var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"autodesk.max.dll\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Autodesk.Max.dll</h1></div><p>With the release of the Subscription Advantage Pack for 3ds Max 2012 there have been significant enhancements to the exposure of 3ds Max SDK to .NET. Previously only a small subset of the 3ds Max SDK was exposed to .NET via the ManagedServices.dll. Now nearly all of the global functions and classes in the SDK can be accessed directly from .NET using the new <code>Autodesk.Max.dll</code>.</p>\n\
<p>The <code>Autodesk.Max.dll</code> assembly is automatically generated from the 3ds Max SDK header files, so most of the functions and classes in the 3ds Max SDK are mapped to similarly named constructs in the DLL.</p>\n\
<p>One of the easiest ways to become familiar with the contents of the DLL is to load it in the Visual Studio object browser. In the rest of this document we will review some key features of this assembly.</p>\n\
<p>You may also consult <a href=\"./../../3ds_max_sdk_-_the_learning_path/lesson_7_writing_dotnet_plug-ins.html\">Lesson 7: Writing .Net Plug-ins</a> for more information on using the <code>Autodesk.Max.dll</code> .NET assembly.</p>\n\
<h2 id=\"accessing-the-services-exposed-by-3ds-max-the-iglobal-interface-and-interface13\">Accessing the services exposed by 3ds Max - the IGlobal Interface and Interface13</h2><p>Before we can do anything interesting with the <code>Autodesk.Max.dll</code> we need a handle to an instance of the &quot;<code>IGlobal</code>&quot; interface. This interface provides access to all of the global functions and variables in the 3ds Max SDK. You might use this from a function as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">IGlobal Global = Autodesk.Max.GlobalInterface.Instance;\n\
IInterface13 Interface = Global.COREInterface13;</code></pre><p>Note the presence of a property &quot;<code>COREInterface13</code>&quot; instead of the expected function &quot;<code>GetCOREInterface13()</code>&quot;. Consult the &quot;Functions to Properties&quot; section below for more information.</p>\n\
<h2 id=\"interfaces-and-classes\">Interfaces and Classes</h2><p>The 3ds Max SDK from C++ uses classes exclusively. What are called &quot;interfaces&quot; in the C++ SDK, are actually &quot;abstract classes&quot;. That means they are instances of classes with one or more pure virtual functions. In  <em>Autodesk.Max.dll</em>  almost every class has a corresponding interface that is used where the class might be expected as an argument type or return type. For example we used <code>IAnimatatable</code> instead of <code>Animatable</code>. The <code>DemoTeapot()</code> function in the next example demonstrates this issue.</p>\n\
<h3 id=\"functions-to-properties\">Functions to Properties</h3><p>The <code>Autodesk.Max.dll</code> maps certain methods in the 3dsMax C++ SDK to .NET properties according to the following rules:</p>\n\
<ul>\n\
<li>The function must have no arguments</li>\n\
<li>The function name must start with &apos;Get&apos;</li>\n\
</ul>\n\
<p>The following code shows how the <code>COREInterface</code> property is used instead of the <code>GetCOREInterface()</code> function.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void DemoTeapot(IGlobal global) \n\
{\n\
    IGlobal global = Autodesk.Max.GlobalInterface.Instance;  //note that global will be an instance of an abstract class.\n\
    var intfc = global.COREInterface13;\n\
    IClass_ID cid = global.Class_ID.Create((uint)BuiltInClassIDA.TEAPOT_CLASS_ID, (uint)BuiltInClassIDB.TEAPOT_CLASS_ID);\n\
    object obj = intfc.CreateInstance(SClass_ID.Geomobject, cid as IClass_ID);\n\
    if (obj == null) throw new Exception(&quot;Failed to create a teapot!&quot;);\n\
    IINode n = global.COREInterface.CreateObjectNode((IObject)obj);\n\
    IAnimatable iobj = (IAnimatable)obj;\n\
    IIParamBlock2 ps = iobj.GetParamBlock(0);\n\
    ps.SetValue(0, global.COREInterface.Time, 20.0f, 0);\n\
    n.Move(global.COREInterface.Time, global.Matrix3.Create(), global.Point3.Create(20, 20, 0), true, true, 0, true);\n\
}</code></pre><h2 id=\"instantiating-types\">Instantiating types</h2><p>In order to instantiate a specific type from the <code>Autodesk.Max.dll</code> we need to use properties of the IGlobal interface. For example in order to instantiate an instance of a Point3 you would write:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">IPoint3 pt = Autodesk.Max.GlobalInterface.Instance.Point3.Create(20, 20, 0)</code></pre><p>Note that <code>IGlobal.Point3</code> is a property that returns an instance of an interface of type <code>IGlobal.IGlobalPoint3</code>. The <code>IGlobalPoint3</code> type is a member type of the <code>IGlobal</code> type and has a <code>Create()</code> function that will return a <code>Point3</code> instance.</p>\n\
<h2 id=\"default-arguments\">Default arguments</h2><p>In <code>Autodesk.Max.dll</code> default arguments are not supported. As a result, some functions require many more arguments than the 3ds Max SDK requires. You can look up the values that are used as default arguments in the 3ds Max SDK documentation.</p>\n\
<h2 id=\"macros\">Macros</h2><p>Many macros (especially function macros) are not ported from 3ds Max SDK to .NET. Instead, you will have to figure out what the macro expands to and then use it directly. For example in C++ the following line:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">EPoly13* epoly = GetEPoly13Interface( myObject );</code></pre><p>expands to:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">EPoly13* epoly = (EPoly*)( myObject )-&gt;GetInterface( EPOLY13_INTERFACE );</code></pre><p>So using <code>Autodesk.Max.dll</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">IEPoly epoly = (IEPoly)myObject.GetInterface( EPOLY13_INTERFACE );</code></pre><p>To do this easily, it is recommended to use the Visual Studio &quot;Find in Files&quot; feature to search the \\maxsdk\\include folder for the correct macro expansion. Alternatively you can use the C++ SDK documentation.</p>\n\
<h2 id=\"faq\">FAQ</h2><ul>\n\
<li><p><em>Where is the DLL?</em></p>\n\
<p>The <code>Autodesk.Max.dll</code> is found in the root directory of 3ds Max.</p>\n\
</li>\n\
<li><p><em>What is Autodesk.Max.Wrappers.dll?</em></p>\n\
<p>It is For internal use only.</p>\n\
</li>\n\
<li><p><em>What is Autodesk.Max.Remoting.dll?</em></p>\n\
<p>It is For internal use only.</p>\n\
</li>\n\
<li><p><em>Isn&apos;t Autodesk.Max.dll the same thing as EPHERE&apos;s Max.NET?</em></p>\n\
<p>Not quite, though they share a common ancestor. Autodesk.Max.dll will likely continue to evolve in a different direction.</p>\n\
</li>\n\
<li><p><em>Can I load plug-ins the same way I did using EPHERE&apos;s built-in plug-in loader?</em></p>\n\
<p>This functionality is not supported, and we strongly recommend against using it.</p>\n\
</li>\n\
<li><p><em>Where can I find class or function X in the .NET API that corresponds to the 3ds Max SDK?</em></p>\n\
<p>Try using the object browser to search for the name. If the name of a function starts with &quot;Get&quot; you should remove that. Note that many macros aren&apos;t carried over from the 3ds Max SDK, so you will have to expand those manually.</p>\n\
</li>\n\
<li><p><em>If I implement an interface found in Autodesk.Max.dll, will it work?</em></p>\n\
<p>No.</p>\n\
</li>\n\
<li><p><em>Can I write a plug-in which overrides classes found in Autodesk.Max.Plugins?</em></p>\n\
<p>We do not recommend overriding these classes when creating plug-ins. Doing so may cause unexpected results.</p>\n\
</li>\n\
<li><p><em>What is the INativeObject.NativePointer property?</em></p>\n\
<p>This is an IntPtr that has a pointer to the unmanaged pointer of an object.</p>\n\
<p><strong>Note</strong>The name of this property changed from <code>.Handle</code> to <code>.NativePointer</code> in 3ds Max 2015.</p>\n\
</li>\n\
<li><p><em>How can I create a Rollout Panel in .NET?</em></p>\n\
<p>You can&apos;t use resource files in .NET. So use the ParamBlock2Flags.autoUI flag to have 3ds Max create the UI for you.</p>\n\
<p>For example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">this.paramBlockDesc = this.global.ParamBlockDesc2.Create( 0, &quot;Parameters&quot;, IntPtr.Zero, this,\n\
 (ParamBlock2Flags)( (int)ParamBlock2Flags.Version + (int)ParamBlock2Flags.AutoConstruct + (int)ParamBlock2Flags.AutoUi ), new object[] { 1, 0 } );</code></pre></li>\n\
<li><p><em>Where can I get Reference Documentation?</em></p>\n\
<p>There are two options:</p>\n\
<ol>\n\
<li>Use the &quot;Object Browser&quot; feature of Visual Studio to look at the <code>Autodesk.Max.dll</code>.</li>\n\
<li>Consult the .NET or C++ Reference adjacent to this Programmer&apos;s Guide.</li>\n\
</ol>\n\
</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_dotnet_assemblies.html\">3ds Max .NET Assemblies</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";