var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"dotnet_action_items\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>.NET Action Items</h1></div><p>It is now possible to create custom action items in a .NET assembly. This is done by creating a public class that implements the interface <code>ICuiActionCommand</code> (found in  <em>MaxCustomControls.dll</em> ) and placing the DLL containing the implemented interface in the  <em>bin\\assemblies</em>  folder. 3ds Max will then dynamically load an instance of this class as an action in its CUI interface. When the action is executed by 3ds Max the method <code>ICommand.Execute()</code> will be called on the loaded class.</p>\n\
<p>To simplify the construction of .NET custom actions, a custom class can inherit from the abstract base class <code>CuiActionCommandAdapter</code>. In the code samples below, we provide an example of such an inheritance using VB.NET and C#. For more information, consult the .NET reference guide.</p>\n\
<h2 id=\"vb-net\">VB.NET</h2><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Imports Autodesk.Max\n\
Imports MaxCustomControls\n\
\n\
Public Class Class1\n\
    Inherits CuiActionCommandAdapter\n\
\n\
    Public Overrides ReadOnly Property ActionText As String\n\
        Get\n\
            Return InternalActionText\n\
        End Get\n\
    End Property\n\
    Public Overrides ReadOnly Property InternalActionText As String\n\
        Get\n\
            Return &quot;Action 1&quot;\n\
        End Get\n\
    End Property\n\
    Public Overrides ReadOnly Property Category As String\n\
        Get\n\
            Return InternalCategory\n\
        End Get\n\
    End Property\n\
    Public Overrides ReadOnly Property InternalCategory As String\n\
        Get\n\
            Return &quot;ADN Samples&quot;\n\
        End Get\n\
    End Property\n\
    Public Overrides Sub Execute(parameter As Object)\n\
        Dim myglobal As IGlobal = Autodesk.Max.GlobalInterface.Instance\n\
        Dim ip As IInterface13 = myglobal.COREInterface13\n\
\n\
        ip.PushPrompt(&quot;Hello 3ds Max .Net API!&quot;)\n\
\n\
    End Sub\n\
End Class</code></pre><h2 id=\"c-\">C#</h2><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">public class MaxDotNetCUIDemo : CuiActionCommandAdapter {\n\
    public override string ActionText {\n\
        get { return InternalActionText; }\n\
    }\n\
\n\
    public override string Category {\n\
        get { return InternalCategory; }\n\
    }\n\
\n\
    public override void Execute(object parameter) {\n\
        Autodesk.Max.Global.Instance.COREInstance13.AddPrompt(&quot;Yeeehaaaaaaaaaa!&quot;);\n\
    }\n\
\n\
    public override string InternalActionText {\n\
        get { return &quot;Prompt me!&quot;; }\n\
    }\n\
\n\
    public override string InternalCategory {\n\
        get { return &quot;3ds Max .NET Demo&quot;; }\n\
    }\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_dotnet_sdk.html\">3ds Max .NET SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";