var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"dotnet_user_interface\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>.NET User Interface</h1></div><p>You can create a user interface (UI) using the .NET API and the Windows Presentation Foundation (WPF) or Windows Forms.</p>\n\
<h2 id=\"creating-the-ui-using-the-wpf\">Creating the UI using the WPF</h2><p>You can create a WPF UI in Visual Studio or another tool such as Microsoft Expression Blend. To setup 3ds Max to work with the WPF window, you need to use <code>ManagedServices.AppSDK.GetMaxHWND</code> and <code>ManagedServices.AppSDK.ConfigureWindowForMax</code>.</p>\n\
<p>The following example shows how to create a floating dialog box:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Create a new managed window to contain the WPF control\n\
System.Windows.Window dialog = new System.Windows.Window();\n\
\n\
// Name the window\n\
dialog.Title = &quot;Sample&quot;;\n\
\n\
// Example of setup size and location\n\
// ...\n\
dialog.SizeToContent = System.Windows.SizeToContent.WidthAndHeight;\n\
dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;\n\
dialog.ResizeMode = System.Windows.ResizeMode.NoResize;\n\
\n\
// Assign the window&apos;s content to be the WPF control\n\
dialog.Content = new CreateTeapotUserControl1();\n\
dialog.ShowInTaskbar = false;\n\
\n\
// Create an interop helper\n\
System.Windows.Interop.WindowInteropHelper windowHandle =\n\
new System.Windows.Interop.WindowInteropHelper(dialog);\n\
\n\
// Assign the 3ds Max HWnd handle to the interop helper\n\
windowHandle.Owner = ManagedServices.AppSDK.GetMaxHWND();\n\
\n\
// Setup 3ds Max to handle the WPF dialog correctly\n\
ManagedServices.AppSDK.ConfigureWindowForMax(dialog);\n\
\n\
// Show the dialog box\n\
dialog.Show();</code></pre><h2 id=\"creating-the-ui-using-the-windows-forms\">Creating the UI using the Windows Forms</h2><p>The .NET scene explorer feature demonstrates how to use a Windows Form in 3ds Max. There are some compatibility issues in handling window events correctly between the native 3ds Max application and any managed Windows Forms opened from it. A .NET plug-in that uses Windows Forms can host their controls in a <code>MaxCustomControls.MaxForm</code> window instead of the .NET Framework standard Form class.</p>\n\
<p>It has been found that not all events are handled properly when using Windows Forms, therefore, it is recommended to setup and test your UI interactivity before coding to make sure that the functionality and events that you need are working properly.</p>\n\
<h2 id=\"creating-dockable-ui-or-new-extended-viewports-using-net\">Creating Dockable UI or new Extended Viewports using .NET</h2><p>A .NET custom user action with a UI component can be created by inheriting from <code>CuiDockableContentAdapter</code> and returning a Windows control from the derived class&apos;s implementation of <code>CreateDockableContent()</code>. The docking style of the new control is controlled by the value returned from the <code>DockingModes</code> property. By returning <code>DockStates.Dock.Viewport</code> from <code>DockingModes</code> enables the control to behave as an extended viewport.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">public class TestAction : CuiDockableContentAdapter \n\
  { \n\
      public override string ActionText { get { return &quot;Test&quot;; } } \n\
      public override string Category { get { return &quot;CuiTest&quot;; } } \n\
      public override Type ContentType \n\
      { \n\
        get { return typeof(TestControl);  \n\
      } \n\
\n\
      public override object CreateDockableContent() \n\
      { \n\
        return new TestControl(); \n\
      } \n\
\n\
      public override DockStates.Dock DockingModes \n\
      { \n\
        get \n\
           { \n\
            return DockStates.Dock.Left | DockStates.Dock.Right | DockStates.Dock.Floating | DockStates.Dock.Viewport; \n\
           } \n\
      }\n\
  }\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_dotnet_sdk.html\">3ds Max .NET SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";