var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"passing_function_callbacks_in_dotnet\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Passing Function Callbacks in .NET</h1></div><p>Registering callbacks in .NET is different than in the C++ SDK. In C++, you can define a single notification handler, register for multiple notifications, and look at the <code>NotifyInfo</code> passed back to the callback to determine what event happened. In .NET, you must define handlers in your code for each notification type you are interested in receiving.</p>\n\
<p>Here&apos;s a C++ example (from the  <em>comsrvgup</em>  sample project included with the 3ds Max SDK - see samples\\gup\\comsrv\\MaxDocument.cpp), notice that the same handler can be used for different notifications:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">//notifications which cause us to revoke our file moniker\n\
RegisterNotification(NotifyProc, this, NOTIFY_SYSTEM_PRE_RESET);\n\
RegisterNotification(NotifyProc, this, NOTIFY_SYSTEM_PRE_NEW);\n\
RegisterNotification(NotifyProc, this, NOTIFY_SYSTEM_SHUTDOWN);</code></pre><p>In .NET, RegisterNotification() is used in a similar way , the difference being you must pass separate delegates of type <code>GlobalDelegates.Delegate5</code> assigned to each callback. Callbacks are unregistered when the plugin is disposed. This ensures that 3ds Max will not attempt to call the function after the plugin has been garbage-collected.</p>\n\
<p>Notification types corresponding to the C++ types defined in  <em>notify.h</em>  are found in the <code>Autodesk.Max.SystemNotificationCode</code> enum.</p>\n\
<p>This simple utility plugin illustrates how to register callbacks in .NET:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">using System;\n\
using System.Diagnostics;\n\
using System.Windows;\n\
\n\
// Max:\n\
using Autodesk.Max;\n\
\n\
\n\
\n\
namespace MaxSDKDocs\n\
{\n\
    public class MyMaxCommand : AbstractCustom_CuiActionCommandAdapter\n\
    {\n\
        static bool bPrompt = true;\n\
        GlobalDelegates.Delegate5 unitsChangeHandler, fileSaveHandler;\n\
        static bool callbackRegistered = false;\n\
\n\
        // Handler methods, one for each delegate:\n\
        private void Global_UnitsChange(IntPtr obj, IntPtr info)\n\
        {\n\
            Debug.WriteLine(&quot;Units changed...&quot;);\n\
        }\n\
        private void Global_PreFileSave(IntPtr obj, IntPtr info)\n\
        {\n\
            Debug.WriteLine(&quot;About to save...&quot;);\n\
        }\n\
\n\
        MessageBoxResult showMessageBoxButton()\n\
        {\n\
            // Configure message box\n\
            string message = &quot;Should I toggle the prompt?&quot;;\n\
            string caption = &quot;Prompt Dialog&quot;;\n\
            MessageBoxButton buttons = MessageBoxButton.OKCancel;\n\
            MessageBoxImage icon = MessageBoxImage.Question;\n\
            MessageBoxResult defaultResult = MessageBoxResult.OK;\n\
            // Show message box\n\
            return MessageBox.Show(message, caption, buttons, icon, defaultResult);\n\
        }\n\
\n\
        public override string CustomActionText\n\
        {\n\
            get { return &quot;Max CUI in .NET&quot;; }\n\
        }\n\
\n\
        public override void CustomExecute(object parameter)\n\
        {\n\
            // Set up global interface:\n\
            IGlobal global = Autodesk.Max.GlobalInterface.Instance;\n\
            IInterface14 ip = global.COREInterface14;\n\
\n\
            // register the notification callbacks once:\n\
            if (!callbackRegistered)\n\
            {\n\
                this.unitsChangeHandler = new GlobalDelegates.Delegate5(this.Global_UnitsChange);\n\
                GlobalInterface.Instance.RegisterNotification(this.unitsChangeHandler, null, SystemNotificationCode.UnitsChange);\n\
                this.fileSaveHandler = new GlobalDelegates.Delegate5(this.Global_PreFileSave);\n\
                GlobalInterface.Instance.RegisterNotification(this.fileSaveHandler, null, SystemNotificationCode.FilePreSave);\n\
\n\
                callbackRegistered = true;\n\
            }\n\
\n\
            MessageBoxResult result = showMessageBoxButton();\n\
            if (result == MessageBoxResult.OK)\n\
            {\n\
                if (bPrompt)\n\
                    ip.PushPrompt(&quot;Hello 3ds Max .Net API&quot;);\n\
                else\n\
                    ip.PopPrompt();\n\
\n\
                bPrompt = !bPrompt;\n\
            }\n\
        }      \n\
    }\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_dotnet_sdk.html\">3ds Max .NET SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";