var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"troubleshooting_dotnet_plug-ins\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Troubleshooting .NET Plug-ins</h1></div><p>This section contains a list of common problems encountered when developing 3ds Max plug-ins in .NET. These particular issues were chosen for discussion here because they tend to have limited documentation available and were more difficult to debug or resolve. Other common issues are not listed here because they are covered by resources elsewhere, either in the MSDN documentation, in the 3ds Max SDK documentation, or in most .NET books.</p>\n\
<h2 id=\"warning-lnk4248-unresolved-typeref-token-for-39-type-39-\">Warning LNK4248: unresolved typeref token for &#39;type.&#39;</h2><p>Generally, warning LNK4248 is signalled when the C++ / CLI compiler must create an empty class wrapper for a forward-declared type. This is not a problem if the class definition is not required in the module. The warning can be suppressed by either including the full class definition for the given type or by explicitly creating an empty class definition. See <em>SceneExplorerExtensionClassDesc.cpp</em>  in the how-to sample project for an example.</p>\n\
<h2 id=\"3ds-max-hangs-or-crashes-on-start-up-or-exit\">3ds Max hangs or crashes on start-up or exit</h2><p>A most common cause for a crash of 3ds Max when a .NET plug-in is being loaded or unloaded, is due to an exception being thrown and not caught. 3ds Max cannot handle an exception thrown during DLL loading or unloading. For .NET plug-ins, an exception is thrown for any call into managed code before the CLR is loaded or after the CLR is unloaded for the given DLL. In other words: an exception is thrown for any managed code executed from DllMain. The solution is to perform initializations in LibInitialize and clean-up in LibShutdown. Often a call to the CLR occurs for any static object instance in the DLL. Temporarily setting the <code>/NOENTRY</code> linker option under: <code>Configuration Properties -&amp;gt; Linker -&amp;gt; Advanced -&amp;gt; No Entry Point</code> will help find any statically allocated instances. See <code>LibShutdown</code> in  <em>dllmain.cpp</em>  in the how-to sample project for an example.</p>\n\
<p>Another common cause of 3ds Max crashing on start-up is the presence of multiple copies of the .NET assemblies in various start-up directories. For example, if  <em>Autodesk.Max.dll</em>  is present in both the 3ds Max application directory and in  <em>bin/assmblies</em> , 3ds Max will crash. Make sure <code>Copy Local</code> is set to False for assembly references in your .NET project if you have set the project&#39;s build output directory to the 3ds Max  <em>/bin/assemblies</em>  directory.</p>\n\
<h2 id=\"error-c3767-39-function-39-candidate-function-s-not-accessible-\">Error C3767: &#39;function&#39;: candidate function(s) not accessible.</h2><p>Native types are private by default in C++/CLI. If a public function takes a private type as a parameter, references to this function from outside the assembly will generate error C3767. The solution is to mark the native type being used as public using a <code>\\#pragma</code> make_public(type) directive. See  <em>NativeInclude.h</em>  in the how-to sample project for an example.</p>\n\
<h2 id=\"error-c3642-39-return_type-args-39-cannot-call-a-function-with-__clrcall-calling-convention-from-native-code\">Error C3642: &#39;return_type/args&#39; : cannot call a function with __clrcall calling convention from native code</h2><p>When writing a .NET plug-in, it is neccessary at one point to call managed code, from native code. You can call an managed class that is not garbage collected (e.g. it is not a &quot;<code>ref class</code>&quot;), but you can only call methods that have no managed types in the signature. If a managed type is in the signature, the calling convention for that methods implicitly becomes &quot;<code>\\__clrcall</code>&quot;.</p>\n\
<h2 id=\"error-lnk2022-metadata-operation-failed-80131195-custom-attributes-are-not-consistent\">Error LNK2022: metadata operation failed (80131195) : Custom attributes are not consistent</h2><p>This linker error occurs if a custom metadata attribute is not applied consistently to a type. This occurs if a native type is marked using a <code>\\#pragma</code> make_public directive in one translation unit, but included without the directive in another translation unit. To avoid this problem, the 3ds Max SDK development team adopted the habit of grouping all of a C++/CLI assembly&#39;s native includes in a single header file and applying all metadata directives in that file. See  <em>NativeInclude.h</em>  in the how-to sample project for an example.</p>\n\
<h2 id=\"deadlock-during-dll-initialization\">Deadlock During DLL Initialization</h2><p>It is possible in some circumstances for deadlock DLL initialization, a situation sometimes called loader lock. Resolving this problem is described on MSDN at <a href=\"http://msdn.microsoft.com/en-us/library/ms173266.aspx\" target=\"_blank\">http://msdn.microsoft.com/en-us/library/ms173266.aspx</a>.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_dotnet_sdk.html\">3ds Max .NET SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";