var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"getting_started_writing_plug-ins\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Getting Started Writing Plug-ins</h1></div><h2 id=\"introduction\">Introduction</h2><p>This document reviews the structure of 3ds Max plug-ins and the interactions between 3ds Max software, the plug-ins and the user. The goal is to provide a clear image of the architecture of a plug-in in high level terms without getting into plug-in specific details. This document is written for C++ developers with no previous experience in writing 3ds Max plug-ins using the 3ds Max SDK. It is expected that the reader is already familiar with the basic features of 3ds Max.</p>\n\
<h2 id=\"what-is-a-plug-in-\">What is a plug-in?</h2><p>A plug-in is code and data that provide additional features or functionality to 3ds Max. There are many plug-ins shipped with 3ds Max whose functionality form the majority of the architecture and behavior of the final product. Examples of plug-ins are custom geometric objects, animation controllers, texture maps, materials and so on. We will explain here how a developer can create a plug-in to accomplish tasks by providing practical lessons on a few types of plug-ins and their features. You can refer to the topics <a href=\"./../overview/overview_plug-ins.html\">Overview: Plug-ins</a> and and <a href=\"./../writing_plug-ins/creating_a_plug-in_project/manually_creating_a_new_plug-in_/plug-in_file_extensions.html\">Plug-in File Extensions</a> in the Programmer&#39;s Guide for a complete review of 3ds Max plug-ins.</p>\n\
<h2 id=\"how-3ds-max-recognizes-your-plug-ins\">How 3ds Max recognizes your plug-ins</h2><p>Programmers implement plug-ins in dynamic link libraries (DLLs). These DLLs include classes extended from one or more predefined base classes in the Max SDK, and override specific member functions according to their needs. At start up, 3ds Max attempts to load all DLL files located in certain 3ds Max installation subfolders (e.g. &lt;3dsMaxInstallFolder&gt;\\plugins). A DLL will only load if it exports the required DLL functions mentioned below. These functions tell 3ds Max how many plug-ins are implemented in the DLL file, and return objects that both describe each of those plug-ins, and allow 3ds Max to instantiate from them.</p>\n\
<h2 id=\"what-3ds-max-expects-from-your-plug-in\">What 3ds Max expects from your plug-in</h2><h3 id=\"required-dll-functions\">Required DLL functions</h3><p>In order for a DLL file to be loaded by 3ds Max it must implement certain functions. Common to any DLL, there must be a DLL entry point function (named <a href=\"./../writing_plug-ins/creating_a_plug-in_project/manually_creating_a_new_plug-in_/required_dll_functions.html\">Required DLL Functions</a>). There are also at least four other functions required by 3ds Max. Three of them return basic information about the plug-in including: (1) the number of plug-in classes in the DLL, (2) the version of the 3ds Max SDK with which the plug-in has been compiled, and (3) a short string that gives the user an idea of what is contained in the DLL (e.g. what the included plug-in does).</p>\n\
<p>The fourth function provides a mechanism with which the actual plug-in can be created (i.e. an object of our plug-in class can be instantiated) in 3ds Max. Once the plug-in is created using this function, 3ds Max will have specialized interfacing to the plug-in based on the plug-in type. This function is explained in more detail below in the Class Descriptors section.</p>\n\
<p>Other than these four functions, without which your plug-in cannot be compiled, there are two other DLL functions needed for initializing and un-initializing the plugin. It is strongly recommended that you include these two functions in your DLL as well. The names and detailed descriptions of these functions are listed in the topic <a href=\"./../writing_plug-ins/creating_a_plug-in_project/manually_creating_a_new_plug-in_/required_dll_functions.html\">Required DLL Functions</a> in the Programmer&#39;s Guide.</p>\n\
<h3 id=\"class-descriptors\">Class descriptors</h3><p>Other than the main plug-in class, every plug-in project contains a class knows as the &#39;class descriptor&#39;. The class descriptor has virtual member functions that provide information about the plug-in to 3ds Max. This information includes the class ID (explained below), category, and name. The class descriptor is usually derived from ClassDesc2, but can also be derived from ClassDesc.</p>\n\
<p>An important task of the class descriptors is to implement a function (named <code>Create()</code>) that provides 3ds Max a pointer to an instance of the plug-in object. 3ds Max will use this pointer to access the plug-in&#39;s properties and functionality. You can refer to <a href=\"./../writing_plug-ins/plug-in_basics/class_descriptors.html\">Class Descriptors</a> in the programmer&#39;s guide for detailed information on the class descriptor.</p>\n\
<h3 id=\"class-id\">Class ID</h3><p>3ds Max recognizes plug-ins not by name but by ID. This means that two different plug-ins with the same name can co-exist as long as they have different IDs. On the other hand, if two plug-ins with different or the same name share the same ID, 3ds Max will load only one of them and will give a warning message regarding the conflict. This is because different developers might accidentally pick up the same names for their plug-ins, but it is practically impossible for them to pick up the same IDs since the ID consists of two random unsigned 32-bit numbers.</p>\n\
<h3 id=\"def-files\">DEF Files</h3><p>A .DEF file can be included in a DLL project to describe various attributes of the DLL such as the name and the order of the functions exposed in the DLL file. By listing the six mandatory plug-in functions in your .DEF file, you tell 3ds Max that the functions are implemented and are ready to be called. Although there are methods to load the plug-ins without including the .DEF file in the projects, we include it in the sample projects accompanying the lessons in this learning path to make them compatible with the plug-ins in the  <em>howto</em>  and  <em>samples</em>  sub-folders.</p>\n\
<h2 id=\"how-to-make-your-plug-ins-do-what-you-want\">How to make your plug-ins do what you want</h2><p>Based on their desired task, plug-ins can extend from many different base classes. The base class will outline the structure of your plug-in. Unlike a standalone application, your plug-in will not work by following a path from a start to end point in its code. Instead, it implements the member functions that are called by either 3ds Max, other plug-ins, or the plug-in itself at specific times or events. Each plug-in is extended from one or more 3ds Max predefined base classes. Each base class (and probably its parents) have a number of virtual methods that your plug-in must implement in order to become functional. Usually these are pure virtual functions. Additional optional virtual methods may be implemented depending on what your needs are, and these can improve the integration of your product with 3ds Max.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_sdk_-_the_learning_path.html\">3ds Max SDK - The Learning Path</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";