var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"lesson_2_the_scene_graph_and_nod\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Lesson 2: The Scene Graph and Nodes</h1></div><p>In a 3ds Max scene the objects that a user can create and manipulate in the viewports are called scene objects. Scene objects derive from the <code>Object</code> class. Examples of scene objects are geometric objects (<code>GeomObject</code>), lights (<code>LightObject</code>), cameras (<code>CameraObject</code>), particles, and world space modifiers (not to be confused with regular modifiers). Scene objects can have parent-child relationships to form a hierarchy and let users create compound objects. Transforms applied to the parent are also transmitted to child objects. You can refer to <em>Animation</em> &gt; <em>Hierarchies and Kinematics</em> &gt; <em>Hierarchies</em> in the 3ds Max online help to review the parent-child relationship among the scene objects from the user&apos;s perspective. To understand the relationship between scene graph nodes in modifier stack, read the topic <a href=\"./../3ds_max_sdk_features/modeling/geometry_pipeline_system/pipeline_overview.html\">Pipeline Overview</a> .</p>\n\
<p>3ds Max stores the information about scene objects as well as their relationship to each other in a data structure called the Scene Graph. Each node of this graph is an object of the class <code>INode</code> that corresponds to one scene object. Each <code>INode</code> object has information about its corresponding scene object including (but not limited to) a pointer to the object, the number of child nodes, the address of the parent and the child nodes, the position, rotation and scale (PRS) transform controllers for that object, and the material applied to the object. Note that the nodes in the scene graph are not the real scene objects but rather have a pointer to their corresponding scene objects. The scene objects themselves do not have any information about their PRS controllers, their materials, their relation to the other objects, etc.</p>\n\
<p>The edges of the scene graph are directed edges that are called links in 3ds Max. There is a link from node A to node B if the state of node A (e.g. position, rotation, scale, material, etc) depends on that of node B. Each object can be followed or linked by many objects but can link to only one object. No loop is allowed in the links of the nodes. These together make the scene graph a tree data structure. The root of this tree is a virtual node that is not linked to any other node (the only exception). The nodes corresponding to the independent scene objects that do not have a parent object in the scene are the first level nodes in this tree. Even if their object is instantiated from their classes, the scene objects do not get displayed in the viewport unless they have a corresponding node in the scene graph.</p>\n\
<p>The <code>INode</code> class is an important class in the 3ds Max SDK and has a large number of member functions. In this lesson we review a small set of <code>INode</code>&apos;s member functions to get familiar with the basics of the scene graph and nodes. You can refer to the programmers&apos; guide topic <a href=\"./../overview/overview_scene_graph_and_nodes.html\">Overview: Scene Graph and Nodes</a> or the <code>INode</code> entry in the C++ API Reference for detailed information about the <code>INode</code> class.</p>\n\
<p>You can access the root node of the scene graph using the function <code>Interface::GetRootNode()</code>. Once you get a pointer to a node, there are many functions that get information about its scene object. A few examples of these functions are:</p>\n\
<ul>\n\
<li><code>INode::IsRootNode()</code> - determines if a node is the root node (does not have a parent node).</li>\n\
<li><code>INode::GetParentNode()</code> - returns the parent node of a child.</li>\n\
<li><code>INode::NumberOfChildren()</code> - gets the number of children of the node.</li>\n\
<li><code>INode::GetChildNode()</code> - returns an <code>INode</code> pointer to the nth child.</li>\n\
<li><code>INode::GetName()</code> - returns the name of the scene object of this node.</li>\n\
<li><code>INode::GetObjectRef()</code> - returns a pointer to the scene object of the node.</li>\n\
<li><code>INode::GetNodeTM()</code> - returns the PRS of the node&apos;s scene object at the given time.</li>\n\
</ul>\n\
<p>In the example plug-in for this lesson, we will create a scene export plug-in that writes information about all the nodes in the scene graph. In the plug-in wizard, select the &apos;File Export&apos; plug-in type, and choose the &apos;Scene Export&apos; for the base class. This class has a number of simple functions that return basic information about the scene exporter plug-in such as the type of the file to be generated, a description of what the exporter does, the number of file types supported, etc. The most important function that we need to implement is <code>SceneExport::DoExport()</code>. 3ds Max calls this function to do the main export job.</p>\n\
<p>The project in the lesson folder  <em>Lesson2</em>  shows the plug-in implementation. We use three helper functions named <code>Lesson2::BeginWriting()</code>, <code>SampleExp::Write()</code> and <code>Lesson2::EndWriting()</code> to open the file to be written to, write a message to the file and to close it. When our plug-in is selected to be used for export, 3ds Max takes care of the file selection UI and gives us the file name in its call to <code>SceneExport::DoExport()</code>. We implement a recursive private member function named <code>Lesson2::Export()</code> to help us exporting the scene graph information. This function takes a pointer to a node in the scene graph, writes its name and the number of its child notes in the text file, and then calls itself on each of the child nodes of the input node. This way all the nodes will have their name and the number of children written in the text file if we call this function for the root node. It will be a depth-first search since the recursive function is called on the children of the node right away after writing the node&apos;s info and before writing the information of the sibling nodes. The following code shows the implementation of this function:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void Lesson2::Export(INode* pNode, int iTreeDepth)\n\
{\n\
    MCHAR * pNodeName = const_cast&lt;MCHAR*&gt;(pNode-&gt;GetName());\n\
    int nChildren = pNode-&gt;NumberOfChildren();\n\
\n\
    Write(iTreeDepth, &quot;[%s] : %i children&quot;, pNodeName, nChildren);\n\
\n\
    iTreeDepth++;\n\
    for (int i = 0; i &lt; nChildren; i++)\n\
    {\n\
        Export(pNode-&gt;GetChildNode(i), iTreeDepth);\n\
    }\n\
}</code></pre><p>The variable <code>iTreeDepth</code> in the above code stores the level of the current node and is used to format the output. The number of tabs printed at the start of each node info is the same as the node level. The lower in the scene graph a node is, more indented its info in the output file will be. Also the <code>Lesson2::Export()</code> is called on the root node in its first call in the <code>Lesson2::DoExport()</code> function:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">int Lesson2::DoExport(const TCHAR *name,ExpInterface *ei,\n\
    Interface *i, BOOL suppressPrompts, DWORD options)\n\
{\n\
    ...\n\
    INode* pRootNode = i-&gt;GetRootNode();\n\
    Export(pRootNode);\n\
    ...\n\
}\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_sdk_-_the_learning_path.html\">3ds Max SDK - The Learning Path</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";