var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"lesson_3_the_reference_system\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Lesson 3: The Reference System</h1></div><p>Plug-ins can use the 3ds Max reference system to be notified when another entity changes in the scene. A plug-in creates a reference to a second plug-in in the reference system schema, so if the second plug-in (the reference target) is changed, deleted or updated, the first plug-in (the reference maker) is automatically notified. A plug-in can be both a reference target and a reference maker at the same time. An example of references is a scene object following a second scene object. If the first object has a reference to the second one, it is notified whenever the coordinates of the second object changes, so it can update its own coordinates. Another example is a camera that is looking at a geometric object and its perspective needs to be updated whenever the parameters of the geometric object, such as position, orientation, size, material, etc. changes or when the object is deleted. We will also see in <a href=\"lesson_6_parameter_blocks.html\"\">Lesson 6: Parameter Blocks</a> that the plug-ins have a reference to their parameters (stored in a separate objects of the class <code>IParamBlock2</code>) and update themselves whenever one of their parameters change. You can refer to the topic <a href=\"./../3ds_max_sdk_features/reference_system.html\">Reference System</a> in the Programmers Guide to read detailed information about this.</p>\n\
<p>In a 3ds Max plug-in, any pointer to an object that is set, accessed, modified or deleted using the specific guidelines of the reference system is called a reference. It is always recommended to maintain a 3ds Max reference rather than a general pointer when keeping track of an object. A general pointer only refers to a memory address and is not capable of determining what 3ds Max or other plug-ins do to the object in that address. The target object can even be deleted, leaving your plug-in with a hanging pointer. When you are using a reference, your plug-in is informed of any changes to its target object. In order to use the references system and register the pointers as references, your plug-in should extend from the class <code>ReferenceMaker</code>. Most of the plug-in types are already extended directly or indirectly from <code>ReferenceMaker</code> (although there are exceptions such as <code>Utility</code> plug-ins).</p>\n\
<p>Being a <code>ReferenceMaker</code>, your plug-in will need to implement four virtual functions from that class. These functions are:</p>\n\
<ul>\n\
<li><p><code>int ReferenceMaker::NumRefs();</code></p>\n\
<p>3ds Max calls this function to know how many references we are holding in our plug-in class.</p>\n\
</li>\n\
<li><p><code>void ReferenceMaker::SetReference(int i, ReferenceTarget* pTarget);</code></p>\n\
<p>3ds Max calls this function to give us a reference. We should store the given pointer in the reference whose index is given, and 3ds Max will notify us of any changes to the target node. <em>Note:</em> Only 3ds Max calls this function. Never call it to set or update one of your references. Use <code>ReferenceMaker::ReplaceReference()</code> instead.</p>\n\
</li>\n\
<li><p><code>RefTargetHandle ReferenceMaker::GetReference(int i);</code></p>\n\
<p>3ds Max calls this function to get the ith reference managed by our plug-in instance.</p>\n\
</li>\n\
<li><p><code>RefResult ReferenceMaker::NotifyRefChanged ( const Interval&amp; changeInt, RefTargetHandle hTarget, PartID&amp; partID, RefMessage message, BOOL propagate);</code></p>\n\
<p>3ds Max calls this function to let us know that one of our references has been changed. The complementary information is provided in the list of arguments:</p>\n\
<ol>\n\
<li>Interval <code>changeInt</code>: this parameter is never used</li>\n\
<li>RefTargetHandle <code>hTarget</code>: Our reference that has been changed or updated</li>\n\
<li>PartID&amp; <code>partID</code>: Additional information depending on the message</li>\n\
<li>RefMessage <code>message</code>: Tells what exactly has happened.</li>\n\
</ol>\n\
</li>\n\
</ul>\n\
<p>Please note that you should never directly assign the address of an object to your reference. Neither should you call <code>ReferenceMaker::SetReference()</code> for this purpose. You will need to call reference handling methods such as <code>ReferenceMaker::DeleteReference()</code>, or <code>ReferenceMaker::ReplaceReference()</code> to modify or update your references.</p>\n\
<h2 id=\"-em-adding-some-functionality-to-your-utility-plug-in-em-\"><em>Adding some functionality to your utility plug-in</em></h2><p>The plug-in project in <a href=\"lesson_1_sample_utility_plug-in.html\"\">Lesson 1: Sample utility plug-in</a> is a valid plug-in but does not perform any specific task other than prompting a string. To demonstrate how you can have your plug-in interact with 3ds Max, lets modify that utility plug-in to show the name of the selected scene object in the UI panel. We will need to add some new functions and edit some existing ones for this. We also add the data member <code>INode* mpMyNode</code> to make a reference to the node in the scene graph that represents the currently selected scene object.</p>\n\
<h3 id=\"creating-a-ui\">Creating a UI</h3><p>Before starting to edit the source code, we need to make sure that we have a placeholder for the name of the selected node in the plug-in UI panel. </p>\n\
<p>If you are using the plug-in wizard, add a static text widget (QLabel) to your dialog within the .ui file to show the name of the selected scene object. To do this, right-click and select <strong>Open</strong> for the plugin_form.ui in Visual Studio.  This will open the form in Qt Designer.  Drag in a new <strong>Label</strong> widget, set its text to &quot;Uninitialized&quot; and re-name it &quot;NodeName_Label&quot;.  You can delete the two place-holder labels (&quot;Generated by Autodesk 3ds Max&quot; and &quot;TODO: Place panel&quot;) created by the plug-in wizard.</p>\n\
<p><img src=\"./images/learning_path_qt_label.png\" alt=\"Qt Label\"></p>\n\
<p>The project Lesson3 implements a utility plug-in class named SampleRef that is also a reference maker, and displays the name of the selected node in the viewport. The following code snippets are copied from that project.</p>\n\
<h3 id=\"using-the-reference-system-in-your-utility-plug-in\">Using the reference system in your utility plug-in</h3><p>After creating the UI panel we need to know when a new scene object is selected by the user. Our data member <code>INode* mpMyNode</code> is a reference to a node in the scene graph, and not to a scene object. Therefore it will be our responsibility to update this reference when the selected object in the viewport changes. The function <code>UtilityObj::SelectionSetChanged()</code> will help us here. 3ds Max calls this function whenever the user selects a new object or deselects the old one. We will write a function to update the caption of the static text we added to the dialog. We name this function <code>SetText()</code> here with the following implementation:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void SampleRef::SetText(const MSTR&amp; s)\n\
{\n\
    // MSTR is the main string class in 3ds Max SDK.\n\
    // ui is the instance of our Rollup UI\n\
\n\
    ui.NodeName_Label-&gt;setText(s);    \n\
}</code></pre><p>We will need to call this function to update the text in the rollout page when the plug-in starts, a new node is selected, a selected node is changed or the node is de-selected. 3ds Max calls different functions in our plug-in when each of these situations happen. These function are:</p>\n\
<ul>\n\
<li><code>UtilityObj::BeginEditParams()</code> is called when the utility plug-in starts and the rollout page appears. We introduced this function in lesson 1. Note that we only update our reference (and do not directly update the text) by calling the <code>ReplaceReference()</code> here. This will consequently cause 3ds Max to call our <code>SetReference()</code>, where we update the text.</li>\n\
<li><code>UtilityObj::SelectionSetChanged()</code> is called when the user selects another object in the viewport or when he deselects a previously selected object. The data member <code>INode* mpMyNode</code> in our plug-in is a reference to a node in the scene graph, and not directly to a scene object. When a new scene object is selected the currently referenced node stays unchanged and our reference will not be notified. We will use this function to update our reference manually in this case. Again, we only update our reference and the text is updated as a result, similar to <code>BeginEditParams()</code> above.</li>\n\
<li><code>ReferenceMaker::NotifyRefChanged()</code> is called when there is a change in one of the reference targets of our plug-in. In this example, when the scene object that our reference node is representing changes its position or its name or gets deleted, the corresponding node (our reference) will be changed or deleted automatically. 3ds Max calls <code>NotifyRefChanged()</code> to notify us of such an event. We change the text in the rollout page in that function to reflect the change.</li>\n\
</ul>\n\
<p>The following code snippets show the implementation of these functions. The <code>Interface::GetSelNode()</code> function used in the implementation of <code>SampleRef::SelectionSetChanged()</code> returns a pointer to the <code>INode</code> object corresponding to the currently selected object in the viewport.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void SampleRef::SelectionSetChanged(Interface *ip,IUtil *iu)\n\
{\n\
    if (ip-&gt;GetSelNodeCount() &gt; 0) \n\
        ReplaceReference(0, ip-&gt;GetSelNode(0));\n\
    else\n\
        ReplaceReference(0, NULL);\n\
}\n\
\n\
void SampleRef::SetReference(int i, ReferenceTarget* pTarget)\n\
{\n\
    switch (i)\n\
    {\n\
    case 0:             //we have only one reference in this plug-in\n\
    {\n\
        INode* tmp = dynamic_cast&lt;INode*&gt;(pTarget);\n\
        if (tmp != NULL) \n\
        {\n\
            mpMyNode = tmp;\n\
            MSTR s(tmp-&gt;GetName());\n\
            s += _T(&quot; is being observed now.&quot;);\n\
            SetText(s);\n\
        }\n\
        else \n\
        {\n\
            SetText(_T(&quot;No node is currently being observed.&quot;));\n\
        }\n\
    }\n\
    break;\n\
}\n\
}</code></pre><p>and:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">RefResult SampleRef::NotifyRefChanged(const Interval&amp; changeInt,    RefTargetHandle hTarget, \n\
    PartID&amp; partID, RefMessage message, BOOL propagate)\n\
{\n\
    // Should be true, but just in case\n\
    if (mpMyNode) \n\
    {\n\
        MSTR s(mpMyNode-&gt;GetName());\n\
        s += _T(&quot; is changed.&quot;);\n\
        SetText(s);\n\
    }\n\
    return REF_SUCCEED;\n\
}</code></pre><p>You can compile the Lesson3 project and then start 3ds Max to see your plug-in (named SampleRef) come to life. You can click on <em>Utilities</em> &gt; <em>More</em> to find it in 3ds Max.</p>\n\
<h2 id=\"references-and-the-track-view\">References and the Track View</h2><p>The animatable references of a 3ds Max plug-in (i.e. references extended from the class <code>Animatable</code>) are displayed in the <a href=\"./../3ds_max_sdk_features/animation_time_and_controllers/track_view.html\">Track View</a>. We will see in <a href=\"lesson_4_animation_controllers.html\"\">Lesson 4: Animation Controllers</a> that such references can also be viewed and possibly edited in the Curve Editor if they are controllers (i.e. extended from the class <code>Control</code>).</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_sdk_-_the_learning_path.html\">3ds Max SDK - The Learning Path</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";