var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"lesson_4_animation_controllers\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Lesson 4: Animation Controllers</h1></div><p>Users animate their objects in 3ds Max by changing the object&apos;s properties over time. 3ds Max uses &quot;animation controllers&quot; for this purpose. Animation controllers - or &quot;controllers&quot; for short - are plug-ins whose main responsibility is to provide 3ds Max with the value of a parameter at a given time, whenever 3ds Max inquires them. In mathematical terms, controllers implement a function of time whose output is a point in either one dimensional space (float controllers) or in a multi dimensional space (e.g. position or rotation controllers). A common application of controllers is to update the position, rotation or scale (PRS) parameters of a node associated with an object. Note that the scene graph does not store the geometric objects, but rather nodes associated with those objects and their own PRS parameters.</p>\n\
<p>The type of the inquired parameter determines the type of the controller. The following is the list of basic controller types (with the type of parameter they control in parentheses):</p>\n\
<ul>\n\
<li>Floating point controllers (<code>float</code>)</li>\n\
<li>Point3 controllers (<code>Point3</code>)</li>\n\
<li>Position controllers (<code>Matrix3</code>/<code>Point3</code>)</li>\n\
<li>Rotation controllers (<code>Quat</code>)</li>\n\
<li>Scale controllers (<code>ScaleValue</code>)</li>\n\
<li>Transform controllers (<code>Matrix3</code>)</li>\n\
</ul>\n\
<p>Controllers are divided into two categories based on their behavior: procedural and keyframe. Procedural controllers typically does not take any input other than the initial value at time zero, and generates outputs when inquired. They use an algorithm that generates outputs for different time values. A keyframe controller on the other hand provides an interface with which the user can set the value of the animated parameter at certain points in time. The controller usually caches these values and uses them to interpolate the value of the animated parameter at other points in time. We will first introduce procedural controllers in a basic example, and then look at keyframe controllers.</p>\n\
<h2 id=\"procedural-controllers\">Procedural Controllers</h2><p>Procedural controllers are extended from the base class <code>Control</code>. The main responsibility of these controllers is to answer a query from 3ds Max on a parameter&apos;s value at the given time, and to include the <a href=\"./../3ds_max_sdk_features/modeling/modifiers/validity_intervals.html\">Validity Intervals</a> for that answer. Including a validity interval reduces the communication cost between 3ds Max and your plug-in since 3ds Max does not query the controller during the intervals in which the value does not change.</p>\n\
<p>Procedural controllers do not need to maintain a cache since 3ds Max never tries to write any value to the parameter they are controlling. Developers can use an algorithm that produces values for different times. This should be done in the member function <code>Control::GetValue()</code> and is explained in the example below, where a procedural controller plug-in named <code>SampleProCtrl</code> is assigning a periodic movement to an object:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void SampleProCtrl::GetValue(TimeValue t, void *ptr, Interval &amp;valid, GetSetMethod method)\n\
{\n\
    Point3 p3OurAbsValue(0, 0, 0);\n\
    p3OurAbsValue.x = 15*sin((float)t/960);\n\
    p3OurAbsValue.y = (float)t/192;\n\
       valid.Set(t,t+1); //This answer is only valid at the calling time.\n\
\n\
       if (method == CTRL_ABSOLUTE)\n\
       {\n\
            Point3* p3InVal = (Point3*)ptr;\n\
              *p3InVal = p3OurAbsValue;\n\
       }\n\
       else // CTRL_RELATIVE\n\
       {\n\
              Matrix3* m3InVal = (Matrix3*)ptr;\n\
              m3InVal-&gt;PreTranslate(p3OurAbsValue);\n\
       }\n\
}</code></pre><p><em>Controller::GetValue() parameters:</em></p>\n\
<ul>\n\
<li><code>TimeValue t</code> is the time at which 3ds Max wants to know the value of the animated parameter. It is essentially an integer number, and each 4800 unit is equal to one second.</li>\n\
<li><code>GetSetMethod method</code> indicates whether your implementation should return the absolute <code>(CTRL_ABSOLUTE)</code> or relative <code>(CTRL_RELATIVE)</code> value of the animated parameter. If a relative answer is required, the reference value will be the value 3ds Max pre-loads in the <code>\\*ptr</code> parameter in its call to this function.</li>\n\
<li><code>Void* ptr</code> points to the address where the value of the animated parameter should be written. If a relative answer is required <code>(method == CTRL_RELATIVE)</code>, it initially holds the reference point. It is a void pointer because the answer can be any of the six types above and needs to be type-casted in your code according to the type of animated parameter.</li>\n\
<li><code>Interval &amp;valid</code> is an output parameter and indicates how long the value you computed for ptr is valid. Failing to update this parameter will be interpreted as &quot;forever&quot; by 3ds Max, and it will not call this function at any later time. This can result in freezing your parameter in its initial value in the course of animation.</li>\n\
</ul>\n\
<p>This controller animates the object with a periodic movement along the x axis with the mathematical function <code>x=a.sin(w.t)</code>. The terms a and w are just two coefficients to control the amplitude and frequency of the periodic movement so the movement can be observed easily in the viewport (a = 15 and w = 1/960 in this example). There is also a linear movement along the y axis (<code>y=b.t</code> where b is a coefficient representing the linear speed and is set to 1/192). The function <code>Matrix3::PreTranslate()</code> is used if a relative answer is required. It applies an incremental translation transformation to the <code>matrix m3InVal</code> that is initialized to the reference point.</p>\n\
<p>The project Lesson4a implements an animation controller plug-in. The class name is <code>SampleProCtrl</code> and it appears in 3ds Max with the name &quot;Sample Procedural Controller&quot;. To assign it to an object in the view port, while the object is selected in 3ds Max click on the <em>Motion</em> tab, expand <em>Assign Controller</em> and select <em>Position</em> from the <em>Transform: Position/Rotation/Scale</em> tree. Then click on the green check mark under the title <em>Assign Controller</em> to find &quot;Sample Procedural Controller&quot; and assign it to your object. You can then click on the <em>Play Animation</em> button to see the effect of your animation controller. You can also use the <em>Curve Editor</em> to assign it to an object.</p>\n\
<p>Although we are not holding any reference in our very basic controller, you can see that the four basic functions discussed in Lesson 3 are implemented here. That is because an animation controller plug-in extends from the control class, which indirectly extends from the <code>ReferenceMaker</code> class. You also need to implement the pure virtual function <code>Control::Copy()</code> to be able to instantiate from your plug-in class. Notice that the x, y and z curves are observed all in the same graph in the curve editor and you cannot have them in separate graphs. You will need to break your position controller into basic independent controllers and implement two other functions, <code>Control::NumSubs()</code> and <code>Control::SubAnim()</code> to observe them as separate graphs in the curve editor. We talk about these functions later in this lesson.</p>\n\
<h2 id=\"keyframe-controllers\">Keyframe Controllers</h2><p>A keyframe controller extends from the base class <code>Control</code>, similar to a procedural controller. It only differs from a procedural controller in the way it implements the member functions. Similar to a procedural controller, 3ds Max calls <code>Control::GetValue()</code> to read values of an animated parameter with a keyframe controller. However, it can also write values to an animated parameter of a keyframe controller. When calling to write a value to the controller, 3ds Max indicates at which time and for how long the value it provides is valid. It is the responsibility of the keyframe controller to maintain a data cache to store those values and interpolate between them for unknown values. 3ds Max has no restriction on how the controller plug-in handles this cache. While it is possible to implement and maintain your own cache to keep this data, it is very common to use multiple default basic controllers to form the aggregated animation controller. In such case, it is a good practice to hold one reference per basic controller and access them through the references.</p>\n\
<p>The simplest type of controller is the float controller. It can be combined to form the elements of an aggregated controller. For example, three float controllers can be used to control each of the x, y and z elements in a point3 or position controller. While developers can implement any interpolator (e.g. constant, linear, polynomial, etc) for float controllers, it is normal in the computer graphics applications to use Bezier interpolation. 3ds Max SDK already has a default implementation of a float controller that uses this type of interpolator. Developers can use the function <code>Control *NewDefaultFloatController()</code> to instantiate a new object of a float controller with Bezier interpolation. We will use this function for our example here.</p>\n\
<h2 id=\"example-an-x-y-controller\">Example: an X-Y controller</h2><p>We will demonstrate creating a simple position controller that only moves the object along the X and Y axis. Instead of maintaining our own X and Y animation values, we are going to use two Bezier float controllers, each responsible for one dimension. We will hold two references in our plug-in class, each making a reference to one of those Bezier float controllers. The following code snippet shows these two references as private data members in our plug-in class:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class SampleKFCtrl: public Control\n\
{\n\
private:\n\
\n\
    Control* mpXCtrl;\n\
    Control* mpYCtrl;\n\
\n\
    enum MyRefs \n\
    {\n\
        kXCtrlRef,\n\
        kYCtrlRef,\n\
        kNumRefs\n\
    };\n\
\n\
public:\n\
\n\
    ...\n\
};</code></pre><p>And we initiate them in the constructor as following:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">SampleKFCtrl::SampleKFCtrl()\n\
{\n\
    ReplaceReference(kXCtrlRef, NewDefaultFloatController());\n\
    ReplaceReference(kYCtrlRef, NewDefaultFloatController());\n\
}</code></pre><p>Now we have two references to two basic float controllers, and we will use them in the functions <code>Control::SetValue()</code> and <code>Control::GetValue()</code>. Our plug-in can be considered a medium between 3ds Max and our basic controllers:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void SampleKFCtrl::GetValue(TimeValue t, void *ptr, Interval &amp;valid, GetSetMethod method)\n\
{\n\
    Point3 p3OurAbsValue(0, 0, 0);\n\
    mpXCtrl-&gt;GetValue(t, &amp;p3OurAbsValue.x, valid, CTRL_ABSOLUTE);\n\
    mpYCtrl-&gt;GetValue(t, &amp;p3OurAbsValue.y, valid, CTRL_ABSOLUTE);\n\
\n\
    if (method == CTRL_ABSOLUTE)\n\
    {\n\
        Point3* p3InVal = (Point3*)ptr;\n\
        *p3InVal = p3OurAbsValue;\n\
    }\n\
    else // CTRL_RELATIVE\n\
    {\n\
        Matrix3* m3InVal = (Matrix3*)ptr;\n\
        m3InVal-&gt;PreTranslate(p3OurAbsValue);\n\
    }\n\
}\n\
\n\
void SampleKFCtrl::SetValue(TimeValue t, void *ptr, int commit, GetSetMethod method)\n\
{\n\
    Point3* p3Val = (Point3*)ptr;\n\
    mpXCtrl-&gt;SetValue(t, &amp;p3Val-&gt;x, commit, CTRL_ABSOLUTE);\n\
    mpYCtrl-&gt;SetValue(t, &amp;p3Val-&gt;y, commit, CTRL_ABSOLUTE);\n\
}</code></pre><p>There are two virtual functions without any default implementation that you need to implement in your plug-in or your plug-in will not compile. The first one is <code>Control::NotifyRefChanged()</code> that we introduced in Lesson 3 of the learning path. This function is called whenever a reference in our class is changed (for example, when the user modifies the curve of one of the float sub-controllers in the curve editor). The other virtual function is <code>Control::Copy()</code>, and is used to let the user smoothly assign our controller to an object that has already been animated by another controller. When the user assigns a different controller to the object in the track view, the function <code>Animatable::AssignController()</code> is invoked, which in turns calls the <code>Control::Copy()</code> in our plug-in so we get a chance to copy the keyframe values from the old controller. We might want to use at least the position of the object at the time zero so the object keeps its original coordinates.</p>\n\
<p>As indicated earlier, we are using two default float controllers that are assigned to our reference variables (<code>mpXCtrl</code> and <code>mpYCtrl</code>) in our constructor. We should remember from our previous lesson that we should use references to the objects derived from <code>ReferenceTarget</code> if we have a dependency on them. This holds true here and we use references and call <code>Control::ReplaceReference()</code> (which then calls <code>Control::setReference()</code> to assign the float controllers to our sub-controllers). We also need to implement the member function <code>Control::setReference()</code> for this purpose.</p>\n\
<p>Our plug-in at this point can handle the values for the position of the object it is assigned to. The user can use either auto key or set key animation mode to animate his objects using this controller. However, he still cannot visualize the individual sub-controller in track view curve editor, and neither can he modify the curves there. Remember that the curve editor plots one or more 2D curves, each demonstrating the value of one float parameter over time. As a result, the curve editor cannot plot an animated parameter unless it can be divided into a finite number of float sub-controllers. Although we are using float sub-controllers, 3ds Max has no way to know it unless we expose them as sub-animatables and implement the two functions <code>Control::NumSubs()</code> and <code>Control::SubAnim()</code>. The first function here returns the number of float sub-controllers in our animation controller (e.g. 2 for an XY position controller), and the second function should return a pointer to the float sub-controller whose index is given. These two functions are sufficient for the curve editor to display the controller. The following shows how these two functions can be implemented (do not forget to include the declaration in your header file):</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">int SampleKFCtrl::NumSubs() { \n\
    return 2; \n\
}\n\
\n\
Animatable* SampleKFCtrl::SubAnim(int n) { \n\
    return (n==0) \n\
        ? mpXCtrl \n\
        : mpYCtrl; \n\
}</code></pre><p>The project Lesson4b implements a key frame controller plug-in. The main class is <code>SampleKFCtrl</code> and it appears in 3ds Max with the name Sample Key Frame Controller.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_sdk_-_the_learning_path.html\">3ds Max SDK - The Learning Path</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";