var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"lesson_5_geometric_objects\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Lesson 5: Geometric Objects</h1></div><p>Geometric objects are one of the most important parts of 3ds Max, without which there cannot be a scene. In this lesson we will show how to create the geometry of an object using a mesh, and then we will show how to use mouse actions to create the object.</p>\n\
<p>Geometric object plug-ins are extended from the <code>GeomObject</code> class. However, they usually extend from a child class of <code>GeomObject</code> such as <code>SimpleObject2</code> that already has the implementation of some of the functions. We can extend from that class if our geometrical object has a deformable mesh. A mesh surface is composed of a set of faces, where each face is a flat surface in 3D space formed by three or more 3D points.</p>\n\
<p>In the project Lesson5a we define and implement a class named <code>SampleGObject</code> extended from SimpleObject2. The object created by our plug-in a geometry object with only 4 vertices and 3 faces. At this point we do not get involved with direct object creation using mouse commands because we want to focus on the mesh creation. To avoid mouse commands, we will use a utility plug-in that creates an instance of our geometry object once selected. We will present a more robust method to create geometric objects using mouse commands later in this lesson.</p>\n\
<p>The class <code>SampleGObject</code> has the following structure:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class SampleGObject : public SimpleObject2 \n\
{\n\
public:\n\
\n\
    SampleGObject() : objSize(2.0) { }\n\
\n\
    // Member variable\n\
    double objSize;\n\
\n\
    // From BaseObject\n\
    CreateMouseCallBack* GetCreateMouseCallBack() { return NULL; }\n\
\n\
    // From SimpleObject\n\
    void BuildMesh(TimeValue t);\n\
\n\
    //From Animatable\n\
    Class_ID ClassID() {return SampleGObject_CLASS_ID;}        \n\
    SClass_ID SuperClassID() { return GEOMOBJECT_CLASS_ID; }\n\
    void GetClassName(TSTR&amp; s) { s = &quot;Sample Geometric Object&quot;;}\n\
\n\
    void DeleteThis() { delete this; }        \n\
};</code></pre><p>We will explain the <code>SimpleObject2::GetCreateMouseCallBack()</code> later in this lesson. The focus here is on the function <code>SimpleObject::BuildMesh()</code> that is called to build the mesh representation of the object. The time input parameter is to implement animated objects, where the mesh parameters change over the time. We will cover that later in lesson 6 (parameter blocks). Any geometrical object should use a data member of type <code>Mesh</code> to store the built mesh. In our example, the parent class (<code>SimpleObject</code>) has this data member (<code>Mesh SimpleObject::mesh</code>) and we do not need to re-define it. The mesh in this example is composed of 4 vertices (each vertex is represented by a point3) and 3 faces. You can modify the number, coordinates and connection of the vertices in the above code to have your own geometrical object.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void SampleGObject::BuildMesh(TimeValue t)\n\
{\n\
    ivalid = FOREVER;\n\
    mesh.setNumVerts(4);\n\
    mesh.setNumFaces(3);\n\
    mesh.setVert(0,objSize*Point3(0.0,0.0,0.0)); \n\
    mesh.setVert(1,objSize*Point3(10.0,0.0,0.0)); \n\
    mesh.setVert(2,objSize*Point3(0.0,10.0,0.0)); \n\
    mesh.setVert(3,objSize*Point3(0.0,0.0,10.0)); \n\
\n\
    mesh.faces[0].setVerts(0, 1, 2);\n\
    mesh.faces[0].setEdgeVisFlags(1,1,0);\n\
    mesh.faces[0].setSmGroup(2);\n\
    mesh.faces[1].setVerts(3, 1, 0);\n\
    mesh.faces[1].setEdgeVisFlags(1,1,0);\n\
    mesh.faces[1].setSmGroup(2);\n\
    mesh.faces[2].setVerts(0, 2, 3);\n\
    mesh.faces[2].setEdgeVisFlags(1,1,0);\n\
    mesh.faces[2].setSmGroup(4);\n\
\n\
    mesh.InvalidateGeomCache();\n\
}</code></pre><p>As you can see in the sample code above, the <code>SimpleObject::BuildMesh()</code> needs to set the number of the vertices and faces of the object, set the coordinates of the vertices, and define the faces by setting which vertices they connect to. Note that the viewport will not be updated as long as all of the objects within are valid. That is why we are calling <code>mesh.InvalidateGeomCache()</code> at the end, so our object gets displayed in the viewport.</p>\n\
<p>Our geometrical object does not use the mouse commands to create the object yet. We utilize a size variable (<code>objSize</code>) in this example (Lesson5a) and initialize it in the plug-in constructor. For now, it will not be changed anywhere in the program and will only be used for generating the coordinates of the mesh&apos;s vertices.</p>\n\
<p>The last step is to have a utility plug-in that creates our geometry object and displays it in the viewport. This utility plug-in class is named Lesson5a and extends from <code>UtilityObj</code>. Once that utility plug-in is selected, in the <code>Lesson5a::BeginEditParams()</code> we create a new geometry object and assign it to a node in the viewport. The following code shows how it is done.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void Lesson5a::BeginEditParams(Interface* ip,IUtil* iu) \n\
{\n\
    ...\n\
\n\
    // Create an object \n\
    SampleGObject* myGeomObj = new SampleGObject();\n\
    INode* node = ip-&gt;CreateObjectNode(myGeomObj);\n\
    TimeValue t (0);\n\
    Matrix3 tm(1);\n\
    node-&gt;SetNodeTM(t,tm);\n\
}</code></pre><p>Note that the created object always appears at the origin of the viewport and with the fixed initial size. We also create our object by calling <code>new</code> on the <code>SampleGObject</code>. With this implementation the geometric object class is invisible from 3ds Max because there is no way for 3ds Max to know that this class exists. The only interface to your geometric object is the utility plug-in Lesson5a. As a result, there will be problems for loading and saving a scene where instances of <code>SampleGObject</code> exist. For a more stable implementation we will need to define the class descriptor (<code>ClassDesc2</code>) class for our geometric object and also change the functions in the DllEntry.cpp file to declare to the 3ds Max that there are 2 plug-in classes in this project (you can keep your geometric object plug-in hidden from user by returning <code>FALSE</code> from it&apos;s <code>&amp;lt;geometric_object_class_descriptor&amp;gt;::IsPublic()</code>. In order to let 3ds Max know that the utility plug-in is creating an instance of the geometric object, we will need to create it as follows, instead of simply calling the <code>new</code> operator:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">SampleGObject* myGeomObj = (SampleGObject*)ip-&gt;CreateInstance(GEOMOBJECT_CLASS_ID, SampleGObject_CLASS_ID); </code></pre><p>By using the above line of code, 3ds Max will have the necessary information to create and manage the geometric object. Our sole purpose in this section is to demonstrate how the <code>SimpleObject::BuildMesh()</code> function works. For information about the <code>Lesson5a::DlgProc()</code> function and to see how to create an object without using the mouse commands, refer to the topic <a href=\"./../writing_plug-ins/writing_simple_geometric_object_/object_creation_methods/dialog_based_creation_of_objects.html\">Dialog Based Creation of Objects</a> in the programmer&apos;s guide. In the next section we explain how to use mouse commands to create a geometric object.</p>\n\
<h2 id=\"reading-mouse-inputs\">Reading mouse inputs</h2><p>Our geometrical object is instantly created and displayed when the user selects the corresponding plug-in. However, users are used to create a geometrical object using mouse actions such as move, click and right click, and we implement a plug-in that does this in the Lesson5b project. 3ds Max receives these mouse actions and sends them to an object of a class named <code>CreateMouseCallBack</code> by calling the <code>CreateMouseCallBack::proc()</code> on that object. It is the responsibility of our plug-in to instantiate and keep a pointer as well as to destroy this object. We will need to implement the pure virtual class <code>CreateMouseCallBack</code> for this reason. Note that this class should have a pointer to our main plug-in class. This will allow it to pass the results of mouse actions to our plug-in. We also need to maintain at least two points in that class, one for keeping the coordinates of the previous point the mouse was pointing to, and one for the current point.</p>\n\
<p>The sequence of 3ds Max calls to the <code>CreateMouseCallBack::proc()</code> function will enable our plug-in to trace the user&apos;s mouse actions. This includes both the viewport coordinate and the screen coordinate of each mouse action, a message for specific action (e.g. button pressed, button released, etc.) and finally an incremental counter on each action. You can refer to the documentation of the class <code>CreateMouseCallBack</code> for more information. In this example we only consider the two values for the <code>msg</code> parameter in 3ds Max call on <code>proc()</code>: <code>MOUSE_POINT</code> and <code>MOUSE_MOVE</code>. The first one is sent whenever the user presses or releases the mouse button and the second one is sent whenever the user moves the mouse pointer. Although the <code>proc()</code> function is called over and over when the user is moving the mouse pointer during the object creation, the message and point parameters do not change until the next time user presses or releases the mouse button. A scenario for using this function is as follows:</p>\n\
<ol>\n\
<li><p>The user moves the mouse on the viewport when the geometrical object plug-in is active. The <code>proc()</code> function is not called.</p>\n\
</li>\n\
<li><p>The user presses and holds the button on some point in the viewport. 3ds Max calls the proc function with message = <code>MOUSE_POINT</code>, point = 0.</p>\n\
</li>\n\
<li><p>The user moves the mouse while the button is pressed. 3ds Max calls the proc function over and over with message = <code>MOUSE_MOVE</code>, point = 0.</p>\n\
</li>\n\
<li><p>The user releases the button. The proc function is called with message = <code>MOUSE_POINT</code>, point = 1 (note that point is incremented comparing to the previous <code>MOUSE_POINT</code> message).</p>\n\
</li>\n\
<li><p>The user moves the mouse again. The proc function is called multiple times with message = <code>MOUSE_MOVE</code>, point = 2 (again, point is incremented compared to the previous <code>MOUSE_MOVE</code> message).</p>\n\
</li>\n\
</ol>\n\
<p>3ds Max finishes the creation of the object when the <code>Proc</code> function returns <code>CREATE_STOP</code>.</p>\n\
<p>We will call <code>SimpleObject2::BuildMesh()</code> from <code>CreateMouseCallBack::proc()</code>. Object creation will use mouse commands, and the mesh will be constantly re-built during the object creation. Our implementation of the class <code>CreateMouseCallBack</code> and its <code>proc()</code> function looks like this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class SampleGObject2CreateCallBack : public CreateMouseCallBack \n\
{\n\
    IPoint2 sp0;          //First point in screen coordinates\n\
    SampleGObject2 *ob;   //Pointer to the object \n\
    Point3 p0;            //First point in world coordinates\n\
    Point3 p1;            //We added this point. Second point in world coordinates.\n\
public:    \n\
    int proc( ViewExp *vpt,int msg, int point, int flags, IPoint2 m, Matrix3&amp; mat);\n\
    void SetObj(SampleGObject2 *obj) {ob = obj;}\n\
};\n\
\n\
int SampleGObject2CreateCallBack::proc(ViewExp *vpt,int msg, int point, int flags, IPoint2 m, Matrix3&amp; mat )\n\
{\n\
    TimeValue t (0);\n\
    if (msg==MOUSE_POINT||msg==MOUSE_MOVE) {\n\
        switch(point) \n\
        {\n\
        case 0: // only happens with MOUSE_POINT msg\n\
            ob-&gt;suspendSnap = TRUE;\n\
            sp0 = m;\n\
            p0 = vpt-&gt;SnapPoint(m,m,NULL,SNAP_IN_PLANE);\n\
            mat.SetTrans(p0); // sets the pivot location\n\
            ob-&gt;objSize = 0.0;\n\
            break;\n\
        case 1:\n\
                                        {\n\
            ob-&gt;suspendSnap = TRUE; \n\
            p1 = vpt-&gt;SnapPoint(m,m,NULL,SNAP_IN_PLANE);\n\
            float speedFactor = 24.0f;\n\
            ob-&gt;objSize = (Length(p1 - p0) / speedFactor);\n\
            ob-&gt;mesh.InvalidateGeomCache();\n\
            ob-&gt;BuildMesh(t);\n\
            if (msg == 1)\n\
                return CREATE_STOP;\n\
            break;\n\
                                        }\n\
        case 2:\n\
            return CREATE_STOP;\n\
        }\n\
        ob-&gt;NotifyDependents(FOREVER, PART_ALL, REFMSG_CHANGE);\n\
    } \n\
    else {\n\
        if (msg == MOUSE_ABORT) return CREATE_ABORT;\n\
    }\n\
    return TRUE;\n\
}</code></pre><p>To link this class to your geometric object plug-in, a static object of this class is created:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static SampleGObject2CreateCallBack SampleGObject2CreateCB;</code></pre><p>The <code>GetCreateMouseCallBack()</code> function of your plug-in registers its address in this static object and returns the address of the static CreateMouseCallBack object. 3ds Max by default will call <code>&amp;lt;yourplugin&amp;gt;::GetCreateMouseCallBack()</code> to access this static object and to call it&apos;s <code>proc()</code> function. This way 3ds Max will read the mouse commands as you have defined to create the geometric object. Note that the <code>objSize</code> is changing in the <code>SampleGObject2CreateCallBack::proc()</code> now. Although the mesh vertices are updated in the call to <code>ob-&amp;gt;BuildMesh()</code>, we still need to call the <code>mesh.InvalidateGeomCache()</code> function to update the geometry of the mesh. Failing to do so will always show the object with the original size (zero, in this case) in the viewport.\n\
At this point the viewport still has no idea that the mesh has been changed. To inform the viewport of the update to the geometry, we need to call <code>ob-&amp;gt;NotifyDependents()</code>. This function sends a signal to all the plug-ins that have a reference to our geometrical object (including the viewport) so they will know it has been updated.</p>\n\
<p>One might expect the object to always appear at the origin because of the following line in the code:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">mesh.setVert(0,objSize*Point3(0.0,0.0,0.0)); </code></pre><p>The above line will return the point3 (0,0,0) no matter what the <code>objSize</code> is. However, it will not be the coordinates of the vertex zero because the pivot of the object is set in the <code>proc()</code> function:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">mat.SetTrans(p0);</code></pre><p>This will set the coordinates relative to the first point in the space user has clicked on.</p>\n\
<p>The geometric object in this lesson had only one parameter, <code>objSize</code> which does not change after the object is created, so the read and write actions can be easily done for that single parameter. However, we need to implement extra features to be able to perform 3ds Max standard actions including but not limited to undo actions, exposing the parameter to MAX Script, updating any dependent plug-ins, responding to changes to other entities in the scene, animating the parameters, or saving the plug-in to disk. There will also be more parameters as the plug-ins expand, so a standard flexible method of handling the parameters that supports all the above-mentioned actions is very useful. In the next lesson we will introduce parameter blocks which support all these actions and make it very easy to manage any number of parameters in your plug-ins.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_sdk_-_the_learning_path.html\">3ds Max SDK - The Learning Path</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";