var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"lesson_6_parameter_blocks\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Lesson 6: Parameter Blocks</h1></div><p>3ds Max refers to the data values associated with plug-ins as &apos;parameters&apos;. The size of a geometric object and the angle of a bend modifier are examples of parameters. 3ds Max has a powerful mechanism to manage plug-in parameters: instead of keeping them as individual data members, they are encapsulated in &quot;parameter blocks&quot; (objects of the class <code>IParamBlock2</code>). Parameter blocks provide a standard method for 3ds Max to read from and write to plug-in parameter values, independent of the plug-in type. They have the following advantages:</p>\n\
<ul>\n\
<li>They can automatically connect to Qt widgets so the user can observe or modify their values (see <a href=\"./../writing_plug-ins/using_qt_with_3ds_max_plug-ins/creating_a_qt-based_plug-in.html\">Creating a Qt-based plug-in</a> for more information).</li>\n\
<li>They automatically interpolate animated parameters and provide a value for any parameter at any valid time.</li>\n\
<li>They are reference targets, so the owner plug-in and other ReferenceMakers holding a reference to them are automatically notified of any change to parameters. Such a change can be caused by 3ds Max, another plug-in, or the user.</li>\n\
<li>They can serialize the data to handle plug-in saving and loading.</li>\n\
<li>They expose the plug-in parameters to MAXScript.</li>\n\
</ul>\n\
<p>The implementation of <code>SimpleObject2::SetReference()</code> and <code>SimpleObject2::GetReference()</code> is an example that shows how parameter blocks are treated as reference targets in the 3ds Max SDK. The class <code>SimpleObject2</code> is a reference maker class and has only one reference (<code>IParamBlock2* pblock2</code>), to its parameter block. The implementations of those two functions are:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">virtual void SimpleObject2::SetReference (int i, RefTargetHandle rtarg)[inline, protected, virtual] \n\
{\n\
    pblock2 = (IParamBlock2*)rtarg; \n\
    SimpleObject::SetReference(i, rtarg);\n\
}\n\
\n\
RefTargetHandle SimpleObject2::GetReference (int i)[inline, virtual] \n\
{\n\
    return (RefTargetHandle)pblock;\n\
}</code></pre><h2 id=\"what-you-need-to-know-about-parameter-blocks\">What you need to know about parameter blocks</h2><p>The number of parameter blocks per plug-in, as well as the number of parameters within a parameter block is not limited. The parameter blocks in a plug-in are identified by their ID, which is usually zero for the first parameter block and increments for the next ones.</p>\n\
<p>Developers do not directly call the parameter block&apos;s constructor to create the parameter blocks for their plug-in. Instead, they instantiate a static parameter block descriptor (a static object of type <code>ParamBlockDesc2</code>) by passing the appropriate arguments to its constructor for each parameter block. These arguments describe the parameters that you want to include in that parameter block. The constructor of <code>ParamBlockDesc2</code> automatically creates the described parameter block (an object of type <code>IParamBlock2</code>) and registers the address of the newly-created parameter block in a table in the plug-in&apos;s class descriptor. The following figure demonstrates this procedure for a geometric object plug-in named Widget. The Widget plug-in project exists in the &quot;how to&quot; folder of the 3ds Max SDK at  <em>&lt;3ds Max Installation folder&gt;\\maxsdk\\howto\\objects\\widget.</em> </p>\n\
<p><img src=\"./images/GUID-C48DAACA-FBF6-4E5F-8948-53EC572497A2-low.png\" alt></p>\n\
<p>The parameters in a parameter block are accessed using their index number or parameter ID&apos;s. For each of its parameters, the parameter block stores at least the parameter value, the parameter ID, an internal name (used by MAXScript), and the parameter type. There can also be additional data for a parameter, such as the default value, the range of the parameter, the UI type, and others. Developers must use the tags defined in the <code>ParamTags</code> enumeration in the call to the parameter block constructor if they want to include these additional data for a parameter. Declaring a parameter in a parameter block is primarily done in the constructor call of the parameter block descriptor, but it can also be done later on by calling the function <code>ParamBlockDesc2::AddParam()</code> as long as there have been no parameter blocks created using that descriptor object.</p>\n\
<p>Parameters within a parameter block can be reference types (references to <code>ReferenceTarget</code> objects), scalar types such as <code>TYPE_FLOAT</code>, aggregated types such as <code>Point3</code> or <code>Matrix3</code>, external assets to the scenes such as file names and so on. Refer to the documentation for <code>ParamType2</code> for a complete list of valid parameter types. If a parameter is a <a href=\"./../3ds_max_sdk_features/animation_time_and_controllers/controllers.html\">Controllers</a>, a default animation controller is automatically assigned to it and the developer can change it later. If animated, you have to provide a value-time pair when you want to write to the parameters (using <code>IParamBlock::SetValue()</code>) and provide a valid time when you want to read from them (using <code>IParamBlock::GetValue()</code>). You can access a non-animated parameter using the current time. The next section describes in detail how a parameter block can be created and accessed.</p>\n\
<h2 id=\"creating-a-parameter-block-for-your-plug-in\">Creating a parameter block for your plug-in</h2><p>Suppose that your plug-in needs to have two float parameters, <code>p0</code> and <code>p1</code>. You will need to instantiate an object of the class <code>ParamBlockDesc2</code> which will describe your parameter block. You will also need to have an <code>IParamBlock2*</code> in your plug-in class. 3ds Max will automatically create the parameter block based on the arguments you provide in your call to the <code>ParamBlockDesc2</code> constructor if you set the <code>P_AUTO_CONSTRUCT</code> flag there. The following is the minimum code you need for this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static ParamBlockDesc2 myPBDesc (\n\
    //Mandatory data\n\
    // ---------------------------------------------------\n\
    0,                   /* The parameter block ID. */\n\
    _T(&quot;myParamBlock&quot;),  /* The internal name exposed to MaxScript for this parameter. */\n\
    0,                   /* The resource ID of a particular string in the string table */\n\
                         /* that will be displayed in the track view for the parameter. */\n\
    &amp;myPluginDesc,       /* The address of your plug-in descriptor (ClassDesc2 object). */\n\
\n\
    //flags\n\
    P_AUTO_CONSTRUCT     /* This flag makes 3ds Max automatically create the */\n\
                         /* parameter block from this parameter block descriptor object*/\n\
\n\
    //Conditional data, depends on the flags defined above\n\
    // ---------------------------------------------------\n\
    0,                   /* Required because P_AUTO_CONSTRUCT was flagged. This means that this */\n\
                         /* parameter block will be the first reference target of your plug-in. */\n\
\n\
    //Optional data - parameters definition starts here\n\
    // ---------------------------------------------------\n\
    0,                   /* Parameter ID. We are defining the first parameter here */\n\
    _T(&quot;p0&quot;),            /* Internal name of the parameter */\n\
    TYPE_FLOAT,          /* Parameter Type. It will be a float parameter */\n\
    P_ANIMATABLE,        /* A constant defined in iparamb2.h. Indicates that the parameter is animatable.*/\n\
    p_end,               /* End of the first parameter definition.*/\n\
    1,                   /* Parameter ID. This will be the second parameter*/\n\
    _T(&quot;p1&quot;),            /* Internal name of the parameter*/\n\
    TYPE_FLOAT,          /* Parameter Type. It will be a float parameter*/\n\
    P_ANIMATABLE,        /* A constant defined in iparamb2.h. Indicates that the parameter is animatable.*/\n\
    p_end,               /* End of the second parameter definition. &apos;end&apos; is an enumerated value defined in \n\
                         /* the &apos;ParamTags&apos; enumerator.*/\n\
\n\
    p_end                 /*End of parameter definition*/\n\
    );</code></pre><p>The above code calls the constructor of <code>ParamBlockDesc2</code> to create the static object <code>myPBDesc</code>. The address of the static <code>ClassDesc2</code> object in your plug-in that is provided in the list of parameters (<code>&amp;myPluginDesc</code>) is used to register this parameter block descriptor in the static <code>ClassDesc2</code> object . By the time the static <code>ClassDesc2</code> object calls its <code>ClassDesc2::Create()</code> function, it has the address of all of its parameter blocks and also knows to which reference in the plug-in each parameter block must be assigned. However, this is not done automatically. It is the responsibility of the plug-in developer to assign the addresses of the parameter blocks (registered in the static <code>ClassDesc2</code> object) to its <code>IParamBlock2</code> pointers as references. This can be done in the constructor of your plug-in as shown below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MyPlugin::MyPlugin()\n\
{\n\
    myPluginDesc.MakeAutoParamBlocks(this);\n\
}</code></pre><p>The function <code>ClassDesc2::MakeAutoParamBlock()</code> uses the address of the parameter blocks as well as the reference index in the plug-in to set them as the reference targets in the plug-in (remember that the class <code>IParamBlock2</code> is extended from the class <code>ReferenceTarget</code>).</p>\n\
<p>After calling <code>ClassDesc2::MakeAutoParamBlock()</code> in the constructor of your plug-in, you can set a parameter (for example, p1) at a specific time (for example, <code>Interval&amp; thisTime</code>) for a plug-in instance (for example, <code>myPluginObj</code>) as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">myPluginObj.pblock2-&gt;SetValue(1, thisTime, 0.0f);</code></pre><p>You can also read the value and validity interval of a parameter using <code>IParamBlock2::GetValue()</code> as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">float&amp; f;\n\
Interval&amp; ivalid;\n\
myPluginObj.pblock2-&gt;GetValue(1, getCOREInterface-&gt;GetTime(), f, ivalid );</code></pre><p>Note that a controller of the appropriate type is automatically assigned to each animatable parameter in the parameter block.</p>\n\
<h3 id=\"user-interface\">User Interface</h3><p>While you can write to and read from the parameters <code>p0</code> and <code>p1</code> in the above code, there is no user interface yet to let the user observe or modify them. Using parameter blocks, it is very easy to create UI panels for parameters. There are two ways to do this: using a Qt UI, or using the legacy Win32 UI method.</p>\n\
<blockquote>\n\
<p><strong>Note</strong>: Although many of the sample projects included in the 3ds Max SDK still use the legacy Win32 UI method, and the Plug-in Wizard does not support Qt for all project types yet, Qt is now the recommended way of building user interfaces for 3ds Max plug-ins.  </p>\n\
</blockquote>\n\
<p>All you need to do is set the <code>P_AUTO_UI_QT</code> flag in the <code>ParamBlockDesc2</code> constructor, and ensure the name of each parameter matches the name of a widget in the associated Qt UI.  See <a href=\"./../writing_plug-ins/using_qt_with_3ds_max_plug-ins.html\">Using Qt with 3ds Max Plug-ins</a> for more details.</p>\n\
<p>Using the legacy Win32 method, you set the <code>P_AUTO_UI</code> flag in the <code>ParamBlockDesc2</code> constructor, link to a rollout window you have previously defined in your  <em>.rc</em>  file, and link each parameter to that rollout window&apos;s components. The two latter links are created by using appropriate flags from the <code>ControlType</code> and <code>EditSpinnerType</code> enumerations in your constructor call.</p>\n\
<p>The following example (copied from the howto <em>QtObjectDemo</em> proejct) demonstrates the code to create a more complex parameter block with several parameters of different types and automaticly connect them to Qt widgets. The <a href=\"./../3ds_max_sdk_features/plug-ins/parameter_blocks/creating/parameter_specification_argument/parameter_tags.html\">Parameter Tags</a> used in the constructor define a default value and a range for each parameter.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// enums for various parameters\n\
enum {\n\
    pb_float, pb_int, pb_bool, pb_texmap, pb_point2, pb_point3, pb_point4, pb_matrix3\n\
};\n\
...\n\
static ParamBlockDesc2 qt_param_blk ( \n\
    // Required arguments\n\
    qt_params, _T(&quot;Qt_params&quot;),  0, GetQtObjectDemoDesc(), \n\
    // flags\n\
    P_AUTO_CONSTRUCT + P_AUTO_UI_QT, \n\
    //Dependent arguments ---------------------------     \n\
    &#x200B;//required because P_AUTO_CONSTRUCT was flagged\n\
    PBLOCK_REF, \n\
    &#x200B;//Parameter Specifications ----------------------\n\
    // For each control create a parameter:\n\
    // size control:\n\
    pb_float,             _T(&quot;qfloatspinbox&quot;),         TYPE_FLOAT,     P_ANIMATABLE, IDS_NUMVERTS, \n\
        p_default,         10.0f, \n\
        p_range,         0.1f,1000.0f, \n\
        p_end,    \n\
    // # of vertices control:\n\
    pb_int,    _T(&quot;dial&quot;), TYPE_INT, P_ANIMATABLE, IDS_NUMVERTS,\n\
        p_default, 10,\n\
        p_range, 4, 100,\n\
        p_end,\n\
    // Raise center control:\n\
    pb_bool, _T(&quot;qcheckbox&quot;), TYPE_BOOL, P_ANIMATABLE, IDS_NUMVERTS, \n\
        p_default, FALSE,\n\
        p_end,\n\
    // Texmap control:\n\
    pb_texmap, _T(&quot;qbutton&quot;), TYPE_TEXMAP, 0, IDS_NUMVERTS, \n\
        p_subtexno, 0,\n\
        p_end,    \n\
    // A point3 spinner control:\n\
    pb_point3, _T(&quot;qspinPoint3&quot;), TYPE_POINT3, P_ANIMATABLE, IDS_NUMVERTS, \n\
        p_default, Point3(0.0f,0.0f,0.0f),\n\
        p_end,\n\
    // A point4 spinner control:\n\
    pb_point4, _T(&quot;qspinPoint4&quot;), TYPE_POINT4, P_ANIMATABLE, IDS_NUMVERTS, \n\
        p_default, Point4(0.0f, 0.0f, 0.0f, 0.0f),\n\
        p_end,\n\
    // A matrix3 control:\n\
    pb_matrix3, _T(&quot;qspinMatrix3&quot;), TYPE_MATRIX3, P_ANIMATABLE, IDS_NUMVERTS, \n\
        p_default, Matrix3(true),\n\
        p_end,\n\
\n\
    p_end\n\
    );</code></pre><p>Each parameter name (such as &quot;qfloatspinbox&quot;) matches the name of a widget in the associated UI.   See <del>{ Qt in Parameter Maps and Parameter Blocks }</del> for a description of the P_AUTO_UI_QT flag, and the supported widgets for various parameter types.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_sdk_-_the_learning_path.html\">3ds Max SDK - The Learning Path</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";