var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"lesson_7_writing_dotnet_plug-ins\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Lesson 7: Writing .Net Plug-ins</h1></div><p>3ds Max uses its integrated .NET assembly loaders at run time to load any valid .Net assembly located in  <em>[3ds Max installation folder]\\bin\\assemblies</em>  that implements a 3ds Max plug-in. There are different DLL libraries providing the necessary APIs for your .NET plug-ins to use the 3ds Max SDK. The important libraries for this purpose are :</p>\n\
<ul>\n\
<li><em>Autodesk.max.dll</em>  - provides a comprehensive library that wraps almost all the features of the 3ds Max C++ SDK.</li>\n\
<li><em>UiViewModel.dll</em>  - contains the classes required to create custom user actions for CUI</li>\n\
</ul>\n\
<p>To write .Net plug-ins, you create a new C# class library project in Visual Studio. Once the new project is created, you can select <em>Add Reference</em> from the <em>Project</em> menu and browse for  <em>UiViewModel.dll</em> , and / or  <em>Autodesk.Max.dll</em>  in the 3ds Max installation folder, depending on what functionality you want to implement.</p>\n\
<h2 id=\"writing-net-action-plug-ins-using-uiviewmodel-dll\">Writing .Net Action plug-ins using UiViewModel.dll</h2><p>To create .NET Action plug-ins using  <em>UiViewModel.dll</em> , your plug-in class must implement <code>CuiActionCommandAdapter</code>. You can automatically implement an abstract class in Visual Studio (if you are using C#) by right clicking on the class name and selecting &quot;Implement Abstract Class&quot;. Doing so for <code>CuiActionCommandAdapter</code> will create stubs for four properties and one function for you. The properties provide basic information about your CUI action plug-in. The function is named <code>Execute()</code> and implements the actual functionality of your plug-in.</p>\n\
<p>Any plug-in class extended from <code>CuiActionCommandAdapter</code> can be independently compiled and loaded in 3ds Max. We will see later that it is not the case for plug-ins implementing the interfaces from  <em>Autodesk.Max.dll</em> . The code samples on <a href=\"./../3ds_max_dotnet_sdk/dotnet_action_items.html\">.NET Action Items</a> provide an example of a simple .Net Action plug-in that prompts a message in a pop up window.</p>\n\
<h2 id=\"writing-net-plug-ins-using-autodesk-max-dll\">Writing .Net plug-ins using Autodesk.max.dll</h2><p>Plug-in developers can use the interfaces defined in <code>Autodesk.max.dll</code> to implement any type of plug-in they need. This is similar to extending from a 3ds Max base class for a C++ plug-in. There is almost a one to one mapping between 3ds Max C++ SDK classes and the interfaces available in  <em>Autodesk.Max.dll</em> . We will create a utility plug-in as a basic example. At the beginning, your code will look like this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">using System;\n\
using Autodesk.Max;\n\
\n\
namespace firstDotNet\n\
{\n\
    public class utilityDotNet : Autodesk.Max.Plugins.UtilityObj\n\
    {        \n\
    }\n\
}</code></pre><p>Using Visual Studio&apos;s &quot;Implement Abstract Class&quot; option for a class extending <code>Autodesk.Max.Plugins.UtilityObj</code> will create two function stubs for you to implement, <code>BeginEditParams()</code> and <code>EndEditParams()</code>. These two functions are explained in <a href=\"lesson_1_sample_utility_plug-in.html\"\">Lesson 1: Sample utility plug-in</a> of the 3ds Max SDK learning path. Similar to the example there, we get this utility plug-in to send some text to the prompt area of 3ds Max. The following code shows what your code looks like after this step:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">using System;\n\
using Autodesk.Max;\n\
\n\
namespace firstDotNet\n\
{\n\
    public class utilityDotNet : Autodesk.Max.Plugins.UtilityObj\n\
    {\n\
        public override void BeginEditParams(IInterface ip, IIUtil iu)\n\
        {\n\
            ip.PushPrompt(&quot;Hello Dot Net!&quot;);\n\
        }\n\
        public override void EndEditParams(IInterface ip, IIUtil iu)\n\
        {\n\
            ip.PopPrompt();\n\
        }\n\
    }\n\
}</code></pre><p>.Net Plug-ins also need class descriptors, similar to unmanaged (C++) plug-ins. Class descriptors are explained in <a href=\"getting_started_writing_plug-ins.html\"\">Getting Started Writing Plug-ins</a>. You can define your own class descriptor that extends from <code>Autodesk.Max.Plugins.ClassDesc2</code>. The descriptor class returns an object of your plug-in class in its <code>Create()</code> function. You can right click on the class descriptor declaration name and select <em>Implement Abstract Class</em> to create stubs for the functions you have to implement.</p>\n\
<p>You do not need to explicitly define a constructor for your plug-in class since your call in <code>Descriptor::Create()</code> does not take any arguments. Your plug-in project can be compiled at this point, but 3ds Max still cannot recognize it even after placing the .dll file in <code>\\[3ds max installation folder]\\\\bin\\\\assemblies</code> and restarting 3ds Max. As explained in <a href=\"./../3ds_max_dotnet_sdk/loading_3ds_max_dotnet_assemblies.html\">Loading 3ds Max .NET Assemblies</a>, you will need to register your plug-in with 3ds Max by exposing a public static function named <code>AssemblyMain()</code>. You can implement this function in any public class in your plug-in&apos;s namespace, including the main plug-in class and the class descriptor. We create a new public class to implement this and other assembly functions for more clarity. Similar to <code>AssemblyMain()</code>, the public static function <code>AssemblyShutdown()</code> is used to perform the required tasks (if any) when your plug-in is being terminated. Adding the following public class to the namespace will make your plug-in be loaded by the .NET Assembly Loader when 3ds Max is starting up:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">public static class AssemblyFunctions\n\
    {\n\
        public static void AssemblyMain()\n\
        {\n\
            var g = Autodesk.Max.GlobalInterface.Instance;\n\
            var i = g.COREInterface13;\n\
            i.AddClass(new Descriptor(g));\n\
        }\n\
\n\
        public static void AssemblyShutdown()\n\
        {\n\
        }\n\
    }</code></pre><p>The .NET project Lesson7.zip provides different examples of using the  <em>Autodesk.Max.dll</em>  assembly along with CUI action items to create .NET plug-ins.</p>\n\
<h3 id=\"interfaces-instead-of-classes-properties-instead-of-functions\">Interfaces instead of Classes, Properties instead of Functions</h3><p>We can see from the above code that 3ds Max includes a handle to its global interface in its call to the <code>BeginEditParams()</code>. We use that interface handle to call the <code>PushPrompt()</code> function. We can also obtain the 3ds Max global interface by defining our own <code>IGlobal</code> Interface as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">IGlobal global = Autodesk.Max.GlobalInterface.Instance;\n\
Interface13 Interface = global.COREInterface13;\n\
global.PushPrompt(&quot;...&quot;);</code></pre><p>The equivalent code in 3ds Max C++ SDK will be:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Interface* ip = GetCOREInterface();\n\
ip-&gt;PushPrompt(&quot;...&quot;);</code></pre><p>By comparing the two pieces of code above, we can see two differences that can be generalized as two rules of thumb:</p>\n\
<ul>\n\
<li><code>COREInterface13</code> is a &apos;property&apos; of the Global interface, while its equivalent in <a href=\"./../writing_plug-ins/plug-in_basics/accessing_core_3ds_max_functiona.html\">Accessing Core 3ds Max Functionality</a>, <code>GetCOREInterface()</code> is a global &apos;function&apos;. In general, all the accessor functions in the 3ds Max SDK with names that start with &apos;Get&apos; and have no argument have equivalent properties in  <em>Autodesk.Max.dll</em> .</li>\n\
<li>The type of the parameter <code>ip</code> in the .Net version is an &apos;interface&apos; named <code>IInterface</code>, as opposed to the equivalent parameter in the 3ds Max C++ SDK where the type is a pointer to a &apos;class&apos; named Interface. In  <em>Autodesk.Max.dll</em>  almost every class has a corresponding interface which is used instead where the class might be expected as an argument type or return type. Remember that &quot;if a method requires an interface as an argument, then any object that implements that interface can be used in the argument&quot;.</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_sdk_-_the_learning_path.html\">3ds Max SDK - The Learning Path</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";