var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"3dxi_controllers_igamecontrol\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>3DXI Controllers: IGameControl</h1></div><p><code>IGameControl</code> provides direct access to standard key frame controllers in 3ds Max. This is particularly useful to game developers, because they know what type of controller they are exporting as defined by their game engine.</p>\n\
<p>To export key frame controllers using the <em>3ds Max SDK</em> requires asking for the position controller, checking if it has the required Key interface, and then extracting the key based on the control type. 3DXI handles this automatically, allowing the developer to simply request a list of bezier position keys for export.</p>\n\
<p>Along with the basic Key frame controllers, <code>IGameControl</code> can sample a controller. This is important for certain controllers that do not support the Key interfaces, such as biped. Each <code>IGameControl</code> interface maintains a control type which allows the developer to determine the control type used by <code>IGameControl</code>, based on the transform style. For example: if the Control is an Unsupported controller or biped, then the developer can sample it at the required rate. <em>3DXI</em> provides two types of sampling: Full and Quick. Full samples the animation over the entire animation range at the specified sampling rate set by the developer. Quick samples only the controller where a key exists. Quick has limitations: it will not work for <code>IGAME_TM</code> sampling or controllers that do not support key setting.</p>\n\
<p><code>IGameControl::Get...Keys()</code> methods return a success flag as a boolean, allowing simple iteration through the key methods. Thus, if all return <code>FALSE</code>, no keys are found. Sampling can then be requested.</p>\n\
<p>The following table lists <code>IGameControl</code> types and their associated meanings:</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th></th>\n\
<th></th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td>IGame Controller Type</td>\n\
<td>Max Controller</td>\n\
</tr>\n\
<tr>\n\
<td>IGAME_UNKNOWN</td>\n\
<td>Procedural Controller / 3rd Party</td>\n\
</tr>\n\
<tr>\n\
<td>IGAME_MAXSTD</td>\n\
<td>Linear, TCB, Bezier</td>\n\
</tr>\n\
<tr>\n\
<td>IGAME_BIPED</td>\n\
<td>Biped Controller</td>\n\
</tr>\n\
<tr>\n\
<td>IGAME_EULER</td>\n\
<td>Euler Controller</td>\n\
</tr>\n\
<tr>\n\
<td>IGAME_ROT_CONSTRAINT</td>\n\
<td>Orientation and Look At Constraint</td>\n\
</tr>\n\
<tr>\n\
<td>IGAME_POS_CONSTRAINT</td>\n\
<td>Path and Position Constraint.</td>\n\
</tr>\n\
<tr>\n\
<td>IGAME_LINK_CONSTRAINT</td>\n\
<td>A Matrix3 TM constraint</td>\n\
</tr>\n\
<tr>\n\
<td>IGAME_LIST</td>\n\
<td>A List Controller</td>\n\
</tr>\n\
<tr>\n\
<td>IGAME_INDEPENDENT</td>\n\
<td>Independent Position controller</td>\n\
</tr>\n\
<tr>\n\
<td>IGAME_MASTER</td>\n\
<td>Master point controller</td>\n\
</tr>\n\
</tbody></table>\n\
<p>Constraints are also supported directly by <code>IGameControl</code>. Like the skin access, all constraints are supported through a unified interface. The constraint is identified as a controller type. If found, the developer simply needs to obtain the constraint interface for that particular controller type. The following code is an example of this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">if(exportControllers)\n\
{\n\
    IGameKeyTab posKeys,rotKeys,\n\
    IGameControl * pGC = node-&gt;GetIGameControl();\n\
    DWORD T = pGC-&gt;GetControlType( IGAME_POS);\n\
    //position\n\
    if(T==IGAME_MAXSTD &amp;&amp; pGC-&gt;GetBezierKeys(poskeys, IGAME_POS))\n\
        DumpBezierKeys( IGAME_POS,poskeys,prsNode);\n\
    else if(T==IGAME_POS_CONSTRAINT &amp;&amp; !forceSample)\n\
    {\n\
         IGameConstraint * cnst = pGC-&gt;GetConstraint( IGAME_POS);\n\
        DumpConstraints(prsNode,cnst);\n\
    }\n\
//rotation\n\
    else if(T==IGAME_MAXSTD &amp;&amp; pGC-&gt;GetTCBKeys(rotkeys, IGAME_ROT)\n\
    {\n\
        DumpTCBKeys( IGAME_ROT,rotkeys,prsNode);\n\
    }\n\
    else if(T==IGAME_ROT_CONSTRAINT &amp;&amp; !forceSample)\n\
    {\n\
         IGameConstraint * cnst = pGC-&gt;GetConstraint( IGAME_ROT);\n\
        DumpConstraints(prsNode,cnst);\n\
    }\n\
    else\n\
        DumpSampleKeys(child,prsNode); // usually Biped\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3dxi.html\">3DXI</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";