var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"3dxi_initialization\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>3DXI Initialization</h1></div><p>Before you can begin using 3DXI, it must be initialized by obtaining a pointer to the main <code>IGameScene</code>. The following code snippet is an example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">IGameScene * pIgame = GetIGameInterface();</code></pre><p>You can also specify the coordinate space during initialization.</p>\n\
<p>3DXI uses a system where the developer can define such coordinate system standards such as 3ds Max, OpenGL and DirectX. By default, the conversion manager defaults to the 3ds Max system which is a right-Handed system with Z-up and Y into the screen.</p>\n\
<p>The developer can also define a custom system with the ability to set the handedness and principle axis orientation. The conversion manager ensures that all matrix and coordinate data is returned in the correct format without the need for further processing. The following code sets-up the conversion manager with the internal database ready to be used with data being converted to a DirectX format. It is now a simple case of enumerating the <code>IGameNode</code>s and extracting the data you need:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// pIGame is an IGameScene pointer\n\
\n\
GameConversionManager * cm = GetConversionManager();\n\
cm-&gt;SetCoordSystem(IGameConversionManager::IGAME_D3D);\n\
Igame-&gt;InitialiseIGame(true); // true- we want selected only\n\
Igame-&gt;SetStaticFrame(0);</code></pre><p>The following code snippet show how to setup a user defined coordinate system:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">UserCoord WhackySystem = {\n\
    1, //Right Handed\n\
    1, //X axis goes right\n\
    4, //Y Axis goes in\n\
    3, //Z Axis goes down.\n\
    0, //U Tex axis is left\n\
    1, //V Tex axis is Down\n\
};\n\
\n\
IGameConversionManager * cm = GetConversionManager();\n\
cm-&gt;SetUserCoordSystem(WhackySystem);</code></pre><p>You can also initialize 3DXI based on a set of <code>INode</code>s. If you do not wish to gather data from every node in the 3ds Max scene, you can initialize 3DXI based on a single node or a list of nodes by supplying the relevant <code>INode</code>(s). This provides you with an <code>IGameScene</code>, maintaining pointers only to the specified nodes. The following is an example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Tab &lt; INode *&gt; interestingNodes;\n\
Interface * ip = GetCOREInterface();\n\
\n\
//use the 3ds Max SDL to provide access to the selected nodes.\n\
INode* node;\n\
for(int i=0;i&lt;ip-&gt;GetSelNodeCount();i++)\n\
{\n\
    node = ip-&gt;GetSelNode(i);\n\
    interestingNodes.Append(1,&amp;node);\n\
}\n\
\n\
//pIGame is an IGameScene pointer\n\
IGameScene * scene = pIgame-&gt;InitialiseIGame(interestingNodes,false);\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3dxi.html\">3DXI</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";