var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"3dxi_mesh\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>3DXI Mesh</h1></div><p>The <code>IGameMesh</code> object provides complete access to all the mesh relevant data including normals, mapping channels, tangents and binormals, vertex information, and texture coordinates. It also takes care of such things as mirroring, which is again something that can cause unexpected results when exporting as the normals will appear to be flipped.</p>\n\
<p>Once you have an <code>IGameMesh</code> pointer, you can use its various methods to extract the data. Most face data (except mapping channel data) is provided in an extended face class called <code>FaceEx</code>. This is an extension to the 3ds Max class <code>Face</code> and includes additional indexing for vertex color, alpha, illumination and normals. Mapping channels are dealt with separately.</p>\n\
<p>The following example shows how to extract the vertices and face data of the given mesh. The other data types are extracted in a similar way.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">int numVerts = gm-&gt;GetNumberOfVerts();\n\
for(int i = 0;i&lt;numVerts;i++)\n\
{\n\
    TSTR data;\n\
    Point3 v;\n\
    if(gm-&gt;GetVertex(i,v))\n\
    {\n\
        //use the data here\n\
    }\n\
}\n\
\n\
int numFaces = gm-&gt;GetNumberOfFaces();\n\
for(int f=0;f&lt;numFaces;f++)\n\
{\n\
    FaceEx * face = gm-&gt;GetFace(f);\n\
    // use data here\n\
} </code></pre><p><code>IGameMesh</code> also provides access to <code>Face</code> data based on smoothing groups. For any particular smoothing group, a list of faces can be returned. This also applies to faces on a per material ID basis, with material lookup into the global material list based on the material ID from the face. This type of access is important for real-time display of the data as hardware has no concept of a multi-material, so the mesh needs to be split into smaller chunks based on the material at the face.</p>\n\
<p>Mapping data found on the object is collected and stored. You simply query which channels have data and then access them in a similar manner to other data channels. Typically, you call <code>GetMapFace()</code> and <code>GetMapFaceIndex()</code> to access all the data. The following code is an example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">//gm is a pointer to IGameMesh\n\
if(exportMappingChannel)\n\
{\n\
    Tab&lt;int&gt; mapNums = gm-&gt;GetActiveMapChannelNum();\n\
    int mapCount = mapNums.Count();\n\
    for(int i=0;i &lt; mapCount;i++)\n\
        int vCount = gm-&gt;GetNumberOfMapVerts(mapNums[i]);\n\
    buf.printf(&quot;%d&quot;,vCount);\n\
    for(int j=0;j&lt;vCount;j++)\n\
    {\n\
        vert = NULL;\n\
        Point3 v;\n\
        if(gm-&gt;GetMapVertex(mapNums[i],j,v))\n\
        {\n\
             //use data here\n\
        }\n\
    }\n\
    int fCount = gm-&gt;GetNumberOfFaces();\n\
    for(int k=0;k&lt;fCount;k++)\n\
    {\n\
        DWORD v[3];\n\
        gm-&gt;GetMapFaceIndex(mapNums[i],k,v);\n\
        //use data here\n\
    }\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3dxi.html\">3DXI</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";