var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"3dxi_property_containers\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>3DXI Property Containers</h1></div><p>Each <code>IExportEntity</code> stored in the 3DXI database supports <code>IPropertyContainer</code>. This interface is responsible for finding the properties on the entity. It loops through the various places where parameter blocks can be stored, gathering custom object data and custom attributes, if applicable. Developers should request this interface if they want to access any plug-in specific parameters.</p>\n\
<p>Generic 3DXI objects (<code>IGameGenObject</code>) also provide access to IPropertyContainer using the function <code>IGameGenObject::GetIPropertyContainer()</code>.</p>\n\
<p><code>IPropertyContainer</code> provides two ways to extract parameters from parameter blocks maintained by plugins, or <code>IGameObject</code>:</p>\n\
<ul>\n\
<li>Query by Name - This method allows the developer to extract all properties defined in ParamBlock, custom attributes, custom object data, and developer-defined plugin data directly and without enumeration.</li>\n\
<li>Simple Enumeration - This method allows direct handling of the ParamBlock, including read/write, and is also the quickest way to enumerate the properties of entities.</li>\n\
</ul>\n\
<p>Both these methods supply individual parameter data wrapped in in <code>IGameProperty</code>. Which method to follow depends on the level of control and the amount of data to extract.</p>\n\
<p>For example: the query-by-name route supplies all parameter block parameters, as well as custom attributes and custom object data that is assigned in the Object Properties dialog of a 3ds Max object. Custom attributes and MAXScript scripted plugins are also supported in the manner. Developers can specify the name of the file to be used when initializing the game export interfaces, thus allowing specific data to be maintained.</p>\n\
<h2 id=\"query-by-name\">Query by Name</h2><p>All properties are collected and accessed by name. The name parameter is corresponding to the MAXScript property name of the object. It provides easy way to identify data available. In Scripting Listener object properties can be listed with <code>ShowProperties()</code> call permitting the developer to query the property by name.</p>\n\
<p>Given a pointer to an <code>IPropertyContainer</code> interface a developer could write a wrapper function as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">IPropertyContainer * pc = material-&gt;GetIPropertyContainer ();\n\
IGameProperty * prop = pc-&gt;QueryPropery(_T(&quot;Ambient&quot;));</code></pre><p>The important aspect of the XML-based extraction is that custom object data can be exported and represented as an <code>IGameProperty</code>. Once the data is defined, it can be accessed in the exact same way as any other property.</p>\n\
<p>Custom object data can be seen in the following listing:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">- &lt;ExportUserData&gt;\n\
- &lt;!--\n\
 Accessto the Node User Data\n\
 - &lt;!--\n\
 Theid mustbe uniqueto the file\n\
- &lt;UserProperty&gt;\n\
&lt;id&gt;101&lt;/id&gt;\n\
&lt;simplename&gt;IGameTestString&lt;/simplename&gt;\n\
  &lt;keyName&gt;IGameTestString&lt;/keyName&gt;\n\
  &lt;type&gt;string&lt;/type&gt;\n\
  &lt;/UserProperty&gt;\n\
  &lt;/ExportUserData&gt;</code></pre><p>Note that the only difference between an <code>IGameProperty</code> object returning a User Property is that the controller interface will be NULL.</p>\n\
<h2 id=\"simple-enumeration\">Simple Enumeration</h2><p>Any <code>IExportEntity</code> can have its parameters enumerated. The developer simply registers a callback for the <code>IPropertyContainer</code> to use, and then every parameter-block based parameter found will be returned to the developer through the callback.</p>\n\
<p>The 3DXI export sample (<code>\\\\samples</code>\\igame\\export) uses this technique to export the names of every property found on an object. This includes modifiers, materials or anything that supports the <code>IPropertyContainer</code> interface. The following code sample shows how to setup the callback.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class MyProc : public PropertyEnum\n\
{\n\
    DumpProperties * dp;\n\
\n\
 public:\n\
    MyProc(DumpProperties * d) {dp = d;}\n\
    bool Proc(IGameProperty * prop)\n\
    {\n\
        TCHAR Buf[256];\n\
        _tcscpy(Buf,prop-&gt;GetName());\n\
\n\
        // if it is null, this is usually because its a IParamBlock and it is\n\
        // not animated, thus No name\n\
        if(_tcscmp(Buf, _T(&quot;&quot;))!=0){\n\
             //use the Property here\n\
        }\n\
        return false; //lets keep going\n\
    }\n\
};</code></pre><p>The callback is set using the following code where material is a pointer to an <code>IGameMaterial</code> object.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">IPropertyContainer *pCont = material-&gt;GetIPropertyContainer();\n\
MyProc * proccy = newMyProc(this);\n\
pCont-&gt;EnumerateProperties(proccy);</code></pre><p>This method will provide access to every parameter that was found during the initialization process. The technique is useful if you want to access the underlying parameter block for any parameter and thus provide read and write access to the data. This is also the quickest way to obtain parameters from the various entities. The developer is responsible for accessing the parameter block in the correct manner using the main 3ds Max SDK if they decide to use the kind of low level access.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3dxi.html\">3DXI</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";