var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"animatable_handles\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Animatable Handles</h1></div><p>The <code>AnimHandle</code> system enhances sorting and searching of 3ds Max scene entities by providing a new type of identifier for <code>Animatable</code> objects. All <code>Animatables</code> have an <code>AnimHandle</code>. It is possible to retrieve any <code>Animatable</code> using its <code>AnimHandle</code>. An <code>Animatable</code>&apos;s <code>AnimHandle</code> is unchanging within a 3ds Max session, and the system does not reuse them for newer <code>Animatables</code> during a 3ds Max session. After an <code>Animatable</code> is deleted, attempting to retrieve it from its handle is completely safe, the system indicates the object is deleted by returning NULL.</p>\n\
<p><code>AnimHandle</code>s are ideal as key values when sorting <code>Animatable</code> objects, including <code>ReferenceTargets</code> and other derived types, into a data structure. However, they are not saved to disk, and a given object will not have the same <code>AnimHandle</code> when the file is next loaded from disk.</p>\n\
<p>An <code>AnimHandle</code> is an unsigned integer set to 1 when the application launches, is incremented by 1 for <code>Animatable</code> created, and is never reset, even if you load a scene and reset repeatedly. Thus, each <code>Animatable</code> receives a greater-valued handle than the previous for the duration of the 3ds Max session.</p>\n\
<p>The only possible case of handle re-use is an error condition called wrap-around, when sufficient <code>Animatables</code> have been created to cause the handle counter to wrap around to zero. This requires creating 2^32 Animatables for 32-bit builds, or 2^64 Animatables for 64-bit builds, in one session without shutting down 3ds Max.</p>\n\
<h2 id=\"using-animhandles\">Using AnimHandles</h2><p><code>Animatables</code> may be fetched from their handle, and the handle may be fetched from an <code>Animatable</code>, using the following two static methods, respectively:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static AnimHandle Animatable::GetHandleByAnim( Animatable* anim );\n\
static Animatable* Animatable::GetAnimByHandle( AnimHandle handle );</code></pre><p>If an <code>Animatable</code> has been deleted, then <code>GetAnimByHandle()</code> on that object returns NULL. Also, <code>GetAnimByHandle()</code> performs a binary search lookup, requiring time based on the number of existing <code>Animatables</code> expressed by:</p>\n\
<p><img src=\"./images/GUID-D535ADBE-C006-4BF7-969A-2AA775071CCE-low.png\" alt></p>\n\
<p><code>AnimHandle</code>s are not pointers to <code>Animatable</code>s they remain safe if the <code>Animatable</code> is deleted, and are ideal for use as sorting/hash keys. Since the handles strictly increase in value each time one is assigned, new <code>Animatables</code> can be added to the end of a list <code>Animatable</code>s sorted by <code>AnimHandle</code>. Since the handles are not re-used, there will never be a key collision between two different <code>Animatable</code>s, even if one or both have been deleted.</p>\n\
<p>A given object in a scene is assigned different handle value each time the scene is loaded. The order of handles could change between loads, an object with a higher handle value on one occasion might have a lower handle value on another occasion.</p>\n\
<h2 id=\"animhandles-are-not-node-handles\">AnimHandles are not Node Handles</h2><p>Do not confuse Node Handles with <code>Animatable</code> Handles: they are actually quite different. A node handle is a unique permanent ID, saved with the scene, and pertains to Node objects only. An <code>AnimHandle</code> is an instance count. It is unique within one session of 3ds Max, but not permanent across file loads. Every <code>Animatable</code> has an <code>AnimHandle</code>, not just nodes.</p>\n\
<h3 id=\"node-handles-\">Node handles:</h3><ul>\n\
<li>uniquely define an object within the scene</li>\n\
<li>are saved with the scene</li>\n\
<li>usually never change; but, when an object is XRef&apos;d into another scene, it is given a new handle.</li>\n\
<li>are NOT assigned in the constructor. After a Node is created, the handle is assigned only after its Load() operation, and after it is added to the scene. Any code using them needs extra logic to ensure node handles are valid.</li>\n\
</ul>\n\
<h3 id=\"animhandles-\">AnimHandles:</h3><ul>\n\
<li>uniquely define an object within a scene.</li>\n\
<li>are NOT saved with the scene.</li>\n\
<li>assigned in the <code>Animatable</code>&apos;s constructor when they become immediately valid and never change.</li>\n\
<li>will NOT have the same value for the same <code>Animatable</code> when the file is next loaded.</li>\n\
<li>strictly increase and never reset until 3ds Max is shut down.</li>\n\
<li>are ideal for sorting and searching, assuming your data structure does not persist across load/save.</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../animation_time_and_controllers.html\">Animation, Time, and Controllers</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";