var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"keyframe_data_access_classes_and\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Keyframe Data Access Classes and Methods</h1></div><p>This section discusses access to keyframe controller data. The 3ds Max API provides a class <code>IKeyControl</code>. This class provides an interface into the TCB, Bezier, and Linear keyframe controllers allowing a developer to add, delete, retrieve and update the keys of the controller.</p>\n\
<p>Use of the <code>IKeyControl</code> methods requires you to include <code>\\\\MAXSDK</code>\\INCLUDE\\ISTDPLUG.H, i.e.:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#include&quot;ISTDPLUG.H&quot;</code></pre><p>The standard 3ds Max PRS keyframe controllers provide access to their keys using this class. To get an interface you can use to call methods of <code>IKeyControl</code> use the following macro (defined in <code>\\\\MAXSDK</code>\\INCLUDE\\ANIMTBL.H):</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#define GetKeyControlInterface(anim)\n\
   ((IKeyControl*)anim-&gt;GetInterface(I_KEYCONTROL))</code></pre><p>You may use this macro as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">IKeyControl *ikc = GetKeyControlInterface(anim);</code></pre><p>The return value will either be NULL or a pointer to a valid controller interface. A NULL pointer indicates the controller does not support this interface to allow access to its data.</p>\n\
<p>The following is an example of getting a position controller interface from a node in the scene. The first thing that happens is the position controller is retrieved from the node (using methods of class <code>INode</code> and <code>Control</code>) and then the controller interface is returned via the <code>GetKeyControlInterface()</code> macro:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Control *c;\n\
c = node-&gt;GetTMController()-&gt;GetPositionController();\n\
\n\
IKeyControl *ikeys = GetKeyControlInterface(c);\n\
if (!ikeys) return; // No interface available to access the keys...</code></pre><p>The methods of <code>IKeyControl</code> may be called using the <code>ikeys</code> pointer. For example the method <code>IKeyControl::GetKey()</code> retrieves the nth You&apos;ll need to check the ClassID of the controller so you can pass the appropriate class to the method to retrieve the keys. There are three type of controllers and five data types that are supported by the <code>IKeyControl</code> interface. The controller types are Tension/Continuity/Bias (TCB), Bezier, and Linear. The data types are floating point (<code>float</code>), Position (<code>Point3</code>), Rotation (<code>Quat</code> or <code>AngAxis</code>), and Scale (<code>ScaleValue</code>). The type of key you pass to the <code>GetKey()</code> or <code>SetKey()</code> methods depends on both the controller and data type.</p>\n\
<p>The following classes are used for key storage for each valid possibility:</p>\n\
<ul>\n\
<li>Tension/Continuity/Bias Controllers: <code>ITCBFloatKey</code>, <code>ITCBPoint3Key</code>, <code>ITCBRotKey</code>, <code>ITCBScaleKey</code>.</li>\n\
<li>Bezier Controllers: <code>IBezFloatKey</code>, <code>IBezPoint3Key</code>, <code>IBezQuatKey</code>, <code>IBezScaleKey</code>.</li>\n\
<li>Linear Controllers: <code>ILinFloatKey</code>, <code>ILinPoint3Key</code>, <code>ILinRotKey</code>, <code>ILinScaleKey</code>.</li>\n\
</ul>\n\
<h2 id=\"example\">Example</h2><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">ITCBPoint3Key tcbPosKey;\n\
int numKeys = ikeys-&gt;GetNumKeys();\n\
\n\
if (c-&gt;ClassID() == Class_ID(TCBINTERP_POSITION_CLASS_ID, 0))\n\
{\n\
   for (i = 0; i &lt; numKeys; i++)\n\
   {\n\
     ikeys-&gt;GetKey(i, &amp;tcbPosKey);\n\
     DebugPrint(_T(&quot;\\nPosition Key: %d=(%.1f, %.1f, %.1f)&quot;),\n\
            i, tcbPosKey.val.x,\n\
            tcbPosKey.val.y,\n\
            tcbPosKey.val.z);\n\
   }\n\
}\n\
</code></pre><p>Note how an instance of the <code>ITCBPoint3Key</code> class was passed to <code>GetKey()</code> since the ClassID indicates the controller is a TCB controller and we want position keys (which operate on <code>Point3</code>s).</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../controller_data_access.html\">Controller Data Access</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";