var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"procedural_controller_data_acces\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Procedural Controller Data Access</h1></div><p>Procedural controllers compute their value algorithmically rather than interpolate between stored keyframes. Developers may need to retrieve values from these controllers but won&apos;t be able to use the <code>IKeyControl</code> interface since no keys are available. In these cases, one must sample the controller to retrieve its value at each frame required. This is done by calling the <code>Control::GetValue()</code> method directly. The following sample code samples the controller for each frame in the current animation interval and <code>DebugPrint()</code>s the values to the VC++ IDE debug window. Note how the parent matrix of the node is passed into the <code>GetValue()</code> call. This allows <code>GetValue()</code> to update the appropriate matrix. In the code below, the position coordinates printed by <code>DebugPrint()</code> will be in world space.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void Utility::SampleController(INode *n, Control *c)\n\
{\n\
   TimeValue t;\n\
   Point3 trans;\n\
   Matrix3 pmat;\n\
   Interval ivalid;\n\
   int tpf = GetTicksPerFrame();\n\
   int s = ip-&gt;GetAnimRange().Start()/tpf,\n\
   e = ip-&gt;GetAnimRange().End()/tpf;\n\
\n\
   // Sample the controller at every frame in the anim range\n\
   for (int f = s; f &lt;= e; f++)\n\
   {\n\
       t = f*tpf;\n\
       ivalid = FOREVER;\n\
       pmat = n-&gt;GetParentTM(t);\n\
       c-&gt;GetValue(t, &amp;pmat, ivalid, CTRL_RELATIVE);\n\
       trans = pmat.GetTrans();\n\
       DebugPrint(_T(&quot;\\nPosition at frame: %d of %d=(%.1f, %.1f,\n\
           %.1f)&quot;), f, e, trans.x, trans.y, trans.z);\n\
   }\n\
}\n\
\n\
// Display the data of the controller. For TCB, Bezier and Linear\n\
// keyframe PRS controllers the position, rotation and scale values\n\
// are displayed in the local space of the controller. For\n\
// procedural controllers (or those that don&apos;t support the\n\
// IKeyControl interface), position data is displayed in world space.\n\
void Utility::KeyTest()\n\
{\n\
   int i, numKeys;\n\
   INode *n;\n\
   Control *c;\n\
   Quat newQuat, prevQuat;\n\
   IKeyControl *ikeys;\n\
   ITCBPoint3Key tcbPosKey;\n\
   ITCBRotKey tcbRotKey;\n\
   ITCBScaleKey tcbScaleKey;\n\
   IBezPoint3Key bezPosKey;\n\
   IBezQuatKey bezRotKey;\n\
   IBezScaleKey bezScaleKey;\n\
   ILinPoint3Key linPosKey;\n\
   ILinRotKey linRotKey;\n\
   ILinScaleKey linScaleKey;\n\
\n\
   // Get the first node in the selection set\n\
   if (!ip-&gt;GetSelNodeCount()) return;\n\
   n = ip-&gt;GetSelNode(0);\n\
\n\
   // --- Process the position keys ---\n\
   c = n-&gt;GetTMController()-&gt;GetPositionController();\n\
   ikeys = GetKeyControlInterface(c);\n\
   if (!ikeys)\n\
   {\n\
       // No interface available to access the keys...\n\
       // Just sample the controller to get the position\n\
       // data at each key...\n\
       SampleController(n, c);\n\
       return;\n\
   }\n\
   numKeys = ikeys-&gt;GetNumKeys();\n\
   DebugPrint(_T(&quot;\\nThere are %d position key(s)&quot;), numKeys);\n\
\n\
   if (c-&gt;ClassID() == Class_ID(TCBINTERP_POSITION_CLASS_ID, 0))\n\
   {\n\
       for (i = 0; i &lt; numKeys; i++)\n\
       {\n\
           ikeys-&gt;GetKey(i, &amp;tcbPosKey);\n\
           DebugPrint(_T(&quot;\\nTCB Position Key: %d=(%.1f, %.1f, %.1f)&quot;),\n\
               i, tcbPosKey.val.x, tcbPosKey.val.y, tcbPosKey.val.z);\n\
       }\n\
   }\n\
   else if (c-&gt;ClassID() == Class_ID(HYBRIDINTERP_POSITION_CLASS_ID, 0))\n\
   {\n\
       for (i = 0; i &lt; numKeys; i++)\n\
       {\n\
           ikeys-&gt;GetKey(i, &amp;bezPosKey);\n\
           DebugPrint(_T(&quot;\\nBezier Position Key: %d=(%.1f, %.1f, %.1f)&quot;),\n\
               i, bezPosKey.val.x, bezPosKey.val.y, bezPosKey.val.z);\n\
       }\n\
   }\n\
   else if (c-&gt;ClassID() == Class_ID(LININTERP_POSITION_CLASS_ID, 0))\n\
   {\n\
     for (i = 0; i &lt; numKeys; i++)\n\
     {\n\
         ikeys-&gt;GetKey(i, &amp;linPosKey);\n\
         DebugPrint(_T(&quot;\\nLinear Position Key: %d=(%.1f, %.1f, %.1f)&quot;),\n\
             i, linPosKey.val.x, linPosKey.val.y, linPosKey.val.z);\n\
     }\n\
   }\n\
\n\
   // --- Process the rotation keys ---\n\
   c = n-&gt;GetTMController()-&gt;GetRotationController();\n\
   ikeys = GetKeyControlInterface(c);\n\
   if (!ikeys) return;\n\
\n\
   numKeys = ikeys-&gt;GetNumKeys();\n\
   DebugPrint(_T(&quot;\\nThere are %d rotation key(s)&quot;), numKeys);\n\
\n\
   if (c-&gt;ClassID() == Class_ID(TCBINTERP_ROTATION_CLASS_ID, 0))\n\
   {\n\
       for (i = 0; i &lt; numKeys; i++)\n\
       {\n\
           ikeys-&gt;GetKey(i, &amp;tcbRotKey);\n\
           newQuat = QFromAngAxis(tcbRotKey.val.angle, tcbRotKey.val.axis);\n\
           if (i) newQuat = prevQuat * newQuat;\n\
           prevQuat = newQuat;\n\
           DebugPrint(_T(&quot;\\nTCB Rotation Key: %d=(%.1f, %.1f, %.1f, %.1f)&quot;),\n\
               i, newQuat.x, newQuat.y, newQuat.z, newQuat.w);\n\
       }\n\
   }\n\
   else if (c-&gt;ClassID() == Class_ID(HYBRIDINTERP_ROTATION_CLASS_ID, 0))\n\
   {\n\
       for (i = 0; i &lt; numKeys; i++)\n\
       {\n\
           ikeys-&gt;GetKey(i, &amp;bezRotKey);\n\
           newQuat = bezRotKey.val;\n\
           if (i) newQuat = prevQuat * newQuat;\n\
           prevQuat = newQuat;\n\
           DebugPrint(_T(&quot;\\nBezier Rotation Key: %d=(%.1f, %.1f, %.1f, %.1f)&quot;),\n\
               i, newQuat.x, newQuat.y, newQuat.z, newQuat.w);\n\
     }\n\
   }\n\
   else if (c-&gt;ClassID() == Class_ID(LININTERP_ROTATION_CLASS_ID, 0))\n\
   {\n\
       for (i = 0; i &lt; numKeys; i++)\n\
       {\n\
           ikeys-&gt;GetKey(i, &amp;linRotKey);\n\
           newQuat = linRotKey.val;\n\
           if (i) newQuat = prevQuat * newQuat;\n\
           prevQuat = newQuat;\n\
           DebugPrint(_T(&quot;\\nLinear Rotation Key: %d=(%.1f, %.1f, %.1f, %.1f)&quot;),\n\
               i, newQuat.x, newQuat.y, newQuat.z, newQuat.w);\n\
       }\n\
   }\n\
\n\
   // --- Process the scale keys ---\n\
   c = n-&gt;GetTMController()-&gt;GetScaleController();\n\
   ikeys = GetKeyControlInterface(c);\n\
   if (!ikeys) return;\n\
\n\
   numKeys = ikeys-&gt;GetNumKeys();\n\
   DebugPrint(_T(&quot;\\nThere are %d scale key(s)&quot;), numKeys);\n\
\n\
   if (c-&gt;ClassID() == Class_ID(TCBINTERP_SCALE_CLASS_ID, 0))\n\
   {\n\
       for (i = 0; i &lt; numKeys; i++)\n\
       {\n\
           ikeys-&gt;GetKey(i, &amp;tcbScaleKey);\n\
           DebugPrint(_T(&quot;\\nTCB Scale Key: %2d=(%.1f, %.1f, %.1f)&quot;),\n\
               i, tcbScaleKey.val.s.x, tcbScaleKey.val.s.y,\n\
               tcbScaleKey.val.s.z);\n\
       }\n\
   }\n\
   else if (c-&gt;ClassID() == Class_ID(HYBRIDINTERP_SCALE_CLASS_ID, 0))\n\
   {\n\
       for (i = 0; i &lt; numKeys; i++)\n\
       {\n\
           ikeys-&gt;GetKey(i, &amp;bezScaleKey);\n\
           DebugPrint(_T(&quot;\\nBezier Scale Key: %2d=(%.1f, %.1f, %.1f)&quot;),\n\
               i, bezScaleKey.val.s.x, bezScaleKey.val.s.y,\n\
               bezScaleKey.val.s.z);\n\
       }\n\
   }\n\
   else if (c-&gt;ClassID() == Class_ID(LININTERP_SCALE_CLASS_ID, 0))\n\
   {\n\
       for (i = 0; i &lt; numKeys; i++)\n\
       {\n\
           ikeys-&gt;GetKey(i, &amp;linScaleKey);\n\
           DebugPrint(_T(&quot;\\nLinear Scale Key: %2d=(%.1f, %.1f, %.1f)&quot;),\n\
               i, linScaleKey.val.s.x, linScaleKey.val.s.y,\n\
               linScaleKey.val.s.z);\n\
       }\n\
   }\n\
}</code></pre><p>A developer could also get the relative values from the PRS controller by passing in the identity matrix and not the parent matrix to <code>GetValue()</code>. This is appropriate for the PRS transform controller but not the Look At transform controller. With a Look At controller, its relative values are actually a function of the input matrix. If you pass in the identity to its <code>GetValue()</code> method, it will think that the object is positioned at the center of the world and the results won&apos;t be meaningful.</p>\n\
<h2 id=\"sample-code\">Sample Code</h2><p>The following sample code demonstrates access to the controller data for the first node in the current selection set. The code checks to see if the controller provides its data via the <code>IKeyControl</code> interface. If it does, this interface is used to get the keys. If it does not, the controller is sampled at each frame to get its data. For the keyframe controllers, note how the Class IDs are checked to ensure the proper classes are used when getting key values. Also note how the rotation keys are derived by multiplying on the left by the previous key. For the procedural controllers, note how the parent matrix of the node is passed to the <code>GetValue()</code> method.</p>\n\
<p>The following code displays the position data of controller in world coordinates for each frame in the animation range:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void Utility::SampleController(INode *n, Control *c)\n\
{\n\
    TimeValue t;\n\
    Point3 trans;\n\
    Matrix3 pmat;\n\
    Interval ivalid;\n\
    int tpf = GetTicksPerFrame();\n\
    int s = ip-&gt;GetAnimRange().Start()/tpf,\n\
    e = ip-&gt;GetAnimRange().End()/tpf;\n\
\n\
    // Sample the controller at every frame in the anim range\n\
    for (int f = s; f &lt;= e; f++) \n\
    {\n\
        t = f*tpf;\n\
        ivalid = FOREVER;\n\
        pmat = n-&gt;GetParentTM(t);\n\
        c-&gt;GetValue(t, &amp;pmat, ivalid, CTRL_RELATIVE);\n\
        trans = pmat.GetTrans();\n\
        DebugPrint(_T(&quot;\\nPosition at frame: %d of %d=(%.1f, %.1f, %.1f)&quot;),\n\
            f, e, trans.x, trans.y, trans.z);\n\
    }\n\
}\n\
\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../controller_data_access.html\">Controller Data Access</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";