var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"intervals\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Intervals</h1></div><p>An Interval is a class that represents a length of time. The most common use is to describe a range of time over which an item is said to be &#39;valid&#39; with regards within the <a href=\"./../modeling/geometry_pipeline_system/cache_system.html\">Cache System</a>. This type of interval is referred to as a v<em>alidity interval</em>.</p>\n\
<p>The validity interval describes the range of time over which the cache accurately reflects the state of the item. When comparing a given time to see if the cache is up to date, if the time is inside the interval, the cache is valid at that time. If it is outside the interval, the cache is invalid.</p>\n\
<p>3ds Max object and modifier plug-ins are responsible for maintaining, updating, and reporting their validity intervals.</p>\n\
<p>Plug-ins must provide 3ds Max with information about when they are changing and when they are not. Below are some examples of methods which plug-in objects or modifiers call or implement that return or modify intervals.</p>\n\
<h2 id=\"retrieving-interval-values\">Retrieving Interval Values</h2><p>The <code>GetValue()</code> method of <code>IParamBlock</code> has several parameters, one of which is a C++ reference to an Interval. This method is frequently used by developers to &#39;whittle&#39; down an interval. When a parameter of a parameter block is animated, for any given time there is an interval over which the parameter is constant. If the parameter is constantly changing the interval is instantaneous. If the parameter does not change for a certain period the interval will be longer. If the parameter never changes the interval will be <code>FOREVER</code>. By passing an interval to the <code>GetValue()</code> method of the parameter block you ask the parameter block to &#39;intersect&#39; the interval passed in with the interval of the parameter. Intersecting two intervals means returning a new interval whose start value is the greater of the two, and whose end value is smaller of the two. In this way, the resulting interval represents a combined period of constancy for the two intervals.</p>\n\
<p>This technique is used frequently to compute a validity interval for an object. The developer starts an interval off as <code>FOREVER</code>, then intersects this interval with each of its animated parameters (by calling <code>GetValue()</code>). <code>GetValue()</code> &#39;whittles&#39; down the interval with each call. When all the parameters have been intersected, the result is the overall validity interval of an object at a specific time.</p>\n\
<h2 id=\"example\">Example</h2><p>Consider the example shown in the diagram below. A validity interval is computed at time 40 for an object with two animated parameters, Radius, and Segments. The interval is computed by first intersecting an initial interval of FOREVER with the Radius parameter. The Radius interval is from 20 to 100. The result of this intersection is the interval from 20 to 100. This interval is intersected with the Segment parameter interval (from 10 to 50). The result of this intersection is an interval from 20 to 50. Thus the validity interval of the object at time 40 is from 20 to 50.</p>\n\
<p><img src=\"./images/GUID-965AB410-DB0B-4A26-9D0E-23FF3A098BF3-low.gif\" alt=\"\"></p>\n\
<p><code>ObjectValidity()</code> - This method, implemented by the plug-in, returns the validity interval of the procedural object around the time passed. This method is computed by the object by starting an interval at <code>FOREVER</code>, and intersecting this interval with the intervals of each of its animated parameters. In this way, an interval is whittled down as it is intersected with each of the parameters. The resulting interval represents a period of constancy about the TimeValue passed. The interval represents how far before the TimeValue and how far after it the object is constant.</p>\n\
<p><code>LocalValidity()</code> - This method returns the validity interval of the modifier itself around the time passed. In general, this would be the intersection of the validity intervals of all controllers that the modifier uses to control its parameters, so if a modifier was not animated, this interval would be <code>FOREVER</code>.</p>\n\
<p>As an object flows up the pipeline, the validity interval of each modifier is intersected into the object state&#39;s validity interval. When the object gets to the end of the pipeline, its validity interval reflects the intersection of all the elements in the pipeline.</p>\n\
<p><code>GetValidity()</code> - The <code>SimpleMod</code> class calls this method to retrieve the validity interval of the modifier. The modifier provides this interval by starting an interval at FOREVER and intersecting each parameter of the modifier with the interval. <code>SimpleMod</code> then intersects the validity intervals of its own controllers with the returned interval in its implementation of <code>LocalValidity()</code>.</p>\n\
<p><code>UpdateValidity()</code> - A modifier calls the <code>UpdateValidity()</code> method of an object. When a modifier is applied to an object, it needs to include its own validity interval. Frequently this is called by the modifier in its <code>ModifyObject()</code> method.</p>\n\
<p><code>SetChannelValidity()</code> - This method is called to specify a validity interval for a certain channel of the pipeline.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../animation_time_and_controllers.html\">Animation, Time, and Controllers</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";