var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"track_view\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Track View</h1></div><p>The track view of 3ds Max provides a view into time. Track view is organized as a nested hierarchy showing both parent/child relationships, and sub-anim (parameter) relationships.</p>\n\
<p>Many different plugin types may participate in track view. For example, system plugins, sound object plugins, and many controller types all may have interfaces in track view. The methods of the <code>Animatable</code> class allow a developer to participate. This section is an overview of the concepts of track view, and an outline of the specific methods involved in providing an interface.</p>\n\
<p>The track view provides a hierarchical view of the scene. All controllers appear in the track view as children of the objects they control. For example, every node has a transform controller and this controller appears under the node in the track view. Also, procedural objects and modifiers appear in the track view. If these have animatable parameters, the controllers plugged in to those parameters appear underneath the object or modifier. If a controller itself has animatable parameters, then any controllers plugged into those parameters would be found underneath it in the track view.</p>\n\
<p>On the right side of each item in the track view there is a long horizontal track window. In this window, a controller can represent the track it controls. For example, a keyframe controller draws marks in the track representing points in time that have keyframes. Other procedural controllers may only present a time line displaying the duration of the animation.</p>\n\
<p>Each controller is also able to display a function curve. This may or may not be editable by the user depending on the type of controller. For example, the standard keyframe controllers allow the user to edit the function curve while the expression controller simply displays a graph of the function that the user specified. The user has the ability to overlay function curves for different controllers on top of each other, regardless of the types of the controllers. For example, the user could place a function curve that represents the red component of a spotlight on top of the function curve that represents the position along a path of one of the nodes in the scene and edit the two.</p>\n\
<p>In addition to function curves and tracks, a modal dialog may be brought up by the controller that contains additional parameters which the user can edit.</p>\n\
<h2 id=\"appearing-in-track-view\">Appearing in Track View</h2><p>By default an animatable (controller, parameter, etc.) will appear in track view. If the animatable should not, the method <code>Animatable::BypassTreeView()</code> can be overridden to return TRUE</p>\n\
<p>This method indicates to the system that this item should not appear in the track view. Note: Track View was formally referred to as Tree View. The default implementation returns FALSE, so items will appear unless this method is implemented to return TRUE.</p>\n\
<h2 id=\"copy-and-paste-operations\">Copy and Paste Operations</h2><p>There are two types of copy and paste in track view. These are available using two buttons in track view dialog. There is the Copy Controller button, and in Edit Time Mode, there is the Copy Tracks button.</p>\n\
<p>Copy Controller lets the user copy the entire controller plugin. A method of <code>Animatable</code> is available to prevent an item from being copied.</p>\n\
<p>virtual BOOL CanCopyAnim()</p>\n\
<p>If an animatable doesn&#39;t want to be copied in track view it can return FALSE from this method, otherwise it can return TRUE.</p>\n\
<p>Copy Tracks lets the user select a block of time and copy keys. An item (usually a controller) can participate in the copy and pasting of tracks. To indicate that it will participate in copying, a controller implements the following method and returns TRUE:</p>\n\
<p>virtual BOOL CanCopyTrack(Interval iv, DWORD flags)</p>\n\
<p>Returns TRUE if this item can copy its data over the specified range; otherwise returns FALSE.</p>\n\
<p>If a plugin can copy its track data, the following method will be called to allow the item to do so:</p>\n\
<p>virtual TrackClipObject *CopyTrack(Interval iv, DWORD flags)</p>\n\
<p>This method is called to copy the item&#39;s track data over the specified interval. A plugin should derive a class from the <code>TrackClipObject</code> base class to store the data associated with the objects tracks, and implement the methods that identify the creator of the clip object.</p>\n\
<p>When the user has asked to paste the track, the system will call the following method to see if the track data can be pasted to the item.</p>\n\
<p>virtual BOOL CanPasteTrack(TrackClipObject *cobj,Interval iv, DWORD flags)</p>\n\
<p>Returns TRUE if this item can paste its data over the specified range; otherwise returns FALSE. The item can check the <code>TrackClipObject</code> creator and see if the data is appropriate for it to accept.</p>\n\
<p>If the data is okay, the following method will be called to paste the data from the <code>TrackClipObject</code> to the item.</p>\n\
<p>virtual void PasteTrack(TrackClipObject *cobj,Interval iv, DWORD flags)</p>\n\
<p>This method is called to paste the specified clip object to this track. This method will not be called unless <code>CanPasteTrack()</code> returned TRUE.</p>\n\
<h2 id=\"time-operations\">Time Operations</h2><p>This section shows several of the methods of <code>Animatable</code> that relate to the manipulation of time in track view:</p>\n\
<p><code>Animatable::SupportTimeOperations()</code> - If an item supports time operations in the track view (cut, copy, paste, etc.), it should implement this method to return TRUE. When it is FALSE the user cannot select blocks of time in the item&#39;s track.</p>\n\
<p><code>Animatable::DeleteTime()</code> - This method is called to delete the specified interval of time (or the keys within the interval).</p>\n\
<p><code>Animatable::ReverseTime()</code> - This method is called to reverse the data within the specified interval.</p>\n\
<p><code>Animatable::ScaleTime()</code> - This method is called to scale an interval of time by the specified scale factor.</p>\n\
<p><code>Animatable::InsertTime()</code> - This method is called to insert the specified amount of time at the specified insertion point.</p>\n\
<h2 id=\"drawing-and-hit-testing-tracks-and-function-curves\">Drawing and Hit Testing Tracks and Function Curves</h2><p>This section discusses how an item may display and hit test itself in the track and function curve modes of track view.</p>\n\
<p>To indicate the amount of space it needs to draw itself, the item implements the following method:</p>\n\
<p>virtual int GetTrackVSpace(int lineHeight) - Returns the vertical space occupied by the track in units of one line.</p>\n\
<p>An item may have a custom appearance for itself in the track view. It does this custom drawing in its implementation of the following method:</p>\n\
<p>virtual int PaintTrack(HDC hdc,Rect&amp; rcTrack, Rect&amp; rcPaint, float zoom, int scroll, DWORD flags) - This method is called to display the item in the track view. If an item needs to draw itself in a special fashion, it implements this method to do so. For example, a sound plugin may draw its waveform using this method. If an item does not need to draw itself, the default implementation may be used. This draws the range bar for the item in the track instead.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> When drawing something to appear in Track View, a developer should not do any clipping of their own. 3ds Max will take care of all clipping itself.</div>\n\
<p>virtual int HitTestTrack(TrackHitTab&amp; hits, Rect&amp; rcHit,Rect&amp; rcTrack, float zoom,int scroll,DWORD flags) -</p>\n\
<p>The next method is used to hit test a track. Hit testing is checking to see which keys have been selected by the user. This selection may be a single pick point or a dragged rectangular region. This method is passed a table of <code>TrackHitRecords</code> to update. Each key that lies within the hit rectangle (is hit) should be added to this table.</p>\n\
<p>This method is called to determine which keys lie within the <code>rcHit</code> rectangle. Keys that are hit are added to the <code>hits</code> table.</p>\n\
<p>When the user has selected Function Curve mode, 3ds Max displays the item in a graph with time along the horizontal axis and the value of the item along the vertical axis. An item can draw itself in track view in any way it wishes. It does so by implementing the following method:</p>\n\
<p>virtual int PaintFCurves(ParamDimensionBase *dim, HDC hdc, Rect&amp; rcGraph, Rect&amp; rcPaint, float tzoom,int tscroll, float vzoom, int vscroll, DWORD flags)</p>\n\
<p>This method is called to draw the function curve of the item.</p>\n\
<p>There are a set of macros that are handy for scaling time and values into and out of screen space. These are often used in the implementations of <code>PaintFCurve()</code> and <code>HitTestFCurves()</code>. See <code>Screen-Time-Value Macros</code>.</p>\n\
<p>When the user executes zoom extents on the item, this method is called:</p>\n\
<p>virtual int GetFCurveExtents(ParamDimensionBase *dim, float &amp;min, float &amp;max, DWORD flags)</p>\n\
<p>This method is called to calculate the largest and smallest values of the item.</p>\n\
<p>The next method is used to hit test the keys of a function curve. This selection may be a single pick point or a dragged rectangular region. This method is passed a table of <code>TrackHitRecords</code> to update. Each key that lies within the hit rectangle (is hit) should be added to this table.</p>\n\
<p>virtual int HitTestFCurves(ParamDimensionBase *dim, TrackHitTab&amp; hits, Rect&amp; rcHit, Rect&amp; rcGraph, float tzoom, int scroll, float vzoom,int vscroll, DWORD flags)</p>\n\
<p>This method is called to hit test the item&#39;s function curves. It is called to determine which keys on the curve lie within the <code>rcHit</code> rectangle. Keys that are hit are added to the <code>hits</code> table.</p>\n\
<h2 id=\"track-view-and-references\">Track View and References</h2><p>3ds Max maintains a reference to every sub-anim in the track view. This is how track view monitors if the sub-anim is changing. This reference is created when the sub-anim appears in track view, and is deleted when the sub-anim is no longer shown.</p>\n\
<h2 id=\"summary\">Summary</h2><p>This section has provided an overview of many of the methods related to participating in track view. The main methods developers must use to work with keys, time ranges, function curves, and copying/pasting operations were discussed. The primary methods associated with track view are from class <code>Animatable</code>. See this class for more detailed information on the methods presented above as well as the others associated with track view.</p>\n\
<h4 id=\"see-also\">See Also</h4><p>-</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
      <div class=\"relinfo relchildren\">\n\
        <strong>Pages in this section</strong>\n\
        <ul class=\"ullinks\">\n\
          <li class=\"link ulchildlink\"><a href=\"track_view/track_view_key_operations.html\">Track View Key Operations</a></li>\n\
          <li class=\"link ulchildlink\"><a href=\"track_view/deleting_controllers.html\">Deleting Controllers</a></li>\n\
        </ul>\n\
      </div>\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../animation_time_and_controllers.html\">Animation, Time, and Controllers</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";