var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"cat_controllers\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>CAT Controllers</h1></div><h2 id=\"rig-controller-setup\">Rig Controller Setup</h2><p>In general, we have followed a design pattern in CAT where functional groups have been separated out into unit pieces.  The functional units are :</p>\n\
<ul>\n\
<li>Hub (pelvis/ribcage/head)</li>\n\
<li>Limb (arm/leg)</li>\n\
<li>Bone</li>\n\
<li>Digit</li>\n\
<li>Tail</li>\n\
<li>Spine (spine/neck)</li>\n\
</ul>\n\
<p>The controllers in a unit are inter-dependent and cannot be constructed or used in isolation.  Apart from a Hub (which is a single bone), there is a central class that holds data common to the unit and provides management functions, called a Data or a Manager class.  There is one data class created to manage each unit. These data classes are <code>LimbData</code>, <code>BoneData</code>, <code>DigitData</code>, <code>SpineData</code>, and <code>TailData</code>.  Each one of these data classes is responsible for the creation and maintenance of its associated node controllers.  </p>\n\
<p>Note from the rig layout diagram – <code>BoneData</code> is somewhat of a special case.  <code>BoneData</code> is the class that provides the transform for an individual limb bone.  It is managed by <code>LimbData</code>, but it is also a manager itself.  It manages an array of 1-to-N <code>BoneSegTrans</code> classes.  This is because although the limb is a functional unit made up of many limb bones, each limb bone can also be broken down into segments to provide twisting.  The <code>BoneSegTrans</code> class looks to <code>BoneData</code> to manage it, and <code>BoneData</code> looks to <code>LimbData</code> to manage it.</p>\n\
<h2 id=\"base-cat-classes\">Base CAT classes</h2><h3 id=\"-code-catcontrol-code-\"><code>CATControl</code></h3><p>This class is at the root of the rig control structure in CAT.  Its job is to provide the common functionality required by all the classes that make up CAT&#39;s rig structure, and a method of iterating all the controls that make up a given CAT rig.</p>\n\
<p>Some major functionality provided:</p>\n\
<ul>\n\
<li>GetCATParentTrans – provides access to the common data pool\n\
Rig iteration - The CATControls in a rig are linked in a tree structure, where every CATControl has a pointer back to the CATControl that &#39;owns&#39; it.  Access is provided via GetParentCATControl and GetChildCATControl</li>\n\
<li>Bone ID : Each CATControl has an identifier to distinguish it from its siblings.  The identifier is unique to siblings of the same class only.  In other words, an Id on a LimbData is unique among sibling LimbData&#39;s.  The Bone ID is usually also the index in the bone in its array, so if a hug has a single tail and a single limb, the Id of both the TailData and the LimbData will be 0.</li>\n\
<li>CATMessage:  The CATParent triggers events on the rig via a message passing system internal to CAT.  Because there is no reference between the rig elements and the CATParentTrans, when something on the CATParent changes (eg CATUnits), we need to manually update the rig.  The message system can also trigger changes on the bones, eg, baking animation, changing length axis, or adding/removing a layer.</li>\n\
<li>Save/Load rig/pose/clip: CATControl provides the base functionality, although derived classes still need to specialize it.</li>\n\
</ul>\n\
<h3 id=\"-code-catnodecontrol-code-code-catcontrol-code-\"><code>CATNodeControl</code> : <code>CATControl</code></h3><p>The <code>CATNodeControl</code> class specializes the <code>CATControl</code> class for the Matrix3 controllers applied to <code>INodes</code>.  This class is the primary workhorse of the <code>CATSystem</code>.  It provides all the functions for managing the rig elements data, and returning values to Max.  In general, if 3ds Max calls a function on a CAT element, <code>CATNodeControl</code> will be the class that implements the function.</p>\n\
<p><strong>Primary tasks:</strong></p>\n\
<ul>\n\
<li>Implement GetValue/SetValue - The primary role of any transform controller is to provide a transform (Matrix3) value to Max.  <code>CATNodeControl</code> provides a framework for evaluating CAT data to generate a Matrix3.</li>\n\
<li>As part of the Get/Set value, the <code>CATNodeControl</code> also provides the functionality for inheritance, rig stretching, locking, limits, etc.\n\
The structure of the GetValue/SetValue function should also be examined.</li>\n\
<li>Add functionality for generating ArbBones - <code>ArbBones</code> is essentially an implementation of <code>CATNodeControl</code> without any element-specific functionality. All <code>ArbBone</code> management happens on <code>CATNodeControl</code></li>\n\
<li>Add functionality for Extra Rig Nodes/Extra rig controllers.</li>\n\
<li>Destruction: When a <code>CATNodeControl</code> is deleted, it is its own responsibility to re-parent any <code>CATNodeControl</code> children it may have to its parent.  This is to allow (for example) removing spine links, and have the hub at the end of the spine re-parented to whatever link is now at the end of the spine. The same concept applies to all CAT nodes, when it is deleted, any children will be reparented to maintain the hierarchy structure.</li>\n\
<li>Pass-through to the Layer Controllers :  An implementation of <code>CATNodeControl</code> contains no animation data.  Therefore, all they keyframing methods (defined by Animatable) need to passed through to the Layer controller which handles the animation data.</li>\n\
</ul>\n\
<h3 id=\"-code-catclipvalue-code-\"><code>CATClipValue</code></h3><p>This class is the base of the layer system.  A layer is essentially a list controller – it contains an array of generic Max controllers, and it merges their values according to the rules of the layer.  The rules are specified by the type of layer, this data is stored on the <code>CATClipRoot</code> LayerDesc classes.\n\
Originally, we had an implementation of <code>CATClipValue</code> for every value type in Max – float, position, rotation, point3 etc.  However, over time we have removed almost all these layer types, and now simply provide implementations of float (<code>CATClipFloat</code>) for scalar values, and Matrix3 (<code>CATClipMatrix3</code>) for transform values.\n\
The basic list behavior functionality is implemented in <code>CATClipValue</code>.  Here you can find the code for adding/removing layers, cloning, maintaining weights etc.  The implementation&#39;s main job is to calculate the value of the appropriate type.  <code>CATClipFloat</code> is fairly simple, but <code>CATClipMatrix3</code> contains a lot of calculation to generate its values and handle things like inheritance correctly.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../cat.html\">CAT (Character Animation Toolkit)</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";