var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"cat_overview\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>CAT Overview</h1></div><p>A CAT rig is a collection of Controllers that generate motion data for Max nodes.  The concept is that CAT is simply a specialization of the functionality that Max already provides.</p>\n\
<p>The controllers are specialized to the job of each particular body-part.  For example, we have a specific controller for the collarbone, a specific controller for spine bones, and a specific controller for digit bones.  Generally each class handles a particular job and only that job.  A hub may be a pelvis, ribcage or head, but that is because the functionality is the same for all these parts.</p>\n\
<div class=\"note note:\"><strong><span class=\"notetitle\">Note:</span></strong> One class that is more generic is the palm/ankle. A single class generates the motion for both palms and ankles.  The reason for this is semi-quadrupeds; for example, a creature like an Ape may be bipedal and quadredal, and we decided to implement the limb in such a generic way so it could handle both.  Unfortunately the needs of the two roles are quite distinct, and we have ended up making quite a few design compromises in order to keep this single class working for both cases.  </div>\n\
<p>The CAT Rig is rooted in the CATParent.  This entity is not part of the rig at all, it is simply a visual container for a set of data and functions that apply to the rig as a whole.  The CATParent is not the root of the hierarchy, it actually has no functional impact on an animated CAT character.  It is just part of the scene to give users a logical place to access CAT data.</p>\n\
<h2 id=\"max-code-reuse\">Max Code Reuse</h2><p>In CAT, we have tried whenever possible to use Max classes instead of implementing our own.  We do not implement any actual animation classes (eg Bezier Float, Position XYZ), we simply use Max implementations of these classes.  </p>\n\
<h2 id=\"catunits\">CATUnits</h2><p>The original goal of CAT was to build a rigging system that was independent of any one scene size. We wanted to have a system that could be easily resized. To enable this, we implemented <code>CATUnits</code>. <code>CATUnits</code> is a scalar value stored on the <code>CATParent</code> that is used to scale most rig-specific data (eg bone size, setup positions, etc).  Animation data is not scaled by <code>CATUnits</code>.</p>\n\
<h2 id=\"modes\">Modes</h2><p>CAT works in two distinct modes.  In setup mode, operations on the rig are intended to change its base properties – object size, length axis, CATUnits etc.  In Animation mode, only animation data should be changeable.</p>\n\
<h2 id=\"length-axis\">Length Axis</h2><p>When CAT was originaly designed, CAT bones were based on Max primitives and the Max Coordinate system. With the Z-axis in Max being &quot;up&quot;, it made sense to use the Z-axis as the length axis for CAT Bones. However, this presented a problem when some physics engines required exported character bones to be aligned on the X-axis. The X-axis preference by game engines, 3ds Max, Maya and almost every 3d animation tool out there originated from the older field of robotics where the Denavit-Hartenberg conventions defines X as being the dominant bone axis.</p>\n\
<p>CAT version 2.0 added the option of using X as the length axis. This introduced a &#39;length axis&#39; setting used by IK solvers, spine, geometry builders, copy-paste tools etc.\n\
Visually, there is no difference between them when switching.  However, you might note that when creating a CATParent via MaxScript, that it is tipped over on its side.  This is because the mesh defaults to being X up.  The mouse creation tools compensate for this by rotating the node over so its X axis aligns with the scene Z axis, but when creating via MaxScript the matrix set is the identity, and the X-up mesh appears to be lying sideways.</p>\n\
<h2 id=\"layers\">Layers</h2><p>All animation data in CAT is stored in its layer system.  The basic CAT rig classes provide functionality, and their Layer controllers provide data.  Each CATControl has 0-N layer classes to store its data of either Matrix3 or float type.  The CAT layer classes just store lists of Max animation classes of the appropriate type.  The layer system is controlled via means of messaging (see <strong>CATMessage</strong> below).  The messages are passed to individual layer controllers, which then take appropriate action to add/remove layers.\n\
The CAT rig elements know nothing about the layers, and simply query the layer controllers for the appropriate values.  The Layer system merges the values returned by the stored animation controllers according to the type of the layer system and its global weight value.</p>\n\
<p>Each layer in CAT is one of these types: Absolute, CATMotion, Relative Local, and Relative world.\n\
The main class for working with the layer system is the <code>CATClipRoot</code>, stored on the <code>CATParentTrans</code>.  The cat clip root is responsible for managing the layer system.  It stores a list of classes containing meta-data describing each of the layers called <code>NLAInfo</code>.  The <code>NLAInfo</code> class stores all the information on what a layer is for, such as its type, name, colour, etc.  The <code>NLAInfo</code> also stores a pointer to a float controller for the timewarp of the layer and an instance of the <code>CATClipWeight</code> class for generating the global weight for the layer.</p>\n\
<h3 id=\"layer-weighting\">Layer Weighting</h3><p>A <code>CATClipValue</code> needs to scale each value in its list by a weight.  To do so, it needs to know the global weight for the layer (stored on the <code>CATClipRoot</code>&#39;s <code>NLAInfo</code>).  It also needs to know what the local weight for the current group is.  Each group generates an instance of the <code>CATClipWeight</code> class, and a pointer to this instance is shared by all <code>CATClipValues</code> in this group.  The <code>CATClipWeight</code> also stores a pointer to the <code>CATClipWeights</code> class applied to the groups parent.  Eg, for the limb group, its weights controller would store a pointer to the owning hub&#39;s weights class.  To generate the final weight for a layer value, the global weight is multiplied by the local weight and by the parent&#39;s local weight.  The parent&#39;s local weight will have been multiplied by its parent&#39;s local weight - in this way the weights are inherited by each group.</p>\n\
<h2 id=\"objects\">Objects</h2><p>CAT provides custom CAT objects for rig elements.  A CAT bone provides no functionality; it is simply a visual aid to help show how far away from each other bones are.  There is a dependency between the controller and the bone, but we could probably remove that dependency and never notice.  For the majority of work, it is not necessary to work with the CAT bones.</p>\n\
<h2 id=\"arbbones-extra-rig-nodes-and-extra-rig-controllers\">ArbBones, Extra Rig Nodes, and Extra Rig Controllers</h2><p>CAT provides a variety of ways to extend the built in rig structure:</p>\n\
<ul>\n\
<li><code>ArbBones</code> : An arb bone is a generic CAT Bone.  It has all the functionality of standard CAT Bones, including stretching, layers &amp; weights, etc, but has no functionality specific to any body part.  Users create these arb bones generally to extend the rig.  Their only use is as bones though, as they rely on the CAT objects (and <code>CATObjects</code> are not replacable).</li>\n\
<li>Extra Rig Nodes: This is primarily for RG3 rig saving purposes.  CAT can include any node as part of its rig.  Including the node is a one-way connection, no part of the extra node is changed and no additional functionality is added to the extra node.  We simply maintain lists of these extra nodes, and when loading &amp; saving RG3 files we load/save the nodes in these lists.  Its standard to specify a skin mesh as an extra node.</li>\n\
<li>Extra Rig Controllers:  CAT can create generic layer controllers that users can use wherever they want to have a value controlled by the CAT layer system.  Prior to 3ds Max 2013, users could only create float controls.  In 3ds Max 2013 we added the ability to generate Matrix3 controllers (using the MAXScript only).  A Matrix3 extra controller is simply an <code>ArbBone</code> controller without a <code>CATBone</code> attached.</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../cat.html\">CAT (Character Animation Toolkit)</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";