var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"action_interfaces\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Action Interfaces</h1></div><p>Action interfaces only contain UI Action Functions that provide a programmatic method of &quot;pressing&quot; buttons and keys in the UI for a plug-in. As mentioned in the opening section of this doc, these action functions have certain special characteristics and possess additional descriptive metadata, relative to the functions we&apos;ve described so far. Here&apos;s an annotated Action Interface example for the EdMesh system we looked at above.</p>\n\
<p>The public interface (in a public header file)</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#include&quot;iFnPub.h&quot;\n\
#define EM_ACT_INTERFACE Interface_ID(0x65678, 0x123)\n\
#define GetEMActionsInterface(cd) \\\n\
   (EMActions*)(cd)-&gt;GetFPInterface(EM_ACT_INTERFACE)\n\
\n\
enum {\n\
   ema_create, ema_create_enabled, ema_create_checked,\n\
   ema_delete, ema_delete_enabled,\n\
};\n\
\n\
class EMActions: public FPInterface\n\
{\n\
   public:\n\
   virtualFPStatus Create()=0;\n\
   virtualFPStatus Delete()=0;\n\
   virtualBOOL IsCreateEnabled()=0; \n\
   virtualBOOL IsCreateChecked()=0; \n\
   virtualBOOL IsDeleteEnabled()=0; \n\
};</code></pre><p>As with normal function interfaces, we we have an <code>Interface_ID</code> defined and interface accessor helpers. There is also a function ID enum, but in this case there are more IDs than published actions. Each action function can have associated with it up to 3 predicate functions that users of the interface can call to determine status for the action. These are the <code>isEnabled</code>, <code>isChecked</code>, and <code>isVisible</code> predicates and they are used mostly by the UI elements that can be bound to an interface action to determine things like whether a button or menu item should greyed or a checkbutton should be highlighted or whether the action should be added to an about-to-be-displayed pop-up menu. The <code>isEnabled</code> &amp; <code>isVisible</code> predicates are optional and default to always TRUE. The <code>isChecked</code> predicate is for actions that toggle a state, for example starting &amp; stopping a mouse command mode. This predicate is also optional and defaults to FALSE, but it is required if the action is bound to <code>TYPE_CHECKBUTTON</code> via the ParamMap2 system.</p>\n\
<p>Declarations for action functions along with any associated predicates are supplied in the virtual interface class. Action functions always take zero arguments and return an <code>FPStatus</code> result which indicates whether the action was succesfully performed or not (success if <code>FPS_OK</code>, but it may return values such as <code>FPS_ACTION_DISABLED</code> if the action was not enabled at the time). Predicate functions always take zero arguments and return a <code>BOOL</code>.</p>\n\
<p>Here are some action function call examples:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">EMActions* emai = GetEMActionsInterface(edmeshCD);\n\
//...\n\
if (emai-&gt;IsCreateEnabled()) // direct calls\n\
   emai-&gt;Create();</code></pre><p>Indirect Calls:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">if (emai-&gt;IsEnabled(ema_delete))\n\
   emai-&gt;Invoke(ema_delete);</code></pre><p>This indirect example uses one of the predicate helper functions to call the isEnabled predicate in the interface for a given function, specified by FunctionID.</p>\n\
<h2 id=\"the-implementation-class-in-the-cpp-implementation-file-\">The implementation class (in the .cpp implementation file)</h2><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class EMActionsImp : publicEMActions\n\
{\n\
   public:\n\
   BOOLcreating;\n\
\n\
   DECLARE_DESCRIPTOR(EMActionsImp)\n\
   BEGIN_FUNCTION_MAP\n\
     FN_ACT(ema_create, Create)\n\
     FN_PRED(ema_create_enabled, IsCreateEnabled)\n\
     FN_PRED(ema_create_checked, IsCreateChecked)\n\
     FN_ACT(ema_delete, Delete)\n\
     FN_PRED(ema_delete_enabled, IsDeleteEnabled)\n\
   END_FUNCTION_MAP\n\
\n\
   voidInit()\n\
   {\n\
     // initialize any state data\n\
     creating = FALSE;\n\
   }\n\
   FPStatus Create()\n\
   {\n\
     //... start or stop the create mode\n\
     //...\n\
     creating = !creating; // toggle creating state\n\
     //...\n\
     return FPS_OK;\n\
   }\n\
   BOOLIsCreateEnabled()\n\
   {\n\
     //... determine if create enable, perhaps\n\
     // vertex subobjlevel test, etc.\n\
   }\n\
   BOOLIsCreateChecked()\n\
   {\n\
     return creating; // the current create state\n\
   }\n\
   //...\n\
};</code></pre><p>The action interface implementation class contains implementations for the actions and predicates, as required by the virtual interface. It also contains a <code>FUNCTION_MAP</code>, but in this case the actions and predicates entries in the map are defined with their own special FN_ macros, <code>FN_ACT</code> for actions and <code>FN_PRED</code> for predicates.</p>\n\
<p>The Create action in this example corresponds to a vertex create mode start button and so it provides an <code>isChecked</code> predicate to tell what state the action is in, in this case checked =&gt; in create mode, not checked =&gt; not in create mode. The implementation interface contains a data element &apos;creating&apos; to help keep track of this state. Note that it is intialized in a method called <code>Init()</code>. Since the varargs constructor is the normal way the interface instance is created, it guarantees to call this (virtual) init method for you; alternatively, you could make it a static data member and statically initialize it.</p>\n\
<h2 id=\"the-interface-definition-in-the-cpp-implementation-file-\">The interface definition (in the .cpp implementation file)</h2><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static EMActionsImp emai\n\
(\n\
   EM_ACT_INTERFACE, _T(&quot;Actions&quot;), IDS_EMAI, &amp;edmeshCD, FP_ACTIONS,\n\
   ema_create, _T(&quot;create&quot;), IDS_CREATE, 0,\n\
   f_predicates, ema_create_enabled, ema_create_checked,\n\
\n\
   FP_NO_FUNCTION,\n\
     f_category, _T(&quot;creation&quot;), 0,\n\
     f_iconRes, IDI_CREATE_ICON,\n\
     f_toolTip, IDS_VERTEX_CREATE_BY_CLICKIN,\n\
     f_ui, em_params, 0, TYPE_CHECKBUTTON, IDC_CREATE,\n\
     GREEN_WASH,\n\
   end,\n\
     ema_delete, _T(&quot;delete&quot;), IDS_DELETE, 0,\n\
     f_isEnabled, ema_delete_enabled,\n\
     f_ui, em_params, 0, BUTTON, IDC_DELETE,\n\
   end,\n\
   end\n\
);</code></pre><p>The constructor for an Action Interface follows a slightly different syntax to a normal function interface. The header parameters are the same (<code>Interface_ID</code>, internal name, descriptor, CD, flags), the <code>FP_ACTIONS</code> flag must be specified to mark this as an action interface.</p>\n\
<p>Each action function is specified by header parameters giving FunctionID, internal name, descriptor resID and flag bits. This is followed by one or more tagged action function options, in the same manner as parameter options in a <code>ParamBlockDesc2</code> constructor. Each option defines a separate item of metadata for the function In the case of <code>ema_create</code>, the following are specified:</p>\n\
<ul>\n\
<li>2 of the 3 possible predicates via their FunctionIDs,</li>\n\
<li>A category string, used by the UI customize dialogs to arrange actions into category groups</li>\n\
<li>A resID for an Icon resource that will be made available for use in toolbar buttons and other UI locations</li>\n\
<li>A toolTip string resID</li>\n\
<li>A UI specification that allows the parammap2 system to automatically connect a button to this action. This is described in more detail in the next section.</li>\n\
</ul>\n\
<p>The possible options include:</p>\n\
<p>f_category category name, as internal TCHAR* and localizable string resID, defaults to interface name</p>\n\
<p>f_predicates supply 3 functionIDs for isEnabled, isChecked, isVisible</p>\n\
<p>predicates</p>\n\
<p>f_isEnabled isEnabled predicate functionID</p>\n\
<p>f_isChecked isChecked predicate functionID</p>\n\
<p>f_isVisible isVisible predicate functionID</p>\n\
<p>f_iconRes icon as resource ID</p>\n\
<p>f_icon icon as UI .bmp filename, index pair, as per CUI icon specifications</p>\n\
<p>f_buttonText button text string resID, defaults to function description</p>\n\
<p>f_toolTip tooltip string resID, defaults to function description</p>\n\
<p>f_menuText menu item text string resID, defaults to buttonText or function description</p>\n\
<p>f_ui UI spec if paramMap2-implemented UI (pmap blockID, mapID, control type, button or checkbutton resID, hilight col if chkbtn)</p>\n\
<p>f_shortcut default keyboard shortcuts for action functions</p>\n\
<p>f_keyArgDefault marks a parameter as optional and supplies a default value. Optional parameters must come after the positional parameters. Example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Meshop::buildMapFaces, _T(&quot;buildMapFaces&quot;), 0, TYPE_VOID, 0, 3,\n\
   _T(&quot;source&quot;),  0, TYPE_FPVALUE_BR,\n\
   _T(&quot;keep&quot;),     0, TYPE_BOOL, f_keyArgDefault, FALSE,\n\
   _T(&quot;channel&quot;),  0, TYPE_INT, f_keyArgDefault, 0,</code></pre><p><code>f_inOut</code> specifies whether <code>\\_BR</code> parameter is just for input, just for output or both via <code>FPP_OUT_PARAM</code>, <code>FPP_IN_PARAM</code>, or <code>FPP_IN_OUT_PARAM</code> argument. Default is <code>FP_IN_OUT_PARAM</code>.</p>\n\
<p><code>f_macroEmitter</code> Supply a pointer to <code>FPMacroEmitter</code> subclass instance to customize macro-recorder emission. The <code>Emit()</code> method is called on this instance with interface and function def pointers for the particular action function to be emitted.</p>\n\
<p>The ActionTable system &apos;table ID&apos; DWORD for Action interfaces is generated automatically from the Interface_ID for the Action interface (by xor-ing the two DWORDS in the interface ID). If this generates an ID that clashes with built-in table IDS (in the range 0-64), an error message is generated and you should choose a more random Interface_ID.</p>\n\
<p>You define the UI context for the interface as a whole, rather than for each individual action function. The context ID should be specified immediately after the flag word in an Action interface descriptor constructor. For example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static IKChainActionsImp sIKChainActionImp(\n\
   IKCHAIN_FP_INTERFACE_ID, _T(&quot;Action&quot;), IDS_IKCHAIN_ACT,\n\
   &amp;theIKChainControlDesc, FP_ACTIONS, kActionMainUIContext,\n\
   IKChainActionsImp::kSnap, _T(&quot;snap&quot;), IDS_IKCHAIN_SNAP, 0,\n\
   //...</code></pre><p>The constant &apos;<code>kActionMainUIContext</code>&apos; is one of the built-in ActionTable system contexts that you can use (defined in <code>\\\\MAXSDK</code>\\include\\<code>ActionTable</code>.h), and is most likely the one to be used. If you create an entirely new context and context ID, you will need to register and control its activation yourself (see the <code>IActionManager</code> in <code>\\\\MAXSDK</code>\\include\\ActioNTable.h for details).</p>\n\
<p>The automatically-generated ActionTable for the Action interface can be accessed via a new FPInterface method: <code>virtual``ActionTable</code>* GetActionTable();</p>\n\
<p>By default, the action table is activated immediately and stays active throughout the 3ds Max session. You can dynamically control this activation via a new FPInterface method: <code>virtual</code> void EnableActions(BOOL onOff);</p>\n\
<p>This enables or disables the entire set of actions in the interface. You might do this if the actions are only to be active during certain times in the running of 3ds Max. Usually, this control is achieved via ActionTable contexts.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../function_publishing.html\">Function Publishing</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";