var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"core_interfaces\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Core Interfaces</h1></div><p>There is an interface descriptor flag, <code>FP_CORE</code>, which must be specified on Core interface descriptors. Static Core interface descriptors are now automatically registered with the Core, so you do not need to explicitly call <code>RegisterCOREInterface()</code> on them. The code examples below include a Core interface using this flag. The following example taken from the DragAndDrop manager interface.</p>\n\
<h2 id=\"the-public-interface-definition-in-a-public-header-file-\">The public interface definition (in a public header file)</h2><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">classIDragAndDropMgr : public  FPStaticInterface\n\
{\n\
   public:\n\
     virtualvoid EnableDandD(BOOL flag)=0;\n\
     virtual BOOL IsEnabled()=0;\n\
     virtual BOOL EnableDandD(HWND hwnd, BOOL flag,\n\
      DragAndDropHandler* handler = NULL)=0;\n\
     virtual BOOL DropPackage(HWND hwnd, POINT&amp; point,\n\
       URLTab&amp; package)=0;\n\
     virtual BOOL DownloadPackage(URLTab&amp; package, TCHAR* directory,\n\
      HWND hwnd = NULL)=0;\n\
     virtual TCHAR* GetStdDownloadDirectory()=0;\n\
};</code></pre><h2 id=\"the-implementation-class-in-the-cpp-implementation-file-\">The implementation class (in the .cpp implementation file)</h2><p>Note in this example, a couple of the interface methods use a new type, URLTab, which is not a ParamType2 type. This is a <code>Tab</code>&lt; TCHAR*&gt; specialization and so provides overloads that take <code>Tab</code>&lt; TCHAR*&gt;&apos;s and convert them to <code>URLTabs</code>, specifically for dispatch-based calls.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">classDragAndDropMgr :  IDragAndDropMgr\n\
{\n\
   ...\n\
   public:\n\
     void EnableDandD(BOOL flag) { global_enable = flag; }\n\
     BOOL IsEnabled() { return global_enable; }\n\
     BOOL EnableDandD(HWND hwnd, BOOL flag,\n\
      DragAndDropHandler* handler = NULL);\n\
     BOOL DropPackage(HWND hwnd, POINT&amp; point, URLTab&amp; package);\n\
     BOOL DropFiles(HWND hwnd, HDROP hDrop);\n\
     BOOL DownloadPackage(URLTab&amp; package, TCHAR* directory,\n\
     HWND hwnd = NULL);\n\
     TCHAR* GetStdDownloadDirectory();\n\
\n\
     // variants for FnPub interface that take Tab&lt;TCHAR*&gt;s\n\
     BOOL DropPackage(HWND hwnd, POINT&amp; point, Tab&lt;TCHAR*&gt;&amp; package);\n\
     BOOL DownloadPackage(Tab&lt;TCHAR*&gt;&amp; package, TCHAR* directory);\n\
     DECLARE_DESCRIPTOR(DragAndDropMgr)\n\
\n\
   // dispatch map\n\
   BEGIN_FUNCTION_MAP\n\
     VFN_1( dndmgr_globalEnableDnD, EnableDandD, TYPE_BOOL);\n\
     FN_0( dndmgr_isEnabled, TYPE_BOOL, IsEnabled);\n\
     FN_2( dndmgr_enableDandD,\n\
       TYPE_BOOL,\n\
      EnableDandD,\n\
       TYPE_HWND,\n\
       TYPE_BOOL);\n\
     FN_3( dndmgr_dropPackage,\n\
       TYPE_BOOL, DropPackage,\n\
       TYPE_HWND,\n\
       TYPE_POINT_BR,\n\
       TYPE_STRING_TAB_BR);\n\
     FN_2( dndmgr_downloadPackage,\n\
       TYPE_BOOL,\n\
      DownloadPackage,\n\
       TYPE_STRING_TAB_BR,\n\
       TYPE_STRING);\n\
     FN_0( dndmgr_downloadDirectory,\n\
       TYPE_STRING,\n\
      GetStdDownloadDirectory);\n\
   END_FUNCTION_MAP\n\
   ...\n\
};</code></pre><h2 id=\"the-descriptor-note-the-fp_core-flag-\">The descriptor, note the FP_CORE flag.</h2><p>FP_CORE descriptors are automatically registered with <code>RegisterCOREInterface()</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">DragAndDropMgrdragAndDropMgr(DND_MGR_INTERFACE, _T(&quot;dragAndDrop&quot;),\n\
   IDS_DND_INTERFACE, NULL, FP_CORE,\n\
    dndmgr_globalEnableDnD, _T(&quot;globalEnableDragAndDrop&quot;), 0,\n\
   TYPE_BOOL, 0, 1, _T(&quot;onOff&quot;), 0,\n\
   TYPE_BOOL,  dndmgr_isEnabled, _T(&quot;isEnabled&quot;), 0,\n\
   TYPE_BOOL, 0, 0,  dndmgr_enableDandD, _T(&quot;enableDragAndDrop&quot;), 0,\n\
   TYPE_BOOL, 0, 2, _T(&quot;window&quot;), 0,\n\
   TYPE_HWND, _T(&quot;onOff&quot;), 0,\n\
   TYPE_BOOL,  dndmgr_dropPackage, _T(&quot;dropPackage&quot;), 0,\n\
   TYPE_BOOL, 0, 3, _T(&quot;window&quot;), 0,\n\
   TYPE_HWND, _T(&quot;mousePoint&quot;), 0,\n\
   TYPE_POINT_BR, _T(&quot;files&quot;), 0,\n\
   TYPE_STRING_TAB_BR,  dndmgr_downloadPackage, _T(&quot;downloadPackage&quot;), 0,\n\
   TYPE_BOOL, 0, 2, _T(&quot;files&quot;), 0,\n\
   TYPE_STRING_TAB_BR, _T(&quot;directory&quot;), 0,\n\
   TYPE_STRING,  dndmgr_downloadDirectory, _T(&quot;geStdtDownloadDirectory&quot;), 0,\n\
   TYPE_STRING, 0, 0,\n\
   end\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../function_publishing.html\">Function Publishing</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";