var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"mesh_example\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Mesh Example</h1></div><p>Within your plug-in, each interface is usually organized into 3 separate sections of code:</p>\n\
<ul>\n\
<li>The public virtual interface along with the Interface_ID and function ID definitions in a public header file.</li>\n\
<li>the implementation interface, which inherits from the virtual interface, and contains both the function implementations and a function dispatch map used by the indirect calling methods. The FnPub system provides a set of macros to ease the definition of these maps.</li>\n\
<li>the interface descriptor, usually a static instance of the implementation interface. The constructor for this uses the same varargs technique used by the ParamBlockDesc2 constructor, enabling descriptive info for all the functions in the interface to be supplied in one constructor call.</li>\n\
</ul>\n\
<p>The following is an (imaginary &amp; simplified) example of an interface on EditMesh called &apos;FaceOps&apos; with two functions, delete and extrude:</p>\n\
<p>It contains three parts:</p>\n\
<ul>\n\
<li>The public interface (in a public header file)</li>\n\
<li>The implementation class (in the .cpp implementation file)</li>\n\
<li>The interface definition (in the .cpp implementation file)</li>\n\
</ul>\n\
<h2 id=\"edit-mesh-interface-example\">Edit Mesh Interface Example</h2><h3 id=\"the-public-interface-in-a-public-header-file-\">The public interface (in a public header file)</h3><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#include&quot;iFnPub.h&quot;\n\
#define EM_FO_INTERFACE Interface_ID(0x434455, 0x65654)\n\
\n\
#define GetEMFaceOpsInterface(cd) \\\n\
   (EMFaceOps *)(cd)-&gt;GetFPInterface(EM_FO_INTERFACE)\n\
\n\
enum { em_delete, em_extrude, };\n\
\n\
class EMFaceOps : public  FPStaticInterface\n\
{\n\
   public:\n\
     virtual int Delete (Mesh* m, BitArray* faces)=0;\n\
     virtual void Extrude(Mesh* m, BitArray* faces, float amt)=0;\n\
};</code></pre><p>This defines the Interface_ID for the &apos;FaceOps&apos; interface and provides a helper macro for getting the interface pointer from the plug-in&apos;s ClassDesc. This is followed by an enum for the two function IDs and then the virtual interface class itself in which each function is declared as a pure virtual function. The return and parameter types of these functions are restricted to a fixed set, defined below. Fixing this set is necessary so that systems that access the interface metadata or use the dispatch form of calling have a known set of types to deal with. External code that wishes to call one of these functions, and has access to this public header can do so as in the following example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">EMFaceOps* efi = GetEMFaceOpsInterface(edmeshCD);\n\
//...\n\
efi-&gt;Extrude(mesh, faces, 10.0);</code></pre><p>To call this function indirectly, you call the Invoke() method on the interface, giving it the function ID and an <em>FPParams</em> object containing all the parameters, as in the following example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">FPParams p (3, TYPE_MESH, mesh,\n\
TYPE_BITARRAY, faces,\n\
TYPE_FLOAT, 10.0);\n\
FPValue result;\n\
FPInterface efi = edmeshCD-&gt;GetFPInterface(&quot;FaceOps&quot;);\n\
efi-&gt;Invoke(em_extrude, result, &amp;p);\n\
x = result.i;</code></pre><p>Any function result is passed back in an <code>FPValue</code> instance, given as one of the args to Invoke, which is a type-tagged variant structure that can hold any one of the FnPub supported types. The FPParams class contains a convenience varargs constructor for building parameter sets as shown. An FPParams instance contains a <code>Tab</code>&lt;&gt; of <code>FPValue</code> instances, each holding a parameter. This example demonstrates how to look up an interface by name in a ClassDesc. The Invoke() function has several overloads for calling functions with and without results and parameters.</p>\n\
<h3 id=\"the-implementation-class-in-the-cpp-implementation-file-\">The implementation class (in the .cpp implementation file)</h3><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class EMFaceOpsImp : publicEMFaceOps\n\
{\n\
   DECLARE_DESCRIPTOR(EMFaceOpsImp)\n\
   BEGIN_FUNCTION_MAP\n\
     FN_2(em_delete, TYPE_INT, Delete, TYPE_MESH, TYPE_BITARRAY)\n\
     VFN_3(em_extrude, Extrude, TYPE_MESH, TYPE_BITARRAY, TYPE_FLOAT)\n\
   END_FUNCTION_MAP\n\
\n\
   intDelete(Mesh* m, BitArray* faces)\n\
   {\n\
     //... do the delete\n\
     return face_count;\n\
   }\n\
   voidExtrude(Mesh* m, BitArray* faces, float amt)\n\
   {\n\
     //... do the extrude\n\
   }\n\
};</code></pre><p>The interface implementation class specializes the virtual interface class defined in the public header. It can contain any implementation-required data members and utility methods, but must at least contain implementations for each of the virtual methods defined in the virtual interface class.</p>\n\
<p>The key component shown above is the function map which generates the indirect call dispatcher used by the <code>Invoke()</code> method. This dispatcher unbundles the parameters from the indirect call parameter structure, forwards them to the correct implementation method, and bundles the return value into an <code>FPValue</code>. It is specified here using the map macros that come with the FnPub system (in  <em>iFnPub.h</em> ), in a manner somewhat similar to the message maps in MFC. You can implement this dispatcher by hand, but it is advised that you use the map macros.</p>\n\
<p>The map is bounded by <code>BEGIN_FUNCTION_MAP</code> and <code>END_FUNCTION_MAP</code> and contains one entry for each function in the interface. The map entry macros used come from a set named according to the number of arguments and whether the function is void. In this case, <code>FN_2</code> is used for a 2 argument function returning a value and <code>VFN_3</code> is used for a void function taking 3 arguments. The <code>FN_2</code> macro, for example, takes the function ID, the return type, the virtual interface method name, and the types of the two arguments. There are more details on these macros in the ref section below. There is a separate declaration macro, <code>DECLARE_DESCRIPTOR</code>(&lt; class&gt;) that must be specified in the interface descriptor class. In the implementation class for a static or action or core interface, you provide a single descriptor declarator with the <code>DECLARE_DESCRIPTOR()</code> macro, giving the current class as the macro parameter and a function map using <code>BEGIN_FUNCTION_MAP</code> and <code>END_FUNCTION_MAP</code> macros as before. The <code>DECLARE_DESCRIPTOR</code>(&lt; class&gt;) is only required for <code>FPStaticInterface</code> subclasses; <code>FPMixinInterface</code> subclasses should only have the <code>FUNCTION_MAP</code>/<code>END_FUNCTION_MAP</code> map table.</p>\n\
<h3 id=\"the-interface-definition-in-the-cpp-implementation-file-\">The interface definition (in the .cpp implementation file)</h3><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static EMFaceOpsImp emfi\n\
(\n\
   EM_FO_INTERFACE, _T(&quot;FaceOps&quot;), IDS_EMFO, &amp;edmeshCD, 0,\n\
   em_delete, _T(&quot;delete&quot;), IDS_DELETE, TYPE_INT, 0, 2,\n\
     _T(&quot;mesh&quot;),     IDS_MESH, TYPE_MESH,\n\
     _T(&quot;faces&quot;),  IDS_FACES, TYPE_BITARRAY,\n\
   em_extrude, _T(&quot;extrude&quot;), IDS_EXTRUDE, TYPE_VOID, 0, 3,\n\
     _T(&quot;mesh&quot;),     IDS_MESH, TYPE_ MESH,\n\
     _T(&quot;faces&quot;),  IDS_FACES, TYPE_BITARRAY,\n\
     _T(&quot;amount&quot;),  IDS_AMOUNT, TYPE_FLOAT,\n\
   p_end\n\
);</code></pre><p>A distinguished instance of the interface implementation class is constructed and registered with the owning ClassDesc. This is the instance given out when the CD is asked for an interface via its <code>GetFPInterface()</code> method. The instance has data members that contain all the metadata for the interface. You normally build this instance as a static in the implementing .cpp file, in a manner similar to ParmBlock2 descriptor instances.</p>\n\
<p>In the example above, the instance &apos;emfi&apos; is statically declared using the FPInterface&apos;s varargs constructor. The initial arguments to this constructor provide the <code>Interface_ID</code>, the internal fixed name, localizable descriptor string resID, owning ClassDesc and flags bits. This is followed by a list of function descriptors which themselves each have lists of parameter descriptors. The <code>em_delete</code> function description, for example, consists of a fixed name, a localizable description string resID, a return type, flag bits, and a count of the number of parameters. For each parameter, there is a line giving parameter name, description string resID, and type.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../function_publishing.html\">Function Publishing</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";