var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"mixin_interfaces\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Mixin Interfaces</h1></div><h2 id=\"object-based-mixin-interfaces\">Object Based Mixin Interfaces</h2><p>A variant of the FPInterface, known as a Mixin interface, is provided and can be multiply-inherited by a plug-in&apos;s class and returned via its implementation of the above new <code>GetInterface</code>(<code>Interface_ID</code>) method, in the way that most existing object-based interfaces are now implemented. Here&apos;s an example setup. First, the public header that would be used by an SDK-level client of the interface:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// interface ID\n\
#define FOO_INTERFACE Interface_ID(0x342323, 0x55664)\n\
#define GetFooInterface(obj) \\\n\
   ((FooInterface*)obj-&gt;GetInterface(FOO_INTERFACE))\n\
\n\
// function IDs\n\
enum { foo_move, foo_setRadius, };\n\
\n\
// mixin interface\n\
class FooInterface : public  FPMixinInterface\n\
{\n\
   BEGIN_FUNCTION_MAP\n\
     VFN_1(foo_move, Move, TYPE_POINT3);\n\
     VFN_1(foo_setRadius, SetRadius, TYPE_FLOAT);\n\
   END_FUNCTION_MAP\n\
\n\
    FPInterfaceDesc* GetDesc();\n\
   virtualvoid SetRadius(float radius)=0;\n\
   virtualvoid Move(Point3 p)=0;\n\
};</code></pre><p>This is much the same as existing stand-alone FnPub interfaces, except that the function map is put into the virtual interface class, rather than the implementing interface class. Then, in the plug-in class, you inherit the mixin interface (you are &quot;mixing&quot; it into the class). You provide implementations for the interface&apos;s virtual methods in the main class and an implementation of <code>GetInterface()</code> that returns the object cast to the interface, exactly as you would have done for old-style object-based interfaces:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class MyObject : public SimpleObject2, publicFooInterface\n\
{\n\
   public:\n\
     //...\n\
     void SetRadius(float radius);\n\
     void Move(Point3 p);\n\
     //...\n\
\n\
     FPInterface* GetInterface(Interface_ID id) {\n\
     if (id == FOO_INTERFACE)\n\
      return (FooInterface*)this;\n\
     else\n\
       return SimpleObject2::GetInterface(id);\n\
     }\n\
     //...\n\
}</code></pre><p>Note that the <code>GetInterface()</code> method needs to cast the &apos;<code>this</code>&apos; to the mixin interface class so the correct vtable is used (this was the case with old-style object interfaces, as well). It also calls <code>SimpleObject2::GetInterface()</code> if the id doesn&apos;t match so that other base class interfaces and stand-alone interfaces, if any, are made available to the caller. Finally, you provide a descriptor for the interface, as with stand-alone interfaces, but in this case, you must make it an instance of <code>FPInterface</code>, not FooInterface or MyObject, and must specify the <code>FN_MIXIN</code> flag to denote the interface as mixin:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static  FPInterfaceDesc foo_mixininterface(FOO_INTERFACE,\n\
   _T(&quot;foo&quot;), 0,\n\
   &amp;myObjDesc, FP_MIXIN,\n\
\n\
   foo_move, _T(&quot;move&quot;), 0, TYPE_VOID, 0, 1,\n\
     _T(&quot;vector&quot;), 0, TYPE_POINT3,\n\
   foo_setRadius, _T(&quot;setRadius&quot;), 0, TYPE_VOID, 0, 1,\n\
     _T(&quot;radius&quot;), 0, TYPE_FLOAT,\n\
   end\n\
);\n\
\n\
FPInterfaceDesc* FooInterface::GetDesc()\n\
{\n\
   return &amp;foo_mixininterface;\n\
}</code></pre><p>This static instance provides the interface metadata and is recorded in the class descriptor. To access the interface metadata, you must get this instance directly from the ClassDesc via <code>ClassDesc2::GetInterface()</code>, and not via <code>Animatable::GetInterface()</code> on an object.</p>\n\
<p>All this is pretty-much the same work you would do to provide an <code>Animatable::GetInterface()</code> interface currently in 3ds Max. The extra stuff is the <code>FUNCTION_MAP</code> and the interface descriptor. The SDK-level clients use the interface in exactly the same way, but now the scripter and other external systems can find and use these interfaces automatically at runtime.</p>\n\
<p>Mixin object-based interfaces are accessible in the scripter in similar way to stand-alone published interfaces. In particular, the functions are available in a struct function package which is itself accessed as a property on the plug-in class object. For example, if the above example plug-in class is named &apos;MyObject&apos; in the scripter, the functions in its &apos;foo&apos; mixin interface would be accessed as:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MyObject.foo.move\n\
MyObject.foo.setRadius</code></pre><p>The functions are effectively &apos;generic&apos; functions that require an instance of the plug-in as the first argument, resulting in:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MyObject.foo.move $baz [10,10,10]\n\
MyObject.foo.setRadius $bar 123.4\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../function_publishing.html\">Function Publishing</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";