var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"parameter_validation\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Parameter Validation</h1></div><p>An interface descriptor can now contain validation information for individual parameters, so that clients such as MAXScript can validate values given as parameters to <code>FPInterface</code> calls, prior to making the call. The validation info can be in the form of a range of values for int and float types, or more generally, a validator object that is called the validate a parameter value.</p>\n\
<p>MAXScript now applies the validations if supplied and generates descriptive runtime errors if the validation fails.</p>\n\
<p>The validation info is specified in the <code>FPInterface</code> descriptor in optional tagged entries following the parameters to be validated. The two possible tags are <code>f_range</code> and <code>f_validator</code>. Here&apos;s an example from a possible mixin interface to Cylinder:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static  FPInterfaceDesc cylfpi (\n\
   CYL_INTERFACE, _T(&quot;cylMixin&quot;), 0, &amp;cylinderDesc, FP_MIXIN,\n\
   //...\n\
\n\
   cyl_setRadius, _T(&quot;setRadius&quot;), 0, TYPE_VOID, 0, 1,\n\
     _T(&quot;radius&quot;), 0, TYPE_FLOAT, f_range, 0.0, 10000.0,\n\
   cyl_setDirection, _T(&quot;setDirection&quot;), 0, TYPE_VOID, 0, 1,\n\
     _T(&quot;vector&quot;), 0, TYPE_POINT3, f_validator, &amp;cylValidator,\n\
   //...\n\
   end\n\
);</code></pre><p>In this above example, the &quot;radius&quot; parameter is defined to have a range 0.0 to 10000.0. An <code>f_range</code> spec can only be used for int (<code>TYPE_INT</code>,<code>TYPE_TIMEVALUE</code>, <code>TYPE_RADIOBTN_INDEX</code>, <code>TYPE_INDEX</code>, <code>TYPE_ENUM</code>) and float parameters (<code>TYPE_FLOAT</code>,<code>TYPE_ANGLE</code>, <code>TYPE_PCNT_FRAC</code>, <code>TYPE_WORLD</code>, <code>TYPE_COLOR_CHANNEL</code>) and is given as a pair of low and high range values. The values must be floating point or integer as needed by the TYPE_xxx code, you cannot specify integer range values for float types and vice versa, hence the 0.0 and 10000.0 in the example above. MAXScript checks parameter values against these supplied ranges and will generate a descriptive error message for out-of-range values.</p>\n\
<p>The &quot;vector&quot; parameter in the above example has a validator object specified. This must be a pointer to an instance of a class derived from the new class, <code>FPValidator</code>, defined in <code>iFnPub</code>.h. This is a virtual base class, containing a single method, <code>Validate()</code>, that is called to validate a prospective value for a parameter. You would typically subclass <code>FPValidator</code> in your code and provide an implementation of <code>Validate()</code> to do the validation. Here is the <code>FPValidator</code> virtual base class:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class FPValidator\n\
{\n\
   public:\n\
     // validate val for the given param in function in interface\n\
     virtual bool Validate(FPInterface* fpi, FunctionID fid,\n\
     int paramNum, FPValue&amp; val, TSTR&amp; msg)=0;\n\
};</code></pre><p>The <code>Validate()</code> function is called with interface, function-within-interface and parameter-within-function identifiers and an <code>FPValue</code> to validate. It can optionally install an error message string in the &apos;msg&apos; TSTR&amp; parameter for the user of the validator to display. If there are many parameters to validate this way, you can choose to provide a separate subclass for each parameter or a single subclass and switch on the parameter identification supplied.</p>\n\
<p>For the Cylinder example above, the &quot;vector&quot; parameter might be required to be given as a unit vector, so the validator might check for this as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class CylValidator : public FPValidator\n\
{\n\
   bool Validate(FPInterface* fpi,\n\
         FunctionID fid,\n\
         int paramNum,\n\
         FPValue&amp; val,\n\
         TSTR&amp; msg)\n\
   {\n\
     if (fabs(Length(*val.p) - 1.0) &gt; 1e-6)\n\
     {\n\
      msg = &quot;Direction vector must be unit length.&quot;\n\
      return false;\n\
     }\n\
     else\n\
     {\n\
      return true;\n\
     }\n\
   }\n\
};</code></pre><p>Note that the type is already checked by the caller, since it has type info for the parameter, so you don&apos;t also need to check the <code>FPValue</code> type.</p>\n\
<p>Note, also, that the <code>FPValue</code>&amp; val argument given to <code>Validate()</code> is a reference to the actual parameter value to be given to the called function, so it is also possible to massage the value rather than reporting an error. For example, the above sample <code>Validate()</code> method could normalize the vector and always return true. The type must not be changed in this process, only the value can be adjusted.</p>\n\
<p>A singleton instance of this <code>FPValidator</code> subclass would created, typically as a static instance, to be given in the <code>f_validator</code> specification: <code>static</code> CylValidator cylValidator;</p>\n\
<p>If you need to get at the validation metadata, it is available in the new &apos;options&apos; data member in an <code>FPParamDef</code> instance for a parameter. If non-NULL, this points to a FPParamOptions instance which contains the range or validator information for the parameter. See class FPParamOptions in <code>iFnPub</code>.h for details.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../function_publishing.html\">Function Publishing</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";