var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"property_accessors\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Property Accessors</h1></div><p>It is possible to expose C++ functions to maxscript interfaces as a property, rather an as a function. A <code>LayerProperties</code> interface for example has properties such as <code>on</code>, <code>lock</code>, <code>current</code> etc. that can be read and written to. These properties remain as C++ level functions, but are accessed as properties in MAXScript by using the standard dot-notation for members.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">L = layermanager.getlayer 1\n\
&lt;MixinInterface:LayerProperties&gt;\n\
showinterface L\n\
  Interface: LayerProperties\n\
   Properties:\n\
    .on : boolean : Read|Write\n\
    .lock : boolean : Read|Write\n\
    .current : boolean : Read|Write\n\
    .wireColor : color by value : Read|Write\n\
    .isGIExcluded : boolean : Read|Write\n\
    ...\n\
--For example\n\
L.on\n\
true\n\
L.on = false\n\
false</code></pre><p>It is possible to define selected methods in an <code>FPInterface</code> as &apos;property accessor&apos; methods so that dispatch-based clients of the interface may present these methods as properties instead of functions. A &apos;property&apos; is typically defined by a pair of accessor methods, one for getting and one for setting, but you can also define read-only methods that have a single getter method. As an example, here&apos;s a rework of the previous Cylinder interface in which the radius and direction are made properties.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">classCylInterface : public  FPMixinInterface\n\
{\n\
   ...\n\
   virtualvoid RemoveCaps()=0;\n\
   virtualvoid AddBend(float offset, float angle, float radius)=0;\n\
   ...\n\
   virtualfloat GetRadius()=0;\n\
   virtualvoid SetRadius(float radius)=0;\n\
   virtualPoint3 GetDirection()=0;\n\
   virtualvoid SetDirection(Point3 dir)=0;\n\
   ...\n\
\n\
   BEGIN_FUNCTION_MAP \n\
     ... \n\
     VFN_0(cyl_removeCaps, RemoveCaps);\n\
     VFN_3(cyl_addBend, AddBend, TYPE_FLOAT, TYPE_ANGLE, TYPE_FLOAT);\n\
     ...\n\
     PROP_FNS(cyl_getRadius, GetRadius, cyl_setRadius, SetRadius, TYPE_FLOAT);\n\
     PROP_FNS(cyl_getDir, GetDirection, cy_setDir, SetDirection, TYPE_POINT3_BV);\n\
     ...\n\
   END_FUNCTION_MAP\n\
\n\
    FPInterfaceDesc* GetDesc();\n\
};</code></pre><p>Each property has a getter and setter virtual method. Their signatures must conform to the convention shown, namely, the getter takes no parameters and the getter one. The getter returns the same type as the setter and the setter type is void. There are also variants that take an explicit time as described below.</p>\n\
<p>Properties have single entries in the function map, using one of the property-related entry macros. In this case, <code>PROP_FNS()</code> takes a getter ID, getter function, setter ID, setter function and finally the property type code. There is also a <code>RO_PROP_FN()</code> macro for read-only properties that takes a getter ID, getter FN and prop type. Finally, the are variants of <code>PROP_FNS</code> and <code>RO_PROP_FN</code> that indicate the accessors take an explicit <code>TimeValue</code> argument, presumably associated with animatable properties and indicating that the property access should be at the given time. These macros are <code>PROP_TFNS</code> and <code>RO_PROP_TFN</code> and takes the same arguments as their corresponding base macros. If the above example properties were time-sensitive, the definitions would be as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">classCylInterface : public  FPMixinInterface\n\
{\n\
   ...\n\
   virtualvoid RemoveCaps()=0;\n\
   virtualvoid AddBend(float offset, float angle, float radius)=0;\n\
   ...\n\
   virtualfloat GetRadius(TimeValue t)=0;\n\
   virtualvoid SetRadius(float radius, TimeValue t)=0;\n\
   virtualPoint3 GetDirection(TimeValue t)=0;\n\
   virtualvoid SetDirection(Point3 dir, TimeValue t)=0;\n\
   ...\n\
\n\
   BEGIN_FUNCTION_MAP \n\
     ... \n\
     VFN_0(cyl_removeCaps, RemoveCaps);\n\
     VFN_3(cyl_addBend, AddBend, TYPE_FLOAT, TYPE_ANGLE, TYPE_FLOAT);\n\
     ...\n\
     PROP_TFNS(cyl_getRadius, GetRadius, cyl_setRadius, SetRadius, TYPE_FLOAT);\n\
     PROP_TFNS(cyl_getDir, GetDirection, cy_setDir, SetDirection, TYPE_POINT3_BV);\n\
     ...\n\
   END_FUNCTION_MAP\n\
    FPInterfaceDesc* GetDesc();\n\
};</code></pre><p>Recall that there are time-sensitive variants for the <code>FN_0</code>, <code>FN_1</code>, etc., macros, namely <code>TFN_0</code>, <code>TFN_1</code>, etc. Functions specified by such macros in the function map have an implicit <code>TimeValue</code> last parameter. Note that in all the time-sensitive variants, the last <code>TimeValue</code> parameter is not defined as an explicit parameter in the <code>FPInterface</code> descriptor entry for that function. Such time-sensitive properties and functions are handled specially in MAXScript. It supplies the current MAXScript time, as defined by the current &apos;at time&apos; context for this implicit parameter, making these functions and properties behave consistently with the rest of MAXScript.</p>\n\
<p>Properties are defined in a special section in the <code>FPInterface</code> descriptor constructor following the function definitions, headed by the special tag &apos;properties&apos;. They are also entered in the function map using new property-specific <code>FUNCTION_MAP</code> macros. Here&apos;s an example descriptor fragment:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">staticFPInterfaceDesc cylfpi (\n\
   CYL_INTERFACE, _T(&quot;cylMixin&quot;), 0, &amp;cylinderDesc, FP_MIXIN,\n\
   ...\n\
   cyl_removeCaps, _T(&quot;removeCaps&quot;), 0, TYPE_VOID, 0, 0,\n\
   cyl_addBend, _T(&quot;addBend&quot;), 0, TYPE_VOID, 0, 3,\n\
   _T(&quot;offset&quot;), 0, TYPE_FLOAT,\n\
   _T(&quot;angle&quot;), 0, TYPE_ANGLE,\n\
   _T(&quot;radius&quot;), 0, TYPE_FLOAT,\n\
\n\
   properties,\n\
   cyl_getRadius, cyl_setRadius, _T(&quot;radius&quot;), IDS_RADIUS,\n\
   TYPE_FLOAT, f_range, 0.0, 10000.0,\n\
   cyl_getDirection, cyl_setDirection, _T(&quot;direction&quot;), IDS_DIR,\n\
   TYPE_POINT3_BV,\n\
   ...\n\
   end\n\
);</code></pre><p>The &apos;properties&apos; section follows the function definitions. Each propery has a single entry defining the function IDs for the getter and setter functions, a fixed internal property name, a descriptor string resource ID and the property type. If the property is read-only and there is no setter function, specify <code>FP_NO_FUNCTION</code> for the setter ID. Each property definition can optionally be followed by parameter validation options, as described in the Parameter Validation section above, and these apply to the parameter given to the setter function during property assignment. In this case, the radius is range checked.</p>\n\
<p>If you need to get at it, the property metadata is accessible in the &apos;props&apos; data member in an <code>FPInterfaceDesc</code>. It is a <code>Tab</code>&lt;&gt; of pointers to <code>FPPropDef</code> class instances, which contain individual property metadata. See the <code>FPPropDef</code> class definition in <code>iFnPub</code>.h for details.</p>\n\
<h2 id=\"maxscript-convention\">Maxscript Convention</h2><p>MAXScript now exposes properties defined in this way as direct dot-notation properties on the interface. So, where before the Get/SetRadius in the example cylinder mixin would have been accessed as functions:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">r = $cyl01.cylMixin.getRadius()\n\
$cyl01.cylMixin.setRadius 23</code></pre><p>you now can use:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">r = $cyl01.cylMixin.radius\n\
$cyl01.cylMixin.radius = 23</code></pre><p>This is true for all the types of FP interfaces that turn up in MAXScript, static, mixin and core. As a futher optimization, MAXScript now effectively promotes all interface methods and properties to the level of the interface, so if individual methods and properties have unqiue names within all the interfaces of an object or</p>\n\
<p>class, you can eliminate the interface name. The above examples could now be written:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">r = $cyl01.getRadius()\n\
$cyl01.setRadius 23</code></pre><p>and:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">r = $cyl01.radius\n\
$cyl01. radius = 23</code></pre><p>If there is a naming conflict, you can always include the interface name level to resolve this.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../function_publishing.html\">Function Publishing</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";