var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"publishing_mixin_interface_on_ar\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Publishing Mixin Interface on Arbitrary Classes</h1></div><h2 id=\"publishing-mixin-interface-on-arbitrary-classes\">Publishing Mixin Interface on Arbitrary Classes</h2><p>You can publish FnPub mixin interfaces on any class, not just Animatable subclasses. To do this, you inherit from, <code>IObject</code>, a new virtual base class in the FnPub system. This provides an API for querying and iterating FPMixinInterfaces that the class wishes to publish, similar to <code>Animatable::GetInterface()</code>. The API that the publishing class must implement is as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class IObject\n\
{\n\
   public:\n\
     // object/class name\n\
     virtual TCHAR* GetName()=0;\n\
\n\
     // iterate over all interfaces...\n\
     virtual int NumInterfaces()=0; \n\
     virtual FPInterface* GetInterface(inti)=0;\n\
\n\
     // get ID&apos;d interface\n\
     virtual FPInterface* GetInterface(Interface_ID id)=0;\n\
\n\
     // IObject ref management (can be implemented by dynamically-\n\
     // allocated IObjects for ref-count based lifetime control)\n\
      virtual void Acquire() { }; \n\
     virtual void Release() { };\n\
};</code></pre><p>There is a corresponding new ParamType2 type code, <code>TYPE_IOBJECT</code>, that allows instances of these classes to be passed and returned in FPInterface methods, providing a simple form of user-defined type, in the sense that these instance collections are passed as interfaces rather than pointers (similar to COM). MAXScript has been extended to provide wrapper value classes for IObjects and so this mechanism provides a light-weight alternative to the MAXScript SDK facilities for adding new wrapper value classes to the scripter.</p>\n\
<p>MAXScript also calls the <code>Acquire()</code> and <code>Release()</code> methods on IObjects as it creates and collects these wrappers, so that IObject objects can keep track of MAXScript&apos;s extant references to them. For example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class IFoo1 : public  FPMixinInterface\n\
{\n\
   virtual void Frabulate(Point3 p)=0;\n\
   //...\n\
};\n\
\n\
class IFoo2 : public  FPMixinInterface\n\
{\n\
   //...\n\
};\n\
\n\
class Foo : public IObject, publicIFoo1, publicIFoo2\n\
{\n\
   // Foo methods\n\
   //...\n\
\n\
   // IObject methods\n\
   TCHAR* GetName() { return _T(&quot;Foo&quot;); }\n\
   int NumInterfaces() { return 2; }\n\
\n\
   FPInterface* GetInterface(inti)\n\
   {\n\
     if (i == 0) return (IFoo1*)this;\n\
     if (i == 1) return (IFoo2*)this;\n\
   }\n\
\n\
   FPInterface* GetInterface(Interface_ID id)\n\
   {\n\
     if (id == FOO_INTERFACE_1) return (IFoo1*)this;\n\
     if (id == FOO_INTERFACE_2) return (IFoo2*)this;\n\
   }\n\
   // IFoo1 methods\n\
   void Frabulate(Point3 p) { ... }\n\
   //...\n\
   // IFoo2 methods\n\
   //...\n\
};</code></pre><p>Instances of Foo can be passed as parameters and results in FnPub interface descriptors and function maps by declaring them as <code>TYPE_IOBJECT</code> values, and they will get cast to <code>IObject</code>* automatically. For example, in a plug-in that uses the Foo class, a method in one of its FPInterfaces might want to return a Foo instance. The method might be defined as:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Foo* GetFoo(INode* object)\n\
{\n\
   Foo* x = new Foo (...);\n\
   //...\n\
   return x;\n\
}</code></pre><p>Since Foo is not a supported base ParamType2 type but is derived from <code>IObject</code>, the return value of the above method can be declared using <code>TYPE_IOBJECT</code> in the <code>FUNCTION_MAP</code> as:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">FN_1(my_getFoo, TYPE_IOBJECT, GetFoo, TYPE_INODE);</code></pre><p>and in the descriptor constructor as:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">my_getFoo, _T(&quot;getFoo&quot;), 0, TYPE_IOBJECT, 0, 1,\n\
     _T(&quot;object&quot;), 0, TYPE_INODE,</code></pre><p>In MAXScript, calling <code>getFoo()</code> would return an <code>IObject</code> value that has two interface properties, ifoo1 and ifoo2, and these in turn would expose their methods as properties:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">f = getFoo $\n\
f.ifoo1.frabulate [10,0,0] --call the IFoo1 frabulate method\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../function_publishing.html\">Function Publishing</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";