var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"symbolic_enumerations\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Symbolic Enumerations</h1></div><p>One or more symbolic enums, similar to C++ enums, can now be added to an FPInterface&apos;s metadata, and individual int parameters and/or results for functions in that interface can be defined as <code>TYPE_ENUM</code> and associated with one of the enum lists. This allows metadata clients to support symbolic encodings for these parameters and results, which MAXScript now does.</p>\n\
<p>Enums are defined in the FPInterface descriptor following the function and property definitions as sets of string/code pairs. Each enum list is identified by a unique integer, similar to function IDs, which is used to associated a <code>TYPE_ENUM</code> parameter or result with its enum. IDs for these would normally be defined somewhere near the function IDs for an interface. For example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// function IDs\n\
enum { bmm_getWidth, bmm_getHeight, bmm_getType, bmm_copyImage, ...};\n\
\n\
// enum IDs\n\
enum { bmm_type, bmm_copy_quality, ...};</code></pre><p>might be some of the IDs for a possible bitmap manager interface. The two enums provide symbolic codes for the bitmap type and copyImage quality defines in the <code>bitmap</code>.h SDK header, such as <code>BMM_PALETTED</code>,<code>BMM_TRUE_32</code>, <code>COPY_IMAGE_RESIZE_LO_QUALITY</code>, etc. In the descriptor for the interface, any enum lists follow the function and property definitions. They are introduced by the special tag, &apos;enums&apos;, as in the following example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static  FPInterfaceDesc bmmfpi (\n\
   BMM_INTERFACE, _T(&quot;bmm&quot;), IDS_BMMI, NULL, FP_CORE,\n\
   //...\n\
   bmm_copyImage, _T(&quot;copyImage&quot;), //...\n\
   _T(&quot;copyType&quot;), IDS_COPYTYPE, TYPE_ENUM, bmm_copy_quality,\n\
   //...\n\
\n\
   properties,\n\
     geo_getType, geo_setType, _T(&quot;type&quot;), 0, TYPE_ENUM, bmm_type,\n\
\n\
   enums,\n\
     bmm_type, 7,\n\
       &quot;lineArt&quot;, BMM_LINE_ART,\n\
       &quot;paletted&quot;, BMM_PALETTED,\n\
       &quot;gray8&quot;, BMM_GRAY_8,\n\
       &quot;gray16&quot;, BMM_GRAY_16 ,\n\
       &quot;true16&quot;, BMM_TRUE_16,\n\
       &quot;true32&quot;, BMM_TRUE_32,\n\
       &quot;true24&quot;, BMM_TRUE_64,\n\
     bmm_copy_quality, 4,\n\
       &quot;crop&quot;, COPY_IMAGE_CROP,\n\
       &quot;resizeLo&quot;, COPY_IMAGE_RESIZE_LO_QUALITY,\n\
       &quot;resizeHi&quot;, COPY_IMAGE_RESIZE_HI_QUALITY,\n\
       &quot;useCustom&quot;, COPY_IMAGE_USE_CUSTOM,\n\
   end\n\
);</code></pre><p>In the above example, the enums are listed following the function &amp; property definitions. They are introduced by the &apos;enums&apos; tag and consist of an enum ID followed by a count of items, followed by that many string and code pairs. By attaching them to the interface like this, any number of functions and properties in the interface can use them.</p>\n\
<p>The above example also has function and property definitions showing the use of TYPE_ENUM. The <code>copyImage</code> function takes a <code>copyType</code> parameter which uses the <code>bmm_copy_quality</code> enum and the type property uses the <code>bmm_type</code> enum. In all situations where TYPE_xxx types can be supplied in a descriptor, including the new property definitions, <code>TYPE_ENUM</code> can be used to indicate an int by-value type. <code>TYPE_ENUM</code> &apos;s must always be followed by an enum ID. This is the only case in which the type is specified as a pair of values. <code>TYPE_ENUM</code> parameters and results show up in MAXScript as # names. For example, if a bmm interface was in the variable &apos;bm1&apos; and the bitmap type was <code>BMM_GRAY_16</code>,</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">bm1.type</code></pre><p>-&gt; #gray16</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">bm1.type = #true32 --set it to #true24 (code is BMM_TRUE_24)\n\
bm2 = bm1.copyImage #resizeHi</code></pre><p>the integer <code>TYPE_ENUM</code> codes are translated back-and-forth to symbolic # names by MAXScript using the definitions in the FPInterface descriptor&apos;s enums. If you need to access the enum metadata in an <code>FPInterfaceDesc</code>, it is available in the &apos;enumerations&apos; data member. This is a <code>Tab</code>&lt;&gt; of pointers to <code>FPEnum</code> class instances which themselves contain a Tab&lt;&gt; of name, code pairs. See class FPEnum in <code>iFnPub</code>.h for details.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
      <div class=\"relinfo relchildren\">\n\
        <strong>Pages in this section</strong>\n\
        <ul class=\"ullinks\">\n\
          <li class=\"link ulchildlink\"><a href=\"symbolic_enumerations/additional_paramtype2_codes_for_.html\">Additional ParamType2 Codes for Function Publishing and MAXScript</a></li>\n\
          <li class=\"link ulchildlink\"><a href=\"symbolic_enumerations/parameterresult_types.html\">Parameter/Result Types</a></li>\n\
          <li class=\"link ulchildlink\"><a href=\"symbolic_enumerations/added_base_types.html\">Added Base Types</a></li>\n\
        </ul>\n\
      </div>\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../function_publishing.html\">Function Publishing</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";