var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"the_function_publishing_api\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>The Function Publishing API</h1></div><p>The Function Publishing API consists of two main components, a function descriptor system and a function calling mechanism. The descriptor system is used by the publishing component to declare the functions it is publishing and provide necessary descriptive data and may be used by a component&#39;s client to enquire about published functions. The calling system is used by a component&#39;s client to invoke one of the published functions.</p>\n\
<h2 id=\"interfaces\">Interfaces</h2><p>Functions are published in one or more <code>Interfaces</code> by your plug-in. If you have a large number of functions to be published, you might organize them functionally into different interfaces to make them more manageable for the user. For example, EditMesh might publish vertex functions, edge functions and face functions in separate interfaces. Action functions must be published in their own (set of) interface(s). Each interface is represented by an instance of a class derived from the base class <code>FPInterface</code>. You normally create these instances as static objects using the FPInterface constructor, in much the same way as ParamBlock2 descriptors are created. Each interface contains a list of the functions it publishes and the parameters they take, along with type and name and other descriptive info for all these things. All the interfaces for a particular plug-in class are kept in its ClassDesc object. An external system can find out about the interfaces you publish by calling various query methods on ClassDesc that access these interface definition objects. As well as these enquiry or &#39;reflection&#39; methods, an FPInterface also has the calling methods for actually invoking a particular function in the interface, so that if something has hold of one of your interfaces, it can call any of its published functions.</p>\n\
<p>Additionally, a <code>Mixin</code> interface is provided which can be multiply-inherited by a plug-in&#39;s class and returned via its implementation of the above new <code>GetInterface</code>(<code>Interface_ID</code>) method, in the way that most existing object-based interfaces are now implemented.</p>\n\
<p>The FPInterface class is defined in the header file<code>iFnPub.h</code>, along with all the other FnPub classes &amp; macros described in these notes.</p>\n\
<h2 id=\"direct-and-indirect-calling\">Direct and Indirect Calling</h2><p>The function publishing system lets you set up interfaces so that functions in them can be called directly, as virtual member functions of the interface object, or indirectly via a dispatching method that takes a runtime function ID and a table of parameters. This is roughly equivalent to the dual vtable and <code>IDispatch</code> interface schemes in COM. Typically, you provide the virtual interface in a public header file so that it can be compiled against by other plug-ins and they can call the virtual functions on an interface object directly. The indirect call mechanism is used by MAXScript and other external systems that look for the published interface functions at runtime using the interface metadata.</p>\n\
<h2 id=\"interface-and-function-ids\">Interface and Function IDs</h2><p>Each interface is uniquely indentified by an <code>Interface_ID</code>, which is a new class in the R4 SDK. This class is structurally very similar to Class_ID, containing two randomly-chosen longwords to provide a unique global ID. It is defined in <code>maxtypes</code>.h</p>\n\
<p>Each function in an interface is identified by an integer ID of type <code>FunctionID</code>. This is similar to the BlockID and ParamID&#39;s used in the paramblock2 system. This ID is used in the dispatch-based calling mechanism to identify the function to call.</p>\n\
<h2 id=\"interface-organization\">Interface Organization</h2><p>The map is bounded by <code>BEGIN_FUNCTION_MAP</code> and <code>END_FUNCTION_MAP</code> and contains one entry for each function in the interface. The map entry macros used come from a set named according to the number of arguments and whether the function is void. In this case, <code>FN_2</code> is used for a 2 argument function returning a value and <code>VFN_3</code> is used for a void function taking 3 arguments. The <code>FN_2</code> macro, for example, takes the function ID, the return type, the virtual interface method name, and the types of the two arguments. <code>FN_VA</code>, <code>VFN_VA</code>, <code>FNT_VA</code>, etc., FUNCTION_MAP entry macro variants have same args as <code>\\_0</code> macros and specify that the function takes a variable number of arguments (passed directly in <code>FPParams</code> instances). There are more details on these macros in the ref section below. There is a separate declaration macro, <code>DECLARE_DESCRIPTOR</code>(&lt; class&gt;) that must be specified in the interface descriptor class. In the implementation class for a static or action or core interface, you provide a single descriptor declarator with the <code>DECLARE_DESCRIPTOR()</code> macro, giving the current class as the macro parameter and a function map using <code>BEGIN_FUNCTION_MAP</code> and <code>END_FUNCTION_MAP</code> macros as before. The <code>DECLARE_DESCRIPTOR</code>(&lt; class&gt;) is only required for <code>FPStaticInterface</code> subclasses; <code>FPMixinInterface</code> subclasses should only have the <code>FUNCTION_MAP</code>/<code>END_FUNCTION_MAP</code> map table.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../function_publishing.html\">Function Publishing</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";