var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"using_fpinterfaceinvoke\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Using FPInterface::Invoke()</h1></div><p>In some cases one might end up with a pointer to an object derived from <code>FPInterface</code>, which has some methods published using the function publishing system, but that are not exported in a .LIB or .DLL file. For example a third party plug-in. In this case one may used <code>FPInterface::Invoke()</code>.</p>\n\
<p>Any call to <code>FPInterface::Invoke()</code> should be surrounded in a try/catch block. This is because the MAXScript system throws exceptions to convey error messages. If an <code>FPInterface</code> function throws an exception for any reason and we do not catch it, then Max will crash.</p>\n\
<p>Another consideration when using <code>FPInterface::Invoke()</code> is that function IDs may change without notice, causing your plug-in to crash in the future. To avoid this you should always use <code>FPInterface::FindFn()</code> to find the appropriate function ID for a function name.</p>\n\
<p>When calling <code>FPInterface::Invoke()</code> the user must supply an <code>FPParams</code> object which is a <code>Tab</code>&lt;&gt; collection of <code>FPParam</code> values.</p>\n\
<p>The example code below gives an example of how to open the track view using <code>ITrackViewArray::OpenTrackViewWindow()</code> which takes six arguments.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> This is just an example of how to set-up a call to <code>FPInterface::Invoke()</code>. We do not recommend using this technique to access 3ds Max functionality that is exposed to MAXScript but not the SDK. In most cases there is a better way to achieve the desired effect using the SDK, or it is simply not supported by 3ds Max.</div>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">\n\
ITrackViewArray* myTrackView = ( ITrackViewArray*)GetCOREInterface(ITRACKVIEWS);\n\
\n\
FPParams fnParams;\n\
FPValue result, param1, param2, param3, param4, param5, param6;\n\
FPValue param1_FPValue;\n\
TCHAR* param1_TCHAR = _T(&quot;Test Track View&quot;);\n\
param1_FPValue.type = TYPE_STRING,\n\
param1_FPValue.s = param1_TCHAR;\n\
\n\
// getTrackView index must be 1-based\n\
// function map indicates parameter 1 is type TYPE_FPVALUE (accepts int or string)\n\
param1.type = (ParamType2)TYPE_FPVALUE,\n\
param1.fpv = &amp;param1_FPValue;\n\
fnParams.params.Append( 1, &amp;param1 );\n\
\n\
// function map indicates parameter 2 is type TYPE_TSTR\n\
TSTR param2_TSTR = TSTR( _T(&quot;&quot;) );\n\
param2.type = TYPE_TSTR,\n\
param2.tstr = &amp;param2_TSTR;\n\
fnParams.params.Append( 1, &amp;param2 );\n\
\n\
// function map indicates parameter 3 is type TYPE_POINT2\n\
Point2 param3_Point2 = Point2(-1.0f,-1.0f);\n\
param3.type = TYPE_POINT2,\n\
param3.p2 = &amp;param3_Point2;\n\
fnParams.params.Append( 1, &amp;param3 );\n\
\n\
// function map indicates parameters 4 and 5 are type TYPE_INT\n\
param4.type = (ParamType2)TYPE_INT,\n\
param4.i = -1;\n\
param5.type = (ParamType2)TYPE_INT,\n\
param5.i = -1;\n\
fnParams.params.Append( 1, &amp;param4 );\n\
fnParams.params.Append( 1, &amp;param5 );\n\
\n\
// function map indicates parameter 6 is type TYPE_ENUM (integer)\n\
param6.type = TYPE_ENUM,\n\
param6.i = (int)TV_FLOAT;\n\
fnParams.params.Append( 1, &amp;param6 );\n\
\n\
try\n\
{\n\
  myTrackView-&gt;Invoke( ITrackViewArray::openTrackView, result, &amp;fnParams );\n\
}\n\
catch (...)\n\
{\n\
  // an error occured\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../function_publishing.html\">Function Publishing</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";