var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"maxscript_functions_with_optiona\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>MAXScript Functions with Optional Arguments</h1></div><p>MAXScript functions also support optional arguments, using a mechanism called keyword argument passing. The keyword arguments are placed in the argument list of the C++ function after all the mandatory positional argument, and are preceded by a marker value. The marker value is accessible in the global variable <code>keyarg_marker</code> which has type <code>Value</code>*. The arguments are passed in pairs, a keyword name and the argument&apos;s value, in the <code>arg_list</code> array.</p>\n\
<p>There are several macros to help extract keyword arguments defined in the header file <code>maxscrpt/maxscrpt.h</code>. These macros assume that the C++ function implementation uses the parameter names <code>arg_list</code> and <code>count</code>.</p>\n\
<ul>\n\
<li><code>key_arg()</code> - gets the named argument value or returns <code>&amp;unsupplied</code> if not present</li>\n\
<li><code>key_arg_or_default()</code> - gets the named argument or the default value &apos;def&apos; if not supplied</li>\n\
<li><code>int_key_arg()</code> - gets the named key argument, and places it into the <code>Value</code>*<code>var</code> variable. The result is retuned as a C++ int, or else the result <code>def</code> is returned if the key argument is not present. There also similar versions for <code>float</code> and <code>BOOL</code> types.</li>\n\
<li><code>check_arg_count_with_keys()</code> - Checks for positional argument counts in the presence of keyword arguments.</li>\n\
</ul>\n\
<h2 id=\"examples\">Examples</h2><p>The following &apos;delete_file&apos; example shows an argument count check, conversion of first argument (arg_list[0]) to a string and the return of MAXScript &apos;true&apos; or &apos;false&apos; values.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">def_visible_primitive(delete_file, &quot;deleteFile&quot;);\n\
\n\
Value* delete_file_cf(Value** arg_list, int count)\n\
{\n\
  // deleteFile &quot;file_name&quot;\n\
  check_arg_count(&quot;deleteFile&quot;, 1, count);\n\
  BOOL result = DeleteFile(arg_list[0]-&gt;to_string());\n\
  return result ? &amp;true_value : &amp;false_value;\n\
}</code></pre><p>The &apos;file_in&apos; example below shows working with an optional keyword argument &apos;quiet&apos;. The required argument count is checked with the macro <code>check_arg_count_with_keys()</code> in this case. Two typed value locals are declared, one to hold the temporary <code>FileStream</code> instance and the other to hold the <code>fileIn</code> result.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">def_visible_primitive(file_in, &quot;fileIn&quot;);\n\
\n\
Value* file_in_cf(Value** arg_list, int count)\n\
{\n\
  // fileIn &quot;filename&quot; [quiet:true]\n\
  check_arg_count_with_keys(&quot;fileIn&quot;, 1, count);\n\
\n\
  // declare local variables\n\
  two_typed_value_locals(FileStream* file, Value* result);\n\
\n\
  Value* quiet = key_arg_or_default(quiet, &amp;true_value);\n\
  type_check(quiet, Boolean, &quot;fileInquiet:&quot;);\n\
  Value* result;\n\
\n\
  // open a temporary fileStream\n\
  vl.file = (new FileStream)-&gt;open( arg_list[0]-&gt;to_string(), &quot;rt&quot;);\n\
  if (vl.file == (FileStream*)&amp;undefined)\n\
  throwRuntimeError (GetString(IDS_FILEIN_CANT_OPEN_FILE), arg_list[0]);\n\
\n\
  // pass it to the stream-based file_in\n\
  try\n\
  {\n\
    vl.result = file_in(vl.file, (quiet == &amp;true_value));\n\
  }\n\
  catch (...)\n\
  {\n\
    // catch any errors and close the temp filestream\n\
    vl.file-&gt;close();\n\
    throw;\n\
  }\n\
\n\
  // pop value locals &amp; return fileIn result\n\
  return_value(vl.result);\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../advanced_maxscript_topics.html\">Advanced MAXScript Topics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";