var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"ik_concepts\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>IK Concepts</h1></div><h2 id=\"degrees-of-freedom\">Degrees of Freedom</h2><p>A node transform contains six degrees of freedom. Three are translational (sliding) and three are rotational. In the API, they have enum names, <code>IKSys::TransX</code> to <code>IKSys::TransZ</code> and <code>IKSys::RotX</code> to <code>IKSys::RotZ</code>. When the context implies rotational or translational, <code>IKSys::DofX</code> to <code>IKSys::DofZ</code> are used.</p>\n\
<p>A degree of freedom can be active or inactive. An inactive degree of freedom is not to be used by IK. In other words, only active degrees of freedom are degrees of freedom as far as IK is concerned. The methods <code>DofActive()</code>, <code>ActiveTrans()</code>, <code>ActiveRot()</code>, and <code>ActiveDofs()</code> are provided to determine which degrees of freedom are active and inactive. The first query answers whether a specific (translational / rotational) axis is active. The following three return a set of DOFs, which is defined in the same file. The set returned by <code>ActiveTrans()</code> and <code>ActiveRot()</code> is to be tested by <code>IKSys::DofX</code> to <code>IKSys::DofZ</code>, and the set returned by <code>ActiveDofs()</code> is to be tested by <code>IKSys::TransX</code> to <code>IKSys::RotZ</code>.</p>\n\
<h2 id=\"relationship-to-ik-chains\">Relationship to IK Chains</h2><p>A node transformation can be conceptually viewed as consisting of two joints, sliding joint (translational degrees of freedom) and rotational joint. The enum type, <code>IKSys::JointType</code>, contains two enums: <code>IKSys::SlidingJoint</code> and <code>IKSys::RotationalJoint</code>. An IK chain starts at the rotational joint of the Start Joint and ends at the sliding joint of the End Joint. Two IK chains covering the same joint, are considered overlapping IK chains.</p>\n\
<p>Let&#39;s call a node an IK chain node if its TM controller supports interface <code>IIKChainControl</code>, i.e., <code>node</code>-&gt;GetTMController()-&gt;<code>GetInterface</code>(I_IKChainControl) != <code>NULL</code>. The following query returns a list of IK chain nodes; <code>InodeTab</code> IKChains(<code>JointType</code>) const</p>\n\
<p>The parameter that decides whether an individual degree of freedom is (IK) active is not animatable. There is an animatable variable of IK chain that decides whether the goal defined in the IK chain actually affects the joints it covers at a specific time, this can be queried using <code>IKBound()</code>.</p>\n\
<h2 id=\"joint-limits\">Joint Limits</h2><p>A degree of freedom can be bound to a limited range. The lower bound and upper bound can be set separately. At each end, there are a boolean parameter, Limited, and a float number that sets the limit. The limit is effective only if the boolean parameter, Limited, is true. Following queries concern the joint limits. When a <code>Point2</code> is returned, it is ordered as the lower limit followed by the upper limit. When a <code>Point3</code> is returned, the order is X, Y, and Z. For example, <code>TransLowerLimits()</code> returns lower limits of <code>TransX</code>, <code>TransY</code>, and <code>TransZ</code>, respectively.</p>\n\
<h2 id=\"preferred-angles\">Preferred Angles</h2><p>Some solver may start off the solution process with joint angles being set to special values, these are called preferred angles. <code>PrefPosition()</code> and <code>PrefRotation()</code> return preferred angles of the translation and rotation joints, respectively. The APIs allows them to be animatable. However, in 3ds Max 4, they are constant with regard to animation time. The methods <code>SetPrefTrans()</code>, <code>SetPrefRot()</code>, and <code>SetPrefTR()</code> set their values at a specific time.</p>\n\
<h2 id=\"joint-angles\">Joint Angles</h2><p><code>TransValue()</code> and <code>RotValue()</code> will query joint angles of sliding and rotational joints, respectively, at a specific time. If the user supplies non-null pointer to a validity interval, the validity interval will be updated.</p>\n\
<p>Their values can be assigned using <code>AssignTrans()</code> and <code>AssignRot()</code>. The methods <code>AssignActiveTrans()</code> and <code>AssignActiveRot()</code> will skip those degrees of freedom that are not active. These same two methods allow active DOFs to be given as the first argument of type <code>DofSet</code>, while having the new values supplied as float array whose size should be the same as the DofSet [<code>DofSet::Count()</code>]. The order in the array is: <code>DofZ</code> if the DofSet includes <code>DofZ</code>, <code>DofY</code> if the DofSet includes it, and followed by <code>DofX</code> if it is included in the <code>DofSet</code>. IK controllers are designed to be controlled by the IK goals (through the internal IK system). Therefore, their values are not to be set as independent variables are. These assignment methods just assign new values to the respective variables and that&#39;s all. They do not adjust the validity interval, this is done through the methods <code>SetTransValid()</code>, <code>SetRotValid()</code> and <code>SetTRValid()</code>.The last method sets the validity intervals for translation and rotation values to the same interval.</p>\n\
<h2 id=\"fk-sub-controller\">FK Sub-controller</h2><p>The Forward Kinematics sub-controller and the pointer to the node that holds this controller as its TM controller can be obtained using <code>Control</code>* FKSubController() and <code>INode</code>* GetNode(). Note that the IK controller is not designed to be instanced. It is expected to have a unique node.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../inverse_kinematics.html\">Inverse Kinematics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";