var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"ik_solvers\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>IK Solvers</h1></div><p>Being a plugin itself, the IK system allows the solver to be a separate plugin. The <code>IKSolver</code> class defines the base class that plugin solver should derive from. The IK solver is a pure mathematical function: it does not hold state and just solves a given, self-contained, mathematical problem. In other words, the plugin solver does not have influence on when IK is invoked and what an IK problem is (what is the goal and what are the joints, etc.), but contributes to IK by answering how to solve. Structurally, it is independent of the SDK and, hence, can be built independently, except for some theoretically independent math library.</p>\n\
<h2 id=\"class-identity\">Class Identity</h2><p>Plugin solvers are recognized by the IK system by the Super Class ID. A unique enum, <code>IK_SOLVER_CLASS_ID</code>, defined in <code>animtbl</code>.h, is given to it. The <code>SClass_ID</code>SuperClassID() method should not to be overridden. Since it is a pure mathematical function that does not hold state, an individual plugin solver is identified by its class name which can be obtained using the <code>GetClassName()</code> method. When class name clashes, a suffix may be appended. The class name will appear in the solver list from that users can pick for or assign to IK chains.</p>\n\
<h2 id=\"solver-traits\">Solver Traits</h2><p>Following methods are meant to be overridden by the plugin solver queried by the IK system.</p>\n\
<p><code>IsInteractive()</code>. IK can be used as a controller or as an interactive manipulation tool. In the former, the relationship between the goal and the joints are permanent: joints are completely controlled by the goal. In the latter, the relationship is transient, existing only during interactive manipulation. In the end, IK solutions are registered at each joint, mostly likely as key-frames, and it no longer matters how joints got their joint angles. Only non-interactive, or controller, IK solvers ares used.</p>\n\
<p><code>IsHistoryDependent()</code>. At a specific animation time, the history dependent solver will reach solutions not only based the state of goal at the time, but also its previous states (hence they are history dependent). On the contrary, the history independent solver does its job based on the state of the goal just at the time. The procedural implication is that, when the goal is changed at time t, the IK system would have to invalidate joints, at time t for the history independent solver, and at all times that are greater or equal to t for the history dependent solver. Only history dependent solvers are used by the IK system.</p>\n\
<p>The methods <code>UseSlidingJoint()</code> and <code>UseSwivelAngle()</code> are used to tell whether the plugin solver intends to use the sliding joint (translational degrees of freedom) or the swivel angle parameter of the IK chain.</p>\n\
<p>When two IK chains overlap, i.e., if there&apos;s a joint belonging to both IK chains, some solvers are able to negotiate between the possibly contending goals and some are not. The method <code>DoesOneChainOnly()</code> is used to determine this. For those that can only solve one chain at a time, the IK system will pass to the solvers one chain at a time in a definitive order. Only solvers that &quot;do one chain only&quot; are used.</p>\n\
<p>The method <code>DoesRootJointLimits()</code> and <code>DoesJointLimitsButRoot()</code> deal with the concern of joint limits. If the solver supports joint limits, the IK system will trust it. Otherwise, the IK system will, after calling the solver, clamp the results according the joint limit constraints. The root joint in <code>DoesRootJointLimits()</code> refers to the Start Joint. It is treated differently from the rest of joints.</p>\n\
<p>The method <code>SolveEERotation()</code> tells whether the rotation part of the goal node will be used. If it returns false, only the position of the goal node is taken as the IK goal and rotation threshold will be irrelevant.</p>\n\
<p>Solvers can reach solutions with closed formula, analytically, or going through iterations. For an analytic solver, thresholds and maximum iteration numbers are not relevant. Checking whether or not a solver is analytical can be done using the <code>IsAnalytic()</code> method.</p>\n\
<h2 id=\"solution-parameters\">Solution Parameters</h2><p>The methods <code>GetPosThreshold()</code>, GetRotThreshold(), GetMaxIteration(), SetPosThreshold(), SetRotThreshold(), SetMaxIteration() are used to get and set the Position Threshold, Rotation Threshold, and the Maximum number allowed for iterations and are not relevant for all solvers. For an analytic solver, for example, these are not used at all. The IK system, however, may set values to them.</p>\n\
<p>As mentioned earlier, plugin solvers may have their own Zero Plane Map. If so, they must override the <code>IKSys::ZeroPlaneMap</code>* GetZeroPlaneMap() method. The IK system will need it to perform IK snapping: setting the swivel angle based on the current pose so that the pose is consistent with the swivel angle.</p>\n\
<h2 id=\"solve\">Solve</h2><p><code>Solve()</code> is the method which the IK system will call when its time to update the joints according to the IK goal and other parameters. The IK system will compile an IK problem represented in the 3ds Max scene data structure, nodes, controllers, etc., into a pure mathematical representation, <code>IKSys::LinkChain</code>. This data structure only represents one IK chain. Overlapping IK chains are not dealt with. The <code>ReturnCondition</code>, returned by the <code>Solve()</code> method is a bit-set. Its definition is copied from <code>IKHierarchy</code>.h:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">typedefunsignedReturnCondition;\n\
enum ConditionBit\n\
{\n\
   bLimitReached = 0x00000001,\n\
   bLimitClamped = 0x00000002,\n\
   bMaxIterationReached = 0x00000004,\n\
\n\
   // The first eight bits are reserved for mild condition.\n\
   // They are still considered successful.\n\
   bGoalTooCloseToEE = 0x00000100,\n\
   bInvalidArgument = 0x00000200,\n\
   bInvalidInitialValue = 0x00000400\n\
};</code></pre><p>The data structure passed to the solver is transient, meaning that it will be discarded once the solution is copied back to the joints. If the return condition indicates failure, (<code>return_condition</code> &gt; 0xff) then the result will not be copied back to the joint nodes in the 3ds Max scene database.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../inverse_kinematics.html\">Inverse Kinematics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";