var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"backward_compatibility\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Backward Compatibility</h1></div><p>As development evolves on a plug-in, the use of its parameter blocks may change. For example, more parameters may be added or some may be deleted. 3ds Max files that were saved using the old version of the plug-in would not normally load properly under the new format. In order to provide backward compatibility with the older format a mechanism exists to allow the old format to be converted to the new format.</p>\n\
<p>To accomplish the conversion you must define both the old and the new format of the parameter block.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static  ParamBlockDescID descVer0[] = {\n\
 { TYPE_FLOAT, NULL, TRUE, 0 },\n\
 { TYPE_INT, NULL, TRUE, 1 },\n\
 { TYPE_INT, NULL, TRUE, 2 } };\n\
\n\
static  ParamBlockDescID descVer1[] = {\n\
 { TYPE_FLOAT, NULL, TRUE, 0 },\n\
 { TYPE_INT, NULL, TRUE, 1 },\n\
 { TYPE_INT, NULL, TRUE, 2 },\n\
 { TYPE_FLOAT, NULL, TRUE, 3 },\n\
 { TYPE_INT, NULL, FALSE, 4 },\n\
 { TYPE_INT, NULL, FALSE, 5 } };\n\
\n\
#define PBLOCK_LENGTH 6</code></pre><p>The fourth argument of the <code>ParamBlockDescID</code> is the ID of the parameter. These are used to match the old version of the parameter to the new version. The code which does the conversion matches the IDs between the two versions, for example, it will match the ID=1 of the old version to the ID=1 of the new version.</p>\n\
<p>You create an array of the class <code>ParamVersionDesc</code>, one element for each old version of the parameter block. This structure describes a version of the parameter block. You also create an instance of <code>ParamVersionDesc</code> for the new version.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Array of old versions\n\
static  ParamVersionDesc versions[] = {\n\
 ParamVersionDesc(descVer0,3,0)\n\
};\n\
#define NUM_OLDVERSIONS 1\n\
\n\
// Current version\n\
static  ParamVersionDesc curVersion(descVer1,PBLOCK_LENGTH, 1);\n\
#define CURRENT_VERSION 1</code></pre><p>When the plug-in is loaded, its <code>Load()</code> method is called. From within this method you call a method of the <code>ILoad</code> class which registers the parameter block post load callback object. The callback object is an instance of the class <code>ParamBlockPLCB</code>. This callback creates a new parameter block. The new parameter block inherits any parameters from the old parameter block whose parameter IDs match. In this way the older format is automatically converted to the new format by matching the corresponding IDs.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">IOResult SphereObject::Load(ILoad *iload)\n\
{\n\
 iload-&gt;RegisterPostLoadCallback(\n\
  new ParamBlockPLCB(versions,NUM_OLDVERSIONS,&amp;curVersion,this,0));\n\
 return IO_OK;\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../parameter_maps.html\">Parameter Maps</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";