var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"snapping\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Snapping</h1></div><p>Objects snaps are a plug-in type that allow objects to provide the 3ds Max snapping system with additional points that may be snapped to. For example, the built in snap system provides the ability to snap to points such as vertices, endpoints, midpoints, etc. A developer of a procedural sphere object might want to provide additional snaps for center point and quadrant points. By deriving a class from <code>Osnap</code> and implement its methods a developer can seamlessly provide this functionality in 3ds Max.</p>\n\
<p>The 3ds Max scene has a single object called the <em>Osnap Manager</em>. At 3ds Max startup it will load any object snap plugins (those with a <code>DLS</code> extension). Object snap plugins are derived from class <code>Osnap</code>. This class has various methods that allow the system to query it about potential objects to be snapped to. That is, when 3ds Max is traversing the scene doing a hit test the Osnap Manger will call each of the plugins to snap against a particular node. The <code>Osnap</code> class has a method that allows it to tell the Osnap Manager if it wants to snap to the specified object type based on its Super Class ID and Class ID. For example, NURBS snaps only respond to NURBS objects. Tangent and Perpendicular snaps only respond to Spline objects. If the object snap plugins do want to snap to the object type, they can register a set of points to snap to. For instance, the sphere above would register the center and quadrant points.</p>\n\
<p>This system allows developers to implement very specific snapping behaviors. For example, a door object might have a special snap mode that always snaps to the hinge point.</p>\n\
<h2 id=\"principal-classes\">Principal Classes</h2><p>The following are the main classes associated with creating object snap plugins.</p>\n\
<ul>\n\
<li><code>Osnap</code> - This is the base class for creating osnap plugins. Conceptually, the osnap class represents a &quot;rule&quot; for locating points in an object&apos;s local space. Typically, an instance of this class will only make sense for certain object types. It&apos;s the job of the <code>ValidInput()</code> method to filter out uninteresting nodes in the scene. When the scene is traversed, each object which passes the input test will be passed into the <code>Snap()</code> method. This method is the workhorse of object snap plugins and is responsible for computing, allocating and recording its hits.</li>\n\
<li><code>IOsnapManager</code> - This class provides an interface to the Osnap Manager. Developers who implement osnaps need to record hits with the osnap manager.</li>\n\
<li><code>OsnapHit</code> - This class encapsulates the data required to record a snapped point. Typically a plugin creates instances of this class and records them with the Osnap Manager. If a snap plugin needs to record additional data for its hits, it should derive from this class and provide a clone method which copies this additional data and calls the base classes clone method.</li>\n\
<li><code>OsnapMarker</code> - This class is used for drawing osnap markers in the viewports. The marker is drawn as a polyline. The Osnap class must implement the <code>GetMarkers()</code> method which typically returns pointers to these static instances.</li>\n\
<li><code>HitMesh</code> - This is a class to hold a list of object space points for highlighting the geometry associated with a hit. In practice, developers need only implement two methods of this class.</li>\n\
<li><code>Interface</code> - There are several methods in this class related to object snap. The most important one returns a pointer to the Osnap Manager. See Interface Osnap Methods.</li>\n\
</ul>\n\
<h2 id=\"handling-snap-preview-and-point-snap-in-creation-procedures\">Handling Snap Preview and Point Snap in Creation Procedures</h2><p>The <code>ViewExp::SnapPoint()</code> method has been enhanced in 3ds Max 2.0 to work with the new object snap system. A change that developers will need to make is that any mouse procs which will be calling <code>ViewExp::SnapPoint()</code> will need to add a call to <code>ViewExp::SnapPreview()</code> as in the following code fragment from  <em>\\MAXSDK\\SAMPLES\\OBJECTS\\CYL.CPP</em> .</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">intCylinderObjCreateCallBack::proc(\n\
          ViewExp *vpt,\n\
          int msg,\n\
          int point,\n\
          int flags,\n\
          IPoint2m,\n\
          Matrix3&amp; mat )\n\
{\n\
   floatr;\n\
   if (msg == MOUSE_FREEMOVE)\n\
   {\n\
     vpt-&gt;SnapPreview(m,m,NULL, SNAP_IN_3D);\n\
   }\n\
   if (msg==MOUSE_POINT||msg==MOUSE_MOVE)\n\
   {\n\
     switch(point)\n\
     {\n\
      case 0:\n\
      ob-&gt;suspendSnap = TRUE;\n\
      sp0 = m;\n\
      p[0 = vpt-&gt;SnapPoint(m,m,NULL,SNAP_IN_3D);\n\
      //...\n\
     }\n\
     //...\n\
   }\n\
   //...\n\
}\n\
</code></pre><h3 id=\"sample-code\">Sample Code</h3><p>The main code where snapping is handled for Geometric Objects, Shape Objects, Cameras, Lights and Helpers is available in  <em>\\MAXSDK\\SAMLES\\SNAPS\\XMESH\\XMESH.CPP</em> .</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../miscellaneous_topics.html\">Miscellaneous Topics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";