var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"writing_pipeline_compatible_obje\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Writing Pipeline Compatible Objects</h1></div><p>The simplest way to design a new plug-in object so that it can flow through the pipeline is to allow it to be converted to one of the commonly supported pipeline objects such as <code>TriObject</code>. For more information see <a href=\"./../../../../writing_plug-ins/writing_object_plug-ins/converting_objects_to_compatible.html\">Converting Objects to Compatible Types</a>.</p>\n\
<p>Most of the methods an object must implement for pipeline compatibility relate to minimizing processing overhead when the object is flowing through the pipeline. To be as efficient as possible, the system avoids unnecessary copying of objects in the pipeline and keeps track of which channels modifiers request to deep copy only those channels and perform a shallow copy of the others. For more information see <a href=\"./../channels/channel_copying.html\">Channel Copying</a>.</p>\n\
<p>Typically all objects with geometry that flow through the pipeline should be able to convert themselves to a triangular mesh (<code>TriObject</code>), but this is not a hard and fast rule. It is conceivable that an object can flow through the pipeline and only support conversion to a NURBS object for example. However, it makes sense to allow your objects to convert to a mesh so that they can work with a wide number of modifiers.</p>\n\
<h2 id=\"channel-copying-and-allocation-methods\">Channel Copying and Allocation Methods</h2><p>The following methods must be implemented by an object that is intended to flow natively (without conversion) through the pipeline:</p>\n\
<ul>\n\
<li><code>Object::MakeShallowCopy</code>() - This method creates a new shell and then creates shallow copies in the channels that are specified. It must also copy the validity intervals of the copied channels, and invalidate the other intervals.</li>\n\
<li><code>Object::ShallowCopy</code>() - This method is passed the shell, and it copies the specified channels into it along with the validity interval. The shallow copy just copies the pointers (for example, the vertices pointer or the faces pointer). No new memory is typically allocated.</li>\n\
<li><code>Object::NewAndCopyChannels</code>() - This method takes the channels specified and clones them, and makes them read only (by locking them).</li>\n\
<li><code>Object::FreeChannels</code>() - This method deletes the memory associated with the specified channels and set the intervals associated with the channels to invalid (empty).</li>\n\
<li><code>Object::CanCacheObject</code>() - This method is only implemented by particle systems. Particle systems bypass a lot of how the pipeline works and so they implement this method to return FALSE to ensure that they are never cached. Particle systems handle their own caching mechanism. All objects other than particle system can use the default implementation which returns TRUE.</li>\n\
</ul>\n\
<h2 id=\"interval-validity-methods\">Interval Validity Methods</h2><p>The following methods must be implemented by an object that is intended to flow natively (without conversion) through the pipeline:</p>\n\
<ul>\n\
<li><code>Object::ChannelValidity()</code> - Retrieve the current validity interval for the <code>nchan</code> channel of the object.</li>\n\
<li><code>Object::SetChannelValidity()</code> - Sets the validity interval of the specified channel.</li>\n\
<li><code>Object::UpdateValidity()</code> - This is implemented by the system, but can be overridden if desired. This method is called to combine in interval <code>with the</code> specified channel validity.</li>\n\
<li><code>Object::InvalidateChannels()</code> - This method invalidates the intervals for the given channel mask. This just sets the validity intervals to empty (calling <code>SetEmpty()</code> on the interval) .</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../objects_flowing_through_the_pipe.html\">Objects Flowing through the Pipeline</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";