var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"pipeline_overview\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Pipeline Overview</h1></div><p>This topic describes the 3ds Max pipeline architecture in high-level terms.</p>\n\
<p>The diagrams in this topic show the relationship between scene nodes and scene objects in the pipeline. Each node (<code>INode</code>) in the scene has a reference to a Max object (<code>Object</code>). The arrows show the references between nodes, objects, modifiers, and world space modifiers.</p>\n\
<p>Objects flow through the pipeline from the bottom up. The first object to enter into the pipeline is called the &apos;Base Object&apos;. The result of applying a modifier to an object is represented by a &apos;Derived Object&apos; (<code>IDerivedObject</code>). The end result of the pipeline is the Node (<code>INode</code>). A derived object has a reference to a base object or another derived object, and has a reference to the modifier.</p>\n\
<p>The following diagram shows the pipeline of an unmodified Cylinder object in the scene. In this case the node&apos;s object reference points directly to the base object, the cylinder. The pipeline flow is against the order of the arrows.</p>\n\
<p><img src=\"./images/GUID-B58329F3-C01C-4E64-BA01-E7152F7DE740-low.gif\" alt></p>\n\
<h2 id=\"shared-objects-in-different-pipelines\">Shared Objects in Different Pipelines</h2><p>The following figure shows two pipelines. Note how the object reference of the instanced copy points to the same base object as the original. If the creation parameters of the Cylinder are changed, both nodes will change in the scene since they both point at the same Base Object.</p>\n\
<p><img src=\"./images/GUID-B3427C28-8CC8-43F9-A345-0DD246F495FA-low.gif\" alt></p>\n\
<h2 id=\"modifiers-in-a-pipeline\">Modifiers in a Pipeline</h2><p>You can apply one or more modifiers to the objects in a pipeline. The following figure shows the pipeline resulting from applying a Bend object space modifier to a Cylinder. When a modifier is first applied, a new derived object (<code>IDerivedObject</code>) is inserted into the node&apos;s pipeline.</p>\n\
<p><img src=\"./images/GUID-09D1A49C-6EEC-4A34-BAC7-EA9CB79EAC0E-low.gif\" alt></p>\n\
<p>This arrangement appears in the modifier stack as shown in the following figure:</p>\n\
<p><img src=\"./images/cylinder_bend_mod.png\" alt></p>\n\
<p>This indicates that the base object of the pipeline is the Cylinder. The horizontal line above the word &quot;Cylinder&quot; in the figure indicates the start of a derived object. The Bend is the modifier referenced by this derived object.</p>\n\
<h2 id=\"modifier-application-modapp-\">Modifier Application (ModApp)</h2><p>A derived object consists of one or more applications of modifiers followed by a reference to another object. This other object may be a base object or another derived object. In the figure preceding the above screen capture, the derived object has a single application of a modifier, the Bend. This application of a modifier is referred to as a <code>ModApp</code>. The ModApp consists of a modification context (MC) and a reference to a modifier -- in this case the Bend. Note that the modifier is not in the pipeline, but is referenced by the ModApp that is in the pipeline.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>   <strong></strong> A ModApp is not directly exposed in the 3ds Max SDK.</p>\n\
\n\
</div></div>\n\
\n\
<h2 id=\"modifier-context-modcontext-\">Modifier Context (ModContext)</h2><p>The modification context (represented by the <code>ModContext</code> class) holds the data needed by each individual instance of the modifier. The MC stores information about the space the modifier was applied in, and allows a modifier instance to store the unique data that it needs to operate.</p>\n\
<p>The MC stores three items, as shown in the diagram below, and then described following the diagram.</p>\n\
<p><img src=\"./images/GUID-ECA64FF2-28A3-46D2-8DAD-227F8EC7F842-low.gif\" alt></p>\n\
<ul>\n\
<li><em>Transformation Matrix</em> - This matrix represents the space the modifier was applied in, used by the modifier plug-in when it deforms an object. The plug-in modifier first transforms the points with this matrix. Next it applies its own deformation. Then it transforms the points back through the inverse of this transformation matrix.</li>\n\
<li><em>Bounding Box</em> - This is the bounding box of the modifiers and represents its scale. For a single object it is the bounding box of the object. If the modifier is being applied to a sub-object selection it represents the bounding box of the sub-object selection. If the modifier is being applied to a selection set of objects (and 3ds Max is not in the &apos;Use Pivot Points&apos; mode), then this is the bounding box of the entire selection set. For a selection set of objects, the bounding box is constant. In the case of a single object, the bounding box is not constant. For example, if the user applies a 90 degree bend to a cylinder, then changes the height of the cylinder, the user would want the cylinder to still be bent 90 degrees. If the bounding box did not adapt, the cylinder would appear to move through the bend causing the bend angle to be incorrect.</li>\n\
<li><em>LocalModData Pointer</em> - This is a pointer to an instance of a class derived from <code>LocalModData</code>, which is the part of the MC that a plug-in developer controls. It is the place where a modifier can store application-specific data. The <code>LocalModData</code> class has two methods the derived class must implement. One is <code>LocalModData::Clone()</code> method, so the system can copy a <code>ModContext</code>. The second is the virtual destructor.</li>\n\
</ul>\n\
<h2 id=\"multiple-modifiers\">Multiple Modifiers</h2><p>More than one modifier may be applied to an object. The following figure shows the previous bent cylinder with an additional Taper modifier applied.</p>\n\
<p><img src=\"./images/GUID-9B4270EF-8575-4372-AEC0-36D740B13F57-low.gif\" alt></p>\n\
<p>The modifier stack for Cylinder01 for the above figure would appear as:</p>\n\
<p><img src=\"./images/cylinder_bend_taper_mod.png\" alt></p>\n\
<p>In this case a new ModApp is inserted into the existing Derived Object. The modifier reference of the new ModApp points to the Taper modifier.</p>\n\
<h2 id=\"reference-copies\">Reference Copies</h2><p>3ds Max also allows Reference Copies of items in the scene. When a Reference Copy is made, a new Derived Object is inserted into the item&apos;s pipeline. Modifiers applied to the Reference Copy will have their ModApps inserted into the new Derived Object. The following figure shows the result of a bent cylinder being Reference-Copied and having a Taper modifier applied. Note that the object reference of the new Derived Object points to the original Derived Object.</p>\n\
<p><img src=\"./images/GUID-9ABC1C7B-6D3E-449F-A202-5C7CF046B2FC-low.gif\" alt></p>\n\
<p>Note the two occurrences of the line (above Cylinder) indicating that two Derived Objects are in use. The original one points to the Bend while the new one points to the Taper.</p>\n\
<h2 id=\"sharing-modifiers\">Sharing Modifiers</h2><p>A modifier object may be used by several objects. When the 3ds Max user applies a modifier to more than one object simultaneously, one instance of the modifier&apos;s class is created and shared among the objects. Each object that the modifier is applied to automatically acquires a new ModApp in its pipeline. These ModApps then reference the same modifier.</p>\n\
<p>In the screen capture and diagram below, the user has selected two independent Cylinders in the scene, checked the &apos;Use Pivot Points&apos; box, and applied a Bend modifier.</p>\n\
<p><img src=\"./images/cylinders_pivot_center_bend.png\" alt></p>\n\
<p><img src=\"./images/GUID-1F7A6243-3D0A-45DA-8E69-BD791E44147B-low.gif\" alt></p>\n\
<p>Note that each cylinder has its own Derived Object. The modifier reference of each ModApp points to the same instanced modifier however. The ModApp also stores the ModContext (labeled &apos;MC&apos; in the diagrams). One data member of this ModContext is the bounding box of the deformation. Because the &apos;Use Pivot Points&apos; button was checked at the time the bend was applied, each bounding box stored in the ModContext is the size of each cylinder alone. Additionally, the transformation matrix of each ModContext reflects that the bend is to be applied to each cylinder in its own space. The result is the bend is applied locally and independently to each cylinder.</p>\n\
<p>Contrast this with the following case. In the screen capture, below, the user applied the Bend modifier to the cylinders as follows: First the independent cylinders were selected. Then the &apos;Use Pivot Points&apos; button was unchecked. Then the Bend modifier was applied. This means the bend will be applied to the entire selection set as a whole. Note that the cylinders are bent about a common center.</p>\n\
<p><img src=\"./images/cylinders_no_pvt_bend.png\" alt></p>\n\
<p>In this case the bend is applied relative to the bounding box of both cylinders. Thus the bounding box of the ModContexts are the same for each cylinder. Note what happens if one cylinder is moved away from the other in the scene. The Modifer is selected so the gizmo shows the bounding box graphically. The bounding box remains the size of both even when the objects no longer share the same world space relationship.</p>\n\
<p><img src=\"./images/cylinders_no_pvt_bend_moved.png\" alt></p>\n\
<p>The separation of the ModApps from the Modifier allows this flexibility. The bounding box stored with the ModContext of the ModApp represents the scale of the application of the modifier. The transformation matrix of the ModContext represents the space the modifier was applied in. The instanced modifier just uses this information to properly modify each input object.</p>\n\
<h2 id=\"world-space-modifiers\">World Space Modifiers</h2><p>Below is a diagram of a Cylinder and a Ripple Space Warp before the Cylinder has been bound to the space warp. Note that world space modifiers have their own nodes in the scene graph and have a gizmo representation in the viewport but are not part of the rendered scene.</p>\n\
<p><img src=\"./images/GUID-5386F67E-20FF-47A4-A272-B6465C5F85A9-low.gif\" alt></p>\n\
<p>The following diagram shows the pipeline of the Cylinder after it has been bound to the Ripple Space Warp.</p>\n\
<p><img src=\"./images/GUID-4CC8A18F-709E-4274-A1EC-49BAED3DECE0-low.gif\" alt></p>\n\
<p>The modifier stack for this condition would appear as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Ripple Binding\n\
----------\n\
Cylinder</code></pre><p>The Cylinder is the Base Object. The line (----------) represents the beginning of a world space modifier derived object. The Ripple Binding is the actual application of the world space modifier.</p>\n\
<p>When the Cylinder is bound to the Ripple a new world space modifier derived object is inserted into the Cylinder&apos;s pipeline. The world space modifier derived object is similar to the derived objects that hold object space modifier ModApps except that these are contained in a specific node (world space modifier object is in fact the exact same class as derived object except for the <code>ClassID()</code>). Since they are associated with a specific node, they cannot be instanced.</p>\n\
<p>The modifier reference of the ModApp points to a newly-created world space modifier. This world space modifier usually has a reference to the node in the scene. In this case it is to the Ripple01 node. This reference is used to retrieve the position of the node (from the node&apos;s world space transformation matrix). It uses this matrix to transform the points of the object it is deforming into the space of the WSM object where it actually performs the deformation.</p>\n\
<h2 id=\"example-of-pipeline-data-flow\">Example of Pipeline Data Flow</h2><p>This following presents a detailed look at the flow of the pipeline from the Base Object to the resulting World Space Cache. This includes the derived objects of the object space portion of the pipeline, the application of the node&apos;s transform controller, and through the world space portion of the pipeline.</p>\n\
<p>The diagram below shows a procedural Cylinder with a Bend modifier applied, and bound to a Ripple space warp.</p>\n\
<p><img src=\"./images/GUID-7706A1E3-5F5D-4B8C-9ECA-DB5374E09907-low.gif\" alt></p>\n\
<p>The data flow in the pipeline follows the object references. In the example above, the Cylinder01 Node has an object reference pointing to the world space modifier object. Its object reference points to the Derived Object. Its object reference points to the Cylinder Base Object. This is the path the data follows as it moves along the pipeline.</p>\n\
<p>The actual object that flows between these references is an <code>ObjectState</code>. The <code>ObjectState</code> is the result of the <code>Object::Eval()</code> method being called on the object reference.</p>\n\
<p>This pipeline starts at the Base Object -- the Cylinder. The system asks the cylinder to evaluate itself. In the cylinder&apos;s implementation of <code>Object::Eval()</code>, it simply returns itself ( <code>return</code>ObjectState(this); ). It returns this ObjectState to the next object reference in the pipeline. This is the object reference of the derived object.</p>\n\
<p>The Derived Object sends this ObjectState through each of its ModApps. In this example, the only ModApp is for the Bend Modifier. The Bend requires Deformable objects (it indicates this in its implementation of <code>Modifier::InputType()</code>). The ModApp handles converting the object to the appropriate type for the Modifier. The cylinder is asked to convert itself to a Deformable object. It does this in its implementation of <code>Object::ConvertToType()</code> by creating a new TriObject and setting the TriObject&apos;s mesh pointer to point at the cylinder&apos;s triangle mesh.</p>\n\
<p>This Deformable TriObject is then sent through the Bend Modifier. The Bend is passed the ModApp&apos;s ModContext as an argument to its <code>Modifier::ModifyObject()</code> method. The Bend first modifies the points of the TriObject using the transformation matrix of the ModContext. This puts the object into the space the modifier was applied in. Next the Bend applies its deformation to the points (it bends them). Then the Bend modifies the points of the TriObject by the inverse of the ModContext transformation matrix. This restores the object, excepting it now has the bend effect applied.</p>\n\
<p>At this point, the Derived Object returns the result to the next Object Reference in the pipeline. In this case the result is a TriObject that has been bent. It is returned to the Object Reference of a World Space Derived Object.</p>\n\
<p>This juncture between the Derived Object and the World Space Modifier Derived Object is where the pipeline crosses from object space to world space. At this point the result of the node&apos;s transform controller, and the object offset transformation are put into the pipeline. For more details on these various transformation matrices see the Advanced Topics section on <a href=\"./../transformation_and_rotation/node_transformations.html\">Node Transformations</a>.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../geometry_pipeline_system.html\">Geometry Pipeline System</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";