var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"world_space_modifiers_and_object\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>World Space Modifiers and Object Transformations</h1></div><p>When world space modifiers are applied to an object the points of the object may have already been transformed into world space. In this case, the points should not be transformed into world space again when they are drawn.</p>\n\
<p>This is handled in the 3ds Max SDK by storing some state information with the node. Before the system calls <code>BaseObject::Display()</code>, <code>BaseObject::itTest()</code>, etc. it sets a flag on the node. The flag indicates if the object has already been transformed into world space. The <code>INode::GetObjectTM()</code> method looks at this flag to determine the proper matrix to return. In this way, when <code>INode::GetObjectTM()</code> is called, it returns the matrix the object needs to be multiplied by in order to get into world space. If the object is already in world space it will return the identity matrix. If it&apos;s not in world space, it will return the matrix to get it there. So all any objects need to do in their <code>BaseObject::Display()</code> methods is call <code>INode::GetObjectTM()</code> and use whatever matrix is returned.</p>\n\
<p>There may be times when a developer needs to access the full object transformation matrix regardless of whether the points of the object have been transformed into world space already. For example, if a developer was creating a utility plug-in to align two objects. In this case, the developer would need to get the node transformation matrix and the object offset transformation matrix. It would not matter if the points of the object had already been transformed into world space. In this case <code>INode::GetObjectTM()</code> would not work. This is because it returns the identity matrix if the object is already in world space.</p>\n\
<p>To solve this problem 3ds Max provides two other methods:</p>\n\
<ul>\n\
<li><code>INode::GetObjTMBeforeWSM()</code> - This method explicitly gets the full node transformation matrix and object-offset transformation effect before the application of any world space modifiers.</li>\n\
<li><code>INode::GetObjTMAfterWSM()</code> - This method explicitly gets the full node transformation matrix and object-offset transformation and world space modifier effect unless the points of the object have already been transformed into world space in which case it will return the identity matrix.</li>\n\
</ul>\n\
<h2 id=\"example\">Example</h2><p>Using the methods <code>INode::GetObjTMBeforeWSM()</code> and <code>INode::GetObjTMAfterWSM()</code> a developer has complete access to any transformation matrix they require. Below is a code example that uses all these methods. This function computes the bounding box of the first object in the current selection set at the current time. It removes anything but scaling from the object transformation matrix. In this way rotation of the node will not affect the bounding box.</p>\n\
<p>To do this we first need to determine if the object is in world space or in object space. Since we are after the object space bounding box (and will later apply scaling) we need to convert the transformation matrix back into object space if it is in world space. To check if the object is in world space we call <code>INode::GetObjTMAfterWSM()</code>. If this matrix is the identity we know we are in world space. This is because when the points of the object get transformed by the <code>ObjectState</code> transformation matrix to put them into world space the ObjectState TM is set to the identity.</p>\n\
<p>If the object is in world space we need to compute the object space transformation matrix. We can do this by taking the inverse of the world space transformation matrix. If the object is not in world space we just need to get its object TM by calling <code>INode::GetObjectTM()</code>.</p>\n\
<p>Once we have the object space transformation matrix we want to extract just the scaling portion of the matrix. 3ds Max provides a set of APIs that make this easy. This is done by calling <code>decomp_affine()</code>. This function decomposes a matrix into its translation, rotation and scaling components. See <code>Structure AffineParts</code> for more details.</p>\n\
<p>Once we have the scaling portion of the matrix we can get the bounding box and apply the scaling by calling <code>GetDeformBBox()</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void Utility::ComputeBBox(Interface *ip)\n\
{\n\
   if (ip-&gt;GetSelNodeCount())\n\
   {\n\
     INode *node = ip-&gt;GetSelNode(0);\n\
     Box3 box; // The computed box\n\
     Matrix3 mat; // The Object TM\n\
     Matrix3 sclMat(1); // This will be used to apply the scaling\n\
\n\
     // Get the result of the pipeline at the current time\n\
     TimeValue t = ip-&gt;GetTime();\n\
     Object *obj = node-&gt;EvalWorldState(t).obj;\n\
\n\
     // Determine if the object is in world space or object space\n\
     // so we can get the correct TM. We can check this by getting\n\
     // the Object TM after the world space modifiers have been\n\
     // applied. It the matrix returned is the identity matrix the\n\
     // points of the object have been transformed into world space.\n\
     if (node-&gt;GetObjTMAfterWSM(t).IsIdentity())\n\
     {\n\
      // It&apos;s in world space, so put it back into object\n\
      // space. We can do this by computing the inverse\n\
      // of the matrix returned before any world space\n\
      // modifiers were applied.\n\
      mat = Inverse(node-&gt;GetObjTMBeforeWSM(t));\n\
     }\n\
     else\n\
     {\n\
      // It&apos;s in object space, get the Object TM.\n\
      mat = node-&gt;GetObjectTM(t);\n\
     }\n\
\n\
     // Extract just the scaling part from the TM\n\
     AffineParts parts;\n\
     decomp_affine(mat, &amp;parts);\n\
     ApplyScaling(sclMat, ScaleValue(parts.k*parts.f, parts.u));\n\
\n\
     // Get the bound box, and affect it by just\n\
     // the scaling portion\n\
     obj-&gt;GetDeformBBox(t, box, &amp;sclMat);\n\
\n\
     // Show the size and frame number\n\
     float sx = box.pmax.x-box.pmin.x;\n\
     float sy = box.pmax.y-box.pmin.y;\n\
     float sz = box.pmax.z-box.pmin.z;\n\
\n\
     TSTR title;\n\
     title.printf(_T(&quot;Result at frame %d&quot;), t/GetTicksPerFrame());\n\
     TSTR buf;\n\
     buf.printf(_T(&quot;The size is: (%.1f, %.1f, %.1f)&quot;), sx, sy, sz);\n\
     MessageBox(NULL, buf, title, MB_ICONINFORMATION|MB_OK);\n\
   }\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../geometry_pipeline_system.html\">Geometry Pipeline System</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";