var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"computing_face_normals\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Computing Face Normals</h1></div><p>A face normal is a vector that defines which way a face is pointing. The direction that the normal points represents the front, or outer surface of the face. To compute the face normal of a face in a mesh you simply take the cross product of two edge vectors of the face. The following code shows how this is done using the 3ds Max API. It loops through each face in a mesh and stores the computed face normal in a table of <code>Point3</code>s named <code>fnorms</code>. At the end of the code, the <code>DebugPrint()</code> API is used to display each normal in the table to the debug window of the VC++ IDE.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void Utility::ComputeFaceNormals(Mesh *mesh)\n\
{\n\
   Face *face;; \n\
   Point3 *verts;\n\
   Point3 v0, v1, v2;\n\
   Tab&lt;Point3&gt; fnorms;\n\
\n\
   // Compute face (surface) normals for the mesh\n\
   face = mesh-&gt;faces; \n\
   verts = mesh-&gt;verts;\n\
   fnorms.SetCount(mesh-&gt;getNumFaces());\n\
   for (int i = 0; i &lt; mesh-&gt;getNumFaces(); i++, face++)\n\
   {\n\
     v0 = verts[face-&gt;v[0]];\n\
     v1 = verts[face-&gt;v[1]];\n\
     v2 = verts[face-&gt;v[2]];\n\
     fnorms[i] = (v1-v0)^(v2-v1);\n\
     fnorms[i] = Normalize(fnorms[i]);\n\
   }\n\
\n\
   // Display the normals in the debug window of the VC++ IDE\n\
   DebugPrint(&quot;\\n\\nFace Normals ---&quot;;);\n\
\n\
   for (i = 0; i &lt; fnorms.Count(); i++)\n\
   {\n\
     DebugPrint(&quot;\\nFaceNormal[%d]=(%.1f, %.1f, %.1f)&quot;,\n\
     i, fnorms[i].x, fnorms[i].y, fnorms[i].z);\n\
   }\n\
   DebugPrint(&quot;\\n\\n&quot;);\n\
}</code></pre><p>As a side note on face normals -- If you take the length of a face normal vector (for example using the <code>Point3::Length()</code> method), you get a quantity equal to twice the surface area of that face.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../meshes.html\">Meshes</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";