var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"computing_vertex_normals\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Computing Vertex Normals</h1></div><p>This section discusses the way vertex normals may be computed. Two algorithms are reviewed. The first ignores smoothing groups and returns an averaged normal at the vertex. The second one looks at the smoothing information and computes multiple normals when there are faces that have different smoothing groups that share a vertex.</p>\n\
<h2 id=\"ignoring-smoothing-groups\">Ignoring Smoothing Groups</h2><p>In the case where smoothing groups are not checked there will be a single vertex normal for each vertex of the mesh. The vertex normal is the average of the face normals of each of the faces that share that vertex. The algorithm to compute such vertex normals is as follows:</p>\n\
<p>First, allocate an array of normals, one for each vertex in the mesh, and initialize them to zero (<code>Point3</code>( 0,0,0)). Then for each face, compute its face normal, and add it into each of the three vertex normals that the face contributes to. For example, if you have a vertex normal shared by five faces, each face will add in its normal to that vertex, and thus the result will be average normal of those five faces. When all the faces in the mesh have been processed, the average normal vector has been computed. As a last step, all the normals in the array can be normalized.</p>\n\
<h2 id=\"considering-smoothing-groups\">Considering Smoothing Groups</h2><p>The above algorithm does not take smoothing groups into account. When smoothing groups are involved, you may have multiple normals for each vertex. For example, if you had a sphere that had the top and bottom hemispheres smoothed separately (i.e. not smoothed across the equator), then the vertices across the equator would have two normals for each vertex while the other vertices would have one. There may be as many normals as there are smoothing groups colliding at a vertex. However, it is by far the most common case to have one, and anything other than one or two is very rare.</p>\n\
<p>The class used to compute vertex normals considering smoothing is shown below. This class, <code>VNormal</code>, is similar to the <code>RNormal</code> class used by 3ds Max internally. The class contains a <code>Point3</code> which is the normal, a <code>DWORD</code> for the smoothing groups, and a pointer to the <code>next</code> normal -- this class is a linked list. The <code>init</code> variable is used as a flag to indicate if the first normal in the list has been initialized.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Linked list of vertex normals\n\
class VNormal\n\
{\n\
   public:\n\
     Point3 norm;\n\
     DWORD smooth;\n\
     VNormal *next;\n\
     BOOL init;\n\
\n\
     VNormal() {smooth=0;next=NULL;init=FALSE;norm=Point3(0,0,0);}\n\
     VNormal(Point3 &amp;n,DWORD s) {next=NULL;init=TRUE;norm=n;smooth=s;}\n\
     ~VNormal() {delete next;}\n\
     void AddNormal(Point3 &amp;n,DWORD s);\n\
     Point3 &amp;GetNormal(DWORD s);\n\
     void Normalize();\n\
};</code></pre><p>A key method to this class is <code>AddNormal()</code>. It is used when a face is going to add its normal to a vertex. This method is passed the normal and the smoothing information for that face. It checks if the normal passed shares smoothing information with the existing normal. If it does, the normal is added in, and the smoothing bits are bitwise OR-ed in. If it does not, a new vertex normal is created. In this way, as normals that share smoothing information are added, they contribute to the overall normal for that smoothing condition at the vertex. If it is a normal whose face does not share smoothing information, a new vertex normal is allocated.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Add a normal to the list if the smoothing group bits overlap,\n\
// otherwise create a new vertex normal in the list\n\
voidVNormal::AddNormal(Point3 &amp;n,DWORD s) {\n\
   if (!(s&amp;smooth) &amp;&amp; init) {\n\
     if (next) next-&gt;AddNormal(n,s);\n\
     else {\n\
      next = new VNormal(n,s);\n\
     }\n\
   }\n\
   else {\n\
     norm += n;\n\
     smooth |= s;\n\
     init = TRUE;\n\
   }\n\
}\n\
\n\
// Retrieves a normal if the smoothing groups overlap or there is\n\
// only one in the list\n\
Point3 &amp;VNormal::GetNormal(DWORD s)\n\
{\n\
   if (smooth&amp;s || !next) return norm;\n\
   else return next-&gt;GetNormal(s); \n\
}\n\
\n\
// Normalize each normal in the list\n\
void VNormal::Normalize() {\n\
   VNormal *ptr = next, *prev = this;\n\
   while (ptr)\n\
   {\n\
     if (ptr-&gt;smooth&amp;smooth) {\n\
      norm += ptr-&gt;norm;\n\
      smooth |= ptr-&gt;smooth;\n\
      prev-&gt;next = ptr-&gt;next;\n\
      delete ptr;\n\
      ptr = prev-&gt;next;\n\
     }\n\
     else {\n\
      prev = ptr;\n\
      ptr = ptr-&gt;next;\n\
     }\n\
   }\n\
   norm = ::Normalize(norm);\n\
   if (next) next-&gt;Normalize();\n\
}</code></pre><p>The method <code>ComputeVertexNormals()</code> shown below is a demonstration method that uses the <code>VNormal</code> class above. The first thing done is to create a table of the vertex normals. Note that since the <code>Tab</code> class does not do any initialization (it only allocates the memory), the code loops through each normal and call the constructor to perform the initialization. Then it goes through each face , calculates the surface normal, and adds it into each of the three vertex normals for the face using <code>AddNormal()</code>. When all the faces have been processed, it goes through each of the vertex normals and normalizes them.</p>\n\
<p>In the code below, the vertex normals are displayed using <code>DebugPrint()</code> to the output window. If there is more than one normal at a vertex, each one is displayed (the <code>DisplayVertexNormal()</code> method recursively calls itself to display each one).</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Compute the face and vertex normals\n\
void Utility::ComputeVertexNormals(Mesh *mesh)\n\
{\n\
   Face *face; \n\
   Point3 *verts;\n\
   Point3 v0, v1, v2;\n\
   Tab&lt;VNormal&gt; vnorms;\n\
   Tab&lt;Point3&gt; fnorms;\n\
   face = mesh-&gt;faces; \n\
   verts = mesh-&gt;verts;\n\
   vnorms.SetCount(mesh-&gt;getNumVerts());\n\
   fnorms.SetCount(mesh-&gt;getNumFaces());\n\
\n\
   // Compute face and vertex surface normals\n\
   for (int i = 0; i &lt; mesh-&gt;getNumVerts(); i++) {\n\
     vnorms[i] = VNormal();\n\
   }\n\
   for (int i = 0; i &lt; mesh-&gt;getNumFaces(); i++, face++) {\n\
     // Calculate the surface normal\n\
     v0 = verts[face-&gt;v[0]];\n\
     v1 = verts[face-&gt;v[1]];\n\
     v2 = verts[face-&gt;v[2]];\n\
     fnorms[i] = (v1-v0)^(v2-v1);\n\
     for (int j=0; j&lt;3; j++) { \n\
      vnorms[face-&gt;v[j]].AddNormal(fnorms[i],face-&gt;smGroup);\n\
     }\n\
     fnorms[i] = Normalize(fnorms[i]);\n\
   }\n\
   for (i=0; i &lt; mesh-&gt;getNumVerts(); i++) {\n\
     vnorms[i].Normalize();\n\
   }\n\
   // Display the normals in the debug window of the VC++ IDE\n\
   DebugPrint(&quot;;\\n\\nVertex Normals ---&quot;;);\n\
   for (i = 0; i &lt; vnorms.Count(); i++) {\n\
     DisplayVertexNormal(vnorms.Addr(i), i, 0);\n\
   }\n\
   DebugPrint(&quot;\\n\\n&quot;);\n\
}\n\
\n\
void Utility::DisplayVertexNormal(VNormal *vn, int i, int n)\n\
{\n\
   DebugPrint(&quot;\\nVertex %d Normal %d=(%.1f, %.1f, %.1f)&amp;&quot;,\n\
     i, n, vn-&gt;norm.x, vn-&gt;norm.y, vn-&gt;norm.z);\n\
   if (vn-&gt;next) DisplayVertexNormal(vn-&gt;next, i, n+1);\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../meshes.html\">Meshes</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";