var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"computing_vertex_normals_by_weig\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Computing Vertex Normals by Weighting</h1></div><p>This next section discusses two additional techniques that may be used in computing vertex normals. The first technique weights the normals by the vertex angle on each face. The second weights the normal using the area of each face.</p>\n\
<h2 id=\"weighting-by-face-angle\">Weighting by Face Angle</h2><p>To understand why using a weighted normal approach might be used consider the following example:</p>\n\
<p>Create a default Box and convert it to an Editable Mesh. Go into SubObject mode, and find vertex 1 (generally found at the front lower left corner). Display the edges, and go into object properties to unclick &quot;edges only&quot; (so you see the hidden edges). You&apos;ll see that vertex 1 is used by bottom faces 1 and 2. (In the SDK, these are vertex 0 and faces 0 and 1.) its also used by front faces 5 and 6, and left face 11. (4,5,10.)</p>\n\
<p>If we simply averaged the normals of all incident faces, we&apos;d get:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Normalize (2*(0,0,-1) + 2*(0,-1,0) + (-1,0,0)), which is (1/3, 2/3, 2/3). </code></pre><p>Because this arrangement of diagonals is haphazard and (inherently) asymmetric, we&apos;d get vertex normals on a box pointing to odd uncoordinated directions.</p>\n\
<p>If we instead weight the normals by the vertex angle on each face, we get:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Normalize (PI/2*(0,0,-1) + PI/2*(0,-1,0) + PI/2*(-1,0,0)) = (1,1,1)/Sqrt(3) </code></pre><p>This is more natural for the user. Each vertex normal points away from all three sides symmetrically. (The individual front and bottom triangles may have varying angles, but the pairs of them always add up to PI/2.)</p>\n\
<p>This seems like the right approach in general -- when you divide a face, for example, you don&apos;t wind up changing the vertex normal when the surface hasn&apos;t essentially changed. If you change vertex angles by dragging neighboring verticies, the normal changes in a natural fashion.</p>\n\
<p>You can compute the vertex angle by using dot products as shown below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Corner is 0, 1, or 2 -- which corner do we want the angle of?\n\
float FindVertexAngle(Mesh *mesh, int face, int corner) {\n\
   int cnext = (corner+1)%3;\n\
   int cprev = (corner+2)%3;\n\
   DWORD *vv = mesh-&gt;faces[face];\n\
\n\
   // Get edge vectors:\n\
   Point3 A = mesh-&gt;verts[vv[cnext]] - mesh-&gt;verts[vv[corner]];\n\
   Point3 B = mesh-&gt;verts[vv[corner]] - mesh-&gt;verts[vv[cprev]];\n\
\n\
   // Normalize the edge-vectors, but return 0 if either has 0 length.\n\
   float len = Length(A);\n\
   if (!len) return len;\n\
   A = A/len;\n\
   len = Length(B);\n\
   if (!len) return len;\n\
   B = B/len;\n\
\n\
   // The dot product gives the cosine of the angle:\n\
   float dp = DotProd (A,B);\n\
   if (dp&gt;1) dp=1.0f; // shouldn&apos;t happen, but might\n\
   if (dp&lt;-1) dp=-1.0f; // shouldn&apos;t happen, but might\n\
   return acos(dp);\n\
}</code></pre><p>To be efficient when computing all normals, you may want to cache the normalized edge directions (A &amp; B). You can index these by an adjacent edge list for the mesh, where edir[i] is the unit vector pointing from vertex ae-&gt;edges[i].v[0] to ae-&gt;edges[i].v[1] (AdjEdgeList *ae).</p>\n\
<h2 id=\"weighting-by-face-area\">Weighting by Face Area</h2><p>Another possibility is to weight the normals by the area of each face. The area of a face is very easy to compute, its just half the length of the normal cross product:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void GetAreaAndNormal (Mesh *mesh, int face, Point3 &amp; N, float area)\n\
{\n\
   DWORD *vv = mesh-&gt;faces[face].v;\n\
   Point3 A = mesh-&gt;verts[vv[1]] - mesh-&gt;verts[vv[0]];\n\
   Point3 B = mesh-&gt;verts[vv[2]] - mesh-&gt;verts[vv[0]];\n\
   N = A^B;\n\
   area = Length (N) / 2.0f;\n\
   Normalize (N);\n\
}</code></pre><p>This works using any two edges for A and B.</p>\n\
<p>To weight face normals by area, you can just use the N=A^B vector directly.</p>\n\
<p>Weighting by area gives an interesting result, but perhaps not as satisfactory as weighting by face angle.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../meshes.html\">Meshes</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";