var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"creating_pipeline-friendly_meshe\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Creating Pipeline-Friendly Meshes</h1></div><p>Developers should follow a few simple but important rules when building meshes to function ideally with modifiers in the Geometry Pipeline. The six basic rules are:</p>\n\
<ol>\n\
<li>Referencing each edge at most once in each direction.</li>\n\
<li>Avoiding self-intersection of faces.</li>\n\
<li>Avoiding creating faces with vertices located at the same place.</li>\n\
<li>Not &#39;bridging&#39; separate mesh components with a single vertex.</li>\n\
<li>Breaking the mesh into sensible elements.</li>\n\
<li>Whenever convenient, close the mesh.</li>\n\
</ol>\n\
<p>3ds Max meshes allow the developer a great deal of flexibility. There are very few rules. Meshes can have holes in them, such as you get when deleting any face from a box or a sphere. They may be composed of separate, disconnected units, which may even self-intersect. A &#39;fan&#39; of 10 faces can use the same edge, simply by using the same two vertices. A grid may be created which is simply a 2-D collection of points with one set of faces pointing up and another pointing down.</p>\n\
<p>However, not all of the possible meshes are appropriate for operations that may be done to them later in the 3ds Max pipeline. For instance, the 3ds Max Boolean, like any other Boolean, regards meshes as the outside boundary of solid objects. A Boolean intersection, for instance, is defined by the intersection of the volumes contained by the two operands. A mesh with a hole in it, such as the teapot, contains no volume, and therefore produces surprising results in Boolean.</p>\n\
<p>Furthermore, Boolean, like several other compound objects and modifiers, converts meshes internally into MNMeshes. The <code>MNMesh</code> winged-edge structure can only handle 2 faces on a single edge, so the &#39;fan&#39; described above gives it some difficulty. A mesh that doesn&#39;t gracefully convert to an <code>MNMesh</code> won&#39;t always behave as expected in Mesh Smooth, Tesselate, Shape Merge, Connect, Boolean, Slice, and possibly other modifiers.</p>\n\
<p>Also, if a mesh is not constructed in a straightforward manner, the user may experience some difficulty working with it in modifiers such as Edit Mesh. Of course, the user can construct very strange meshes themselves in Edit or Editable Mesh, but its best if parametric objects and modifiers avoid introducing this sort of complexity when it isn&#39;t needed.</p>\n\
<p>To that end, the following style guidelines for creating pipeline-friendly meshes in 3ds Max are presented. None of these are hard-and-fast rules, but the more you are able to follow, the more flexible your mesh will be within the 3ds Max environment.</p>\n\
<h2 id=\"1-reference-each-edge-at-most-once-in-each-direction-\">1. Reference each edge at most once in each direction.</h2><p>3ds Max meshes don&#39;t really have edges, so what this means is that if you use the vertices (a,b) in a face, you should only have one other face that uses both of these vertices, and it should reference them in the other direction (b,a). This other face is the &#39;other side&#39; of the edge (a,b). If the same edge is used more than twice, or more than once in the same direction, the MeshSmooth modifier and other MNMesh-based effects have to compensate by duplicating the vertices used in the edge, once for each extra face using this edge.</p>\n\
<p><img src=\"./images/GUID-8943DEE9-B756-43EB-BDB2-A0FFB7EBA66B-low.gif\" alt=\"\"></p>\n\
<p>Figure 1A: Typical use of an edge</p>\n\
<p>Face 0, which has vertices (0,3,2), uses the hidden edge in the (0,3) direction. Face 1, with vertices (0,1,3), uses the same edge in the (3,0) direction. The other edges are &#39;1-sided&#39;, since there&#39;s no face on their other side.</p>\n\
<p><img src=\"./images/GUID-5DC1B357-9059-4FF3-A1BE-945F44E22CF1-low.gif\" alt=\"\"></p>\n\
<p>Figure 1B: Overuse of an edge</p>\n\
<p>The face on the left uses the central edge in the (7,1) direction; the other 5 use it in the (1,7) direction. This can be inconvenient for the user and for many pipeline modifiers. Also, it generally doesn&#39;t respond well to smoothing groups.</p>\n\
<p>Another situation in which edges are referenced too many times is when a zero-thickness mesh is constructed, with faces on both sides.</p>\n\
<p><img src=\"./images/GUID-AC917D47-456B-4001-8ADF-502F94F3F233-low.gif\" alt=\"\"></p>\n\
<p><img src=\"./images/GUID-68400875-4346-41E9-974B-E795B1BB1812-low.gif\" alt=\"\"></p>\n\
<p>The above two views show the same zero-thickness mesh, with 9 points, that has faces representing both the front and back. The edges connecting the outside to the central point are used twice by front-facing faces and twice by back-facing edges. This also can cause problems in the pipeline. Additionally, some systems may not display edges properly, as shown above at right: the edge connecting the top middle and base middle vertices should be visible, but doesn&#39;t show up because both the front-facing and back-facing faces have the same Z-depth.</p>\n\
<h2 id=\"2-avoid-self-intersection\">2. Avoid self-intersection</h2><p>For an example of self-intersecting meshes, we need look no further than the standard 3ds Max primitive Teapot. Each of the four Teapot components is a separate mesh element, but the handle and spout elements both intersect the main body. Even if this mesh is closed with the CapHoles modifier, it causes trouble for Booleans because once again, what is &#39;inside&#39; and what is &#39;outside&#39; the teapot isn&#39;t very well defined.</p>\n\
<p>For an alternative to self-intersection, when (as in the teapot) computing the proper, connected result would be taxing, you may want to use the Boolean itself within the SDK. If separate mesh elements are closed and otherwise acceptable to Boolean, you can just Union them together. Boolean is a bit slow, however, and there are still some known cases where it actually produces an open result. See the <code>MakeBoolean</code> method in class <code>MNMesh</code> (<code>MNMesh Boolean Operations</code>).</p>\n\
<h2 id=\"3-avoid-creating-faces-with-vertices-located-at-the-same-place-\">3. Avoid creating faces with vertices located at the same place.</h2><p>A face referencing the same vertex twice, or two different vertices located in the same place, makes no contribution to your mesh and can interfere with display and rendering. Also, it confuses algorithms that traverse faces, such as ShapeMerge or Editable Mesh&#39;s new Cut function. Finally, Edit Mesh users can find it very frustrating to have to sort through multiple vertices at the same location in order to move the one they want.</p>\n\
<h2 id=\"4-try-not-to-39-bridge-39-separate-mesh-components-with-a-single-vertex-\">4. Try not to &#39;bridge&#39; separate mesh components with a single vertex.</h2><p><img src=\"./images/GUID-E4A14576-6CDF-4A1D-A802-6D803C88F04D-low.gif\" alt=\"\"></p>\n\
<p>The above two boxes share a single vertex, which is selected. Such vertices are split by <code>MNMesh</code> conversions. <code>MNMesh</code> effects deal with such vertices adequately, but the duplication is occasionally a surprise.</p>\n\
<h2 id=\"5-if-a-mesh-looks-like-a-single-element-it-probably-should-be-\">5. If a mesh looks like a single element, it probably should be.</h2><p>A user shouldn&#39;t be able to take what looks like a single object apart by moving &#39;element&#39; selections in Edit Mesh. If a model looks like a box, it shouldn&#39;t behave like a collection of 6 separate faces. This will give the user more reasonable results not only in modifiers like MeshSmooth, but also when they edit the object in Edit Mesh.</p>\n\
<h2 id=\"6-whenever-convenient-close-your-meshes-\">6. Whenever convenient, close your meshes.</h2><p>A &#39;closed&#39; mesh is one in which every edge has exactly two faces: one on each side. Closed meshes represent 3-dimensional objects. Open meshes are mere shells. Closed objects respond well to Boolean operations; open ones can&#39;t, since there&#39;s no way for the Boolean to know which parts of operand A are supposed to be &#39;inside&#39; operand B, and vice versa. Closed meshes have measurable volume; open ones do not.</p>\n\
<p>This style point is less important than the others, because users are used to dealing with it. Any user who has models in Editable Mesh, for instance, deletes faces and constructs new ones all the time. However, avoid catching users off-guard: try not to leave tiny holes that can&#39;t be seen, say by putting open edges very close to each other. Users may not see them, but any hole will cause problems in pipeline effects that depend on solid, closed meshes, and may affect rendering adversely.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../meshes.html\">Meshes</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";