var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"extracting_the_mesh_from_a_node\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Extracting the Mesh from a Node</h1></div><p>The <code>INode::EvalWorldState()</code> method should be called when a developer needs to work with an object that is the result of the node&apos;s pipeline. This is the object that appears in the scene. This may be an object that is not referenced - it may just be an object that has flowed down the pipeline.</p>\n\
<p>For example, if there is a Sphere object in the scene that has a Bend modifier and Taper modifier applied, <code>INode::EvalWorldState()</code> would return an <code>ObjectState</code> containing a <code>TriObject</code>. This is the result of the sphere turning into a <code>TriObject</code> and being bent and tapered (just as it appeared in the scene). For more information on the <code>ObjectState</code> see the topic <a href=\"./../geometry_pipeline_system/object_state.html\">Object State</a>.</p>\n\
<p>If a developer needs to access the object that the node in the scene references, then the method <code>INode::GetObjectRef()</code> should be used instead.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Get the object from the node\n\
ObjectState os = node-&gt;EvalWorldState(ip-&gt;GetTime());\n\
if (os.obj-&gt;SuperClassID()==GEOMOBJECT_CLASS_ID)\n\
{\n\
  obj = (GeomObject*)os.obj;\n\
  //...\n\
}</code></pre><p>The following code shows how a <code>TriObject</code> can be retrieved from a node. Note on the code that if you call <code>Object::ConvertToType()</code> on an object and it returns a pointer other than itself, you are responsible for deleting that object.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Return a pointer to a TriObject given an INode or return NULL\n\
// if the node cannot be converted to a TriObject\n\
TriObject *Utility::GetTriObjectFromNode(INode *node, BOOL &amp;deleteIt)\n\
{\n\
  deleteIt = FALSE;\n\
  Object *obj = node-&gt;EvalWorldState(ip-&gt;GetTime()).obj;\n\
  if (obj-&gt;CanConvertToType(Class_ID(TRIOBJ_CLASS_ID, 0)))\n\
  {\n\
    TriObject *tri = (TriObject *) obj-&gt;ConvertToType(ip-&gt;GetTime(),\n\
    Class_ID(TRIOBJ_CLASS_ID, 0));\n\
\n\
    // Note that the TriObject should only be deleted\n\
    // if the pointer to it is not equal to the object\n\
    // pointer that called ConvertToType()\n\
    if (obj != tri) deleteIt = TRUE;\n\
    return tri;\n\
  }\n\
  else\n\
  {\n\
    return NULL;\n\
  }\n\
}</code></pre><p>The following code demonstrates usage of the above function.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Retrieve the TriObject from the node\n\
BOOL deleteIt = FALSE;\n\
TriObject *triObject = GetTriObjectFromNode(ip-&gt;GetSelNode(0), deleteIt);\n\
\n\
// Use the TriObject if available\n\
if (!triObject) return;\n\
\n\
// perform processing\n\
// ...\n\
\n\
// Delete it when done if we own it\n\
if (deleteIt) triObject-&gt;DeleteMe();\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../meshes.html\">Meshes</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";