var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"mapping_coordinate_modifiers\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Mapping Coordinate Modifiers</h1></div><p>These modifiers alter the texture mapping coordinates of an objects. They can add new mapping coordinates to unmapped objects or modify the existing mapping coordinates of objects. A modifier that adds new mapping coordinates is the UVW Mapping Modifier. The full source code is available in  <em>\\MAXSDK\\SAMPLES\\MODIFIERS\\MAPMOD.CPP</em> . Below are the implementations of <code>InputType()</code>, <code>ChannelsUsed()</code>, and <code>ChannelsChanged()</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Class_ID InputType() { return mapObjectClassID; }\n\
ChannelMask ChannelsUsed()\n\
{\n\
   return PART_GEOM | PART_TOPO | PART_SELECT |\n\
   PART_SUBSEL_TYPE | PART_VERTCOLOR;\n\
}\n\
ChannelMask ChannelsChanged()\n\
{\n\
   return TEXMAP_CHANNEL|PART_VERTCOLOR;\n\
}</code></pre><p>The mapping modifier requests an object that is mappable by specifying the <code>Class_ID``mapObjectClassID</code>. Objects that are mappable (or know how to convert themselves to a mappable object) will return <code>TRUE</code> in the method <code>CObject::CanConvertToType()</code> when passed this <code>Class_ID</code>. This indicates that they may have texture coordinates assigned to them and they implement the method <code>Object::ApplyUVWMap()</code>. Note how the Vertex Color channel is needed and changed as well as the texture map channel. This is because 3ds Max uses the second mapping channel as the vertex color channel and the user is able to specify that the second mapping channel may be used for the mapping coordinates.</p>\n\
<p>Here is the implementation of <code>ModifyObject()</code>. Note that when the modifier is done it updates the validity of the texture map channel by calling <code>UpdateValidity()</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void MapMod::ModifyObject(TimeValue t, ModContext &amp;mc,\n\
            ObjectState *os, INode *node)\n\
{\n\
   // If it&apos;s not a mappable object then we can&apos;t help\n\
   Object *obj = os-&gt;obj;\n\
   if (!obj-&gt;IsMappable()) return;\n\
\n\
   // Get pblock values \n\
   int type, uflip, vflip, wflip, cap, channel; \n\
   float utile, vtile, wtile;\n\
   pblock-&gt;GetValue(PB_MAPTYPE, t,type,FOREVER);\n\
   pblock-&gt;GetValue(PB_UTILE,t,utile,FOREVER);\n\
   pblock-&gt;GetValue(PB_VTILE,t,vtile,FOREVER);\n\
   pblock-&gt;GetValue(PB_WTILE,t,wtile,FOREVER);\n\
   pblock-&gt;GetValue(PB_UFLIP,t,uflip,FOREVER);\n\
   pblock-&gt;GetValue(PB_VFLIP,t,vflip,FOREVER);\n\
   pblock-&gt;GetValue(PB_WFLIP,t,wflip,FOREVER);\n\
   pblock-&gt;GetValue(PB_CAP,t,cap,FOREVER);\n\
   pblock-&gt;GetValue(PB_CHANNEL, t,channel,FOREVER);\n\
\n\
   // Prepare the controller and set up mats\n\
   if (!tmControl || (flags&amp;CONTROL_OP) || (flags&amp;CONTROL_INITPARAMS))\n\
   {\n\
     InitControl(mc,obj,type,t);\n\
   }\n\
   Matrix3 tm; \n\
   tm = Inverse(CompMatrix(t,&amp;mc,NULL));\n\
\n\
   obj-&gt;ApplyUVWMap(type,utile,vtile,wtile,uflip,vflip,wflip,cap,tm,channel);\n\
\n\
   // The tex mapping depends on the geom and topo so make sure the validity interval reflects this.\n\
   Interval iv = LocalValidity(t);\n\
   iv = iv &amp; os-&gt;obj-&gt;ChannelValidity(t,GEOM_CHAN_NUM);\n\
   iv = iv &amp; os-&gt;obj-&gt;ChannelValidity(t,TOPO_CHAN_NUM);\n\
\n\
   os-&gt;obj-&gt;UpdateValidity(TEXMAP_CHAN_NUM,iv); \n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../modifiers.html\">Modifiers</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";