var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"modifier_channels\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Modifier Channels</h1></div><p>Objects flowing down the geometry pipeline are broken into separate channels. That is, the pipeline can operate on only portions of objects and not the entire thing as a whole. Some examples of channels that objects are broken into are the geometry portion (points), the topology portion (polygons or faces), the selection level of the object (object, face, vertex, etc.), and its texture coordinates. This allows the pipeline to operate more efficiently, by only copying the necessary data.</p>\n\
<h2 id=\"indicating-required-input-channels\">Indicating Required Input Channels</h2><p>Modifiers must inform 3ds Max which channels they require to perform their modification. Copies of the needed channels are made and passed up the pipeline. 3ds Max will only pass copies of a minimum set of needed channels up the pipeline. If a certain channel is not required by any modifiers anywhere in a pipeline it is not copied and passed along.</p>\n\
<p>Modifiers inform 3ds Max which channels they require by implementing the method <code>Modifier::ChannelsUsed()</code>. This list of channels must include the channels the modifier actually alters but may include more. Here is the implementation of <code>Modifier::ChannelsUsed()</code> by the Volume Select modifier:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">ChannelMask ChannelsUsed()\n\
{\n\
   return OBJ_CHANNELS;\n\
}</code></pre><p>Note that <code>OBJ_CHANNELS</code> is defined as:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#define OBJ_CHANNELS \\\n\
(TOPO_CHANNEL | GEOM_CHANNEL | SELECT_CHANNEL | TEXMAP_CHANNEL | MTL_CHANNEL | SUBSEL_TYPE_CHANNEL | DISP_ATTRIB_CHANNEL | VERTCOLOR_CHANNEL | GFX_DATA_CHANNEL)</code></pre><p>This means that the pipeline must have all these channels up to date in the object that gets passed to the modifier before it can properly do its work. Technically, this is actually overkill for the Normals modifier. It wouldn&apos;t need the <code>TEXMAP_CHANNEL</code> and <code>VERTCOLOR_CHANNEL</code> since they aren&apos;t in fact used. It really only needs the <code>GEOM_CHANNEL</code>, <code>TOPO_CHANNEL</code>, and the <code>SELECT_CHANNEL</code>.</p>\n\
<p>For more information see the topic <a href=\"./../../../writing_plug-ins/writing_modifier_plug-ins/indicating_required_channels.html\">Indicating Required Channels</a>.</p>\n\
<h2 id=\"indicating-changed-channels\">Indicating Changed Channels</h2><p>A modifier must also inform 3ds Max which channels it will actually alter its function as a modifier. It does this by implementing the method <code>Modifer::ChannelsChanged()</code>. Here is the implementation of <code>ChannelsChanged()</code> by the Volume Select modifier:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">ChannelMask ChannelsChanged()\n\
{\n\
   return SELECT_CHANNEL|SUBSEL_TYPE_CHANNEL|GEOM_CHANNEL;\n\
}</code></pre><p>This indicates that this modifier alters the selection channel, the sub-object selection channel, and the geometry channel. Note that the geometry channel is modified because the vertices themselves get selected via the <code>vertSel``BitArray</code> in the <code>Mesh</code> class and are thus modified.</p>\n\
<p>For more information see the topic <a href=\"./../../../writing_plug-ins/writing_modifier_plug-ins/indicating_changed_channels.html\">Indicating Changed Channels</a>.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../modifiers.html\">Modifiers</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";