var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"simple_modifiers\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Simple Modifiers</h1></div><p>A simple modifier is a special case of a geometry modifier. Simple modifiers can derive from <code>SimpleMod2</code> instead of <code>Modifier</code> and will have fewer methods to implement. The Bend modifier example is derived from <code>SimpleMod2</code>.</p>\n\
<p>To be a &apos;simple&apos; modifier, the following assumptions are made:</p>\n\
<ul>\n\
<li>The modifier only modifies the geometry channel. In other words, the modifier is of the general type of modifiers that operate on <a href=\"./../../../geometric_objects/deformable_objects.html\">Deformable Objects</a>.</li>\n\
<li>The modifier uses an instance of a class derived from <code>Deformer</code> to do the modification.</li>\n\
<li>The modifier&apos;s gizmo is represented as a 3D box that has had the modifier applied to it. For example, the Bend modifier&apos;s gizmo is a bent 3D box.</li>\n\
</ul>\n\
<p>Simple Modifiers don&apos;t directly implement <code>Modifier::ModifyObject()</code> instead they implement the method <code>SimpleMod::GetDeformer()</code>. This method returns a callback object which is an instance of the <code>Deformer</code> class. The <code>Deformer</code> class provides a virtual method <code>Deformer::Map()</code> that performs the alteration of a single point of an object. To see an example of an implementation of this method see <code>\\\\MAXSDK</code>\\SAMPLES\\MODIFIERS\\BEND.CPP.</p>\n\
<p>The base class <code>SimpleMod</code> provides the following default implementations for <code>Modifier::InputType()</code>, <code>Modifier::ChannelsUsed()</code> and <code>Modifier::ChannelsChanged()</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Class_ID InputType() {\n\
  return defObjectClassID;\n\
}\n\
\n\
ChannelMask ChannelsUsed() {\n\
   return PART_GEOM|PART_TOPO|SELECT_CHANNEL|SUBSEL_TYPE_CHANNEL;\n\
}\n\
\n\
ChannelMask ChannelsChanged() {\n\
  return PART_GEOM;\n\
}\n\
</code></pre><p>Note that <code>Modifier::InputType()</code> return the <code>Class_ID</code> of deformable objects. As noted above, these are a type of objects that have &apos;points&apos; to deform. Also notice that simple modifiers require the geometry, topology, and selection channels. They need the topology channel up to date because these modifiers can work on a selection set.</p>\n\
<p>In this case a modifier needs to find the vertices that are selected because they are part of selected faces. To make sure the face selection is up to date the topology channel is specified. The selection channels are required for the same reason -- the modifier may operate on only the selection set if that&apos;s what the object is providing in the pipeline.</p>\n\
<p>The base class <code>SimpleMod</code> also provides an implementation of <code>ModifyObject()</code> that calls <code>Object::Deform()</code> method on the input object. The object in turn calls <code>Deformer::Map()</code> on all points. Here is the implementation of <code>SimpleMod::ModifyObject()</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void SimpleMod::ModifyObject(TimeValue t, ModContext &amp;mc, ObjectState *os, INode *node)\n\
{\n\
   Interval valid = GetValidity(t);\n\
   Matrix3 modmat,minv;\n\
\n\
   // These are inverted because that&apos;s what is usually needed\n\
   // for displaying/hit testing\n\
   minv = CompMatrix(t,mc,idTM,valid,TRUE);\n\
   modmat = Inverse(minv);\n\
\n\
   os-&gt;obj-&gt;Deform(&amp;GetDeformer(t,mc,modmat, minv), TRUE);\n\
   os-&gt;obj-&gt;UpdateValidity(GEOM_CHAN_NUM,valid);\n\
}</code></pre><p>Here is the default implementation of <code>Object::Deform()</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void Object::Deform(Deformer *defProc,int useSel)\n\
{\n\
   int nv = NumPoints();\n\
   for (int i=0; i&lt;nv; i++)\n\
     SetPoint(i,defProc-&gt;Map(i,GetPoint(i)));\n\
   PointsWereChanged();\n\
}</code></pre><p>The method <code>Object::PointsWereChanged()</code> is called to let that object know that its points have been altered.</p>\n\
<p>Some objects override the base class definition of <code>Object::Deform()</code> to provide other ways of modifying the object, such as the <code>PatchObject</code> and <code>SplineShape</code> objects. If an object only wants to modify selected points only for example, it would need to override <code>Object::Deform()</code> and observe the <code>useSel</code> parameter that indicates it should use the current selection. The default method ignores the <code>useSel</code> parameter.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../object_modifiers.html\">Object Modifiers</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";