var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"whole_object_modifiers\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Whole Object Modifiers</h1></div><p>Whole object modifiers that don&apos;t alter any parts of the object, instead they completely replace an object with a new object. Examples of this are the Extrude and Lathe modifiers. Both Extrude and Lathe convert a spline object into a mesh, patch or NURBS object, completely replacing the entire object. The full source code for Extrude is available in  <em>\\MAXSDK\\SAMPLES\\MODIFIERS\\EXTRUDE.CPP</em> .</p>\n\
<p>Below are the implementations of <code>Modifier::InputType()</code>, <code>Modifier::ChannelsUsed()</code>, and <code>Modifier::ChannelsChanged()</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Class_ID InputType() {\n\
  return genericShapeClassID;\n\
}\n\
\n\
ChannelMask ChannelsUsed() {\n\
  return PART_GEOM|PART_TOPO;\n\
}\n\
\n\
ChannelMask ChannelsChanged() {\n\
  return PART_ALL;\n\
}</code></pre><p>Note that the modifier indicates it changes all the channels by returning <code>PART_ALL</code> from <code>Modifier::ChannelsChanged()</code>.</p>\n\
<p>Below is a subset of the implementation of <code>Modifier::ModifyObject()</code> for the Extrude modifier with the code for the NURBS and Patch cases are removed for brevity and simplicity. In the mesh case, the key thing to notice is that a new <code>TriObject</code> is created, the <code>TriObject</code>&apos;s mesh is created from the spline, and the new object is placed into the pipeline by assigning its pointer to the <code>ObjectState</code>&apos;s object pointer.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void ExtrudeMod::ModifyObject(TimeValue t, ModContext &amp;mc,\n\
              ObjectState *os, INode *node)\n\
{ \n\
   // Get our personal validity interval...\n\
   Interval valid = GetValidity(t);\n\
   valid &amp;= os-&gt;obj-&gt;ChannelValidity(t,TOPO_CHAN_NUM);\n\
   valid &amp;= os-&gt;obj-&gt;ChannelValidity(t,GEOM_CHAN_NUM);\n\
\n\
   int output;\n\
   pblock-&gt;GetValue(PB_OUTPUT, TimeValue(0), output, FOREVER);\n\
   switch (output)\n\
   {\n\
     case NURBS_OUTPUT:\n\
     {\n\
      // ...\n\
      break;\n\
     }\n\
     case PATCH_OUTPUT:\n\
     {\n\
      // ...\n\
      break;\n\
     }\n\
     case MESH_OUTPUT:\n\
     {\n\
      // BuildMeshFromShape fills in the TriObject&apos;s mesh,\n\
      // then we stuff the TriObj into the pipeline.\n\
       TriObject *tri = CreateNewTriObject();\n\
      BuildMeshFromShape(t, mc, os, tri-&gt;mesh);\n\
      tri-&gt;SetChannelValidity(TOPO_CHAN_NUM, valid);\n\
      tri-&gt;SetChannelValidity(GEOM_CHAN_NUM, valid);\n\
      tri-&gt;SetChannelValidity(TEXMAP_CHAN_NUM, valid);\n\
      tri-&gt;SetChannelValidity(MTL_CHAN_NUM, valid);\n\
      tri-&gt;SetChannelValidity(SELECT_CHAN_NUM, valid);\n\
      tri-&gt;SetChannelValidity(SUBSEL_TYPE_CHAN_NUM, valid);\n\
      tri-&gt;SetChannelValidity(DISP_ATTRIB_CHAN_NUM, valid);\n\
      os-&gt;obj = tri;\n\
      break;\n\
     }\n\
   }\n\
   os-&gt;obj-&gt;UnlockObject();\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../object_modifiers.html\">Object Modifiers</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";