var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"validity_intervals\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Validity Intervals</h1></div><p>A validity intervals describes a range of time over which the modification preformed by the modifier is accurate. Many things in 3ds Max can be animated and thus change over time. Validity intervals allow 3ds Max to minimize redundant evaluations of modifications.</p>\n\
<p>For example, consider the case above of the procedural sphere and the volume select modifier applied to select a set of vertices. A 3ds Max user might animate the number of segments in the sphere as an animated camera gets closer to it -- this keeps the silhouette of the sphere from appearing faceted. The volume select modifier applied to the sphere needs to select all the vertices in the region defined by its gizmo which represents its volume. Since the segment count in the sphere is changing, the number of vertices in the region defined by the modifier&apos;s gizmo is changing. Therefore the selection done at one frame may not be valid at the next frame. In order to tell 3ds Max how long the modification is valid, what&apos;s called a validity interval is computed and returned. This describes the range of time over which the modification is accurate and up to date.</p>\n\
<p>There are two methods that a modifier needs to call or implement. These are <code>Object::UpdateValidity()</code> and <code>Modifier::LocalValidity()</code> (or <code>SimpleMod::GetValidity()</code>)</p>\n\
<p>When a modifier is finished altering an object it needs to include its interval in the validity interval of the object. This way if an object was static, but had an animated modifier applied to it, 3ds Max would know that the modifier would need to be re-evaluated if the user moves to a new time. To do this, the modifier calls the <code>UpdateValidity()</code> method on the object, specifying the channel and the modifier&apos;s interval. Modifiers that only affect the geometry channel would specify <code>GEOM_CHAN_NUM</code>. Modifiers that affect other channels would have to call this method once for each channel modified. The interval that is passed in to this method is then intersected with the interval that the object keeps for each channel. So as the object travels through the pipeline, its validity intervals are potentially getting smaller as (possibly) animated modifiers are applied to it.</p>\n\
<p>Here are the calls to <code>UpdateValidity()</code> that the Volume Select Modifier makes to the object it modifies. These calls are made inside the modifier&apos;s <code>ModifyObject()</code> method.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> obj-&gt;UpdateValidity(SUBSEL_TYPE_CHAN_NUM,FOREVER);\n\
 obj-&gt;UpdateValidity(SELECT_CHAN_NUM,valid);\n\
 obj-&gt;UpdateValidity(GEOM_CHAN_NUM,valid);\n\
 obj-&gt;UpdateValidity(TOPO_CHAN_NUM,valid);</code></pre><p>The modifier must also implement a method to return to 3ds Max its own validity, independent of the object it is modifying. This method is <code>Modifier::LocalValidity()</code>. This value is computed by starting an interval off at <code>FOREVER</code>, and intersecting the validity intervals of all the animated parameters of the modifier. In the case of the Volume Select Modifer, it has a single animated parameter -- its gizmo which represents the actual extents of the volume it selects. Below is the implementation of this method by Volume Select.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Interval SelMod::LocalValidity(TimeValue t)\n\
{ \n\
   Interval valid = FOREVER;\n\
   if (tmControl) {\n\
     Matrix3 tm(1);\n\
     tmControl-&gt;GetValue(t,&amp;tm,valid,CTRL_RELATIVE);\n\
   }\n\
   return valid;\n\
}</code></pre><p>Note the interval is started as <code>FOREVER</code>. Then the gizmo&apos;s transform controller&apos;s <code>GetValue()</code> method is called passing it this initial interval of <code>FOREVER</code>. The controller&apos;s <code>GetValue()</code> method will update this interval to reflect the validity of the gizmo. This interval is then returned. Note that if the volume has not been animated yet, and thus there is not a controller assigned to the gizmo yet, <code>FOREVER</code> is returned. This means the modifier is valid at all times.</p>\n\
<p>Some modifiers are sub-classed from SimpleMod and not Modifier. To return the validity interval of the modifier to <code>SimpleMod</code>, the developer must implement a method named <code>GetValidity()</code>. <code>SimpleMod</code> then provides the implementation of <code>LocalValidity()</code> itself but calls <code>GetValidity()</code> on the <code>SimpleMod</code>ifier.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../modifiers.html\">Modifiers</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";