var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"creating_a_patch\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Creating a Patch</h1></div><p>A good sample to look at to see how patches are created is the patch grid plugin. The source code is in <em>\\MAXSDK\\SAMPLES\\OBJECTS\\PATCHGRD.CPP</em> .</p>\n\
<p>The code below is a portion of the source of this plugin. This section discusses some of the key methods used in the creation of the patch.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Quad patch layout\n\
//\n\
// A---&gt; ad ----- da &lt;---D\n\
// | |\n\
// | |\n\
// v v\n\
// ab i1 i4 dc\n\
//\n\
// | |\n\
// | |\n\
//\n\
// ba i2 i3 cd\n\
// ^ ^\n\
// | |\n\
// | |\n\
// B---&gt; bc ----- cb &lt;---C\n\
//\n\
// Vertices ( a b c d ) are in counter clockwise order when viewed from\n\
// outside the surface\n\
void QuadPatchObject::BuildPatch(TimeValue t,PatchMesh&amp; amesh)\n\
{\n\
   intix,iy,np,kv;\n\
   intwsegs,lsegs,nv;\n\
   Point3 v,p;\n\
   floatl, w;\n\
\n\
   // Start the validity interval at forever and widdle it down.\n\
   ivalid = FOREVER;\n\
   pblock-&gt;GetValue( PB_LENGTH, t, l, ivalid );\n\
   pblock-&gt;GetValue( PB_WIDTH, t, w, ivalid );\n\
   pblock-&gt;GetValue( PB_LSEGS, t, lsegs, ivalid );\n\
   pblock-&gt;GetValue( PB_WSEGS, t, wsegs, ivalid );\n\
\n\
   intlv = lsegs + 1;\n\
   intwv = wsegs + 1;\n\
   intnverts = lv * wv;\n\
   intnpatches = lsegs * wsegs;\n\
   intnexteriors = npatches * 4 + lsegs * 2 + wsegs * 2;\n\
   intninteriors = npatches * 4;\n\
   intnvecs = ninteriors + nexteriors;</code></pre><p>Here the number of vertices, texture vertices, vectors, patches and texture patches are established in the <code>PatchMesh</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">   amesh.setNumVerts(nverts);\n\
   amesh.setNumTVerts(textured ? nverts : 0);\n\
   amesh.setNumVecs(nvecs);\n\
   amesh.setNumPatches(npatches);\n\
   amesh.setNumTVPatches(textured ? npatches : 0);\n\
\n\
   v = Point3(-w, -l, 0.0f) / 2.0f;\n\
\n\
   floatdx = w/wsegs;\n\
   floatdy = l/lsegs;\n\
   floatfws = (float)wsegs;\n\
   floatfls = (float)lsegs;</code></pre><p>Next the vertices are created. The <code>setVert()</code> method of the <code>PatchMesh</code> is used to do this.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Create the vertices.\n\
nv = 0;\n\
p.z = v.z;\n\
p.y = v.y;\n\
for(iy = 0; iy &lt;= lsegs; iy++)\n\
{\n\
   p.x = v.x;\n\
   for (ix = 0; ix &lt;= wsegs; ix++)\n\
   {\n\
   if(textured)\n\
     amesh.tVerts[nv] = UVVert((float)ix / fws, (float)iy / fls, 0.0f);\n\
     amesh.verts[nv].flags = PVERT_COPLANAR;\n\
     amesh.setVert(nv++, p);\n\
     p.x += dx;\n\
   }\n\
   p.y += dy;\n\
}</code></pre><p>Next the patches are created. The <code>PatchMesh</code> method <code>MakeQuadPatch()</code> is used once the vertices, vectors, and interiors are set up.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Create patches.\n\
np = 0;\n\
int interior = nexteriors;\n\
int vecRowInc = lsegs * 2;\n\
int vecColInc = wsegs * 2;\n\
\n\
for(iy=0; iy&lt;lsegs; iy++)\n\
{\n\
   kv = iy*(wsegs+1);\n\
   intrv = iy * vecColInc; // Row vector start\n\
   intcv = vecColInc * lv + iy * 2; // column vector start\n\
   for (ix=0; ix&lt;wsegs; ix++,++np)\n\
   {\n\
     Patch &amp;p = amesh.patches[np];\n\
     int a = kv, b = kv+1, c = kv+wsegs+2, d = kv + wsegs + 1;\n\
     int ab = rv, ba = rv+1;\n\
     int bc = cv+vecRowInc, cb = cv + vecRowInc + 1;\n\
     int cd = rv+vecColInc+1, dc = rv+vecColInc;\n\
     int da = cv + 1, ad = cv;\n\
\n\
     amesh.MakeQuadPatch(np, a, ab, ba, b, bc,\n\
              cb, c, cd, dc, d, da, ad,\n\
              interior, interior+1, interior+2,\n\
              interior+3, 1);</code></pre><p>If textures are being used, then <code>setTVerts()</code> is called.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">if(textured)\n\
{\n\
   TVPatch &amp;tp = amesh.tvPatches[np];\n\
   tp.setTVerts(a,b,c,d);\n\
}</code></pre><p>Next the vectors are set using <code>setVec()</code>. One third of the distance from vertex <code>a</code> to vertex <code>b</code> is vector <code>ab</code>, and one third of the distance from vertex <code>b</code> to vertex <code>a</code> is <code>ba</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">     // Create the default vectors\n\
     Point3 pa = amesh.getVert(a).p;\n\
     Point3 pb = amesh.getVert(b).p;\n\
     Point3 pc = amesh.getVert(c).p;\n\
     Point3 pd = amesh.getVert(d).p;\n\
\n\
     amesh.setVec(ab, pa + (pb - pa) / 3.0f);\n\
     amesh.setVec(ba, pb - (pb - pa) / 3.0f);\n\
     amesh.setVec(bc, pb + (pc - pb) / 3.0f);\n\
     amesh.setVec(cb, pc - (pc - pb) / 3.0f);\n\
     amesh.setVec(cd, pc + (pd - pc) / 3.0f);\n\
     amesh.setVec(dc, pd - (pd - pc) / 3.0f);\n\
     amesh.setVec(da, pd + (pa - pd) / 3.0f);\n\
     amesh.setVec(ad, pa - (pa - pd) / 3.0f);\n\
     kv++;\n\
     cv += vecRowInc;\n\
     rv += 2;\n\
     interior += 4;\n\
   }\n\
}\n\
// Verify that we have the right number of parts!\n\
assert(np==npatches);\n\
assert(nv==nverts);</code></pre><p>Next the <code>PatchMesh</code> method <code>buildLinkages()</code> is called. This makes sure that everything is connected correctly and there are not any edges that are used by more than two patches. It will return FALSE if these conditions are not met.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Finish up patch internal linkages (and bail out if it fails!)\n\
assert(amesh.buildLinkages());</code></pre><p>Next the method <code>computeInteriors()</code> is called. For any automatic patches, the interior vertices will be computed.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Calculate the interior bezier points on the PatchMesh&apos;s patches\n\
amesh.computeInteriors();</code></pre><p>Then the geometry cache is cleared to make sure that any cache that might have been in the <code>PatchMesh</code> is emptied.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> amesh.InvalidateGeomCache();</code></pre><p>Finally the validity of the <code>Mesh</code> cache is set to FALSE to indicate the <code>PatchMesh</code> has been changed.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">   // Tell the PatchObject its mesh just got changed\n\
   meshValid = FALSE;\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../patches_and_faces.html\">Patches and Faces</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";