var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"capping_a_shape_with_a_mesh\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Capping a Shape with a Mesh</h1></div><p>This section discusses the capping of shapes using a mesh. The SDK provides tools for easily creating these mesh caps for shapes.</p>\n\
<p>Typically for an extruded object, the capping on the front of the object is exactly the same as the capping on the back of the object. For example, if you extrude the letter &apos;M&apos;. It&apos;s redundant to have to cap this separately for the front and back (as they are the same). When dealing with complex shapes, having to go back and do the second cap is very time consuming. So the capping system tries to cache information to speed up the process.</p>\n\
<p>Sample code for capping can be found in the extrude and lathe modifiers. These plugins are in  <em>\\MAXSDK\\SAMPLES\\MODIFIERS\\SURFREV.CPP</em>  and <em>\\MAXSDK\\SAMPLES\\MODIFIERS\\EXTRUDE.CPP</em> .</p>\n\
<p>The following code fragments are taken from <em>EXTRUDE.CPP</em> . This demonstrates the basic process for capping a shape. This code is part of the method <code>ExtrudeMod::BuildMeshFromShape()</code>.</p>\n\
<p>For this example, we are starting of with a ShapeObject <code>shape</code> that we want capped with a mesh. The first thing to do is convert the shape to a <code>PolyShape</code>. This greatly simplifies the mesh conversion. The code below shows how this is done.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Make the shape convert itself to a PolyShape.\n\
// This makes our mesh conversion MUCH easier!\n\
PolyShape pShape;\n\
shape-&gt;MakePolyShape(t, pShape);</code></pre><p>Next, we organize the curves into a hierarchy. This automatically figures out the shape nesting and directions for proper capping.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">ShapeHierarchy hier = pShape.OrganizeCurves(t);</code></pre><p>Next we need to reverse the shapes whose directions are incorrect for the hierarchy. The hierarchy calculated above contains a <code>BitArray</code> that describes which shapes need to be reversed. This is passed into the <code>Reverse()</code> method of the <code>PolyShape</code> to tell it which shapes to reverse.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Need to flip the reversed curves in the shape!\n\
pShape.Reverse(hier.reverse);</code></pre><p>At this point the <code>PolyShape</code> is all set up with the proper clockwise/counter-clockwise ordering on all the polygons so all that needs to be done is to generate the faces. You may refer to the full source of <code>EXTRUDE</code>.CPP to see how this is done.</p>\n\
<p>The next thing is just to create the caps. This begins by instantiating a <code>MeshCapInfo</code> class and asking the <code>PolyShape</code> to make a cap. This fills up the <code>MeshCapInfo</code> class with all the information it needs to create a cap. The type can be morph or grid capping. Morph capping only uses the existing vertices in the <code>PolyShape</code> to generate the cap. The capping code does the best job it can given this constraint, however it is possible to wind up with long sliver-like faces on the cap. This is referred to as a morph cap because if you cap a shape using this method it does not generate any new vertices and you can then morph between shapes with the same number of vertices. A Grid cap generates new vertices in the interior of the shape in a grid pattern. This helps to break up the shape and helps reduce slivering. Grid capping will generate different number of vertices based on the shape and thus the shapes are not morphable.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MeshCapInfo capInfo;\n\
pShape.MakeCap(t, capInfo, capType);</code></pre><p>After this is done, the <code>MeshCapInfo</code> is cached within the shape. Therefore if this is needed again, no work needs to be done.</p>\n\
<p>Next, a <code>MeshCapper</code> object is created. This is done by passing the <code>PolyShape</code> as an argument to the constructor. This gets the <code>MeshCapper</code> ready for the topology of the shape. Developers don&apos;t need to understand the inner workings of the <code>MeshCapper</code>, it&apos;s just a tool used to aide in capping.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Build information for capping\n\
MeshCapper capper(pShape);</code></pre><p>Below is the code where the start of the extrusion is capped. Inside the capper is a <code>MeshCapPoly</code>. There is one for each polygon in the shape. The <code>MeshCapPoly</code> needs to know the corresponding mesh vertex for each vertex in the <code>PolyLine</code>. This is done by calling <code>SetVert()</code> on the <code>MeshCapPoly</code>. For example, this might associate vertex 0 in the <code>PolyLine</code> with vertex 200 in the mesh, vertex 1 with mesh vertex 220, etc.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">if(capStart)\n\
{\n\
   vert = 0;\n\
   for(poly = 0; poly &lt; polys; ++poly)\n\
   {\n\
     PolyLine &amp;line = pShape.lines[poly];\n\
     MeshCapPoly &amp;capline = capper[poly];\n\
     int lverts = line.numPts;\n\
     for(int v = 0; v &lt; lverts; ++v)\n\
      // Gives this vert&apos;s location in the mesh!\n\
       capline.SetVert(v, vert++);\n\
     vert += lverts * levels;\n\
   }</code></pre><p>The next thing that is done is used only for grid capping. A grid cap generates new vertices inside the shape that make up the grid. In the case of a SurfRev for example, the end cap might be rotated, or scaled in some manner. A matrix is required so the capper knows how to orient the vertices into the correct location. This matrix is ignored for non-grid capping.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Create a work matrix for grid capping\n\
Matrix3 gridMat = TransMatrix(offset1);</code></pre><p>The final step is to cap the mesh. This is done using a method of the capper named <code>CapMesh()</code>. This method is passed the output mesh, the <code>MeshCapInfo</code>, a flag that indicates if the cap should be flipped, the smoothing group number for all the faces in the cap, and a pointer to the orientation matrix.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> capper.CapMesh(mesh, capInfo, TRUE, 16, &amp;gridMat);</code></pre><p>Once this is done the shape has been capped with a mesh.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../shapes_and_splines.html\">Shapes and Splines</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";