var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"capping_a_shape_with_a_patch\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Capping a Shape with a Patch</h1></div><p>This section discusses the capping of shapes with patches. This is very similar to the mesh capping discussed above.</p>\n\
<p>Again, the following code fragments are taken from <em>EXTRUDE.CPP</em> . This demonstrates the basic process for capping a shape with a patch.</p>\n\
<p>Here we try to use a <code>BezierShape</code> to create the cap. First a <code>BezierShape</code> object is created. Then the shape to be capped is asked if it can make a transformation from itself to a <code>BezierShape</code>. If it can, we just ask it to make a <code>BezierShape</code>. If it cannot, we are forced to use a <code>PolyShape</code>. For patch capping this is much less desirable, but if we can&apos;t make a <code>BezierShape</code>, then this is used. The shape is asked to convert to a <code>PolyShape</code>, and then the <code>PolyShape</code> is converted to a <code>BezierShape</code>. This is a very poor conversion, as all it does is make a linear spline out of the <code>PolyShape</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// If the shape can convert itself to a BezierShape, have it do so!\n\
BezierShape bShape;\n\
\n\
if (shape-&gt;CanMakeBezier())\n\
   shape-&gt;MakeBezier(t, bShape);\n\
else\n\
{\n\
   PolyShape pShape;\n\
   shape-&gt;MakePolyShape(t, pShape);\n\
   bShape = pShape; // UGH -- Convert it from a PolyShape -- not good!\n\
}</code></pre><p>Next, the curves are organized into a hierarchy. This automatically figures out the shape nesting and directions for proper capping.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">ShapeHierarchy hier;\n\
bShape.OrganizeCurves(t, &amp;hier);</code></pre><p>Next we need to reverse the shapes whose direction is incorrect for the hierarchy. The hierarchy calculated above contains a <code>BitArray</code> that describes which shapes need to be reversed. This is passed into the reverse method of the <code>BezierShape</code> to tell it which shapes to reverse.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Need to flip the reversed polys...\n\
bShape.Reverse(hier.reverse);</code></pre><p>At this point the <code>BezierShape</code> is all set up with the proper clockwise/counter-clockwise ordering on all the polygons. Next the extrude modifier generates the patches. You may refer to the full source of <code>EXTRUDE</code>.CPP to see how this is done.</p>\n\
<p>The next step is to create the caps. This begins by instantiaing a <code>PatchCapInfo</code> class and asking the <code>BezierShape</code> to make a cap. This fills up the <code>PatchCapInfo</code> class with all the information it needs to create a cap.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">PatchCapInfo capInfo;\n\
bShape.MakeCap(t, capInfo);</code></pre><p>After this is done, the <code>PatchCapInfo</code> is cached within the shape. Therefore if this is needed again, no work needs to be done.</p>\n\
<p>Next, a <code>PatchCapper</code> object is created. This is done by passing the <code>BezierShape</code> as an argument to the constructor. This gets the <code>PatchCapper</code> ready for the topology of the shape. Developers don&apos;t need to understand the inner workings of the <code>PatchCapper</code>, it&apos;s just a tool used to aide in capping.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Build information for capping\n\
PatchCapper capper(bShape);</code></pre><p>Below is the code where the start of the extrusion is capped. Inside the capper is a <code>PatchCapPoly</code>. There is one for each polygon in the shape. The <code>PatchCapPoly</code> needs to know the vertex number in the patch mesh that corresponds to the given knot in the bezier spline. This is done by calling <code>SetVert()</code> on the <code>PatchCapPoly</code>. After this is done, the same thing happens for each of the vectors when <code>SetVec()</code> is called.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">if(capStart)\n\
{\n\
   vert = 0;\n\
   intbaseVec = 0;\n\
   for(poly = 0; poly &lt; polys; ++poly)\n\
   {\n\
     Spline3D *spline = bShape.splines[poly];\n\
     PatchCapPoly &amp;capline = capper[poly];\n\
     int lverts = spline-&gt;KnotCount();\n\
     for(int v = 0; v &lt; lverts; ++v)\n\
     {\n\
      // Gives this vert&apos;s location in the mesh!\n\
      capline.SetVert(v, vert++);\n\
     }\n\
     vert += lverts * levels;\n\
     vec = baseVec;\n\
     int lvecs = spline-&gt;Segments() * 2;\n\
     for(v = 0; v &lt; lvecs; ++v)\n\
     {\n\
      // Gives this vec&apos;s location in the mesh!\n\
      capline.SetVec(v, vec++);\n\
     }\n\
     baseVec += lvecs * (levels + 1) + spline-&gt;KnotCount() * levels * 2;\n\
   }\n\
}</code></pre><p>There are vectors generated inside the patch cap. In the case of a SurfRev for example, the end cap might be rotated, or scaled in some manner. A matrix is required so the capper knows how to orient the vectors into the correct location.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> // Create a work matrix for capping\n\
 Matrix3 mat = TransMatrix(offset1);</code></pre><p>The final step is to cap the patch mesh. This is done using a method of the capper named <code>CapPatchMesh()</code>. This method is passed the output patch mesh, the <code>PatchCapInfo</code>, a flag that indicates if the cap should be flipped, the smoothing group number for all the patches in the cap, and a pointer to the orientation matrix.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> capper.CapPatchMesh(pmesh, capInfo, TRUE, 16, &amp;mat);</code></pre><p>Once this is done the shape has been capped with a patch.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../shapes_and_splines.html\">Shapes and Splines</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";