var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"creating_a_spline\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Creating a Spline</h1></div><p>Below is a section of code from the Circle plug-in. This plug-in is derived from class <code>SimpleSpline</code>. This code demonstrates the key methods used in building a shape. The method <code>SimpleSpline::BuildShape()</code> is called to build the shape at the specified time and store the results in <code>ashape</code>.</p>\n\
<h2 id=\"buildshape-\">BuildShape()</h2><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">voidCircleObject::BuildShape(TimeValue t, BezierShape&amp; ashape)\n\
{\n\
   // Start the validity interval at forever and whittle it down.\n\
   ivalid = FOREVER;\n\
   floatradius;\n\
   pblock-&gt;GetValue(PB_RADIUS, t, radius, ivalid);\n\
   LimitValue( radius, MIN_RADIUS, MAX_RADIUS );\n\
   ashape.NewShape();</code></pre><p>The first thing to notice is the call to <code>NewShape()</code>. This ensures the shape is flushed out and emptied.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">   // Get parameters from SimpleSpline and place them in the BezierShape\n\
   intsteps;\n\
   BOOLoptimize,adaptive;\n\
   ipblock-&gt;GetValue(IPB_STEPS, t, steps, ivalid);\n\
   ipblock-&gt;GetValue(IPB_OPTIMIZE, t, optimize, ivalid);\n\
   ipblock-&gt;GetValue(IPB_ADAPTIVE, t, adaptive, ivalid);\n\
   ashape.steps = adaptive ? -1 : steps;\n\
   ashape.optimize = optimize;\n\
   MakeCircle(ashape,radius);</code></pre><p>Then this method calls <code>MakeCircle()</code>. See the code for this method below.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">   ashape.UpdateSels(); // Make sure it readies the selection set info</code></pre><p>After <code>MakeCircle()</code> has returned, another important call is made. This is <code>UpdateSels()</code>. This should be called when you are done adding all the polygons to the shape. This method updates the selection set information maintained by the shape. It is vital to call this before you are done.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">   ashape.InvalidateGeomCache();\n\
}</code></pre><p>Finally we clear any caches from the shape.</p>\n\
<h2 id=\"makecircle-\">MakeCircle()</h2><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">staticvoidMakeCircle(BezierShape&amp; ashape, float radius)\n\
{\n\
   floatvector = CIRCLE_VECTOR_LENGTH * radius;\n\
   Spline3D *spline = ashape.NewSpline();</code></pre><p>First create a new spline. This is done by calling the <code>NewSpline()</code> method on the shape. The shape adds a polygon to itself and returns a pointer to it.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">   for(int ix=0; ix&lt;4; ++ix)\n\
   {\n\
     float angle = 6.2831853f * (float)ix / 4.0f;\n\
     float sinfac = (float)sin(angle), cosfac = (float)cos(angle);\n\
     Point3 p(cosfac * radius, sinfac * radius, 0.0f);\n\
     Point3 rotvec = Point3(sinfac * vector, -cosfac * vector, 0.0f);\n\
     spline-&gt;AddKnot(SplineKnot(\n\
                KTYPE_BEZIER,\n\
                LTYPE_CURVE,\n\
                p,\n\
                p + rotvec,\n\
                p - rotvec ));\n\
   }</code></pre><p>Next points or knots are added to the spline by calling <code>AddKnot()</code>. This allows you to add different types of knots and line segments.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">   spline-&gt;SetClosed();</code></pre><p>After adding four knots, the <code>SetClosed()</code> method is called to make sure it&apos;s a closed circle.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">   spline-&gt;ComputeBezPoints();\n\
}</code></pre><p>Next, the <code>ComputeBezPoints()</code> call is made. The spline has a cached set of bezier points inside it. This needs to be called if you change the points of the spline. This methods updates the information internal to the spline.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../shapes_and_splines.html\">Shapes and Splines</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";