var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"local_transformation_matrix\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Local Transformation Matrix</h1></div><p>To retrieve the local transformation matrix of a node (the transformation of a node relative to its parent) you must perform some matrix arithmetic on the node&apos;s world space transformation matrix. This is because 3ds Max does not store the local transformation matrix.</p>\n\
<p>The transformation of a node relative to its parent may be some function of the node&apos;s parent&apos;s transform. For example, a transform controller takes the parent&apos;s transformation matrix and modifies it. When a node evaluates itself it takes the parent&apos;s transformation matrix and passes it as an argument when it calls <code>GetValue()</code> on its transform controller. The task of a transform controller in its implementation of <code>GetValue()</code> is to modify this matrix. It applies its transformation to the parent transformation passed in.</p>\n\
<p>In some cases this modification may be just a simple pre-multiplication by what is considered the local transformation matrix. But it may be some other more-complicated process. For example, if rotation or scale inheritance is turned off, the transform controller takes the parent&apos;s matrix and perhaps removes rotation from it, or removes scaling from it, and then applies itself. As another example, it may use the parent&apos;s position as a function to derive rotation -- as in a Look At controller. Thus, what is considered the local transformation is a function of the transform controller and is not stored by 3ds Max.</p>\n\
<p>3ds Max stores the node&apos;s world space transformation matrix. This matrix includes the parent&apos;s transformation. To understand how the local transformation can be extracted from the world space transformation consider the following:</p>\n\
<p>Any transformation is made up by starting with the node, and then multiply it by its parent, and then by its parent, and so on, all the way through the ancestors. This is shown below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">NodeWorldTM = NodeLocalTM * ParentLocalTM * ParentLocalTM * ParentLocalTM, etc.</code></pre><p>The parents world transformation is equal to the product of all its ancestors, so:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">ParentWorldTM = ParentLocalTM * ParentsLocalTM * ParentsLocalTM, etc.</code></pre><p>The nodes world transform can then be expressed as:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">NodeWorldTM = NodesLocalTM * ParentWorldTM.</code></pre><p>If we multiply both sides of this equation by the Inverse of the ParentsWorldTM and simplify we get:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">NodeLocalTM = NodeWorldTM * Inverse(ParentWorldTM)</code></pre><p>So, to retrieve the local transformation you must get the parent&apos;s transform and multiply the node&apos;s transform by the inverse of the parent. The way this is computed using the methods of 3ds Max is shown in the following code fragment. This example assumes <code>node</code> is the <code>INode</code>* that we want the local transformation matrix from, and the result will get stored in <code>localTM</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">...\n\
Matrix3 nodeTM = node-&gt;GetNodeTM(0);\n\
INode* parent = node-&gt;GetParentNode();\n\
Matrix3 parentTM = parent-&gt;GetNodeTM(0);\n\
Matrix3 localTM = nodeTM*Inverse(parentTM);\n\
...\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../transformation_and_rotation.html\">Transformation and Rotation</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";