var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"matrix_fundamentals\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Matrix Fundamentals</h1></div><p>A matrix is a two-dimensional array of numbers. In 3ds Max, 4x3 matrices are used. For 3ds Max matrices, the first number is the number of rows (4) and the second number is the number of columns (3). Thus there are a total of 12 elements.</p>\n\
<p>An instance of <code>Matrix3</code> has a private data member that contains the values:</p>\n\
<p><code>float</code> m[4][3];</p>\n\
<p>The layout of these elements is shown in the following diagram:</p>\n\
<p><img src=\"./images/GUID-D9639BE8-F1CB-4FC7-80BD-D476987C1DF3-low.gif\" alt></p>\n\
<p>The following matrix is called the &apos;Identity Matrix&apos;. This special matrix, when multiplied by a vector or another matrix, yields the original matrix.</p>\n\
<p><img src=\"./images/GUID-BC2F195E-7755-45BF-9295-FE890EB4FB33-low.gif\" alt></p>\n\
<p>The 3ds Max API provides several ways to create identity matrices. If a value of 1 is passed to the constructor of a <code>Matrix3</code> it is initialized to the identity. For example</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Matrix3 tmat(1); // Identity matrix</code></pre><p>You may also use a method of <code>Matrix3</code> to reset a matrix to the identity, for example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">tmat.IdentityMatrix(); // Reset to the identity matrix</code></pre><p>You can verify if a matrix is equal to the identity by using:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">BOOL isIdent = tmat.IsIdentity();</code></pre><p>The following sections discuss the ways matrices can be used in transformations.</p>\n\
<h2 id=\"multiplying-a-vector-by-a-matrix\">Multiplying a Vector by a Matrix</h2><p>One thing that can be done is to multiply a vector (<code>Point3</code>) by a matrix (<code>Matrix3</code>). This results in a new vector as a result. This is often used to transform vectors from one coordinate space to another. There is a global function in the SDK that does this.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Point3 VectorTransform(constMatrix3&amp; M, const Point3&amp; V);</code></pre><p>Transform the vector (<code>Point3</code>) with the specified matrix.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> In 3ds Max, all vectors are assumed to be row vectors. Under this assumption, multiplication of a vector with a matrix using the * operator can be written either way (Matrix*Vector or Vector*Matrix), for ease of use, and the result is the same -- the (row) vector transformed by the matrix.</div>\n\
<h2 id=\"multiplying-two-matrices\">Multiplying Two Matrices</h2><p>It is often also very useful to compute the multiplication of a matrix by another matrix. This produces a matrix as the result. Matrix multiplication is associative but not commutative. That is, AB is not the same as BA, but (AB)C is the same as A(BC). This property is very useful since separate matrix transformations can be concatenated together and then all applied at once. 3ds Max provides several methods to perform matrix multiplication; each are operators of the <code>Matrix3</code> class.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Matrix3&amp; operator*=(const Matrix3&amp; M);</code></pre><p>Multiplies this Matrix3 by the specified Matrix3.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Matrix3 operator*(const Matrix3&amp;) const;</code></pre><p>Performs matrix multiplication.</p>\n\
<h2 id=\"transformation-matrix-demonstration-program\">Transformation Matrix Demonstration Program</h2><p>There is a program available in  <em>\\MAXSDK\\HOWTO\\UTILITIES\\TRANSFORMMATRIXVIZ\\TMATTEST.CPP</em>  that shows visually how matrices are constructed for move/rotate/scale transformations. This program also shows the results of some of the <code>INode</code> methods that return transformation matrices (for example, <code>GetNodeTM()</code>, <code>GetObjectTM()</code>, etc.). Developers can load this project into VC++ and build it. Then copy  <em>TMATTEST.DLU</em>  into the <code>STDPLUGS</code> directory where 3ds Max can load it. To run it select <em>TMatTest</em> from the <em>HowTo</em> section of the Utilities menu. The program operates on the first object in the selection set. Create a node in the scene (for instance a Box) and experiment with animating the node, binding it to space warps, etc., and viewing the resulting matrices returned from the INode methods. You can also build transformations from scratch and apply them to the first selected object.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../matrix_representations_of_3d_tra.html\">Matrix Representations of 3D Transformations</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";