var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"object_offset_transformation\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Object Offset Transformation</h1></div><p>The object offset transformation provides an offset of the geometry of an object from its node. This section describes the object offset transformation and its function in 3ds Max.</p>\n\
<p>One can see a node&apos;s pivot point graphically represented in 3ds Max by selecting a node, going to the Hierarchy branch of the command panel, selecting the &apos;Pivot&apos; button, and choosing either the &apos;Affect Pivot Only&apos; or &apos;Affect Object Only&apos; button. This displays a large axis tripod that shows the location and orientation of the node&apos;s pivot point: the Node transformation matrix (<code>INode::GetNodeTM()</code>). By choosing one of these buttons, and using the Move / Rotate / Scale tool bar controls, a user can manipulate the position of the geometry of the object independent of the pivot point. Or they may manipulate the pivot point independent of the geometry of the object position. One can think of the geometry of the object as the &apos;mesh&apos; of the object.</p>\n\
<p>The way the user is able to independently manipulate the pivot and the object is managed internally using the object-offset transformation. The object-offset transformation affects how the geometry of the object is related to the node. The object-offset transformation transforms the geometry of the object itself some amount from the node.</p>\n\
<p>To understand how the object-offset is used, consider the following example from the 3ds Max user interface. In the Hierarchy branch under &apos;Pivot&apos;, when the user has chosen the &apos;Affect Object Only&apos; button they are free to move the geometry of the object around independent of the node. The pivot point does not move -- only how the geometry of the object is oriented relative to the pivot. What is happening internally is the object-offset transformation is being manipulated. This transformation is simply an additional offset that may be applied to the geometry of the object that is independent of the node. The object-offset transformation is not inherited by any child nodes.</p>\n\
<p>As another example consider the use of the &apos;Affect Pivot Only&apos; button. This mode lets the user move the pivot without affecting the position of the geometry of the object. When the system allows the user to move the pivot point, what is actually happening is the node&apos;s transformation is being altered (to re-orient the pivot point), then the object-offset transformation is adjusted to counter the node transformation. This lets the geometry of the object stay in the same place while the pivot point moves around. So again, when the user is moving the pivot point, 3ds Max is actually adjusting the node transformation matrix and counter-adjusting the object-offset transformation.</p>\n\
<h2 id=\"construction-of-the-object-offset-transformation\">Construction of the Object Offset Transformation</h2><p>The object offset transformation consists of separate position, rotation and scale transformations. Like the node transformation, these are applied by pre-multiplying position, then rotation, then scale. Thus the object offset transformation is:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Object Offset Transformation = OffsetScale * OffsetRotation * OffsetPosition</code></pre><p>Unlike the node transformation matrix, the object offset transformation is not inherited by children of the node.</p>\n\
<h2 id=\"methods\">Methods</h2><p>The <code>INode</code> class has methods to get and set individual transformations of the object offset. These are:</p>\n\
<ul>\n\
<li><code>void</code> <code>INode::SetObjOffsetPos</code>(<code>Point3</code> p) - Sets the position portion of the object offset from the node.</li>\n\
<li><code>Point3</code> <code>INode::GetObjOffsetPos</code>() - Returns the position portion of the object-offset from the node as a Point3.</li>\n\
<li><code>void</code> <code>INode::SetObjOffsetRot</code>(<code>Quat</code> q) - Sets the rotation portion of the object offset from the node.</li>\n\
<li><code>Quat</code> <code>INode::GetObjOffsetRot</code>() - Returns the rotation portion of the object-offset from the node.</li>\n\
<li><code>void</code> <code>INode::SetObjOffsetScale</code>(<code>ScaleValue</code> sv) - Sets the scale portion of the object offset from the node.</li>\n\
<li><code>ScaleValue</code> <code>INode::GetObjOffsetScale</code>() - Returns the scale portion of the object-offset from the node.</li>\n\
</ul>\n\
<p>Constructing an offset transformation matrix for a node can be done with the following code:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Matrix3 tm(1);\n\
Point3 pos = node-&gt;GetObjOffsetPos();\n\
tm.PreTranslate(pos);\n\
Quat quat = node-&gt;GetObjOffsetRot();\n\
PreRotateMatrix(tm, quat);\n\
ScaleValue scaleValue = node-&gt;GetObjOffsetScale();\n\
ApplyScaling(tm, scaleValue);</code></pre><p>The <code>INode</code> class has a set of methods that allow a developer to transform the node or geometry in the same way as the hierarchy &apos;Affect Pivot Only&apos; and &apos;Affect Object Only&apos; modes do. The entire node and object may be transformed, just the pivot, or just the geometry of the object. These methods are:</p>\n\
<ul>\n\
<li><code>void</code> <code>INode::Move</code>() - This method may be called to move the node about the specified axis system. Either the pivot point, or the geometry of the object, or both the pivot and the object may be transformed. Optionally, any children of the node can be counter transformed so they don&apos;t move.</li>\n\
<li><code>void</code> <code>INode::Rotate</code>() - This method may be called to rotate the node about the specified axis system. Either the pivot point, or the geometry of the object, or both the pivot and the object may be transformed. Optionally, any children of the node can be counter transformed so they don&apos;t rotate. There is also a Quat version of this method.</li>\n\
<li><code>void</code> <code>INode::Scale</code>() - This method may be called to scale the node about the specified axis system. Either the pivot point, or the geometry of the object, or both the pivot and the object may be transformed. Optionally, any children of the node can be counter transformed so they don&apos;t scale.</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../transformation_and_rotation.html\">Transformation and Rotation</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";