var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"using_the_api_to_create_and_modi\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Using the API to Create and Modify 3ds Max NURBS Models</h1></div><p>As mentioned above, 3ds Max itself uses a different internal representation of NURBS objects than the API does. The API is simply a means to access these internal 3ds Max NURBS objects and allow them to be manipulated.</p>\n\
<p>The object used in communication between the internal 3ds Max NURBS objects and the classes in the SDK available to work with these objects is the <code>NURBSSet</code>. There are also two global functions used in this communication. These are <code>CreateNURBSObject()</code> and <code>GetNURBSSet()</code>. How these are used is described below:</p>\n\
<h2 id=\"creating-new-nurbs-objects\">Creating New NURBS Objects</h2><p>The API can be used to create new NURBS objects. This is done by adding the objects (points, curves and surfaces) to an item called a NURBS &apos;set&apos; and calling the global function <code>CreateNURBSObject()</code>. The <code>NURBSSet</code> is a container for the objects. Then the global function <code>CreateNURBSObject()</code> makes an internal 3ds Max NURBS object out of the set.</p>\n\
<p>As an example, consider the following code fragment that creates a <code>NURBSCVCurve</code> object (with <code>NURBSControlVertex</code> sub-objects) and puts it in the scene:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Create an empty NURBSSet object\n\
NURBSSet nset;\n\
\n\
// Allocate a new NURBSCVCurve and set the knots and CVs\n\
NURBSCVCurve *c = new NURBSCVCurve();\n\
c-&gt;SetName(_T(&quot;CV Curve&quot;));\n\
c-&gt;SetNumCVs(4);\n\
c-&gt;SetOrder(4);\n\
c-&gt;SetNumKnots(8);\n\
for (int k = 0; k &lt; 4; k++)\n\
{\n\
   c-&gt;SetKnot(k, 0.0);\n\
   c-&gt;SetKnot(k+4, 1.0);\n\
}\n\
\n\
NURBSControlVertex cv;\n\
cv.SetPosition(0, Point3(0, 0, 50));\n\
c-&gt;SetCV(0, cv);\n\
cv.SetPosition(0, Point3(-100, 0, 50));\n\
c-&gt;SetCV(1, cv);\n\
cv.SetPosition(0, Point3(-100, 100, 50));\n\
c-&gt;SetCV(2, cv);\n\
cv.SetPosition(0, Point3(0, 100, 50));\n\
c-&gt;SetCV(3, cv);\n\
\n\
// Add the NURBSCVCurve object to the set\n\
nset.AppendObject(c);\n\
\n\
// Create the NURBS object from the NURBSSet\n\
Matrix3 mat(1);\n\
Object *obj = CreateNURBSObject(ip, &amp;nset, mat);\n\
\n\
// Create the node in the scene and point it at the NURBS object\n\
INode *node = ip-&gt;CreateObjectNode(obj);\n\
node-&gt;SetName(_T(&quot;NURBS Set 1&quot;));</code></pre><p>The NURBS object created (<code>obj</code>) will have the Class_ID: <code>EDITABLE_SURF_CLASS_ID</code>.</p>\n\
<h2 id=\"modifying-existing-nurbs-objects\">Modifying Existing NURBS Objects</h2><p>The API can also be used to modify the properties of existing NURBS objects in the scene. To gain access to an internal 3ds Max NURBS object you must have the data copied into a <code>NURBSSet</code>. This is done using the global function <code>GetNURBSSet()</code>. You can then use the methods of the <code>NURBSSet</code> to access the objects. After checking the type of the object you can cast it to the appropriate class and use that class&apos;s methods to alter the item.</p>\n\
<p>For example, the following sample code attempts to grab the first NURBS sub-object from the first node in the current selection set. It then checks if it&apos;s a blend surface and if so adjust one of the tension parameters.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Get the first selected node\n\
INode* node = ip-&gt;GetSelNode(0);\n\
if (!node) return;\n\
\n\
// Get the object reference of the node\n\
Object* obj = node-&gt;GetObjectRef();\n\
\n\
// Make sure it&apos;s a NURBS object\n\
if (obj-&gt;ClassID() == EDITABLE_SURF_CLASS_ID)\n\
{\n\
   NURBSSet getSet;\n\
   BOOL okay = GetNURBSSet(obj, ip-&gt;GetTime(), getSet, TRUE);\n\
   if (okay)\n\
   {\n\
      NURBSObject *nObj;\n\
     nObj = getSet.GetNURBSObject(0), ip-&gt;GetTime());\n\
     if (nObj-&gt;GetType() == kNBlendSurface)\n\
     {\n\
      // It&apos;s a blend, adjust the tension\n\
      NURBSBlendSurface *bSurf = ( NURBSBlendSurface *)nObj;\n\
      bSurf-&gt;SetTension(ip-&gt;GetTime(), 0, 2.0);\n\
     }\n\
   }\n\
}</code></pre><p>There are other global functions that may be used to modify NURBS objects. These functions are documented at the end of this topic. For instance, developers can use the function <code>BreakSurface()</code> to take an existing NURBS surface and break it into two separate surfaces.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../working_with_nurbs.html\">Working with NURBS</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";