var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"describing_a_parameter_block\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Describing a Parameter Block</h1></div><p>A parameter block is created with a parameter block descriptor as an argument. There is exactly one parameter block descriptor for every parameter block. A programmer must create a singleton parameter block descriptor that remains valid throughout the life of the DLL. This can be done by declaring a static instance in the DLL. The <code>ParamBlockDesc2</code> constructor takes a variable amount of arguments that describe the parameter block. Alternatively you can construct the parameter block descriptor incrementally using methods such <code>ParamBlockDesc2::AddParam()</code> and <code>ParamBlockDesc2::ParamOption()</code>.</p>\n\
<p>Of the arguments that need to be listed, the first five arguments are required. After that, some arguments are included only if you supplied some special flags in the 5th argument (<code>flags</code>). The parameter flags are documented in the topic <a href=\"parameter_block_flags.html\"\">Parameter Block Flags</a>.</p>\n\
<p>Afterwards, for each parameter for which you want a user interface control, you will supply mandatory and optional arguments. At the end of each parameter description section you will supply the special argument tag <code>p_end</code>. At the end of the entire constructor you must also supply the tag <code>p_end</code>.</p>\n\
<h2 id=\"parameter-block-descriptor-2-constructor\">Parameter Block Descriptor 2 Constructor</h2><pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">ParamBlockDesc2\n\
(\n\
   // Required specifications\n\
   BlockID     ID,\n\
   TCHAR*      int_name,\n\
   int        local_name,\n\
   ClassDesc2* cd,\n\
   BYTE       flags,\n\
\n\
   // ONLY IF P_VERSION is specified as a flag\n\
   int  version_number,\n\
\n\
   // ONLY IF P_AUTO_CONSTRUCT is specified as a flag\n\
   // This is the same reference number that specifies the\n\
   // parameter block used in GetReference() and SetReference().\n\
   int  reference_number,\n\
\n\
   // ONLY IF P_AUTO_UI is specified above\n\
   // Automatic UI parameter map specifications\n\
   int  dialog_template_ID,\n\
   int  dialog_title_res_ID,\n\
   int  flag_mask,\n\
   int  rollup_flags,\n\
    ParamMap2UserDlgProc* proc,\n\
\n\
   // ONLY IF P_INCLUDE_PARAMS or P_USE_PARAMS is specified as a flag\n\
   // Address of source IParamblockDesc to use\n\
    ParamBlockDesc2* pSourceDesc,\n\
\n\
     // OPTIONAL\n\
     // FOR EACH variable control create a parameter specification:\n\
     // There is one of these parameter specifications for each control.\n\
     ParamID   id,\n\
     TCHAR*    internal_name,\n\
     ParamType type,\n\
     // Optional. Use this argument ONLY IF you use a Tab&lt;&gt; type\n\
     [int      table_size,\n\
     int       flags,\n\
     int      local_name_res_ID,\n\
\n\
       // FOR EACH parameter specification, specify zero or more parameter tags.        \n\
        // and optional parameter information\n\
       Parameter_Tag\n\
\n\
     p_end, //End for each parameter specification\n\
   p_end //End for all the variable arguments\n\
);</code></pre><p>Below is a sample parameter block:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static  ParamBlockDesc2 widget_param_blk (\n\
   //Required arguments ----------------------------\n\
   widget_params, _T(&quot;params&quot;), 0, &amp;WidgetDesc,\n\
   //flags\n\
   P_AUTO_CONSTRUCT + P_AUTO_UI,\n\
\n\
   //Dependent arguments ---------------------------\n\
   //required because P_AUTO_CONSTRUCT was flagged\n\
   //This declares the reference number of the Parameter Block.\n\
   PBLOCK_REF,\n\
   //required because P_AUTO_UI was flagged.\n\
   //This is the Rollout description\n\
   IDD_PANEL, IDS_PARAMS, 0, 0, NULL,\n\
\n\
   //Parameter Specifications ----------------------\n\
   // For each control create a parameter:\n\
   widget_size,      _T(&quot;size&quot;),      TYPE_FLOAT,    P_ANIMATABLE,    IDS_SPIN,\n\
     //3 optional tags\n\
     p_default,      1.0f,\n\
     p_range,      0.0f,1000.0f,\n\
     p_ui,        TYPE_SPINNER,     EDITTYPE_FLOAT, IDC_SIZE_EDIT,   IDC_SIZE_SPIN, 0.50f,\n\
     p_end,\n\
   widget_left,     _T(&quot;Left&quot;),      TYPE_FLOAT,    P_ANIMATABLE,    IDS_SPIN,\n\
     //3 optional tags  \n\
     p_default,      0.0f,\n\
     p_range,      0.0f,100.0f,\n\
     p_ui,        TYPE_SPINNER,     EDITTYPE_UNIVERSE, IDC_LEFT_EDIT,   IDC_LEFT_SPIN, 0.50f,\n\
     p_end,\n\
   widget_right,     _T(&quot;Right&quot;),      TYPE_FLOAT,    P_ANIMATABLE,    IDS_SPIN,\n\
     //3 optional tags  \n\
     p_default,      0.0f,\n\
     p_range,      0.0f,100.0f,\n\
     p_ui,        TYPE_SPINNER,     EDITTYPE_UNIVERSE, IDC_RIGHT_EDIT,   IDC_RIGHT_SPIN, 0.50f,\n\
     p_end,\n\
   p_end\n\
);\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../creating.html\">Creating a Parameter Block</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";