var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"getting_and_setting_pb_values\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Getting and Setting Parameter Block Values</h1></div><p>Parameter block values are get and set from an <code>IParamBlock2</code> object using a <code>ParamID</code> that identifies the parameter value. This must be the same parameter identifier that is supplied as part of the <a href=\"creating/parameter_specification_argument.html\">Parameter Specification Arguments</a>.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> It is important not to confuse <code>IParamBlock2::GetValue()</code> and <code>IParamBlock2::SetValue()</code> with the parameter block 1 system for setting and retrieving values (<code>IParamBlock::GetValue()</code> and <code>IParamBlock::SetValue()</code>) which uses integer indexes instead of <code>ParamID</code>.</div>\n\
<h2 id=\"getting-parameter-values\">Getting Parameter Values</h2><p>There are two sets of functions in class <code>IParamBlock2</code> for retrieving values from the parameter block.</p>\n\
<ul>\n\
<li><em>BOOL GetValue functions:</em> There are a number of overloaded <code>IParamBlock2::GetValue</code> functions used for retrieving various data-types in the parameter block. These functions take as one of their arguments an address of a value with the appropriate data type that you want to fill, and return a boolean value indicating success or failure.</li>\n\
<li><em>&lt;type&gt; Get&lt;type&gt; functions:</em> There are overloaded functions for each type of value to retrieve (<code>int</code>, <code>float</code>, <code>Point3</code>, etc...). For example: <code>IParamBlock2::GetColor</code>, <code>IParamBlock2::GetFloat</code>, or <code>IParamBlock2::GetPoint3</code>.</li>\n\
</ul>\n\
<p>When accessing a value from the parameter block an <code>Interval</code> reference must be supplied. This method is frequently used by developers to &apos;whittle&apos; down an interval. When a parameter of a parameter block is animated, for any given time there is a interval over which the parameter is constant. If the parameter is constantly changing the interval is instantaneous. If the parameter does not change for a certain period the interval will be longer. If the parameter never changes the interval will be <code>FOREVER</code>. By passing an interval to the <code>GetValue()</code> method you ask the parameter block to &apos;intersect&apos; the interval passed in with the interval of the parameter. Intersecting two intervals means returning a new interval whose start value is the greater of the two, and whose end value is smaller of the two. In this way, the resulting interval represents a combined period of constancy for the two intervals.</p>\n\
<p>This technique is used frequently to compute a validity interval for an object. The developer starts an interval off as FOREVER, then intersects this interval with each of its animated parameters (by calling <code>GetValue()</code>). <code>GetValue()</code> &apos;whittles&apos; down the interval with each call. When all the parameters have been intersected the result is the overall validity interval of an object at a specific time. For more information see the section on <a href=\"./../../animation_time_and_controllers/intervals.html\">Intervals</a>.</p>\n\
<p>The following code example is taken from the Bend modified example at <code>maxsdk</code>\\howto\\modifiers\\bend\\bendmod.cpp and demonstrates this technique:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Interval BendMod::GetValidity(TimeValue t)\n\
{\n\
  float f = 0.0f;\n\
  // Start our interval at forever...\n\
  Interval valid = FOREVER;\n\
  // Intersect each parameters interval to narrow it down.\n\
  pblock2-&gt;GetValue(bend_angle,t,f,valid);\n\
  pblock2-&gt;GetValue(bend_dir,t,f,valid);\n\
  pblock2-&gt;GetValue(bend_from,t,f,valid);\n\
  pblock2-&gt;GetValue(bend_to,t,f,valid);\n\
  // Return the final validity interval.\n\
  return valid;\n\
}</code></pre><p>The <code>IParamBlock2::GetValue</code> methods might be called in the following scenarios:</p>\n\
<ul>\n\
<li>Modifying an object via a modifier based on user supplied parameter values.</li>\n\
<li>Computing the validity interval of your Modifier&apos;s state (see <code>SimpleMod::GetValidity()</code>).</li>\n\
<li>Computing the mesh and world bounding box of an object.</li>\n\
</ul>\n\
<h2 id=\"setting-parameter-values\">Setting Parameter Values</h2><p>Functions for setting values in a parameter blocks.</p>\n\
<ul>\n\
<li><em>BOOL SetValue functions:</em> There are a number of overloaded <code>IParamBlock2::SetValue</code> functions used for setting the parameter value in the parameter block. These functions return a boolean value indicating success or failure.</li>\n\
</ul>\n\
<p>Most of the time parameter values are modified automatically through the associated UI element on the roll-out window, or by an animation controller.</p>\n\
<p>In some circumstances the parameter block value must be set manually. For example during interactive creation and modification of the object via the viewport. See <code>CreateMouseCallback::proc()</code> for an example of this.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../parameter_blocks.html\">Parameter Blocks</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";