var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"tutorial1\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Tutorial 1 - Converting the Bend Modifier</h1></div><p>The aim of this tutorial is to explain the steps required in converting an existing plug-in from ParamBlock to ParamBlock2.  To aid in this the Bend modifier found in the samples under &quot;maxsdk\\howto&quot; will be converted.  At each point the key issues will be addressed.  The final converted plug-in is contained in the accompanying zip file.  The source code should be read in conjunction with this document.  </p>\n\
<p>The tutorial is a descriptive step by step guide on how to convert an existing plug-in to support Paramblock2 interface.  However the new system has many methods and variables so it is advisable to cross-reference this document with the 3ds Max SDK and the accompanying source code.</p>\n\
<p><em>Note</em> that at the time the Bend modifier was converted from Paramblock to Paramblock2, the conversion did not support Save To Previous.  All plug-ins should always properly support Save to Previous.  For an example of doing so, see method <code>bool BoxObject::SpecifySaveReferences(ReferenceSaveManager&amp; referenceSaveManager)</code> in Tutorial 2 - Converting the Box Object.</p>\n\
<h2 id=\"step-1-project-settings\">Step 1 - Project Settings</h2><p>To use the new methods, the following need to be added to the project:</p>\n\
<ul>\n\
<li>Include File - iparamm2.h</li>\n\
<li>Library file - paramblk2.lib</li>\n\
</ul>\n\
<h2 id=\"step-2-convert-simplemod\">Step 2 - Convert SimpleMod</h2><p>The bend Modifier is sub classed from <code>SimpleMod</code>.   However, there is a new class that adds support for Paramblock2 called SimpleMod2.  This has a public data member, <code>IParamBlock2 *pblock2</code>, used instead of the <code>IParamBlock *pblock</code> provided by <code>SimpleMod</code>.  It also provides implementations of <code>ReferenceMaker::GetReference()</code> and <code>SetReference()</code> which get and set the pblock2 pointer.</p>\n\
<p>Original interfaces that are now redundant and can be deleted are as follows:</p>\n\
<ul>\n\
<li><code>ParamArray</code></li>\n\
<li><code>ParamMaps</code></li>\n\
<li><code>GetParamName()</code></li>\n\
<li><code>GetParamDim()</code></li>\n\
</ul>\n\
<p>Also, the notification messages <code>REFMSG_GET_PARAM_NAME</code> and <code>REFMSG_GET_PARAM_DIM</code> are no longer required\n\
There are three additions to be made and these add direct access to the Parameter blocks maintained by the plug-in.   These methods are from Class Animatable</p>\n\
<ul>\n\
<li><code>NumParamBlocks()</code></li>\n\
<li><code>IParamBlock2* GetParamBlock(int i)</code></li>\n\
<li><code>IParamBlock2* GetParamBlockByID(BlockID id)</code></li>\n\
</ul>\n\
<p>The complete definition of BendMod is as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">class BendMod : public SimpleMod2 {\n\
    public:\n\
\n\
        BendMod();\n\
\n\
        // --- Interhited virtual methods of Animatable\n\
        void DeleteThis() { delete this; }\n\
        void GetClassName(TSTR&amp; s) { s= GetString(IDS_RB_BENDMOD); }  \n\
        virtual Class_ID ClassID() { return BEND_CID;}\n\
\n\
        void BeginEditParams( IObjParam  *ip, ULONG flags,Animatable *prev);\n\
        void EndEditParams( IObjParam *ip,ULONG flags,Animatable *next);\n\
\n\
        // Add Direct Paramblock2 Support\n\
\n\
        int  NumParamBlocks() { return 1; }\n\
        IParamBlock2* GetParamBlock(int i) { return pblock2; }\n\
        IParamBlock2* GetParamBlockByID(BlockID id) { return (pblock2-&gt;ID() == id) ? pblock2 : NULL; }\n\
\n\
        // --- Interhited virtual methods of ReferenceMaker\n\
        IOResult Load(ILoad* iload);\n\
\n\
        // --- Interhited virtual methods of ReferenceTarget\n\
        RefTargetHandle Clone(RemapDir&amp; remap = NoRemap());\n\
\n\
        // --- Interhited virtual methods of BaseObject\n\
        TCHAR *GetObjectName() { return GetString(IDS_RB_BEND2);}\n\
\n\
        // --- Interhited virtual methods of SimpleMod\n\
        Deformer&amp; GetDeformer(TimeValue t,ModContext &amp;mc,Matrix3&amp; mat,Matrix3&amp; invmat);        \n\
        Interval GetValidity(TimeValue t);\n\
\n\
        BOOL GetModLimits(TimeValue t,float &amp;zmin, float &amp;zmax, int &amp;axis);\n\
        void InvalidateUI();\n\
};</code></pre>\n\
<h2 id=\"step-3-build-the-code-paramblockdesc2-code-\">Step 3 - Build the <code>ParamBlockDesc2</code></h2><p>The original modifier created its interface by using <code>ParamUIDesc</code> and <code>ParamBlockDescID</code>.  Now it is all controlled by <code>ParamBlockDesc2</code>.    The bend modifier has a relatively simple interface; for a more detailed look at ParamBlockDesc2 please refer to the PB2Utility sample.</p>\n\
<p>First of all a list of parameter Ids are defined for use with the block descriptor and also for use with GetValue and SetValue.  The order you declare these in is the order that they must be defined in the descriptor.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">enum { bend_params,};\n\
enum { bend_angle,\n\
    bend_dir,\n\
    bend_axis,\n\
    bend_fromto,\n\
    bend_to,\n\
    bend_from,\n\
};</code></pre>\n\
<p>The first line for the <code>ParamBlockDesc2</code> constructor defines the block:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">static ParamBlockDesc2 bend_param_blk ( bend_params, _T(&quot;Bend Parameters&quot;),  0, &amp;bendDesc, P_AUTO_CONSTRUCT + P_AUTO_UI, SIMPMOD_PBLOCKREF,\n\
    //rollout\n\
    IDD_BENDPARAM, IDS_RB_PARAMETERS, 0, 0, NULL,</code></pre>\n\
<p>The flag <code>P_AUTO_CONSTRUCT</code> tells the system that the reference will be created and handled by the owner.  If it is set then the reference number must be given after the flags, in this case <code>SIMPMOD_PBLOCKREF</code>.  The actual creation is achieved in the call to C`lassDesc2::MakeAutoParamBlocks().  This will create the Parameter map and also create the references using the ref number supplied.</p>\n\
<p>If <code>P_AUTO_UI</code> is used, then this tells the system that UI will be automatically created during calls to <code>ClassDesc2::BeginEditParams()</code>.  If this is specified then further dialog information is required including Dialog ID, Dialog Name, Append Closed Flag, and the Dialog proc to handle additional initialization (in this case it is set to NULL).\n\
Once the block has been defined, the parameters need to be defined.  The definitions for all the parameters are very similar so only one will be described here:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">bend_to,    _T(&quot;BendTo&quot;),    TYPE_FLOAT,    P_ANIMATABLE,    IDS_TO,\n\
    p_default,    0.0f,\n\
    p_range,     0.0f,     BIGFLOAT,\n\
    p_ui,        TYPE_SPINNER, EDITTYPE_UNIVERSE, IDC_BEND_TO, IDC_BEND_TOSPIN, SPIN_AUTOSCALE,\n\
    p_accessor,    &amp;bendPBAccessor,\n\
    end,</code></pre>\n\
<p>The first line contains required elements.  The first two elements are ID and internal name.  The next defines the parameter type; in this case it is a float value.  A list of flags follows in which <code>P_ANIMATIBLE</code> is defined which means that the parameter can be animated.  The last entry is the local name which will be used by Trackview, Schematic View and MAXScript. When assigning the internal and localized names, be careful that the names given are not the same as any of the MAXScript node level property names, such a &apos;Position&apos;.  To see the complete list of MAXScript node level property names type into the Scripting Listener: <code>print (getpropnames node)</code>. Following this is a list of details, which define how the parameter will work.  In particular is <code>P_UI</code>, which specifies what Dialog Resource will control the parameter.  <code>p_accessor</code> holds the pointer of a <code>PBAccessor</code> class which allows you to check the data of the parameter during calls to <code>GetValue()</code> and <code>SetValue()</code>.  More details on <code>PBAccessor</code> can be found in Step 4.</p>\n\
<p>When assigning the non-localized parameter name for each parameter, use the non-localized name in the explicit MAXClass descriptor for the class.  The p_ms_default values also come from the explicit MAXClass descriptor.</p>\n\
<ul>\n\
<li>The <code>p_ui</code> IDC_* values are controls in the dialog definition in the resources file, and come from corresponding pb1 code&apos;s ParamUIDesc instances.</li>\n\
<li>The <code>p_range</code> values come from corresponding pb1 code&apos;s ParamUIDesc instances.</li>\n\
<li>The <code>p_default</code> values typically come from the static initialization values from pb1 version of the class and its <code>ResetClassParams</code> method, if present.</li>\n\
</ul>\n\
<p>When specifying the type of the parameter, and the pb1 type is <code>TYPE_FLOAT</code>, look at pb1 code&apos;s <code>GetParameterDim</code> method to see what dimensioning is used. The following table shows the pb2 parameter type to use for each parameter dimension type.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>PB1 Parameter Dimension Type</th>\n\
<th>PB2 Parameter Type</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>defaultDim</code></td>\n\
<td><code>TYPE_FLOAT</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>stdWorldDim</code></td>\n\
<td><code>TYPE_WORLD</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>stdAngleDim</code></td>\n\
<td><code>TYPE_ANGLE</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>stdColorDim</code></td>\n\
<td><code>TYPE_FLOAT</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>stdColor255Dim</code></td>\n\
<td><code>TYPE_COLOR_CHANNEL</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>stdPercentDim</code></td>\n\
<td><code>TYPE_PCNT_FRAC</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>stdNormalizedDim</code></td>\n\
<td><code>TYPE_FLOAT</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>stdSegmentsDim</code></td>\n\
<td><code>TYPE_FLOAT</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>stdTimeDim</code></td>\n\
<td><code>TYPE_TIMEVALUE</code></td>\n\
</tr>\n\
</tbody></table>\n\
<p>The full ParamBlockDesc2 is listed below.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">static ParamBlockDesc2 bend_param_blk ( bend_params, _T(&quot;Bend Parameters&quot;),  0, &amp;bendDesc, P_AUTO_CONSTRUCT + P_AUTO_UI, SIMPMOD_PBLOCKREF,\n\
\n\
//Dlalog rollout\n\
    IDD_BENDPARAM, IDS_RB_PARAMETERS, 0, 0, NULL,\n\
\n\
// params\n\
\n\
    bend_angle,         _T(&quot;BendAngle&quot;),    TYPE_FLOAT,    P_ANIMATABLE,    IDS_ANGLE,\n\
        p_default,    0.0f,\n\
        p_range,     -BIGFLOAT, BIGFLOAT,\n\
        p_ui,        TYPE_SPINNER, EDITTYPE_FLOAT, IDC_ANGLE, IDC_ANGLESPINNER, 0.5f,\n\
        end,\n\
\n\
    bend_dir,        _T(&quot;BendDir&quot;),    TYPE_FLOAT,    P_ANIMATABLE,    IDS_DIR,\n\
        p_default,    0.0f,\n\
        p_range,     -BIGFLOAT, BIGFLOAT,\n\
        p_ui,        TYPE_SPINNER, EDITTYPE_FLOAT, IDC_DIR, IDC_DIRSPINNER, 0.5f,\n\
        end,\n\
\n\
\n\
    bend_axis,         _T(&quot;bendAxis&quot;), TYPE_BOOL, 0,    IDS_AXIS,     \n\
        p_default,     2,\n\
        p_ui,         TYPE_RADIO, 3,IDC_X,IDC_Y,IDC_Z,\n\
        p_vals,        0,1,2,\n\
        end,\n\
\n\
\n\
    bend_fromto,         _T(&quot;FromTo&quot;), TYPE_BOOL, 0,    IDS_FROMTO,     \n\
        p_default,     FALSE,\n\
        p_ui,         TYPE_SINGLECHEKBOX, IDC_BEND_AFFECTREGION,\n\
        end,\n\
\n\
\n\
    bend_to,            _T(&quot;BendTo&quot;),    TYPE_FLOAT,    P_ANIMATABLE,    IDS_TO,\n\
        p_default,    0.0f,\n\
        p_range,     0.0f, BIGFLOAT,\n\
        p_ui,        TYPE_SPINNER, EDITTYPE_UNIVERSE, IDC_BEND_TO, IDC_BEND_TOSPIN, SPIN_AUTOSCALE,\n\
        p_accessor,    &amp;bendPBAccessor,\n\
        end,\n\
\n\
    bend_from,        _T(&quot;BendFrom&quot;),    TYPE_FLOAT,    P_ANIMATABLE,    IDS_FROM,\n\
        p_default,    0.0f,\n\
        p_range,     -BIGFLOAT, 0.0f,\n\
        p_ui,        TYPE_SPINNER, EDITTYPE_UNIVERSE, IDC_BEND_FROM, IDC_BEND_FROMSPIN, SPIN_AUTOSCALE,\n\
        p_accessor,    &amp;bendPBAccessor,\n\
        end,\n\
\n\
\n\
    end\n\
    );</code></pre>\n\
<p>If the plug-in that you are converting maintains references that are handled in a non-trivial manner, then it may be easier to specify the parameter is of type P_OWNERS_REF.  This means that the owner of the paramblock will handle the references not the block itself.  When using the P_OWNERS_REF flag the p_ref specification needs to filled out with the ref number to use.  Using this system means that the references will be handled in the same way as in your original plug-in.</p>\n\
<h2 id=\"step-4-parameter-checking\">Step 4 - Parameter Checking</h2><p>In the original Bend Modifier a dialog procedure was used to control the values of the &quot;to/From&quot; parameters.  With the ParamBlock2 system, a new class called <code>PBAccessor</code> has been implemented to allow developers to have a SetValue and GetValue call back mechanism.  This allows continuous monitoring of the parameter change.  <code>PBAccessor</code> has two methods, <code>Get()</code> and <code>Set()</code>.  In the bend modifier case, <code>Set()</code> has been used to check the values of  &apos;bend_from&apos; and &apos;bend_to&apos;.</p>\n\
<p><code>PBAccessor::Set()</code> is passed a PB2Value Structure which holds the data being changed and an ID of the parameter changing.  The following PBAccessor changes the value of bend_from and bend_to depending on a comparison to each other.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">class bendPBAccessor : public PBAccessor\n\
{\n\
public:\n\
    void Set(PB2Value&amp; v, ReferenceMaker* owner, ParamID id, int tabIndex, TimeValue t)    // set from v\n\
    {\n\
        BendMod* u = (BendMod*)owner;\n\
        IParamMap2* pmap = u-&gt;pblock2-&gt;GetMap();\n\
        float from, to;\n\
\n\
        switch(id)\n\
        {\n\
            case bend_from:\n\
                u-&gt;pblock2-&gt;GetValue(bend_to,t,to,FOREVER);\n\
                from = v.f;\n\
                if (from &gt;to) {\n\
                    u-&gt;pblock2-&gt;SetValue(bend_to,t,from);\n\
                }\n\
                break;\n\
            case bend_to:\n\
                u-&gt;pblock2-&gt;GetValue(bend_from,t,from,FOREVER);\n\
                to = v.f;\n\
                if (from&gt;to) {\n\
                    u-&gt;pblock2-&gt;SetValue(bend_from,t,to);\n\
                }\n\
                break;\n\
        }\n\
    }\n\
};</code></pre>\n\
<h2 id=\"step-5-loading-old-data\">Step 5 - Loading Old Data</h2><p>Paramblock2 provides an automatic mechanism for loading in old paramap data.  To take advantage of this system the original ParamBlockDesc is used, but now using the newly created parameter Ids.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">static ParamBlockDescID descVer0[] = {\n\
    { TYPE_FLOAT, NULL, TRUE, bend_angle },\n\
    { TYPE_FLOAT, NULL, TRUE, bend_dir },\n\
    { TYPE_INT, NULL, FALSE, bend_axis } };\n\
\n\
// The current version\n\
static ParamBlockDescID descVer1[] = {\n\
    { TYPE_FLOAT, NULL, TRUE, bend_angle },\n\
    { TYPE_FLOAT, NULL, TRUE, bend_dir },\n\
    { TYPE_INT, NULL, FALSE, bend_axis },\n\
    { TYPE_INT, NULL, FALSE, bend_fromto },\n\
    { TYPE_FLOAT, NULL, TRUE, bend_to },\n\
    { TYPE_FLOAT, NULL, TRUE, bend_from } };\n\
\n\
static ParamVersionDesc versions[] = {\n\
    ParamVersionDesc(descVer0,3,0)\n\
    };</code></pre>\n\
<p>In the <code>BendMod::Load()</code> method a callback is registered which maps the incoming IDs to new enumerated IDs.   It is important to note that old Parameter maps still need to be loaded using the original mapping techniques before the new parmblock Ids are used.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">ParamBlock2PLCB* plcb = new ParamBlock2PLCB(versions, 1, &amp;bend_param_blk, this, SIMPMOD_PBLOCKREF);\n\
iload-&gt;RegisterPostLoadCallback(plcb);</code></pre>\n\
<h2 id=\"step-6-derive-from-classdesc2\">Step 6 - Derive from ClassDesc2</h2><p><code>ClassDesc2</code> is subclassed from <code>ClassDesc</code> and it is used for plug-ins using the Paramblock2 system.  It contains a table of ParamBlockDesc2s for all the parameter blocks used by the plug-in and a number of methods, including access to the block descriptors, auto user interface management, auto param block2 construction, and access to any automatically-maintained ParamMap2s.  To use this class, simply replace all reference of <code>ClassDesc</code> with <code>ClassDesc2</code>.</p>\n\
<h2 id=\"step-7-rename-pblock-to-pblock2\">Step 7 - Rename pblock to pblock2</h2><p>All <code>GetValue()</code> and  <code>SetValue()</code> calls used the original pblock pointer to IParamBlock.  This needs to be<code>changed so that it uses the</code>IPramBlock2<code>pointer</code>pblock2`.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../pb1_to_pb2_conversion.html\">Updating from ParamBlock1 to Paramblock2</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";