var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"using_indirect_references_instea\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Using Indirect References Instead of Pointers</h1></div><p>Using versions of the <em>3ds Max SDK</em> prior to Version 8, when a developer needed to make a regular reference that would have led to a circular reference, there was no choice but to use a pointer to a node instead. Pointers are not managed by the <em>Reference System</em> and thus not stored properly to <em>Max</em> files. This meant that entities created by plugins that used node pointers would cause 3ds Max to crash when the plug-in data was XREF&#39;d and the XREF loaded. The only work-around was to avoid XREFing stored data for those plugins.</p>\n\
<p>Developers can now use <a href=\"./../indirect_references.html\">Indirect References</a> in situations where Regular References would create circular references.</p>\n\
<p>This topic provides the steps for converting node pointers to <em>Indirect References</em> and testing the resultant code. Also see the topic <a href=\"indirect_references_and_object_x.html\"\">Indirect References and Object XRefs</a> for more information.</p>\n\
<h2 id=\"conversion-steps\">Conversion Steps</h2><h3 id=\"a-add-a-nodemonitor-to-the-plug-in-as-a-new-data-member\">A. Add a NodeMonitor to the plug-in as a new data member</h3><ol>\n\
<li>Declare a new ReferenceTarget* member in the plug-in class and in the plug-in&#39;s ctor create and instance of it by calling <code>Interface::CreateInstance(REF_TARGET_CLASS_ID, NODEMONITOR_CLASS_ID)</code></li>\n\
<li>Add two public methods to the plug-in class for setting and getting the node</li>\n\
<li>Replace usage of the node pointer with set\\get methods described above</li>\n\
</ol>\n\
<h3 id=\"b-modify-plugin-to-reference-the-nodemonitor-object\">B. Modify Plugin to Reference the NodeMonitor object</h3><ol>\n\
<li>Define new <code>refids</code> and modify the <code>NumRefs</code>, <code>SetReference</code>, <code>GetReference</code> methods to report and work with the new reference</li>\n\
</ol>\n\
<h3 id=\"c-modify-the-clone-method-to-also-clone-the-nodemonitor\">C. Modify the Clone method to also clone the NodeMonitor</h3><ol>\n\
<li>Clone the node monitor in the same way it clones other references the plug-in makes using the <code>RemapDir</code> parameter passed to it</li>\n\
</ol>\n\
<h3 id=\"d-modify-file-io-methods\">D. Modify File IO Methods</h3><ol>\n\
<li>Modify the Save method to:</li>\n\
</ol>\n\
<ul>\n\
<li>Save out a version chunk</li>\n\
<li>Not save the pointer to the 3ds Max file</li>\n\
</ul>\n\
<ol start=\"2\">\n\
<li>Modify the Load method to:</li>\n\
</ol>\n\
<ul>\n\
<li>Load legacy versions of the plug-in (the ones with no version chunk but with a pointer saved to the 3ds Max file), and register a postload-callback that would transform that pointer into an indirect reference. (The postload-callback would set the node in the <code>NodeMonitor</code>. <code>maxsdk</code>\\samples\\objects\\extendedobjects\\hose.cpp and <code>maxsdk</code>\\samples\\systems\\sunlight\\sunlight.cpp are examples of this procedure.</li>\n\
<li>Load the new version of the plug-in (the one with the version chunk and without the pointer saved to the 3ds Max file)</li>\n\
<li>Define a postload-callback class that would transform the raw node pointer read from a legacy 3ds Max file, into a Node Monitor and register this PLC from within the load method that loads legacy files</li>\n\
</ul>\n\
<h3 id=\"e-create-a-postload-callback-class\">E. Create a postload-callback class</h3><ol>\n\
<li>The NodeMonitor is set-up to point to the node pointer read from legacy files using a postload-callback. (The other place where the NodeMonitor is initialized with the node is usually a creation procedure.)</li>\n\
<li>In order to ensure that the reference held by the NodeMonitor is remapped correctly when this plug-in is XREF&#39;d, the postload-callback also needs to record a node-ref-remap item with the ILoad object passed into the postload-callback&#39;s procedure (<code>PostLoadCallback::proc</code>). Use <code>ILoad::RecordNodeRefRemap</code> to record remap information for Indirect References to nodes. For more information, see <code>ioapi</code>.h.</li>\n\
</ol>\n\
<h3 id=\"f-undo-redo\">F. Undo\\Redo</h3><ol>\n\
<li>A <code>NodeMonitor</code> has built-in undo/redo support. This means that, in general, your plug-in does not need to do anything special about undoing/redoing the monitoring of nodes.</li>\n\
</ol>\n\
<h2 id=\"unit-testing\">Unit Testing</h2><h3 id=\"a-ensure-the-new-version-of-the-plug-in-object-works\">A. Ensure the new version of the plug-in object works</h3><ol>\n\
<li>Create plug-in object (undo\\redo\\undo)</li>\n\
<li>Delete plug-in object (undo\\redo\\undo)</li>\n\
<li>Clone the plug-in object (undo\\redo\\undo)</li>\n\
<li>Persist the plug-in object (save\\load\\merge)</li>\n\
<li>Object and Scene XREF the plug-in object, reloading the xref, using nested XREFing, etc</li>\n\
<li>Plugin specific tests - white box tests based on the the code where the node pointer was being used</li>\n\
</ol>\n\
<h3 id=\"b-ensure-legacy-versions-of-the-plug-in-object-work\">B. Ensure legacy versions of the plug-in object work</h3><ol>\n\
<li>Load max files with the legacy version of the plug-in object, then delete, clone, render, work with the object</li>\n\
<li>Save the scene, then load it again</li>\n\
<li>Object\\Scene XREF 3ds Max files with the legacy version of the plug-in object</li>\n\
<li>Reload the XREF</li>\n\
</ol>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../indirect_references.html\">Indirect References</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";