var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"accessing_material_properties\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Accessing Material Properties</h1></div><p>This section covers how the various properties of a material may be retrieved from a node in the scene. These are properties such as color, mapping parameters, bitmaps used, etc.</p>\n\
<h2 id=\"3ds-max-multi-sub-object-material\">3ds Max Multi/Sub-Object Material</h2><p>Next, let&apos;s consider access to the properties a multi-material. If the <code>Class_ID</code> returned is <code>MULTI_CLASS_ID</code> then it&apos;s a m material. The multi-material is a plugin that uses the <code>MtlID</code> assigned to each face of a mesh as an index into the list of sub-materials. There are methods of the <code>Face</code> class <code>getMatID()</code> and <code>setMatID()</code> to access the MtlID for each face. Additionally, the <code>Mesh</code> class has methods</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">getFaceMtlIndex(inti)\n\
setFaceMtlIndex(inti, MtlID id).</code></pre><p>Note that the meaning of this material index assigned to the face is specific to the type of material. A third party developer could write a material that used the ID in an entirely different manner than 3ds Max does.</p>\n\
<p>For a multi-material, a developer can use the methods of <code>MtlBase</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">NumSubMtls()\n\
GetSubMtl(i)\n\
SetSubMtl(i, Mtl* m)</code></pre><p>to access the sub materials. Note that some 3ds Max primitive have pre-assigned material ids (such as the Box and the Hedra). Since the object is not really aware of the material, it can happen that the material ID on a face is higher than the total number of sub materials in a material. Developers should look at the number of sub materials on the material, and use a modulo function to bring the material ID of the face down to a legal value before trying to access it.</p>\n\
<p>The following code demonstrates access to material and texture properties. It shows how the ClassIDs are checked for the material and the texture map. It also demonstrates how to access materials properties using the <code>StdMat</code> class and the bitmap properties using the <code>BitmapTex</code> class.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Test of access to material and texture map properties of a selected node.\n\
// This code determines if the two sided flag is set for the material\n\
// and retrieves the U and V tiling options for the diffuse texture.\n\
// It checks if the material is a 3ds Max Standard material, and if the diffuse\n\
// texture map is a 3ds Max Bitmap texture.\n\
#include &quot;stdmat.h&quot;\n\
void Utility::Test (Interface *ip)\n\
{\n\
   BOOLtwo;\n\
   floatutile, vtile;\n\
   TSTR buf;\n\
\n\
   // Get the material and texture properties from the node.\n\
   if (ip-&gt;GetSelNodeCount() &lt; 1) return;\n\
   INode *node = ip-&gt;GetSelNode(0);\n\
\n\
   // Get the material from the node\n\
   Mtl *m = node-&gt;GetMtl();\n\
   if (!m) return; // No material assigned\n\
\n\
   // See if it&apos;s a Standard material\n\
   if (m-&gt;ClassID() == Class_ID(DMTL_CLASS_ID, 0))\n\
   {\n\
     // It is -- Access the two sided property of the material\n\
     StdMat* std = (StdMat *)m;\n\
     two = std-&gt;GetTwoSided();\n\
\n\
     // Access the Diffuse map and see if it&apos;s a Bitmap texture\n\
     Texmap *tmap = m-&gt;GetSubTexmap( ID_DI);\n\
     if (tmap-&gt;ClassID() == Class_ID(BMTEX_CLASS_ID, 0))\n\
     {\n\
      // It is -- Access the UV tiling settings at time 0.\n\
       BitmapTex *bmt = (BitmapTex*) tmap;\n\
       StdUVGen *uv = bmt-&gt;GetUVGen();\n\
      utile = uv-&gt;GetUScl(0);\n\
      vtile = uv-&gt;GetVScl(0);\n\
      buf.printf(_T(&quot;Two sided=%d, U Tile = %.1f, V Tile = %.1f&quot;),\n\
      two, utile, vtile);\n\
      MessageBox(ip-&gt;GetMAXHWnd(), buf, _T(&quot;Info...&quot;),\n\
      MB_ICONINFORMATION);\n\
     }\n\
   }\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../materials_textures_and_maps.html\">Materials, Textures and Maps</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";