var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"editing_material_and_texture_par\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Editing Material and Texture Parameters</h1></div><p>This section discusses the manner in which plug-in texture and materials manage their user interface in 3ds Max. This is only a concern of developers creating plug-in materials or textures.</p>\n\
<p>The way materials and texture maps handle their user interface is different than the way other plug-ins handle their UI in the command panel of 3ds Max. For materials and textures, a developer derives a class from <code>ParamDlg</code> and implements its methods.</p>\n\
<p>A method of <code>MtlBase</code> named <code>CreateParamDlg()</code> is called by the system when the material or texture is to be displayed in the material editor parameters area. This method is expected to create a new instance of a class derived from <code>ParamDlg</code>. The system then maintains the <code>ParamDlg</code> pointer. When the system needs to delete the memory associated with this instance it calls the method <code>ParamDlg::DeleteThis()</code>.</p>\n\
<p>Within the instance of the class derived from <code>ParamDlg</code> a developer can store any data needed to handle the user interface (for example, spinner control handles, window handles, etc.). The class will also need to have a pointer to the &apos;thing&apos; that is being edited. This &apos;thing&apos; is either a texture or the material.</p>\n\
<p>As the user works with different materials in the materials editor, these materials have to put up their user interfaces. For example, if the user is editing a Standard material in one sample window, then selects another Standard material in another sample window, the user interface changes to reflect the new material settings. The user interface does not &apos;flash&apos; however -- in other words the entire rollup page is not deleted and replaced. Rather the fields are simply updated to reflect the new values. What the system is doing is effectively &apos;passing off&apos; the user interface from one material to another. Two methods of <code>ParamDlg</code> allow this to happen. These are <code>SetThing()</code> and <code>GetThing()</code>.</p>\n\
<p>When a system calls <code>SetThing()</code> it passes a pointer to a <code>ReferenceTarget</code>. This is the item that is being edited. So normally a developer would implement this method to store the item being edited (the &apos;thing&apos;) and update the user interface controls to reflect the state of the new &apos;thing&apos;.</p>\n\
<p>When the system calls <code>GetThing(),</code> the plug-in returns the &apos;thing&apos; that is currently being edited.</p>\n\
<p>As an example, consider the following code from the Checker texture map. Checker derives a class from <code>ParamDlg</code> and uses data members to store the data needed for its operation. A portion of this code is shown below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">classCheckerDlg: public ParamDlg\n\
{\n\
   public:\n\
     HWND hwmedit; // window handle of the materials editor dialog\n\
     IMtlParams *ip;\n\
     Checker *theTex; // current Checker being edited.\n\
     HWND hPanel; // Rollup pane\n\
      ISpinnerControl *blurSpin;\n\
     IColorSwatch *cs[2];\n\
     TimeValue curTime;\n\
     ParamDlg *uvGenDlg;\n\
     int isActive;\n\
     ...</code></pre><p>Note that <code>CheckerDlg</code> has a data member that is a pointer to an instance of Checker (<code>theTex</code>). This is where it stores the &apos;thing&apos;, i.e. the current Checker being edited. Shown below are Checker&apos;s implementations of <code>GetThing()</code> and <code>SetThing()</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">ReferenceTarget* GetThing() { return ( ReferenceTarget *)theTex; }</code></pre><p>When the system calls <code>GetThing()</code>, Checker just returns the pointer to the item that is currently being edited.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">voidCheckerDlg::SetThing( ReferenceTarget *m)\n\
{\n\
   assert (m-&gt;ClassID()==checkerClassID);\n\
   assert (m-&gt;SuperClassID()==TEXMAP_CLASS_ID);\n\
\n\
   if (theTex) theTex-&gt;paramDlg = NULL;\n\
\n\
   theTex = (Checker *)m;\n\
   uvGenDlg-&gt;SetThing(theTex-&gt;uvGen);\n\
\n\
   if (theTex)\n\
     theTex-&gt;paramDlg = this;\n\
   LoadDialog(TRUE);\n\
}</code></pre><p>When the system calls <code>SetThing()</code>, Checker store the pointer to the item that is currently being edited into <code>theTex</code>. The Checker class itself maintains a pointer to the instance of <code>ParamDlg</code> that is handling the user interface. Note that Checker stores the <code>this</code> pointer (<code>theTex</code>-&gt;paramDlg = this;).</p>\n\
<p>Also note that <code>CheckerDlg</code> maintains a pointer to a <code>UVGen</code> (<code>UVGenDlg</code>). <code>UVGen</code> is a class developers use to encapsulate the user interface for UV coordinates. This pointer is initialized in the <code>CheckerDlg</code> constructor as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">uvGenDlg = theTex-&gt;uvGen-&gt;CreateParamDlg(hwmedit, imp);</code></pre><p>In the <code>CheckerDlg</code> implementation of <code>SetThing()</code> it also calls the <code>SetThing()</code> method on the <code>uvGenDlg</code>. This allows the <code>UVGen</code> to update its user interface in the dialog. These are the controls in the &apos;Coordinate&apos; and &apos;Noise&apos; rollups.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../materials_textures_and_maps.html\">Materials, Textures and Maps</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";