var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"render_session\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>The Render Session</h1></div><p>Whereas the Unified Renderer is the object that lives in the scene and manages user interaction, the render session is a transient object that controls the interval rendering process. It represents one running instance of the render engine - several of which may co-exist and render in parallel, for example to update the material editor preview while an interactive session is active.</p>\n\
<p><img src=\"./images/render_session.png\" alt=\"Render Session\"></p>\n\
<p>The render session takes, in essence, the scene definition as input and produces a rendered image as ouput. Both of these input and output are abstracted by the Render Session Context, to be covered in a following section.</p>\n\
<h2 id=\"implementation-the-render-session\">Implementation the Render Session</h2><p>The plugin developer implements the render sessions by deriving from two interfaces: IOfflineRenderSession and IInteractiveRenderSession. As seen previously, instances of these interfaces are created using methods on class UnifiedRenderer:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">virtual std::unique_ptr&lt;IOfflineRenderSession&gt; CreateOfflineSession(\n\
    IRenderSessionContext&amp; sessionContext) = 0;\n\
virtual std::unique_ptr&lt;IInteractiveRenderSession&gt; CreateInteractiveSession(\n\
    IRenderSessionContext&amp; sessionContext) = 0;</code></pre><p>Each method may return null if it does not support that particular mode of operation - e.g. if it doesn&apos;t support interactive rendering.</p>\n\
<h2 id=\"the-offline-render-session\">The Offline Render Session</h2><p>The offline render session is tasked with render still images and multi-frame animations. Its  emphasis is on overall rendering performance. No interactive scene changes may occur; only animation-related changes may occur from animation frame to the next.\n\
Class IOfflineRenderSession exposes the following methods, to be implemented by the plugin developer:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">virtual bool TranslateScene(const TimeValue t) = 0;\n\
virtual bool RenderOfflineFrame(const TimeValue t) = 0;\n\
virtual bool StoreRenderElementResult(const TimeValue t, IRenderElement&amp; render_element, IFrameBufferProcessor&amp; frame_buffer_processor) = 0;\n\
\n\
virtual void StopRendering() = 0;\n\
virtual void PauseRendering() = 0;\n\
virtual void ResumeRendering() = 0;</code></pre><p>The offline render session is invoked strictly from the main thread. It is the plugin developer&apos;s responsibility to implement any kind of multi-threading support.</p>\n\
<h2 id=\"the-interactive-render-session\">The Interactive Render Session</h2><p>The interactive render session renders a single still image progressively while taking into account concurrent user interaction (modifications of the scene). The emphasis is on interactivity, rather than on pure rendering performance. Great care must be taken to process scene changes in a way which does not de-stabilize 3ds Max, with special attention to threading and re-entrant change notifications.\n\
Class IInteractiveRenderSession exposes the following methods, to be implemented by the plugin developer:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">virtual bool InitiateInteractiveSession(const TimeValue t) = 0;\n\
virtual bool UpdateInteractiveSession(const TimeValue t, bool&amp; doneRendering) = 0;\n\
virtual void TerminateInteractiveSession() = 0;\n\
virtual bool IsInteractiveSessionUpToDate(const TimeValue t) = 0;</code></pre><p>The interactive render session is initialized from the main thread, but the <code>UpdateInteractiveSession()</code> method is pooled (at regular intervals) from a separate thread. This enables the application to remain active while the frame buffer is updated or while the scene is checked for validity. Nevertheless, care should be taken to avoid accessing the 3ds Max from this thread, as the 3ds Max SDK is generally not thread safe (see class IRenderingProcess for a mechanism that enables running tasks from the main thread).</p>\n\
<h2 id=\"concurrent-render-sessions\">Concurrent Render sessions</h2><p>Render Sessions were designed with encapsulation in mind. By separating the session from the plugin, it becomes possible to have multiple render sessions exist simultaneously for a single plugin. As of today, this scheme is used to update the material editor preview while an interactive render is active. By running two sessions in parallel, we avoid having to stop and restart a render.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../new_rendering_api.html\">New Rendering API</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";