var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"render_session_context\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>The Render Session Context</h1></div><p>The Render Session Context is the interface through which the Unified Renderer and the Render Session communicate. It is the glue that binds the various components of the Rendering API.</p>\n\
<p><img src=\"./images/render_session_context.png\" alt=\"Render Session Context\"></p>\n\
<p>The Render Session Context is created by the Unified Renderer. It is a two-way communication channel between the Rendering API core (the system) and the render session (the plugin). It encapsulates the following:</p>\n\
<ul>\n\
<li>The scene definition, abstracted into a common, simplified representation - regardless of the render target type (offline or interactive).</li>\n\
<li>Logging, progress reporting, and process control.</li>\n\
<li>Frame buffer management.</li>\n\
<li>Instancing management.</li>\n\
<li>Notification management.</li>\n\
<li>Etc.</li>\n\
</ul>\n\
<h2 id=\"scene-definition\">Scene Definition</h2><p>The Render Session Context exposes a set of methods for accessing the scene to be rendered:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual const ISceneContainer&amp; GetScene() const = 0;\n\
virtual const ICameraContainer&amp; GetCamera() const = 0;\n\
virtual IEnvironmentContainer* GetEnvironment() const = 0;\n\
virtual const IRenderSettingsContainer&amp; GetRenderSettings() const = 0;\n\
virtual std::vector&lt;IRenderElement*&gt; GetRenderElements() const = 0;\n\
virtual std::vector&lt;Atmospheric*&gt; GetAtmospherics() const = 0;\n\
virtual std::vector&lt;Effect*&gt; GetEffects() const = 0;</code></pre>\n\
<p>The first four methods return interfaces that are defined by the Rendering API. These further abstract / encapsulate specific aspects of the scene definition.</p>\n\
<h2 id=\"the-scene-container\">The Scene Container</h2><p>Class <code>ISceneContainer</code> exposes the exact set of objects to be rendered:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual std::vector&lt;INode*&gt; GetGeometricNodes(const TimeValue t, Interval&amp; validity) const = 0;\n\
virtual std::vector&lt;INode*&gt; GetLightNodes(const TimeValue t, Interval&amp; validity) const = 0;\n\
virtual std::vector&lt;INode*&gt; GetHelperNodes(const TimeValue t, Interval&amp; validity) const = 0;</code></pre>\n\
<p>Internally, this interface handles the scene traversal and all object flags or rendering options which may include or exclude specific objects from the render. For example, it handles the fact that hidden geometric objects should be exclude (unless render hidden object is enabled), while hidden light objects should not.</p>\n\
<p>The root node is also available, should the renderer need to do its own traversal:</p>\n\
<p><code>virtual INode* GetRootNode() const = 0;</code></p>\n\
<h2 id=\"the-camera-container\">The Camera Container</h2><p>Class <code>ICameraContainer</code> exposes the properties of the camera in a way that abstracts the type of camera being rendered - whether it be a GenCamera object, ViewParam, ViewExp, IPhysicalCamera, or else. The logic for interpreting these different types of cameras/views has always been a source of confusion for plugin developers.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">//! \\name Camera Projection\n\
virtual MotionTransforms EvaluateCameraTransform(\n\
const TimeValue t,\n\
Interval&amp; validity,\n\
const MotionBlurSettings* camera_motion_blur_settings = nullptr) const = 0;\n\
virtual ProjectionType GetProjectionType(const TimeValue t, Interval&amp; validity) const = 0;\n\
virtual float GetPerspectiveFOVRadians(const TimeValue t, Interval&amp; validity) const = 0;\n\
virtual float GetOrthographicApertureWidth(const TimeValue t, Interval&amp; validity) const = 0;\n\
virtual float GetOrthographicApertureHeight(const TimeValue t, Interval&amp; validity) const = 0;\n\
\n\
//! \\name Focus and Depth-of-Field\n\
virtual bool GetDOFEnabled(const TimeValue t, Interval&amp; validity) const = 0;\n\
virtual float GetLensFocalLength(const TimeValue t, Interval&amp; validity) const = 0;\n\
virtual float GetFocusPlaneDistance(const TimeValue t, Interval&amp; validity) const = 0;\n\
virtual float GetLensApertureRadius(const TimeValue t, Interval&amp; validity) const = 0;\n\
\n\
//! \\name Clipping Planes\n\
virtual bool GetClipEnabled(const TimeValue t, Interval&amp; validity) const = 0;\n\
virtual float GetClipNear(const TimeValue t, Interval&amp; validity) const = 0;\n\
virtual float GetClipFar(const TimeValue t, Interval&amp; validity) const = 0;\n\
\n\
//! \\name Resolution, Region, and Bitmap\n\
virtual IPoint2 GetResolution() const = 0;\n\
virtual float GetPixelAspectRatio() const = 0;\n\
virtual Bitmap* GetBitmap() const = 0;\n\
virtual Box2 GetRegion() const = 0;\n\
virtual Point2 GetImagePlaneOffset(const TimeValue t, Interval&amp; validity) const = 0;</code></pre>\n\
<p>The camera can also be accessed directly if needed:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual INode* GetCameraNode() const = 0;\n\
virtual IPhysicalCamera* GetPhysicalCamera(const TimeValue t) const = 0;\n\
virtual const View&amp; GetView(const TimeValue t, Interval&amp; validity) const = 0;\n\
virtual ViewParams GetViewParams(const TimeValue t, Interval&amp; validity) const = 0;</code></pre>\n\
<p>The camera container also exposes the global motion blur settings:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual MotionBlurSettings GetGlobalMotionBlurSettings(const TimeValue, Interval&amp;) const = 0;</code></pre>\n\
<h2 id=\"the-environment-container\">The Environment Container</h2><p>The Rendering API introduces a new type of plugin to 3ds Max: the Environment. This expands the traditional environment, represented by a single texture map, through the introduction of separate environment and background textures and colors.\n\
Class IEnvironmentContainer exposes these methods:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual BackgroundMode GetBackgroundMode() const = 0;\n\
virtual Texmap* GetBackgroundTexture() const = 0;\n\
virtual AColor GetBackgroundColor(const TimeValue t, Interval&amp; validity) const = 0;</code></pre>\n\
<p>Class <code>ITexturedEnvironment</code> expands <code>IEnvironment</code> with these methods:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual EnvironmentMode GetEnvironmentMode() const = 0;\n\
virtual Texmap* GetEnvironmentTexture() const = 0;\n\
virtual AColor GetEnvironmentColor(const TimeValue t, Interval&amp; validity) const = 0;</code></pre>\n\
<h2 id=\"the-render-settings-container\">The Render Settings Container</h2><p>Class IRenderSettingsContainer exposes various parameters that control he rendering process:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual bool GetRenderOnlySelected() const = 0;\n\
virtual TimeValue GetFrameDuration() const = 0;\n\
bool GetIsMEditRender() const;\n\
virtual RenderTargetType GetRenderTargetType() const = 0;\n\
virtual Renderer&amp; GetRenderer() const = 0;\n\
virtual ToneOperator* GetToneOperator() const = 0;\n\
virtual float GetPhysicalScale(const TimeValue t, Interval&amp; validity) const = 0;</code></pre>\n\
<h2 id=\"logging-amp-progress-reporting\">Logging &amp; Progress Reporting</h2><p>The Render Session Context provides access to interfaces that specifically handle logging, progress reporting, and more:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual IRenderingProcess&amp; GetRenderingProcess() const = 0;\n\
virtual IRenderingLogger&amp; GetLogger() const = 0;</code></pre>\n\
<h3 id=\"logging\">Logging</h3><p>Class <code>IRenderingLogger</code> provides simple message logging functionality. The messages are internally forwarded to the proper systems - for example, to the Production or ActiveShade message windows, as appropriate.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual void LogMessage(const MessageType type, const MCHAR* message) = 0;\n\
virtual void LogMessageOnce(const MessageType type, const MCHAR* message) = 0;</code></pre>\n\
<h3 id=\"progress-reporting-amp-process-control\">Progress Reporting &amp; Process Control</h3><p>Class IRenderingProcess handles progress reporting:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual void SetRenderingProgressTitle(const MCHAR* title) = 0;\n\
virtual void SetRenderingProgress(const size_t done, const size_t total, const ProgressType progressType) = 0;\n\
virtual void SetInfiniteProgress(const size_t done, const ProgressType progresstType) = 0;</code></pre>\n\
<p>It also handles timing and reporting of milestones:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual void TranslationStarted() = 0;\n\
virtual void TranslationFinished() = 0;\n\
virtual void RenderingStarted() = 0;\n\
virtual void RenderingPaused() = 0;\n\
virtual void RenderingResumed() = 0;\n\
virtual void RenderingFinished() = 0;\n\
virtual float GetTranslsationTimeInSeconds() const = 0;\n\
virtual float GetRenderingTimeInSeconds() const = 0;</code></pre>\n\
<p>It is responsible for reporting user aborts:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">virtual bool HasAbortBeenRequested() = 0;</code></pre><p>And, finally, this class implements a system for running tasks from the main thread - to be used when, for example, the renderer needs to access the 3ds Max scene, something which only safe when done from the main thread.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">virtual bool RunJobFromMainThread(IMainThreadJob&amp; job) = 0;</code></pre><h2 id=\"scene-evaluation\">Scene Evaluation</h2><p>The Render Session Context provides methods for evaluating scene nodes while taking motion blur settings into account:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual MotionTransforms EvaluateMotionTransforms(\n\
INode&amp; node,\n\
const TimeValue t,\n\
Interval&amp; validity,\n\
const MotionBlurSettings* node_motion_blur_settings = nullptr) const = 0;\n\
virtual MotionBlurSettings GetMotionBlurSettingsForNode(\n\
INode&amp; node,\n\
const TimeValue t,\n\
Interval&amp; validity,\n\
const MotionBlurSettings* global_motion_blur_settings = nullptr) const = 0;</code></pre>\n\
<p>These methods internally fetch all the motion blur properties from the node and the camera, but still allow the renderer to provide its own.\n\
The Context also provides methods for handling of RenderBegin() and RenderEnd(). These methods are designed to function correctly with both offline and interactive rendering:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual void CallRenderBegin(ReferenceMaker&amp; refMaker, const TimeValue t) = 0;\n\
virtual void CallRenderEnd(const TimeValue t) = 0;</code></pre>\n\
<h2 id=\"processing-of-frame-buffer\">Processing of Frame Buffer</h2><p>The Render Session may store its rendered frame buffer directly, or may use these methods to use the built-in mechanism:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual IFrameBufferProcessor&amp; GetMainFrameBufferProcessor() = 0;\n\
virtual void UpdateBitmapDisplay() = 0;</code></pre>\n\
<p>Class <code>IFrameBufferProcessor</code> is optimized to make use of threads and can handle processing of the ToneOperator.</p>\n\
<h2 id=\"interactive-change-notifications\">Interactive Change Notifications</h2><p>Classes <code>IRenderSessionContext</code>, <code>ISceneContainer</code>, <code>ICameraContainer</code>, and <code>IRenderSettingsContainer</code> all expose methods for registering notification callbacks - to be called whenever the contents of the container changes. For example, if render selected is being used, and the selection set changes, then ISceneContainer would notify the renderer that the contents of the scene has changed.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual void RegisterChangeNotifier(IChangeNotifier&amp; notifier) const = 0;\n\
virtual void UnregisterChangeNotifier(IChangeNotifier&amp; notifier) const = 0;</code></pre>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../new_rendering_api.html\">New Rendering API</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";