var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"translator_component\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>The Translator Component</h1></div><p>The Translator Component offers a framework for building a translator upon. It facilitates scene translation and updates through its built-in mechanisms for tracking dependencies between translated elements. It also offers base classes and various helpers that take some of the translation task out of the hands of the plugin developer.</p>\n\
<p><img src=\"./images/render_api_translator.png\" alt=\"Render Session Context\"></p>\n\
<p>This component is entirely optional. A renderer built on top of the Rendering API does not have to use the Translator Component.</p>\n\
<h2 id=\"the-translator\">The Translator</h2><p>The translation system is built on the concept of a graph of translators. The translators are responsible for translating a 3ds Max entity into the rendering engine&apos;s own representation. Thus, the translators take 3ds Max scene elements as inputs and produce the rendering engine&apos;s elements as outputs. Each translator is dependent on the output of other translators, thus forming a dependency graph.</p>\n\
<h3 id=\"translation\">Translation</h3><p>The translator&apos;s main task is to convert a 3ds Max scene element into the rendering engine&apos;s representation. As such, its foremost method is:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual TranslationResult Translate(\n\
        const TimeValue translationTime,\n\
        Interval&amp; newValidity,             \n\
        ITranslationProgress&amp; translationProgress,\n\
        KeyframeList&amp; keyframesNeeded) = 0;</code></pre>\n\
<p>This method does the actual translation work, and may also be called to update an item which has already translated. Updates happen in the following cases:</p>\n\
<ul>\n\
<li>The time value, at which the rendering is to occur, has changed and has fallen outside of the translator&apos;s validity interval (as output by Translate()). This occurs when rendering animations, or when the user moves the time slider.</li>\n\
<li>The translator has been explicitly invalidate, by calling Invalidate(), usually as the result of an interactive change in the scene.</li>\n\
<li>The output of a translator, on which there is a dependency, has changed. The dependent translator needs to be updated to take the new output into account. This mechanism is automatic.</li>\n\
</ul>\n\
<h3 id=\"translator-keys\">Translator Keys</h3><p>Translator Keys are the input to a Translator, and as such encapsulate everything the Translator needs to perform its task. This may be an INode<em>, a Mtl</em>, or else. The Translator Key is named so because it is also used as a key for a lookup table. As such, it is a unique identifier for a Translator, enabling automatic elimination of duplicates.</p>\n\
<p>Class <code>TranslatorKey</code> is the base class for all Translator Keys, though in general it should not be used directly. Instead, it is recommended to use the built-in sub-classes which implement specializations for keys storing a pointer, a reference, or an arbitrary struct.</p>\n\
<p>Class <code>TranslatorKey</code> uses hashing to perform efficient lookups. Its main methods are:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    TranslatorKey(const size_t hash);\n\
        size_t get_hash() const;\n\
    virtual bool operator==(const TranslatorKey&amp; rhs) const = 0;\n\
    virtual std::unique_ptr&lt;const TranslatorKey&gt; CreateClone() const = 0;</code></pre>\n\
<p>The <code>CreateClone()</code> method enables the plugin developer to allocate temporary keys, on the stack, for efficiency; the system will clone the key if and only if it needs to create a new entry in the lookup table.</p>\n\
<p>The built-in specializations of class <code>TranslatorKey</code> are:</p>\n\
<ul>\n\
<li>Class <code>GenericTranslatorKey_Empty</code>: A key which stores no data, but which instead relies on its type to identify entities unique to a scene.</li>\n\
<li>Class <code>GenericTranslatorKey_SinglePointer</code>: A key which stores a single pointer value.</li>\n\
<li>Class <code>GenericTranslatorKey_SingleReference</code>: A key which stores a single reference (non-null pointer).</li>\n\
<li>Class <code>GenericTranslatorKey_Struct</code>: A key which stores an arbitrary struct.</li>\n\
</ul>\n\
<h3 id=\"translator-outputs\">Translator Outputs</h3><p>The call to <code>Translate()</code> will typically result in the creation of one or more outputs, on the Translator. Class <code>Translator</code> exposes these methods for managing outputs are:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    size_t GetNumOutputs() const;\n\
    template&lt;typename OutputType&gt;\n\
    std::shared_ptr&lt;const OutputType&gt; GetOutput(const size_t index) const;\n\
    void SetNumOutputs(const size_t num);\n\
    void SetOutput(const size_t index, std::shared_ptr&lt;const ITranslatorOutput&gt; output);\n\
    void ResetOutput(const size_t index);\n\
    void ResetAllOutputs();</code></pre>\n\
<p>Outputs may require construction and destruction - for example, if the output is a handle to an entity in the rendering engine&apos;s database, then destruction of the output requires the handle to be freed. For this reason, outputs derive from ITranslatorOutput - which defines a virtual destructor.</p>\n\
<p>Outputs are stored using shared pointers. This is to facilitate the management of outputs which end up being shared across translators. For example, an instance translator may be dependent on a material translator; the material handle is output by the material translator and used by the instance translator. Internally, the system shares the outputs of a Translator with its dependents; using a shared pointer results in the output (e.g. the material handle) being destroyed only once it is no longer in used.</p>\n\
<p>Class <code>Translator</code> also provides functionality for storing simple output values - i.e., outputs which contain a POD value, which does not require constructor nor destruction (mainly integers, bools, floats, etc.). These avoid creating specialized outputs for simple things:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    template&lt;typename SimpleValueType&gt;\n\
    void SetOutput_SimpleValue(const size_t output_index, const SimpleValueType&amp; value);\n\
    template&lt;typename SimpleValueType&gt;\n\
    SimpleValueType GetOutput_SimpleValue(const size_t output_index, const SimpleValueType&amp; default_value) const;</code></pre>\n\
<p>Note that, when Translate() is being called to update an already-translated entity, the outputs need not be re-set. For example, if a material translator were to be re-translated to update a single parameter, then output (a material handle) may still remain valid, as the material handle would not have to be re-created. But if the material changed in a way that required a whole new material handle to be created, then the old output must be replaced. The system would then automatically invalidate dependent Translators, and the shared pointer would take care of garbage collecting the old material handle once it were no longer referenced by these dependents - i.e., after all dependents would be fully updates.</p>\n\
<h3 id=\"acquiring-dependencies\">Acquiring Dependencies</h3><p>When a Translator requires the output of another Translator to perform its own translation, it must acquire a dependency to that other Translator. Following this acquisition, the dependent Translator may use the outputs of the Translator it just acquired.\n\
The acquisition is performed using this method from class <code>Translator</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    template&lt;typename TranslatorType&gt;\n\
    const TranslatorType* AcquireChildTranslator(\n\
        const TranslatorKey&amp; key,\n\
        const TimeValue t,\n\
        ITranslationProgress&amp; translation_progress,\n\
        TranslationResult&amp; result);</code></pre>\n\
<p>The dependent translator must provide a key, which it builds according to the entity that needs to be translated. The system will then automatically:</p>\n\
<ol>\n\
<li>Check if a Translator already exists for the given key, create one if not.</li>\n\
<li>Translate or update the Translator if needed (based on validity).</li>\n\
<li>Have the dependent Translator acquire all the outputs of the acquired Translator, such that they are not garbage collected while in use.\n\
Allocation of New Translator\n\
New Translators are allocated using a method on class TranslatorKey:</li>\n\
</ol>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    virtual std::unique_ptr&lt;Translator&gt; AllocateNewTranslator(\n\
        TranslatorGraphNode&amp; translator_graph_node,\n\
        const IRenderSessionContext&amp; render_session_context,\n\
        const TimeValue initial_time) const = 0;</code></pre>\n\
<p>Thus, in addition to encapsulating a Translator&apos;s input and being a lookup key, class TranslatorKey behaves as a factory for class Translator.</p>\n\
<h2 id=\"the-translation-manager\">The Translation Manager</h2><p>The Translation Manager is the entry point to the Translator Component. It is where the root of the translation graph is acquired. Class <code>ITranslationManager</code> exposes these methods for translating and updating the root of the translation graph:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    template&lt;typename TranslatorType&gt;\n\
    TranslatorType* TranslateScene(\n\
        const TranslatorKey&amp; key,\n\
        const TimeValue t,\n\
        TranslationResult&amp; result);\n\
    virtual TranslationResult UpdateScene(\n\
        Translator&amp; rootTranslator,\n\
        const TimeValue t) = 0;</code></pre>\n\
<p>The plugin developer may check whether any part of the graph is invalid, and would benefit from an update, using this method:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    virtual bool DoesSceneNeedUpdate(\n\
        const Translator&amp; rootTranslator,\n\
        const TimeValue t) = 0;</code></pre>\n\
<p>When the plugin is done rendering a scene, the root translator is to be released using:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    virtual void ReleaseSceneTranslator(const Translator&amp; rootTranslator) = 0;</code></pre>\n\
<p>An instance to a the Translation Manager may be acquired from this method on class <code>IRenderSessionContext</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    virtual ITranslationManager&amp; GetTranslationManager() const = 0;</code></pre>\n\
<h2 id=\"translator-base-classes\">Translator Base Classes</h2><p>To facilitate the implementation of a full translator, the Rendering API provides base classes for commonly used translators. These perform the basic tasks of: monitoring changes, calling <code>RenderBegin()</code> before translating/updating, and providing basic evaluation functionality. The API currently offers base translator classes for:</p>\n\
<ul>\n\
<li>Camera (class <code>ICameraContainer</code>)</li>\n\
<li>Environment (class <code>IEnvironment</code>)</li>\n\
<li>Instances (class <code>INode</code>)</li>\n\
<li>Material (class <code>Mtl</code>)</li>\n\
<li>Texture Plugins (class <code>Texmap</code>)</li>\n\
<li>Generic Reference Targets (class <code>ReferenceTarget</code>)</li>\n\
<li>Generic Objects (class <code>Object</code>)</li>\n\
</ul>\n\
<h2 id=\"translation-helpers\">Translation Helpers</h2><p>The Rendering API also provides specialized helper classes that implement common, but difficult tasks. The API currently offers these helper classes:</p>\n\
<ul>\n\
<li>Class <code>MeshFlattener</code> flattens a mesh into a list of vertices with normal and UV coordinates, and enables splitting up the mesh according to material IDs. Because this process takes a significant part of the overall translation time, it may be expected to receive significant performance attention in the future, potentially including multi-threading support.</li>\n\
<li>Class <code>NodeInstancingPool</code> implements logic for pooling nodes that are instances of a common object. There is no other such built-in mechanism in the 3ds Max SDK.</li>\n\
<li>Class <code>IMeshCollector</code> abstracts the logic necessary for reading and interpreting class <code>Mesh</code>.</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../new_rendering_api.html\">New Rendering API</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";