var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"osl\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Open Shading Language (OSL) Support</h1></div><h2 id=\"overview\">Overview</h2><p>3ds Max supports the Open Shading Language (OSL), in <strong>3ds Max 2019 and higher</strong>.  OSL is implemented as an &quot;OSL Map&quot;, which is an execution environment for OSL shaders inside of 3d Max, and is exposed to the API as a regular C++ 3d Max shader (that is, it is a subclass of Texmap and implements <code>EvalColor()</code>, <code>EvalNormal()</code> and <code>EvalNormalPerturb()</code> methods).</p>\n\
<p>OSL works in any renderer that supports the 3ds Max shader API, such as Scanline, vRay, Corona, and so on. It can also work with renderers that support OSL natively, such as Arnold. In this case, the renderer rather than 3ds Max executes the OSL code.</p>\n\
<p>OSL also works outside of renderers, anywhere in 3ds Max where a regular map is requested, such as in the Displacement modifier.</p>\n\
<p>OSL is a simple to understand shading language, and writing a shader in OSL is orders of magnitude less effort than developing the equivalent functionality as a 3ds Max C++ shader. Instead of dealing with complex plug-ins, building the UI manually, and working with &quot;validity intervals&quot; and other details of the 3ds Max APIs, you simply type some OSL code in a text file.</p>\n\
<p>OSL also benefits from JIT compilation and optimization of entire shade trees at once, as long as all the shaders in the shade tree are OSL shaders.&#x2003;</p>\n\
<h2 id=\"new-in-3ds-max-2021\">New in 3ds Max 2021</h2><p>Several things were added for OSL in 3ds Max 2021, mostly around UI refinement. The default layout engine was improved to right-justify parameter headings and have some more user control of the layout, but it now also has the ability to load a .ui file that completely defines the user interface of an OSL shader. </p>\n\
<p>There are also several new options for shader metadata; not only can the shader itself can be versioned with the <code>version</code> attribute, but parameters appearance and behavior can be tweaked with the new attributes <code>autopop</code>, <code>connectable</code>, <code>step</code>, <code>packName</code>, <code>widgetWidth</code> and <code>worldunits</code>.\n\
We now correctly support the widget type &quot;null&quot;, as well as a few custom UI widgets &quot;max:actionButton&quot; and &quot;max:ramp0&quot;.</p>\n\
<p>Also, parameter support was extended. The new &quot;semi-types&quot; vector2, vector4, color2, color4 and matrix33 can now be used. These types are not built-in OSL types, but defined in header files. Including these header files and using these types is now supported (see the section about include files). Also, parameters of type array are supported in certain cases (see the ramp widget).</p>\n\
<p>Support was added for adding OSL shaders from a 3ds Max &quot;Package&quot;. This is the preferred method to distribute sets of OSL shaders. </p>\n\
<p>Finally, the support for OSL in the viewport has been vastly improved. The majority of OSL language functions are automatically converted to HLSL code and will display correctly in the viewport in &apos;Realistic&apos; material display mode. You can now effectively develop OSL shaders without ever launching an actual rendering! Not everything is supported (for example, shaders passing string values to each other, or doing operations on string values) but the confidence level is displayed at the bottom of the OSL map as a percentage value. If this reads 100%, the shader will most likely display in the viewport identically to in the final render.</p>\n\
<h2 id=\"developing-osl-maps\">Developing OSL Maps</h2><p>There are two fundamental workflows for the OSL Map, using OSL Maps in the Material Browser, and developing (editing, changing, or writing) OSL Maps.  We will discuss the development workflow here.  See the 3ds Max User Guide for general usage information.</p>\n\
<p>OSL shaders that populate the material browser come from two places. </p>\n\
<p>The first way to load OSL shaders is from a package defined in a <em>PackageContents.xml</em> file. The component description must be &quot;osl shaders parts&quot;. For more information, see <a href=\"./../../writing_plug-ins/plugin_package.html\">Packaging Plug-ins</a>. This is the preferred method for third parties to introduce OSL shaders to 3ds Max.</p>\n\
<p>The second way is from subfolders named &quot;OSL&quot; inside plug-in directories, both the standard 3ds Max plug-in directories, and third-party plug-in directories added on the <strong>Customize &gt; Configure System Paths</strong> dialog. There is also a &quot;system&quot; OSL folder which contains basic shaders shipped directly by Autodesk, where things like the OSL basic headers reside.\n\
Shaders are loaded from:</p>\n\
<ul>\n\
<li>&lt;3dsmax&gt;\\<strong>OSL</strong>    (do not touch files in this folder)</li>\n\
<li>&lt;3dsmax&gt;\\StdPlugs\\<strong>OSL</strong></li>\n\
<li>&lt;3dsmax&gt;\\Plugins\\<strong>OSL</strong></li>\n\
<li><configured plug-in directory>\\<strong>OSL</strong></configured></li>\n\
</ul>\n\
<p><strong>Tip</strong>: If the OSL source file is put in a subfolder of one of these OSL folders, it will show up in the Map Browser in a subcategory named after the folder.</p>\n\
<p>In this workflow, all shaders are hosted in files in the plug-in folders or in packages, and the code in the file is what is being used at render time. Add a parameter to the code in the file, and the shader will update with the new parameter. Change the algorithm, and the rendering result will change. (See Shader Versioning below.)</p>\n\
<p>There are two ways you can work with OSL Map source files, in memory or on disk.  When the OSL Map is linked to the file on disk, changes to that file are detected and re-loaded and re-evaluated.  This is true whether the file is loaded in the OSL source editor, or in another external editor. </p>\n\
<p>When OSL source files are &quot;unlinked&quot; from the source file on disk, they are held in memory, and persist in the scene file across save/load operations.  Changes are not saved to the original OSL file, unless you re-link or save under another file name. The scene can be rendered on other machines, since there are no external file dependencies.  To make the shader available to other scenes, the map can be saved to one of the OSL directories as a new file, and will then appear in the OSL section of the Material Map Browser.</p>\n\
<p>You can also start with an OSL Map template, which contains skeleton code for creating a new OSL shader from scratch.  This shader is in the General maps section named &quot;OSL Map&quot;.  This is an empty OSL shader that is in &quot;unlinked&quot; mode by default. </p>\n\
<p>You can switch between linked and unlinked mode during the shader development process, depending on how you want to manage the OSL source.  In general, operating in unlinked mode allows you to iterate and test changes as you progress, without altering the original source.  You can re-link to either save as a new shader or update the original source, and to make the shader available to other scenes.</p>\n\
<h3 id=\"shader-versioning\">Shader Versioning</h3><p>When a scene is loaded containing shaders in &quot;linked&quot; mode, the shaders will find their corresponding source on disk, and &quot;update&quot; themselves from there. But when moving such a scene to a different older 3ds Max installation, there is a risk of an older version on disk accidentaly overwriting a newer version embedded in the scene. </p>\n\
<p>The big risk with getting shaders &quot;downgraded&quot; is if the newer shader contans a parameter the older does not. This parameter value (or any connections to it) will then be lost forever.</p>\n\
<p>To avoid this, there is a <strong>version</strong> metadata attribute (new in 3ds Max 2021) that give some additional control over this process. If the OSL shader stored in the scene is newer than the one on disk, it will not be updated. The newer shader in the scene will continue to be used.</p>\n\
<p>In general, when making new versions of shaders, make sure your <em>newer</em> version works when loading <em>older</em> scenes, by defaulting any <em>new</em> parameters such a way that the result is compatible with the old version. If your newer version is different, simply name the shader something different. Shaders are considered linked if both the filename <em>and</em> the name of the shader <em>in</em> the file is the same. If you make a new variant of a shader that is not at all compatible with the old one, renaming it with something like &quot;V2&quot; at the end will not ever confuse your new version with the old. Files containing the old shader will still work, since it will use the old copy of the shader embedded in the file.</p>\n\
<h2 id=\"writing-an-osl-shader-for-3ds-max\">Writing an OSL shader for 3ds Max</h2><p>A complete description of the latest version of OSL is available in the <a href=\"https://github.com/imageworks/OpenShadingLanguage/blob/master/src/doc/osl-languagespec.pdf\" target=\"_blank\">Open Shading Language Specification</a>.</p>\n\
<p><img src=\"./images/osl-parameters.png\" alt=\"osl shader example\"></p>\n\
<p>General points about OSL in 3ds Max:\n\
&#x2003;</p>\n\
<ul>\n\
<li>A shader looks very similar to a C/C++ function definition.</li>\n\
<li>Shaders have inputs and outputs. Both inputs and outputs always have to have a default value assigned, without which an error is emitted when compiling.</li>\n\
<li>Inputs can sometimes have computed defaults. This is the value the input gets if it is not connected to anything. Most procedural texture shaders have an input for the coordinate for which it is texturing, but also uses a computed default so that the shader does something reasonable without being forced to connect, for example a UV generating shader to its input.</li>\n\
<li>Since computed defaults are used for parameters that are inherently varying across the texture, having &quot;fixed value&quot; inputs for them is meaningless. Hence, no value spinners are shown for these in the UI, but only the slot for connecting them to some other shader.</li>\n\
</ul>\n\
<h2 id=\"osl-specifics-in-3ds-max\">OSL Specifics in 3ds Max</h2><p>This section covers some of the specifics about how OSL is supported in 3ds Max, including some of its limitations.</p>\n\
<h3 id=\"osl-closures-not-supported\">OSL Closures Not Supported</h3><p>Most importantly, it can only be used for procedural texturing. OSL Closures are not supported in this version which means it is useful for texturing, but not for making materials.</p>\n\
<p>There are several reasons for this, mostly technical, but one of them is that while the OSL language is well specified in its feature-set, the set of closures supported is renderer dependent, and can shift from renderer to renderer. We at Autodesk have a vision of making textures and materials interchangeable, and before a standard for closures is established, that is hard to accomplish. </p>\n\
<p>Therefore, the recommended workflow is to build your procedural texture maps in OSL, but connect the outputs to a standardized, well defined, renderer-independent material, such as the 3ds Max Physical Material.</p>\n\
<h3 id=\"parameter-type-support\">Parameter Type Support</h3><p>A second limitation is that only base types are supported for parameters. This means integers, floats, strings, and point/vector/normal data types. Complex types like structs or arrays are not generally supported. However, OSL has some &quot;special&quot; types that actually are defined as structs with operators, and these <em>are</em> supported. These types are vector2, vector4, color2, color4 and matrix33. Also, arrays can exist as parameters but have no automatic UI, with the exception of the specific &quot;max:ramp0&quot; widget described in the UI section.</p>\n\
<p>Note that this limitation only applies to parameters, any kind of structs and arrays can be used inside the shader code itself. \n\
&#x2003;</p>\n\
<h3 id=\"limited-use-of-osl-include-directive\">Limited use of OSL Include Directive</h3><p>Finally, due to the mode of embedding the OSL code into the OSL Map itself, file inclusion is discouraged. The standard OSL include file &quot;stdosl.h&quot; is implicitly included automatically by the compiler, you do not need to include it explicitly. If you have this line in your source, you can delete it: </p>\n\
<p><code>#include &lt;stdosl.h&gt;</code></p>\n\
<p>Including other files will <em>only</em> work if those files are in the 3ds Max system OSL folder. So the &quot;system&quot; includes will work (e.g. vector2.h, vector4.h, color2.h, color4.h and matrix33.h).</p>\n\
<p>If you write a shader that uses custom include files, the safest and most portable way is to copy the relevant content from that file into the OSL source itself.</p>\n\
<h3 id=\"global-variables-and-attributes\">Global Variables and Attributes</h3><p>OSL by default works in a &quot;common&quot; coordinate space that can (in theory) be different from renderer to renderer. In practice, most renderers use world space for this coordinate space, and this is true also for the OSL Map. </p>\n\
<p>This means that the OSL global parameter <code>P</code> is in world space. Many shaders use the above mentioned feature of a computed default so that the shader works immediately, by having a line such as</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    vector Input = P</code></pre>\n\
<p>This will make the <code>Input</code> parameter have the world space point as a default. Since texturing things in world space may not be what a user expects (since moving the object would drag it &quot;through&quot; the texture space, it is probably more useful to most users to instead use object space, which would look like this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    vector Input = transform(&quot;object&quot;, P),</code></pre>\n\
<p>Or, if it is a 2D texture, use the default UV space since 3d Max populates the global OSL variables u and v with the default 3d Max UVW map channel 1:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    vector Input = vector(u, v, 0),</code></pre>\n\
<p>Also, since in theory (according to the OSL specification) P is in &quot;common&quot; space, and you actually want it in world space, the correct way of transforming it is:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    vector Input = transform(&quot;world&quot;, P),</code></pre>\n\
<p>However, in the OSL Map&apos;s execution environment, this does nothing, since P is already in world space.\n\
&#x2003;</p>\n\
<h3 id=\"scene-attributes\">Scene Attributes</h3><p>The OSL rendering state is filled in before 3d Max executes the shader. The standard variables such as <code>P</code>, <code>N</code>, and <code>I</code> are filled in, where the <code>u</code> and <code>v</code> variables gets pre-populated with map channel 1&apos;s UV coordinate.</p>\n\
<p>OSL also allows the generic <code>getattribute()</code> function to get a named attribute from the scene. It is up to the renderer executing the OSL code to populate these named attributes, which means it is outside the control of the OSL Map itself to guarantee whether these attributes are defined. </p>\n\
<p>This is the set of attributes the OSL Map itself uses, that can be guaranteed to exist when rendering inside of 3ds Max and using the OSL Map as the execution environment:</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Attribute Name</th>\n\
<th>Data type</th>\n\
<th>Description</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>UVxx</code></td>\n\
<td>point</td>\n\
<td>The 3D UVW coordinate for map channel <em>xx</em> (UV0 to UV99), for example <code>UV1</code>.</td>\n\
</tr>\n\
<tr>\n\
<td><code>mtlID</code></td>\n\
<td>int</td>\n\
<td>The material ID of a face, or particle ID of a particle.</td>\n\
</tr>\n\
<tr>\n\
<td><code>gBufID</code></td>\n\
<td>int</td>\n\
<td>The <strong>Object ID</strong> in the 3d Max object properties dialog.</td>\n\
</tr>\n\
<tr>\n\
<td><code>nodeName</code></td>\n\
<td>string</td>\n\
<td>The name of the instance in 3d Max. OSL has a built-in attribute <code>geom:name</code>, but that will be what the renderer calls the object, which isn&apos;t necessarily the same thing.</td>\n\
</tr>\n\
<tr>\n\
<td><code>nodeID</code></td>\n\
<td>int</td>\n\
<td>What <code>NodeID()</code> in the ShadeContext returns.</td>\n\
</tr>\n\
<tr>\n\
<td><code>nodeRenderID</code></td>\n\
<td>int</td>\n\
<td>What <code>INode::GetRenderID()</code> returns.</td>\n\
</tr>\n\
<tr>\n\
<td><code>nodeHandle</code></td>\n\
<td>int</td>\n\
<td>What <code>INode::GetHandle()</code> returns.</td>\n\
</tr>\n\
<tr>\n\
<td><code>wireColor</code></td>\n\
<td>color</td>\n\
<td>The instances wireframe color.</td>\n\
</tr>\n\
<tr>\n\
<td><code>paLife</code></td>\n\
<td>float</td>\n\
<td>Normalized particle age (0.0-1.0 over the age of the particle)</td>\n\
</tr>\n\
<tr>\n\
<td><code>usr_xxxxx</code></td>\n\
<td>int <br>float <br>string <br>float,float,float</td>\n\
<td>Any value from the 3d Max Object Properties <strong>User Defined</strong> page, as an integer, float, string, or three comma-separated floats. <br>For example,<code>Hello=3.0</code>&quot;`, will allow you to get the float attribute usr_Hello from the OSL code to retrieve the value 3.0.</td>\n\
</tr>\n\
</tbody></table>\n\
<p>Remember this list is defined by the renderer, and is only guaranteed when run inside the OSL Map. For example, Arnold uses a slightly different naming to store UV coordinates. Therefore we recommend you do not deal with UV channels directly in your OSL code. Instead, add an input for the coordinates, defaulting it to <code>vector(u, v, 0)</code>, which is guaranteed to yield a meaningful default value for 2D textures. Then, let the user connect the Autodesk-supplied <strong>UVW Channel</strong> shader, which has code in it to understand the different UV attribute formats for the supported renderers.</p>\n\
<p>&#x2003;</p>\n\
<h3 id=\"shader-metadata\">Shader Metadata</h3><p>In OSL metadata can be assigned to whole shaders, or individual parameters. 3ds Max supports a subset of those metadata settings, as well as some of its own.</p>\n\
<p>For the shader, the following metadata are supported</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Name</th>\n\
<th>Meaning</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td>label</td>\n\
<td>The &quot;display name&quot; of the shader. This is what is displayed the Material Browser, as well as the title in the Material Editor and on the node in Slate Material Editor.</td>\n\
</tr>\n\
<tr>\n\
<td>help</td>\n\
<td>A short help text displayed the top of the material editor. <br>This can contain a subset of HTML formatting.</td>\n\
</tr>\n\
<tr>\n\
<td>URL</td>\n\
<td>A link to a more extensive help page about the shader.</td>\n\
</tr>\n\
<tr>\n\
<td>logo</td>\n\
<td>An image loaded from the same folder as the OSL shader, and displayed at the bottom of the material editor in place of the default logo. This Allows you to personalize the look of your shaders, or even include a sample image of what it does directly in the UI. If the file cannot be found, the default logo is displayed.</td>\n\
</tr>\n\
<tr>\n\
<td>category</td>\n\
<td>Sorts this shader in a particular category in the 3d Max Map browser. <br>By default, categories are determined by the subdirectory the shader is in, but this can be overridden by a specific category metadata value. It allows nested categories separated by a backslash, for example &quot;Math\\Color&quot; puts the in the &quot;Math&quot; category, with a sub-category of &quot;Color&quot;.</td>\n\
</tr>\n\
<tr>\n\
<td>version</td>\n\
<td>The version number of the shader. It should be a string of up to three dotted numbers (e.g. &quot;1.3.11&quot;) to encode the version of the shader, although a float or integer version number also works. A shader with no version metadata is considered as version zero.</td>\n\
</tr>\n\
</tbody></table>\n\
<p>For individual parameters, the following metadata are supported:</p>\n\
<table>\n\
<tbody>\n\
<tr><th>Name</th><th>Description</th></tr>\n\
<tr><td>label</td><td>The &quot;display name&quot; of the parameter. The parameter name displayed in the Material Editor.</td></tr>\n\
<tr><td>help</td><td>The tooltip of the control. This can contain a subset of HTML formatting. </td></tr>\n\
<tr><td>min, max</td><td>The minimum and maximum value of the parameter.</td></tr>\n\
<tr><td>step</td><td>The step size for spinners.</td></tr>\n\
<tr><td>connectable</td><td>If 1 (the default), the parameter will have a corresponding map input. If 0, there will be no map input. I.e. the parameter can not be connected to, only set by the widget.</td></tr>\n\
<tr><td> widget</td><td>The kind of UI control to use for the parameter. Supports the following OSL widget types:\n\
    <ul>\n\
    <li>&quot;filename&quot; - displays a file selector button beside the text field, and makes the pointed-to file a 3ds Max &quot;asset&quot; editable via the asset tracker. A shader using this also automatically pops up a file dialog when the it is instanced, unless the <b>autopop</b> metadata is 0.</li>\n\
    <li>&quot;checkBox&quot; - displays an integer value as a checkbox. </li>\n\
    <li>&quot;popup&quot; - displays a popup list of choices.</li>\n\
    <li>&quot;mapper&quot; - displays a popup list of choices that maps to other values such as integers</li>\n\
    <li>&quot;null&quot; - displays no widget at all. There is still a map input (unless <b>connectable</b> is 0).</li>\n\
    <li>&quot;max:xxx&quot; - additional 3ds Max specific widgets. See the user interface section.</li>\n\
    </ul>\n\
<p>See the OSL Specification for details</p></td></tr>\n\
<tr><td>timeValue</td><td>If enabled (set to 1), any integer parameter this metadata is assigned to automatically gets populated with the current frame number, and any float parameter gets automatically populated with the current scene time, in seconds.</td></tr>\n\
<tr><td>worldunits</td><td>If enabled (set to 1), any float parameter this metadata is assigned to will be displayed using display units, seen by the shader in system units, and the default value will always be in <b>meters</b>.</td></tr>\n\
<tr><td>autopop</td><td>If the parameter has a widget of type &quot;filename&quot;, a file browser will automatically pop up when creating the shader, similar to how the classic <b>Bitmap</b> behaves. If this is not desired, you can set autpop to 0 and disable the behavior. The default is 1.</td></tr>\n\
<tr><td>packName</td><td>Allows grouping parameters on the same row. Any parameters with the same packName will show up on the same row in the UI, headed by that name. The regular parameter labels are ignored.</td></tr>\n\
<tr><td>widgetWidth</td><td>Adjusts the width of the widget (measured in pixels at 100% UI scaling). </td></tr>\n\
</tbody></table>\n\
\n\
<h3 id=\"metadata-example\">Metadata Example</h3><p>Here is a visual example of how metadata shows up in an OSL shader:</p>\n\
<p><img src=\"./images/metadata.jpg\" alt=\"metadata example\"></p>\n\
<h2 id=\"advanced-user-interface-3ds-max-2021-\">Advanced User Interface (3ds Max 2021-)</h2><p>Several improvements have been made to the UI engine for OSL, adding some components that are not strictly standard. However, care has been taken so these additions are only extensions. Meaning, the shader itself will work &quot;as is&quot; in any standard compliant OSL host - or an earlier 3ds Max version - it&apos;s just that the UI will be less &quot;fancy&quot; than what it is starting from 3ds Max 2021.</p>\n\
<h3 id=\"metadata-driven-ui\">Metadata-driven UI</h3><p>Additional metadata can be used to make more complex UIs. </p>\n\
<ul>\n\
<li>The widget type &quot;null&quot; will make the parameter invisible (although it is still acessible through maxscript, and controllers can be assigned to it).</li>\n\
<li>The metadata <code>connectable</code>, when set to 0, will remove the map input for the parameter. \n\
The parameter itself still exists, but it can&apos;t be connected to by another shader. This reduces clutter and makes the node smaller in SME, and can be used for parameters where it&apos;s unlikly it will ever be useful to connect it to another shader (like a mode switch).</li>\n\
<li>The metadata <code>worldunits</code>, when set to 1, will make the parameter type a <code>TYPE_WORLD</code>, which means it will be displayed in the currently selected display unit, its <em>value</em> (as seen by the shader) will always be in scene units, and the <em>default value</em> will <em>always be given in meters</em>.</li>\n\
</ul>\n\
<h4 id=\"row-packing\">Row Packing</h4><p>By default, one parameter per row is displayed. This tends to make the UI very &quot;tall&quot; with a lot of scrolling around for shaders with lots of parameters. But sometimes parameters are related to each other (e.g. a multiplier and a color) which makes it desirable to be able to put multiple controls on the same row.</p>\n\
<p>To facilitate this, use the metadata <code>packName</code>. It decides which parmeters to &quot;pack&quot; into the same row. Any two or more parameters that have the same &quot;packName&quot; will show up on the same row, with that &quot;packName&quot; as the row header. To additionally size the columns as needed, the metadata <code>widgetWidth</code> allows some amount of adjustment of the relative size of the widgets, to keep the layout looking balanced.</p>\n\
<p>In the below example, on the left, the <code>Dark</code> and <code>Bright</code> parameters are on separate rows. But on the right, both of them have been given the <code>packName</code> of &quot;Dark / Bright&quot;, putting both on the same row, with that heading:</p>\n\
<p><img src=\"./images/packing.png\" alt=\"example of packing two parameters on one line\"></p>\n\
<h4 id=\"custom-widgets\">Custom Widgets</h4><p>Two custom widgets are supported, <strong>max:ramp0</strong> and <strong>max:actionButton</strong>.</p>\n\
<p><strong>max:ActionButton</strong></p>\n\
<p>The widget type &quot;max:actionButton&quot; must be applied to a string parameter. This will replace the normal string input field with a push button control with the string&apos;s value as the label. The additional <code>actionID</code> string metadata denotes the ID of a helper script that is run when the button is clicked. </p>\n\
<p>For security reasons, only predefined scripts are allowed (see the file OSLHelpers.ms), and most available functions are specific to some of the included shaders. However, there is a generic way to make dynamid multi-input shaders using this method, explained below.</p>\n\
<p><strong>max:ramp0</strong></p>\n\
<p>The widget type &quot;max:ramp0&quot; uses the same UI code as the Arnold ramp widget. This widget requires some additional metadata to work. There are three <em>required</em> string metadata items:</p>\n\
<ul>\n\
<li><code>interpolation</code> should give the name of an integer array of interpolation types. 0 = Constant, 1 = Linear, 2 = Catmull-Rom and 3 = Monotone Cubic.</li>\n\
<li><code>values</code> should give the name of an array of either float or color values (the type will determine the kind of ramp).</li>\n\
<li><code>position</code> should give the name of a float array giving the position along the ramp of each item.</li>\n\
</ul>\n\
<p>Each metadata is a string containing the name of another parameter which in turn must be an array of the appropriate type. Each array must be initialized, and all three arrays must have the same length. The &quot;max:ramp0&quot; widget type should be applied to the the interpolation array, and the widget type of the other two arrays should be &quot;null&quot; and have connectable=0 to hide the widget (although there are no widgets for arrays currently, other than these ramps) and disable connectivity respectively.</p>\n\
<p>See the source code of the <strong>Curves (color)</strong> shader for an example. </p>\n\
<h3 id=\"dynamic-ui\">Dynamic UI</h3><p>With the help of the &quot;max:actionButton&quot; shown above, a few helper scripts have been introduced that makes shaders dynamic - as in - the shader is actually <em>modifying its own sourcecode</em>!</p>\n\
<p>Most of the available actions are specific to some of the shipping shaders, but there is one generic action usable for any kind of shader you want to have a <em>variable number of inputs</em>. This action is named &quot;UpdateGenericLayering&quot;. It works as follows:</p>\n\
<ul>\n\
<li>The shader <em>must</em> have an integer parameter named exactly <code>Layers</code>, that is set to widget type &quot;null&quot; and <code>connectable=0</code>. The parameter must exist, even if the shader code never actually needs to use the parameter value itself.</li>\n\
<li>Two controls of widget type &quot;max:actionButton&quot; with the actions set to &quot;UpdateGenericLayering(1)&quot; to add a layer, and &quot;UpdateGenericLayering(-1)&quot; to remove a layer.</li>\n\
<li>Special <code>LAYER(x)</code> macros in the OSL shader code, and &quot;marker&quot; comments before (<code>//====LAYER-BEGIN====</code>) and after (<code>//====LAYER-END====</code>) the use of these macros.</li>\n\
<li>Between the begin and end markers, only a line that reads <code>LAYER(x)</code> must exist, expanding the macro, where x is the number, starting from 1 for the first layer and so on.</li>\n\
<li>Whatever the default value of the <code>Layers</code> parameter, exactly that many copies of the <code>LAYER(x)</code> macro should exist in the default version of the shader</li>\n\
</ul>\n\
<p>When the user pushes one of the two action buttons, the &quot;UpdateGenericLayering&quot; action will actually perform the following actions:</p>\n\
<ul>\n\
<li>Increase or decrease the parameter named <code>Layers</code>. It will never be allowed to be less than 1 or more than 99.</li>\n\
<li>&quot;Unlink&quot; the shader if linked, and modify the source code of the shader. It will find the begin / end marker lines, and insert as many copies of the <code>LAYER(x)</code> macro as the <code>Layers</code> parameter says. So if <code>Layers = 3</code>, it will insert <code>LAYER(1) LAYER(2) LAYER(3)</code> in the code.</li>\n\
</ul>\n\
<p>The rest is up to the shader code, and clever use of preprocessing macros (OSL uses the same preprocessor syntax as C/C++). Understanding preprocessing macros is beyond the scope of this documentation, but you have as many layer sections as you want by defining the <code>LAYER</code> macro before and undefining it after.</p>\n\
<p>A simple use case of the generic layering can be studied in the source of the switcher shader <strong>1-of-N (color)</strong>. A more complex use case is in the <strong>HDRI Lights</strong> shader.</p>\n\
<h3 id=\"fully-custom-ui-via-qt\">Fully Custom UI via QT</h3><p>The user interface for OSL now allows a custom layout to be provided in the form of a <code>.ui</code> file. </p>\n\
<p><strong>HDRI Environment</strong> shader with no custom UI (left) and the custom UI (right):</p>\n\
<p><img src=\"./images/env-ui.png\" alt=\"automatic vs custom ui\"></p>\n\
<p>If there is no <code>.ui</code> file, or if the shader was loaded into an earlier version of 3ds Max (or even another software entirely) it would look like on the left. But by supplying the file <code>HDRIEnviron.ui</code> in the same folder as <code>HDRIEnviron.osl</code>, we get the result on the right.</p>\n\
<p>To design custom UI, use the free <strong>Qt Designer</strong> app (downloadable from <a href=\"https://www.qt.io/download\" target=\"_blank\">https://www.qt.io/download</a>). </p>\n\
<p>Parmeter binding is <em>by name</em>, so by giving the widget in the <code>.ui</code> file the same <em>exact</em> name as the parameter in the OSL shader, the value is automatically connected. </p>\n\
<p>3ds Max uses some custom Qt widget classes, and it is very important to use the proper class names when designing the UI. The following Qt widget class names <em>must</em> be used:</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Parameter type</th>\n\
<th>Qt class name</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>int</code></td>\n\
<td>In the general case <code>MaxSDK::QMaxSpinBox</code> but if <code>widget = &quot;checkBox&quot;</code> any checkable Qt widget (e.g. <code>QCheckBox</code> or a <code>QGroupBox</code> with the attribute <code>checkable = true</code>).</td>\n\
</tr>\n\
<tr>\n\
<td><code>float</code></td>\n\
<td>In the general case a <code>MaxSDK::QMaxDoubleSpinBox</code> but if <code>worldunits = 1</code> then a <code>MaxSDK::QMaxWorldSpinBox</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>color</code></td>\n\
<td><code>MaxSDK::QMaxColorSwatch</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>point</code>, <code>vector</code>, <code>normal</code></td>\n\
<td><code>MaxSDK::QMaxPoint3Spinner</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>string</code></td>\n\
<td>In the general case a <code>MaxSDK::UI::QmaxDeferredLineEdit</code> but if <code>widget = &quot;filename&quot;</code> a <code>MaxSDK::UI::QmaxFileChooser</code>, and if <code>widget = &quot;max:actionButton&quot;</code>, a Qt button control (e.g. <code>QPushButton</code> or <code>QToolButton</code>).</td>\n\
</tr>\n\
<tr>\n\
<td>- <em>any</em> -</td>\n\
<td>Regardless of type, if the <code>widget = &quot;popup&quot;</code> or <code>widget = &quot;mapper&quot;</code>, use a <code>QComboBox</code>.  If <code>widget = &quot;max:ramp0&quot;</code>, a <code>QWidget</code></td>\n\
</tr>\n\
</tbody></table>\n\
<p>Every parameter (that does not have <code>connectable = 0</code>) also comes with a matching parameter with &quot;_map&quot; added at the end. The widget for this is either a plain <code>QPushButton</code> or <code>MaxSDK::UI::QmaxTexmapToolbutton</code> for the smaller &quot;mini&quot; buttons that only display an <strong>M</strong> when populated.</p>\n\
<p>Any paramater that exists in the <em>shader</em> but is not represented in the <code>.ui</code> file will be automatically added below the Custom UI. This assures that all parameters are visible even if they are missing from the <code>.ui</code> file. If this is not desirable <code>widget = &quot;null&quot;</code> (for parameters) or <code>connectable = 0</code> (for map inputs) can be used to hide them.</p>\n\
<p><strong>Tip</strong>: The simplest case to get all these widget types &quot;right&quot;, is to simply open up a few of the <code>.ui</code> files shipping with 3ds Max and cutting and pasting the widgets into your own layout and simply renaming the controls to match the parameters of your shader.</p>\n\
<h2 id=\"3ds-max-osl-shaders-on-github\">3ds Max OSL Shaders on GitHub</h2><p>There is a public GitHub for exchange of 3ds Max OSL shaders on the <a href=\"https://github.com/ADN-DevTech/3dsMax-OSL-Shaders\" target=\"_blank\">ADN Dev Tech Repository</a>. </p>\n\
<h2 id=\"osl-sdk-for-renderer-developers\">OSL SDK for Renderer Developers</h2><p>As a developer of a <em>renderer</em> you have a few choices when it comes to supporting OSL. If your renderer can call the classic <code>EvalColor</code>, <code>EvalMono</code> and <code>EvalNormalPerturb</code> funtions on a Texmap, you effectively have OSL support for free. </p>\n\
<p>But if your renderer translates the scene to another format, or comes with its own OSL support, or you want to implement your own raytracing for the OSL <code>trace()</code> call, there is a helper SDK available in the <code>MAXOSLInterface.h</code> header. This allows interrogating the OSL shaders for various things, reading the per-node data cache, and contains callbacks for <code>trace()</code>.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../rendering.html\">Rendering</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";