var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"obtaining_shader_parameters\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Obtaining Shader Parameters</h1></div><p>The <code>IParameterManager</code> provides access to the data acquired by the parsers. The following code snippet shows how to access an <code>IParameterManager</code> from an <code>IDxMaterial3</code> pointer obtained from an <code>Mtl</code> object.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Mtl * l_pMtl = l_pINode-&gt;GetMtl();\n\
IDxMaterial3 *l_pIDxMaterial3 = (IDxMaterial*)l_pMtl-&gt;GetInterface(IDXMATERIAL3_INTERFACE);\n\
IParameterManager *l_pIParameterManager = l_pIDxMaterial3-&gt;GetCurrentParameterManager();\n\
</code></pre><p>A specific parameter value can be accessed using the method <code>IParameterName::GetParameterByName</code>() :</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">texHandle = pEffect-&gt;GetParameterByName(NULL, paramName);</code></pre><p>The following code sample shows how to access data either by name or iteratively. The choice of approach depends on how the parser holds its internal data.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">switch(pm-&gt;GetParamType(i))\n\
{\n\
    case  IParameterManager::kPType_Float:\n\
    {\n\
        float fval;\n\
        pm-&gt;GetParamData((void*)&amp;fval,i);\n\
        pEffect-&gt;SetFloat(pm-&gt;GetParamName(i), fval);\n\
    }\n\
    break;\n\
\n\
    case  IParameterManager::kPType_Color:\n\
    case  IParameterManager::kPType_Point4:\n\
    {\n\
        D3DCOLORVALUE cval;\n\
        pm-&gt;GetParamData((void*)&amp;cval,i);\n\
        pEffect-&gt;SetVector(pm-&gt;GetParamName(i), (D3DXVECTOR4*)&amp;cval);\n\
    }\n\
    break;\n\
\n\
    case  IParameterManager::kPType_Bool:\n\
    {\n\
        BOOL bval;\n\
        pm-&gt;GetParamData((void*)&amp;bval,i);\n\
        pEffect-&gt;SetBool(pm-&gt;GetParamName(i), bval);\n\
    }\n\
    break;\n\
\n\
    case  IParameterManager::kPType_Int:\n\
    {\n\
        int ival;\n\
        pm-&gt;GetParamData((void*)&amp;ival,i);\n\
        pEffect-&gt;SetInt(pm-&gt;GetParamName(i), ival);\n\
    }\n\
    break;\n\
}</code></pre><p>As more shader types become available, materials must be able to display these in the File Open dialog. To accommodate this, new methods have been added to the <code>EffectDescriptor</code> class. The parser defines supported file extensions in the descriptor since the actual parser has not been loaded until the file has been accessed to determine which parser to load. File extensions are supplied as a localizable strings.</p>\n\
<p>The material is responsible for displaying any errors generated, so additional parser methods need to provide access to error buffers.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../programming_hardware_shaders.html\">Programming Hardware Shaders</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";