var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"shader_semantics_and_annotations\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Shader Semantics and Annotations</h1></div><p>Semantics are user supplied hints in a shader effect file, that describe a variable. The material uses semantics to define the transforms required by the effect, and also for what style of texture map is present.</p>\n\
<p>Annotations are provided as additional information about variables, and are used by 3ds Max to generate UI elements for hardware materials.</p>\n\
<p>In the following HLSL parser example, there are two semantics <code>Direction</code> and <code>LightColor</code>. The <code>Direction</code> semantic has three annotations: <code>UIName</code>, <code>Object</code>, and <code>RefID</code>. The <code>LightColor</code> semantic has two annotations <code>UIName</code> and <code>LightRef</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// light direction (view space)\n\
float3 lightDir : Direction &lt;  \n\
  string UIName = &quot;Light Direction&quot;;\n\
  string Object = &quot;TargetLight&quot;;\n\
  int RefID = 0;\n\
&gt; = {-0.577, -0.577, 0.577};\n\
\n\
//diffuse setting controlled by the light if available\n\
float4 k_d : LightColor &lt;\n\
  string UIName = &quot;Diffuse&quot;;\n\
  int LightRef = 0;\n\
&gt; = float4( 1.0f, 1.0f, 1.0f, 1.0f );    </code></pre><h2 id=\"parsers\">Parsers</h2><p>The supported semantics and annotations depend on the parser used. The parsers are separated into HLSL and SAS shaders for the purpose of the documentation. The parsers are identified by the <code>paramID</code> value in a shader effect file.</p>\n\
<ul>\n\
<li>0x0000 (default) - The original HLSL parser for 3ds Max.</li>\n\
<li>0x0001 - Microsoft Standard Annotations and Semantics (SAS) specification version 0.8.</li>\n\
<li>0x0002 - CgFX compliant parser using the DirectX profiles. The CgFX parser uses a subset of the SAS specification. The main difference of CgFX and SAS is that it does not support the scripting interfaces.</li>\n\
<li>0x0003 - Microsoft SAS specification version 0.8 with scene effect support.</li>\n\
</ul>\n\
<h2 id=\"about-standard-annotations-and-semantics-sas-\">About Standard Annotations and Semantics (SAS)</h2><p>For more information on the SAS specification see the NVIDIA site (<a href=\"http://www.nvidia.ca/object/using_sas.html\" target=\"_blank\">http://www.nvidia.ca/object/using_sas.html</a>). Note that 3ds Max does not support camera semantics and annotations, and has its own lighting setup. 3ds Max specific information can be added to an effects file using <code>\\#ifdef</code> / <code>\\#endif</code> blocks using the <code>\\_MAX_</code> predefined value. For more information see <a href=\"shader_semantics_and_annotations/supported_sas_shader_semantics.html\">Supported SAS Shader Semantics</a>.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
      <div class=\"relinfo relchildren\">\n\
        <strong>Pages in this section</strong>\n\
        <ul class=\"ullinks\">\n\
          <li class=\"link ulchildlink\"><a href=\"shader_semantics_and_annotations/supported_hlsl_shader_semantics.html\">Supported HLSL Shader Semantics</a></li>\n\
          <li class=\"link ulchildlink\"><a href=\"shader_semantics_and_annotations/supported_hlsl_shader_annotation.html\">Supported HLSL Shader Annotations</a></li>\n\
          <li class=\"link ulchildlink\"><a href=\"shader_semantics_and_annotations/supported_sas_shader_semantics.html\">Supported SAS Shader Semantics</a></li>\n\
          <li class=\"link ulchildlink\"><a href=\"shader_semantics_and_annotations/supported_sas_shader_annotations.html\">Supported SAS Shader Annotations</a></li>\n\
          <li class=\"link ulchildlink\"><a href=\"shader_semantics_and_annotations/supported_direct3d_texture_forma.html\">Supported Direct3D Texture Formats</a></li>\n\
        </ul>\n\
      </div>\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../programming_hardware_shaders.html\">Programming Hardware Shaders</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";