var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"bump_mapping\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Bump Mapping</h1></div><p>Bump mapping is a technique that enables a surface to appear wrinkled or dimpled without the need to model these depressions geometrically. Rather, the surface normal is perturbed according to the information in the <em>bump map</em> to create variations in the smooth surface.</p>\n\
<p>A 2D bump map is applied to the surface of an object. The space along the surface is called the UV space. The 2D bump map can be thought of as its own surface with bright areas as hills and dark areas as valleys. You can then define a gradient on the surface, which can be thought of as the <em>downhill</em> or <em>uphill</em> directions. This gradient of values gives a direction in UV space where you can perturb the surface normal. If you have a bump in a texture, for example a steep uphill, then you can perturb the normal to make it look like there is a steep hill. Given the gradient in UV space, you need to determine what direction in 3D (XYZ) space to perturb the corresponding normal. The <em>bump basis vectors</em> are used for this purpose. These are the vectors that represent the UVW axes of the texture in 3D and can be used to perturb the normal.</p>\n\
<p>To understand how this is done, look at the following sample code. The method responsible for returning the perturbed normal is <code>Texmap::EvalNormalPerturb()</code>. The code below is from <code>\\\\MAXSDK\\\\SAMPLES\\\\MATERIALS\\\\CHECKER.CPP</code>, but all the other 2D textures use a similar approach.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Point3 Checker::EvalNormalPerturb(ShadeContext&amp; sc)\n\
{\n\
   Point3 dPdu, dPdv;\n\
   if (!sc.doMaps) return Point3(0,0,0);\n\
   if (gbufID) sc.SetGBufferID(gbufID);\n\
\n\
   uvGen-&gt;GetBumpDP(sc,dPdu,dPdv);\n\
   Point2 dM = uvGen-&gt;EvalDeriv(sc,&amp;mysamp);\n\
\n\
   returnd M.x*dPdu+dM.y*dPdv;\n\
}</code></pre><p>The first significant line related to bump mapping is:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> uvGen-&gt;GetBumpDP(sc,dPdu,dPdv);</code></pre><p>This method of <code>UVGen</code> gets the bump vectors. You can also get the bump vectors directly from the method <code>ShadeContext::DPdUVW()</code>, although these are not be affected by the <code>UVGen</code> transformations.</p>\n\
<p>In this case, all the coordinates are coming through <code>UVGen</code>. You must use <code>GetBumpDP()</code> because the <code>UVGen</code> has rotated things around or has transformed the bump vectors to a new position. Basically, the <code>UVGen</code> has rotated the UV space into another position locally, so this method gets the bump basis vectors, which are really the U and V axes in 3D space (unit vectors in the U direction and the V direction, but in 3D space).</p>\n\
<p>The next line computes <code>dM</code>. This is the derivative of the function across the pixel. This is the rate of change of the function in the U direction (<code>dM</code>.x) and the V direction (<code>dM</code>.y). If this is a flat function, both of these are zero. If the function is increasing in U but is constant in V, then the value in the U direction corresponds to how fast it is changing while V is still zero. Thus, <code>dM</code> can be thought of as the gradient or how fast things are changing up and down.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> Point2 dM = uvGen-&gt;EvalDeriv(sc,&amp;mysamp);</code></pre><p>Next, you need to compute the perturbation to the normal. This can be thought of as a small vector that is added to the end of the existing normal to move it a little bit. There are several ways to do this. The common textbook algorithm (Blinn&apos;s algorithm for bump mapping) is not used by the 3ds Max textures. Rather, the calculation shown below is simpler and faster with no visual difference. To compute the perturbation to apply to the normal, the following code is used:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> return dM.x*dPdu+dM.y*dPdv;</code></pre><p>This takes the sum of the U component (<code>dM</code>.x) multiplied by the U basis vector (<code>dPdu</code>) and the V component (<code>dM</code>.y) multiplied by the basis vector in the V direction. This gives the change (perturbation) to the normal as a unit vector.</p>\n\
<p>The result of <code>EvalNormalPerturb()</code> (the perturbation to apply to the surface normal) is used by 3ds Max as follows: Outside the procedural texture, for example in the Standard material, the value returned is added to the surface normal. The normal is then re-normalized (made a unit vector again). This altered normal results in the surface appearing <em>bumped</em> when rendered.</p>\n\
<h2 id=\"tangents-and-bitangents-generation\">Tangents and Bitangents Generation</h2><p>The following example shows how the tangent and bitangent vectors are calculated in Nitrous, Quicksilver, and Scanline.</p>\n\
<h3 id=\"-em-calculating-tangents-and-bitangents-for-vertices-on-a-triangle-face-em-\"><em>Calculating Tangents and Bitangents for Vertices on a Triangle Face</em></h3><p>The inputs are <code>pos</code> (vertex positions), <code>norm</code> (vertex normals), and <code>uv</code> (texture coordinates).</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void CalculateTangentsBitangents(\n\
Point3 pos[3], Point3 norm[3], Point3 uv[3],\n\
Point3 tangent[3], Point3 bitangent[3])\n\
{\n\
    float uv1x = uv[1].x - uv[0].x;\n\
    float uv2x = uv[2].x - uv[0].x;\n\
    float uv1y = uv[1].y - uv[0].y;\n\
    float uv2y = uv[2].y - uv[0].y;\n\
    float uvk = uv2x * uv1y - uv1x * uv2y;\n\
\n\
    Point3 v1 = pos[1] - pos[0];\n\
    Point3 v2 = pos[2] - pos[0];\n\
\n\
    Point3 faceTangent;\n\
    if (uvk != 0) {\n\
       faceTangent = (uv1y * v2 - uv2y * v1) / uvk;\n\
    } else {\n\
       if (uv1x != 0) faceTangent = v1 / uv1x;\n\
       else if (uv2x != 0) faceTangent = v2 / uv2x;\n\
       else faceTangent = Point3(0.0f, 0.0f, 0.0f);\n\
    }\n\
    Normalize(faceTangent);\n\
\n\
    Point3 mapNormal = CrossProduct(uv[1] - uv[0], uv[2] - uv[1]);\n\
    bool flip = mapNormal.z &lt; 0;\n\
\n\
    for (int i = 0; i &lt; 3; ++i)\n\
    {\n\
     // Make tangent perpendicular to normal\n\
     tangent[i] = faceTangent - DotProduct(norm[i], faceTangent) * norm[i];\n\
     Normalize(tangent[i]);\n\
\n\
     bitangent[i] = CrossProduct(norm[i], tangent[i]);\n\
     if (flip) bitangent[i] = -bitangent[i];\n\
    }\n\
}</code></pre><h3 id=\"smoothing\">Smoothing</h3><p>If a vertex is shared among multiple faces, and the texture coordinate and smooth group of the vertex are the same among all the faces, the tangents and bitangents calculated for this vertex on each face is accumulated and normalized to form the final tangent and bitangent of the vertex.</p>\n\
<h3 id=\"normal-bump-modes\">Normal Bump Modes</h3><p>3ds Max supports the following normal bump modes: <strong>3ds Max Mode</strong>, <strong>Maya Mode</strong>, and <strong>DirectX Mode</strong>. You can select a normal bump mode in the <strong>General</strong> tab of the <strong>Preferences Settings</strong> dialog, which can be accessed by selecting <strong>Customize <img src=\"./images/ac.menuaro.gif\" alt> Preferences</strong>. The calculation of tangents and bitangents vary based on the normal bump mode.</p>\n\
<p>The following figure shows the three normal bump modes in the <strong>Preferences Settings</strong> dialog.</p>\n\
<p><img src=\"./images/GUID-15E413F1-7D03-4FFD-833B-2EC5562051FE-low.png\" alt></p>\n\
<p>These normal bump modes are explained in the following table.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Mode</th>\n\
<th>Description</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td>*<em>3ds Max Mode *</em></td>\n\
<td>The face normal that is always perpendicular to the triangle face is used in the calculation. During pixel shading, the tangent and bitangent interpolated from the triangle vertices are used in the normal bump calculation.</td>\n\
</tr>\n\
<tr>\n\
<td>*<em>Maya Mode *</em></td>\n\
<td>The vertex normal that considers smooth group and which can be tuned using the Edit Normals modifier is used in the calculation. During pixel shading, the tangent and bitangent interpolated from the triangle vertices are orthogonalized against the normal before the normal bump calculation.</td>\n\
</tr>\n\
<tr>\n\
<td>*<em>DirectX Mode *</em></td>\n\
<td>The vertex tangent and bitangent are perpendicular to the vertex normal. During pixel shading, the tangent and bitangent interpolated from the triangle vertices are used in the normal bump calculation.</td>\n\
</tr>\n\
</tbody></table>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../textures_maps.html\">Textures Maps</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";