var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"creating_bitmaps\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Creating Bitmaps</h1></div><p>To create a bitmap from scratch, we need to declare a pointer to point to it:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Bitmap *bmap;</code></pre><p>Remember that the <code>Bitmap</code> class represents the bitmap itself. Next, we need to declare an instance of the <code>BitmapInfo</code> class to describe the properties of the bitmap to create:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">BitmapInfo bi;</code></pre><p>Then we initialize the <code>BitmapInfo</code> with the properties of the bitmap we wish to create. This is done using the methods of <code>BitmapInfo</code> such as <code>SetType()</code>, SetWidth(), etc.:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> // Initialize the BitmapInfo instance\n\
 bi.SetType(BMM_TRUE_64);\n\
 bi.SetWidth(320);\n\
 bi.SetHeight(200);\n\
 bi.SetFlags(MAP_HAS_ALPHA);\n\
 bi.SetCustomFlag(0);</code></pre><p>Note in the above code how the type is set. The type describes the number of bits per pixel used to describe the image, whether the images is paletted (has a color map), and if it has an alpha channel. To see a list of the types of bitmaps that may be created see <code>List of Bitmap Types</code>. In this example, we use <code>BMM_TRUE_64</code>. This format has 16-bits for each color (RGB) and alpha component. This is the format that 3ds Max uses internally in the renderer.</p>\n\
<p>Once the <code>BitmapInfo</code> is initialized, we can call a method of the bitmap manager to create the bitmap. A global instance of the <code>BitmapManager</code> class exists called <code>TheManager</code>. This is what we use to call methods of <code>BitmapManager</code> as shown below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> // Create a new bitmap\n\
 bmap = TheManager-&gt;Create(&amp;bi);</code></pre><p>Note that we pass it our <code>BitmapInfo</code> instance. This is where the bitmap manager gets the information on the bitmap to create. If the pointer returned is <code>NULL</code>, an error has occurred in creating the bitmap. If the pointer is non-NULL, its valid and we can use it to call methods of the <code>Bitmap</code> class to work with the bitmap. The code below shows how the <code>PutPixels()</code> method is used to write data to the bitmap. This code sets every pixel of the image to the color and opacity specified by <code>r</code>, g, b and <code>alpha</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">//...\n\
BMM_Color_64 *line, *lp, color = {r, g, b, alpha};\n\
int bmapWidth = bmap-&gt;Width(), bmapHeight = bmap-&gt;Height();\n\
if ((line = ( BMM_Color_64 *)calloc(bmapWidth, sizeof( BMM_Color_64))))\n\
{\n\
   intix, iy;\n\
   for(ix = 0, lp = line; ix &lt; bmapWidth; ix++, lp++)\n\
     *lp = color;\n\
   for(iy = 0; iy &lt; bmapHeight; iy++)\n\
     int res = bmap-&gt;PutPixels(0, iy, bmapWidth, line);\n\
   free(line);\n\
}</code></pre><p>When we are done using the bitmap, we need to delete it. This is done by simply using the delete operator:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">if (bmap) bmap-&gt;DeleteThis();\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../working_with_bitmaps.html\">Working With Bitmaps</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";