var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"memory_management_for_plug-ins_t\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Memory Management for Plug-ins that Work with Bitmaps</h1></div><h2 id=\"bitmap-memory-management\">Bitmap Memory Management</h2><p>Memory is allocated and de-allocated by the bitmap classes in various ways. It is important to understand when memory needs to be freed by the developer, and when the system will take care of freeing it. This section discusses this issue.</p>\n\
<p>The bitmap manager methods <code>BitmapManager::Create()</code> and <code>BitmapManager::Load()</code> both return pointers to bitmaps that need to be freed when the developer is done working with them. This is accomplished by simply using the delete operator on the pointer returned from the methods. The pseudo code below shows both of these cases.</p>\n\
<p>This is the <code>BitmapManager::Create()</code> case:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Bitmap *bmap;\n\
BitmapInfo bi;\n\
\n\
bmap = TheManager-&gt;Create(&amp;bi);\n\
// Work with the bitmap...\n\
// ...\n\
// Free the bitmap when done\n\
if (bmap) bmap-&gt;DeleteThis();</code></pre><p>This is the <code>BitmapManager::Load()</code> case:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">BMMRES status;\n\
Bitmap *bmap;\n\
BitmapInfo bi;\n\
\n\
BOOL res = TheManager-&gt;SelectFileInput(&amp;bi, ip-&gt;GetMAXHWnd(), _T(&quot;Open File&quot;));\n\
bmap = TheManager-&gt;Load(&amp;bi, &amp;status);\n\
// Work with the bitmap...\n\
// ...\n\
// Free the bitmap when done\n\
if (bmap) bmap-&gt;DeleteThis();</code></pre><p>As the code above shows, the developer is responsible for freeing memory from both these methods.</p>\n\
<p>Another example of getting a pointer to a bitmap and needing to free it is the pointer returned from the method Bitmap::ToDib(). This method creates a new Windows Device Independent Bitmap (DIB) and returns a pointer to it. The DIB bitmap is created from the bitmap whose method is called. The DIB is allocated internally using <code>Bitmap::LocalAlloc()</code> and must be freed by the developer using <code>Bitmap::LocalFree()</code>. The pseudo-code below show how this is done.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">PBITMAPINFOpDib;\n\
\n\
pDib = bmap-&gt;ToDib();\n\
// Work with the bitmap...\n\
// ...\n\
// Free the bitmap when done\n\
LocalFree(pDib);</code></pre><p>There is a different case where the developer receives a pointer to a bitmap, but is not responsible for deleting it when done. If you have a Image Filter plug-in, it receives a pointer to the video post bitmap queue. This bitmap should not be deleted as the video post system is using it internally. For example, in the <code>ImageFilter::Render()</code> method, the filter plug-in has access to a source bitmap named <code>srcmap</code>. The methods of this pointer may be called, but the bitmap itself should not be deleted by the plug-in. Video post will take care of it.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../working_with_bitmaps.html\">Working With Bitmaps</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";