var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"monitoring_the_node_events\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Monitoring the Node Events</h1></div><p>You can monitor the node events in a scene using one of these methods:</p>\n\
<ul>\n\
<li><p>Reference System - The reference system is an implementation of the subject-observer pattern. Objects (observers) that must be notified of changes to target objects (subjects) are called reference makers and are derived from the <code>ReferenceMaker</code> class. The target object is called the reference target and is derived from the <code>ReferenceTarget</code> class. Reference relationships are not represented by a specific class, but are managed by methods of the <code>ReferenceMaker</code> class. For example, if a node is deleted from the scene, the reference maker that references the deleted node will receive the <code>REFMSG_TARGET_DELETED</code> message through the <code>ReferenceMaker::NotifyRefChanged()</code> method, and it can respond appropriately to the node deletion event. Obtaining complete information about each node in a scene requires managing several references, which can be complex and in this case using one of the other alternative systems is recommended.</p>\n\
<p>See <a href=\"./../../reference_system.html\">Reference System</a> for more information.</p>\n\
</li>\n\
<li><p>Scene Event System - The scene event system provides a simplified mechanism for monitoring all nodes in a scene simultaneously using the <code>ISceneEventManager</code> class. This system supports a broader range of messages than the other systems, but provides fewer details regarding single events. It is useful in situations that require high-level information about the node changes and where precision timing of messages is not a priority. For example, it is an ideal method to use for monitoring the scene to trigger a UI refresh.</p>\n\
<p>See <code>ISceneEventManager</code> class for more information</p>\n\
</li>\n\
<li><p>Broadcast Notification System - The broadcast notification system uses callback functions when certain events occur. The header file <code>notify.h</code> contains the list of notification event IDs that can be handled.</p>\n\
<p>The following example shows how to handle a node deletion event by registering a notification callback with the broadcast notification system. When a node is deleted, the <code>NOTIFY_SCENE_PRE_DELETED_NODE</code> and <code>NOTIFY_SCENE_POST_DELETED_NODE</code> messages are sent with a <code>callParam</code> containing an <code>INode*</code>. The following code snippet demonstrates how to register a callback to handle the <code>NOTIFY_SCENE_PRE_DELETED_NODE</code> message.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">//Declare the callback function  \n\
static void NodeDeleteNotify(void *param, NotifyInfo *info)\n\
{\n\
  // Get the nodes being deleted\n\
  INode* deleted_node = (INode*)info-&gt;callParam;\n\
\n\
  // Do custom processing\n\
   ...\n\
}\n\
\n\
// Register the callback\n\
RegisterNotification(NodeDeleteNotify, this, NOTIFY_SCENE_PRE_DELETED_NODE);\n\
\n\
...\n\
\n\
// When done, unregister the callback\n\
UnRegisterNotification(NodeDeleteNotify, this, NOTIFY_SCENE_PRE_DELETED_NODE);</code></pre><p>See <a href=\"./../../broadcast_notification_system.html\">Broadcast Notification System</a> for more information.</p>\n\
</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../nodes.html\">Nodes</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";