var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"deriving_from_restoreobj\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Deriving from RestoreObj</h1></div><p>A restore object must be defined by the developer that derives from RestoreObject. An instance of this class must be registered with the <code>theHold</code> global object by passing it to the <code>Hold::Put()</code> method. The restore object should save just enough data to restore the current scene state to where it was before it was modified by an action. In other words, the restore object must restore the state of the scene to the one it was in when <code>theHold</code>. Put() was called with this restore object because other actions may have occurred since <code>Begin()</code> was called. The restore object also saves data to allow a redo operation to occur.</p>\n\
<p>In addition to saving the data it needs to restore the scene state, the restore object has three main methods it must implement:</p>\n\
<ul>\n\
<li><code>RestoreObj::Restore()</code></li>\n\
<li><code>RestoreObj::Redo()</code></li>\n\
<li><code>RestoreObj::Size()</code></li>\n\
</ul>\n\
<h2 id=\"restoreobj-restore-int-isundo-\">RestoreObj::Restore(int isUndo)</h2><p>This is called to restore the scene state to the state it was in when <code>theHold</code>. Begin() was called. In the earlier example, the <code>Restore()</code> method would reset the wireframe color to the previous color. A flag, <code>isUndo</code>, is passed into this method to indicate if it was called in response to the Undo command. In the above example, if the flag is TRUE the developer must save the current color before restoring the previous color so if the user selects the Redo command the color could be reset to the state before the undo.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> Regardless of the value of the <code>isUndo</code> parameter, <code>Restore()</code> must restore the state of the scene to the one it was in when <code>theHold</code>. Put() was called with this restore object.</div>\n\
<p>The <code>Restore()</code> method of the restore object can be called without the user selecting the Undo command. The undo system also allows values to be held in a buffer and then restored programmatically. For example, consider the operation of moving a node in the scene using the mouse. When the user first clicks the mouse button down to begin the operation, the state of the node&apos;s position is saved as part of a restore object. As the user moves the mouse, the plug-in tracks its position and updates the position of the node in the scene. The plug-in receives repeated messages that the mouse has moved and the node in the scene must be re-positioned. Rather than storing incremental moves from the previously calculated position (which can accumulate error) it is more accurate to restore the mouse position to where it started and recalculate the transformation based on the distance from the original mouse point to the new mouse point. To reset the original position, the system calls <code>Restore()</code>. to put the node back into its original position. Then when the new position is calculated, this is applied to the node. This happens repeated until the user releases the mouse. When the mouse is released, <code>theHold</code>. Accept() is called to register an undo object with the system.</p>\n\
<p>In iterative operations such as the one described above, it is often useful to set one of the flags of Animatable to indicate that a restore object is being held. When the user first clicks down on the mouse, the developer checks if <code>theHold</code> is holding and if it is calls <code>theHold</code>. Put() to register a restore object. Then the developer calls a method of Animatable <code>SetAFlag</code>(<code>A_HELD</code>). Note, that the flag is set on the object being worked with, <em>NOT</em> the restore object. This sets the <code>A_HELD</code> bit of the Animatable <code>aflag</code> data member to indicate the restore object is held. Then on each iteration the bit is tested to see if it is set and if so another restore object is not registered. A single restore object can be restored repeatedly in this manner.</p>\n\
<p>When <code>theHold</code>. Accept() or <code>theHold</code>. Cancel() is called, the system calls a method of the restore object called <code>EndHold()</code>. The developer may then clear the <code>A_HELD</code> bit to indicate the restore object is no longer being held.</p>\n\
<h2 id=\"restoreobj-redo-\">RestoreObj::Redo()</h2><p>This is called by the system if the user selects the Redo command. The developer restores the scene state to the state it was in before the last undo was called on the restore object.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> <code>RestoreObj::Restore</code> and <code>RestoreObj::Redo</code> can be called multiple times in succession, therefore these methods must be prepared for such an eventuality. Specifically, they must guard against multiple undo attempts that have been undone already.</div>\n\
<h2 id=\"restoreobj-size-\">RestoreObj::Size()</h2><p>This is called by the system to retrieve the size in bytes of the restore object. This is used to make sure all the accumulated restore objects to not grow beyond a manageable size. Even though this is not pure virtual, you should override the method.</p>\n\
<h3 id=\"example\">Example</h3><p>The following example demonstrates an implementation of a restore object and its registration with undo system.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class NodeRest : public RestoreObj {\n\
   public:\n\
     Orient offs;\n\
     Orient redoOffs;\n\
     Node *cur;\n\
     NodeRest() { cur = NULL; }\n\
     NodeRest(Node *node) {\n\
      cur = node;\n\
      offs = node-&gt;modOffset;\n\
     }\n\
     ~NodeRest() {}\n\
     void Restore(int isUndo);\n\
     void Redo();\n\
     int Size() { return 2*sizeof(Orient) + sizeof(INode*); }\n\
     void EndHold() { cur-&gt;ClearAFlag(A_HELD); }\n\
     virtual TSTR Description() { return TSTR(_T(&quot; Node State&quot;)); }\n\
};\n\
\n\
void Node::HoldState() {\n\
   if (theHold.Holding() &amp;&amp; !TestAFlag(A_HELD))\n\
   {\n\
     theHold.Put(new NodeRest(this));\n\
     SetAFlag(A_HELD);\n\
   }\n\
}\n\
\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../the_undo_and_redo_system.html\">The Undo and Redo System</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";