var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"super-holds\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Super-holds</h1></div><p>The methods <code>Hold::SuperBegin()</code>, <code>Hold::SuperAccept()</code>, and <code>Hold::SuperCancel()</code> together form the concept of a super-hold. This allows a developer to group a set of <code>Hold::Begin()</code> / <code>Hold::Accept()</code> sequences to be undone in one operation.</p>\n\
<p>Consider the case of the user using the Shift-Move command to create a new node in the scene. There are two parts to this process. First the node must be cloned and second the position must be tracked as the user moves the mouse to place the new node in the scene. Naturally if the user wanted to Undo this operation, they would expect a single selection of the Undo command would accomplish it. However the process was not a single operation. There was the initial cloning of the node, and then the iterative process of moving the node in the scene, restoring its position, moving it again, restoring it again, etc. Using a super-hold these actions can be grouped together. Note that in this example it is only necessary to use <code>Hold::SuperBegin()</code> and <code>Hold::SuperAccept()</code> because the move was restoring interactively. Normally a developer does NOT need to use these methods even if they have several operations that modify the database. The undo system will automatically register all the restore objects needed as part of the undo object when <code>Hold::Accept()</code> is called and these may be undone using a single undo command.</p>\n\
<p>The sample program  <em>/maxsdk/samples/objects/bones.cpp</em>  demonstrates the usage of super-holds.</p>\n\
<h2 id=\"super-hold-example\">Super-Hold Example</h2><p>In the following code, the <code>SuperHoldControl</code> class is instanced, and then, if the system is already holding, you must start a super-begin or begin and add some restore objects. If the operation is successful, you then call <code>Accept()</code> on the instance, which performs an accept or super accept. If the operation is not successful, or an exception is thrown here or in <code>addVar</code> or <code>assignObject</code>, a cancel or super cancel is performed. The string for accept and cancel is <code>NULL</code> because we are already inside a begin/accept block, so our string would never appear in the undo and redo drop-down control.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// helper class used to control the start/cancel/accept of a superhold.\n\
class SuperHoldControl {\n\
public:\n\
   SuperHoldControl() : started(false) { }\n\
   ~SuperHoldControl() { Cancel(); }\n\
   void Begin()\n\
   {\n\
     if (theHold.Holding())\n\
     {\n\
      theHold.SuperBegin();\n\
      theHold.Begin();\n\
      started = true;\n\
     }\n\
   }\n\
   void Cancel()\n\
   {\n\
     if (started)\n\
     {\n\
      theHold.Cancel();\n\
      theHold.SuperCancel();\n\
      started = false;\n\
     }\n\
   }\n\
   void Accept()\n\
   {\n\
     if (started)\n\
     {\n\
      theHold.Accept(NULL);\n\
      theHold.SuperAccept(NULL);\n\
      started = false;\n\
     }\n\
   }\n\
private:\n\
   bool started;\n\
};\n\
\n\
SuperHoldControl shc;\n\
\n\
if (theHold.Holding())\n\
{\n\
   shc.Begin();\n\
   theHold.Put(new ScriptInvalidateRestore(this));\n\
   theHold.Put(new ScriptVarRestore(this));\n\
}\n\
\n\
int slot = getVarIndex(name);\n\
\n\
if (slot &gt;= 0 &amp;&amp; slot &lt; NUM_READONLY_VARS) {\n\
   if (bThrowOnError)\n\
     throw RuntimeError(GetString(IDS_SCRIPT_CTRL_ADDOBJECT_VAR_READONLY),name);\n\
   returnFALSE;\n\
}\n\
if (slot &gt;= 0)\n\
{\n\
   BOOL res = assignObject(obj, slot);\n\
   if (res)\n\
     shc.Accept();\n\
   return res;\n\
}\n\
\n\
slot = addVar(name);\n\
BOOL res = assignObject(obj, slot);\n\
\n\
if (res)\n\
   shc.Accept();\n\
else\n\
   deleteVar(slot);\n\
\n\
return res;\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../the_undo_and_redo_system.html\">The Undo and Redo System</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";