var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"tips_and_tricks\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2024/ENU/Max-Developer-Help/scripts/prettify/lang-csharp.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Tips and Tricks</h1></div><h4 id=\"this-section-contains-a-set-of-useful-tips-and-tricks-for-working-with-the-3ds-max-sdk-\">This section contains a set of useful tips and tricks for working with the 3ds Max SDK.</h4><ul>\n\
<li><p><em>Accessing the Parameter Block of Any Plug-in</em> - If a plug-in does not explicitly provide access to its parameter blocks, you can still access it by iterating over the references. The following returns a pointer to the first parameter block of any plug-in:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">IParamBlock2* GetParamBlock(ReferenceMaker* obj) {\n\
    int nRefs = obj-&gt;NumRefs();\n\
    for ( int i = 0; i &lt; nRefs; ++i ) {\n\
        ReferenceTarget* ref = obj-&gt;GetReference(j);\n\
    if ( ref &amp;&amp; ref-&gt;SuperClassID() == PARAMETER_BLOCK2_CLASS_ID ) {\n\
        returndynamic_cast&lt;IParamBlock2*&gt;( ref );\n\
    }\n\
    }\n\
    return NULL;\n\
}</code></pre>\n\
</li>\n\
<li><p><em>Referencing a Global Instance of a Non-plug-in Class</em> - If a plug-in needs to contain some global instance that its member objects refer to, but the plug-in itself is not of a specific type, then the super class ID of <code>REF_TARGET_CLASS_ID</code> can be used. The class of the global instance can be derived from <code>ReferenceTarget</code>.</p>\n\
</li>\n\
<li><p><em>Executing Code in Main Thread</em> - You can execute code safely in the main thread of 3ds Max by either creating a Windows timer object using the main 3ds Max window as the window handle, or by using the following function to trigger code when 3ds Max is next idle.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">#define WM_TRIGGER_CALLBACK WM_USER+4764\n\
void PostCallback( void (*funcPtr)(UINT_PTR), UINT_PTR param )\n\
{\n\
    PostMessage( GetAppHWnd(), WM_TRIGGER_CALLBACK, (UINT_PTR)funcPtr, (UINT_PTR)param );\n\
}</code></pre>\n\
<p>In .NET, the System.Windows.Dispatcher class can be used to achieve the same effect. See MSDN (<a href=\"http://msdn.microsoft.com/en-us/library/system.windows.threading.dispatcher.aspx\" target=\"_blank\"></a>) for more information about this class.</p>\n\
</li>\n\
<li><p><em>Executing a Python Script from a .NET Assembly</em> - After downloading and installing IronPython, you can execute a Python script from a .NET assembly with a few lines of code as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-csharp prettyprint\">using IronPython.Hosting;\n\
using Microsoft.Scripting.Hosting;\n\
public static void RunPythonFile(string filename) {\n\
    try  {\n\
        var options = new Dictionary&lt;string, object&gt;();\n\
        options[&quot;Debug&quot;] = true;\n\
        ScriptEngine se = Python.CreateEngine(options);\n\
        ScriptSource ss = se.CreateScriptSourceFromFile(filename);\n\
        CompiledCode cc = ss.Compile();\n\
        cc.Execute();\n\
    }\n\
    catch (Exception e){\n\
    MessageBox.Show(&quot;Error occurred: &quot; + e.Message);\n\
    }\n\
}</code></pre>\n\
</li>\n\
<li><p><em>Drag and Drop Functionality</em> - You can extend 3ds Max with drag and drop functionality by deriving from the <code>DragAndDropHandler</code> class.</p>\n\
</li>\n\
<li><p><em>Downloading from a URL</em> - You can download a file referenced by a URL to disk using the <code>IDragAndDropMgr::DownloadUrlToDisk()</code> method.</p>\n\
</li>\n\
<li><p><em>Logging Messages</em> - You can log messages to a log file and print messages to the Visual Studio console using the <code>LogSys</code> class. 3ds Max maintains a log file that contains the error, warning, information, and debug messages generated by the system and plug-ins. The log file is placed in the  <em>Network</em>  directory and is named  <em>Max.Log</em> . The <code>LogSys</code> class is used for sending messages to the log.</p>\n\
</li>\n\
<li><p><em>Evaluate Mathematical Expressions</em> - You can evaluate mathematical expressions stored as strings using the <code>Expr</code> class. The expressions can consist of operators (+, -, *, /, and others), literal constants (numbers like 180, 2.718, and others), variables (single floating point values or vector (<code>Point3</code>) values), and functions (mathematical functions that take one or more arguments and return a result). The return value from the expression might be a floating point value or a vector. There are many built-in functions, operators and constants available for use.</p>\n\
</li>\n\
<li><p><em>Pseudo-random Number Generator</em> - You can generate repeatable sequences of random numbers using the <code>Random</code> class. This class defines a pseudo-random number generator that precisely matches the behavior of the MSVCRT 6.0 random routines.</p>\n\
</li>\n\
<li><p><em>Container Templates</em> - You can store objects in one of the following container template types: <code>Stack</code>, <code>LinkedList</code>, <code>Array</code>, and <code>Tab</code>.</p>\n\
</li>\n\
<li><p><em>Reading and writing Unicode files</em> - You can read and write Unicode files (see <a href=\"./../what_s_new/archive/what_s_new_3ds_max_2013_sdk/unicode.html\">Unicode</a>) easily and correctly using the <code>FileReaderWriter</code> class.</p>\n\
</li>\n\
<li><p><em>Compute the Time to Complete a Task</em> - You can compute the time taken to complete a task using the <code>Timer</code> class. The <code>EndTimer</code> method returns the time interval that has passed in milliseconds since the <code>StartTimer</code> method was used to start the timer.</p>\n\
</li>\n\
<li><p><em>Automatically Delete the Allocated Objects when Done</em> - Objects can be deleted automatically when you are done with them using the <code>AutoPtr</code> template. The object pointed to by the <code>AutoPtr</code> is deleted when the <code>AutoPtr</code> goes out of scope. <code>AutoPtr</code> helps prevent memory leaks and leads to exception safe code.</p>\n\
</li>\n\
<li><p><em>Display Status Messages</em> - You can display unobtrusive status messages to the user using the <code>Interface::PushPrompt()</code> function. See also <a href=\"maxscript_scripting_engine/useful_maxscript_tips.html\">Useful MAXScript Tips</a>.</p>\n\
</li>\n\
<li><p><em>Custom Colors for UI Elements</em> - You can add and retrieve user defined colors for the UI elements using the <code>IColorManager</code> class.</p>\n\
</li>\n\
<li><p><em>Launch a Web Browser</em> - You can use the <code>IBrowserMgr</code> class to launch a web browser.</p>\n\
</li>\n\
<li><p><em>Access File Properties</em> - You can retrieve the properties of a .MAX file using <code>Interface11::OpenMaxStorageFile()</code>. A max file is stored as a structured storage file and the property values can be accessed through the standard structured storage property interfaces. See MSDN (<a href=\"http://msdn.microsoft.com/en-us/library/windows/desktop/aa380369%28v=vs.85%29.aspx\" target=\"_blank\"></a>) for more information about structured storage.</p>\n\
</li>\n\
<li><p><em>Format a Time Value as a String</em> - You can format a time value as a string using <code>Interface10::FormatRenderTime()</code>.</p>\n\
</li>\n\
<li><p><em>Open Windows Explorer</em> - You can open the windows explorer to a specific folder or file path using <code>Interface8::RevealInExplorer()</code>. If the file path does not exist, a warning dialog appears and the Windows Explorer is not opened. The Windows Explorer is independent of the calling application, which means that it does not close when 3ds Max is closed.</p>\n\
</li>\n\
<li><p><em>Quick Render</em> - You can perform a quick render to a bitmap using <code>Interface8::QuickRender()</code>.</p>\n\
</li>\n\
<li><p><em>Execute Code in C#</em> - You can execute code in C# after 3ds Max is fully initialized by registering a callback with the notification listener as shown in the following example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-csharp prettyprint\">using ManagedServices;\n\
public class My3dsMaxAssembly\n\
{\n\
    public const int StartUpNotification = 0x50;\n\
    public static void AssemblyMain()\n\
    {\n\
    // This causes an event to be raised once the 3ds Max application has been started \n\
    var m = new MaxNotificationListener(StartUpNotification);\n\
    m.NotificationRaised += new EventHandler&lt;MaxNotificationEventArgs&gt;(AfterStartup); \n\
    } \n\
    public static void AfterStartup(object sender, MaxNotificationEventArgs e) { \n\
    if (e.NotificationCode == StartUpNotification) { \n\
        // Do whatever \n\
    } \n\
    } \n\
} </code></pre>\n\
</li>\n\
<li><p><em>Generate C++ Code from Mesh Data</em> - If you create a mesh using 3ds Max and wish to capture the data as a sequence of C++ instructions for manually constructing the mesh can be very time consuming if the object contains a large number of vertices.</p>\n\
<p>The MAXScript script found in the SDK under  <em>howto\\objects\\widget\\mesh2cpp.ms</em>  can be used to write all the mesh generating instructing information to a header (.h) file that you can include in your project. The script can be found Here is how to use the script:</p>\n\
<ol>\n\
<li>Open the script and evaluate it. This will compile the script and create an action entry in the 3dsmax action system.</li>\n\
<li>In the Customize &gt; Customize User Interface ... dialog, look up the command Mesh2CPP under the &apos;SDK Tools&quot; category.</li>\n\
<li>Assign this command to a button or hotkey.</li>\n\
<li>Select your object in the viewport.</li>\n\
<li>Run the script. A Save As dialog will appear.</li>\n\
<li>Specify the header filename and select Save.</li>\n\
</ol>\n\
<p>This will generate a header (.h) file that you can copy or include into the source of your plug-in.</p>\n\
</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../3ds_max_sdk_features.html\">3ds Max SDK Features</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";