var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"building_action_tables\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Building Action Tables</h1></div><p>This section discusses the approach developers may use to build action tables and make them available to the system. In most cases building an action table is fairly easy. There is more work involved if you choose to implement your own class derived from <code>ActionItem</code>, but in most cases that isn&apos;t needed.</p>\n\
<p>An action table is built by making a static table of action descriptions ( <code>ActionDescription</code> ) and passing it to the constructor for <code>ActionTable</code>. For example, here is the code that builds the action table for the <em>maxMenuDemo</em> HowTo sample:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// This packages up our actions into an ActionDescription for our ActionTable\n\
#define NumElements(array) (sizeof(array)/sizeof(array[0]))\n\
static ActionDescription spActions[] = {\n\
    ID_MENU1, // ID\n\
    IDS_MENU1, // menu1\n\
    IDS_MENU1, \n\
    IDS_CATEGORY,\n\
\n\
    ID_MENU2, \n\
    IDS_MENU2,\n\
    IDS_MENU2,\n\
    IDS_CATEGORY\n\
};\n\
\n\
// Finally, a method that registers and returns our ActionTable. \n\
ActionTable* GetActions() {\n\
    TSTR name = _T(&quot;SDKDocsActionTable&quot;);\n\
    HACCEL hAccel = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDR_ACCELERATOR1));\n\
    int numOps = NumElements(spActions);\n\
    ActionTable* pTab;\n\
    pTab = new ActionTable(kMyActions, kMyContext, name, hAccel, numOps,\n\
        spActions, hInstance);\n\
    GetCOREInterface()-&gt;GetActionManager()-&gt;RegisterActionContext(kMyContext, name.data());\n\
    return pTab;\n\
}</code></pre><p>The constructor for <code>ActionTable</code> takes the ID of the table, the context id, a name for the table, a windows accelerator table that gives default keyboard assignments for the operations, the number of items, the table of operation descriptions, and the instance of the module where the string resources in the table are stored.</p>\n\
<p>At the same time the table is built, you also need to register the action context ID with the system. This is done with the <code>IActionManager::RegisterActionContext()</code> method.</p>\n\
<p>The next part of implementing an <code>ActionTable</code> is to derive a class from <code>ActionCallback</code>. This is an abstract class. You will need to pass an instance of the derived class to 3ds Max when you activate the <code>ActionTable</code> by calling <code>IActionManager::ActivateActionTable()</code>. Then when the system wants to execute an action, you will get a callback to <code>ActionCallback::ExecuteAction()</code>.</p>\n\
<p>For the <em>maxMenuDemo</em> sample, this looks like:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class ExampleActionCB : public ActionCallback {\n\
public:\n\
    ExampleActionCB(){};\n\
    BOOL ExecuteAction(int id);\n\
};\n\
\n\
// This is where the callbacks are executed.  Here we simply print a message to the status prompt.\n\
BOOL ExampleActionCB::ExecuteAction (int id) {\n\
    switch (id) {\n\
    case ID_MENU1:\n\
        GetCOREInterface()-&gt;PushPrompt(_T(&quot;Menu 1 executed&quot;));\n\
        return true;\n\
    case ID_MENU2:\n\
        GetCOREInterface()-&gt;PushPrompt(_T(&quot;Menu 2 executed&quot;));\n\
        return true;\n\
    }\n\
    return false;\n\
}</code></pre><p>Finally, the system needs to activate and deactivate the table at the appropriate time. When to do this depends on the scope of applicability of the table. If your ActionTable is exported from an editable object or modifier plugin, then you typically want it only to be active when in the context of the command. This is done by activating it in your plugin&apos;s <code>Animatable::BeginEditParams()</code> method, and deactivating it in <code>Animatable::EndEditParams()</code>. See the FFD modifier in the SDK samples directory for an example of this approach. In other plugin types, you can perform the ActionTable activation and deactivation elsewhere, for example if you want to toggle your menu while the plugin is still running.</p>\n\
<p>For the maxMenuDemo sample, activation looks like this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">myActionCB = new ExampleActionCB();\n\
GetCOREInterface()-&gt;GetActionManager()-&gt;ActivateActionTable( myActionCB, kMyActions );</code></pre><p>The first parameter is the ID of the table to activate, and the second is an instance of the <code>ActionCallback</code> class that is responsible for executing actions.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../action_system.html\">Action System</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";