var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"command_modes_and_mouse_procedur\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Command Modes and Mouse Procedures</h1></div><p>In addition to any user interface that a plug-in may provide in the command panel via rollup pages, a plug-in may want to process mouse interaction in any of the viewports. Command modes allow the plug-in developer to define custom user / mouse interaction procedures. The system uses command modes as well. Examples are the viewport manipulation commands such as zoom and arc-rotate. Move, rotate and scale are implemented as command modes as well.</p>\n\
<p>This section discusses command modes and the methods used to work with them.</p>\n\
<h2 id=\"methods\">Methods</h2><p>In 3ds Max, there is always a current command mode. This is the instance of the class <code>CommandMode</code> that is currently at the top of the system&#39;s command stack. Command modes can either be pushed on the stack or replace the item on the top of the stack.</p>\n\
<p>There are methods in 3ds Max&#39;s <code>Interface</code> class to set and get the current command mode as well as getting the size of the command stack and finding a particular mode by index. See the methods:</p>\n\
<ul>\n\
<li><code>Interface::PushCommandMode()</code></li>\n\
<li><code>Interface::SetCommandMode()</code></li>\n\
<li><code>Interface::PopCommandMode()</code></li>\n\
<li><code>Interface::GetCommandMode()</code></li>\n\
<li><code>Interface::PushStdCommandMode()</code></li>\n\
<li><code>Interface::RemoveMode()</code></li>\n\
<li><code>Interface::DeleteMode()</code></li>\n\
<li><code>Interface::GetCommandStackSize()</code></li>\n\
<li><code>Interface::GetCommandStackEntry()</code></li>\n\
</ul>\n\
<p>Viewport manipulation commands, for instance, are always pushed on the stack so that if the user right clicks while in a viewport mode, the viewport command mode is popped off the stack, restoring the previous command mode in effect before the viewport command was engaged.</p>\n\
<p>A command mode provides the system with two major things:</p>\n\
<ul>\n\
<li>A callback procedure which flags nodes that belong in the foreground plane.</li>\n\
<li>A mouse proc which handles mouse input in the viewports.</li>\n\
</ul>\n\
<p>For the first item, plug-ins typically use a standard callback object provided by the system that flags all nodes dependent on the plug-in object. So when the plug-in object changes, any nodes that change as a result will be in the foreground plane, making redraw time faster. For more details see the section <a href=\"./../viewports_and_graphics_windows/graphics_window/foreground__background_planes.html\">Foreground / Background Planes</a>.</p>\n\
<p>The mouse proc is the object that allows plug-ins to process viewport mouse input. The <code>MouseCallback</code> class is a pure virtual class with a method named <code>proc()</code>. This method is called when a mouse event takes place. It gets passed several parameters:</p>\n\
<ul>\n\
<li>The window handle of the window the user clicked in. This will be one of the viewports. An interface to the viewport can be obtained from the system, given this window handle.</li>\n\
<li>A message parameter. This describes the type of event such as mouse down, mouse up, mouse move, etc.</li>\n\
<li>The point number. this is 0 for the first click, 1 for the second, etc.</li>\n\
<li>Flags. These specify the state of the Shift, Ctrl, and Alt keys.</li>\n\
<li>The 2D screen point that the user clicked on. Methods in the viewport interface allow this point to be converted into a world space ray or a 3D view space point. A world space ray can be intersected with the active construction plane which results in a point on the active construction plane.</li>\n\
</ul>\n\
<p>The mouse proc can specify how many points it wants for its specific task. For instance, the move transform needs two points. The first point represents the point where the user clicked down and the second point represents where the user released the mouse. A mouse proc can specify that it needs a large number of points and then return an abort code when it has completed its operation. For instance, the mouse proc that handles the creation of a spline expects to get some arbitrary number of points. When the user completes the spline, the mouse proc signals the system that its operation is completed.</p>\n\
<p>In addition to providing the system with a foreground callback and a mouse proc, a command mode also has some other methods.</p>\n\
<p>When a command mode becomes active its <code>EnterMode()</code> method is called. Usually it would respond by changing the state of a control somewhere to reflect to the user that they were in that mode. Typically this means pushing in a tool button. When the mode is finished the button should be returned to normal. The user may have activated the mode by pushing in the tool button, in which case it is redundant for the command mode to also set the button&#39;s state to &#39;in&#39;, however the mode could have been entered by right clicking while in a viewport manipulation mode in which case, the tool button would have been in the &#39;out&#39; state. Note: A developer should use the standard color <code>GREEN_WASH</code> for check buttons that instigate a command mode. While the command mode is active the button should be displayed in <code>GREEN_WASH</code>. See Class <code>ICustButton</code> (specifically the method <code>SetHighlightColor()</code>) for more details.</p>\n\
<p>When the active mode is replaced by a different mode, its <code>ExitMode()</code> method is called. Typically, the command mode would respond by setting its corresponding tool button to the &#39;out&#39; state.</p>\n\
<p>Command modes are identified by a class ID and an ID. The class ID is usually chosen from a pre-defined set of class IDs although developers may create their own. These define the &#39;type&#39; of mode. The ID is the mode&#39;s unique ID.</p>\n\
<p>There are two other methods a developer may find useful. These methods allow a plug-in to receive notification when the command mode has been changed by the user. These methods of class <code>Interface</code> are:</p>\n\
<p><code>Interface::RegisterCommandModeChangedCallback()</code> - Register a callback object that will get called when the user changes the command mode. Also see <code>CommandModeChangedCallback</code>.</p>\n\
<p><code>Interface::UnRegisterCommandModeChangedCallback()</code> - Unregister the command mode change callback object.</p>\n\
<h2 id=\"display-of-messages-in-command-modes\">Display of Messages in Command Modes</h2><p>Developers cannot put up a message box using the Windows <code>MessageBox()</code> API while in the middle of a mouse operation inside a command mode. For example, if the user is dragging the mouse, it will cause problems to put up a message box. The prompt line may be used if a message needs to be sent to the user while in this state. This is done using the methods of class <code>Interface</code>. See Class <code>Interface</code> to review these methods.</p>\n\
<h2 id=\"summary\">Summary</h2><p>Command modes allow the plug-in developer to process mouse interaction in any of the viewports. A command mode provides the system with a foreground callback and a mouse proc. See the section <a href=\"./../../writing_plug-ins/writing_simple_geometric_object_/object_creation_methods.html\">Object Creation Methods</a> for a discussion of how command modes participate in custom object creation procedures.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../user_interface.html\">User Interface</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";