var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"methods_and_classes_of_custom_co\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Methods and Classes of Custom Controls</h1></div><p>For each control, there are two functions that you&apos;ll always use:</p>\n\
<p><em>1.</em> A function to initialize the pointer to the control so you may call its methods (usually when the dialog is initialized).</p>\n\
<p><em>2.</em> A function to release the control when you are done with it (when the dialog is destroyed).</p>\n\
<p>The sample code below demonstrates this for a spinner control (the other controls are similar):</p>\n\
<p>First, declare a pointer to the control:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static  ISpinnerControl *radSpin;</code></pre><p>To initialize the pointer call:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">radSpin=GetISpinner(GetDlgItem(hDlg,IDC_RAD_SPIN));</code></pre><p>Each custom control has a Get function to return a handle to the control.</p>\n\
<p>In the example above, two functions are called from a single statement. The <code>GetDlgItem</code> function retrieves the handle of a control in the specified dialog box. The parameter <code>hDlg</code> is the dialog handle. The <code>IDC_RAD_SPIN</code> parameter is the ID for the control. This is the name entered in the ID field of the Custom Control Properties dialog box when the control was created.</p>\n\
<p>When you are finished using the control, call:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">ReleaseISpinner(radSpin);</code></pre><p>Each control has a Release function to release it.</p>\n\
<p>See <code>ICustomControl</code> for a list of the methods that are available for all the custom controls.</p>\n\
<p>The following functions are not a part of class <code>ICustomControl</code> but may also be used in conjunction with the custom controls and UI design in 3ds Max:</p>\n\
<p>This is a bitmap brush where the bitmap is a gray and white checker board.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">HBRUSH GetLTGrayBrush();\n\
HBRUSH GetDKGrayBrush();</code></pre><p>This returns the standard font.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">HFONT GetFixedFont();</code></pre><p>This returns the handle of the hand cursor used for panning.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">HCURSOR GetPanCursor();</code></pre><p>The following section provides links to the custom control classes that document how to use the specific methods of each control.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>How to use the</th>\n\
<th>See</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td>Custom Edit Control</td>\n\
<td><code>ICustEdit</code></td>\n\
</tr>\n\
<tr>\n\
<td>Spinner Control</td>\n\
<td><code>ISpinnerControl</code></td>\n\
</tr>\n\
<tr>\n\
<td>Custom Image Control</td>\n\
<td><code>ICustImage</code></td>\n\
</tr>\n\
<tr>\n\
<td>Custom Status Control</td>\n\
<td><code>ICustStatus</code></td>\n\
</tr>\n\
<tr>\n\
<td>Color Swatch Control</td>\n\
<td><code>IColorSwatch</code></td>\n\
</tr>\n\
<tr>\n\
<td>Custom Button Control</td>\n\
<td><code>ICustButton</code></td>\n\
</tr>\n\
<tr>\n\
<td>Custom Toolbar Control</td>\n\
<td><code>ICustToolbar</code></td>\n\
</tr>\n\
<tr>\n\
<td>Rollup Window Control</td>\n\
<td><code>IRollupWindow</code></td>\n\
</tr>\n\
<tr>\n\
<td>Off Screen Buffer Control</td>\n\
<td><code>IOffScreenBuf</code></td>\n\
</tr>\n\
<tr>\n\
<td>TCB Graph Control</td>\n\
<td><code>TCBGraphParams</code></td>\n\
</tr>\n\
<tr>\n\
<td>Thumb Tack Control</td>\n\
<td>This control installs a thumb tack into a window title bar which allows the user to make the window &apos;Always On Top&apos;. There are two global functions for managing this control: <code>InstallThumbTack()</code> - This function installs a thumb tack in the title bar of a window. NOTE: The window class for the window should have 4 extra bytes in the window structure for <code>SetWindowLong()</code>.  <code>RemoveThumbTack()</code> - This function is used to remove the thumb tack from the window title bar.</td>\n\
</tr>\n\
</tbody></table>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../custom_user_interface_controls.html\">Custom User Interface Controls</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";