var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"processing_user_input_from_custo\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Processing User Input from Custom Controls</h1></div><p>Windows uses a &apos;Dialog Procedure&apos; created by the developer to handle the users manipulation of the controls in the dialog,. As the user works with the controls, Windows sends messages to the dialog procedure. The developer is responsible for responding to these messages and implementing the logic to process the user input. A full description of this messaging system is beyond the scope of this documentation, but below is a brief overview.</p>\n\
<p>When you create a dialog box or add a rollup page, you specify the dialog procedure to process the input. The basic structure of this dialog proc is listed below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">BOOL CALLBACK DialogProc(HWND hDlg, UINT message,\n\
            WPARAM wParam, LPARAM lParam)\n\
{\n\
   switch (message)\n\
   { // Respond to the message ...\n\
     case WM_INITDIALOG: // Initialize the Controls here.\n\
      return TRUE;\n\
     case WM_DESTROY: // Release the Controls here.\n\
      return FALSE;\n\
     case WM_COMMAND: // Various messages come in this way.\n\
      break;\n\
     case WM_NOTIFY: // Others this way...\n\
      break;\n\
     // Other cases...\n\
     default:\n\
     break;\n\
   }\n\
   return FALSE;\n\
}</code></pre><p>Windows passes in four parameters to the dialog procedure. These are the handle of dialog box, the message, and two parameters which hold message-specific information. Except in response to the <code>WM_INITDIALOG</code> message, the dialog box procedure should return TRUE if it processes the message, and FALSE if it does not.</p>\n\
<p>When the dialog box is initialized, the <code>WM_INITDIALOG</code> message is sent. At this time the custom controls should be initialized. Each control has methods to initialize it. The methods used to initialize the control are discussed in the next section, or you may see the sample program <code>CUSTCTRL</code>.CPP for how this is done for each control.</p>\n\
<p>When the user is finished with the dialog box, the <code>WM_DESTROY</code> message is sent. This is where you release the controls. Again, the next section discusses this, or you may take a look at the sample program to see how this is done.</p>\n\
<p>When the user works with any of the custom controls, Windows sends in specific messages to the dialog procedure. For example, when the user changes a spinner control, Windows sends in a <code>CC_SPINNER_CHANGE</code> message. The developer would add this case to the dialog proc code above and handle the processing required when the spinner changed. The dialog box may have several spinner controls: how does the developer know which spinner changed? The <code>lParam</code> and <code>wParam</code> arguments to the dialog procedure contain message-specific information. It is here that the ID of the control is provided. (This is the ID entered in the ID field of the user control properties dialog when the control was created). For example, the developer could add a case such as the one below to handle spinner change messages:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">case CC_SPINNER_CHANGE:\n\
   switch (LOWORD(wParam)) { // Switch on ID\n\
case IDC_ANGLE_SPINNER: // A specific spinner ID.\n\
   angle = (( ISpinnerControl *)lParam)-&gt;GetFVal();\n\
   break;\n\
case IDC_RADIUS_SPINNER: // A specific spinner ID.\n\
   // Code to handle the Radius spinner...\n\
   break;\n\
};\n\
break;</code></pre><p>This code fragment presents several important concepts. The <code>LOWORD</code> macro used above retrieves the low-order word from the given 32-bit value. The spinner control provides the ID of the spinner which changed in the low order word of <code>wParam</code>. (Another macro, <code>HIWORD</code>, retrieves the high-order word from the given 32-bit value). Splitting the argument into low and high order words allows more information to be packed into the parameter.</p>\n\
<p>A pointer to the spinner control is provided in the <code>lParam</code> parameter. This pointer is used to call a method of the spinner to get the new floating point value (<code>(ISpinnerControl</code> *) lParam &gt;GetFVal()).</p>\n\
<p>As another example, let&apos;s look at the color change message sent in from the Color Swatch control. When the user is changing the colors in the dialog box, and the developer has asked to be notified on each change, Windows sends the <code>CC_COLOR_CHANGE</code> message. Again, the developer would add a case to the switch to handle this message. The <code>LOWORD</code> of <code>wParam</code> contains the ID of the color swatch control. The <code>HIWORD</code> of <code>wParam</code> contains 0 if the mouse button is down as the user is changing colors, and 1 if the mouse is up.</p>\n\
<p>Sometimes, the message sent to the dialog proc is <code>WM_COMMAND</code>, and the programmer must look at both the high and low words of <code>wParam</code> to determine the nature of the message and the ID of the control which sent the message. For example, the code below demonstrates how custom button messages may be handled.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">   caseWM_COMMAND:\n\
     switch(LOWORD(wParam))\n\
     { // Switch on ID\n\
      case IDC_BUTTON: // A specific button&apos;s ID.\n\
         switch (HIWORD(wParam)) { // Notification codes\n\
      case BN_BUTTONDOWN: // Button is pressed.\n\
         break;\n\
      case BN_BUTTONUP: // Button is released.\n\
         break;\n\
      case BN_RIGHTCLICK: // User right clicked.\n\
         break;\n\
     };\n\
     break;\n\
   caseIDC_FLYOFF: // A specific fly off control ID\n\
     switch (HIWORD(wParam))\n\
     { // Notification codes\n\
      case BN_FLYOFF:\n\
     // This notification code is sent when the\n\
     // user chooses a new fly off.\n\
         break;\n\
     };\n\
     break;</code></pre><p>In summary, when the user works with controls in a dialog, Windows sends messages to the plugin&apos;s dialog proc. The developer responds to these messages to process the user input.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../custom_user_interface_controls.html\">Custom User Interface Controls</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";