var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"guidelines_for_handling_characte\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Guidelines for Handling Character Strings</h1></div><p>This section presents information developers need to know when using character strings in 3ds Max. It discusses the meaning of the <code>TSTR</code>, <code>\\_T</code>, and <code>TCHAR</code> macros that are commonly seen in the source code examples. This section also covers important substitutes for the standard C <code>str</code>* functions, and some general functions that are available for use with character strings.</p>\n\
<h2 id=\"the-tstr-_t-and-tchar-macros\">The TSTR, _T and TCHAR Macros</h2><p>When 3ds Max was designed there was originally the possibility of using UNICODE for the storage of character strings. During development it was decided that 3ds Max would use MBCS (multi-byte character sets) instead. To provide the flexibility to switch between the two (perhaps some day to make it possible to also support UNICODE once it is fully supported under Windows NT), a macro is used to substitute either wide character strings (UNICODE) or single character strings (MBCS). This macro is <code>TSTR</code>. Its definition is shown below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#ifdef _UNICODE\n\
   #defineTSTRWStr\n\
#else\n\
   #defineTSTRCStr\n\
#endif</code></pre><p>If UNICODE is in use, the macro is defined to mean <code>Class``WStr</code>. If UNICODE is not in use, <code>Class``CStr</code> is used. As mentioned above, the decision was made to not go with UNICODE, and thus <code>TSTR</code> is equivalent to <code>CStr</code>.</p>\n\
<p><code>Class</code> <code>WStr</code> is used for wide character strings (16 bits per character). <code>Class``CStr</code> is for single and double-byte character sets. DBCS <em>may</em> need 2 bytes to represent a single character, whereas UNICODE <em>always</em> uses two bytes per character. Note: Windows code often refers to &quot;MBCS&quot;, but in reality, the only multi-byte type supported is &quot;double-byte&quot;. I.e. Windows doesn&apos;t support any character sets that need 3 bytes for some characters.</p>\n\
<p>Both <code>Cstr</code> and <code>WStr</code> provide methods and operators for calculating lengths, concatenation, sub-string operations, character searching, case conversion, comparison, and formatted writing. For additional information on these classes see Class <code>CStr</code> andClass <code>WStr</code>.</p>\n\
<p>Developers are encouraged to use the <code>TSTR</code> macro for string definitions, rather than going straight to <code>CStr</code>. In this way if a future version of 3ds Max is developed that uses UNICODE, the text definitions will convert with a simple re-compile.</p>\n\
<p>Developers will also encounter the <code>\\_T</code> macro in the source code examples. Literal string definitions use <code>\\_T</code> preceding the string, for example <code>\\_T</code>(&quot;Parameters&quot;). This macro is defined in a Microsoft include file <code>\\\\MSDEV</code>\\INCLUDE\\TCHAR.H. This macro will convert a string to the proper form for either UNICODE or non-UNICODE. In general, any code that is written for internationalization should use the <code>\\_T</code> macro for literal string definitions.</p>\n\
<p>A developer may also encounter the <code>TCHAR</code> macro in the source code. This macro will be either a <code>char</code> (8 bit) or a <code>wchar_t</code> (16 bit UNICODE char) depending if UNICODE is in use.</p>\n\
<h2 id=\"standard-string-functions-from-c\">Standard String Functions from C</h2><p>The proper way to deal with strings (other than the methods of the <code>CStr</code> and <code>WStr</code> classes) is to use the <code>\\_t</code> version of the standard string functions. Because 3ds Max uses MCBS (multi-byte character sets), the <code>\\_t</code> versions sometimes map to functions other than the traditional <code>str</code>* ones (which are only guaranteed to work with single-byte character sets). Below is a list of several common mappings from the standard Microsoft VC++ include file (usually <code>\\\\MSDEV</code>\\INCLUDE\\TCHAR.H).</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">strcat maps to _tcscat\n\
strcpy maps to _tcscpy\n\
strlen maps to _tcslen\n\
sprintf maps to _stprintf\n\
sscanf maps to _stscanf</code></pre><p>For example, instead of using the standard C <code>strcpy</code> function, use the one shown below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">TCHAR buf[64];\n\
_tcscpy(buf, _T(&quot;[ 0, 0, 0 ]&quot;));</code></pre><p>Developers who need to use the standard C string functions in their 3ds Max code should refer to <code>\\\\MSDEV</code>\\INCLUDE\\TCHAR.H to see if there is a <code>\\_t</code> version of the string function defined for use.</p>\n\
<h2 id=\"the-getstring-function\">The GetString() Function</h2><p>Developers may also see the <code>GetString()</code> functions used in the sample source code. This function is simply used to return a string from a resource library. It is defined as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">TCHAR *GetString(int id)\n\
{\n\
   static TCHAR buf[256];\n\
   if (hInstance)\n\
     return LoadString(hInstance, id, buf, sizeof(buf)) ? buf : NULL;\n\
   return NULL;\n\
}</code></pre><p>This function is used so that the 3ds Max program can be easily translated into other non-English languages. By loading the strings from a resource table, only the resource string table has to be updated to allow the user interface of 3ds Max to appear translated into another language. For more details on using resource libraries, see <a href=\"./../../writing_plug-ins/advanced_plug-in_topics/facilitating_translation_and_loc.html\">Facilitating Translation and Localization</a>.</p>\n\
<h2 id=\"summary-of-rules-for-using-strings\">Summary of Rules for Using Strings</h2><p>The following rules summarize what developers need to be aware of when working with characters strings.</p>\n\
<ul>\n\
<li><p>Use the <code>TSTR</code> string class as much as possible, since these strings automatically allocate the right amount of memory for their storage. (As an added bonus, they&apos;re also a lot easier to use than standard strings, because a lot of functionality is built into the string class.)</p>\n\
</li>\n\
<li><p>If you have to use static character arrays, make sure the array allocation is adequate. Also, make sure that system limits (like 259 for path+filename) are always met or exceeded.</p>\n\
</li>\n\
<li><p>Don&apos;t use <code>char</code> or <code>w_char</code> arrays: instead use <code>TCHAR</code> as the character type.</p>\n\
</li>\n\
<li><p>Don&apos;t use any standard string manipulation functions (like <code>strcat</code>, <code>strcpy</code>, <code>sprintf</code>, etc.) directly. Rather, use the character-set independent versions found in <code>\\\\MSDEV</code>\\INCLUDE\\TCHAR.H. For example, use <code>\\_tcscat</code> instead of <code>strcat</code>. Note that even routines like <code>fopen()</code> that take a string have character-set independent versions in <code>TCHAR</code>.H.</p>\n\
</li>\n\
<li><p>Put all literal strings inside the <code>\\_T</code> macro, as in <code>\\_T</code>(&quot;This is literal&quot;). The <code>\\_T</code> macro will convert strings to Unicode automatically if we ever choose to compile with that enabled.</p>\n\
</li>\n\
<li><p>When comparing characters, use the same macro as for strings but with single quotes. Also, if for any reason you are comparing with a slash for directory separation, make sure to also check for forward slashes as they are valid in NT and used when mounting Unix drives. For example:</p>\n\
</li>\n\
<li><p>if (my_path[x] == _T(&apos;/&apos;) || my_path[x] == _T(&apos;\\&apos;)) {</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">   // do something...\n\
}</code></pre><p>For internationalization / localization consider the following when working with strings:</p>\n\
</li>\n\
<li><p>Try to avoid any string-specific manipulation (like keying off of the first letter in a string, or concatenating strings to form a longer token), since much of this won&apos;t translate easily into other languages.</p>\n\
</li>\n\
<li><p>Keep all literal strings in a resource string table. This allows the internationalization/localization process to consist of only editing resource files, not code.</p>\n\
</li>\n\
</ul>\n\
<h2 id=\"miscellaneous-string-functions\">Miscellaneous String Functions</h2><p>The following functions are part of the 3ds Max SDK and are useful for breaking up filenames.</p>\n\
<ul>\n\
<li><code>SplitFilename()</code> - Splits filename <code>name</code> into its path, filename, and extension.</li>\n\
<li><code>SplitPathFile()</code> - Splits filename <code>name</code> into its path and a filename.extension.</li>\n\
<li><code>MatchPattern()</code> - This method is used to check if a string matches a wildcard pattern.</li>\n\
<li><code>MaxAlphaNumComp()</code> - A &quot;smart&quot; alphanumeric compare that sorts things so that numerical suffixes come out in numerical order. This version is case sensetive.</li>\n\
<li><code>MaxAlphaNumCompI()</code> - A case insensitive &quot;smart&quot; alphanumeric compare that sorts things so that numerical suffixes come out in numerical order. This version is case insensitive.</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../user_interface.html\">User Interface</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";