var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"parameter_editing_on_the_command\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Parameter Editing on the Command Panel</h1></div><p>This section discusses the editing of an item&apos;s parameters in the command panel. Plug-in types such as procedural objects, modifiers, space warps, and controllers may all present their user interface in the command panel.</p>\n\
<h2 id=\"methods-called-during-parameter-editing\">Methods Called During Parameter Editing</h2><p>There are two methods that are called when a user begins and ends editing an item&apos;s parameters. In the case of procedural objects, modifiers, and controllers, these methods are from class <code>Animatable</code>. They are named <code>BeginEditParams()</code> and <code>EndEditParams()</code>. Other plug-in types, such as utilities, may have their own version of these methods (for instance <code>UtilityObj::BeginEditParams()</code>). The text below discusses the more common <code>Animatable</code> version.</p>\n\
<h2 id=\"beginning-parameter-editing\">Beginning Parameter Editing</h2><p>When the user begins to edit an item the <code>BeginEditParams()</code> method is called on it. At this point it is expected to do two things:</p>\n\
<ul>\n\
<li>Adding or displaying its user interface via rollup pages.</li>\n\
<li>Register any sub-object selection types it may have.</li>\n\
</ul>\n\
<p>Each of these is discussed below.</p>\n\
<p>Adding rollup pages can be done using the method <code>Interface::AddRollupPage()</code> or by creating a parameter map (<code>CreateCPParamMap()</code>) which puts up the rollup pages itself.</p>\n\
<p>The <code>BeginEditParams()</code> prototype looks like this:</p>\n\
<p>virtual void BeginEditParams(IObjParam *ip,ULONG flags, Animatable *prev=NULL);</p>\n\
<p>The <code>flags</code> parameter passed is used to indicate which branch of the command panel the item is being edited in. For instance it may be the Create branch, the Modifier branch, the Hierarchy branch, or the Motion branch.</p>\n\
<p>Some items may have specific parameters that are particular to the creation process. The <code>flags</code> parameter indicates whether the user is in the create branch or not. For example, procedural spheres have an option to create them by dragging out the radius or the diameter. These parameters shouldn&apos;t be displayed in the Modify branch since they are only for creating. The sphere checks the flag to see if rollup should be added. Here&apos;s the code fragment from the sphere code where that is done:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void SphereObject::BeginEditParams(\n\
                 IObjParam *ip,\n\
                ULONG flags,\n\
                Animatable *prev)\n\
{\n\
   // ...\n\
   if (flags &amp; BEGIN_EDIT_CREATE)\n\
   {\n\
     pmapCreate = CreateCPParamMap(\n\
      descCreate,CREATEDESC_LENGH,\n\
      this,\n\
      ip,\n\
       hInstance,\n\
      MAKEINTRESOURCE(IDD_SPHEREPARAM1),\n\
       GetString(IDS_RB_CREATIONMETHOD),\n\
      0);\n\
\n\
     pmapTypeIn = CreateCPParamMap(\n\
      descTypeIn,TYPEINDESC_LENGH,\n\
      this,\n\
      ip,\n\
       hInstance,\n\
      MAKEINTRESOURCE(IDD_SPHEREPARAM3),\n\
       GetString(IDS_RB_KEYBOARDENTRY),\n\
       APPENDROLL_CLOSED);\n\
   }\n\
\n\
\n\
   pmapParam = CreateCPParamMap(\n\
     descParam,\n\
     PARAMDESC_LENGH,\n\
     pblock,\n\
     ip,\n\
     hInstance,\n\
     MAKEINTRESOURCE(IDD_SPHEREPARAM2),\n\
     GetString(IDS_RB_PARAMETERS),\n\
     0);\n\
}</code></pre><p>Note above how the &apos;Creation Method&apos; or &apos;Keyboard Entry&apos; rollups are only added if the <code>BEGIN_EDIT_CREATE</code> flag is set. In either case the &apos;Parameters&apos; rollup is added.</p>\n\
<p>The following bit flags may be compared with the <code>flag</code> parameter to test the branch (or sub-task of the branch).</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th></th>\n\
<th></th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td>Flags</td>\n\
<td>Description</td>\n\
</tr>\n\
<tr>\n\
<td><code>BEGIN_EDIT_CREATE</code></td>\n\
<td>Indicates the item is being edited in the create branch.</td>\n\
</tr>\n\
<tr>\n\
<td><code>BEGIN_EDIT_MOTION</code></td>\n\
<td>Indicates a controller is being edited in the motion branch.</td>\n\
</tr>\n\
<tr>\n\
<td><code>BEGIN_EDIT_HIERARCHY</code></td>\n\
<td>Indicates a controller is being edited in the Pivot subtask of the hierarchy branch.</td>\n\
</tr>\n\
<tr>\n\
<td><code>BEGIN_EDIT_IK</code></td>\n\
<td>Indicates a controller is being edited in the IK subtask of the hierarchy branch.</td>\n\
</tr>\n\
<tr>\n\
<td><code>BEGIN_EDIT_LINKINFO</code></td>\n\
<td>Indicates a controller is being edited in the Link Info subtask of the hierarchy branch.</td>\n\
</tr>\n\
<tr>\n\
<td><code>BEGIN_EDIT_SUPPRESS_SO</code></td>\n\
<td>Used by VIZ composites to suppress sub-ojbect editing.</td>\n\
</tr>\n\
<tr>\n\
<td><code>BEGIN_EDIT_SHAPE_NO_RENDPARAM</code></td>\n\
<td>Used by LoftSpline</td>\n\
</tr>\n\
</tbody></table>\n\
<p>Inside <code>BeginEditParams()</code> the item is also expected to register any sub-object selection levels with the system. . For example, a modifier plugin may have sub-object selection levels like &apos;Gizmo&apos; and &apos;Center&apos;. These need to appear in the sub-object drop down allowing the user to select them. In the case of a bend mofifier for example, this is accomplished by calling <code>BeginEditParams()</code> on the base class <code>SimpleMod</code>, i.e.:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void BendMod::BeginEditParams(IObjParam *ip, ULONG flags, Animatable *prev)\n\
{\n\
   SimpleMod::BeginEditParams(ip,flags,prev);\n\
   // ...</code></pre><p>What the base class <code>SimpleMod</code> does is the following:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void SimpleMod::BeginEditParams(IObjParam *ip, ULONG flags, Animatable *prev)\n\
{\n\
   // ...\n\
   // Add our sub object type\n\
   TSTR type1(GetResString(IDS_RB_APPARATUS) ); \n\
   TSTR type2(GetResString(IDS_RB_CENTER) ); \n\
   constTCHAR *ptype[ = { type1, type2 };\n\
   ip-&gt;RegisterSubObjectTypes( ptype, 2 );\n\
   // ...</code></pre><p>Note how it retrieves two string resources using <code>GetResString()</code> (&quot;Gizmo&quot; and &quot;Center&quot;) and registers them using <code>Interface::RegisterSubObjectTypes()</code>.</p>\n\
<h2 id=\"ending-parameter-editing\">Ending Parameter Editing</h2><p>When the user is finished editing the item&apos;s parameters its <code>EndEditParams()</code> method is called. For example, if the user switches branches of the command panel, or selects a different object type to be created, it&apos;s called on the item currently being edited.</p>\n\
<p>The prototype looks like this:</p>\n\
<p>virtual void EndEditParams(IObjParam *ip, ULONG flags, Animatable *next=NULL);</p>\n\
<p>This method is also passed a <code>flag</code> value. If the <code>END_EDIT_REMOVEUI</code> flag is set, the item should remove its rollup pages from the command panel. For example, when in the create branch, procedural objects can be created one after the other so there is no need to remove the UI when one instance of a class is finished with its create stage if another instance of that same class is about to be created. In such a case, <code>END_EDIT_REMOVEUI</code> won&apos;t be set since the user interface should stay in place.</p>\n\
<h2 id=\"user-interface-related-variables\">User Interface Related Variables</h2><p>The best way to handle user interface-related variables is to make them class variables (by declaring them static). Class variables are <em>shared</em> by every instance of the class (rather than stored with each instantiated object of the class). This way, an object doesn&apos;t suffer the overhead of these variables that are only used when it is editing. Only one item may have its parameters being edited at any one time so the class variables work well.</p>\n\
<p>For example, consider a procedural sphere. It may have class variables that contain the handles to its parameter maps in the command panel. When a sphere is first created, it is immediately edited so its <code>BeginEditParams()</code> method is called. It creates the parameter maps and stores the handles to them in its class variables. If the user chooses to make another sphere, the first sphere&apos;s <code>EndEditParams()</code> method is called, but <code>END_EDIT_REMOVEUI</code> flag is set to <code>FALSE</code>, indicating that the sphere should leave the rollup pages in the command panel. A new instance of the Sphere class is then created and its <code>BeginEditParams()</code> method is called. It notices that the parameter map handles are not NULL so it doesn&apos;t need to create them. For instance, consider how the procedural sphere checks this in the code below. If its pointer to the &apos;Creation Method&apos; and &apos;Parameters&apos; parameter maps are still valid (non-NULL) it doesn&apos;t recreate the rollup pages. Rather it simply updates the parameber block pointer stored by the parameter map to point to the new sphere object.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">if (pmapCreate &amp;&amp; pmapParam)\n\
{\n\
   // Left over from last sphere ceated\n\
   pmapCreate-&gt;SetParamBlock(this);\n\
   pmapTypeIn-&gt;SetParamBlock(this);\n\
   pmapParam-&gt;SetParamBlock(pblock);\n\
}\n\
else\n\
{\n\
// ...</code></pre><h3 id=\"interface-pointer-validity\">Interface Pointer Validity</h3><p>The interface pointer passed to <code>BeginEditParams()</code> is usally stored by the plugin since it is likely that the plugin will need it to call its methods. This pointer is only valid while the item is being editied. Therefore, another important thing to do when <code>EndEditParams()</code> is called is to set the object&apos;s interface pointer to NULL. This pointer is only valid between the <code>BeginEditParams()</code> and <code>EndEditParams()</code> methods. A plugin should not call methods on this pointer outside of this interval.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../user_interface.html\">User Interface</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";