var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"tabbed_dialogs\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Tabbed Dialogs</h1></div><p>Developers can use the tabbed dialog system to create their own tabbed dialogs as well as working with 3ds Max&#39;s own tabbed dialogs. The steps below present a how-to guide for doing this. Theses step</p>\n\
<h2 id=\"1-define-id-numbers\">1. Define ID numbers</h2><p>Define a unique <code>Class_ID</code> for each tab, and for each tabbed dialog, you are creating.</p>\n\
<p>There are predefined IDs for some standard dialogs:</p>\n\
<ul>\n\
<li><code>TAB_DIALOG_PREFERENCES_ID</code> - preferences dialog</li>\n\
<li><code>TAB_DIALOG_PROPERTIES_ID</code> - object properties dialog</li>\n\
<li><code>TAB_DIALOG_PATH_ID</code> - customize paths dialog</li>\n\
<li><code>TAB_DIALOG_RENDER_ID</code> - render dialog</li>\n\
<li><code>TAB_DIALOG_VIDEO_POST_ID</code> - video post</li>\n\
<li><code>TAB_DIALOG_ENVIRONMENT_ID</code> - environment dialog</li>\n\
</ul>\n\
<h2 id=\"2-itabdialogplugintab-class\">2. ITabDialogPluginTab class</h2><p>For each tab, create one implementation of <code>ITabDialogPluginTab</code>. If you have a group of tabs that will always appear together, you need only one implementation for the whole group.</p>\n\
<h2 id=\"3-add-tabs-to-the-dialog\">3. Add tabs to the dialog</h2><p>Your <code>ITabDialogPluginTab::AddTabToDialog()</code> method should call <code>dialog</code>-&gt;AddPage() or <code>dialog</code>-&gt;AddRollout().</p>\n\
<p>Pass the tab&#39;s name, your dialog template, and the message procedure. Make sure to pass the class ID you&#39;ve defined for the tab.</p>\n\
<p>If you have a group of tabs appearing together, you can add multiple pages or rollouts to the dialog.</p>\n\
<h2 id=\"4-tab-filters-optional-\">4. Tab filters (optional)</h2><p>These steps are optional. You can implement some or all of these options if you want tabs to show and hide dynamically according to the state of the dialog</p>\n\
<ol>\n\
<li>Implement <code>ITabDialogPluginTab::AcceptDialog()</code>. This method indicates whether a tab is willing to reside in a given dialog</li>\n\
<li>Implement an <code>ITabDialogFilter</code> and override its <code>ITabDialogFilter::AcceptTab()</code> method. The return value indicates if a dialog is willing to accept a given tab. The return value <code>TAB_DIALOG_ADD_TAB</code> means the tab should be displayed. The return value <code>TAB_DIALOG_REMOVE_TAB</code> means the tab should not be displayed and will be removed. Returning both values combined using &quot;or&quot; means the tab will be removed, and immediately re-displayed. Note: A tab only appears if both the dialog and the tab agree</li>\n\
<li>Both the <code>PluginTab</code> and the <code>DialogFilter</code> have some optional methods that you can implement if your <code>Accept()</code> methods need them. These are: <code>GetReferenceMaker()</code>, <code>GetAnimatable()</code>, <code>GetClassID()</code>, and <code>GetSuperClassID()</code>. These allow your <code>Accept()</code> methods to distinguish one tab from another, or one dialog from another. The <code>ITabDialogPluginTab</code> can return the class ID of the tab it&#39;s associated with, and the <code>ITabDialogFilter</code> can return the class ID of the dialog it is associated with.</li>\n\
</ol>\n\
<h2 id=\"5-registering-with-the-manager\">5. Registering with the manager</h2><p>Your <code>ITabDialogPluginTab</code> and <code>ITabDialogFilter</code> (if any) need to be registered with the manager. The manager itself can be obtained with the global function <code>GetTabDialogManager()</code> .</p>\n\
<p>First call <code>ITabDialogManager::RegisterTab()</code>to register a <code>ITabDialogPluginTab</code> , indicating the ID of the dialog tjat the tab should be used with. You may register a tab with multiple different dialogs.</p>\n\
<p>Next, if you have <code>ITabDialogFilter</code>then call <code>ITabDialogManager::SetTabFilter()</code>to register it indicating the ID of the dialog</p>\n\
<h2 id=\"6-message-handlers\">6. Message handlers</h2><p>To handle window messages for your tabs and dialogs, you can to create implementations of <code>ITabPageProc</code> (one for each tab) and <code>ITabDialogProc</code> (one for each dialog). For each of these, the only method to implement is <code>dialogProc()</code>.</p>\n\
<p>These classes have a number of convenience methods, like <code>Commit()</code>, <code>Close()</code>, and <code>Cancel()</code>, which simply call the <code>dialogProc()</code> with certain preset parameters.</p>\n\
<h2 id=\"7-launch-the-dialog\">7. Launch the dialog</h2><p>At this point, you are ready to launch the dialog.</p>\n\
<p>First call <code>ITabDialogManager::CreateTabbedDialog()</code> on the dialog manager returned from <code>GetTabDialogManager()</code> passing the dialog&#39;s <code>ITabDialogProc</code> and the class ID defined for the dialog. If your dialog template already contains a tab control UI element, pass its ID in the <code>tabID</code> parameter, otherwise pass -1. The other parameters are explained in the method&#39;s code comment in the header file.</p>\n\
<p>Next call the <code>ITabbedDialog::DoFloater()</code>or <code>ITabbedDialog::DoModal()</code>method on the <code>ITabbedDialog</code>that is returned from <code>CreateTabbedDialog()</code>. For modeless dialogs, each call to <code>ITabbedDialog::SyncRegisteredTabs()</code>will refresh the tab set, hiding and showing tabs as needed.</p>\n\
<h2 id=\"summary-of-classes\">Summary of Classes</h2><p>The following classes are implemented by the system:</p>\n\
<ul>\n\
<li><code>ITabDialogManager</code></li>\n\
<li><code>ITabPage</code></li>\n\
<li><code>ITabbedDialog</code></li>\n\
<li><code>TabDialogPointer</code> - This is a &quot;smart pointer&quot; provided for convenience, it allows the dialog to be disposed when the pointer goes out of scope</li>\n\
</ul>\n\
<p>The following classes are implemented by the plug-in:</p>\n\
<ul>\n\
<li><code>ITabDialogPluginTab</code></li>\n\
<li><code>ITabDialogFilter</code></li>\n\
<li><code>ITabDialogProc</code></li>\n\
<li><code>ITabPageProc</code></li>\n\
<li><code>ITabDialogObject</code> - this is specifically for use with renderer plug-ins, see <code>IRendParams</code>.</li>\n\
</ul>\n\
<h2 id=\"about-tabbed-dialogs-in-the-renderer\">About Tabbed dialogs in the Renderer</h2><p>Renderers should use <code>AddTabRollupPage()</code> to add their rollup to the render dialog, but <code>AddRollupPage()</code> should be used when adding a rollout to the progress dialog.</p>\n\
<p>In 3ds Max, the render dialog will always display the &quot;Common&quot; tab and the renderer&#39;s individual parameter tab. It will display the &quot;Raytracer&quot; and &quot;Advanced Lighting&quot; tab only if the renderer chooses to display them. The &quot;Render Elements&quot; tab is displayed only if the renderer supports the <code>IRenderElementCompatible</code> interface.</p>\n\
<p>To control which tabs are displayed, a renderer can provide an ITabDialogObject interface and implement AcceptTab().</p>\n\
<h2 id=\"for-more-information\">For More Information</h2><p>For an example of tabbed dialogs see cjrender sample located in <code>\\\\maxsdk</code>\\samples\\render\\cjrender.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../user_interface.html\">User Interface</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";