var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"using_sethitcode_and_sethitdista\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Using setHitCode and setHitDistance</h1></div><p>The <code>GraphicsWindow::setHitCode()</code> and <code>GraphicsWindows::setHitDistance()</code> are methods that make it possible to do hit-testing in some otherwise impossible situations. An example of such a situation arises when hit-testing patch regions that consist of bezier spline-based edges.</p>\n\
<h2 id=\"sample-code\">Sample Code</h2><p>The patch object (<code>PatchObject</code>) contains bezier spline-based edges which can consist of up to 102 vertices. Since the <code>GraphicsWindow::polyline()</code> function can only plot lines with up to 32 vertices, it is impossible to plot these in a single call to the function. Multiple calls do not return a proper hit-code when using a &quot;window&quot;-type hit region. By using the <code>GraphicsWindows::setHitCode()</code> method, code can properly handle this situation. The code below shows the function in use from the <code>PatchMesh::renderEdge</code> method:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">int steps = GetMeshSteps();\n\
int segNum = steps+2;\n\
float fsegNum = (float) (segNum-1);\n\
\n\
// If steps are too high for GraphicsWindow&apos;s buffer,\n\
// we must draw it manually\n\
if((steps + 2) &gt; GW_MAX_VERTS)\n\
{\n\
   Point3 line[2;\n\
   Point3 prev,current(.0f,.0f,.0f);\n\
   BOOLhitAll = TRUE;\n\
   BOOLhitAny = FALSE;\n\
   DWORDhitDist = 0xffffffff;\n\
   for(int terp = 0; terp &lt; segNum; terp++)\n\
   {\n\
     prev = current;\n\
     current = work.InterpCurve3D((float)terp / fsegNum);\n\
     if (terp != 0)\n\
     {\n\
      line[0 = prev;\n\
      line[1 = current;\n\
      gw-&gt;clearHitCode();\n\
      gw-&gt;polyline(2, line, NULL, NULL, 0, NULL);\n\
      if(gw-&gt;checkHitCode())\n\
       {\n\
         hitAny = TRUE;\n\
         if(gw-&gt;getHitDistance() &lt; hitDist)\n\
          hitDist = gw-&gt;getHitDistance();\n\
       }\n\
      else hitAll = FALSE;\n\
     } \n\
   }\n\
\n\
   if(hr &amp;&amp; !hr-&gt;crossing &amp;&amp; hr-&gt;type != POINT_RGN)\n\
     gw-&gt;setHitCode(hitAll);\n\
   else\n\
     gw-&gt;setHitCode(hitAny);\n\
     gw-&gt;setHitDistance(hitDist);\n\
}\n\
else\n\
{\n\
   for(int terp = 0; terp &lt; segNum; terp++)\n\
     fixedBuf[terp = work.InterpCurve3D((float) terp / fsegNum);\n\
\n\
   gw-&gt;polyline(steps+2, fixedBuf, NULL, NULL, 0, NULL);\n\
}\n\
</code></pre><p>Note that the <code>GraphicsWindow::polyline()</code> call is preceded by a call to <code>GraphicsWindows::clearHitCode()</code>, and followed by code which checks the hit code, maintaining <code>hitAny</code> and <code>hitAll</code> flags. When all the segments are drawn, the <code>GraphicsWindow::setHitCode()</code> call is made, setting the hit code depending on the hit region type. When the code which called this function checks the hit code, it will contain the proper value. The sample code also keeps track of the closest hit distance and places that into the <code>GraphicsWindow</code> when all line segments are drawn.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../hit_testing.html\">Hit Testing</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";