var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"about_smarthandle\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>About SmartHandle</h1></div><p>Nitrous is designed to run in a separate nitrous rendering thread. For thread-safety, plug-ins must use <code>SmartHandle</code> to access the pointers of Nitrous resources including buffers, textures, render items, materials, and others in main thread or nitrous rendering thread.</p>\n\
<p><code>SmartHandle</code> can be considered as smart pointers. Each <code>SmartHandle</code> holds a reference to a resource. More than one <code>SmartHandle</code> can refer to the same resource. In this case, the resource is not freed until the last <code>SmartHandle</code> releases its reference.</p>\n\
<p>The Nitrous SDK provides a list of strong typed handles. These handles are derived from <code>SmartHandle</code> and they have no data members, but several methods for plug-ins to manipulate the underlying resources.</p>\n\
<p>The following figure shows the <code>smarthandle</code> types in the Nitrous SDK:</p>\n\
<p><img src=\"./images/GUID-D8E278D7-780B-4C3F-948F-59C728E7E622-low.png\" alt></p>\n\
<h2 id=\"creating-code-smarthandle-code-\">Creating <code>SmartHandle</code></h2><p>By default, a <code>SmartHandle</code> references a null object and is invalid. To make the <code>SmartHandle</code> valid, the plug-ins must initialize the <code>SmartHandle</code> or assign another valid <code>SmartHandle</code> to the <code>SmartHandle</code>.</p>\n\
<p>The following example illustrates how to create a valid <code>GeometryRenderItemHandle</code> and how to use it:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">  GeometryRenderItemHandle hRenderItem;\n\
  assert(!hRenderItem.IsValid()); // Is invalid at this time\n\
\n\
  hRenderItem.Initialize();\n\
  assert(hRenderItem.IsValid()); // Is valid now\n\
\n\
  GeometryRenderItemHandle hRenderItem2;\n\
  // Both hRenderItem and hRenderItem2 reference to the same \n\
  // underlying geometry resource\n\
  hRenderItem2 = hRenderItem;\n\
  // Two render items are equal to each other now\n\
  assert(hRenderItem == hRenderItem2);\n\
\n\
  hRenderItem.SetMaterialID(31583); // Set a random material ID\n\
  assert(hRenderItem2.GetMaterialID() == 31583); // Should be the same</code></pre><h2 id=\"write-type-and-read-type-functions\">Write Type and Read Type Functions</h2><p>The <code>SmartHandle</code> methods can be categorized into two groups: write type and read type.</p>\n\
<p>Any function call that does not send data from the underlying resource to the calling thread is considered a write type function. All calls to these functions are pushed to a thread-safe command queue. Nitrous chooses a proper time to flush all commands in the command queue to synchronize the underlying resource.</p>\n\
<p>The <code>SmartHandle</code> caches the data members that cannot be changed by the rendering thread. If the plug-in needs the member data of the underlying resource, the <code>SmartHandle</code> returns the cached data when applicable. However, in certain cases, if the plug-in wants to retrieve a member data that requires computation in the rendering thread, Nitrous flushes all commands in the command queue, sends a data retrieving command, and waits for the rendering thread to return the result. During this process, the calling thread is blocked and the command queue is locked for exclusive use by the <code>SmartHandle</code>.</p>\n\
<p>Plug-ins must avoid using the <em>read</em> functions of <code>SmartHandle</code> whenever possible because <code>SmartHandle</code> is <em>write</em> friendly but not <em>read</em> friendly.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../nitrous.html\">Nitrous</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";