var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"implementing_simple_material_and\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Implementing Simple Material and Texture Map Plug-ins with Nitrous</h1></div><h2 id=\"simple-material-and-texture-map-display-api\">Simple Material and Texture Map Display API</h2><p>Nitrous SDK provides automatic support for displaying material and texture map plug-ins in both the Nitrous viewport mode and in Quicksilver renderings. 3ds Max will create a simple visual representation of these plug-ins with basic colors, diffuse and opacity texture maps which can be rendered quickly, and which will appear the same in both Nitrous viewport mode and in Quicksilver.</p>\n\
<p>The following are the APIs to support simple and fast material and texture map display:</p>\n\
<ul>\n\
<li><code>ISimpleMaterial</code> - An interface implemented by 3ds Max that represents a basic material or texture in the Nitrous viewport display system.</li>\n\
<li><code>ITextureDisplay</code> - An interface implemented by plug-ins that require special processing for proper display of their textures in the viewport.</li>\n\
<li><code>DisplayTextureHelper</code> - A class implemented by 3ds Max, and derived from class <code>TexHandleMaker</code>, that is passed to<code>ITextureDisplay::SetupTexture()</code> and used to generate texture handle (<code>TexHandle</code>) instances as needed.</li>\n\
</ul>\n\
<p>Materials and texture maps with complex parameters or requiring richer visual representations in the Nitrous viewport display or in the Quicksilver renderer can be developed using HLSL (High Level Shader Language) shaders. For more information see the section <a href=\"./../../../what_s_new/archive/what_s_new_3ds_max_2012_sdk/porting_advanced_material_and_te.html\">Porting Advanced Material and Texture Map Plug-ins to Nitrous</a>.</p>\n\
<h2 id=\"isimplematerial\">ISimpleMaterial</h2><p>Any material plug-ins can directly get <code>ISimpleMaterial</code> interface from <code>MtlBase</code> to customize its display in shaded mode. The following code shows how to get the <code>ISimpleMaterial</code> interface.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">ISimpleMaterial* pSimpleMaterial = (ISimpleMaterial *)pMtlBase-&gt;GetProperty(PROPID_SIMPLE_MATERIAL);</code></pre><p><code>ISimpleMaterial</code> supports basic material and texture color properties such as ambient, diffuse, specular, and emissive, and supports a few basic shading and fill modes. The Nitrous viewport will query the <code>ISimpleMaterial</code> properties when it displays <code>MtlBase</code> plug-ins.</p>\n\
<p><code>ISimpleMaterial</code> supports two texture map channels: diffuse and opacity. The Nitrous viewport display driver will also set these up on an <code>ISimpleMaterial</code> instance, but it is the plug-in&apos;s responsibility to create and manage the lifetime of the texture resource associated with a <code>TexHandle</code> object. The <code>TexHandle</code> can be generated by calling <code>TexHandleMaker::MakeHandle()</code>.</p>\n\
<h2 id=\"itexturedisplay-and-displaytexturehelper\">ITextureDisplay and DisplayTextureHelper</h2><p><code>ITextureDisplay</code> is an interface that allows texture map and material plug-ins to control how their diffuse and opacity textures appear in the Nitrous viewport. This can be useful when a plug-in needs to do custom blending of textures, or when the texture maps have different requirements such as different texture coordinates. Creating and maintaining the handles to these textures is the responsibility of the <code>DisplayTextureHelper</code> interface.</p>\n\
<p>To have its diffuse and opacity texture appear correctly in the Nitrous viewport, the plug-in must create them by implementing <code>ISimpleMaterial::SetTexture()</code>. For this, the plug-in needs to derive from the <code>ITextureDisplay</code> interface and use <code>DisplayTextureHelper::MakeHandle()</code> to implement <code>ITextureDisplay::SetupTextures()</code> in that interface.</p>\n\
<h2 id=\"impact-on-plug-ins\">Impact on plug-ins</h2><p>In practice, all but the most simple materials and texture map plug-ins will require additional code to provide a correct simple visual representation in the Nitrous viewport. The following scenarios provide more details.</p>\n\
<p><em>Scenario 1: Plug-in supports the display of at most one texture map</em></p>\n\
<p>If the material or texture map plug-in supports displaying only one texture map, Nitrous will use that map in the diffuse texture map channel by default. If this default behavior is acceptable, there is no additional code required by the plug-in other than the regular code for supporting the display of the texture map. See Scenario 2 below if the default behavior is not acceptable. Note that in order for the plug-in to be able to override Nitrous&apos; default support, it will need to return <code>TRUE</code> from an overridden implementation of the <code>MtlBase::SupportsMultiMapsInViewport()</code> virtual method even if it has only one map.</p>\n\
<p>The key methods a plug-in should implement for supporting one texture map are outlined below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class Scenario1 : public Texmap \n\
{ \n\
    TexHandle* texHandle;\n\
    // Details elided for brevity \n\
}\n\
\n\
// Supports displaying textures\n\
BOOL Scenario1::SupportTexDisplay() {return TRUE;}\n\
\n\
// Does not support displaying multiple textures\n\
BOOL Scenario1::SupportsMultiMapsInViewport() {return FALSE;}\n\
\n\
// Helper method\n\
void Scenario1::DiscardTexHandle() \n\
{\n\
    if (texHandle) \n\
    {   \n\
        texHandle-&gt;DeleteThis();\n\
        texHandle = NULL;\n\
    }\n\
}\n\
\n\
DWORD_PTR Scenario1::GetActiveTexHandle(TimeValue t, TexHandleMaker&amp; maker) \n\
{\n\
    if (texHandle) \n\
    {\n\
        if (texHandleValid.InInterval(t))\n\
                return texHandle-&gt;GetHandle();\n\
            else \n\
            DiscardTexHandle();\n\
        }\n\
    texHandle = maker.MakeHandle(GetVPDisplayDIB(t, maker, texHandleValid));\n\
    return texHandle-&gt;GetHandle();\n\
}</code></pre><p>The class <code>Scenario1</code> above uses <code>Texmap::GetVPDisplayDIB()</code> to generate a representation of itself. The returned <code>BITMAPINFO</code> is then used by the <code>TexHandleMaker::MakeHandle()</code> to access the raw bitmap data and generate the texture resource needed by the display system. If the texture map is procedural in nature, like marble, it is important to note that the generated result will not look the same as the one rendered by a production renderer; it is a mere interpretation of the procedural effect. For a more accurate viewport representation of the plug-in, HLSL shaders must be used.</p>\n\
<p><em>Scenario 2: Plug-in supports displaying multiple texture maps</em></p>\n\
<p>To display multiple texture maps in the viewport, a plug-in must override <code>MtlBase::SupportsMultiMapsInViewport()</code> to return <code>TRUE</code>. If it does not, <code>ITextureDisplay::SetupTextures()</code> will not be called.</p>\n\
<p>It is important to note that <code>ITextureDisplay</code> supports only the diffuse and opacity texture maps, although the plug-in might support more. For advanced texture map support in the Nitrous viewport, the HLSL shader language needs to be used.</p>\n\
<p>The key code changes a plug-in should implement for supporting multiple texture maps in the Nitrous viewport mode are outlined below:</p>\n\
<ul>\n\
<li><p>Derive the plug-in from class <code>ITextureDisplay</code> and make sure its <code>InterfaceServer::GetInterface()</code> handles the requests for <code>ITEXTURE_DISPLAY_INTERFACE_ID</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#include &lt;.\\Graphics\\ITextureDisplay.h&gt;\n\
\n\
// Material plug-in with multimap support derives from the class ITextureDisplay\n\
class Scenario2 : public Mtl, public MaxSDK::Graphics::ITextureDisplay\n\
{ \n\
    TexHandle* texHandle1;\n\
    TexHandle* textHandle2;\n\
    // Details elided for brevity \n\
}\n\
\n\
BaseInterface* Scenario2::GetInterface(Interface_ID id)\n\
{\n\
    // Handle request for an ITextureDisplay interface\n\
    if (id == ITEXTURE_DISPLAY_INTERFACE_ID)\n\
    {\n\
        return static_cast&lt;ITextureDisplay*&gt;(this);\n\
    }\n\
\n\
    return Mtl::GetInterface(id);  // If Scenario2 is a texture return Texmap::GetInterface\n\
}</code></pre></li>\n\
<li><p>The plug-in&apos;s override of <code>Mtlbase::SupportsMultiMapsInViewport()</code> must return <code>TRUE</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Supports displaying textures\n\
BOOL Scenario2::SupportTexDisplay() {return TRUE;}\n\
\n\
// Supports displaying multiple textures\n\
// Must return TRUE even if the plug-in has only one map but wants to \n\
// override the Nitrous default support for texture maps.\n\
BOOL Scenario2::SupportsMultiMapsInViewport() {return TRUE;}</code></pre></li>\n\
<li><p>Implement <code>ITextureDisplay::SetupTextures()</code>.</p>\n\
<p>This method is called whenever a change to the material or texture is detected by the display system. Call <code>ISimpleMaterial::SetTexture()</code> to specify the actual texture map to be displayed as diffuse and opacity maps. Use the <code>DisplayTextureHelper</code> object derived from class <code>TexHandleMaker</code> to wrap the texture resource into a <code>TexHandle</code> that you can pass to <code>ISimpleMaterial::SetTexture()</code>. <code>DisplayTextureHelper::UpdateTextureMapInfo()</code> needs to be called for the Nitrous graphics driver to extract key data about the texture map to be displayed, such as texture transformations and mapping channels information. This step is important as each texture map can use different UV coordinates based on the mapping channel defined by the plug-in.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void Scenario2::SetupTextures(TimeValue t, DisplayTextureHelper &amp;cb)\n\
{\n\
    ISimpleMaterial *pISimpleMtl = (ISimpleMaterial*)\n\
        GetProperty(PROPID_SIMPLE_MATERIAL);\n\
\n\
    // Initialize texHandles is needed\n\
    if (!texHandle1Valid.InInterval(t))\n\
    {\n\
        PBITMAPINFO bmi;\n\
        // Bitmap creation code elided for brevity - see Scenario 1\n\
        // Create the texture resource\n\
        texHandle1 = cb.MakeHandle(bmi);\n\
    }\n\
    if (!texHandle2Valid.InInterval(t))\n\
    {\n\
        PBITMAPINFO bmi;\n\
        // Bitmap creation code elided for brevity - see Scenario 1\n\
        // Create the texture resource\n\
        texHandle2 = cb.MakeHandle(bmi);\n\
    }\n\
\n\
    // Clear the textures\n\
    pISimpleMtl-&gt;ClearTextures();\n\
\n\
    // Setup texture maps for simple display in Nitrous viewport\n\
    if (texHandle1)\n\
    {\n\
        cb.UpdateTextureMapInfo(t, ISimpleMaterial::UsageDiffuse, diffuseMap);\n\
        pISimpleMtl-&gt;SetTexture(texHandle1, ISimpleMaterial::UsageDiffuse); \n\
    }\n\
    if (texHandle2)\n\
    {\n\
        cb.UpdateTextureMapInfo(t, ISimpleMaterial::UsageOpacity, opacityMap);\n\
        pISimpleMtl-&gt;SetTexture(texHandle2, ISimpleMaterial::UsageOpacity); \n\
    }\n\
}</code></pre></li>\n\
</ul>\n\
<p><em>Scenario 3: Plug-in&apos;s viewport and rendered representation differs</em></p>\n\
<p>It is common for a material plug-in to change its appearance in the viewport compared to its rendered representation, as it might want to display debugging information or additional visual hints in the viewport. In this case the plug-in will want to override the default representation that Nitrous created for it by implementing <code>ITextureDisplay::SetupTextures()</code> as in Scenario 2. In addition to setting up the texture maps, the plug-in can call any of the <code>Set</code> functions in <code>ISimpleMaterial</code> to override the various colors, shade, and fill modes Nitrous has set automatically.</p>\n\
<h2 id=\"texture-resource-management\">Texture resource management</h2><p>It is important to note that in all of the above scenarios the texture resource used by the Nitrous viewport display system is maintained as a <code>TexHandle</code> instance. The plug-in must create a <code>TexHandle</code> for each texture map that is to be displayed in the viewport, even if Nitrous&apos; default texture map handling is not overwritten by the plug-in (see Scenario 1 above). If changes in the plug-in&apos;s parameters render the texture map invalid, then the <code>TexHandle</code> must be deleted by calling <code>TexHandle::DeleteThis()</code>, which ensures texture resources that may be used by the GPU are freed. Next time the system requests a <code>TexHandle</code>, or when setting up the textures for a <code>ISimpleMaterial</code> instance, if the <code>TexHandle</code> is invalid, the plug-in must create a new handle by calling <code>TexHandleMaker::MakeHandle()</code>.</p>\n\
<p>For an example that demonstrates the use of these interfaces, see  <em>maxsdk/samples/materials/stdmtl2.cpp</em> .</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../nitrous.html\">Nitrous</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";