var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"low-level_device_api\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Low-level Device API</h1></div><p>The following interfaces can be used in the <code>CustomRenderItemHandle</code>:</p>\n\
<ul>\n\
<li><code>IVirtualDevice</code> - This is very similar to Direct3D (D3D) <code>IDirect3DDevice</code>. You can use it to create, set, and draw graphics data.</li>\n\
<li><code>BaseMaterialHandle</code> and concrete material handles - These are different material classes. A material handle can be used for shading a render item.</li>\n\
<li><code>VertexBufferHandle</code> - This is a buffer that stores vertices with a given format. It is similar to D3D&apos;s vertex buffer.</li>\n\
<li><code>IndexBufferHandle</code> - This is a buffer that stores indices. It is similar to D3D&apos;s index buffer.</li>\n\
<li><code>TextureHandle</code> and <code>TargetHandle</code> - These two classes can be used as texture parameter for material.</li>\n\
</ul>\n\
<p>The plug-ins can provide complex display in Nitrous using these interfaces.</p>\n\
<h2 id=\"built-in-stock-material\">Built-in Stock Material</h2><p>Nitrous provides the following material classes that can be used by both the object and material plug-ins:</p>\n\
<ul>\n\
<li><code>SolidColorMaterialHandle</code></li>\n\
<li><code>VertexColorMaterialHandle</code></li>\n\
<li><code>TextureMaterialHandle</code></li>\n\
<li><code>StandardMaterialHandle</code></li>\n\
</ul>\n\
<p>To use these materials to shade vertex buffers, the plug-ins need to initialize the handle in proper time. The plug-ins must make sure that the <code>MeshRequiredStreams</code> of the material handle and the vertex buffer format is consistent, and use the <code>BaseMaterialHandle</code> functions to draw. Following is an example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Use a material handle to draw something\n\
void XXXPlugin_CustomItem::Display(DrawContext&amp; dc)\n\
{\n\
    // Assuming that the material handle is properly initialized\n\
    BaseMaterialHandle hMaterial;\n\
\n\
    hMaterial.Activate(dc);\n\
    int numPass = hMaterial.GetPassCount(dc);\n\
    for (int i = 0; i &lt; numPass; ++i)\n\
    {\n\
        // Assuming that the vertex buffer has a consistent format\n\
        hMaterial.ActivatePass(dc, i);\n\
        // Use IVirtualDevice to draw the vertex buffer\n\
    }\n\
    hMaterial.PassFinished(dc);\n\
    hMaterial.Terminate();\n\
}</code></pre><h2 id=\"customize-material-shaders\">Customize Material Shaders</h2><p>Nitrous also provides a <code>HLSLMaterialHandle</code> class that allows plug-ins to use the DirectX effect. Plug-ins can directly use the DirectX effect string or file ( <em>\\</em>.fx* ) to initialize the material handle and use it as other material handles.</p>\n\
<p>When Nitrous is in DirectX 9 (DX9) mode, this class only supports DX9 effects, and when Nitrous is in DirectX 11 (DX11) mode, this class only supports DX11 effects. Plug-ins can get the device capability through <code>IVirtualDevice::FeatureLevel()</code>.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../nitrous.html\">Nitrous</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";