var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"plug-in_display_interface\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Plug-in Display Interface</h1></div><h2 id=\"overview\">Overview</h2><p>The Plugin Display SDK (previously called the Nitrous SDK) can be used by object/modifier plugins and material plugins to customize the way they display in the Nitrous viewport. The following diagram is a high-level overview of the classes and interfaces in this SDK.</p>\n\
<p><img src=\"./images/GUID-7DEBAC3F-EA13-4E28-B15C-146711114584-low.png\" alt></p>\n\
<h2 id=\"legacy-object-modifier-plug-in-display-interface\">Legacy Object/Modifier Plug-in Display Interface</h2>\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>   <strong></strong> The <code>IObjectDisplay</code> interface is removed as of 3ds Max 2015, and <code>IObjectDisplay2</code> must now be used.</p>\n\
\n\
</div></div>\n\
\n\
<p>Traditionally in 3ds Max, the object/modifier plug-in is displayed by implementing its <code>Display()</code> function. In this function, the plug-in uses the <code>GraphicsWindow</code> interface to draw lines, points, markers, meshes, and text as shown in the following example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">int MyObject::Display(\n\
    TimeValue t, \n\
    INode* inode, \n\
    ViewExp* vpt, \n\
    int flags)\n\
{\n\
    GraphicsWindow* gw = vpt-&gt;getGW();\n\
    gw-&gt;setTransform(inode-&gt;GetObjectTM(t));\n\
    mMesh.render(gw, inode-&gt;Mtls(), NULL, COMP_ALL, inode-&gt;NumMtls());\n\
    // draw some lines\n\
    gw-&gt;polyline(...);\n\
}</code></pre><p>This example code works only for legacy display drivers, such as legacy software driver, legacy d3d driver, and legacy OpenGL driver.</p>\n\
<h2 id=\"implementing-code-iobjectdisplay2-code-\">Implementing <code>IObjectDisplay2</code></h2><p>To make an object display correctly under Nitrous, the plug-in must implement the Nitrous display interface, <code>IObjectDisplay2</code>. specifically, the plug-in needs to:</p>\n\
<ul>\n\
<li>Make sure its <code>GetInterface()</code> method returns the <code>IObjectDisplay2</code> interface.</li>\n\
<li>Override the <code>GetObjectDisplayRequirement()</code> method to reuse or skip the whole legacy <code>Display()</code> function. If the legacy <code>Display()</code> function is still needed in Nitrous, the plug-in needs to skip the unsupported legacy display code in its <code>Display()</code> method.</li>\n\
<li>As a replacement for the skipped legacy display code, the plug-in needs to override <code>PrepareDisplay()</code>, <code>UpdatePerNodeItems()</code>, <code>UpdatePerViewItems()</code>, and provide <code>RenderItems</code> to Nitrous.</li>\n\
</ul>\n\
<p>Below is the declaration of IObjectDisplay2:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class IObjectDisplay2 : public BaseInterface\n\
{\n\
public:\n\
    virtual unsigned long GetObjectDisplayRequirement() const;\n\
virtual bool PrepareDisplay( \n\
    const UpdateDisplayContext&amp; updateDisplayContext) = 0;\n\
virtual bool UpdatePerNodeItems( \n\
    const UpdateDisplayContext&amp; updateDisplayContext,\n\
    UpdateNodeContext&amp; nodeContext,\n\
    IRenderItemContainer&amp; targetRenderItemContainer) = 0;\n\
virtual bool UpdatePerViewItems( \n\
    const UpdateDisplayContext&amp; updateDisplayContext,\n\
    UpdateNodeContext&amp; nodeContext, \n\
    UpdateViewContext&amp; viewContext,\n\
    IRenderItemContainer&amp; targetRenderItemContainer);\n\
};</code></pre><h3 id=\"code-flow-for-object-plug-in-display\">Code Flow for Object Plug-in Display</h3><p>The following pseudo-example shows the steps in displaying an object plug-in in the Nitrous viewport:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void Nitrous::DrawAllViewports()\n\
{\n\
    // Preparation of each object. Irrespective of the object being referenced \n\
    // by many nodes, each object prepares the display data \n\
    // only once.\n\
    foreach object in scene\n\
    {\n\
      object.PrepareDisplay\n\
    }\n\
    // Add per-node display items\n\
    foreach node in scene\n\
    {\n\
      node.object.UpdatePerNodeItems\n\
    }\n\
    // display pass for all views\n\
    foreach view \n\
    {\n\
      foreach node in scene\n\
      {\n\
        node.object.UpdatePerViewItems\n\
      }\n\
      foreach RenderItem\n\
      {\n\
        RenderItem.Draw\n\
      }\n\
    }\n\
}</code></pre><h3 id=\"making-the-code-iobjectdisplay2-code-interface-accessible\">Making the <code>IObjectDisplay2</code> Interface Accessible</h3><p>The <code>IObjectDisplay2</code> interface is exposed by the <code>BaseObject</code> class, which is the root class for all objects and modifiers. The plug-in needs to redirect the interface ID to <code>BaseObject::GetInterface()</code>, when it meets an <code>IOBJECT_DISPLAY_INTERFACE_ID</code> in its <code>GetInterface()</code> method as shown in the following example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">BaseInterface* TriObject::GetInterface(Interface_ID iid)\n\
{\n\
    if (iid == IOBJECT_DISPLAY2_INTERFACE_ID)\n\
    {\n\
        // Redirect the IObjectDisplay2 interface ID to parent class\n\
        return GeomObject::GetInterface(iid);\n\
    }\n\
    ...\n\
}</code></pre><h3 id=\"reusing-legacy-display-code\">Reusing Legacy Display Code</h3><p>Legacy display code (<code>XXXObject::Display()</code> that uses <code>GraphicsWindow</code> interface) for drawing lines, points, markers, and text is supported by Nitrous. However, the performance is not good.</p>\n\
<p>All legacy display code that uses <code>Mesh::render()</code>, <code>MNMesh::render()</code>, or any other mesh display functions that use <code>GraphicsWindow</code> is not supported by Nitrous. There might be some display, but these functions must no longer be used. These function calls must be skipped in <code>XXXObject::Display</code>.</p>\n\
<p>By default, the legacy display function of the plug-in is called. Plug-ins can override <code>GetObjectDisplayRequirement()</code> to change this behavior and prevent Nitrous from calling the legacy display function. The following example shows how to handle the legacy display code:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Unsigned long MyObject:: GetObjectDisplayRequirement() const\n\
{\n\
    // Make sure legacy Display code be called under Nitrous\n\
    // Because we still want to reuse some legacy display code\n\
    return ObjectDisplayRequireLegacyDisplayMode;\n\
}\n\
\n\
int MyObject::Display(...)\n\
{\n\
gw-&gt;setRndLimits(...);\n\
if (!MaxSDK::Graphics::IsRetainedModeEnabled())\n\
{\n\
    // only render mesh if Nitrous is not enabled.\n\
        mMesh.render(...);\n\
}\n\
gw-&gt;polyline(...);\n\
}</code></pre><p>As a replacement for <code>Mesh::render()</code>, the plug-ins must override the <code>IObjectDisplay2</code> methods to provide render items to Nitrous.</p>\n\
<h3 id=\"-code-iobjectdisplay2-getobjectdisplayrequirement-code-\"><code>IObjectDisplay2::GetObjectDisplayRequirement()</code></h3><p>This function is an extension of the <code>IObjectDisplay::RequiresSupportForLegacyDisplayMode()</code> method. It returns a flag that can be a combination of the following values:</p>\n\
<table>\n\
<colgroup>\n\
<col width=\"50%\">\n\
<col width=\"50%\">\n\
</colgroup>\n\
<thead>\n\
<tr class=\"header\">\n\
<th>Flag</th>\n\
<th>Description</th>\n\
</tr>\n\
</thead>\n\
<tbody>\n\
<tr class=\"odd\">\n\
<td>`ObjectDisplayRequireLegacyDisplayMode`</td>\n\
<td>If this flag is set, the plug-in\\&apos;s legacy `Display()` function is called in Nitrous.</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>`ObjectDisplayRequireUpdatePerViewItems`</td>\n\
<td>If this flag is set, the `IObjectDisplay2::UpdatePerViewItems()` function is called.</td>\n\
</tr>\n\
</tbody>\n\
</table>\n\
\n\
<p>By default, the <code>IObjectDisplay2::GetObjectDisplayRequirement()</code> function returns <code>ObjectDisplayRequireLegacyDisplayMode</code>, which means that the legacy display code is not used, and <code>IObjectDisplay2::UpdatePerViewItems()</code> is not called.</p>\n\
<h3 id=\"providing-render-items-for-code-mesh-code-and-code-mnmesh-code-\">Providing Render Items for <code>Mesh</code> and <code>MNMesh</code></h3><p>To provide mesh render items under Nitrous, object plugins should override the methods in the <code>IMeshDisplay2</code> interface to build the render items, and pass the built render items to the <code>targetRenderItemContainer</code> parameter in <code>IObjectDisplay2::UpdatePerNodeItems()</code> or <code>IObjectDisplay2::UpdatePerViewItems()</code>.</p>\n\
<p>For more details of render items, see the <a href=\"about_renderitem.html\"\">About RenderItem</a>.</p>\n\
<h3 id=\"-code-iobjectdisplay2-preparedisplay-code-\"><code>IObjectDisplay2::PrepareDisplay()</code></h3><p>An object can be referenced by multiple nodes at the same time. Therefore, different nodes might display each object differently (for example, using backface cull or non-backface cull, vertex color shading or material shading, and so on). The <code>IObjectDisplay2::PrepareDisplay()</code> function provides the object plug-ins a chance to prepare display data for all nodes, thus avoiding duplicate display data generation for each node or each view.</p>\n\
<p>If an object contains a <code>Mesh</code> or <code>MNMesh</code> to display, the <code>IMeshDisplay2::PrepareDisplay()</code> is usually called in this function.</p>\n\
<h3 id=\"-code-iobjectdisplay2-updatepernodeitems-code-\"><code>IObjectDisplay2::UpdatePerNodeItems()</code></h3><p>This function is called for each owner node of an object plug-in. Plug-ins can create different render items depending on the input node properties. For example, if an object wants to display a high-resolution mesh when the node is selected, and display a low-resolution mesh when node is not selected, the plug-in can use this function to create different render items.</p>\n\
<h3 id=\"-code-iobjectdisplay2-updateperviewitems-code-\"><code>IObjectDisplay2::UpdatePerViewItems()</code></h3><p>This function is called for each viewport and each owner node of an object plug-in. If the plug-in needs to display or hide certain render items for a view property, the plug-in can override this function and update the render item based on the input view properties.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>   <strong></strong> This function is optional. If the plug-in does not have a view-dependent render item, it does not need to override this function, and therefore the performance is not affected.</p>\n\
\n\
</div></div>\n\
\n\
<h3 id=\"example-implementation-of-code-iobjectdisplay2-code-interface\">Example Implementation of <code>IObjectDisplay2</code> Interface</h3><p>The following code shows an example of object display. In this example, a higher resolution mesh is shown when the object is selected.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class MyObject:public Object, public IObjectDisplay2\n\
{\n\
public:\n\
    // Object declaration code.\n\
private:\n\
    // High resolution mesh is displayed when node is selected\n\
    Mesh mHighResMesh;\n\
    // Low resolution mesh is displayed when node is unselected\n\
    Mesh mLowResMesh;\n\
};\n\
\n\
BaseInterface* MyObject::GetInterface(Interface_ID iid)\n\
{\n\
    if (iid == IOBJECT_DISPLAY2_INTERFACE_ID)\n\
    {\n\
        return (IObjectDisplay2*)this;\n\
    }\n\
\n\
    return __super::GetInterface(iid);\n\
}\n\
\n\
unsigned long MyObject::GetObjectDisplayRequirement() const\n\
{\n\
    // we do not want legacy display code called\n\
    // we also do not have any per-view items\n\
    return 0;\n\
}\n\
\n\
bool MyObject::PrepareDisplay(    \n\
    const UpdateDisplayContext&amp; displayContext)\n\
{\n\
    GenerateMeshRenderItemsContext renderItemContext;\n\
    renderItemContext.GenerateDefaultContext(displayContext);\n\
\n\
    IMeshDisplay2* pMeshDisplay = NULL;\n\
\n\
    pMeshDisplay = static_cast&lt;IMeshDisplay2*&gt;(\n\
        mHighResMesh.GetInterface(IMesh_DISPLAY2_INTERFACE_ID));\n\
    if (pMeshDisplay != NULL)\n\
        pMeshDisplay-&gt;PrepareDisplay(renderItemContext);\n\
    pMeshDisplay = static_cast&lt;IMeshDisplay2*&gt;(\n\
        mLowResMesh.GetInterface(IMesh_DISPLAY2_INTERFACE_ID));\n\
    if (pMeshDisplay != NULL)\n\
        pMeshDisplay-&gt;PrepareDisplay(renderItemContext);\n\
    return true;\n\
}\n\
\n\
bool MyObject::UpdatePerNodeItems(\n\
    const UpdateDisplayContext&amp; displayContext,\n\
    UpdateNodeContext&amp; nodeContext,\n\
    IRenderItemContainer&amp; targetRenderItemContainer)\n\
{\n\
    GenerateMeshRenderItemsContext renderItemContext;\n\
    renderItemContext.GenerateDefaultContext(displayContext);\n\
    renderItemContext.RemoveInvisibleMeshElementDescriptions(\n\
        nodeContext.GetRenderNode());\n\
\n\
    IMeshDisplay2* pMeshDisplay = NULL;\n\
    RenderItemHandleArray renderItems;\n\
    if (nodeContext.GetRenderNode().GetSelected())\n\
    {\n\
        // If node is selected, use high resolution mesh\n\
        pMeshDisplay = static_cast&lt;IMeshDisplay2*&gt;(\n\
            mHighResMesh.GetInterface(IMesh_DISPLAY2_INTERFACE_ID));\n\
    }\n\
    else\n\
    {\n\
        // Otherwise use low resolution mesh\n\
        pMeshDisplay = static_cast&lt;IMeshDisplay2*&gt;(\n\
            mLowResMesh.GetInterface(IMesh_DISPLAY2_INTERFACE_ID));\n\
    }\n\
    if (pMeshDisplay != NULL)\n\
    {\n\
        pMeshDisplay-&gt;GetRenderItems(\n\
            renderItemContext,\n\
            nodeContext,\n\
            renderItems);\n\
    }\n\
    targetRenderItemContainer.ClearAllRenderItems();\n\
    targetRenderItemContainer.AddRenderItems(renderItems);\n\
    return true;\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../nitrous.html\">Nitrous</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";