var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"automatic_gamma_correction_best_\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Automatic Gamma Correction Best Practices</h1></div><p>The automatic gamma correction method chooses a gamma value based on the file type when loading a bitmap file (in certain cases) and when saving a bitmap file. You do not need to care about the gamma correction because it is automatically done based on the file type, bit depth, and other properties.</p>\n\
<p>Following are the limitations in the previous method for gamma correction:</p>\n\
<ul>\n\
<li>If a file header does not contain a gamma value when loading, the globally defined gamma value is always used and the file type is ignored. The next file that you attempt to load from the File dialog also uses the globally defined gamma value even if it contains a better value in the file header.</li>\n\
<li>The system default gamma value is always used for saving files and the file type is ignored.</li>\n\
</ul>\n\
<p>In the automatic gamma correction method, if a file header does not contain a gamma value, the file type is considered to determine whether the file must be stored linearly. For more information about automatic gamma correction, see the <a href=\"./../what_s_new/archive/what_s_new_3ds_max_2014_sdk/automatic_gamma_correction.html\">Automatic Gamma Correction</a> topic.</p>\n\
<p>The mapping of the flags (<code>BMM_CUSTOM_FILEGAMMA</code> and <code>BMM_CUSTOM_GAMMA</code>) with the gamma options in the File dialog has changed in the automatic gamma correction method. Additionally, the meaning of the <code>BMM_CUSTOM_FILEGAMMA</code> flag has also changed. A new <code>BitmapIOMetaData</code> interface in the <code>BitmapIO</code> class is used to determine whether the file types are saved and loaded linearly or with an embedded gamma. For more information about the SDK changes, see the <a href=\"./../what_s_new/archive/what_s_new_3ds_max_2014_sdk/automatic_gamma_correction/sdk_changes_for_automatic_gamma_.html\">SDK Changes for Automatic Gamma Correction</a> topic.</p>\n\
<h2 id=\"best-practices\">Best Practices</h2><p>Following are some best practices for avoiding gamma related issues when writing code:</p>\n\
<ul>\n\
<li>Use <code>BitmapInfo</code> and not filenames. If you need to use filenames, avoid changing the extension. If you change the extension, call <code>BitmapInfo::ResetPiData()</code>.</li>\n\
<li>Use <code>BitmapInfo::GetEffectiveGamma()</code> if you need to know the gamma of a bitmap and to untangle the various flags correctly. Moreover, it is better to ignore the gamma and always use <code>GetLinearPixels()</code> (instead of <code>GetPixels()</code>) to obtain the proper linear value.</li>\n\
<li>Implement the <code>BitmapIOMetaData</code> interface if you are writing a <code>BitmapIO</code> plug-in that needs linear data.</li>\n\
<li>Bitmaps loaded from the file system might have a different encoding or gamma values and might be non-linear.</li>\n\
<li>Only linear bitmaps can be saved with proper gamma correction applied, so you must linearize all bitmaps before saving. You can copy the bitmap to make it linear and at least 16-bit.</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../best_practices.html\">Best Practices</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";