var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"general_best_practices\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>General Best Practices</h1></div><p>Following is a list of general best practices to follow, and pitfalls to avoid.</p>\n\
<ul>\n\
<li><p>Instead of using stack-allocated character buffers, it is a best practice to use the TSTR, which in Unicode builds maps to the <code>WStr</code> class. For example instead of using the following:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void process_long_string(TCHAR* longstr) \n\
{ \n\
  TCHAR buf[256]; \n\
  __tcscpy (buf, longstr); \n\
}</code></pre><p>You can use:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void process_long_string(TCHAR* longstr) \n\
{ \n\
  TSTR buf = longstr; \n\
}</code></pre></li>\n\
<li><p>Compile your plug-in with the supported version of Visual C++. See <a href=\"./../about_the_3ds_max_sdk/sdk_requirements.html\">SDK Requirements</a> for a list of supported compilers.</p>\n\
</li>\n\
<li><p>Controller plug-ins must implement <code>ILockedTrack</code>. See <a href=\"./../3ds_max_sdk_features/animation_time_and_controllers/animation_track_locking.html\">Animation Track Locking</a> for more information.</p>\n\
</li>\n\
<li><p>Use <code>Interface::GetAppHFont()</code> to get the appropriate default font settings.</p>\n\
</li>\n\
<li><p>Use <code>ClassDesc2</code> instead of <code>ClassDesc</code> to take advantage of the <a href=\"./../3ds_max_sdk_features/function_publishing.html\">Function Publishing</a> API.</p>\n\
</li>\n\
<li><p>Never store asset paths as strings. Instead use an instance of <code>MaxSDK::AssetManagement::AssetUser</code>.</p>\n\
</li>\n\
<li><p>Use <code>DbgAssert()</code> instead of <code>assert()</code>.</p>\n\
</li>\n\
<li><p>Plug-ins must always store parameters and serializable data in a parameter block.</p>\n\
</li>\n\
<li><p>Always use gencid.exe to generate new class IDs for code based on samples and tutorials.</p>\n\
</li>\n\
<li><p>Pay special attention to interpreting correctly the <code>lParam</code> parameter send with the <code>WM_NOTIFY</code> message passed to hook procedures registered for file I/O dialogs. See <a href=\"handling_wm_notify_messages.html\"\">Handling WM_NOTIFY Messages</a> for more information.</p>\n\
</li>\n\
<li><p>Plug-in finalization code must not make a call to a global utility plug-in (a class derived from <code>GUP</code>) because these are unloaded first.</p>\n\
</li>\n\
<li><p>When writing objects that support interfaces, it is a best practice to derive from <code>FPInterface</code> so that you can provide function publishing functionality either immediately or in the future.</p>\n\
</li>\n\
<li><p>Be careful when storing interfaces passed to a method by 3ds Max. In certain cases, the validity of these interfaces is limited to the duration of the function call. Some examples include:</p>\n\
<ul>\n\
<li>When a modifier becomes active in the modify branch of the command panel, its <code>BeginEditParams()</code> method is called and an <code>IObjParams</code> interface is passed in. This interface pointer is defined to be valid until (and including) the modifier&apos;s <code>EndEditParams()</code> method is called. If a plug-in were to hang on to the pointer outside of this interval and then call one of its methods, the result is undefined.</li>\n\
<li>The <code>INode</code> interface pointer, which when passed in to <code>Object::Display()</code> is valid for only the duration of the function call.</li>\n\
</ul>\n\
</li>\n\
<li><p>Use <code>dynamic_cast</code> instead of using the unsafe <code>static_cast</code> and always check the result for NULL to make sure that it succeeded.</p>\n\
</li>\n\
<li><p>Use <code>Interface::ReleaseViewport</code> after you finish with a <code>ViewExp</code> pointer to avoid causing a memory leak.</p>\n\
</li>\n\
<li><p>Be careful to always use the <code>ParamID</code> to access individual parameters in a parameter block (<code>IParamBlock</code>).</p>\n\
</li>\n\
<li><p>Only use a simple value type (for example, primitive types or classes that do not manage memory) or pointers in a <code>Tab</code>&lt;&gt; collection.</p>\n\
</li>\n\
<li><p>Use <code>MaxSDK::WindowsMessageFilter</code> for filtering windows messages in progress dialogs. See <a href=\"./../3ds_max_sdk_features/user_interface/windows_message_filtering.html\">Windows Message Filtering</a> for more information.</p>\n\
</li>\n\
<li><p>Use <code>P_VERSION</code> for parameter block descriptors and update the version number whenever the <code>ParamBlockDesc2</code> is changed.</p>\n\
</li>\n\
<li><p>Never change parameter IDs, instead just add new parameter IDs and deprecate old ones.</p>\n\
</li>\n\
<li><p>When writing modifiers avoid using <code>PART_ALL</code> when specifying the channels required and modified. Instead, explicitly set the correct channel flags.</p>\n\
</li>\n\
<li><p>Do not throw uncaught exceptions in 3ds Max plug-ins.</p>\n\
</li>\n\
<li><p>Surround all calls to <code>ShapeObject::CopyBaseData()</code> with calls to <code>Hold::Suspend()</code> and <code>Hold::Resume()</code>. See <a href=\"./../3ds_max_sdk_features/modeling/shapes_and_splines/calling_copybasedata.html\">Calling CopyBaseData()</a>.</p>\n\
</li>\n\
<li><p>If a scene evaluation is performed in a post-load callback, it is possible that other objects are not sufficiently initialized to be evaluated. When implementing a post-load callback (a class derived from <code>PostLoadCallBack</code>) there are two considerations to assure 3ds Max stability during scene file loads:</p>\n\
<ul>\n\
<li>Your object must be safe to evaluate if it is evaluated before your post-load callback is run.</li>\n\
<li>You must not do a scene/node evaluation (for example, do not change the scene hierarchy and do not call <code>INode::EvalWorldState()</code>) in a post-load callback.</li>\n\
</ul>\n\
</li>\n\
<li><p>Do not place objects with non-trivial constructors, copy constructors, or destructors in a <code>Tab&amp;lt;&amp;gt;</code> template.</p>\n\
</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../best_practices.html\">Best Practices</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";