var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"handling_requests_for_interfaces\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Handling Requests for Interfaces</h1></div><p>As mentioned in the <a href=\"./../what_s_new/archive/what_s_new_3ds_max_2012_sdk/interface_querying_2012.html\">Interface Querying</a> topic, incorrect implementation of <code>InterfaceServer::GetInterface()</code> might lead to incorrect display of the plug-in in the Nitrous viewport.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> BaseInterface*\n\
InterfaceServer::GetInterface(Interface_ID iid);</code></pre><p>Often a plug-in class inherits from multiple base classes, some of which are <a href=\"./../3ds_max_sdk_features/function_publishing/mixin_interfaces.html\">Mixin Interfaces</a> while others are base classes representing the plug-ins main type (class <code>SimpleObject</code>, class <code>Modifier</code>, etc). The plug-in&apos;s implementation of <code>InterfaceServer::GetInterface()</code> needs to allow for correct handling of requests for:</p>\n\
<ul>\n\
<li>One or more specific mixin interfaces it implements (such as those derived from class <code>FPMixinInterface</code> or class <code>BaseInterface</code>).</li>\n\
<li>The base mixin interfaces (such as class <code>FPMixinInterface</code>, class <code>FPInterfaceDesc</code>, class <code>BaseInterface</code>, class <code>InterfaceServer</code>).</li>\n\
<li>Any mixin interfaces their main parent classes may support (such as class <code>MaxSDK::Graphics::IObjectDisplay</code>). The classes derived from class <code>Animatable</code> are considered main parent classes which define the type of the plug-in. Examples include class <code>SimpleObject</code>, class <code>Modifier</code>, etc.</li>\n\
</ul>\n\
<p>A couple of examples of incorrect but typical implementations of <code>InterfaceServer::GetInterface()</code> are provided below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// A mixin interface\n\
#define IID_SOME_MIXIN_INTERFACE Interface_ID (0x627c2f9c, 0x52671832)\n\
class SomeMixinInterface : public FPMixinInterface  \n\
{ \n\
    // Does not implement GetInterface(Interface_ID iid)    \n\
    // Details elided for brevity \n\
}\n\
\n\
// A plugin class that derives from a non-mixin class and a mixin interface class\n\
class MyPlugin : public SimpleObject, public SomeMixinInterface  \n\
{\n\
    // details elided for brevity \n\
}\n\
\n\
// Incorrect GetInterface() implementation - does not handle requests for base mixin interfaces\n\
BaseInterface* MyPlugin::GetInterface(Interface_ID iid)\n\
{\n\
    if (iid == IID_SOME_MIXIN_INTERFACE) return (SomeMixinInterface*)this;\n\
    return SimpleObject::GetInterface(iid);\n\
}\n\
\n\
// Incorrect GetInterface() implementation - does not handle requests for \n\
// base mixin interfaces supported by the main parent class SimpleObject\n\
BaseInterface* MyPlugin::GetInterface(Interface_ID iid)\n\
{\n\
    if (iid == IID_SOME_MIXIN_INTERFACE) return (SomeMixinInterface*)this;\n\
    return FPMixinInterface::GetInterface(iid);\n\
}</code></pre><p>A correct implementation of the plug-in&apos;s implementations of <code>InterfaceServer::GetInterface()</code> would look like this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Correct GetInterface() implementation \n\
BaseInterface* MyPlugin::GetInterface(Interface_ID iid)\n\
{\n\
    // Handle request for specific mixin interface\n\
    if (iid == IID_SOME_MIXIN_INTERFACE) return (SomeMixinInterface*)this;\n\
    else \n\
    {\n\
        // Handle requests for base mixin interfaces\n\
        BaseInterface* baseIfc = FPMixinInterface::GetInterface(iid);\n\
        if (baseIfc) return intf;\n\
    }\n\
    // Handle requests for mixin interfaces supported by main base class\n\
    return SimpleObject::GetInterface(iid);\n\
}</code></pre><h2 id=\"how-to-test-for-correct-handling-of-interface-requests\">How to Test for Correct Handling of Interface Requests</h2><p>A simple test can be written in MAXScript to ensure that the plug-in overriding of <code>InterfaceServer::GetInterface()</code> is correct. The test works by querying a plug-in instance for a specific interface. The following MAXScript code shows the test for ensuring that requests for interface <code>IObjectDislay</code> identified by interface id <code>IOBJECT_DISPLAY_INTERFACE_ID</code> are serviced correctly by the Box plug-in:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">-- create an instance of a box\n\
boxInstance = Box -- &quot;Box&quot; is the name of the class of the box plug-in object\n\
-- define the value of the interface id to query; see maxsdk\\include\\graphics\\IObjectDisplay.h\n\
IOBJECT_DISPLAY_INTERFACE_ID = #(0x39820fa0,0x6311340d) \n\
-- query the box instance for IOBJECT_DISPLAY_INTERFACE_ID\n\
res = getInterface boxInstance IOBJECT_DISPLAY_INTERFACE_ID\n\
-- Use the MAXScript assertion mechanism to signal a test failure\n\
assert_true (res) message: (&quot;Box failed to handle requests for IOBJECT_DISPLAY_INTERFACE_ID&quot;)\n\
-- Print all assertion messages\n\
for msg in AssertReporter.GetMessages() do print msg</code></pre><p>The test uses an undocumented feature of the <code>getInterface</code> MAXScript method which interprets an array of two integers as an <code>Interface_ID</code> value and call the plug-in&apos;s override of <code>InterfaceServer::GetInterface(Interface_ID)</code> with it. It returns true if the plug-in returns a valid (non-NULL) pointer, false otherwise.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../best_practices.html\">Best Practices</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";