var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"reference_system_best_practices\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Reference System Best Practices</h1></div><p>The following is a list of best practices regarding usage of the reference system:</p>\n\
<ul>\n\
<li><p>Make sure you choose correctly between storing a pointer or a reference in your plug-in to a scene entity. Storing a pointer to a scene entity is seldom safe because the pointer could become invalid if the scene entity is deleted and your plug-in fails to update the pointer. Although pointers to scene entities could be used as local variables in plug-in member functions, they should not be cached as plug-in data members and their life span should be as short as possible.</p>\n\
<p>For information on converting pointers to weak or indirect references see <a href=\"./../3ds_max_sdk_features/reference_system/kinds_of_references/indirect_references/using_indirect_references_instea.html\">Using Indirect References Instead of Pointers</a>.</p>\n\
<p>References can be stored as plug-in member variables but it is the plug-in&#39;s responsibility to properly implement their management, so that they do get updated, do not prevent the dependent entity to be destroyed, etc.</p>\n\
<p>Be very careful if holding a pointer to a scene entity in a local variable when making system calls that may cause that scene entity to be deleted. Do not rely on Restore Objects being created to prevent scene entity deletion as, particularly when dealing with MAXScript, the hold system may be suspended. In such cases, instead of directly holding a pointer you should hold the pointer via a <code>SingleRefMaker</code>, <code>SingleWeakRefMaker</code>, <code>TypedSingleRefMaker</code>, or <code>TypedSingleWeakRefMaker</code>.</p>\n\
</li>\n\
<li><p>When observing nodes always use weak references instead of strong references. This is because nodes can be deleted by the user.</p>\n\
</li>\n\
<li><p>Make sure you choose the right type of reference your plug-in needs to create to a scene entity it depends on. See the topic <a href=\"./../3ds_max_sdk_features/reference_system/kinds_of_references.html\">Kinds of References</a> for more information.</p>\n\
</li>\n\
<li><p>Never store pointers to scene nodes as plug-in data members because they can be invalidated at any time (e.g. due to the user deleting them). Use a node monitor instead (<code>INodeMonitor</code> or <code>INodeTransformMonitor</code>).</p>\n\
</li>\n\
<li><p>A plug-in&#39;s implementation of <code>ReferenceMaker::SetReference()</code> should always assign the <code>ReferenceTarget</code> passed in by the caller to the corresponding plug-in data member, as opposed to implementing its own logic to update that reference.</p>\n\
</li>\n\
<li><p>A plug-in that needs to maintain a variable number of references, should use the class <code>IRefTargContainer</code>.</p>\n\
</li>\n\
<li><p>In order to allow 3ds Max to manage the life-time of your plug-in instances, the plug-in would not override <code>Animatable::DeleteThis()</code> or <code>ReferenceTarget::AutoDelete()</code>. When a plug-in needs to behave as a singleton, it should overwrite <code>ReferenceTarget::AutoDelete()</code> to return <code>REF_SUCCEED</code>. In this case, the plug-in should take care of deleting itself its sole instance since 3ds Max will not be able to do it.</p>\n\
</li>\n\
<li><p>Do not call the delete operator on a pointer to an object derived from <code>Animatable</code> or any of its sub-classes. For more information see <a href=\"./../3ds_max_sdk_features/reference_system/reference_object_lifetime_manage.html\">Reference Object Lifetime Management</a>.</p>\n\
</li>\n\
<li><p>For managing one single reference, a plug-in should consider using an instance of class <code>SingleRefMaker</code> or class <code>SingleWeakRefMaker</code>.</p>\n\
</li>\n\
<li><p>Initialize <code>ReferenceTarget</code> pointers to <code>NULL</code> before the first call to <code>ReferenceMaker::ReplaceReference()</code>. This should be done in the member initialization list of the constructor.</p>\n\
</li>\n\
<li><p>When receiving a <code>REFMSG_TARGET_DELETED</code> message a <code>ReferenceMaker</code> should always set its reference to the <code>ReferenceTarget</code> instance being deleted to <code>NULL</code>.</p>\n\
</li>\n\
<li><p>If the plug-in wishes to delete itself in response to <code>REFMSG_TARGET_DELETED</code>, it should return <code>REF_AUTO_DELETE</code> from its implementation of <code>ReferenceMaker::NotifyRefChanged()</code>.</p>\n\
</li>\n\
<li><p>Never call <code>ReferenceMaker::SetReference()</code>, always call <code>ReferenceMaker::ReplaceReference()</code> instead.</p>\n\
</li>\n\
<li><p>Use <code>Animatable::GetAnimByHandle()</code> and compare the result to <code>NULL</code> to verify that an animatable (or <code>ReferenceTarget</code>) pointer is valid.</p>\n\
</li>\n\
<li><p>If appropriate call <code>ReferenceMaker::DeleteAllRefs()</code> in the destructor of a plug-in that derives from reference maker, to assure that all references are deleted. Normally if an object is deleted correctly then this shouldn&#39;t be necessary.</p>\n\
</li>\n\
<li><p>Send reference messages sparingly. They can slow down the system. For example if a large number of changes are made to a plug-in at once, then the plug-in should try to send only a single <code>REFMSG_CHANGE</code> message for the set of changes.</p>\n\
</li>\n\
<li><p>When setting <code>ReferenceTarget</code> pointers to <code>NULL</code> in repsonse to <code>REFMSG_TARGET_DELETED</code>, be sure to only assign one pointer to the <code>ReferenceTarget</code> to <code>NULL</code> in the case that you have multiple references to the same target.</p>\n\
</li>\n\
<li><p>A reference maker should support the setting of <code>NULL</code> references for any of its reference indexes.</p>\n\
</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../best_practices.html\">Best Practices</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";