var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"rtti_and_getinterface\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>RTTI and GetInterface()</h1></div><p>Although run-time type information (RTTI) allows for querying polymorphic objects at run-time and safely casting down a class hierarchy, RTTI is not a replacement for the 3ds Max SDK <code>GetInterface</code> mechanism. RTTI improves code robustness, as illustrated by the following example that uses <code>BaseInterface::GetInterface()</code> with <code>dynamic_cast</code>, instead of <code>Animatable::ClassID()</code> with <code>static_cast</code>.</p>\n\
<p>The following is the recommended way to cast an object to a particular interface:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">template&lt;typename Interface_T&gt;\n\
Interface_T* GetInterface(BaseInterface* base, Interface_ID id)\n\
{\n\
  if (base == NULL)\n\
    return NULL;\n\
\n\
  return dynamic_cast&lt;Interface_T*&gt;(base-&gt;GetInterface(id));\n\
}</code></pre><p><code>Animatable::Class_ID()</code> and <code>static_cast</code> should only be used in the case where you need to retrieve a class that does not have an interface id.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">template&lt;typename Interface_T&gt;\n\
Interface_T* GetAnimInterface(Animatable* anim, Class_ID cid)\n\
{\n\
  if (anim == NULL)\n\
    return NULL;\n\
\n\
  if (anim-&gt;ClassID() != cid))\n\
    return NULL;\n\
\n\
  return static_cast&lt;Interface_T*&gt;(anim);\n\
}</code></pre><p>In addition, superclass ids (<code>SClass_ID</code>) and class ids (<code>Class_ID</code>) are necessary to create instances of objects based on type information using <code>Interface::CreateInstance()</code>.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../best_practices.html\">Best Practices</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";