var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"security_best_practices\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Security Best Practices</h1></div><p>In the <strong>3ds Max 2020</strong> SDK we introduced the header <code>banned.h</code> to help plug-in developers identify where their code uses potentially insecure Windows APIs.  The 3ds Max team uses these same headers to improve the security of 3ds Max, and third party developers are encouraged to consider using this resource to reduce vulnerabilities in their code.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p><strong></strong> The implementation of <code>banned.h</code> is a continuing effort.  Not all code in the 3ds Max SDK has been compiled with <code>banned.h</code>, so it is likely that there are 3ds Max APIs that will not work when compiled with this header.  Third party developers can start using <code>banned.h</code> where appropriate, but may not be able to fully implement it for plug-ins yet.</p>\n\
\n\
</div></div>\n\
\n\
<p>There are two additional headers in the 3ds Max SDK to support <code>banned.h</code>:</p>\n\
<ul>\n\
<li><code>3dsmax_banned.h</code> - a wrapper for <code>banned.h</code> that pre-includes some headers</li>\n\
<li><code>3dsmax_qt_pragma_warning.h</code> - a header to suppress warnings in Qt based projects</li>\n\
</ul>\n\
<h2 id=\"overview-of-code-banned-h-code-and-code-3dsmax_banned-h-code-\">Overview of <code>banned.h</code> and <code>3dsmax_banned.h</code></h2><p>Microsoft describes <code>banned.h</code> as &quot;a sanitizing resource that is designed to help developers avoid using and help identify and remove banned functions from code that may lead to vulnerabilities. Banned functions are those calls in code that have been deemed dangerous by making it relatively easy to introduce vulnerabilities into code during development.&quot;  </p>\n\
<p>See the <a href=\"https://cloudblogs.microsoft.com/microsoftsecure/2012/08/30/microsofts-free-security-tools-banned-h/\" target=\"_blank\">Article on banned.h</a>.</p>\n\
<p>The <code>3dsmax_banned.h</code> is a minimal wrapper for <code>banned.h</code> intended as a common starting point to compile 3ds Max plug-ins in Visual Studio. It pre-includes a minimal set of common system and 3rd-party library headers, to ignore compilation errors caused by non-compliant external code. It conditionally pre-includes a non-compliant subset of Qt headers (added to <code>3dsmax_banned.h</code> to avoid redundant code, since many projects depend on Qt). </p>\n\
<p>You can create your own project-specific wrapper around <code>3dsmax_banned.h</code> to ignore other non-compliant third-party headers, or to temporarily ignore those of your own headers that are not yet fixed.  Additionally, you can use this header to configure custom function deprecations which are not detected by <code>banned.h</code>.</p>\n\
<h2 id=\"using-code-3dsmax_banned-h-code-\">Using <code>3dsmax_banned.h</code></h2><p>There are two ways to add <code>3dsmax_banned.h</code> to your project:</p>\n\
<ol>\n\
<li>Use the &quot;Forced Include File&quot; compiler option (/FI) to automatically force include the header in every .cpp file in the project.  You can either enable /FI using the project Property Pages for each build configuration, or by adding this section to your .vcxproj file:</li>\n\
</ol>\n\
<pre class=\"pre codeblock\"><code class=\"lang-xml prettyprint\">&lt;ItemDefinitionGroup&gt;\n\
  &lt;ClCompile&gt;    \n\
    &lt;ForcedIncludeFiles&gt;3dsmax_banned.h&lt;/ForcedIncludeFiles&gt;\n\
  &lt;/ClCompile&gt;\n\
&lt;/ItemDefinitionGroup&gt;</code></pre>\n\
<ol start=\"2\">\n\
<li>Manually <code>#include</code> the header in each .cpp file after all other <code>#include</code> directives.  This approach requires more maintenance, but can be used to fix one file at a time without generating an overwhelming number of errors.</li>\n\
</ol>\n\
<h2 id=\"integrating-code-3dsmax_banned-h-code-\">Integrating <code>3dsmax_banned.h</code></h2><ol>\n\
<li>Check the project-level build configuration and modify if needed:\n\
Enable the /sdl compiler option.  This enables treating C4995 warnings (deprecated insecure functions) as errors. Check that the C4995 warning is not disabled by other project settings, or by <code>#pragma warning(disable)</code> directives.\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p><strong></strong> The 3dsmax.common.tools.settings.props property sheet already enables the /sdl option. </p>\n\
\n\
</div></div>\n\
\n\
</li>\n\
<li>If you are manually including <code>3dsmax_banned.h</code> (or your own wrapper header), add it to the .cpp files you want to check <em>after</em> all other <code>#include</code> directives.</li>\n\
<li>Rebuild the project and analyze any resulting errors.  See <strong>Fixing Errors</strong> below.</li>\n\
</ol>\n\
<div class=\"head-block\"><h1>Fixing Errors</h1></div><p>We can classify errors as either &quot;side-effect&quot; errors (errors that are not directly attributable to deprecated insecure functions), and actual C4995 function deprecation errors.</p>\n\
<h2 id=\"fixing-quot-side-effect-quot-errors\">Fixing &quot;Side-effect&quot; Errors</h2><p>It is a good idea to fix or suppress these various &quot;side-effect&quot; errors to reduce noise before dealing with C4995 errors.</p>\n\
<p><strong>Macro redefinition errors:</strong> Adding <code>3dsmax_banned.h</code> to a project can change the overall inclusion order of header files, which can lead to macro redefinition errors.  This is where two unrelated headers use the same name to define different macros. \n\
Isolate a conflicting macro definition within its library header file by surrounding the affected <code>#include</code> directive with <code>#pragma push_macro</code> and <code>pop_macro</code> directives.\n\
Then, add the encapsulated include directive to the appropriate <code>banned.h</code> wrapper header.</p>\n\
<p>Here is an example of handling a conflicting macro called <code>EPS</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">// Pre-include the main Example SDK header (included by &quot;dll/MaxExample/ExampleSim.h&quot;)\n\
#include &lt;Example.h&gt;  // Note: &quot;banned&quot; function calls are actually located in nested header &quot;Example/Example/common/include/core/ExampleParamExtractor.h&quot;\n\
\n\
// Include other Example/Example headers to avoid macro conflicts errors\n\
#pragma push_macro(&quot;EPS&quot;) // conflicting macro definitions between &lt;Examplemath/bbox.h&gt; and &lt;maxscript\\maxscript.h&gt;\n\
#undef EPS\n\
#include &lt;Exampleapi/Example_math.h&gt; // Note: macro conflict is actually located in nested header &lt;Examplemath/bbox.h&gt;\n\
#pragma pop_macro(&quot;EPS&quot;)\n\
\n\
\n\
#include &lt;3dsmax_banned.h&gt;</code></pre>\n\
<p>Redundant macro definition errors can show up if your project .cpp files define the <code>STRICT</code> macro, which <code>3dsmax_banned.h</code> already defines before pre-including windows.h.  You can fix these errors by removing the <code>STRICT</code> macro definitions from your project files, or surround them with <code>#ifdef / #endif</code> directives.</p>\n\
<p><strong>Qt Related Warnings:</strong> Many Qt 3rd-party library headers do not compile with warning level 4, which can result in numerous compilation warnings (for example 4127, 4251, 4800, 4275, and 4244).   Ideally, compilation warnings that originate in Qt 3rd-party headers should be suppressed locally, only where needed, by surrounding the affected Qt header includes with #pragma warning push/disable/pop directives.  For example (from <strong>maxsdk/include/Qt/QmaxTranslationLoader.h</strong>):</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">#include &lt;utilexp.h&gt;\n\
\n\
#pragma warning( push )\n\
#pragma warning( disable: 4127 4251 4800 )\n\
#include &lt;QtCore/QObject&gt;\n\
#include &lt;QtCore/QString&gt;\n\
#pragma warning( pop )</code></pre>\n\
<p>However, this can require significant effort, especially in projects with a large number of files.  An easier (though less optimal) approach is to post-include the <code>maxsdk/include/3dsmax_qt_pragma_warning.h</code> header within your wrapper header.  This header takes care of conditionally suppressing compilation warnings for commonly used Qt modules.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p><strong></strong> This header depends on the existence of certain Qt preprocessor symbols, such as QT_CORE_LIB, which are defined for projects using Qt Visual Studio Tools.</p>\n\
\n\
</div></div>\n\
\n\
<h2 id=\"fixing-c4995-errors\">Fixing C4995 Errors</h2><p>You can start by suppressing those C4995 function deprecation errors that occur in system or 3rd-party headers.  This is done by pre-including the affected headers before the <code>#include 3dsmax_banned.h</code> directive in your banned.h wrapper.</p>\n\
<p>As the implementation of <code>banned.h</code> in the 3ds Max SDK is a continuing effort, some 3ds Max SDK headers are not yet compliant and must therefore also be pre-included if used in your plugin code (e.g. <code>&lt;maxscript/maxscript.h&gt;</code> must be pre-included because it contains the <code>MXS_range_check</code> macro that uses some banned functions).</p>\n\
<p>Next, fix (or temporarily suppress) C4995 function deprecation errors that occur in your project header files and source files.</p>\n\
<p>To view the list of &quot;banned&quot; functions and their safer alternatives, see:</p>\n\
<ul>\n\
<li>The official MSDN article <a href=\"https://msdn.microsoft.com/en-us/library/bb288454.aspx?f=255&amp;MSPPError=-2147217396\" target=\"_blank\">Security Development Lifecycle (SDL) Banned Function Calls</a></li>\n\
<li>An unofficial (but more recent) list of banned functions and safe alternatives: <a href=\"https://github.com/intel/safestringlib/wiki/SDL-List-of-Banned-Functions\" target=\"_blank\">SDL List of Banned Functions</a></li>\n\
</ul>\n\
<p>Where possible, replace raw <code>TCHAR*</code> string buffers with the 3ds Max <code>TSTR</code> class, and use class methods like <code>TSTR::printf()</code> instead of low-level string manipulation functions. If the same <code>TSTR</code> instance is reused for several string write operations, verify that the allocated data buffer size is large enough for each write operation.<br>For example, some maxsdk functions like <code>GetSpecDir()</code> or <code>TryCSIDLDir()</code>, accept a <code>TSTR&amp;</code> argument by reference and may reallocate (and possibly reduce) the data buffer allocation.  This could affect subsequent write operations that require a larger buffer.</p>\n\
<p>Some tips:</p>\n\
<ul>\n\
<li>When passing buffer size to secure variants of memory or string functions, take care to specify the expected value, as some functions expect buffer size in bytes (such as <code>memcpy_s()</code>), while others expect string buffer length in <code>TCHAR</code> units (for example <code>_tcscpy_s()</code>).</li>\n\
<li>When using <code>_tcscat_s()</code> or one of its variants, note that the second parameter is expected to be the total size of the buffer, not the remaining size.</li>\n\
<li>When converting a raw string buffer to an instance of type <code>TSTR</code>, and then passing its raw <code>.data()</code> pointer and buffer length as arguments to a function, be careful to account for the order of function argument evaluation.  For example, do not do this:</li>\n\
</ul>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">TSTR buf;\n\
getCfgMgr().getString(INI_KEY_ARCHIVE_ARGS, buf.dataForWrite(1024), (int)buf.AllocatedChars());\n\
// C/C++ standard does not specify order of evaluation for function arguments.\n\
// In MSVC2015, buf.AllocatedChars() is evaluated before buf.dataForWrite(), hence buffer size of zero is passed to function</code></pre>\n\
<p>Instead, re-write your code in a way that&apos;s similar to this snippet:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">getCfgMgr().getString(INI_KEY_ARCHIVE_ARGS, buf.dataForWrite(1024), 1024);\n\
// or:\n\
buf.Resize(1024);\n\
getCfgMgr().getString(INI_KEY_ARCHIVE_ARGS, buf.dataForWrite(), buf.AllocatedChars());\n\
// or better yet, modify the function to accept a TSTR&amp; argument and have the function allocate the appropriate buffer size internally:\n\
getCfgMgr().getString(INI_KEY_ARCHIVE_ARGS, buf);</code></pre>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../best_practices.html\">Best Practices</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";