var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"porting_object_plug-ins_to_nitro\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Porting Object Plug-ins to Nitrous</h1></div><h2 id=\"object-display-api\">Object Display API</h2><p>The object display API is a retained mode graphics API whose functionality is similar to the cached mesh support found with the DirectX viewport driver. This means that plug-ins describe the data to be displayed in the viewport and update it when needed in a declarative manner. The graphics driver stores an internal representation of this data that it retains from frame to frame and uses for rendering to the viewports. This design allows the graphics driver to optimize the rendering and related processing, and to perform rendering in a separate thread. A retained mode graphics API contrasts with an immediate mode graphics API where the drawing commands are immediately rendered.</p>\n\
<p>The Nitrous display driver provides a compatibility layer for plug-ins developed using the legacy immediate mode <code>GraphicsWindow</code> API. As a result, most plug-ins will not require code changes in order to display in the Nitrous viewport display mode. The cases when plug-in code needs to be changed are described in the following section, Impact on Plug-ins.</p>\n\
<p>The object display API consists of the interface class <code>IObjectDisplay</code>. The Nitrous graphics driver queries plug-ins for this interface in order to get their viewport representation as <code>RenderItemHandles</code> (see<code>IObjectDisplay::GetRenderItems()</code>) and asks them to update it (see <code>IObjectDisplay::UpdateDisplay()</code>). The <code>BaseObject</code> class - which is the base class for all procedural objects and modifiers - exposes the <code>IObjectDisplay</code> interface and provides storage for render item handles (see <code>BaseObject::mRenderItemHandles</code>). It means that even those plug-ins that require changes to their <code>BaseObject::Display()</code> function in order to be displayed correctly in the Nitrous viewport display mode, will only need to override the <code>BaseObject::UpdateDisplay()</code> function.</p>\n\
<h2 id=\"impact-on-plug-ins\">Impact on Plug-ins</h2><p>Most procedural object plug-ins are supported as-is by the Nitrous viewport display driver, except for those that have a lit or textured mesh representation and use <code>Mesh::render()</code> in their implementation of <code>BaseObject::Display()</code>. In these cases, the mesh will display flat shaded and unlit in the Nitrous viewport, but any other part of the plug-ins viewport representation drawn using the <code>GraphicsWindow</code> API will display correctly. If the plug-in explicitly sets the <code>GW_WIREFRAME</code> flag and resets <code>GW_ILLUM</code> or <code>GW_TEXTURE</code>, then the mesh will render correctly in the Nitrous viewport since no lighting or texturing information needs to be translated from the immediate mode graphics driver to the retained mode one.</p>\n\
<h2 id=\"plug-in-porting-checklist\">Plug-in Porting Checklist</h2><ul>\n\
<li>Do nothing if the plug-in&apos;s implementation of <code>BaseObject::Display()</code> doesn&apos;t make calls to <code>Mesh::render()</code> , otherwise see the other items on this list.</li>\n\
<li>Do nothing if the plug-in&apos;s implementation of <code>BaseObject::Display()</code> calls <code>Mesh::render()</code> to display a wireframe (<code>GW_WIREFRAME</code> is set) unlit or un-textured mesh (<code>GW_ILLUM</code> or <code>GW_TEXTURE</code> are reset).</li>\n\
<li>Otherwise, the following code changes are required:<ul>\n\
<li>Change the plug-in&apos;s implementation of <code>BaseObject::Display()</code> to skip calling <code>Mesh::render()</code> when the Nitrous viewport graphics driver is active - see <code>MaxSDK::Graphics::IsRetainedModeEnabled()</code>. Leave the rest of the implementation of the Display member function unchanged.</li>\n\
<li>Override <code>BaseObject::RequiresSupportForLegacyDisplayMode()</code> to return true if the plug-in&apos;s override of <code>BaseObject::Display()</code> displays other than a mesh using <code>Mesh::Render()</code>, i.e. contains calls to the <code>GraphicsWindow</code> API, and false, if the plug-in&apos;s override of <code>BaseObject::Display()</code> skips all calls to the <code>GraphicsWindow</code> API when the Nitrous viewport mode is the active one (when <code>MaxSDK::Graphics::IsRetainedModeEnabled()</code> returns true).</li>\n\
<li>Override <code>BaseObject::UpdateDisplay()</code> to generate the plug-in&apos;s mesh representation and the render items based on that.</li>\n\
<li>Add  <em>GraphicsDriver.lib</em>  and  <em>DataBridge.lib</em>  as a linker dependency to the plug-in&apos;s project settings.</li>\n\
<li><em>Important</em>: The plug-in&apos;s override of the <code>InterfaceServer::GetInterface(Interface_ID iid)</code> virtual function must route requests for all interfaces not directly implemented by the plug-in (such as <code>IOBJECT_DISPLAY_INTERFACE_ID</code>) to its <code>BaseObject</code> derived base class. Failure to do so can result in the plug-in not displaying properly in the Nitrous viewport graphics mode.</li>\n\
</ul>\n\
</li>\n\
</ul>\n\
<h2 id=\"sample-code\">Sample Code</h2><p>Following is the <code>ShapeObject</code> display code before the upgrade to support Nitrous viewport graphics mode:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">//ShapeObject display code before the upgrade to support Nitrous viewport graphics mode.\n\
int ShapeObject::Display(TimeValue t, INode *inode, ViewExp* vpt, int flags) \n\
{\n\
    if (!GetDispRenderMesh()) \n\
    return 0;\n\
    // Create a mesh to display and cache it\n\
    GenerateMesh(t, GENMESH_DEFAULT, NULL);\n\
    GraphicsWindow *gw = vpt-&gt;getGW();\n\
    // Call Mesh::render() to display the mesh\n\
    meshCache.render( gw, inode-&gt;Mtls(),\n\
      (flags&amp;USE_DAMAGE_RECT) ? &amp;vpt-&gt;GetDammageRect():NULL, \n\
      COMP_ALL | (inode-&gt;Selected()?COMP_OBJSELECTED:0), inode-&gt;NumMtls(), \n\
      ((ReferenceTarget*)this));\n\
    return 0;\n\
}</code></pre><p>Following is the <code>ShapeObject</code> display code after the upgrade to support Nitrous viewport graphics mode:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">//After the upgrade to support Nitrous viewport graphics mode.\n\
\n\
int ShapeObject::Display(TimeValue t, INode *inode, ViewExp* vpt, int flags) {\n\
    if (MaxGraphics::IsNitrousGraphicsEnabled()) \n\
    return 0;\n\
\n\
    if(!GetDispRenderMesh())\n\
    return 0;\n\
\n\
    // Create a mesh to display and cache it\n\
    GenerateMesh(t, GENMESH_DEFAULT, NULL);\n\
    GraphicsWindow *gw = vpt-&gt;getGW();\n\
    meshCache.render( gw, inode-&gt;Mtls(),\n\
      (flags&amp;USE_DAMAGE_RECT) ? &amp;vpt-&gt;GetDammageRect():NULL,\n\
      COMP_ALL | (inode-&gt;Selected()?COMP_OBJSELECTED:0), inode-&gt;NumMtls(),\n\
      ((ReferenceTarget*)this));\n\
    return 0;\n\
}\n\
\n\
bool ShapeObject::UpdateDisplay(const MaxSDK::Graphics::MaxContext&amp; maxContext,\n\
    const MaxSDK::Graphics::UpdateDisplayContext&amp; displayContext)\n\
{\n\
    using namespace MaxSDK::Graphics;\n\
\n\
    if (!GetDispRenderMesh()) {\n\
      mRenderItemHandles.ClearAllRenderItems();\n\
      return true;\n\
    }\n\
\n\
    // create a mesh to display (leave it in cache)\n\
    GenerateMesh(displayContext.GetDisplayTime(), GENMESH_DEFAULT, NULL);\n\
\n\
    GenerateMeshRenderItemsContext generateRenderItemsContext;\n\
    generateRenderItemsContext.GenerateDefaultContext(displayContext);\n\
    meshCache.GenerateRenderItems(mRenderItemHandles,generateRenderItemsContext);\n\
\n\
    return true;\n\
}</code></pre><h2 id=\"known-issues\">Known Issues</h2><ul>\n\
<li>Compound objects may not display correctly - Objects that have a branched geometry pipeline may not have all of their branches displayed correctly in the Nitrous viewport. This can happen, for example, when the Display method of a certain branch changes some of the parameters of <code>GraphicsWindow</code> such as <code>LINE_COLOR</code> (<code>GraphicsWindow::SetLineColor()</code>).</li>\n\
<li>Object may fail to display correctly - If an object&apos;s mesh depends on parameters whose changes do not trigger notifications the Nitrous graphics driver is aware of, the mesh will not update in the Nitrous viewport since <code>BaseObject::UpdateDisplay()</code> won&apos;t be called for that object.</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../what_s_new_3ds_max_2012_sdk.html\">What&#x27;s New: 3ds Max 2012 SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";