var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"language_packs\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Language Packs</h1></div><h2 id=\"overview\">Overview</h2><p>A Language Pack (LP) is a set of files that can be installed on top of a base product to allow that product to present its User Interface (UI) in another language. An LP is distributed as a separate installer that can only install on top of an already installed Language Neutral (LN) base product. English is just another language in this regard so without an LP installed the product won&apos;t work.</p>\n\
<h2 id=\"what-39-s-in-a-3ds-max-language-pack-\">What&apos;s in a 3ds Max Language Pack?</h2><p>The short answer is anything that&apos;s in any of the various &quot;en-US&quot; directories (&lt;langdir&gt; below refers to &quot;en-US&quot; for English or the appropriate directory for other languages) spread throughout Max&apos;s installed directory structure. There are several types of content:</p>\n\
<ol>\n\
<li><em>3ds Max binaries:</em> Binaries that contain localizable resources are split into two parts, the original DLL (modified to be language neutral) and a resource only DLL with the extension .mui (e.g. 3dsmax.exe and &lt;langdir&gt;/3dsmax.exe.mui). We use a Microsoft supplied tool MUIRCT.exe to do this split but must post process the files slightly to support Windows XP. The Microsoft MUI system requires that the .mui file be in a subdirectory (&lt;langdir&gt;) of the directory that contains the Language Neutral (LN) binary itself. NOTE: some DLLs (the .Net DLLs for example) don&apos;t use the MUI system and their resource DLLs are stored, unchanged, in the &lt;langdir&gt;.</li>\n\
<li><em>MAXScript files (.ms, .mcr, etc):</em> These files are split into the language neutral .ms file and the Maxscript resource file .ms.res. The .res files live in the root &lt;langdir&gt; in the same relative place as the original file. For third parties the .res files can also live relative to the base file so they don&apos;t need to install on top of the LN installer. Please refer to Maxscript&apos;s documentation for further explanations.</li>\n\
<li><em>UI files (.cui, .kbd, .mnu, .clr, .qop, .xaml):</em> These file, in fact the entire UI directory, are in the root &lt;langdir&gt;</li>\n\
<li><em>Essential Skills Movies:</em> These are large and constitute 90% of the size of the LPs.</li>\n\
<li><em>Configuration files and miscellaneous files (.ini, max.tres, maxscrpt.lcl):</em> These files are in various locations under the language directory.</li>\n\
</ol>\n\
<h2 id=\"quick-and-dirty-implementation-notes-for-plug-in-developers\">Quick-and-Dirty Implementation Notes for Plug-in Developers</h2><p>If you only want to implement English-only plug-ins (i.e. not localize them), or your plug-in doesn&apos;t have any localizable resources, you may ignore the following and not implement any of the language pack related code or other build or installer language pack functionality. However, we recommend testing all plug-ins with language packs as a precaution.</p>\n\
<p>If you want to localize your plug-in and match it up with Max&apos;s running locale as best as possible, but do not want to be limited by the XP compromise or any other aspect of Max&apos;s language packs implementation, you can still take advantage of some of its functionality. For instance, you can probably get by with retrieving Max&apos;s runtime locale to implement your own localization functionality, as long as you have a hard coded default locale (usually en-US) if the locale returned by Max is not supported by your plug-in. This is especially valid if you want to implement language packs with the older method provided by Microsoft, which required implementing custom versions of the resource retrieval calls (FindResource(), LockResource(), etc.). This is explained in more detail in MSDN.</p>\n\
<p>On the other hand, if you want to implement functionality that takes advantage of Max&apos;s implementation of Language Packs, and aware of the XP compromise (and plan accordingly), the following gives you a quick-and-dirty explanation at each step that builds on the information in previous sections.</p>\n\
<h3 id=\"step-1\">Step 1</h3><p>Add the required call to <code>MaxSDK::Util::UseLanguagePackLocale()</code> below to your DllMain or _beginthread() and _beginthreadex() functions.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#include &quot;systemutilities.h&quot;\n\
\n\
BOOL WINAPI DllMain(HINSTANCE hinstDLL,ULONG fdwReason,LPVOID lpvReserved) {\n\
\n\
    switch(fdwReason) {\n\
\n\
        case DLL_PROCESS_ATTACH:\n\
\n\
            MaxSDK::Util::UseLanguagePackLocale(); // Required!!\n\
\n\
            // ...\n\
\n\
            break;\n\
\n\
    // Etc...\n\
}\n\
\n\
-   The header file systemutilities.h must be added as an `\\#include`\n\
-   The directory search list for headers must have the Max SDK inclue directory to get to the include file.\n\
-   The library file maxutil.lib must be linked to the project.</code></pre><h3 id=\"step-2\">Step 2</h3><p>After getting your program to compile and link, use MUIRCT.exe to create satellite DLLs for all of your DLLs that need to be localized.</p>\n\
<h3 id=\"step-3\">Step 3</h3><p>Have your resource-only satellite DLLs localized, along with other content, in order to have a fully usable set of localized files.</p>\n\
<h3 id=\"step-4\">Step 4</h3><p>In order for the XP compromise to work with your plug-in, retrofit your en-US localized content into the LN binaries and set them to the &apos;neutral-neutral&apos; locale. Otherwise, your plug-in will not work at all in Windows XP, as the resource retrieval code will fail to find any resources.</p>\n\
<h3 id=\"step-5\">Step 5</h3><p>Test your localized file set by putting them in appropriate directories. The satellite-only DLLs must match up with their locale directory name; any mismatch will result in the plug-in failing on being unable to find resources in Vista and Windows 7. Do not forget to also test in XP for the compromise functionality, as it cannot be simulated in Vista and Windows 7 under normal circumstances.</p>\n\
<h3 id=\"step-6\">Step 6</h3><p>Implement your core LN installer and your language pack installers. Make sure the core is installed first, and then one or more language pack installers.</p>\n\
<h3 id=\"step-7\">Step 7</h3><p>Test the resulting installers with the language switching option, and various combinations. Please remember that Max 32 bit, Max 64 bit, Design 32 bit, and Design 64 bit are all separate targets, each one with their own &apos;core&apos; LN installer, and set of language pack installers. To be clear, that means there are four &apos;core&apos; LN installers and six language pack installers for each of those four. Your plug-ins must follow this format to be fully compliant with all possible product installations.</p>\n\
<h2 id=\"table-of-localized-languages\">Table of Localized Languages</h2><p>3ds Max is localized for the following languages:</p>\n\
<table>\n\
<colgroup>\n\
<col width=\"25%\">\n\
<col width=\"25%\">\n\
<col width=\"25%\">\n\
<col width=\"25%\">\n\
</colgroup>\n\
<thead>\n\
<tr class=\"header\">\n\
<th>Locale</th>\n\
<th>LCID (Hex)</th>\n\
<th>LCID (Decimal)</th>\n\
<th>Name</th>\n\
</tr>\n\
</thead>\n\
<tbody>\n\
<tr class=\"odd\">\n\
<td>en-US</td>\n\
<td>0409</td>\n\
<td>1033</td>\n\
<td>English</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>de-DE</td>\n\
<td>0407</td>\n\
<td>1031</td>\n\
<td>Deutsch (German)</td>\n\
</tr>\n\
<tr class=\"odd\">\n\
<td>fr-FR</td>\n\
<td>040C</td>\n\
<td>1036</td>\n\
<td>Fran&#xE7;ais (French)</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>ja-JP</td>\n\
<td>0411</td>\n\
<td>1041</td>\n\
<td><p>&#x65E5;&#x672C; (Japanese)</p></td>\n\
</tr>\n\
<tr class=\"odd\">\n\
<td>ko-KR</td>\n\
<td>0412</td>\n\
<td>1042</td>\n\
<td><p>&#xD55C;&#xAD6D;&#xC5B4; (Korean)</p></td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>zh-CN</td>\n\
<td>0804</td>\n\
<td>2052</td>\n\
<td><p>&#x4E2D;&#x6587; (Simplified Chinese)</p></td>\n\
</tr>\n\
</tbody>\n\
</table>\n\
\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>   <strong></strong> Code pages have become an obsolete localization mechanism in 3ds Max now that its strings are in Unicode. Code pages cannot be used in conjunction with Language Packs.</p>\n\
\n\
</div></div>\n\
\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../what_s_new_3ds_max_2013_sdk.html\">What&#x27;s New: 3ds Max 2013 SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";