var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"unicode\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Unicode</h1></div><h2 id=\"what-is-unicode-\">What is Unicode?</h2><p>Unicode is a computing industry standard for the consistent encoding, representation and handling of text expressed in most of the world&apos;s writing systems. The characters in Unicode are mapped to numerical values named &apos;code points&apos;. Letter &apos;A&apos; for example is mapped to the code point U+0041 (U+ for Unicode). Code points have no restriction on the implementation in memory. For example, the numerical (hexadecimal) value 0041 can be either stored as 0x00-0x41 (UTF-16, small endian), or as 0x41-0x00 (UTF-16, big endian) or simply 0x41 (UTF-8). There are different methods to encode Unicode characters. The important point is that we cannot assume anymore that all characters take a constant number of bytes. Some encodings of Unicode can use up to 6 byte per character. Unicode itself can be represented by different encodings. The most common are UTF-16, UTF-8, and even UTF-32.</p>\n\
<p>A Unicode string is a sequence of Unicode characters, each taking a potentially variable number of bytes. Unicode code points can use any encoding methods such as ASCII, ANSI or different code pages. If a Unicode character is about to get displayed in an environment where a specific non-Unicode encoding is used, a special character called the replacement character (looks like a question mark - see <a href=\"https://en.wikipedia.org/wiki/Specials_(Unicode_block)#Replacement_character\" target=\"_blank\">http://en.wikipedia.org/wiki/Replacement_character</a> or a box will be displayed for the characters that are not defined in that encoding. There is a chance that the characters defined in both encodings will show up correctly. For example, the following table shows how the string &quot;R&#xE9;sum&#xE9;&quot; created in different encoding systems.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Encoding in which the string was saved</th>\n\
<th>Display of the string in an ASCII environment</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>UTF-16</code>, small endian</td>\n\
<td>R&#xE9;sum&#xE9;</td>\n\
</tr>\n\
<tr>\n\
<td><code>UTF-16</code>, big endian</td>\n\
<td>R&#x398;sum&#x398;</td>\n\
</tr>\n\
<tr>\n\
<td><code>UTF8</code></td>\n\
<td>R&#x251C;&#x2310;sum&#x251C;&#x2310;</td>\n\
</tr>\n\
<tr>\n\
<td><code>ANSI</code></td>\n\
<td>R&#x398;sum&#x398;</td>\n\
</tr>\n\
</tbody></table>\n\
<h2 id=\"character-and-string-types-in-3ds-max\">Character and String Types in 3ds Max</h2><p>Strings are not considered as arrays of plain <code>char</code> data types anymore. If you are sure that your string will always be ASCII strings and do not include any extra symbols, or if your code will always be running on an operating system with the same MBCS encoding, you can still use char arrays, however this is not recommended.</p>\n\
<p>You can use arrays of <code>wchar</code> (wide character) if you are using an encoding such as UTF-16. You can compile the same code using either <code>char</code> or <code>wchar</code> arrays for strings if you define a macro and ask the compiler to replace it by either <code>char</code> or <code>wchar</code> before compile-time. In fact, this is what the <code>TCHAR</code> macro in the Windows API does.</p>\n\
<p>When you are programming for a specific product such as 3ds Max, there might be a case where you need to compile against a non-Unicode API (such as 3ds Max 2012), but you want to have Unicode strings in your code or plugin for internal purposes. This is why a seperate macro <code>MCHAR</code> is defined, so you can use <code>MCHAR</code> for interfacing with the 3ds Max API, and <code>TCHAR</code> for code internal to your plugin. Similarly, use <code>MSTR</code> if interfacing with the 3ds Max API and <code>TSTR</code> for code internal to your plugin when dealing with strings.</p>\n\
<p>The following table shows the mapping for these types in both MBCS and Unicode modes. You can also refer to the basic <code>strbasic.h</code> and <code>strclass.h</code> files for more information.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Name</th>\n\
<th>MBCS</th>\n\
<th>Unicode</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>TCHAR</code></td>\n\
<td><code>char</code></td>\n\
<td><code>wchar_t</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>MCHAR</code></td>\n\
<td><code>char</code></td>\n\
<td><code>wchar_t</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>TSTR</code></td>\n\
<td><code>CStr</code></td>\n\
<td><code>WStr</code></td>\n\
</tr>\n\
</tbody></table>\n\
<h2 id=\"arrays-of-characters-c-strings-\">Arrays of Characters (C Strings)</h2><p>A basic C string data type is a null-terminating array of fixed-length characters. For 3ds Max, you have to use either TCHAR* or MCHAR* to point to such an object. The use of ASCII or ANSI strings for text and files should be avoided. These strings are not compatible with TCHAR or MCHAR when built with Unicode. The following table lists a sample of functions which should be avoided, and their suggested alternative(s).</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Function to Avoid</th>\n\
<th>Reason</th>\n\
<th>Replacement Function</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>fopen</code></td>\n\
<td>3ds Max must support opening files which contain any valid Unicode character.</td>\n\
<td><code>\\_tfopen</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>tolower</code>, <code>toupper</code></td>\n\
<td>Those functions don&apos;t work with many MBCS code pages, like Chinese or Japanese</td>\n\
<td><code>\\_tcsupr</code>, <code>\\_tcslwr</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>fprintf</code></td>\n\
<td>Has issues handling Unicode. Can be replaced by _ftprintf, but even that function produces MBCS output by default.</td>\n\
<td><code>MaxSDK::Util::TextFile::Writer</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>fwrite</code>, <code>fread</code></td>\n\
<td>Does not perform any kind of conversion. Using this function to read or write TCHAR will produce files that are not interchangeable between ASCII or Unicode builds.</td>\n\
<td><code>MaxSDK::Util::TextFile::Writer</code>, <code>MaxSDK::Util::TextFile::Reader</code>,<code>MaxSDK::Util::TextFile::ReaderWriter</code></td>\n\
</tr>\n\
</tbody></table>\n\
<h2 id=\"c-runtime-tchar-enabled-functions\">C Runtime TCHAR-enabled functions</h2><p>The standard C runtime library functions use MBCS strings that cannot be easily converted to Unicode. It is recommended to use their equivalent TCHAR functions to make your project compatible with Unicode. The following tables contain a partial list of MBCS-only functions and their TCHAR equivalent.</p>\n\
<h3 id=\"file-functions\">File Functions</h3><table>\n\
<colgroup>\n\
<col width=\"40%\">\n\
<col width=\"60%\">\n\
</colgroup>\n\
<thead>\n\
<tr class=\"header\">\n\
<th>MBCS function name</th>\n\
<th>TCHAR function name</th>\n\
</tr>\n\
</thead>\n\
<tbody>\n\
<tr class=\"odd\">\n\
<td>`_fprintf_l`</td>\n\
<td>`_ftprintf_l`</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>`fgets`</td>\n\
<td>`_fgetts`</td>\n\
</tr>\n\
<tr class=\"odd\">\n\
<td>`fopen`</td>\n\
<td>`_ftopen`</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>`fprintf`</td>\n\
<td>`_ftprintf`</td>\n\
</tr>\n\
<tr class=\"odd\">\n\
<td>`fputs`</td>\n\
<td>`_fputts`</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>`unlink`</td>\n\
<td>`_tunlink`</td>\n\
</tr>\n\
</tbody>\n\
</table>\n\
\n\
<h3 id=\"string-functions\">String Functions</h3><table>\n\
<thead>\n\
<tr>\n\
<th>MBCS function name</th>\n\
<th>TCHAR function name</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>sprintf</code></td>\n\
<td><code>\\_stprintf</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>sscanf</code></td>\n\
<td><code>\\_stscanf</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strcat</code></td>\n\
<td><code>\\_tcscat</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strchr</code></td>\n\
<td><code>\\_tcschr</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strcmp</code></td>\n\
<td><code>\\_tcscmp</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strcpy</code></td>\n\
<td><code>\\_tcscpy</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strdup</code></td>\n\
<td><code>\\_tcsdup</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>stricmp</code></td>\n\
<td><code>\\_tcsicmp</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strlen</code></td>\n\
<td><code>\\_tcslen</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strlwr</code></td>\n\
<td><code>\\_tcslwr</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strncat</code></td>\n\
<td><code>\\_tcsncat</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strncmp</code></td>\n\
<td><code>\\_tcsncmp</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strncpy</code></td>\n\
<td><code>\\_tcsncpy</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strstr</code></td>\n\
<td><code>\\_tcsstr</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>strupr</code></td>\n\
<td><code>\\_tcsupr</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>vsprintf</code></td>\n\
<td><code>\\_vstprintf</code></td>\n\
</tr>\n\
</tbody></table>\n\
<h3 id=\"date-functions\">Date Functions</h3><table>\n\
<colgroup>\n\
<col width=\"40%\">\n\
<col width=\"60%\">\n\
</colgroup>\n\
<thead>\n\
<tr class=\"header\">\n\
<th>MBCS function name</th>\n\
<th>TCHAR function name</th>\n\
</tr>\n\
</thead>\n\
<tbody>\n\
<tr class=\"odd\">\n\
<td>`_ctime32`</td>\n\
<td>`_tctime32`</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>`_ctime64`</td>\n\
<td>`_tctime64`</td>\n\
</tr>\n\
<tr class=\"odd\">\n\
<td>`_strdate`</td>\n\
<td>`_tstrdate`</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>`_strtime`</td>\n\
<td>`_tstrtime`</td>\n\
</tr>\n\
<tr class=\"odd\">\n\
<td>`_utime`</td>\n\
<td>`_tutime`</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>`_utime32`</td>\n\
<td>`_tutime32`</td>\n\
</tr>\n\
<tr class=\"odd\">\n\
<td>`_utime64`</td>\n\
<td>`_tutime64`</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>`asctime`</td>\n\
<td>`_tasctime`</td>\n\
</tr>\n\
<tr class=\"odd\">\n\
<td>`ctime`</td>\n\
<td>`_tctime`</td>\n\
</tr>\n\
</tbody>\n\
</table>\n\
\n\
<h3 id=\"conversion-functions\">Conversion Functions</h3><table>\n\
<colgroup>\n\
<col width=\"40%\">\n\
<col width=\"60%\">\n\
</colgroup>\n\
<thead>\n\
<tr class=\"header\">\n\
<th>MBCS function name</th>\n\
<th>TCHAR function name</th>\n\
</tr>\n\
</thead>\n\
<tbody>\n\
<tr class=\"odd\">\n\
<td>`atoi`</td>\n\
<td>`_tstoi`</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>`atof`</td>\n\
<td>`_tstof`</td>\n\
</tr>\n\
</tbody>\n\
</table>\n\
\n\
<h2 id=\"string-objects-c-strings-\">String Objects (C++ Strings)</h2><p>String objects are abstract data types composed of a sequence of characters. However, each character can take different memory sizes in the implementation. Moreover, there can be extra information in a string object. It is recommended to use TSTR or <code>MaxString</code> objects when you need a string object.</p>\n\
<h3 id=\"tstr-strings\">TSTR Strings</h3><p>TSTR is a macro replaced either by WStr or CStr depending on if you have defined the Unicode symbol or not. You can still directly use CStr or WStr if you have to specifically store data in a particular format independent of the build mode. In addition, WStr and CStr implement a lazy-copying algorithm which makes it fast and cheap to copy strings inside the application.</p>\n\
<h3 id=\"converting-c-strings-to-tstr\">Converting C Strings to TSTR</h3><p>There are several distinct ways of storing and referencing a string in a Windows program. Examples are:</p>\n\
<ul>\n\
<li>Strings stored in ACP (active code page) format and referenced by <code>char*</code>.</li>\n\
<li>Strings stored in UTF-16 format and referenced by <code>wchar*</code>.</li>\n\
<li>Strings stored in BSTR (basic string objects that hold the size of the string in the object itself) and referenced by BSTR and so on.</li>\n\
</ul>\n\
<p>The TSTR class provides some methods on converting between different types of string objects. You have to be careful, because conversion to ACP and TCHAR (in MBCS mode) can lead to losing some character information. The following static methods of TSTR will construct a TSTR object from different data types:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static TSTR TSTR::FromBSTR  (BSTR           string [, size_t length]);\n\
static TSTR TSTR::FromACP   (const char*    string [, size_t length]);\n\
static TSTR TSTR::FromUTF8  (const char*    string [, size_t length]);\n\
static TSTR TSTR::FromOLESTR(LPCOLESTR      string [, size_t length]);\n\
static TSTR TSTR::FromUTF16 (const wchar_t* string [, size_t length]);\n\
static TSTR TSTR::FromUTF32 (const unsigned int* string [, size_t length]);\n\
static TSTR TSTR::FromTCHAR (const TCHAR*   string [, size_t length]);\n\
static TSTR TSTR::FromCP    (UINT cp, const char* string [, size_t length]);\n\
static TSTR TSTR::FromCStr  (const CStr&amp; string);\n\
static TSTR TSTR::FromWStr  (const WStr&amp; string);\n\
static TSTR TSTR::FromMCHAR (const wchar_t* string [, size_t length]);\n\
static TSTR TSTR::FromMSTR  (const WStr&amp; string);</code></pre><h3 id=\"converting-from-tstr-to-other-strings\">Converting from TSTR to Other Strings</h3><p>The following is a list of TSTR member functions (methods) which return different string objects from the caller TSTR object. Except for &quot;ToBSTR&quot;, all pointers returned by TSTR are valid as long as the TSTR object remains alive and unchanged. <em>Note that these do not return copies of the data. Ensure you use the returned pointers promptly and avoid caching them.</em></p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Return type</th>\n\
<th>Method</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>wchar_t*</code></td>\n\
<td><code>ToBSTR() const</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>MaxSDK::Util::MaxStringCastCP</code></td>\n\
<td><code>ToCP(UINT cp, size_t* length = NULL) const</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>MaxSDK::Util::MaxStringCast&amp;lt;char&amp;gt;</code></td>\n\
<td><code>ToACP(size_t* length = NULL) const</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>MaxSDK::Util::MaxStringCastUTF8</code></td>\n\
<td><code>ToUTF8(size_t* length = NULL) const</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>MaxSDK::Util::MaxStringCast&amp;lt;WCHAR&amp;gt;</code></td>\n\
<td><code>ToOLESTR(size_t* length = NULL) const</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>MaxSDK::Util::MaxStringCast&amp;lt;WCHAR&amp;gt;</code></td>\n\
<td><code>ToUTF16(size_t* length = NULL) const</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>MaxSDK::Util::MaxStringCast&amp;lt;unsigned_int&amp;gt;</code></td>\n\
<td><code>ToUTF32(size_t* length = NULL) const</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>MaxSDK::Util::MaxString</code></td>\n\
<td><code>ToMaxString() const</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>CStr</code></td>\n\
<td><code>ToCStr() const</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>WStr</code></td>\n\
<td><code>ToWStr() const</code></td>\n\
</tr>\n\
<tr>\n\
<td>Compiler dependent</td>\n\
<td><code>ToMCHAR(size_t* length = NULL) const</code></td>\n\
</tr>\n\
<tr>\n\
<td>Compiler dependent</td>\n\
<td><code>ToMSTR() const</code></td>\n\
</tr>\n\
</tbody></table>\n\
<h3 id=\"c-generic-strings\">C++ Generic Strings</h3><p>You can also safely use the following macros to target generic C++ data types. These macros are compatible with both MBCS and Unicode configurations. These are defined in <code>strbasic.h</code>.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Macro</th>\n\
<th>MBCS</th>\n\
<th>Unicode</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>M_STD_STRING</code></td>\n\
<td><code>std::string</code></td>\n\
<td><code>std::wstring</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>M_STD_OSTRINGSTREAM</code></td>\n\
<td><code>std::ostringstream</code></td>\n\
<td><code>std::wostringstream</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>M_STD_ISTRINGSTREAM</code></td>\n\
<td><code>std::istringstream</code></td>\n\
<td><code>std::wistringstream</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>M_STD_OSTREAM</code></td>\n\
<td><code>std::ostream</code></td>\n\
<td><code>std::wostream</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>M_STD_ISTREAM</code></td>\n\
<td><code>std::istream</code></td>\n\
<td><code>std::wistream</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>M_STD_FOSTREAM</code></td>\n\
<td><code>std::fostream</code></td>\n\
<td><code>std::wofstream</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>M_STD_FISTREAM</code></td>\n\
<td><code>std::fistream</code></td>\n\
<td><code>std::wfistream</code></td>\n\
</tr>\n\
</tbody></table>\n\
<p>There are also several global objects provided by the C++ Standard Library and are specific to either MBCS or Unicode. They have been aliased to a common name that works for both and enable users to use the same type name independent of the used standard for the strings.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Macro</th>\n\
<th>MBCS</th>\n\
<th>Unicode</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>M_STD_CERR</code></td>\n\
<td><code>std::cerr</code></td>\n\
<td><code>std::wcerr</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>M_STD_CIN</code></td>\n\
<td><code>std::cin</code></td>\n\
<td><code>std::wcin</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>M_STD_COUT</code></td>\n\
<td><code>std::cout</code></td>\n\
<td><code>std::wcout</code></td>\n\
</tr>\n\
</tbody></table>\n\
<h3 id=\"maxstring-strings\">MaxString Strings</h3><p><code>MaxString</code> is the internal string data type for TSTR. CStr and WStr use this class to perform their internal string operations. This class uses an internal buffer that can hold a string in different encodings, and has a data member that tells which encoding is being used. Plug-in developers can use this class when they need to have string objects passed to/from 3ds Max. Some advantages of this class are:</p>\n\
<ul>\n\
<li>It is capable of holding the same string in different encodings.</li>\n\
<li>It reduces the number of copies of the same string in the memory by using a lazy copy algorithm. Several <code>MaxString</code> objects will point to the same physical memory location unless they need to modify the string. It is only then that an actual copy is performed.</li>\n\
<li>It prevents loss of information by performing optimized string operations. For example, concatenating a UTF-16 string with an ACP string will give a UTF-16 string.</li>\n\
</ul>\n\
<h3 id=\"converting-strings-from-to-maxstring\">Converting Strings From/To MaxString</h3><p>The following static functions create and return a <code>MaxString</code> from other string types.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static MaxString MaxString::FromCP         (UINT codepage, const char*, size_t length = (size_t)-1);\n\
static MaxString MaxString::FromACP        (const char*, size_t length = (size_t)-1);\n\
static MaxString MaxString::FromUTF8       (const char*, size_t length = (size_t)-1);\n\
static MaxString MaxString::FromUTF16      (const WCHAR*, size_t length = (size_t)-1);\n\
static MaxString MaxString::FromUTF32      (const unsigned int*, size_t length = (size_t)-1);\n\
static MaxString MaxString::FromWin32Error (DWORD err);\n\
static MaxString MaxString::FromAnsiError  (int err);</code></pre><p>You can use the <code>MaxStringCast</code> Template class to convert your <code>MaxString</code> objects to encoding-dependant string objects. The following functions can convert <code>MaxStrings</code> to <code>MaxStringCast&amp;lt;typename ChType&amp;gt;</code> or <code>MaxStringCastUTF8</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MaxStringCast&lt;char&gt;         MaxString::ToACP  (size_t* length) const\n\
MaxStringCastUTF8           MaxString::ToUTF8 (size_t* length) const\n\
MaxStringCast&lt;WCHAR&gt;        MaxString::ToUTF16(size_t* length) const\n\
MaxStringCast&lt;unsigned int&gt; MaxString::ToUTF32(size_t* length) const</code></pre><p>You can then use the <code>MaxStringCast&amp;lt;typename CHType&amp;gt;::data()</code> to return an encoding-dependent string object:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">inline const ChType* data() const {\n\
    return (buf) ? buf: null_data();\n\
}</code></pre><p><code>buf</code> is a protected data member pointing to the actual string of the specific type.</p>\n\
<h2 id=\"reading-and-writing-text-files\">Reading and Writing Text Files</h2><p>There are three new classes inside the <code>MaxSDK::Util::TextFile</code> namespace that were designed to intelligently read Unicode-enabled text files. Those classes are able to detect the encoding used in a particular file, either by identifying the Byte Order Mark (BOM) at the beginning of the file, validating that the file is either a valid Unicode encoding file such as UTF-16 or UTF-8 (it doesn&apos;t contain any invalid character) or it is simply an ACP MBCS file.</p>\n\
<p>The following table describes the mapping between the function <code>fopen</code> and its open mode parameter and the 3ds Max API class to use:</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>fopen Open Mode</th>\n\
<th>Class and Open method</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>&quot;r&quot;, &quot;rt&quot;, &quot;rb&quot;</code></td>\n\
<td>TextFile::Reader reader;   reader.Open(file);</td>\n\
</tr>\n\
<tr>\n\
<td><code>&quot;r+t&quot;, &quot;r+b&quot;</code></td>\n\
<td>TextFile::ReaderWriter readerWriter;  readerWriter.Open(file);</td>\n\
</tr>\n\
<tr>\n\
<td><code>&quot;w&quot;, &quot;wt&quot;, &quot;wb&quot;</code></td>\n\
<td>TextFile::Writer writer; writer.Open(file, false);</td>\n\
</tr>\n\
<tr>\n\
<td><code>&quot;w+t&quot;, &quot;w+b&quot;</code></td>\n\
<td>TextFile::ReaderWriter readerWriter; readerWriter.Open(file, false);</td>\n\
</tr>\n\
<tr>\n\
<td><code>&quot;a&quot;, &quot;at&quot;</code></td>\n\
<td>TextFile::Writer writer;  writer.Open(file, true);</td>\n\
</tr>\n\
<tr>\n\
<td><code>&quot;a+t&quot;, &quot;a+b&quot;</code></td>\n\
<td>TextFile::ReaderWriter readerWriter; readerWriter.Open(file);</td>\n\
</tr>\n\
</tbody></table>\n\
<h3 id=\"maxsdk-util-textfile-reader\">MaxSDK::Util::TextFile::Reader</h3><p>This class reads and interprets text files, and it was designed to perform file and stream I/O in a code page neutral way. It was designed to resolve the following problems:</p>\n\
<ul>\n\
<li>Reads and interprets correctly the Byte Order Mark (BOM, an invisible character at the beginning of Unicode files).</li>\n\
<li>Correctly detect UTF-8 and UTF-16 files, even if the file is not signed.</li>\n\
<li>Detect encoding cookies. XML files usually begin with &quot;&lt;?xml encoding=&apos;????&apos;&gt;&quot; The detection algorithm will interpret this directive correctly.</li>\n\
<li>Prevent from splitting a character. In UTF-16, UTF-8 and some ANSI codepages, characters can be stored on 1 to 6 bytes. All the operations of this object avoid returning a partial character.</li>\n\
</ul>\n\
<p>Plugin developers should consider using this class to perform File I/O to ensure that the files they generate remain compatible with previous versions of 3ds Max.</p>\n\
<p><em>Opening Reader Files</em></p>\n\
<p>The following lists the various methods for opening a text file using the <code>MaxSDK::Util::TextFile::Reader</code> class.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">bool TextFile::Reader::Open(\n\
    const MCHAR* fileName,\n\
    unsigned int encoding = 0,\n\
    LineEndMode mode = Text\n\
);\n\
\n\
bool TextFile::Reader::Open(\n\
    FILE* file,\n\
    unsigned int encoding = 0,\n\
    LineEndMode mode = Text\n\
);\n\
\n\
bool TextFile::Reader::Open(\n\
    HANDLE fileHandle,\n\
    unsigned int encoding = 0,\n\
    LineEndMode mode = Text\n\
);</code></pre><p>The parameters of the above functions are described as follows:</p>\n\
<ul>\n\
<li><p><em>fileName/file/fileHandle</em> - The first parameter specifies the file to open. The <code>TextFile::Reader</code> class will parse them to determine their format.</p>\n\
</li>\n\
<li><p><em>encoding</em> - A flag to give hints to the detection algorithm. Acceptable values for this parameter are all of the code page numbers recognized by MS Windows. In addition to that, you can also set one of the following flags:</p>\n\
<ul>\n\
<li><code>FAVOR_UTF8 = 0x10000000</code> - If not used, the function assumes file encoding is ACP when the file&apos;s encoding cannot be detected. Setting this will force this function to use UTF8 when the encoding cannot be automatically detected.</li>\n\
<li><code>FOUND_BOM = 0x20000000</code> - Tells the function that there is a BOM mask at the beginning of the file.</li>\n\
<li><code>FOUND_COOKIE = 0x40000000</code> - Tells the function that there is a BOM mask at the beginning of the file.</li>\n\
<li><code>FLIPPED = 0x80000000</code> - Tells the file if flipped (big endian) UTF-16.</li>\n\
</ul>\n\
<p>For example, if you specify <code>CP_ACP | FAVOR_UTF8</code>, the detection algorithm will treat any non-UTF8 data as ACP.</p>\n\
</li>\n\
<li><p><em>mode</em> - the last parameter specifies type of the end of line characters used in the file. Different arguments are:</p>\n\
<ul>\n\
<li><em>Unchanged</em> - Do not alter the end of line character sequence when reading the file.</li>\n\
<li><em>Enforce_CRLF</em> - Ensure that all lines are terminated by the CRLF character sequence.</li>\n\
<li><em>Enforce_LF</em> - Ensure that all lines are terminated by the LF character.</li>\n\
<li><em>Text</em> - Enforce_CRLF (default in Windows).</li>\n\
</ul>\n\
</li>\n\
</ul>\n\
<p><em>Querying Reader Encoding</em></p>\n\
<p>You can also use the function unsigned <code>int MaxSDK::Util::TextFile::Reader::Encoding()</code> to inquire about the encoding of this object. For reading from the text files, you can use one of the following functions:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">char TextFile::Reader::ReadChar              (bool peek = false);\n\
MaxString TextFile::Reader::ReadChars        (size_t nchars);\n\
unsigned int TextFile::Reader::ReadCharUTF32 (bool peek = false);\n\
MaxString TextFile::Reader::ReadChunk        (size_t len, bool dontReturnLastEOL = false);\n\
MaxString TextFile::Reader::ReadFull         ();\n\
MaxString TextFile::Reader::ReadLine         (size_t nchars = (size_t)-1, bool dontReturnEOL = false);</code></pre><h3 id=\"maxsdk-util-textfile-writer\">MaxSDK::Util::TextFile::Writer</h3><p><code>TextFile::Writer</code> is used to write to any Unicode and MBCS text file format. It automatically performs the conversion between TCHAR and the underlying format. As for the <code>TextFile::Reader</code> class, it will produce a MBCS ACP file by default unless you change the &quot;encoding&quot; parameter to another format like CP_UTF8 or MSDE_CP_UTF16. If <code>TextFile::Writer</code> is set to append or overwrite a file, it will detect the file format of the existing file and continue using that particular format. This class avoids mixing different encodings within the same file, and also intelligently converts strings into binary data. Its open functions are similar to <code>TextFile::Reader</code>&apos;s. It has the following file writing functions:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">size_t Write (const char *string, size_t nbchars=(size_t)-1);\n\
size_t Write (const wchar_t *string, size_t nbchars=(size_t)-1);\n\
size_t Write (const MaxString &amp;string);</code></pre><h3 id=\"maxsdk-util-textfile-readerwriter\">MaxSDK::Util::TextFile::ReaderWriter</h3><p>While the TextFile::Reader and TextFile::Writer are used for the occasions where you only need to either read or write from/to Unicode text files, the TextFile::ReaderWriter class both reads and writes Unicode files. Please refer to the class documentation for a detailed list of its functionality.</p>\n\
<h2 id=\"porting-existing-plug-ins-to-unicode-builds-of-3ds-max\">Porting Existing Plug-ins to Unicode Builds of 3ds Max</h2><p>Plugins have to be ported to Unicode because 3ds Max 2013 is built with Unicode. Therefore the library files that ship with the SDK are only built Unicode. The API will not support MBCS encoding anymore.</p>\n\
<p>The first step for porting existing plug-ins to the Unicode build of 3ds Max is to define UNICODE and _UNICODE symbols in your project. Remove any MBCS and _MBCS definitions from your plugin project. This will replace the TCHAR, MCHAR and TSTR macros by their equivalent Unicode types during the pre-compile phase. Failing to do so will cause linker errors because your functions will have a different signature than those defined in the 3ds Max SDK.</p>\n\
<p>To define the UNICODE and _UNICODE symbols, you can either define the character set in the general settings of your project&apos;s property to &quot;use Unicode character set&quot;, or add the swtiches /D &quot;UNICODE&quot; /D &quot;_UNICODE&quot; to the command line options in project properties &gt; Configuration Properties &gt; C/C++ &gt; Command Line &gt; Additional Options.</p>\n\
<p>Make sure you are using only Unicode compatible characters and strings when you are passing parameters to/from the Max API. Your plug-in will crash or at least will fail to work properly if you use non-Unicode strings such as char* or CStr. Use either TSTR, MSTR, MaxString for strings and TCHAR and MCHAR for characters.</p>\n\
<p>You have to use Unicode encoded files if your plug-in reads from or writes to text files or uses database interfaces. It is recommended to use MaxSDK::Util::TextFile::Reader and MaxSDK::Util::TextFile::Writer. If you are not using those file utilities, you have to pay extra attention to avoid using file utilities that use non-Unicode strings.</p>\n\
<p>Check all the usages of sizeof() and string length determinations within your code. Remember that the memory size for different characters in your string is not the same anymore. Avoid the common mistakes mentioned in the &quot;False Assumptions&quot; section below.</p>\n\
<p>You can now build your plug-in. You might be still missing a few lines in your code where you are using non-Unicode strings or character types. The reason for many linker errors (e.g. fatal error LNK1000) is using incompatible string or character types.</p>\n\
<h2 id=\"common-pitfalls\">Common Pitfalls</h2><h3 id=\"sizeof-and-_countof-\">sizeof() and _countof()</h3><ul>\n\
<li><code>sizeof</code> is used in conjunction with memory allocation functions such as: <code>malloc</code>, <code>memset</code>, <code>memcpy</code>, etc.</li>\n\
<li><code>_countof</code> is used in functions which take both a TCHAR pointer and a size parameter. This is a standard across Microsoft and 3ds Max code. Every time a function needs the length of a TCHAR pointer, it&apos;s always in number of TCHAR, and not a number of bytes. For example: LoadString.</li>\n\
</ul>\n\
<h3 id=\"varying-character-memory-usage\">Varying Character Memory Usage</h3><p>Developers can make mistakes when they write code which processes strings. For instance, it is possible to assume that characters use a constant amount of memory. In the case of WCHAR, this is assumption is false. For example:</p>\n\
<ul>\n\
<li>MBCS characters can take 1 to 2 bytes.</li>\n\
<li>UTF-8 characters can take 1 to 6 bytes.</li>\n\
<li>UTF-16 (WCHAR) characters might take from 2 to 4 bytes.</li>\n\
</ul>\n\
<p>Operations which deal with strings might also yield different string lengths. As such, do not assume that 1 character is equal to 1 byte. For example, changing the casing of a string might change its size in memory. Avoid using <code>sizeof</code> to determine the length of a string.</p>\n\
<h3 id=\"reinterpret_cast-\">reinterpret_cast()</h3><p>When adding an offset in bytes to wchar_t pointers, you will effectively add twice the value. This is why you will see many occurrences of:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">reinterpret_cast&lt;TCHAR *&gt;(reinterpret_cast&lt;char *&gt;(file) + sizeof(astruct));</code></pre><p>Note that sometimes, <code>reinterpret_cast&amp;lt;char *&amp;gt;</code> is missing (only the TCHAR version is present). It was decided that all occurrences of <code>reinterpret_cast&amp;lt;char *&amp;gt;</code> are to be replaced with <code>reinterpet_cast&amp;lt;byte *&amp;gt;</code>.</p>\n\
<h3 id=\"unsigned-char-\">unsigned char*</h3><p>If for any reason you must convert a <code>char*</code> to an <code>unsigned char*</code>, you have to use _TUCHAR. It&apos;s the TCHAR equivalent which guarantees that in both Unicode and ASCII versions, all characters are unsigned and non-negative.</p>\n\
<h3 id=\"tstr-data-is-now-const\">TSTR::data() is now const</h3><p>To prevent invalidating the pointers inside TSTR, the data() method now returns a const MCHAR*. Avoid casting the const string to a normal string.</p>\n\
<h3 id=\"getprocaddress-\">GetProcAddress()</h3><p>The Windows function <code>GetProcAddress()</code> function does not perform any kind of handling when it comes to retrieving Unicode or ASCII functions. You must explicitly handle both cases, whether you have a Unicode or an ASCII build. Also, <code>GetProcAddress()</code> only accepts &quot;char&quot; (NOT Unicode) as input. If you have a TCHAR pointer, you need to convert it before using it. For example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#ifdef UNICODE\n\
functionPointer = (functionProto)GetProcAddress(&quot;FunctionW&quot;);\n\
#else\n\
functionPointer = (functionProto)GetProcAddress(&quot;FunctionA&quot;);\n\
#endif</code></pre><h3 id=\"avoid-calling-tstr-constructors-without-the-_t-macro\">Avoid calling TSTR constructors without the _T macro</h3><p>When you instantiate TSTR object, you have to take attention to implicit conversion. The following code should be avoided. In ASCII mode, it&apos;s fine because the TSTR object is natively storing all its data in ACP. In Unicode mode, this source code will trigger a conversion from ACP to UTF-16.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Avoid this:\n\
TSTR stringObject = &quot;some string&quot;;\n\
TSTR stringObject2(&quot;another string&quot;);</code></pre><p>You should enclose your string inside the _T macro. The conversion to UTF-16, if needed, is performed at compile-time instead. The following is proper use of the _T() macro to wrap string literals:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Do this:\n\
TSTR stringObject = _T(&quot;some string&quot;);\n\
TSTR stringObject2(_T(&quot;another string&quot;));</code></pre><h2 id=\"conventions\">Conventions</h2><p>The following conventions will likely make your code easier to read and manage.</p>\n\
<h3 id=\"implicit-tstrs\">Implicit TSTRs</h3><p>If an operation doesn&apos;t imply a form of convention, for example ACP to UTF16, we favor the implicit constructors and conversions of string objects.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Avoid this:\n\
FunctionCall(TSTR(GetString(IDS_STRING)));\n\
TSTR stringObject = TSTR(_T(&quot;string&quot;));\n\
\n\
// Do this:\n\
FunctionCall(GetString(IDS_STRING));\n\
TSTR stringObject = _T(&quot;string&quot;);</code></pre><h3 id=\"string-concatenation\">String Concatenation</h3><p>TSTR objects have concatenation operators for both TCHAR* and TSTR. If you concatenate a TCHAR* to a TSTR, you don&apos;t have to explicitly construct an intermediate TSTR object.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Avoid this:\n\
TSTR o1 = FunctionCall();\n\
TSTR o2 = o1 + TSTR(FunctionCall()) + TSTR(_T(&quot;string&quot;));\n\
\n\
// Do this:\n\
TSTR o1 = FunctionCall();\n\
TSTR o2 = o1 + FunctionCall() + _T(&quot;string&quot;);</code></pre><h3 id=\"variants\">VARIANTs</h3><p>Since 3ds Max is relies on Windows, many places inside 3ds Max use the OLE variant types: VARIANT, VARIANTARG, and PROPVARIANT. Inside the 3ds Max SDK, we have functions to convert intelligently between VARIANTs and TSTR objects.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">namespace MaxSDK { namespace Util {\n\
    TSTR VariantToString(const PROPVARIANT*, UINT encoding=CP_ACP, USHORT flags=0);\n\
    bool VariantIsString(const PROPVARIANT*);\n\
    bool VariantIsStringVector(const PROPVARIANT*);\n\
    bool SetStringToVariant(PROPVARIANT*, const char* str, bool clear = true);\n\
    bool SetStringToVariant(PROPVARIANT*, const wchar_t* str, bool clear = true);\n\
}}</code></pre><h2 id=\"glossary\">Glossary</h2><p><em>ACP: &quot;Active Code Page&quot;</em> - In Windows or Linux, the active code page always refers to the code page that the underlying operating system would expect to get from an application. It always depends on the current user&apos;s configuration. For example, in US-Windows, the code page is set to 1232 by default. The Chinese Windows code page is set to 936.</p>\n\
<p><em>BOM: &quot;Byte Order Mark&quot;</em> - A sequence of bytes at the beginning of a file that is used to determine the actual format of a file. It identifies if the file is written either UTF-16 Little Endian, UTF-16 Big Endian or UTF-8.</p>\n\
<p><em>MBCS: &quot;Multi Bytes Character System&quot;</em> - An MBCS string is a string where characters use bytes as their base data type but aren&apos;t limited to use a single one. In MBCS, all the control characters such as the null terminator, the carriage return, and the line feed always use a single byte.</p>\n\
<p><em>MCHAR</em> - A synonym of TCHAR that is used in Max SDK&apos;s header files.</p>\n\
<p><em>TCHAR</em> - A primitive data type. When compiled in &quot;Unicode&quot; mode, it becomes a WCHAR (unsigned short), otherwise, in MBCS mode, it&apos;s defined as a signed byte.</p>\n\
<p><em>UTF-16 (UCS-2)</em> - The wide char standard implemented by Windows. It stores characters on 2 or 4 bytes.</p>\n\
<p><em>UTF-32 (UCS-4)</em> - Unicode character encoding that takes exactly 32 bits.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../what_s_new_3ds_max_2013_sdk.html\">What&#x27;s New: 3ds Max 2013 SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";