var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"vc10_compiler_porting\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>VC10 Compiler Porting</h1></div><h2 id=\"introduction\">Introduction</h2><p>3ds Max 2013 has been upgraded to use version 10.0 of the Microsoft C/C++ compiler. This is the compiler that comes with Visual Studio 2010 Service Pack 1. Plug-in developers can now use Visual Studio 2010 with Platform Toolset vc100 to compile their projects. This document is a guide for upgrading plugins to compile with Visual Studio Service Pack 1.</p>\n\
<h2 id=\"breaking-changes-in-3ds-max\">Breaking Changes in 3ds Max</h2><p>With the introduction of VC 10.0 there were few breaking changes in the max code base. These are specific, or mostly specific to the 3ds Max code base. Simple issues such as adding includes will not be discussed here are they are simple to diagnose, and already described on the MSDN.</p>\n\
<h2 id=\"renamed-code-paramtags-end-code-to-code-paramtags-p_end-code-\">Renamed <code>ParamTags::end</code> to <code>ParamTags::p_end</code></h2><p>Inside of <code>ParamType.h</code> is an enum called <code>ParamTags</code> with an enumerator called <code>end</code>. This conflicts with new symbols in the STL <code>std</code> namespace. For instance,</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">1&gt; ...\\xmlmtlexporter.cpp(233): error C2872: &apos;end&apos;: ambiguous symbol\n\
1&gt;    could be &apos;...\\paramtype.h(665): ParamTags end&apos;\n\
1&gt;    or    &apos;end&apos;</code></pre><p>The compiler is not going to show what the conflicting symbol is. However this is caused by adding a &apos;<code>using namespace std</code>&apos; declaration in a header file. It is generally not recommended to add a using namespace declaration in a header file. Also, the symbol name &apos;end&apos; was too generic, so the enumeration was renamed to avoid potential conflicts here and in the future.</p>\n\
<h2 id=\"small-block-heap-no-longer-supported\">Small block heap no longer supported</h2><p>The C Runtime (CRT) header file <code>malloc.h</code> no longer declares nor defines the functions <code>\\_get_sbh_threshold</code> or <code>\\_set_sbh_threshold</code>. As such, any usage of these functions should be removed.</p>\n\
<h2 id=\"stl-set-iterators-are-all-code-const-code-\">STL Set Iterators are all <code>const</code></h2><p>The C++ Standard Template Library in VC 10.0 no longer supports non-<code>const</code> iterators for sets and <code>hash_sets</code>. Thus, the <code>std::set&amp;lt;&amp;gt;::iterator</code> behaves identically to the <code>const_iterator</code>.</p>\n\
<p>In particular, de-referencing the iterators will now return const types instead of non-<code>const</code> types. For instance, the following code may no longer work for an <code>std::set</code> iterator.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MyType&amp; t = *it;</code></pre><p>Instead, you would have to make the type <code>const</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">const MyType&amp; t = *it;</code></pre><p>You could alternatively perform a copy using the assignment operator, and drop the reference symbol.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MyType t = *it;</code></pre><p>To ensure that iterators are not altered within a loop, <em>only <code>const</code> methods can be called on <code>const</code> types</em>. Because set and hash_set is keyed on the type itself, performing non-const operations on the instance is now forbidden.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Avoid the following for an iterator &apos;it&apos;:\n\
(*it)-&gt;changeThis();\n\
\n\
// Proper call of a const method:\n\
(*it)-&gt;MyConstMethod();</code></pre><p>One way to fix this is to make the method declaration const. For instance, the virtual method <code>SelectFilterCallback::IsFiltered</code> in <code>maxapi.h</code> was not <code>const</code> and failed to compile because the filters were stored in a set, and iterated using iterators. Changing this method to const enabled the code to compile properly. Note however that the compiler does not always flag such instances as issues. For instance, the following code compiles properly:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">const AnimatableNotifierPtr&amp; pNotifier = (*it);\n\
pNotifier-&gt;SetOwnerSet(NULL);</code></pre><p>In cases where the iterator will be modified, a useful alternative to <code>std::set&amp;lt;&amp;gt;</code> is <code>std::map&amp;lt;&amp;gt;</code>. For instance, a hash_set can be replaced by a hash_map.</p>\n\
<h2 id=\"named-template-parameters\">Named Template Parameters</h2><p>Passing in arguments to functions through the template type instead of the normal function parameter list is no longer supported. The following example illustrates the proper use of templates.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Wrong: the &apos;AccumulatorType wholeScale&apos; is included in the template definition.\n\
template &lt;class AccumulatorType, class DataType, AccumulatorType wholeScale&gt;\n\
static inline void scale(const DataType *source, int sourceCount, DataType *dest, int destCount) { \n\
    //...\n\
}\n\
\n\
// Correct: the &apos;AccumulatorType wholeScale&apos; is a parameter in the scale() function.\n\
template &lt;class AccumulatorType, class DataType&gt;\n\
static inline void scale(const DataType* source, int sourceCount, DataType* dest, int destCount, AccumulatorType wholeScale) {\n\
    // ...\n\
}</code></pre><h2 id=\"-net-4-0\">.NET 4.0</h2><p>Previous versions of 3ds Max used a range of .NET 2.0 and .NET 4.0 functionality. 3ds Max 2013 now only uses .NET 4.0.</p>\n\
<h2 id=\"possible-compiler-errors\">Possible Compiler Errors</h2><p>The following is a list of frequently encountered compiler errors encountered while porting 3ds Max to the VC 10.0 compiler.</p>\n\
<p><em>error C2678</em> - <code>std::set&amp;lt;&amp;gt;::iterator</code> is now const; it cannot have non-const methods invoked on it.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">binary &apos;=&apos; : no operator found which takes a left-hand operand of type &apos;const &lt;type&gt;&apos; (or there is no acceptable conversion)</code></pre><p>Operations such as the following ones are therefore disallowed:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Will not compile:\n\
MaterialTemplates::iterator p = MaterialTemplates::msTemplates.find(info.mID);\n\
*p = info;</code></pre><p><em>error C2872</em> or <em>error C2664</em> - <code>ParamTags::end</code> was renamed to <code>ParamTags::p_end</code> in <code>paramtype.h</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">error C2872: &apos;end&apos; : ambiguous symbol\n\
    could be &apos;&lt;filepath&gt; : ParamTags end&apos;\n\
    or       &apos;end&apos;</code></pre><p><em>error C1189</em> - The use of <code>\\#define _WIN32_WINNT 0x0400</code> targets Windows 95 and below. Note that 3ds Max is not supported on Windows 95, nor Windows 98, nor Windows 2000. Plugin developers should adhere to <code>\\#define _WIN32_WINNT 0x0502</code> (MaxWindowsVersion.h in the sdk), corresponding to Windows XP Service Pack 2.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">fatal error C1189:\n\
#error : This file requires _WIN32_WINNT to be #defined at least to 0x0403. The value 0x0501 or higher is recommended.</code></pre><p><em>error C2440</em> - This may occur if you are attempting to pass a NULL or zero (0) value to an STL method or constructor which requires a real pointer. Note that the keyword <code>nullptr</code> has been introduced in VC 10.0 to account for null value instantiations.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">error C2440: &apos;initializing&apos; : cannot convert from &apos;int&apos; to &apos;StPathObject *&apos;</code></pre><p>For example, an <code>std::map</code> can be defined in the following way:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">std::map&lt;int*, int*&gt; mymap;</code></pre><p>If you tried to insert NULL into it, the compiler would complain with the warning above.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Will not compile:\n\
mymap.insert(std::pair&lt;int*, int*&gt;(NULL, NULL));</code></pre><p>Similarly, defining a standard pair is also invalid:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Will not compile:\n\
std::pair&lt;int*, int*&gt;mypair(NULL, NULL);</code></pre><p>Both of these approaches can be fixed by passing in properly typed arguments. For instance, you can now pass in <code>nullptr</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Will compile:\n\
mymap.insert(std::pair&lt;int*, int*&gt;(nullptr, nullptr));</code></pre><p><em>error C2039</em> - Resolved by using boost version 1.44 or newer.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">1&gt; ...\\file.hpp(141): error C2039: &apos;unchecked_copy&apos; : is not a member of &apos;stdext&apos;</code></pre><p>This is documented by Microsoft as a breaking change, where it says &quot;In the &lt;algorithm&gt; header, the checked_* and unchecked_* functions are removed&quot;.</p>\n\
<h2 id=\"possible-linker-errors\">Possible Linker Errors</h2><p><em>error LNK2019</em> - A result of linking against libraries which contain STL code compiled with VC 9.0.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">error: LNK2019: unresolved external symbol &quot;__declspec(dllimport) public:class std::basic_string&lt;whcar_t,struct std::char_traits&lt;wchar_t&gt;,...&quot;</code></pre><p><em>error LNK2005</em> - Occurs with DLLs which make extensive use of MFC. Ensure that stdafx.h is included first at the top of every .cpp file in the project. Consult the knowledge base article on Microsoft: <a href=\"http://support.microsoft.com/kb/148652\" target=\"_blank\">http://support.microsoft.com/kb/148652</a>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">error: LNK2005: _DllMain@12 already defined in MSVCRTD.LIB (dllmain.obj)\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../what_s_new_3ds_max_2013_sdk.html\">What&#x27;s New: 3ds Max 2013 SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";