var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"sdk_changes_for_automatic_gamma_\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>SDK Changes for Automatic Gamma Correction</h1></div><p>This topic first defines how things work in general, and then describes how automatic gamma correction affects the SDK.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>  <strong></strong> Color Management has been substantially changed in 3ds Max 2024. See <a href=\"topicid=bitmap_gamma_correction\">Bitmap Color Management</a>.</p>\n\
\n\
</div></div>\n\
\n\
<h2 id=\"the-code-bitmapinfo-code-class\">The <code>BitmapInfo</code> Class</h2><p>The <code>BitmapInfo</code> class contains all the information about a bitmap and is used in many cases:</p>\n\
<ul>\n\
<li>The <code>Bitmap</code> class has a <code>BitmapInfo</code> member that provides information about a bitmap. The <code>BitmapInfo</code> is used when loading and saving bitmaps, for example, to define what to be loaded and saved and how, what settings to use, and others.</li>\n\
<li>You can edit the <code>BitmapInfo</code> in the file dialog boxes (for example, <code>BitmapManager::SelectFileInput()</code> or <code>BitmapManager::Output()</code>)</li>\n\
</ul>\n\
<p>The <code>BitmapInfo</code> contains the filename and asset tag for a bitmap and the associated gamma values and flags. It also contains an opaque data pointer with additional information that is used by a <code>BitmapIO</code> plug-in when saving or loading a file. For example, the plug-in stores information about the JPEG compression settings as additional information.</p>\n\
<p>You must never set, modify, or alter the plug-in data directly. It is edited by the particular setup dialog of a given <code>BitmapIO</code> plug-in (the dialog that pops up when the <strong>Setup</strong> button is clicked in a bitmap file dialog).</p>\n\
<h3 id=\"guidelines\">Guidelines</h3><p>Instead of working with file names, you must use <code>BitmapInfo</code>. In general, the code must avoid dealing only with filenames. When dealing with filenames, these guidelines must be followed:</p>\n\
<ul>\n\
<li>You can change the filename of a <code>BitmapInfo</code> if the file type remains the same, for example, changing the path of a given .JPG.</li>\n\
<li>If you are changing the file type (extension), you must flush the plug-in data by calling <code>BitmapInfo::ResetPiData()</code>.</li>\n\
</ul>\n\
<p>The following cases indicate what this means:</p>\n\
<ul>\n\
<li>You can obtain a <code>BitmapInfo</code> from the user through a dialog such as <code>BitmapManager::SelectFileInput()</code> when loading a bitmap file. You can use this <code>BitmapInfo</code> and pass it to <code>BitmapManager::Open()</code>.</li>\n\
<li>Similarly, you can obtain a <code>BitmapInfo</code> from the user through a dialog such as <code>BitmapManager::SelectFileOutput()</code> when saving a bitmap file. You can use this <code>BitmapInfo</code> and pass it to <code>Bitmap::OpenOutput()</code>.</li>\n\
<li>Base things on a <code>BitmapInfo</code> obtained from the user when saving after modifying names or paths, and avoid modifying the path to a different extension.</li>\n\
<li>Store the <code>BitmapInfo</code> and not the filename when saving data to the scene persistently.</li>\n\
</ul>\n\
<p>You do not need to modify the code to support the new automatic gamma correction if these guidelines are followed. However, if your code is creating its own <code>BitmapInfo</code> or modifying them extensively, or is working with only filenames, you might have to make necessary changes. See the following section for more information.</p>\n\
<h2 id=\"understanding-the-changes-in-the-sdk\">Understanding the Changes in the SDK</h2><p>In the <code>BitmapInfo</code> class, gamma correction is handled by these members: <code>gamma</code>, <code>custgamma</code>, and <code>customflags</code>.</p>\n\
<ul>\n\
<li>Unless overridden by the user, <code>gamma</code> is the gamma value. It is the value determined by 3ds Max for file read/write. It is retrieved with <code>BitmapInfo::Gamma()</code> and set with <code>BitmapInfo::SetGamma()</code>.</li>\n\
<li>When overridden by the user, <code>custgamma</code> is the custom gamma value. It maps to the <strong>Override</strong> option in the bitmap file dialog. It is retrieved with <code>BitmapInfo::GetCustomGamma()</code> and set with <code>BitmapInfo::SetCustomGamma()</code>.</li>\n\
<li>The various flags which decide which gamma value to use are contained in <code>customflags</code>. It is retrieved with <code>BitmapInfo::GetCustomFlags()</code>, and set with <code>BitmapInfo::SetCustomFlag()</code> and <code>BitmapInfo::ResetCustomFlag()</code>.</li>\n\
</ul>\n\
<h3 id=\"gamma-flags\">Gamma Flags</h3>\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>  <strong></strong> This information applies only when using gamma-based color management. See <a href=\"topicid=6BCD1EAA-1F16-4758-A07B-C2E31F94BEB2\">Interface: ColorPipelineMgr</a>.</p>\n\
\n\
</div></div>\n\
\n\
<p>There are a set of flags related to gamma in the <code>customflags</code> member. Two of them (<code>BMM_CUSTOM_GAMMA</code> and <code>BMM_CUSTOM_FILEGAMMA</code>) are important for how gamma is actually read or written. There are three valid combinations:</p>\n\
<ul>\n\
<li><code>BMM_CUSTOM_FILEGAMMA</code> - When set, it indicates that the actual gamma to use is in the <code>gamma</code> member, and after reading it indicates that it is from the file.</li>\n\
<li><code>BMM_CUSTOM_GAMMA</code> - When set, it indicates that the actual gamma to use is in the <code>custgamma</code> member, and that it is from a user override.</li>\n\
<li>If neither flags are set, it indicates that the gamma is the globally configured value (generally 2.2).</li>\n\
</ul>\n\
<p>These flags map to the options in the file dialog as described in <a href=\"understanding_the_automatic_gamm.html\"\">Understanding the Automatic Gamma Correction</a>. The following table shows the previous mapping.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Option</th>\n\
<th>Flag</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><strong>Use image&#39;s own gamma</strong></td>\n\
<td><code>BMM_CUSTOM_FILEGAMMA</code> is set</td>\n\
</tr>\n\
<tr>\n\
<td><strong>Override</strong></td>\n\
<td><code>BMM_CUSTOM_GAMMA</code> is set</td>\n\
</tr>\n\
<tr>\n\
<td><strong>Use System default gamma</strong></td>\n\
<td>No flag is set</td>\n\
</tr>\n\
</tbody></table>\n\
<p>The following table shows the current mapping.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Option</th>\n\
<th>Flag</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><strong>Automatic</strong></td>\n\
<td><code>BMM_CUSTOM_FILEGAMMA</code> is set</td>\n\
</tr>\n\
<tr>\n\
<td><strong>Override</strong></td>\n\
<td><code>BMM_CUSTOM_GAMMA</code> is set</td>\n\
</tr>\n\
</tbody></table>\n\
<p>Currently, the third option where neither flag is set is no longer valid. However, it is still adhered to for compatibility reasons (files will continue to work as they did in the past), but passing a <code>BitmapInfo</code> with neither flag set to any file dialog function actually converts to a matching override setting.</p>\n\
<h3 id=\"meaning-of-the-code-bmm_custom_filegamma-code-flag\">Meaning of the <code>BMM_CUSTOM_FILEGAMMA</code> Flag</h3><p>The change in the meaning of the <code>BMM_CUSTOM_FILEGAMMA</code> flag is important and is explained in the following table.</p>\n\
<table>\n\
<colgroup>\n\
<col width=\"50%\">\n\
<col width=\"50%\">\n\
</colgroup>\n\
<thead>\n\
<tr class=\"header\">\n\
<th>Before</th>\n\
<th>Current</th>\n\
</tr>\n\
</thead>\n\
<tbody>\n\
<tr class=\"odd\">\n\
<td>When set before loading a file, it indicates that gamma is loaded from the file header, if it is available.</td>\n\
<td>When set before loading a file, it flags the **Automatic** option. As before, the gamma value in the file header is used. If the file header does not have a gamma value, the gamma is determined based on the file type.</td>\n\
</tr>\n\
<tr class=\"even\">\n\
<td>When the flag remained set after loading the file, it indicates that gamma is read from the file header. If not, the flag is cleared.</td>\n\
<td>The flag remains set after loading the file, and if gamma value is missing in the file header, the flag is not cleared. Instead, the new information flag `BMM_CUSTOM_INFERREDGAMMA` is set to simply inform any caller that gamma is inferred and is not read from the file header.</td>\n\
</tr>\n\
<tr class=\"odd\">\n\
<td>It is not valid to set the flag before saving.</td>\n\
<td>The flag is valid when saving, and is actually the default behaviour. The gamma used for saving the file is based on the file type, either sRGB (= gamma 2.2) or linear (= gamma 1.0).</td>\n\
</tr>\n\
</tbody>\n\
</table>\n\
\n\
<h3 id=\"determining-how-to-load-or-save-file-types\">Determining How to Load or Save File Types</h3><p>A new interface in the <code>BitmapIO</code> class called <code>BitmapIOMetaData</code> is used to determine whether the file types are saved and loaded linearly or with an embedded gamma.</p>\n\
<p>The bitmap loading and saving code can inquire the appropriate <code>BitmapIO</code> with <code>GetInterface(BITMAPIOMETADATA_INTERFACE_ID)</code> to obtain this interface. If the interface is missing, the globally defined gamma value is used. This means that for any unmodified <code>BitmapIO</code> plugin, the behavior is the same as before.</p>\n\
<p><code>BitmapIO</code> plug-ins dealing with floating point data, log data, or other data that must be stored linearly can expose this interface. It has a <code>BitmapIOMetaData::UseLinearStorage()</code> function which is called after loading or before saving. It returns true if linear storage is preferred, and false if not.</p>\n\
<p>For information about avoiding gamma related issues when writing code, see the <a href=\"./../../../../best_practices/automatic_gamma_correction_best_.html\">Automatic Gamma Correction Best Practices</a> topic.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../automatic_gamma_correction.html\">Automatic Gamma Correction</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";