var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"nvidia_cg_support\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>NVIDIA Cg Support</h1></div><p>The 3ds Max 8 SDK shader interface required that the shader be loaded and compiled by the material. The parser simply got the compiled object. NVIDIA has a different compile system, so without fundamental changes to the shader interface design, the material would have to include a special case for Cg support - a definite limitation. Moving the loading and compilation of the shaders removes the restriction of the type of shader the system can support. The material simply supplies a filename or buffer to compile.</p>\n\
<p>Beyond the need to introduce Cg support, new shader languages are often introduced with the release of new gaming platforms. The new 3ds Max 9 shader language interface breaks a material&apos;s reliance on any specific interface, in this case the LPD3DEffect interface, allowing runtime bindings to take place in the parser DLLs. This approach makes the materials and the 3ds Max-centric aspects of shading independent of specific shader languages with D3D runtime support.</p>\n\
<p>Within the 3ds Max 9 SDK itself, changes were made to define parameter access by name, and not by handle. Plug-in developers can maintain a list of handles internally for use directly with the runtime, but any access to 3ds Max is by name. Even though, in theory, the <code>D3DXHANDLE</code> is a pointer to a string, it can be ambiguous as to whether it is a handle or an actual string - and this could cause problems if the interface assumes a string.</p>\n\
<p>Previously all data that 3ds Max provided was set by 3ds Max in the material. This is now impossible as the parser maintains the link to the runtime, so it needs to set the data itself. The new system now provides all the data by means of the<code>IParameterManager</code>. This is a new interface that provides access to the data requested by the parsers. The following code sample shows how to access data either by name or iteratively. The choice of approach depends on how the parser holds its internal data.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">switch(pm-&gt;GetParamType(i))\n\
{\n\
    case  IParameterManager::kPType_Float:\n\
    {\n\
        float fval;\n\
        pm-&gt;GetParamData((void*)&amp;fval,i);\n\
        pEffect-&gt;SetFloat(pm-&gt;GetParamName(i), fval);\n\
    }\n\
    break;\n\
    case  IParameterManager::kPType_Color:\n\
    case  IParameterManager::kPType_Point4:\n\
    {\n\
        D3DCOLORVALUE cval;\n\
        pm-&gt;GetParamData((void*)&amp;cval,i);\n\
        pEffect-&gt;SetVector(pm-&gt;GetParamName(i), (D3DXVECTOR4*)&amp;cval);\n\
    }\n\
    break;\n\
    case  IParameterManager::kPType_Bool:\n\
    {\n\
        BOOL bval;\n\
        pm-&gt;GetParamData((void*)&amp;bval,i);\n\
        pEffect-&gt;SetBool(pm-&gt;GetParamName(i), bval);\n\
    }\n\
    break;\n\
    case  IParameterManager::kPType_Int:\n\
    {\n\
        int ival;\n\
        pm-&gt;GetParamData((void*)&amp;ival,i);\n\
        pEffect-&gt;SetInt(pm-&gt;GetParamName(i), ival);\n\
    }\n\
    break;\n\
}</code></pre><p>As more shader types become available, materials must be able to display these in the File Opendialog. To accommodate this, new methods have been added to the <code>EffectDescriptor</code>. The parser defines supported file extensions in the descriptor since the actual parser has not been loaded until the file has been accessed to determine which parser to load. File extensions are supplied as a localizable strings.</p>\n\
<p>To allow the future support of DirectX10, some methods now use <code>void</code> data types, which are then cast accordingly. For example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">bool MaxEffectParser::LoadTexture(LPVOID pDevice, LPD3DXEFFECT pEffect,TCHAR * paramName, TCHAR * filename, bool forceReload)\n\
{\n\
    LPDIRECT3DDEVICE9 pd3dDevice = static_cast&lt;LPDIRECT3DDEVICE9&gt;(pDevice);\n\
}</code></pre><p>Plug-in developers can query which version of DirectX is running so they can perform the appropriate cast. This approach kept the API changes smaller, otherwise a copy of the <code>IEffectParser</code> interface would be needed ( <code>IEffectParser10</code>) that had the same methods but with Dx10 counterparts.</p>\n\
<p>The material is responsible for displaying any errors generated, so additional parser methods need to provide access to error buffers.</p>\n\
<h2 id=\"technical-solutions\">Technical solutions</h2><p>As noted above, the parser needs to compile and set all the parameters of the shader. All data is transferred around the system by using the <code>void</code> data type. The calling function must make sure that the buffer is large enough for the data to be copied. Currently, 3ds Max does not check this condition, it just fills the buffer.</p>\n\
<p>Here is an example of data defined asLPVOID:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">D3DXMATRIXMatInvWorld;\n\
D3DXMatrixInverse(&amp;MatInvWorld,NULL,&amp;MatWorld);\n\
*(D3DXMATRIX*)data = MatInvWorld;</code></pre><p>The internal D3DXMATRIXinternally is used for convenience and is defined as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">typedef struct D3DXMATRIX {\n\
    FLOAT _11, FLOAT _12, FLOAT _13, FLOAT _14,\n\
    FLOAT _21, FLOAT _22, FLOAT _23, FLOAT _24,\n\
    FLOAT _31, FLOAT _32, FLOAT _33, FLOAT _34,\n\
    FLOAT _41, FLOAT _42, FLOAT _43, FLOAT _44 );\n\
} D3DXMATRIX;</code></pre><p>This is a fairly standard representation. All loading and parsing for the shaders now takes place in the parser. A typical function that uses the new method of <code>IEffectParser</code> is shown in the following code sample:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">bool MaxEffectParser::LoadEffectFile(LPVOID pDevice, IEffectManager * em,TCHAR * fileName, bool forceReload)\n\
{\n\
    int index;\n\
    HRESULT hr;\n\
    LPDIRECT3DDEVICE9 pd3dDevice = static_cast&lt;LPDIRECT3DDEVICE9&gt;(pDevice); //Used later when we have a Dx10 version of the parser.\n\
    bDirectX9 = em-&gt;GetDirectXVersion() ==  IEffectManager::kDirectX9?true:false;\n\
    index = IDxShaderCache::GetIDxShaderCache()-&gt;FindShader(fileName);\n\
    if(index&lt;0)\n\
    {\n\
        D3DXMACRO macros[5 =\n\
        {\n\
            { &quot;_3DSMAX_&quot;, NULL },\n\
            { &quot;_MAX_&quot;,NULL },\n\
            { &quot;MAX&quot;,NULL },\n\
            { &quot;3DSMAX&quot;,NULL },\n\
            { NULL,NULL }\n\
        };\n\
        hr = D3DXCreateEffectFromFile(pd3dDevice,fileName,macros,NULL,NULL,NULL,&amp;m_pEffect,NULL);\n\
        if(FAILED(hr) || !m_pEffect){\n\
             return false ;\n\
        }\n\
        IDxShaderCache::GetIDxShaderCache()-&gt;SetShader(fileName,(LPCVOID)m_pEffect);\n\
    }\n\
    else\n\
    {\n\
             m_pEffect = (LPD3DXEFFECT) IDxShaderCache::GetIDxShaderCache()-&gt;GetShader(index);\n\
    }\n\
    return true;\n\
}</code></pre><h2 id=\"parameter-access-is-by-name-only\">Parameter Access is By Name Only</h2><p>The most the developer will need to do is the following (assuming D3D):</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">texHandle = pEffect-&gt;GetParameterByName(NULL,paramName);</code></pre><p>from then on, the internal method can be used as it was before. This is exactly 3ds Max 9 works internally. The only other major code change is to use the <code>IParameterManager</code>to access all the data held by 3ds Max.</p>\n\
<p>Any <code>D3DXHANDLE</code>s in the interfaces point to strings, not handles.</p>\n\
<p>As the DirectX Shader material is no longer responsible for loading and maintaining the effect, the <code>IDxMaterial3::GetCurrentEffect</code>() method is no longer available. Plug-in developers should use the filename to load the correct effect interface using the appropriate run time.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../what_s_new_3ds_max_9_sdk.html\">What&#x27;s New: 3ds Max 9 SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";