var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"vault_and_asset_tracking\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Vault and Asset Tracking</h1></div><p>We have improved the public <em>3ds Max SDK</em> interface to the <em>Asset Tracking System</em>: methods and parameters have been added and method usage simplified.</p>\n\
<h2 id=\"iatsprovider-porting-guide\">IATSProvider Porting Guide</h2><p>This document outlines changes to the <code>IATSProvider</code> related classes that might require changes to your plug-ins. This applies to code implementing <code>IATSProvider</code> plug-ins or interacting with them.</p>\n\
<p><em>New Status Flags</em></p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">const  ATSStatus  kATSStatusHide = 0x0010;</code></pre><p><code>ATSFileEntry</code> objects that return the bit <code>kATSStatusHide</code> from <code>ATSFileEntry::GetFlagsIn()</code> have been requested to be hidden when added to the version control provider. The implementation of hidden will depend on the version control provider. For instance, in the case of <em>Autodesk Vault</em> any file marked as hidden would not be visible in <em>Autodesk Vault Explorer</em> . It is optional for a version control provider to support this flag.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">const  ATSStatus  kATSStatusHidden = 0x00000100;</code></pre><p><code>ATSFileEntry</code> objects that return the bit <code>kATSStatusHidden</code> from <code>ATSFileEntry::GetFlagsOut()</code> have been marked as hidden by the version control system. This status would be set from a call to <code>IATSProvider::GetFileStatus</code>. It is optional for a version control provider to support this flag.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">const  ATSStatus  kATSStatusShareLock = 0x00000200;</code></pre><p><code>ATSFileEntry</code> objects that return the bit <code>kATSStatusShareLock</code> from <code>ATSFileEntry::GetFlagsOut()</code> have been marked as <em>Share Locked</em> by the version control system. Files marked as <em>Share Locked</em> should be treated as read only files by the system. The actual conditions that signify a <em>Share Locked</em> file is determined by the version control provider. In <em>Autodesk Vault</em>, a <em>Share Locked</em> file is marked as such when a file is checked out to a shared location and a second user attempts to open that file. The <em>Share Locked</em> status is used to prevent the second user from overwriting the first user&apos;s file. It is optional for a version control provider to support this flag.</p>\n\
<p><em>Specifying Attachments for ATSFileEntry Objects</em></p>\n\
<p>Attachments to <code>ATSFileEntry</code> objects are now stored as <code>ATSFileEntry</code> objects instead of a character string.</p>\n\
<p>Attachments can be added directly with a supplied <code>ATSFileEntry</code> object:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">UINT  ATSFileEntry::AddAttachment(  ATSFileEntry *pEntry );</code></pre><p>Or by specifying the parameters for the <code>ATSFileEntry</code> object with:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">UINT  ATSFileEntry::AddAttachment( const TCHAR *szFile,  ATSFileEntry *pEntryParent = NULL,  ATSStatus dwFlagsIn = ATS::kATSStatusActive,  ATSClientPtr pClientPtr = NULL );</code></pre><p>Attachments are now returned as <code>ATSFileEntry</code> object instead of character strings:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">ATSFileEntry* GetAttachment( UINT iIndex );</code></pre><p><em>Working Folder</em></p>\n\
<p>New pure virtual methods were added to <code>IATSProvider</code> to allow for access to the <em>Working Folder</em> associated with the version control provider. The ability to access or alter the working folder is determined by the version control provider. If the functionality is not supported, <code>kATSResErrorNotSupported</code> should be returned.</p>\n\
<p>virtualconst TCHAR* GetWorkingFolder() = 0;</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">virtual  ATSResult SetWorkingFolder( const TCHAR *szWorkingFolder ) = 0;\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../what_s_new_3ds_max_9_sdk.html\">What&#x27;s New: 3ds Max 9 SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";