var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"what_s_new_3ds_max_2020_sdk\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>What&apos;s New: 3ds Max 2020 SDK</h1></div><p>This topic describes new items and changes that affect the 3ds Max 2020 C++ SDK.</p>\n\
<!-- TOC -->\n\
\n\
<ul>\n\
<li><a href=\"#sdk-break\">SDK Break</a></li>\n\
<li><a href=\"#microsoft-visual-studio-2017\">Microsoft Visual Studio 2017</a></li>\n\
<li><a href=\"#ui\">UI</a></li>\n\
<li><a href=\"#plugin-package-requirements\">Plugin Package Requirements</a></li>\n\
<li><a href=\"#modifier-plugin-class-changes-to-support-pb1-to-pb2-migration\">Modifier Plugin Class Changes to Support Pb1 to Pb2 Migration</a></li>\n\
<li><a href=\"#modeling\">Modeling</a></li>\n\
<li><a href=\"#rendering\">Rendering</a></li>\n\
<li><a href=\"#miscellaneous\">Miscellaneous</a></li>\n\
<li><a href=\"#deprecated-apis\">Deprecated APIs</a></li>\n\
</ul>\n\
<!-- /TOC -->\n\
\n\
<h2 id=\"sdk-break\">SDK Break</h2><p>The 3ds Max 2020 SDK is not binary backward compatible with the 3ds Max 2019 SDK. This means that plugins will need to be re-compiled in order to work in 3ds Max 2020.  See the <a href=\"./../about_the_3ds_max_sdk/sdk_requirements.html\">SDK Requirements</a> topic for more information on the platform requirements for building plugins in 3ds Max 2020.</p>\n\
<h2 id=\"microsoft-visual-studio-2017\">Microsoft Visual Studio 2017</h2><p>3ds Max 2020 is compiled with Microsoft Visual Studio 2017 version 15.8.3, Platform Toolset v141, and Windows Platform SDK version 10.0.17134.0. plugins need to be built using the same Platform Toolset version. Any version of Microsoft Visual Studio can be used as long as the Platform Toolset used is v141.</p>\n\
<p>Note that installers for earlier versions of Microsoft Visual Studio 2017 are available <a href=\"https://docs.microsoft.com/en-us/visualstudio/productinfo/installing-an-earlier-release-of-vs2017\" target=\"_blank\">here</a>.</p>\n\
<h3 id=\"switch-from-fast-to-precise-math-computation\">Switch from Fast to Precise Math Computation</h3><p>3ds Max 2019 and many previous versions have been compiled using the &quot;fast math&quot; optimization of the Visual Studio C++ compiler (see <a href=\"https://docs.microsoft.com/en-us/cpp/build/reference/fp-specify-floating-point-behavior?view=vs-2017\" target=\"_blank\"><code>/fp:fast</code></a> VC++ compile option documentation).</p>\n\
<p>In Visual Studio 2017 version 15.3 and 15.5, the code optimization for the <code>/fp:fast</code> option has been further modified to provide even faster code than in Visual Studio 2015. </p>\n\
<p>Because the speed improvement comes with some loss in precision, after careful consideration of the impact on both 3ds Max and 3rd party plugins, we decided to switch from using the &quot;fast math&quot; optimization to building with the &quot;precise math&quot; compiler option (see <a href=\"https://docs.microsoft.com/en-us/cpp/build/reference/fp-specify-floating-point-behavior?view=vs-2017\" target=\"_blank\">&apos;/fp:precise&apos;</a> VC++ compile option documentation).</p>\n\
<p>Based on our testing, switching to &quot;precise math&quot; does not impact 3ds Max&apos;s performance in any significant way, and it has the following benefits:</p>\n\
<ul>\n\
<li>Protects from future optimizations in the Visual Studio compilers, which might lead to visible differences in 3ds Max features that can inflate small discrepancies in floating point numbers into large and very visible artifacts on meshes, for example;</li>\n\
<li>Enables a compiler warning when a floating point value is truncated/rounded and a loss of accuracy happens. Under &quot;fast math&quot; this was previously ignored by the compiler. We have fixed over 900 such warnings.</li>\n\
</ul>\n\
<blockquote>\n\
<p>Note that 3rd party code can continue to use &quot;fast math&quot; if needed; switching to &quot;precise math&quot; is not mandatory.  However, be aware that there is a significant change in the way <code>Sin()</code> and <code>Cos()</code> are optimized when called side by side with the same argument when using &quot;fast math&quot; that reduces the accuracy of the result.  The new behavior is documented <a href=\"https://blogs.msdn.microsoft.com/vcblog/2017/12/03/msvc-code-optimizer-improvements-in-visual-studio-2017-versions-15-5-and-15-3/\" target=\"_blank\">here</a>.</p>\n\
</blockquote>\n\
<h2 id=\"ui\">UI</h2><h3 id=\"qt-support-for-parameter-blocks-based-ui\">Qt Support for Parameter Blocks Based UI</h3><p>Class <code>ClassDesc2_Extension2017SP2</code> has been deprecated, and the method below has been moved to class <code>ClassDesc2</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">// Deprecated. Please overwrite the method with the same signature from class ClassDesc2\n\
virtual MaxSDK::QMaxParamBlockWidget* ClassDesc2_Extension2017SP2::CreateQtWidget(\n\
    ReferenceMaker&amp; owner,\n\
    IParamBlock2&amp; paramBlock,\n\
    const MapID paramMapID,\n\
    MSTR&amp; rollupTitle,\n\
    int&amp; rollupFlags,\n\
    int&amp; rollupCategory,\n\
    Class_ID&amp; tabID) = 0;</code></pre>\n\
<h3 id=\"qt-support-in-the-3ds-max-plugin-wizard\">Qt Support in the 3ds Max Plugin Wizard</h3><p>The 3ds Max Plugin Wizard has been updated to support generating code for Qt-based user interfaces for Utility Object, Geometric Object and Modifier plugin types.  See the Plugin Wizard readme and the <a href=\"./../writing_plug-ins/using_qt_with_3ds_max_plug-ins.html\">Using Qt with 3ds Max Plug-ins</a> topic for more information about using Qt for plug-in user interfaces.</p>\n\
<h3 id=\"maxscript-code-rolloutcontrol-code-\">MAXScript <code>RolloutControl</code></h3><p>If a third party rollout control supports a controller object (an instance of class Control), it will need to derive from the new class <code>AnimatableRolloutControl</code> rather than class <code>RolloutControl</code>. Any implementation code that currently accesses directly the &quot;controller&quot; data member, will need to call <code>Control* RolloutControl::get_controller()</code> instead. </p>\n\
<h2 id=\"plugin-package-requirements\">Plugin Package Requirements</h2><p>In versions of 3ds Max prior to 2020, the <code>UpgradeCode</code> specified by a plugin package&apos;s <em>PackageContents.xml</em> manifest file was ignored since plugin packages were identified uniquely by their <code>ProductCode</code>.</p>\n\
<p>Since the technology used by the Autodesk Developer Network to generate installers for plugin packages relies on the <code>UpgradeCode</code> to remain unchanged when a package with an updated plugin is released, while the <code>ProductCode</code> needs to change every time the plugin (or application) version is updated, 3ds Max 2020&apos;s plugin package discovery and loading mechanism has been updated.</p>\n\
<p>The manifest of plugin packages need to adhere to the following requirements:</p>\n\
<ul>\n\
<li>The <code>UpgradeCode</code> is required and should not change between updates of a particular plugin. \n\
  For example, if the first release of a plugin is <strong>plugin_v1.msi</strong>, and the second version is <strong>plugin_v2.msi</strong>, the UpgradeCode for the two versions should stay the same. This allows the Windows installer technology to find <strong>plugin_v1</strong> on the system and update it to <strong>plugin_v2</strong>, without asking the user to uninstall <strong>plugin_v1</strong> first.</li>\n\
<li>The <code>ProductCode</code> is required by the Autodesk AppStore (though not required by 3ds Max) and should change for each new version of the plugin, i.e. whenever the <code>AppVersion</code> is incremented\n\
  For example, if the first release of a plugin is <strong>plugin_v1.msi</strong>, and the second version is <strong>plugin_v2.msi</strong>, the <code>ProductCode</code> for the two versions should be different. </li>\n\
</ul>\n\
<p>3ds Max 2020 and newer will not load plugin packages that have no <code>UpgradeCode</code>. When two or more plugin packages with the same <code>UpgradeCode</code> are found, the package with the higher <code>AppVersion</code> is loaded.</p>\n\
<blockquote>\n\
<p>Note that the plugin package loading mechanism has been exposed to the SDK; see class <code>PluginPackageManager</code> in <em>maxsdk\\include\\pluginpackagemanager\\pluginpackagemanager.h</em></p>\n\
</blockquote>\n\
<blockquote>\n\
<p>Note that the MAXScript exposure of the package loading mechanism has been changed:</p>\n\
</blockquote>\n\
<p>The &quot;ExchangeStorePackageManager&quot; interface has been renamed to &quot;PluginPackageManager&quot;, although the previous name will be aliased to the new one</p>\n\
<p>The method <code>pathConfig.GetExchangeStorePlugInInstallPath()</code> has been removed. Please use instead <code>GetDir #publicExchangeStoreInstallPath()</code></p>\n\
<h2 id=\"modifier-plugin-class-changes-to-support-pb1-to-pb2-migration\">Modifier Plugin Class Changes to Support Pb1 to Pb2 Migration</h2><p>In order to ease the migration of parameter block 1 (PB1) based object space modifiers (OSM) to parameter block 2 (PB2), the <code>SimpleMod</code> and <code>SimpleMod2</code> classes both derive from a new class called <code>SimpleModBase</code>. Furtheremore, the PB1 and PB2 members of class <code>SimpleMod</code>, and class <code>SimpleMod2</code> respectively were made protected. Class <code>SimpleModBase</code> is basically handling everything class <code>SimpleMod</code> used to, except the PB1, which is handled by class <code>SimpleMod</code>.</p>\n\
<p><strong>Impact on plugins that derive from class SimpleMod</strong>:\n\
The only impact on 3rd party plugins that derive from class <code>SimpleMod</code> is a small non-mandatory change in their Clone method: remove the call that clones their PB1 since <code>SimpleMod::SimpleModClone()</code> now takes care of it.</p>\n\
<p>In the following sample, the code that needs to be removed from current implementations of <code>ReferenceTarget::Clone()</code> overrides is commented out.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">RefTargetHandle MyModifier::Clone(RemapDir&amp; remap)\n\
{\n\
   MyModifier* newmod = new MyModifier();\n\
   // Cloning of the PB1 is taken care of by SimpleMod::SimpleModClone()\n\
   // newmod-&gt;ReplaceReference(SIMPMOD_PBLOCKREF,remap.CloneRef(pblock));\n\
   newmod-&gt;SimpleModClone(this, remap);\n\
   BaseClone(this, newmod, remap);\n\
   return (newmod);\n\
}</code></pre>\n\
<p>There&apos;s no impact on plugins that derive from class <code>SimpleMod2</code>.</p>\n\
<p><strong>Impact on code that accesses <code>SimpleMod</code>&apos;s parameter block</strong>:\n\
Code that needs access to the PB1 held by a <code>SimpleMod</code> instance needs to use the <code>IParamArray* BaseObject::GetParamBlock()</code> accessor method. One can then use the <code>IParamBlock* IParamArray::GetParamBlock()</code> method to get to the PB1.</p>\n\
<p><strong>Impact on code that accesses <code>SimpleMod2</code>&apos;s parameter block</strong>:\n\
Code that needs access to the PB2 held by an <code>Animatable</code> instance needs to use one of the following two methods:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">IParamBlock2* Animatabe::GetParamBlock(int i)\n\
IParamBlock2* Animatable::GetParamBlockByID(BlockID id)</code></pre>\n\
<h2 id=\"modeling\">Modeling</h2><h3 id=\"triangulation-algorithms\">Triangulation Algorithms</h3><p>The algorithms used for triangulating mesh faces have been exposed through new functions: <code>BestConvexDiagonals()</code>, <code>FindDiagonals()</code>, and <code>GetTriangles()</code>.</p>\n\
<p>Furthermore, the old <code>DiagSort()</code> function has been renamed to <code>SortPolygonDiagonals()</code> and moved next to the new functions.</p>\n\
<h3 id=\"api-to-calculate-intersection-of-3d-lines\">API to Calculate Intersection of 3D Lines</h3><p>The following function was added to the SDK to compute the point of intersection, or a least squares approximation to it, for an arbitrary number N of lines in three dimensions. See <em>maxsdk\\include\\gutil.h</em>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">GEOMEXPORT bool ComputeIntersectionPoint(Point3&amp; intersectionPoint, const Tab&lt;Point3&gt;&amp; fixedCoeffs, const Tab&lt;Point3&gt;&amp; directionVecs, bool doNormalize = true);</code></pre>\n\
<h3 id=\"find-node-from-modifier-apis\">Find Node From Modifier APIs</h3><p>Two new functions have been introduced in the <code>Modifier</code> class to help find the node an object space modifier is modifying. Often these modifiers need context information available through the node object, such as transform or material related information. These functions neatly encapsulate the not so trivial logic of walking the modifier stack to find the node a given modifier application modifies. For more in depth information, please see the documentation for the following two methods:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">CoreExport INode* Modifier::GetNodeFromModData(LocalModData* data, int&amp; index);\n\
\n\
CoreExport INode* Modifier::GetNodeFromModContext(ModContext* mc, int&amp; index);</code></pre>\n\
<h2 id=\"rendering\">Rendering</h2><h3 id=\"activeshade-in-the-viewports\">ActiveShade in the Viewports</h3><p>The <code>IActiveShadeFragmentManager</code> class is for those developers who would like to control the ActiveShade in the viewport layer, or the ActiveShade fragment. This is a layer that is mixed into the viewport representation of the scene. The user can edit the scene while ActiveShade is running and see the results updated in real time in the viewport.</p>\n\
<h3 id=\"preview-rendering\">Preview Rendering</h3><p><code>Interface::CreatePreview()</code> now takes two additional optional parameters: a filename string to override the filename specified on the Preview Rendering dialog, and a snippet string that can contain MAXScript code to execute on each frame of the render.  The  return value from the MAXScript code snippet is displayed in the Preview output.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual void CreatePreview(PreviewParams *pvp=NULL, MSTR *filename=NULL, MSTR *snippet=NULL)=0;</code></pre>\n\
<h3 id=\"3ds-max-osl-related-apis\">3ds Max OSL related APIs</h3><p>An API has been added to the 3ds Max 2020 SDK, primarily intended for third-party renderer developers. A 3rd party renderer can integrate with 3ds Max&apos;s OSL in a few ways. Either the renderer uses the classic C++ shader API, and simply calls <code>Texmap::EvalColor()</code>, <code>Texmap::EvalMono()</code>, and <code>Texmap::EvalNormalPerturb()</code> directly on the OSL map. Or, the it may choose to translate scene properties, shaders, and their parameters to their own OSL implementation. The new OSL API helps in both situations, and aims to solve the following problems:</p>\n\
<ul>\n\
<li>Return the set of object properties that the 3ds Max OSL shaders expect to find in the scene, i.e. standard scene attributes like &quot;nodeHandle&quot;, &quot;wireColor&quot;, and the user-defined attributes from the object property dialog. A third party can either enumerate these and attach to their own scene representation, or directly call these functions at render time to look up the values on the fly.</li>\n\
<li>Return information about the shader (the source code, compiled bytecode, etc.) and its parameters and connections, so that a renderer with its own OSL implementation can make sure to execute the right code and assign the right parameter set.</li>\n\
<li>For renderers using the built-in OSL runtime inside 3ds Max, calls to the OSL <code>trace()</code> function can now call the third-party renderer by an interface added to the <code>ShadeContext</code>. This allows the optimized ray tracing routines for the third party renderer to be used from within OSL code, even though 3ds Max is executing the shader.</li>\n\
</ul>\n\
<p>For more information, see the <em>MaxOSLInterface.h</em> header file in the 3ds Max SDK.</p>\n\
<h3 id=\"render-element-visibility-opt-in\">Render Element Visibility Opt-in</h3><p>Before 3ds Max 2020, render elements were visible to the scanline renderer by default.</p>\n\
<p>In 3ds Max 2020, render elements need to explicitly declare themselves visible to the scanline renderer by implementing (overwriting) the following method to return <code>TRUE</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual BOOL MaxRenderElement::IsVisibleToScanlineRenderer() </code></pre>\n\
<h2 id=\"miscellaneous\">Miscellaneous</h2><h3 id=\"-code-paramblock2-code-id-to-index-mapping\"><code>ParamBlock2</code> id to index mapping</h3><p>The performance of looking up a parameter index based on its id has been improved. This should impact positively those parameter blocks whose descriptor contains many parameters, such as the parameter block descriptors powering OSL shaders with many parameters.</p>\n\
<h3 id=\"user-locale-access\">User Locale Access</h3><p>To ensure that the same decimal separator is used everywhere in the 3ds Max user interface, we exposed the following function which allows access to a cached value of the user locale 3ds Max acquired from Windows.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">//! \\brief GetUserLocale retrieves user specific locale specifications\n\
UtilExport const _locale_t&amp; GetUserLocale();</code></pre>\n\
<p>Please note that when exporting numeric values to various data files, it is a best practice to always use &quot;.&quot; as the decimal separator.</p>\n\
<h3 id=\"archiving\">Archiving</h3><p>Archiving a scene file has been exposed to the SDK, through the following API:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual bool Interface18::ArchiveSceneFile(const MCHAR* archiveFileName,unsigned long saveAsVersion = MAX_RELEASE);</code></pre>\n\
<h3 id=\"compression\">Compression</h3><p>Setting and getting the scene file compression on save flag has been exposed to the SDK through the following APIs:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">virtual bool Interface18::GetSceneFileCompressOnSave();\n\
virtual void Interface18::SetSceneFileCompressOnSave(bool compress, bool persist);</code></pre>\n\
<h3 id=\"tablet-support\">Tablet Support</h3><p>The <code>TabletManager</code> class centralizes support for using pressure sensitive devices with 3ds Max.</p>\n\
<h3 id=\"notifications\">Notifications</h3><p>The following new notifications have been added:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">//! \\brief Sent when ActiveShade in the viewport is toggled on/off.\n\
#define NOTIFY_ACTIVESHADE_IN_VIEWPORT_TOGGLED 0x00000107\n\
\n\
//! \\brief Sent when potentially starting to shut 3ds Max down, before checking for things that can cancel the shutdown, such as scene dirty or ExitMAXCallback callback object returning false.\n\
//! This notification is sent before NOTIFY_SYSTEM_SHUTDOWN\n\
#define NOTIFY_SYSTEM_SHUTDOWN_CHECK 0x00000108\n\
\n\
//! \\brief Sent if system shutdown was cancelled. In this case NOTIFY_SYSTEM_SHUTDOWN is not sent.\n\
#define NOTIFY_SYSTEM_SHUTDOWN_CHECK_FAILED 0x00000109\n\
\n\
//! \\brief Sent if system shutdown was not cancelled, and system shutdown is about to start.\n\
//! This notification is sent before NOTIFY_SYSTEM_SHUTDOWN\n\
#define NOTIFY_SYSTEM_SHUTDOWN_CHECK_PASSED 0x0000010A\n\
\n\
//! \\brief Sent immediately before MAXScript loads its startup scripts. stdscripts will have already been read.\n\
#define NOTIFY_PRE_MXS_STARTUP_SCRIPT_LOAD 0x00000106</code></pre>\n\
<h3 id=\"temporary-object-passing-to-non-const-parameters\">Temporary Object Passing to Non-Const Parameters</h3><p>In order to make our APIs comply with newer versions of the compiler, we have started to eliminate default parameters of non-trivial types passed by non-const reference.</p>\n\
<p>One example of such an API is:</p>\n\
<p><code>virtual float CameraObject::GetFOV(TimeValue t, Interval&amp; valid = Interval(0,0))</code></p>\n\
<p>This API has been changed to </p>\n\
<p><code>virtual float CameraObject::GetFOV(TimeValue t, Interval&amp; valid)</code></p>\n\
<p>In order to minimize changes to client code, we also defined the following overload, which will be called any time the first API listed above was called with the default value for its &quot;valid&quot; parameter.</p>\n\
<p><code>float CameraObject::GetFOV(TimeValue t) { Interval valid(0,0); return GetFOV(t, valid); }</code></p>\n\
<p>Note that this change should not require code changes in clients of the affected APIs.</p>\n\
<h3 id=\"new-application-folder-definitions\">New Application Folder Definitions</h3><p>New application folder for user defined tools, such as MCG graphs or Scene Converter add-ins:</p>\n\
<p><code>APP_USER_TOOLS_DIR</code></p>\n\
<p>New application folder for fluid simulations:</p>\n\
<p><code>APP_FLUID_SIMULATION_DIR</code></p>\n\
<p>New application folder where user settings files will be placed:</p>\n\
<p><code>APP_USER_SETTINGS_DIR</code> </p>\n\
<h3 id=\"binary-module-version-access\">Binary Module Version Access</h3><p>Getting the version of a binary module is now possible by calling the following API:</p>\n\
<p><code>UtilExport MSTR MaxSDK::Util::GetVersionOfFile(const MCHAR * fileName);</code></p>\n\
<h3 id=\"3ds-max-build-number-access\">3ds Max Build Number Access</h3><p>The following API returns the 3ds Max build number as a string of the form &quot;#.#.#.#&quot;, where the numbers are the major version, the update version, the hot fix number, and the build number.</p>\n\
<p><code>UtilExport MSTR MaxSDK::Util::GetMaxBuildNumber();</code></p>\n\
<h3 id=\"floating-point-string-conversion\">Floating Point / String Conversion</h3><p>Transforming between floating point values and strings taking into account the current locale is allowed by these APIs:</p>\n\
<p><code>CoreExport float GetFloatFromText(const TCHAR* buf, BOOL *valid);</code></p>\n\
<p><code>CoreExport TSTR GetTextFromFloat(float f, int precision = 3);</code></p>\n\
<h3 id=\"g-buffer-id-change-message\">G-buffer ID Change Message</h3><p>New reference message sent when a node&apos;s user defined properties of G-buffer id changed:</p>\n\
<p><code>REFMSG_NODE_USER_PROPERTY_CHANGED</code></p>\n\
<h2 id=\"deprecated-apis\">Deprecated APIs</h2><p><code>PARTICLE_SYS_CLASS_ID</code> has been deprecated. Please use <code>virtual BOOL Object::IsParticleSystem()</code> to determine if an object is a particle system.</p>\n\
<p><code>APP_EXCHANGE_STORE_PRIVATE_DIR</code> has been deprecated. Please use <code>APP_PLUGIN_PACKAGE_PRIVATE_DIR</code> instead.</p>\n\
<p><code>APP_EXCHANGE_STORE_PUBLIC_DIR</code> has been deprecated. Please use <code>APP_PLUGIN_PACKAGE_PUBLIC_DIR</code> instead.</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">// Replaced by const MCHAR* PluginPackageManager::GetPublicPackageInstallPath() const;\n\
virtual const MCHAR* IPathConfigMgrEx::GetExchangeStorePlugInInstallPath(const MSTR&amp; productCode)</code></pre>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../what_s_new.html\">What&#x27;s New</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";