var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"other\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Other Improvements and Changes</h1></div><ul>\n\
<li><a href=\"#osl-improvements\">OSL improvements</a></li>\n\
<li><a href=\"#tab-supports-random-access-iterators\">Tab supports random access iterators</a></li>\n\
<li><a href=\"#fixed-conversion-of-integer-parameters-with-large-values\">Fixed conversion of integer parameters with large values</a></li>\n\
<li><a href=\"#disabling-asserts-in-hybrid-and-debug-builds\">Disabling asserts in hybrid and debug builds</a></li>\n\
</ul>\n\
<h2 id=\"osl-improvements\">OSL improvements</h2><p>Several things were added for OSL in 3ds Max 2021, mostly around UI refinement.  For more detail about the following improvements, see the <a href=\"./../../3ds_max_sdk_features/rendering/osl.html\">Open Shading Language (OSL) Support</a> topic.</p>\n\
<p>The default layout engine was improved to right-justify parameter headings and have some more user control of the layout, but it now also has the ability to load a <em>.ui</em> file that completely defines the user interface of an OSL shader. </p>\n\
<p>There are also several new options for shader metadata: not only can the shader itself can be versioned with the new <code>version</code> attribute, but parameters&apos; appearance and behavior can be tweaked with the new attributes <code>autopop</code>, <code>connectable</code>, <code>step</code>, <code>packName</code>, <code>widgetWidth</code> and <code>worldunits</code>.</p>\n\
<p>3ds Max now correctly supports the widget type &quot;null&quot;, as well as a few custom UI widgets &quot;max:actionButton&quot; and &quot;max:ramp0&quot;.</p>\n\
<p>Parameter support was extended. The new &quot;semi-types&quot; <code>vector2</code>, <code>vector4</code>, <code>color2</code>, <code>color4</code> and <code>matrix33</code> can now be used. These types are not built-in OSL types, but defined in header files. Including these header files and using these types is now supported (see the section about include files). Also, parameters of type array are supported in certain cases (see the ramp widget).</p>\n\
<p>Support for OSL in the viewport has been vastly improved. The majority of OSL language functions are automatically converted to HLSL code and will display correctly in the viewport in &apos;Realistic&apos; material display mode. You can now effectively develop OSL shaders without ever launching an actual rendering. Not everything is supported (for example, shaders passing string values to each other, or doing operations on string values) but the confidence level is displayed at the bottom of the OSL map as a percentage value. If this reads 100%, the shader will most likely display identically in the viewport and the final render.</p>\n\
<h2 id=\"tab-supports-random-access-iterators\">Tab supports random access iterators</h2><p><code>Tab</code> now returns standard compatible random access iterators (raw pointers) through newly added <code>begin()</code> and <code>end()</code> methods. Here are a few examples of supported algorithms: </p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">Tab&lt;size_t&gt; t; \n\
for (size_t i = 0; i &lt; 100; ++i) { \n\
&#x202F;&#x202F;&#x202F;&#x202F; t.Append(1, &amp;i); \n\
} \n\
\n\
// range for loop \n\
for (size_t val : t) { \n\
&#x202F;&#x202F;&#x202F; // ... \n\
} \n\
\n\
// copy \n\
Tab&lt;size_t&gt; copy_t; \n\
copy_t.SetCount(100); \n\
std::copy(t.begin(), t.end(), copy_t.begin()); \n\
\n\
// find an element \n\
auto it = std::find(t.begin(), t.end(), 42); \n\
if (it != t.end()) { \n\
&#x202F;&#x202F;&#x202F; // ... \n\
} \n\
\n\
// sort \n\
std::sort(t.begin(), t.end(), [](const size_t&amp; lhs, const size_t&amp; rhs) { return lhs &lt; rhs; }); </code></pre>\n\
<h2 id=\"fixed-conversion-of-integer-parameters-with-large-values\">Fixed conversion of integer parameters with large values</h2><p>In order to ensure that integers larger than 2 ^ 24 - 1 are converted properly, use the two new APIs below instead of <code>ParamDimensionBase::Convert(float value)</code> and <code>ParamDimensionBase::Unconvert(float value)</code> :</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">int ParamDimensionBase::Convert()\n\
int ParamDimensionBase::UnConvert(int value); </code></pre>\n\
<h2 id=\"disabling-asserts-in-hybrid-and-debug-builds\">Disabling asserts in hybrid and debug builds</h2><p>Asserts can now be disabled in Hybrid and Debug build configurations of a plug-in by defining the <code>MAX_ASSERTS_NOT_ACTIVE_IN_HYBRID_BUILD</code> and <code>MAX_ASSERTS_NOT_ACTIVE_IN_DEBUG_BUILD</code> pre-processor symbols respectively. One way to do this is by setting them as an environment variable for the compiler: </p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">set CL=/DMAX_ASSERTS_NOT_ACTIVE_IN_HYBRID_BUILD </code></pre>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../what_s_new_3ds_max_2021_sdk.html\">What&#x27;s New: 3ds Max 2021 SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";