var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"security\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Software Security</h1></div><p><a name=\"toc\"></a></p>\n\
<!-- toc -->\n\
\n\
<ul>\n\
<li><a href=\"#safe-scene-script-execution\">Safe Scene Script Execution</a><ul>\n\
<li><a href=\"#new-iscenescriptsecuritymanager-interface\">New <code>ISceneScriptSecurityManager</code> interface</a></li>\n\
<li><a href=\"#embedded-vs-nonembedded-scripts\">Embedded vs NonEmbedded scripts</a></li>\n\
<li><a href=\"#security-exceptions\">Security exceptions</a></li>\n\
<li><a href=\"#3rd-party-plugin-s-participation-in-safe-scene-script-execution\">3rd party plugin&#x2019;s participation in Safe Scene Script Execution</a></li>\n\
</ul>\n\
</li>\n\
</ul>\n\
<!-- tocstop -->\n\
\n\
<h2 id=\"safe-scene-script-execution\">Safe Scene Script Execution</h2><p>3ds Max&#x2019;s Safe Scene Script Execution feature protects users against potentially malicious code embedded in scene files by blocking the execution of unsafe commands. For more information on what commands are considered unsafe, see the topic called &quot;Safe Scene Script Execution&quot; in the 3ds Max 2022 User Guide. </p>\n\
<h3 id=\"new-code-iscenescriptsecuritymanager-code-interface\">New <code>ISceneScriptSecurityManager</code> interface</h3><p>The <code>ISceneScriptSecurityManager</code> interface allows for querying the state of the various options of the Safe Scene Script Execution. Note that plugins are not allowed to programmatically modify the security settings, and therefore APIs that can modify the options are not available. </p>\n\
<h3 id=\"embedded-vs-nonembedded-scripts\">Embedded vs NonEmbedded scripts</h3><p>To support Safe Scene Script Execution, APIs that take as parameters a string or a CharStream representing a script, now take an additional parameter of type <code>MAXScript::ScriptSource</code>, which describes the scripts origin:  </p>\n\
<ul>\n\
<li><code>MAXScript::ScriptSource::Embedded</code>  - script is saved to and loaded from a scene  </li>\n\
<li><code>MAXScript::ScriptSource::NonEmbedded</code>  - script is not embedded in the scene  </li>\n\
<li><code>MAXScript::ScriptSource::NotSpecified</code> &#x2013; scripts origin is not specified; it&#x2019;s treated as an embedded script </li>\n\
</ul>\n\
<p>Callers of script execution APIs should specify a value of <code>MAXScript::ScriptSource::Embedded</code> if the script is embedded in the scene file, or if the script can be specified by a script and that script is running as an embedded script. An example of the former is a scripted particle flow operator where the script is embedded in the scene and is specifiable by the user.  An example of the latter is the registering of a script to be run when a scene event occurs. The code that registers the script needs to persist the ScriptSource of the script currently being run via the following, and pass that to the evaluation method: </p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">MAXScript_TLS* _tls = (MAXScript_TLS*)TlsGetValue(thread_locals_index); \n\
MAXScript::ScriptSource::ScriptSource scriptSource = _tls-&gt;scriptSource; </code></pre>\n\
<p>The following APIs take a <code>MAXScript::ScriptSource</code> parameter </p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">BOOL ExecuteMAXScriptScript(const MCHAR* s, MAXScript::ScriptSource scriptSource, BOOL quietErrors = FALSE, FPValue* fpv = NULL, BOOL logQuietErrors = TRUE); \n\
Value* ExecuteScript(CharStream* source, bool* res, MAXScript::ScriptSource scriptSource); \n\
virtual bool GUP::ExecuteStringScript(const MCHAR *string, MAXScript::ScriptSource scriptSource); \n\
Value* Parser::compile(CharStream* stream, MAXScript::ScriptSource scriptSource); \n\
Value* Parser::compile_factor(CharStream* stream, MAXScript::ScriptSource scriptSource); \n\
Value* Parser::compile_all(CharStream* stream, MAXScript::ScriptSource scriptSource); \n\
virtual MacroEntry* MacroDir::AddMacro(const MCHAR* category, const MCHAR* internalCategory, const MCHAR* name, const MCHAR* tooltip, const MCHAR* buttonText,                 const MCHAR* sourceText, MAXScript::ScriptSource scriptSource) = 0; \n\
virtual void Interface::CreatePreview(PreviewParams* pvp = nullptr, MSTR* filename = nullptr, MSTR* snippet = nullptr, MAXScript::ScriptSource scriptSource = MAXScript::ScriptSource::NotSpecified) = 0; </code></pre>\n\
<h3 id=\"security-exceptions\">Security exceptions</h3><p>When an unsafe MAXScript command is blocked, an instance of class <code>SecurityException</code> is thrown, and it should be handled by the code that called the scripted command, as any other exceptions raised by the execution of the script. </p>\n\
<h3 id=\"3rd-party-plugin-s-participation-in-safe-scene-script-execution\">3rd party plugin&#x2019;s participation in Safe Scene Script Execution</h3><p>3rd party plugins that expose their functionality to MAXScript are expected to ensure that they adhere to 3ds Max&#x2019;s security settings. That is, if the implementation of 3rd party plugin&#x2019;s MAXScript-exposed functionality allows the user to access operating system resources (such as the file system) and services (such as execute arbitrary commands), load into 3ds Max external code, create files with code that later 3ds Max will load and execute, access the internet to upload or download payloads, access email clients, etc, then the command is considered unsafe when called from a scene embedded script, and its implementation needs to follow the guidelines described below. Not following these guidelines can result in 3ds Max users falling victims of software security exploits. </p>\n\
<p>There are two ways to expose functionality to MAXScript: by implementing a MAXScript primitive, or by using function publishing.  </p>\n\
<p><strong>Implementing a MAXScript primitive</strong></p>\n\
<p>If the implementation of a MAXScript primitive matches the description given above of an unsafe command, it should call one of the overrides of the <code>ValidateCanRunMAXScriptSystemCommand()</code> function: </p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">void ValidateCanRunMAXScriptSystemCommand(const MCHAR* command, Value** arg_list, int count); \n\
void ValidateCanRunMAXScriptSystemCommand(const MCHAR* command, const MCHAR* arguments); </code></pre>\n\
<p>If execution of unsafe MAXScript commands embedded in scene files is not allowed, <code>ValidateCanRunMAXScriptSystemCommand()</code> will throw a security exception. The command name and list of arguments is reported by this function as a security message, logged as an error in Scripting Listener and the 3ds Max log file. </p>\n\
<p>If the implementation of a MAXScript primitive calls Python scripts, they must call one of the following two functions: </p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">void ValidateCanRunPythonScripts(const MCHAR* command, const MCHAR* arguments); \n\
void ValidateCanRunPythonScripts(const MCHAR* command, Value** arg_list, int count); </code></pre>\n\
<p>If execution of Python scripts embedded in scene files is not allowed, <code>ValidateCanRunPythonScripts()</code> will throw a security exception. The command name and list of arguments is reported by this function as a security message, logged as an error in Scripting Listener and the 3ds Max log file. </p>\n\
<p><strong>Implementing a function published interface</strong></p>\n\
<p>If a function published method, property or action matches the description given above of unsafe code, it should specify one of the following flags in the publishing code: </p>\n\
<ul>\n\
<li><p><code>FP_EMBEDDEDSCRIPTSECURITYFEATURE</code> &#x2013; specifying this flag for a function/action tells 3ds Max that the function/action is subject to 3ds Max&#x2019;s software security settings. The execution of the function/action will be blocked if execution of unsafe MAXScript commands embedded in scene files is enabled, and a security exception will be raised. When this flag is specified, the next value should be a <code>MaxSDK::ISceneScriptSecurityManager::ScriptingFeature</code> value. </p>\n\
</li>\n\
<li><p><code>FPP_HAS_EMBEDDEDSCRIPTSECURITYFEATURE_R</code> &#x2013; specifying this flag for a function published property tells 3ds Max that reading the property is subject 3ds Max&#x2019;s software security settings. Reading of the property will be blocked if execution of unsafe MAXScript commands embedded in scene files is enabled, and a security exception will be raised. When this flag is specified, next value is a <code>MaxSDK::ISceneScriptSecurityManager::ScriptingFeature</code> value.  </p>\n\
</li>\n\
<li><p><code>FPP_HAS_EMBEDDEDSCRIPTSECURITYFEATURE_W</code> - specifying this flag for a function published property tells 3ds Max that writing the property is subject 3ds Max&#x2019;s software security settings. Writing of the property will be blocked if execution of unsafe MAXScript commands embedded in scene files is enabled, and a security exception will be raised. When this flag is specified, next value is a <code>MaxSDK::ISceneScriptSecurityManager::ScriptingFeature</code> value.  </p>\n\
</li>\n\
<li><p><code>FPP_HAS_EMBEDDEDSCRIPTSECURITYFEATURE_RW</code> - specifying this flag for a function published property tells 3ds Max that reading and writing the property is subject 3ds Max&#x2019;s software security settings. Both reading and writing of the property will be blocked if execution of unsafe MAXScript commands embedded in scene files is enabled, and a security exception will be raised. When this flag is specified, next value is a <code>MaxSDK::ISceneScriptSecurityManager::ScriptingFeature</code> value.</p>\n\
</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../whats_new_3dsmax_2022_sdk.html\">What&#x27;s New: 3ds Max 2022 SDK</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";