var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"deferred_loading_of_plug-ins\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Deferred Loading of Plug-ins</h1></div><h2 id=\"plug-in-dll-defer-loading\">Plug-in DLL Defer Loading</h2><p>A defer loaded plug-in DLL will not be loaded into memory until required by 3ds Max for the first time. Deferring a plug-in is a useful technique for improving the start-up time and reducing the memory footprint of 3ds Max. This is why 3ds Max uses automatic defer loading system. If a plug-in DLL is deferred, it should not impact the user experience. For example:</p>\n\
<ul>\n\
<li>The set of user interface elements (menus, hotkeys, buttons, list of available modifiers, list of file types available to import/export, etc), the set of MAXScript commands, etc, should be the same as if the plug-in would be loaded.</li>\n\
<li>Support for loading scenes, material libraries or other file based assets should be the same as if the plug-in would be loaded.</li>\n\
</ul>\n\
<p>Certain plug-in types that are cached (proxied out) by 3ds Max need all their data to be constant to prevent run time issues when defer loaded. 3ds Max will still defer load the plug-in if those values are not constant, but the user may experience unexpected behavior in certain 3ds Max features. One example is the input type of a modifier plugin. If the type of an input object used by a modifier changes at run-time, then the user may not be able to apply the deferred modifier on certain object types. For the correct behavior to re-establish, the modifier plug-in needs to be loaded.</p>\n\
<p>If it is not possible for a plug-in to be defer loaded without impacting the user experience, then it is the responsibility of a plug-in developer to opt out of the automatic defer loading system. This can be done by returning <code>FALSE</code> from the DLL exported function <code>CanAutoDefer()</code> as following:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// The plug-in opts out from 3ds Max\\&apos;s defer loading mechanism, \n\
// i.e. it\\&apos;s always loaded when 3ds Max starts up.\n\
__declspec( dllexport ) ULONG CanAutoDefer()\n\
{\n\
   return FALSE;\n\
}</code></pre><p>You will also need to add this line to the  <em>\\</em>.DEF*  file. Example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\"> CanAutoDefer @5</code></pre><p>Loading of a plugin however will not be defered, even if CanAutoDefer() function returns true, if the plug-in:</p>\n\
<ul>\n\
<li>exposes structures, functions, properties, etc to Maxscript using pre-Function Publishing technology, such as the <code>def_struct_primitive</code>, <code>def_visible_primitive</code>, <code>def_property</code>, etc macros. See the documentation of the file  <em>define_abstract_functions.h</em> </li>\n\
<li>register with 3ds Max app-data readers - see <code>ObjectDataReaderCallback</code></li>\n\
</ul>\n\
<p>Although 3ds Max implements some heuristics to detect if the plug-in can be safely deferred, plug-in developers are ultimately responsible for detecting violations of this requirement and explicitly disallow 3ds Max from defer loading the plug-in.</p>\n\
<h2 id=\"the-3dsmax-ini-flags\">The 3dsmax.ini Flags</h2><p>Developers can turn on or off specific plug-in defer loading features using certain flags defined in the file 3dsmax.ini. These features are:</p>\n\
<ul>\n\
<li>The ability to toggle between the legacy opt-in mechanism and the new opt-out mechanism.</li>\n\
<li>The ability to defer load plug-ins that expose action tables (action items - an action table contains action items, action items always live in an action table).</li>\n\
<li>The ability to defer load plug-in that expose core interfaces. See <a href=\"./../../3ds_max_sdk_features/function_publishing/core_interfaces.html\">Core Interfaces</a>.</li>\n\
<li>The ability to defer load plug-ins that listen to startup the <code>NOTIFY_SYSTEM_STARTUP</code> notification. See the <code>RegisterNotification()</code> global function.</li>\n\
</ul>\n\
<p>These flags are introduced in the example below.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">[PluginSettings]\n\
; Controls whether plug-ins DLL that do not implement ULONG CanAutoDefer are defer loaded (1) or not (0)\n\
CanAutoDeferDefaultValue=1 \n\
; Controls whether plug-in DLLs that expose Action Table (action items) can be defer loaded (1) or not (0)\n\
EnableActionItemProxySystem=1\n\
; Controls whether plug-in DLLs that function publish &quot;core&quot; interfaces can be defer loaded (1) or not (0)\n\
EnableCoreInterfaceProxySystem=1\n\
; Controls whether plug-in DLLs that rely on startup notifications can be defer loaded (1) or not (0)\n\
EnableStartupNotificationReplaySystem=1</code></pre><p>The Plug-In Manager feature of 3ds Max shows the user whether a plug-in DLL has been deferred or loaded.</p>\n\
<h2 id=\"common-problems-and-solutions\">Common Problems and Solutions</h2><h2 id=\"plug-in-registers-callbacks-and-objects-with-3ds-max\">Plug-in Registers Callbacks and Objects with 3ds Max</h2><p>Besides action tables and core interfaces, a plug-in may need to register with 3ds Max other callbacks and objects that affect the user interface such as, but not limited to:</p>\n\
<ul>\n\
<li>Action item override names: See<code>IActionItemOverrideManager::ActivateActionItemOverride()</code>.</li>\n\
<li>Geometry Checkers: See <code>IGeometryCheckerManager::RegisterGeometryChecker()</code>.</li>\n\
<li>Drag and drop handlers: See class <code>DragAndDropHandler</code>.</li>\n\
<li>Menus other than those based on action items: See <code>IMenuManager::RegisterMenu()</code>.</li>\n\
<li>Viewport buttons: See <code>IViewportButtonManager::RegisterButton()</code>.</li>\n\
<li>Trackbar filters: See <code>ITrackBarFilterManager:: RegisterFilter()</code>.</li>\n\
<li>Object converters and &quot;Collapse&quot; types: See <code>RegisterObjectConverter()</code>, <code>RegisterStaticEditTri()</code> , <code>RegisterEditTriObjDesc()</code> and <code>RegisterEditPolyObjDesc()</code>.</li>\n\
</ul>\n\
<p>At this time 3ds Max&apos;s plug-in loading mechanism does not detect these cases and as a result, it will defer load such a plug-in DLL unless it explicitly disallows deferring.</p>\n\
<h2 id=\"load-order-dependencies-between-plug-ins\">Load Order Dependencies between Plug-ins</h2><p>The deferred plug-in DLLs are loaded when a certain user action requires them to be present. Because the sequence of user actions is not always predictable, the order in which deferred plug-in DLLs are loaded by 3ds Max will not be deterministic. In general, Plug-in DLL A has a load order dependency on plug-in DLL B if B needs to be loaded in 3ds Max&apos;s address space before A. If plug-in DLL A calls a function exported from plug-in DLL B, a load order dependency will exist between the two plug-in DLLs.</p>\n\
<p>If the plug-in DLL B is defer loaded:</p>\n\
<ul>\n\
<li>Plug-in DLL A may fail to be loaded by the operating system&apos;s loader. In thiscase error 126 (The specified module could not be found) is displayed by the operating system. This is the case if plug-in DLL A links against plug-in DLL B.</li>\n\
<li>A run-time error may occur if plug-in DLL A tries to acquire the address of a function exported by plug-in DLL B (with a call to WINAPI <code>GetProcAddress()</code>) and call that function.</li>\n\
</ul>\n\
<p>The solution to this problem is to eliminate the load order dependency between the two plug-in DLLs by:</p>\n\
<ul>\n\
<li>Moving the functionality that the plug-in DLL A (and possibly plug-in DLL B) relies on into a new DLL C that is not a plug-in and that any plug-in DLLs can either link against or load at runtime.<ul>\n\
<li>The DLL containing the common functionality should not be <a href=\"http://msdn.microsoft.com/en-us/library/151kt790.aspx\" target=\"_blank\">delay loaded</a> if it exposes core interfaces to MAXScript. The delay loading mechanism may result in the core interface not being properly registered and 3ds Max may crash when client code requests such an interface.</li>\n\
</ul>\n\
</li>\n\
<li>Exposing the functionality plug-in DLL A relies on as a core interface (an object that derives from <code>FPStaticInterface</code> and is created with the<code>FP_CORE</code> flag). Plug-in DLLs that expose core interfaces are guaranteed to be loaded on demand, when client code acquires the core interface they expose.</li>\n\
</ul>\n\
<h2 id=\"user-defined-plug-in-architectures\">User Defined Plug-in Architectures</h2><p>One example of a plug-in architecture that does not work well with plug-in DLL defer loading is when a 3ds Max plug-in loads a DLL at run-time that extend the plug-in&apos;s functionality and the plug-in exposes this functionality in the 3ds Max user interface.</p>\n\
<p>When the plug-in DLL is deferred, extensions in the form of DLLs installed since the last session of 3ds Max may not show up in the user interface, until some user action triggers the loading of the plug-in DLL that depends on these extensions.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../advanced_plug-in_topics.html\">Advanced Plug-in Topics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";