var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"facilitating_translation_and_loc\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Facilitating Translation and Localization</h1></div><p>3ds Max is sold in numerous countries outside Canada and the United States. To allow your plug-in to have the largest possible audience, it is a good practice to ensure that text strings used in the plug-in can be easily translated to another language, or easily adapted to differences in usage between countries that speak the same language.</p>\n\
<p>To accomplish this, you should put all your language-specific strings into a separate resource-only DLL. In this way, only the resource DLL, and not the rest of the code, needs to be changed to move the application to another language.</p>\n\
<p>The approach first involves creating a VC++ project that builds a separate resource DLL containing only the resources of the plug-in.</p>\n\
<p>Then you must arrange for the resource DLL to be loaded. Among the various steps taken by 3ds Max to initialize a plug-in, it calls your <code>LibDescription()</code>, <code>LibInitialize()</code> and <code>LibClassDesc()</code> methods each of which should load your resource DLL if it is not already loaded. When 3ds Max is finished with your plug-in, it calls your <code>LibShutdown()</code> method which frees the resource DLL.</p>\n\
<p>The <code>DllMain()</code> function is also called but it does very little processing. See <a href=\"./../creating_a_plug-in_project/manually_creating_a_new_plug-in_/required_dll_functions.html\">Required DLL Functions</a> for more information. The EPS image processing plug-in source code (in <code>maxsdk</code>\\samples\\images\\eps) provides examples of how to separate resources that should be localized from processing.</p>\n\
<p>The final step is to acquire the resource strings from the resource DLL as they are needed. The following code sample (taken from the EPS plug-in) shows how to load and use one of the strings from the resource DLL:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">const TCHAR *EPSClassDesc::ClassName ()\n\
{\n\
   static int loaded = 0;\n\
   static TCHAR stringBuf[MAX_STRING_LENGTH;\n\
   if (! loaded)\n\
   {\n\
     LoadString (hResource, IDS_CLASS_NAME, stringBuf, MAX_STRING_LENGTH);\n\
     loaded = 1;\n\
   }\n\
   return stringBuf;\n\
}</code></pre><p>In summary, it is a good design practice to separate the literal strings of a plug-in into a separate resource-only DLL. This creates a globalized application with the potential to reach the largest possible audience.</p>\n\
<h2 id=\"localization-codes\">Localization Codes</h2><p>The following is the table of localization codes and three letter acronyms for the different localizations of 3ds Max 2011.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th></th>\n\
<th></th>\n\
<th></th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td>CHS</td>\n\
<td>Chinese - PRC</td>\n\
<td>0804</td>\n\
</tr>\n\
<tr>\n\
<td>DEU</td>\n\
<td>German - Germany</td>\n\
<td>0407</td>\n\
</tr>\n\
<tr>\n\
<td>ENU</td>\n\
<td>United States English</td>\n\
<td>0409</td>\n\
</tr>\n\
<tr>\n\
<td>FRA</td>\n\
<td>French - France</td>\n\
<td>040C</td>\n\
</tr>\n\
<tr>\n\
<td>JPN</td>\n\
<td>Japanese - Japan</td>\n\
<td>0411</td>\n\
</tr>\n\
<tr>\n\
<td>KOR</td>\n\
<td>Korean (Extended Wansung) - Korea</td>\n\
<td>0412</td>\n\
</tr>\n\
</tbody></table>\n\
<p>To access the locale information for the current installation of 3ds Max you can use the functions <code>MaxSDK::Util::GetLanguageID()</code> and <code>MaxSDK::Util::GetLanguageTla()</code>.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../advanced_plug-in_topics.html\">Advanced Plug-in Topics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";