var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"setwindowlong_and_getwindowlong\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>SetWindowLong() and GetWindowLong()</h1></div><p>It is typical of windows UI code to store a pointer to a C++ object into the <code>GWL_USERDATA</code> slot of an <code>HWND</code>, using code like this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">long oldValue = SetWindowLong(hWnd, GWL_USERDATA, (long) this);</code></pre><p>And, to get it back to the <code>winproc</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MyClass* me = (MyClass*) GetWindowLong(hWnd, GWL_USERDATA);</code></pre><p>This is not compatible with <em>x64</em>: <code>GWL_USERDATA</code> is not even defined when building for <em>x64</em>. You are forced to use the <code>GWLP_USERDATA</code> define, which should be used with the <code>LONG_PTR</code>-friendly versions as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">LONG_PTRoldValue = SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR) this);\n\
MyClass* me = (MyClass*) GetWindowLongPtr(hWnd, GWLP_USERDATA);</code></pre><p>This is better, but, because of issues in the header files, this can still generate portability warnings in <em>Win32</em> when building with the /Wp64 switch which is enabled for 3ds Max projects now.</p>\n\
<p>To obtain a clean, warning-free compile of the above code, you should write it as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#if !defined( _WIN64 )\n\
// SetWindowLongPtr() maps to SetWindowLong() in 32 bit land; react accordingly to\n\
// keep the compiler happy, even with /Wp64.\n\
LONG_PTRoldValue = static_cast&lt;LONG_PTR&gt;(::SetWindowLongPtr(hWnd, n, (LONG)((LONG_PTR)(ptr))));\n\
#else\n\
LONG_PTRoldValue = static_cast&lt;LONG_PTR&gt;(::SetWindowLongPtr(hWnd, n, (LONG_PTR)(ptr)));\n\
#endif\n\
\n\
MyClass* me = reinterpret_cast&lt;MyClass*&gt;(static_cast&lt;LONG_PTR&gt;(::GetWindowLongPtr(hWnd, n)));</code></pre><p>While this works, it is complicated. To simplify there are a number of functions in <code>maxsdk</code>\\include\\3dsmaxport.h which aid in portability. Using these functions we can rewrite this as:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MyClass* oldValue = DLSetWindowLongPtr(hWnd, this); \n\
MyClass* me = DLGetWindowLongPtr&lt;MyClass*&gt;(hWnd); </code></pre><p>These default to <code>GWLP_USERDATA</code>, since this is the main reason these are used, but you can also specify it explicitly as:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">DLSetWindowLongPtr(hWnd, this, GWLP_USERDATA);\n\
MyClass* me = DLGetWindowLongPtr&lt;MyClass*&gt;(hWnd, GWLP_USERDATA); \n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../upgrading_to_64-bit.html\">Upgrading to 64-Bit</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";