var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"required_dll_functions\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Required DLL Functions</h1></div><p>This topic describes the functions expected by 3ds Max for most plug-in DLL.</p>\n\
<ul>\n\
<li><p><code>DllMain()</code> - This function is the DLL&apos;s entry point function called by Windows when the DLL is loaded. It may be called many times during time critical operations like rendering. <code>DllMain()</code> should do nothing except store the DLL&apos;s <code>HINSTANCE</code>, and call <code>DisableThreadLibraryCalls()</code> if nothing special needs to be done during <code>DLL_THREAD_ATTACH</code> and <code>DLL_THREAD_DETACH</code>. Note that you should never throw an exception in <code>DllMain()</code>. You should also never call any managed code in this method either. Also you should never call code that loads another DLL either. Doing anything more complicated in this function is risky and may crash the application. Please defer any plugin startup or shutdown code to the <code>LibInitialize</code> and <code>LibShutdown</code> code described below.</p>\n\
<p>Once called for the first time, two main actions are usually performed:</p>\n\
<ol>\n\
<li><ol>\n\
<li>A handle to the DLL module in the OS memory space is assigned to a global variable in the plug-in. This handle is provided by OS as an argument in its call of <code>DLLMain()</code>, and will be used later by other functions in the plug-in.</li>\n\
</ol>\n\
</li>\n\
<li>Further notifications from OS get disabled for this DLL. This means that if other threads are created in the same OS process and decide to load the same DLL, the <code>DLLMain()</code> will not be called anymore</li>\n\
</ol>\n\
</li>\n\
<li><p><code>LibNumberClasses()</code> - This function returns the number of plug-in classes this DLL uses.</p>\n\
</li>\n\
<li><p><code>LibClassDesc()</code> - This function returns a class descriptor (<code>ClassDesc2</code>) for the nth class exported by the DLL. See the topic on <a href=\"./../../plug-in_basics/class_descriptors.html\">Class Descriptors</a>.</p>\n\
</li>\n\
<li><p><code>LibDescription()</code> - This function returns a string that describes the DLL. This is used by 3ds Max to identify entities referenced in a scene when the implementing DLL is not available. For example if a particular material is used in a scene which is shared with another 3ds Max user who does not have the appropriate DLL on their system. The string should therefore mention where the user could obtain the DLL.</p>\n\
</li>\n\
<li><p><code>LibVersion()</code> - This function returns a pre-defined constant indicating the version of the system under which it was compiled. It is used to allow 3ds Max identify obsolete DLLs. This function is called once, right after your plug-in has been loaded by 3ds Max. Normally you would just return <code>Get3DSMAXVersion()</code>.</p>\n\
</li>\n\
</ul>\n\
<!-- -->\n\
\n\
<ul>\n\
<li><code>LibInitialize()</code> - Perform one-time plug-in initialization in this method. Return TRUE if your plug-in has initialization was successful (e.g. the plug-in has loaded correctly), or FALSE otherwise. If the function returns FALSE, the system will not load the plug-in, it will then call <code>FreeLibrary()</code> on your DLL, and send you a message.</li>\n\
<li><code>LibShutdown()</code> - Called once, just before the plug-in is unloaded. Perform one-time plug-in uninitialization in this method. The system ignores the return value.</li>\n\
</ul>\n\
<p>Example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">HINSTANCE hInstance = 0;\n\
\n\
BOOL WINAPI DllMain(HINSTANCE hinstDLL,ULONG fdwReason,LPVOID /*lpvReserved*/)\n\
{\n\
  if( fdwReason == DLL_PROCESS_ATTACH )\n\
  {\n\
     // Hang on to this DLL&apos;s instance handle.\n\
     hInstance = hinstDLL;\n\
     DisableThreadLibraryCalls(hInstance);\n\
  }\n\
  return(TRUE);\n\
}\n\
\n\
__declspec( dllexport ) const TCHAR* LibDescription()\n\
{\n\
   // Retrieve astring from the resource string table\n\
   static TCHAR buf[256];\n\
   if (hInstance)\n\
      return LoadString(hInstance, IDS_LIBDESCRIPTION, buf, sizeof(buf)) ? buf : NULL;\n\
   return NULL;   \n\
}\n\
\n\
__declspec( dllexport ) int LibNumberClasses()\n\
{\n\
  return 1;\n\
}\n\
\n\
__declspec( dllexport ) ClassDesc* LibClassDesc(int i)\n\
{\n\
  switch(i)\n\
  {\n\
    case 0:\n\
       return GetSimpleWidgetDesc();\n\
  }\n\
\n\
  return 0;\n\
}\n\
\n\
__declspec( dllexport ) ULONG LibVersion()\n\
{\n\
  return Get3DSMAXVersion();\n\
}\n\
\n\
__declspec( dllexport ) int LibInitialize()\n\
{\n\
  // TODO: Perform initialization here.\n\
  return TRUE;\n\
}\n\
\n\
__declspec( dllexport ) int LibShutdown()\n\
{\n\
  // TODO: Perform uninitialization here.\n\
  return TRUE;\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../manually_creating_a_new_plug-in_.html\">Manually Creating a New Plug-in Project</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";