var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"class_descriptors\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Class Descriptors</h1></div><p>Class descriptors provide 3ds Max with information about each plug-ins that is implemented in a DLL, and allows 3ds Max to instantiate them. A plug-in will implement its own class descriptor by deriving from <code>ClassDesc2</code> which in turn derives from <code>ClassDesc</code>.  </p>\n\
<p>A <code>ClassDesc2</code> class descriptor contains a a table of <code>ParamBlockDesc2</code>s for all the parameter blocks used in the plug-in, and a number of methods for accessing block descriptors, UI management, auto param block2 construction, and access to any <code>ParamMap2</code>s.</p>\n\
<p>3ds Max queries the DLL for the number of class descriptors contained in the plug-in using <code>LibNumberClasses()</code> and will retrieve them using <code>LibClassDesc()</code>. For more information see <a href=\"./../creating_a_plug-in_project/manually_creating_a_new_plug-in_/required_dll_functions.html\">Required DLL Functions</a>.</p>\n\
<p>Here is an example of a plug-ins class descriptor:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class WidgetClassDesc : public ClassDesc2 {\n\
public:\n\
   int          IsPublic()     { return TRUE; }\n\
   void*        Create(BOOL loading = FALSE) { return new Widget(); }\n\
   const TCHAR* ClassName()    { return GetResourceString(IDS_CLASS_NAME); }\n\
   SClass_ID    SuperClassID() { return GEOMOBJECT_CLASS_ID; }\n\
   Class_ID     ClassID()      { return Widget_CLASS_ID; }\n\
   const TCHAR* Category()     { return GetResourceString(IDS_CATEGORY); }\n\
\n\
   // returns fixed parsable name (scripter-visible name)\n\
   const TCHAR* InternalName() { return _T(&quot;Widget&quot;); }   \n\
\n\
   // returns owning module handle\n\
   HINSTANCE    HInstance()    { return hInstance; }     \n\
};</code></pre><h2 id=\"ispublic-\">IsPublic()</h2><p>The <code>ClassDesc::IsPublic</code>() member function returns a boolean value to indicate whether the plug-in should be exposed to the user through the 3ds Max user interface. If the plug-in can be selected by the user, as is usually the case, return TRUE otherwise return FALSE. The following code example demonstrates how a plug-in that is only intended for 3ds Max Design might prevent itself from being display in non-Design versions of 3ds Max.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">int IsPublic() {\n\
  return GetProductID() == k3dsMaxDesign ? 1 : 0;\n\
}</code></pre><h2 id=\"create-\">Create()</h2><p>3ds Max calls the <code>ClassDesc::Create</code>() member function when it needs a pointer to a new instance of the plug-in class. For example, if 3ds Max is loading a file from disk containing a previously used plug-in (procedural object, modifier, controller, etc.), it will call the plug-in&apos;s <code>ClassDesc::Create()</code> member function. The plug-in responds by allocating a new instance of its plug-in class. Typically this would be done using the new operator.</p>\n\
<p>The optional parameter passed to <code>ClassDesc::Create</code>() is a flag indicating if the class being created is going to be loaded from a disk file. If the flag is TRUE, the plug-in may not have to perform any initialization of the object because the loading process will take care of it. See the section on <a href=\"loading_and_saving_plug-in_data.html\"\">Loading and Saving Plug-in Data</a> for more information.</p>\n\
<p>Some plug-ins allow the system to manage the creation process so their implementation of this member function simply returns NULL. This indicates to the system that it should invoke the default object creation process.</p>\n\
<h2 id=\"classname-\">ClassName()</h2><p>The <code>ClassDesc::ClassName()</code> member function returns the potentially localized name of the class. This name appears in the button for the plug-in in the 3ds Max user interface. This name is also used by MAXScript as a name for the class. However, since the name returned by this method may be a localized name, scripts written in one locale may not work correctly in other locales.</p>\n\
<p>In general, your plug-in should implement <code>ClassDesc::UseOnlyInternalNameForMAXScriptExposure()</code> and return true, in which case only the non-localized name returned by <code>InternalName()</code> is used by MAXScript as the name of the class.  If <code>UseOnlyInternalNameForMAXScriptExposure()</code> is not implemented, or returns false, both the ClassName and InternalName are used as names for the class.</p>\n\
<h2 id=\"superclassid-\">SuperClassID()</h2><p>The <code>ClassDesc::SuperClassID</code>() member function returns a system-defined constant describing the class that this plug-in class was derived from. For example, the Bend modifier returns <code>OSM_CLASS_ID</code>. This super class ID is used by all object space modifiers. Some other example super class IDs are: <code>CAMERA_CLASS_ID</code>, <code>LIGHT_CLASS_ID</code>, <code>SHAPE_CLASS_ID</code>, <code>HELPER_CLASS_ID</code>, and <code>SYSTEM_CLASS_ID</code>. Consult the C++ Reference&apos;s &quot;<code>Related Pages &amp;gt; Lists and Functions &amp;gt; List of Super Class IDs</code>&quot; for the entire list of available super class IDs.</p>\n\
<h2 id=\"classid-\">ClassID()</h2><p>The <code>ClassDesc::ClassID</code>() member function must return the <em>unique</em> ID for the plug-in object. A program is provided with the SDK called <a href=\"generating_class_ids.html\"\">Generating Class IDs</a> to generate these IDs, to avoid conflicts with existing plug-ins. It is very important you use this program to create the class ids for your plug-ins.</p>\n\
<div class=\"note note:\"><strong>Note</strong>: If you use one of the source code examples to create your plug-in, you MUST change the existing <code>Class_ID</code>. If you don&apos;t, you&apos;ll get a conflict. If two class IDs conflict, the system will only load the first one it finds (and will post a message when it attempts to load the second one noting that there is a Class_ID conflict).</div>\n\
<p>A Class_ID consists of two unsigned 32-bit quantities. The constructor assigns a value to each of these, for example <code>Class_ID</code>( 0xA1C864D1, 0xE7AA2BE5). See the<code>Class_ID</code> documentation for reference information.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> sample code plug-ins used in 3ds Max use 0 as the second 32-bit quantity of the Class_ID. <em>Only the built-in classes (those that ship with 3ds Max) should have the second 32 bits equal to 0</em> . All plug-in developers should use <em>both</em> 32 bit quantities.</div>\n\
<h2 id=\"category-\">Category()</h2><p>The <code>ClassDesc::Category</code>() category is selected in the bottom-most drop down list in the create branch of the command panel. If this is set to be an existing category (i.e. &quot;Standard Primitives&quot;, &quot;Particle Systems&quot;, etc.) then the plug-in will appear in that category. Developers should not add to the categories provided by 3ds Max (see the note below). If the category doesn&apos;t yet exist then it is created. If the plug-in does not need to appear in the list, it may simply return an empty string as in <code>\\_T</code>(&quot;&quot;). <code>ClassDesc::Category()</code> is also used for modifiers to classify them in the button sets dialog.</p>\n\
<h2 id=\"internalname-\">InternalName()</h2><p>The <code>ClassDesc::InternalName</code>() returns a fixed, parsable internal name for the plug-in.\n\
By default, Plug-in names exposed to MAXScript are generated using both the strings from <code>ClassName()</code> and <code>InternalName()</code>.  You can change this behavior by returning true from <code>ClassDesc::UseOnlyInternalNameForMAXScriptExposure()</code>, in which case the plug-in exposes only its <code>InternalName()</code>.   We recommend that new plugins implement this method and return true.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../plug-in_basics.html\">Plug-in Basics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";