var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"creating_and_destroying_plug-in_\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Creating and Destroying Plug-in Instances</h1></div><p>The 3ds Max API provides a method used to allocate memory when a new instance of a plug-in class is required. It also provides several different ways to deallocate this memory depending on the plug-in type. In the most common case, these methods are <code>ClassDesc::Create()</code> and <code>Animatable::DeleteThis()</code>.</p>\n\
<div class=\"note note:\"><em>Note</em>: While the <code>Create()</code> method is a member of <code>ClassDesc</code>, developers should use <code>ClassDesc2</code> which inherits from <code>ClassDesc</code>.</div>\n\
<p>When 3ds Max needs to create a new instance of a plug-in class, it calls the <code>Create()</code> method of the class descriptor. This method returns a new instance of the plug-in class. The memory for the class may be allocated by using the <code>new</code> operator as shown below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void* Create(BOOL loading = FALSE) { return new MyObject; }</code></pre><p>For example, when 3ds Max loads a file from disk it needs to create instances of the plug-ins that are used (the geometric objects, lights, cameras, materials, controllers, etc.). It does this by calling <code>ClassDesc::Create()</code>.</p>\n\
<p>The memory must be freed when 3ds Max is done with the item. Usually a method named <code>DeleteThis()</code> is called. This method is called to free the memory associated with the plug-in class. Since the memory was allocated with the <code>new</code> operator, it must be de-allocated using the <code>delete</code> operator as shown below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void Plugin::DeleteThis() { deletethis; }</code></pre><div class=\"note note:\"><span class=\"notetitle\">Note:</span> Plug-in&apos;s should not put their <code>DeleteThis</code> methods inline. First the <code>DeleteThis</code> method is virtual, and so conflicts with the whole idea of inline methods. Second, when put inline, this method can cause incorrect destruction of objects and memory leaks.</div>\n\
<p>For plug-ins that are part of the <code>Animatable</code> class hierarchy, <code>DeleteThis()</code> is a method of <code>Animatable</code>. For plug-ins that are <em>not</em> derived from <code>Animatable</code> there may be a non-inherited <code>DeleteThis()</code> method. For example, the <code>UtilityObj</code> class used to create Utility plug-ins is not derived from <code>Animatable</code> and has its own <code>DeleteThis()</code>.</p>\n\
<p>The <code>DeleteThis()</code> method also gives the developer control over deleting. For example, a Utility plug-in object may be statically declared and not actually declared in the heap. A utility plug-in such as <em>\\MAXSDK\\SAMPLES\\RETIRED\\ASCIIOUT.CPP</em>  would implement <code>DeleteThis()</code> to do nothing since there is no heap memory to free. For example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void DeleteThis() {}</code></pre><p>A few plug-in types have no <code>DeleteThis()</code> method at all -- 3ds Max deletes the memory directly. The Image Filter and Compositor plug-in types are derived from <code>ImageFilter</code>. They are allocated by system using <code>ClassDesc::Create()</code> but have no <code>DeleteThis()</code> method to override. The system just calls the <code>operator</code>delete() operator on the object instance.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../plug-in_basics.html\">Plug-in Basics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";