var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"creating_parameter_blocks\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Creating Parameter Blocks</h1></div><p>Parameter blocks are covered in detail in the <a href=\"./../../3ds_max_sdk_features/plug-ins/parameter_blocks.html\">Parameter Blocks</a> section of the documentation. This section walks through the parameter block construction used by the widget how-to sample. See the file  <em>howto/objects/widget/widget.cpp</em> .</p>\n\
<h2 id=\"defining-parameter-block-and-parameter-identifiers\">Defining Parameter Block and Parameter Identifiers</h2><p>The first step in creating a parameter block is to define identifiers for the parameter blocks and individual parameters. This is usually done by creating enumerations.</p>\n\
<p>The enumerated value that identifies the different parameter block is:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">enum { widget_params };</code></pre><p>The next set of enumerated values identify the individual parameters:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">enum {\n\
  widget_size,\n\
  widget_left,\n\
  widget_right\n\
};</code></pre><h2 id=\"describing-the-parameter-block\">Describing the Parameter Block</h2><p>The next step in creating a parameter block is to create a parameter block descriptor. This is done using a <code>ParamBlockDesc2</code>. The Widget samples declares a static <code>ParamBlockDesc2</code> object named <code>widget_param_blk</code>.</p>\n\
<p>The following is the first component of the parameter block:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">\n\
1    static  ParamBlockDesc2 widget_param_blk ( widget_params, _T(&quot;params&quot;), 0, &amp;WidgetDesc,\n\
2        P_AUTO_CONSTRUCT + P_AUTO_UI, PBLOCK_REF,\n\
3         //rollout\n\
4         IDD_PANEL, IDS_PARAMS, 0, 0, NULL,\n\
5         // params\n\
6         widget_size,         _T(&quot;size&quot;),         TYPE_FLOAT,     P_ANIMATABLE,     IDS_SPIN,\n\
7            p_default,       0.1f,\n\
8            p_range,         0.0f,1000.0f,\n\
9            p_ui,            TYPE_SPINNER,       EDITTYPE_FLOAT, IDC_EDIT,    IDC_SPIN, 0.50f,\n\
10           p_end,</code></pre><p><em>Line 1</em></p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static  ParamBlockDesc2 widget_param_blk ( widget_params, _T(&quot;params&quot;), 0, &amp;WidgetDesc,</code></pre><p>The first four arguments are required. They describe the plug-in to 3ds Max, and register a rollout control to display your parameters.</p>\n\
<p>widget_params<br>This the parameter block ID.</p>\n\
<p>_T(&quot;params&quot;)<br>This is a string containing a name that the parameter block uses internally.</p>\n\
<p> 0<br>This the resource ID of the localized string in the resource&apos;s string table.</p>\n\
<p> &amp;WidgetDesc<br>The is the address of a <code>ClassDesc2</code> object that describes the plug-in.</p>\n\
<p><em>Line 2</em></p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">P_AUTO_CONSTRUCT + P_AUTO_UI, PBLOCK_REF,</code></pre><p>The optional arguments on the second line are telling 3ds Max to automatically handle the user interface.</p>\n\
<p><em>Lines 3-4</em></p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">//rollout\n\
IDD_PANEL, IDS_PARAMS, 0, 0, NULL,</code></pre><p>The rollout comment points to more optional arguments.</p>\n\
<ul>\n\
<li>The first argument, IDD_PANEL tells 3ds Max the resource ID of the dialog that will be displayed for the rollout. This parameter will be further discussed further below in the section dealing with setting up the user interface.</li>\n\
<li>The second argument is a string table resource ID, and points to the name that will be on the rollout bar.</li>\n\
<li>The rest of the parameters on the fourth line are not discussed here, see</li>\n\
</ul>\n\
<p><em>Lines 5-10</em></p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">5         // params\n\
6         widget_size,   _T(&quot;size&quot;), TYPE_FLOAT, P_ANIMATABLE, IDS_SPIN,\n\
7            p_default, 0.1f,\n\
8            p_range,   0.0f, 1000.0f,\n\
9            p_ui,      TYPE_SPINNER, EDITTYPE_FLOAT, IDC_SIZE_EDIT, IDC_SIZE_SPIN, 0.50f,\n\
10           p_end,</code></pre><p>The individual parameter specification for the widget size begins after the comment. A parameter block descriptor can be a variable number of individual parameter specs, ours at this point holds only one.</p>\n\
<ul>\n\
<li>To start a new parameter you will first supply the enumerated value for your parameter. In this case <code>widget_size</code>.</li>\n\
<li>The text describing the control is next and is a string. In this <code>case</code> _T(&quot;<code>size</code>&quot;).</li>\n\
<li>The argument <code>TYPE_FLOAT</code> tells what type of value it is. This tells 3ds Max what type of value to store internally for this control.</li>\n\
<li>The argument <code>P_ANIMATABLE</code>, says that the parameter can be animated.</li>\n\
<li>The last argument on this line, <code>IDS_SPIN</code> specifies a string table resource ID.</li>\n\
<li>The next line contains an optional parameter tag, followed by 1 argument specifying the default value of the control.</li>\n\
<li>The next line contains an optional parameter tag, followed by 2 argument specifying the range of values for the control.</li>\n\
<li>The next line contains an optional parameter tag, followed by 5 arguments specifying what type of control this parameter is linked to:<ul>\n\
<li><code>p_ui</code> is the user interface control spec. This is followed by a variable list of arguments depending on the type of UI control specified.</li>\n\
<li><code>TYPE_SPINNER</code> tells what kind of control this is.</li>\n\
<li><code>EDITTYPE_FLOAT</code> tells how to display the values in the edit box that will be linked with this spinner. If we wanted to display these floating point values in inches or in the user specified system units, we would use <code>EDITTYPE_UNIVERSE</code>.</li>\n\
<li><code>IDC_SIZE_EDIT</code> is the identifier of the edit control in the resource editor.</li>\n\
<li><code>IDC_SIZE_SPIN</code> is the identifier of the spinner control in the resource editor.</li>\n\
<li>0.5f is a increment value for the spinner. We will change this to 0.50f so we will get a better viewport response.</li>\n\
</ul>\n\
</li>\n\
<li><code>p_ui</code> is the user interface control spec. This is followed by a variable list of arguments depending on the type of UI control specified.</li>\n\
<li><code>TYPE_SPINNER</code> tells what kind of control this is.</li>\n\
<li><code>EDITTYPE_FLOAT</code> tells how to display the values in the edit box that will be linked with this spinner. If we wanted to display these floating point values in inches or in the user specified system units, we would use <code>EDITTYPE_UNIVERSE</code>.</li>\n\
<li><code>IDC_SIZE_EDIT</code> is the identifier of the edit control in the resource editor.</li>\n\
<li><code>IDC_SIZE_SPIN</code> is the identifier of the spinner control in the resource editor.</li>\n\
<li>0.5f is a increment value for the spinner. We will change this to 0.50f so we will get a better viewport response.</li>\n\
<li>The parameter tag <code>end</code> indicates the end of the parameter description.</li>\n\
</ul>\n\
<p>The rest of the parameter block describes the two other parameter: <code>widget_left</code> and <code>widget_right</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">    widget_left,   _T(&quot;Left&quot;), TYPE_FLOAT, P_ANIMATABLE, IDS_SPIN,\n\
        p_default, 0.0f,\n\
        p_range,   0.0f,100.0f,\n\
        p_ui,      TYPE_SPINNER, EDITTYPE_UNIVERSE,IDC_LEFT_EDIT, IDC_LEFT_SPIN, 0.50f,\n\
        p_end,\n\
    widget_right,  _T(&quot;Right&quot;),  TYPE_FLOAT,     P_ANIMATABLE,     IDS_SPIN,\n\
        p_default, 0.0f,\n\
        p_range,   0.0f,100.0f,\n\
        p_ui,      TYPE_SPINNER, EDITTYPE_UNIVERSE, IDC_RIGHT_EDIT,    IDC_RIGHT_SPIN, 0.50f,\n\
        p_end,</code></pre><p>The end of the entire parameter block is indicated by an additional <code>p_end</code> parameter tag.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">    p_end\n\
    );</code></pre><h2 id=\"creating-the-iparamblock2\">Creating the IParamBlock2</h2><p>The final step in creating a parameter block is to instantiate the <code>IParamBlock2</code> object. If we set the <code>P_AUTO_CONSTRUCT</code> flag in the <code>ParamBlockDesc2</code> constructor the construction is handled by 3ds Max. However the method <code>ClassDesc2::MakeAutoParamBlocks()</code> must be called in a plug-in&apos;s constructor:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Widget::Widget()\n\
{\n\
  WidgetDesc.MakeAutoParamBlocks(this);\n\
}</code></pre><p>Once 3ds Max creates the parameter block is will call <code>Animatable::SetReference()</code> (implemented by your plug-in) passing the parameter block as a <code>ReferenceTarget</code> parameter.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../plug-in_basics.html\">Plug-in Basics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";