var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"enumerating_and_identifying_sub-\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Enumerating and Identifying Sub-Animatables</h1></div><p>If your plug-in wishes to support the animatable interface and appear in the track view you must identify which sub-objects are animatable. These are called sub-animatables. You can do this by overriding the following methods from the <code>Animatable</code> class.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> a parameter block is considered a single animatable sub-object, and any parameters it manages are not counted as animatable sub-objects for the purposes of the following functions.</div>\n\
<ul>\n\
<li><code>Animatable::NumSubs()</code> - Returns the total number of animatable sub-objects maintained by the plug-in. If a plug-in is using a parameter block to manage its parameters it should just count the parameter block as one even if it contains multiple animatable objects. By default this value returns 0.</li>\n\
<li><code>Animatable::SubAnim()</code> - Returns a pointer to the nth animatable sub-object. If a plug-in is using a parameter block to manage its parameters it should just return a pointer to the parameter block itself from this method. This method may return <code>NULL</code> so developers need to check the return value before calling other animatable sub-object methods (such as <code>Animatable::SubAnimName()</code>). By default this method returns <code>NULL</code>.</li>\n\
<li><code>Animatable::SubAnimName()</code> - Returns the name of the nth animatable sub-object to appear in track view. This method returns the name of the nth sub-animatable to appear in track view. A parameter block name does not show up in track view, only its sub-controllers. This is because the parameter block returns true from its override of <code>Animatable::BypassTreeView()</code> to tell the system to not show it in track view. This prevents the user from having to navigate another nested level to simply get to the parameters.</li>\n\
<li><code>Animatable::SubNumToRefNum()</code> - Used for copying and pasting in the track view. It converts an animatable index to a reference index or returns -1 if there is no correspondence. If a client does not wish an animatable to be copied or pasted then it can return -1 even if there is a corresponding reference number. By default this method returns -1.</li>\n\
</ul>\n\
<h2 id=\"example-1\">Example 1</h2><p>The following example is the implementation used by the position constraint sample:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Return one animatable sub-objects, which correspond the parameter block\n\
int NumSubs() {\n\
  return 1;\n\
}\n\
\n\
// Returns the nth animatable sub-object\n\
Animatable* SubAnim(int i) {\n\
  return i != 0 ? pblock : NULL;\n\
}\n\
\n\
// Return the name of the nth animatable sub-object\n\
MSTR SubAnimName(int i) {\n\
  return GetString(IDS_AG_POSITIONPARAMS);\n\
}\n\
\n\
// Returns a reference number of the nth animatable sub-object\n\
int SubNumToRefNum(int i) {\n\
   if (subNum==0)\n\
     return 0; // depends on the number and order of references\n\
   else\n\
     return -1;\n\
}</code></pre><h2 id=\"example-2\">Example 2</h2><p>The following example demonstrates multiple sub-animatables from the simple modifier base class (<code>SimpleMod</code>):</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">int SimpleMod::NumSubs() {\n\
  return3;\n\
}\n\
\n\
Animatable* SimpleMod::SubAnim(inti)\n\
{\n\
   switch (i)\n\
   {\n\
     case 0: return posControl; // Center\n\
     case 1: return tmControl;  // Gizmo\n\
     case 2: return pblock;     // Parameter block\n\
     default: return NULL;\n\
   }\n\
}\n\
\n\
MSTR SimpleMod::SubAnimName(inti)\n\
{\n\
   switch (i)\n\
   {\n\
     case 0: return MSTR(_M(&quot;Center&quot;));\n\
     case 1: return MSTR(_M(&quot;Gizmo&quot;));\n\
     case 2: return MSTR(_M(&quot;Parameters&quot;));\n\
     default: return MSTR(_M(&quot;&quot;));\n\
   }\n\
}\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../plug-in_basics.html\">Plug-in Basics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";