var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"implementing_clone_in_a_plug-in\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Implementing Clone in a Plug-in</h1></div><p>The following code illustrates the recommended method to implement <code>ReferenceTarget::Clone()</code> and <code>ReferenceTarget::BaseClone()</code> in a plug-in when you only have strong references.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class MyDerivedPlugin\n\
: public MyBasePlugin\n\
{\n\
  const int MY_REFERENCE = 1;\n\
\n\
  ReferenceTarget* Clone(RemapDir&amp; remap)\n\
  {\n\
    ReferenceTarget* result = new MyDerivedPlugin();\n\
    BaseClone(this, result, remap);\n\
    return result;\n\
  }\n\
\n\
  void BaseClone(ReferenceTarget* from, ReferenceTarget* to, RemapDir&amp; remap)\n\
  {\n\
    if (!to || !from || from == to)\n\
      return;    \n\
    MyBasePlugin::BaseClone(from, to, remap);\n\
    to-&gt;ReplaceReference(MY_REFERENCE, remap-&gt;CloneRef(from-&gt;GetReference(MY_REFERENCE)));\n\
  }\n\
};</code></pre><p>It is also possible to perform calls to <code>ReferenceMaker::ReplaceReference()</code> in the <code>Clone</code> method itself, but this can lead to complications if you ever want to derive a new plug-in from your old plug-in. Specifically, you cannot call base plug-in&apos;s <code>Clone</code> method and would have to be sure to update all references managed by the derived plug-in correctly in your new derived <code>Clone()</code> method.</p>\n\
<h2 id=\"cloning-of-weakly-and-indirectly-referenced-objects\">Cloning of Weakly and Indirectly Referenced Objects</h2><p>Weakly or indirectly referenced objects should not be cloned, only the pointers should be copied. This means that you do not want to use <code>CloneRefHierarchy()</code> or <code>RemapDir::CloneRef()</code> on a weakly or indirectly referenced object.</p>\n\
<p>Consider the scenario that if two nodes A and B are cloned at the same time, creating nodes C and D. Under node A is a weak reference to node B. After the clone, if you look at the weak reference in C, it should be pointing at D, not at B. To ensure this you have to use the method <code>RemapDir::AddPostPatchProc()</code> passing a class derived from <code>PostPatchProc</code>, to replace weak references that exist to objects which were part of the cloned hierarchy.</p>\n\
<p>The following code demonstrates a method for implementing <code>BaseClone()</code> in a plug-in derived from <code>BaseClass</code> that that has no data other than references to copy.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// This class is used to patch up references at the end of a clone operation.\n\
// An instance of this class is created and registered with the RemapDir if\n\
// a target has not been cloned when the owner is cloned. The Proc\n\
// will see if the target has been cloned at the end of the clone operation, \n\
// and replace the target ref with the clone ref if so.\n\
class PostPatchReference : public PostPatchProc \n\
{\n\
  ReferenceMaker* owner;\n\
  ReferenceTarget* target; \n\
\n\
public:\n\
\n\
    PostPatchReference(ReferenceMaker* owner, ReferenceTarget* target) \n\
    : owner(owner), target(target) \n\
  { }\n\
\n\
  virtual int Proc(RemapDir&amp;)\n\
  {\n\
    // Make sure we still hold a ref to the original target. In some cases we won&apos;t. For example,\n\
    // if owner is a controller on pos track, and target is part of object channel, and you shift-drag to \n\
    // clone, object ref is cloned by NodeSelectionProcessor::ChangeCloneType and it then remaps \n\
    // references, and then the PostPatchProcs are called. So by the time we are called, the original\n\
    // target has been replaced with its clone.\n\
    int refNum = owner-&gt;FindRef(target); \n\
    if (refNum &gt;= 0)\n\
    {\n\
      ReferenceTarget *newTarget = remap.FindMapping(target); // see if target was cloned\n\
      if (newTarget != NULL &amp;&amp; target != newTarget) // yep, update owner\n\
        owner-&gt;ReplaceReference(refNum, newTarget);\n\
    }\n\
    return TRUE;\n\
  }\n\
};      \n\
\n\
virtual void MyPlugin::BaseClone(ReferenceTarget* from, ReferenceTarget* to, RemapDir&amp; remap) \n\
{\n\
    if (!from || !to || from == to)\n\
        return;\n\
\n\
    int numRefs = NumRefs();\n\
    for (int i=0; i &lt; numRefs; ++i)\n\
    {\n\
        ReferenceTarget* fromTarget = from-&gt;GetReference(i);\n\
\n\
        if (from-&gt;IsRealDependency(fromTarget))\n\
            to-&gt;ReplaceReference(i, remap.CloneRef(fromTarget));\n\
        else\n\
        {\n\
          // see if target is already cloned. If so, point at it. If not, we don&apos;t want to \n\
          // force a clone, so we will register a PostPatchProc to check again at the end\n\
          // of the clone, and update our ref to the clone if present.\n\
          ReferenceTarget *newTarget = remap.FindMapping(fromTarget);\n\
          if (newTarget)\n\
            to-&gt;ReplaceReference(i, newTarget);\n\
          else\n\
          {\n\
            to-&gt;ReplaceReference(i, fromTarget);\n\
            remap.AddPostPatchProc(new PostPatchReference(to, fromTarget), true);\n\
          }\n\
       }                   \n\
    }\n\
\n\
    // Call the base class&apos;s implementation of BaseClone\n\
    BaseClass::BaseClone(from, to, remap);\n\
}\n\
\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../plug-in_basics.html\">Plug-in Basics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";