var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"loading_and_saving_plug-in_data\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Loading and Saving Plug-in Data</h1></div><p>3ds Max automatically saves all items referenced by a plug-in when saving the plug-in to a scene file. All parameter blocks are expected to be exposed as references and will therefore be saved and loaded automatically. Any additional data that is not explicitly referenced by the plug-in or contained in a parameter block must be saved or loaded manually.</p>\n\
<p>There are two methods of the <code>ReferenceMaker</code> class which are called by the system when the plug-in&apos;s data needs to be loaded or saved. These methods are named <code>ReferenceMaker::Load()</code> and <code>ReferenceMaker::Save()</code>. The plug-in implements these methods to load or save any special data it has, that is not managed by the parameter blocks.</p>\n\
<p>When a 3ds Max file is saved, each plug-in object is asked to save its data and is provided with an <code>ISave</code> pointer. This data can be partitioned into chunks. When an object needs to be saved, 3ds Max creates a chunk for the object. Inside that chunk, another chunk is created to contain reference information which the system writes. The plug-in&apos;s <code>ReferenceMaker::Save()</code> method is then called. Before <code>ReferenceMaker::Save()</code> is called the file pointer is positioned inside the plug-in&apos;s chunk but after the first chunk containing the reference data. The plug-in can then create its own chunks. Each chunk may contain sub-chunks or data, but not both. If you want to put both data and chunks inside another chunk, the data needs to be contained in a separate chunk. To save a block of binary data, the <code>ISave::Write()</code> method is used that writes a block of bytes to the output stream.</p>\n\
<p>Loading a 3ds Max file is similar. The plug-in is asked to load its data and is given an <code>ILoad</code> pointer which allows it to read chunks. When a plug-in DLL is asked to create a new instance of its class (using the <code>ClassDesc::Create()</code> method), a flag is passed to indicate whether the new instance is being created with the intention of loading an object from a disk file. In this way, if the flag is <code>TRUE</code>, the object doesn&apos;t have to perform any initialization because it is guaranteed that it will be asked to read data from a file.</p>\n\
<h2 id=\"loading-and-saving-class-data\">Loading and Saving Class Data</h2><p>The following three methods of the <code>ClassDesc</code> class may be used to save data associated with a class in a 3ds Max file. This is appropriate for preference data and other similar information associated with a class. If you want to save data associated with the class then you must override <code>ClassDesc::NeedsToSave()</code> to return <code>TRUE</code> and implement the <code>Save()</code> and <code>Load()</code> methods. See the topic on <a href=\"loading_and_saving_plug-in_data.html\"\">Loading and Saving Plug-in Data</a> for more details on the way data is written to and read from 3ds Max files.</p>\n\
<ul>\n\
<li><code>ClassDesc::NeedsToSave()</code> - Returns <code>TRUE</code> if there is data associated with the class that needs to be saved in the 3ds Max file. If this is so, implement the <code>Save()</code> and <code>Load()</code> methods below. If there is no class data to save, return <code>FALSE</code>.</li>\n\
<li><code>ClassDesc::Save()</code> - If <code>ClassDesc::NeedsToSave()</code> returns <code>TRUE</code> then this method should be implemented to save the data associated with the class.</li>\n\
<li><code>ClassDesc::Load()</code> - If <code>NeedsToSave()</code> returns <code>TRUE</code> then this method should be implemented to load the data associated with the class.</li>\n\
</ul>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> One peculiarity with the loading and saving of the class data is that if the user does a File New or File Reset then the class data is not cleared.</div>\n\
<h2 id=\"loading-and-saving-hierarchical-data\">Loading and Saving Hierarchical Data</h2><p>There are two classes available for writing and reading hierarchical data structures to a linear stream, such as an <code>AppData</code> block. These are <code>AppSave</code> and <code>AppLoad</code>. Please see those classes for more details.</p>\n\
<h2 id=\"sample-code\">Sample Code</h2><p>The following code demonstrates how both <code>ReferenceMaker::Save()</code> and <code>ReferenceMaker::Load()</code> may be implemented for a plug-in with two pieces of data it needs to save and restore.</p>\n\
<p>This example writes an array of 10 floating point values and a single <code>DWORD</code> containing flags. The process of saving is very simple. The output file is already open and ready to be written to. A pointer to the <code>ISave</code> class has been passed in with which the plug-in can call methods to save data. The plug-in begins by creating a chunk using the <code>ISave::BeginChunk()</code> method and passing an ID it has defined. This ID can be any <code>USHORT</code>, as only the plug-in&apos;s loading and saving procedures will ever use it. It then writes the data using the <code>ISave::Write()</code> method. It then closes the chunk using <code>ISave::EndChunk()</code>. It begins a new chunk and writes the flag data. After ending this chunk it returns <code>IO_OK</code> to indicate a successful save.</p>\n\
<p>The data is saved in chunks like this so that it may be loaded in a manner which is not order dependent. One could write the flags first and the array second and it would still be read correctly.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#define SAMPLE_DATA_CHUNK 1000\n\
#define SAMPLE_FLAGS_CHUNK 1010\n\
\n\
IOResult Sample::Save(ISave* isave)\n\
{\n\
   ULONG nb;\n\
   isave-&gt;BeginChunk(SAMPLE_DATA_CHUNK);\n\
   isave-&gt;Write(myArray, sizeof(float) * 10, &amp;nb);\n\
   isave-&gt;EndChunk();\n\
   isave-&gt;BeginChunk(SAMPLE_FLAGS_CHUNK);\n\
   isave-&gt;Write(&amp;flags, sizeof(DWORD), &amp;nb);\n\
   isave-&gt;EndChunk();\n\
   return IO_OK;\n\
}</code></pre><p>The following code demonstrates the loading process. Again, in preparation for this call the system has opened the file and positioned the file pointer at the plug-in&apos;s objects chunk. The code loops, opening chunks and reading the data until <code>ILoad::OpenChunk()</code> no longer returns <code>IO_OK</code>. This indicates there are no more chunk at this level and the process is done.</p>\n\
<p>Inside the loop, the code switches on the chunk ID. For each ID it recognizes it reads the data using <code>ILoad::Read()</code>. The chunk is then closed using <code>ILoad::CloseChunk()</code>. If <code>Read()</code> did not return <code>IO_OK</code> an error occurred and this error code is returned. Otherwise, the loop begins again.</p>\n\
<p>If the loading was successful, <code>IO_OK</code> is returned to indicate so.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">IOResult Sample::Load(ILoad* iload)\n\
{\n\
   ULONG nb;\n\
   IOResult res;\n\
   while (IO_OK==(res=iload-&gt;OpenChunk()))\n\
   {\n\
     switch(iload-&gt;CurChunkID())\n\
     {\n\
      case SAMPLE_DATA_CHUNK:\n\
         res=iload-&gt;Read(myArray, sizeof(float)*10, &amp;nb);\n\
         break;\n\
\n\
      case SAMPLE_FLAGS_CHUNK:\n\
         res=iload-&gt;Read(&amp;flags, sizeof(DWORD), &amp;nb);\n\
         break;\n\
     }\n\
     iload-&gt;CloseChunk();\n\
     if (res!=IO_OK) return res;\n\
   }\n\
   return IO_OK;\n\
}</code></pre><p>The possible return values for <code>IOResult</code> are:</p>\n\
<ul>\n\
<li><code>IO_OK</code> - The result was acceptable - no errors.</li>\n\
<li><code>IO_END</code> - This is returned from <code>OpenChunk()</code> when the end of the chunks at a certain level have been reached. It is used as a signal to terminate the processing of chunks at that level.</li>\n\
<li><code>IO_ERROR</code> - This is returned if an error occurred. Note that the plug-in should not put up a message box if a read error occurred. It should simply return the error status. This prevents a overabundance of messages from appearing.</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../plug-in_basics.html\">Plug-in Basics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";