var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"supporting_save_to_previous\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Supporting Save to Previous</h1></div><p>The save to previous feature allows a user to save a scene created in one version of 3ds Max (e.g. 3ds Max 2011) into a file version compatible with an earlier version of the product (e.g. 3ds Max 2010).</p>\n\
<p>A plug-in developer does not need to do anything when the same version of the plug-in is used in both the old and new versions of 3ds Max, or when there has been no change to how the plug-in handles parameters and references. If plug-ins change how they handle parameter or references between versions, they should follow the guidelines listed below, and also those mentioned in the <a href=\"./../../../3ds_max_sdk_features/reference_system/guidelines_for_changing_the_refe.html\">Guidelines for Changing the Reference Structure</a> topic in order to properly support save to previous.</p>\n\
<h2 id=\"guidelines-for-changing-parameter-descriptors\">Guidelines for Changing Parameter Descriptors</h2><ul>\n\
<li>Use the <code>P_VERSION</code> flag and update the version number when changing the parameter block descriptor.</li>\n\
<li>Do not change the parameter block descriptor ID or owner reference ID.</li>\n\
<li>Do not change the type of an existent parameter.</li>\n\
<li>You may add a new parameter of the desired type at the end of the list of parameters and mark an old one obsolete using <code>P_OBSOLETE</code> .</li>\n\
<li>Do not change parameter IDs.</li>\n\
<li>Do not delete or move items in the enum defining the parameter IDs. Instead, add new parameter IDs at the end of the enum.</li>\n\
<li>Do not change the animatable state of a parameter (i.e. do not add or remove the <code>P_ANIMATABLE</code> flag). Instead, mark the old parameter as obsolete and create a new parameter at the end of the parameter list with the desired animatable state.</li>\n\
<li>You may add - but not remove or change - any of the following flags: <code>P_NO_REF</code>, <code>P_OWNERS_REF</code> , <code>P_TRANSIENT</code> , <code>P_SUBANIM</code> , <code>P_READ_ONLY</code> , and <code>P_OBSOLETE</code>.</li>\n\
<li>Non-animatable, and non-reference parameters can be added anywhere in the parameter block descriptor. Reference parameters are: <code>TYPE_MTL</code>, <code>TYPE_TEXMAP</code>, <code>TYPE_INODE</code>, <code>TYPE_REFTARG</code>, <code>TYPE_PBLOCK2</code>, <code>TYPE_OBJECT</code>, and <code>TYPE_CONTROL</code>, along with the corresponding TAB version. Animatable parameters are marked with <code>P_ANIMATABLE</code>.</li>\n\
</ul>\n\
<h2 id=\"guidelines-for-saving-plug-ins\">Guidelines for Saving Plug-ins</h2><ul>\n\
<li>Use <code>ISave::SavingVersion()</code> or <code>::GetSavingVersion()</code> function to determine which Max file format is used for saving your data.</li>\n\
<li>Adding a new chunk should be fine if your plug-in ignores unrecognized chunk IDs in its implementation of <code>ReferenceMaker::Load()</code>.</li>\n\
<li>Removing an existent chunk is not acceptable.</li>\n\
<li>Be prepared to save out the data corresponding to the old chunk ID when 3ds Max signals that it is saving the file to a format compatible with a previous version of 3ds Max. The plug-in must call <code>ISave::SavingVersion()</code> to find this out.</li>\n\
<li>Changing the contents of an existent chunk is best handled as an addition of a new chunk and deprecation of the current chunk (e.g. ignoring it on load).</li>\n\
</ul>\n\
<h2 id=\"animatable-structure-change-guidelines\">Animatable Structure Change Guidelines</h2><p>You cannot change the sub-animatable structure of a plug-in and support save to previous. There&#39;s currently no general way to inform the previous release of what the new mapping is. Therefore, anything that depends on sub-animatable numbering (e.g. expression and wire controllers) will most probsbly crash or at least will not work properly.</p>\n\
<h2 id=\"copying-a-parameter-block-2-to-a-parameter-block-1\">Copying a Parameter Block 2 to a Parameter Block 1</h2><p>When a plug-in is migrated from using parameter block 1 (class IParamBlock) to using parameter block 2 (class IParamBlock2) two responsibilities are particularly important:</p>\n\
<p>The plug-in must support loading parameter block 1 versions of itself. This task is facilitated by the already-existing global function: <code>IParamBlock2* UpdateParameterBlock2(ParamBlockDescID *pdescOld, int oldCount, IParamBlock *oldPB, ParamBlockDesc2* pdescNew, IParamBlock2* newPB);</code></p>\n\
<p>The plug-in must also support saving parameter block 1 versions of itself (for supporting the save to previous functionality of 3ds Max). This task is facilitated by the following global function: <code>int CopyParamBlock2ToParamBlock(IParamBlock2* pb2, IParamBlock *pb1, ParamBlockDescID *pdescPB1, int pb1Count);</code></p>\n\
<p>Although parameter block 1 based plug-ins are still supported in 3ds Max, plug-in developers are encouraged to migrate their plug-ins to use the parameter block 2 system.</p>\n\
<p>The Swirl plug-in is an example of a plug-in that uses these APIs to save a parameter block 1 based version of itself to previous versions of 3ds Max, while maintaining support for loading its old parameter block 1 version.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../plug-in_versioning.html\">Plug-in Versioning</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";