var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"common_problems_and_solutions\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Common Problems and Solutions</h1></div><p>This topic provides a series of commonly-encountered problems and their solutions when writing 3ds Max plug-ins.</p>\n\
<p>Problem 1: Your DLL compiles and links fine, but when you run 3ds Max it complains that <code>LibVersion()</code> is not implemented. You know that you implemented this, so what is going on?</p>\n\
<p>Solution: Did you remember to create a <code>.DEF</code> file with this listed, and if so did you remember to include the file in your project?</p>\n\
<p>Problem 2: When you link <code>InitCommonControls</code> comes up unresolved.</p>\n\
<p>Solution: Did you remember to link in <code>COMCTL32</code>.LIB (which is not included by default by Visual C++ in the list of libraries to link)?</p>\n\
<p>Problem 3: You have a dialog proc to process a button in your user interface. The code is in place to process the button but it never gets called.</p>\n\
<p>Solution: You may have a macro redefinition for one of the buttons in another header file in your project. The compiler takes the first definition and uses it instead of your definition. Therefore yours never gets processed. The compiler will flag this condition as a warning but not as an error.</p>\n\
<p>Problem 4: When you de-allocate memory the system crashes. You have checked that the item being de-allocated exists and is OK to delete. What is happening?</p>\n\
<p>Solution: In Visual C++ the debug and release C runtime libraries use different heap management. This means that allocating an object in debug mode and de-allocating it in release mode (and vice versa) can cause a crash. You need to change your settings for your DEBUG configuration. Choose <code>Build</code>/Settings... from the pulldown menus. From the <code>Settings</code> For: section choose your Debug configuration. From the <code>C</code>/C++ tab, under the <code>Categories</code>: drop down list choose <code>Code</code> Generation. From the <code>Use</code> runtimelibrary: drop down list choose <code>Multithreaded</code> DLL instead of <code>Debug</code> Multithreaded DLL.</p>\n\
<p>Problem 5: You have a plug-in which when compiled under the hybrid settings works fine. When you go to make it in release mode, it works fine except for any time it is used and 3ds Max is exited you get a &quot;Runtime error. R6017 unexpected multithread lock error&quot;.</p>\n\
<p>Solution: The project settings for the Release configuration in the code generation section for run-time library are set to DLL instead of Multithreaded DLL. Changing to Multithreaded DLL corrects the problem.</p>\n\
<p>Problem 6: You have created a plug-in that runs fine on certain machines but on other machines it simply won&#39;t load.</p>\n\
<p>Solution: You probably have a missing system DLL. The easiest way to see what DLLs your plug-in requires is to use the <code>DUMPBIN</code>.EXE program (which comes with Visual C++ and is usually found in the bin folder) and use the <code>/IMPORTS</code> switch. This will tell you all of the DLLs that your DLL depends upon. Make sure the machines that need to run the plug-in have the proper DLLs.</p>\n\
<p>For example: <code>dumpbin</code> importsutility.dlu</p>\n\
<p>Problem 7: You have a plug-in whose interface won&#39;t show up in the command panel yet everything seems fine code wise. What&#39;s wrong?</p>\n\
<p>Solution: You may have forgotten to enter the class names for the custom controls in the VC++ dialog editor. See the topic <a href=\"./../../3ds_max_sdk_features/user_interface/custom_user_interface_controls.html\">Custom User Interface Controls</a> -- especially the section How to a Create a Rollup Page using the Custom Controls for a description of the values to be used in the Class field of the Custom Control Properties dialog.</p>\n\
<p>Problem 7: AppWizard doesn&#39;t work as expected in Vista.</p>\n\
<p>Solution: You may not be running Visual Studio with administrator privileges. When the wizard tries to save its options (e.g. 3ds Max SDK path) it may get a permission denied error because it is trying to write to the Program Files directory, and it doesn&#39;t have the appropriate level of privilege.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../troubleshooting.html\">Troubleshooting</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";