var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"debugging_3ds_max_plug-ins_using\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Debugging 3ds Max plug-ins using Visual Studio</h1></div><h2 id=\"overview\">Overview</h2><p>Developers may use the Visual Studio IDE to debug 3ds Max plug-ins. This provides source code level debugging capabilities while execution is taking place inside the context of the plug-in. One can do things such as set breakpoints (specific lines in the source code where execution stops temporarily so the developer can examine variables, review the call stack, etc.) and establish variables to &apos;watch&apos;. These variables appear in a separate window and display their current contents as they change.</p>\n\
<h2 id=\"setting-up-for-debugging\">Setting Up for Debugging</h2><p>Developers must use the plug-in project&apos;s Hybrid configuration for debugging. The Release build does not contain any source level information and thus cannot be used. The Debug configuration is only available for registered members of the Autodesk Developer Network using a debug build of 3ds Max. For more information see <a href=\"debug_builds_of_3ds_max.html\"\">Debug Builds of 3ds Max</a>.</p>\n\
<p>You can set the Hybrid configuration as the current one by choosing Hybrid from the Configuration drop down list on the standard toolbar. Then choose <strong>Build &gt; Build</strong> from the main menu. This will be create a DLL and associated files appropriate for use while debugging.</p>\n\
<p>Next, VC++ needs to know where the 3ds Max executable is. You can set this through the IDE by opening the project property pages dialog. Then choose the <strong>Configuration &gt; Debugging</strong> folder. In the Command field, enter in the full pathname to 3DSMAX.EXE. In the &apos;Working directory&apos; field enter the full path to the 3ds Max install.</p>\n\
<h2 id=\"setting-up-public-symbols\">Setting Up Public Symbols</h2><p>Autodesk provides public debugging symbols for 3ds Max. These symbols contain useful debugging information, and allow plug-in developers to see their function calls in the context of the calls 3ds Max makes. For more information on symbols, see <a href=\"https://msdn.microsoft.com/en-us/library/windows/hardware/ff551076(v=vs.85).aspx\" target=\"_blank\">Introduction to Symbols</a> on MSDN.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>   <strong></strong> Public symbols are useful for debugging with the release version of 3ds Max. The debug build available to ADN members already contains these symbols, and does not need the public symbol server.</p>\n\
\n\
</div></div>\n\
\n\
<p>To set up Autodesk public symbols in Visual Studio 2012:</p>\n\
<ol>\n\
<li>Go to <strong>Tools <img src=\"./images/ac.menuaro.gif\" alt> Options <img src=\"./images/ac.menuaro.gif\" alt> Debugging <img src=\"./images/ac.menuaro.gif\" alt> Symbols</strong></li>\n\
<li>Add <a href=\"http://symbols.autodesk.com/symbols\" target=\"_blank\">http://symbols.autodesk.com/symbols</a> to the list of symbol file locations, and make sure it is selected (checked)</li>\n\
<li>As you start debugging, Visual Studio will pull the symbols corresponding to the modules being loaded into the debugger from the symbol server into the local symbol cache.</li>\n\
</ol>\n\
<p>For other debuggers, set the <code>\\_NT_SYMBOL_PATH</code> environment variable as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">set _NT_SYMBOL_PATH =\n\
srv*path_to_local_symbol_cache*http://symbols.autodesk.com/symbols</code></pre><h2 id=\"executing-3ds-max-and-your-plug-in\">Executing 3ds Max and Your Plug-in</h2><p>There are several ways to cause execution to halt at a specific spot in your code. One is to set a breakpoint at the spot. This is done by placing the cursor on the line where you want execution to stop and pressing F9. Then, to run the debugger simply press F5. This will launch 3ds Max and get things going. 3ds Max will run and load your plug-in as usual upon startup. Begin execution of your plug-in as you normally would inside 3ds Max (for example, if your plug-in is a modifier apply it to an object). When execution of your plug-in&apos;s code reaches the breakpoint, control will return to VC++ and you may use its tool to examine the state of things (see the section below for more details).</p>\n\
<p>A second way to reach a certain line in your code is to use &apos;Run to Cursor&apos; option. This is chosen by placing the cursor on the desired line of your source file and selecting &apos;Build/Start Debug/Run to Cursor&apos; (or by pressing Ctrl+F10). This will begin execution and stop when the point in your source code where the cursor sits.</p>\n\
<p>Once execution is stopped, you can begin again by pressing F5, or pressing F11 to step to the next line of source or into the next function or method. Note that if you attempt to step into 3ds Max&apos;s code (not your own) you&apos;ll wind up with a window showing the disassembly of the source (since no debugging information is available for 3ds Max&apos;s internal code.) If this happens you can simply close this window and set another breakpoint inside your own code.</p>\n\
<h2 id=\"examining-variables-and-the-call-stack\">Examining Variables and the Call Stack</h2><p>A Variables Windows is available to examine variables within the program&apos;s current context. This window also has a dropdown list for the call stack that shows the hierarchy of functions that are pending completion. This is handy in the case of a program crash since it can often be used to show the sequence of calls made up to the point of the crash.</p>\n\
<p>Also available is a Watch Window. This will display the contents of any variable and updates automatically as the variable changes.</p>\n\
<p>For additional details on these and other debugging tools see the VC++ IDE online help.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../troubleshooting.html\">Troubleshooting</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";