var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"mnmesh_note_on_debugging\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>MNMesh Note on Debugging</h1></div><p>The functions <code>MNVert::MNDebugPrint()</code>, <code>MNEdge::MNDebugPrint()</code>, <code>MNFace::MNDebugPrint()</code>, <code>MNMesh::MNDebugPrint()</code>, and <code>MNMeshBorder::MNDebugPrint()</code> print out detailed ASCII summaries of the component or mesh, which can be an invaluable aid in tracking bugs.</p>\n\
<p>Here is the <code>MNMesh::MNDebugPrint</code> output from a standard 3ds Max box, after applying <code>MakePolyMesh</code>. Mapping coordinates, vertex colors, and face triangulation may also be included.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MNMesh Debug Output: 8 verts, 12 edges, 6 faces\n\
Vertex 0:( -17.185, -26.798, 0.000) Edges:(0 3 9 ) Faces:(0 2 5 )\n\
Vertex 1:( 17.185, -26.798, 0.000) Edges:(2 3 8 ) Faces:(0 2 3 )\n\
Vertex 2:( -17.185, 26.798, 0.000) Edges:(0 1 11 ) Faces:(0 4 5 )\n\
Vertex 3:( 17.185, 26.798, 0.000) Edges:(1 2 10 ) Faces:(0 3 4 )\n\
Vertex 4:( -17.185, -26.798, 24.984) Edges:(4 7 9 ) Faces:(1 2 5 )\n\
Vertex 5:( 17.185, -26.798, 24.984) Edges:(4 5 8 ) Faces:(1 2 3 )\n\
Vertex 6:( -17.185, 26.798, 24.984) Edges:(6 7 11 ) Faces:(1 4 5 )\n\
Vertex 7:( 17.185, 26.798, 24.984) Edges:(5 6 10 ) Faces:(1 3 4 )\n\
Edge 0: verts(0 2), face 0, rev-face 5\n\
Edge 1: verts(2 3), face 0, rev-face 4\n\
Edge 2: verts(3 1), face 0, rev-face 3\n\
Edge 3: verts(1 0), face 0, rev-face 2\n\
Edge 4: verts(4 5), face 1, rev-face 2\n\
Edge 5: verts(5 7), face 1, rev-face 3\n\
Edge 6: verts(7 6), face 1, rev-face 4\n\
Edge 7: verts(6 4), face 1, rev-face 5\n\
Edge 8: verts(1 5), face 2, rev-face 3\n\
Edge 9: verts(4 0), face 2, rev-face 5\n\
Edge 10: verts(3 7), face 3, rev-face 4\n\
Edge 11: verts(2 6), face 4, rev-face 5\n\
Face 0: verts(0 2 3 1 ) edges(0v 1v 2v 3v )\n\
Face 1: verts(4 5 7 6 ) edges(4v 5v 6v 7v )\n\
Face 2: verts(0 1 5 4 ) edges(3v 8v 4v 9v )\n\
Face 3: verts(1 3 7 5 ) edges(2v 10v 5v 8v )\n\
Face 4: verts(3 2 6 7 ) edges(1v 11v 6v 10v )\n\
Face 5: verts(2 0 4 6 ) edges(0v 9v 7v 11v )</code></pre><p><code>MNMesh</code>es have a complex, interlinked topology. Without frequent assertions, a mistake in one part of the code can result in a failure much further down the line. Since an assertion failure in itself provides little diagnostic information, each assertion failure in MNMath.dll is accompanied by a <code>DebugPrint()</code> message that gives more information in the visual studio output pane about the problesm. If this information does not immediately lead to a solution, try using assert to verify <code>MNMesh::CheckAllData()</code> is true between each of your <code>MNMesh</code> operations.</p>\n\
<p>If you call an <code>MNMesh</code> function that produces an inexplicable error after your <code>MNMesh</code> has successfully had all its data checked with <code>MNMesh::CheckAllData()</code>, please contact SDK support.</p>\n\
<p>To give a simple example, suppose you had an application which created an <code>MNMesh</code> from a <code>Mesh</code> , made it into a polygonal mesh, split a single face, and returned.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">SplitUp(Mesh &amp; mesh) {\n\
   MNMesh mm(mesh);\n\
  mm.MakePolyMesh();\n\
  mm.SplitTriEdge(0, .5f);\n\
  mm.OutToTri(mesh);\n\
}</code></pre><p>You try it out on a geosphere, and it works fine. But then you try it out on a box, and for some reason you get an assertion failure somewhere in the MNMath.dll source. By running 3ds Max under the debugger, you can reproduce the error. If you check the debug output pane in Visual Studio, you&apos;ll find the message:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MNMesh::SplitTriEdge error: edge\\&apos;s face 1(0) not a triangle.</code></pre><p>So you check with the documentation and realize that <code>MNMesh::SplitTriEdge</code> is an edge-splitting function that only works on triangulated meshes. To do the same thing to a <code>PolyMesh</code>, you need to use one of the variants of the method <code>SplitEdge</code>. So you can fix your code by either removing <code>MNMesh::MakePolyMesh()</code>, triangulating with a call to <code>MNMesh::Triangulate()</code>, or just calling <code>MNMesh::SplitEdge()</code> with the values 0.0f and 0.5f instead of <code>MNMesh::SplitTriEdge()</code>.</p>\n\
<p>Imagine a more complicated case: in the middle of a series of complex <code>MNMesh</code> modifications made by your program, you need to recompute the triangulation of one of the higher-degree faces with a call to MNMesh::RetriangulateFace. For some reason, this generates an assertion failure. Checking the Debug output, you find the cryptic message:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MNMesh::RetriangulateFace internal error.</code></pre><p>This could mean one of two things. First of all, the MNMesh could have had some of its data scrambled before the call to RetriangulateFace. You can check this by adding an assertion that verifies <code>CheckAllData()</code> is true, immediately before the call to RetriangulateFace. CheckAllData performs a time-intensive series of checks on the mesh structure, checking each mesh component against the components it&apos;s supposed to be linked to, checking that nothing references a dead component, and so on. If any errors are found, the check will fail, and CheckAllData will print diagnostic information to the Debug output.</p>\n\
<p>If <code>CheckAllData()</code> gives your mesh a clean bill of health, but you&apos;re still getting an &quot;internal error&quot;, or just an &quot;error&quot; with no further info, you may have found a bug. Please contact SDK support.</p>\n\
<p>While during your debugging phase, frequent calls to <code>MNMesh::CheckAllData()</code> can be very useful, but the <code>MNDebugPrint()</code> and <code>MNMEsh::CheckAllData()</code> functions are highly time intensive and serve no useful purpose for a released plug-in, you should check to remove them all before release.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../troubleshooting.html\">Troubleshooting</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";