var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"creating_a_qt-based_plug-in\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Creating a Qt-based plug-in</h1></div><p>Once you have installed Qt 5 and the <strong>Qt VS Tools</strong> for Visual Studio 2019 you can start using Qt to build UIs for your 3ds Max plug-ins.  See the <a href=\"./../../about_the_3ds_max_sdk/sdk_requirements.html\">SDK Requirements</a> topic for the Qt version required for your target version of 3ds Max.</p>\n\
<p>3ds Max plug-ins will recognize Qt widgets and hook them into your code when configured properly. In general:</p>\n\
<ul>\n\
<li>The name of the parent Qt widget representing the rollout must match a param block name.</li>\n\
<li>The name of Qt widgets on the rollout should match their corresponding param block members.<ul>\n\
<li>In the case of radio buttons, the Qt radio button group must match a param block method name.</li>\n\
<li>In the case of tab-type param block members, widgets must be in the form of <code>[parametername]_tab_[index]</code>.  For example, with a tab parameter named tex with 7 elements, the last element would match a widget named <code>tex_tab_7</code>.  <strong>Note:</strong> The index is 1-based.</li>\n\
</ul>\n\
</li>\n\
<li>You can use the 3ds Max version of Qt widgets, such as <code>QmaxDoubleSpinBox</code> instead of the generic <code>QFloatSpinBox</code>. These widgets respect various Max settings (such as number of decimal places). See  <em>&lt;maxsdk\\include\\Qt&gt;</em>  for the available widgets.</li>\n\
<li>Each Qt rollout needs a wrapper class derived from <code>MaxSDK::QMaxParamBlockWidget</code> defined in the plug-in. This class allows your code to react to setting or re-setting a new paramblock.</li>\n\
<li>The plug-in&apos;s ClassDesc must implement <code>MaxSDK::QMaxParamBlockWidget* CreateQtWidget()</code>.</li>\n\
<li>The param blocks are created using the <code>P_AUTO_UI_QT</code> flag.</li>\n\
<li>For param block types that don&apos;t match exactly to Qt widget types (for example, a Point3 mapped to three float spinboxes), you will need to implement a <code>PBAccessor</code> class to manage translation to and from the param value.</li>\n\
<li>Finally, your plug-in will need to link  <em>MaxQtBridge.lib</em> .</li>\n\
</ul>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p> As of <strong>3ds Max 2024</strong> you can also manually connect Qt widgets to param blocks in the <code>QMaxParamBlockWidget::PostConnectUI()</code> callback using <code>IParamMap2::ConnectUI()</code>.  See <a href=\"manual_widget_binding.html\"\">Manual Widget Binding</a></p>\n\
\n\
</div></div>\n\
\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>    The plug-in must use <code>IParamBlock2</code>, <code>IParamBlock</code> does not support Qt integration.</p>\n\
\n\
</div></div>\n\
\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>   You can also use Qt for plug-ins that are not param-block based, for example <code>UtilityObj</code> plug-ins.  In that case your wrapper class will derive from <code>QWidget</code> instead of <code>QMaxParamBlockWidget</code>, and you add it to the Max UI using the version of <code>AddRollupPage()</code> that takes a QWidget pointer.  You can specify the <code>ROLLUP_DONT_ADD_TO_CP</code> flag for rollups that do not reside on the command panel, for example for a floating Qt dialog.</p>\n\
\n\
</div></div>\n\
\n\
<p>In this topic, we&apos;ll walk through updating an existing traditional plug-in to use Qt for its rollouts. This will illustrate the basics of using Qt in the 3ds Max plug-in environment. We are going to use the <strong>Qt Designer</strong> to create our Qt forms; although you can do this by hand, it is much easier and faster to use the Designer.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>    A final version of the code in this tutorial is available on the ADN github: <a href=\"https://github.com/ADN-DevTech\" target=\"_blank\">https://github.com/ADN-DevTech</a>.</p>\n\
\n\
</div></div>\n\
\n\
<p>We are going to work on the Geodesic Sphere plug-in, located in  <em>masdk\\samples\\objects</em> .</p>\n\
<p>Make a copy of the  <em>gsphere</em>  directory and name it <em>gsphereqt</em>. To simplify the process, locate the gsphereqt directory in the  <em>&lt;maxsdk&gt;\\samples\\objects</em>  directory, the same directory as  <em>gsphere</em></p>\n\
<p>Open the  <em>gsphereqt\\gsphere.vcxproj</em>  in Visual Studio.</p>\n\
<p>Under <strong>Qt VS Tools &gt; Qt Options</strong>, add the target Qt version, with a path pointing to the installed version of Qt.</p>\n\
<p>We need to manually edit the project file, to convert it to a Qt project so that the Visual Studio Qt extension will recognize it.</p>\n\
<p>Open the <em>gsphere.vcxproj</em> file in a text editor.  Find the &quot;Globals&quot; PropertyGroup element, and after the <code>&lt;ProjectGuid&gt;</code> element, add this element:\n\
<code>&lt;Keyword&gt;Qt4VS&lt;/Keyword&gt;</code></p>\n\
<p>At the bottom of the file, before the final <code>&lt;Project&gt;</code> element add:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-xml prettyprint\">  &lt;ProjectExtensions&gt;\n\
    &lt;VisualStudio&gt;\n\
      &lt;UserProperties Qt5Version_x0020_x64=&quot;5.12.5&quot; /&gt;\n\
    &lt;/VisualStudio&gt;\n\
  &lt;/ProjectExtensions&gt;</code></pre>\n\
<p>Save the project file and re-load the project in Visual Studio.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p> Qt VS Tools presents the option to convert the project from a &quot;custom build steps&quot; project to a Qt/MSBuild project.  For this tutorial we will stay with the custom build steps method, but Qt/MSBuild is supported.  See the <em>&lt;maxsdk&gt;\\samples\\materials\\CameraMapTexture\\CameraMapTexture.vcxproj</em> sample project to see how to configure a project this way.</p>\n\
\n\
</div></div>\n\
\n\
<p>Under <strong>Qt VS Tools &gt; Qt Project Settings</strong>, select the <strong>Qt Modules</strong> tab, and\n\
check <em>Core</em>, <em>GUI</em> and <em>Widgets</em> modules. This will add  the appropriate Qt libraries to the additional dependencies for each configuration.</p>\n\
<p>The final step we need to take to upgrade our project is to add <code>$(QTDIR)\\include</code> under Properties &gt; C/C++ &gt; Additional Include Directories, and <code>$(QTDIR)\\lib</code> under Properties &gt; Linker &gt; Additional Library Directories.</p>\n\
<p>Select <strong>Qt VS Tools &gt; Launch Qt Designer</strong>, and create a new form based on the Widget template. This is our first rollout, and should look something like this:</p>\n\
<p><img src=\"./images/GUID-A699238D-170B-4503-B52C-A8D134D547F4-low.png\" alt></p>\n\
<p>We don&apos;t need to create the title label, just the radio buttons. Add two radio buttons and select <strong>Layout in a Grid</strong> and then <strong>Adjust Size</strong>. Change the text labels to &quot;Diameter&quot; and &quot;Center&quot;, and the radio button <code>objectName</code> properties to &quot;diameter&quot; and &quot;center&quot; respectively.</p>\n\
<p>The radio buttons need to be part of a Qt RadiobuttonGroup with a name that matches a corresponding param name, and the widget itself should match the name of a param block. In this case, we want to hook up the &quot;creationMethod&quot; param to the radio buttons, and the widget with the &quot;GeosphereCreationType&quot; paramblock:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">static ParamBlockDesc2 geo_crtype_blk ( geo_creation_type, _T(&quot;GeosphereCreationType&quot;), 0, &amp;gsphereDesc, P_CLASS_PARAMS + P_AUTO_UI,\n\
    //rollout\n\
    IDD_GSPHERE1, IDS_CREATION_METHOD, BEGIN_EDIT_CREATE, 0, NULL,\n\
    // params\n\
    geo_create_meth,  _T(&quot;creationMethod&quot;),         TYPE_INT,       0, IDS_CREATION_METHOD,\n\
        p_default,      1,\n\
        p_range,        0, 1,\n\
        p_ui,           TYPE_RADIO,     2, IDC_CREATEDIAMETER, IDC_CREATERADIUS,\n\
        p_end,\n\
    p_end\n\
    );</code></pre>\n\
<p>So, we need to name the widget &quot;GeosphereCreationType&quot;, and add the radio buttons to a RadioButtonGroup named &quot;creationMethod&quot;. In Qt Designer, select both buttons, right click, and select <strong>Assign to button group</strong>.  Rename the QButtonGroup &quot;creationMethod&quot;. It should look something like this:</p>\n\
<p><img src=\"./images/QT_Designer_creation_type.png\" alt></p>\n\
<p>Save the form as  <em>gsphere_creation_type.ui</em>  in the project folder, and then add it to the Resource Files in the Solution Explorer in Visual Studio. Because this is a Qt Add-in project, Visual Studio recognizes the UI file and creates an associated header for it:</p>\n\
<p><img src=\"./images/GUID-19B52F7F-1E72-4F9C-B7E5-F669A99F2F3A-low.png\" alt></p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> this header file does not yet exist, because the .ui file has not been compiled. You can force it to compile by right-clicking on the associated <em>.ui</em> file and selecting <strong>Compile</strong>.</div>\n\
<p>Next, we will add a wrapper class for our new rollout. Add a new class to the project, and name it <em>GeosphereCreationType</em>.</p>\n\
<p>Edit  <em>GeosphereCreationType.h</em>  to look like this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#pragma once\n\
#include &lt;maxapi.h&gt;\n\
#include &lt;Qt/QMaxParamBlockWidget.h&gt;\n\
\n\
// This namespace is defined in the header generated by Qt\n\
namespace Ui {\n\
    class GeosphereCreationType;\n\
};\n\
\n\
// Inherit from QMaxParamBockWidget\n\
class GeosphereCreationType :public MaxSDK::QMaxParamBlockWidget\n\
{\n\
    // Connects us to Qt&apos;s meta-object system\n\
    Q_OBJECT\n\
\n\
public:\n\
    explicit GeosphereCreationType(QWidget* parent = nullptr);\n\
    ~GeosphereCreationType(void);\n\
\n\
    // Virtual functions required by QMaxParamBockWidget\n\
    virtual void SetParamBlock(ReferenceMaker* owner, IParamBlock2* const param_block) {};\n\
    virtual void UpdateUI(const TimeValue t) {};\n\
    virtual void UpdateParameterUI(const TimeValue t, const ParamID param_id, const int tab_index) {};\n\
\n\
private:\n\
    Ui::GeosphereCreationType* ui;\n\
\n\
};</code></pre><p>The <code>Q_OBJECT</code> macro causes Visual Studio to recognize this class as part of the Qt system, and will generate the associated source and .moc files. We also have to implement some virtual methods from <code>QMaxParamBlockWidget</code>.</p>\n\
<p>Edit  <em>GeosphereCreationType.cpp</em>  to look like this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#include &quot;GeosphereCreationType.h&quot;\n\
#include &quot;ui_gsphere_creation_type.h&quot;\n\
\n\
GeosphereCreationType::GeosphereCreationType(QWidget* parent)\n\
    :QMaxParamBlockWidget(/*parent*/),\n\
    ui(new Ui::GeosphereCreationType())\n\
{\n\
    ui-&gt;setupUi(this);\n\
    // map radio button IDs to ints in the paramblock:\n\
    // If a button is clicked, the button group will emit a change\n\
    // which will trigger a change in the param block, and vice versa.\n\
    ui-&gt;creationMethod-&gt;setId(ui-&gt;diameter, 0);\n\
    ui-&gt;creationMethod-&gt;setId(ui-&gt;center, 1);\n\
}\n\
\n\
\n\
GeosphereCreationType::~GeosphereCreationType(void)\n\
{   delete ui; }</code></pre><p>Here we set up the ids of the <code>creationMethod</code> Radio Button Group, which is required to hook it up to the <code>creationMethod</code> parameter.</p>\n\
<p>Visual Studio should recognize that our new wrapper class is a <code>Q_OBJECT</code> and to generate Qt moc files for it. Check that a moc file is now auto-generated:</p>\n\
<p><img src=\"./images/qt_tutorial_moc_files.png\" alt>\n\
Once the files are configured correctly, you can right-click .ui files and select &quot;compile&quot; in Visual Studio to actually generate the layouts header moc files based on your design.</p>\n\
<p>Right-click the  <em>gsphere_creation_type.ui</em>  file, and select Compile. You should now be able to open and verify that  <em>ui_gsphere_creation_type.h</em>  is generated.</p>\n\
<p>Finally, we need to add an implementation of <code>CreateQtWidget()</code>. In  <em>GSPHERE.CPP</em> , add this function to the <code>GsphereClassDesc</code> definition:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">virtual MaxSDK::QMaxParamBlockWidget* CreateQtWidget(\n\
        ReferenceMaker&amp; /*owner*/,\n\
        IParamBlock2&amp; paramBlock,\n\
        const MapID paramMapID,\n\
        MSTR&amp; rollupTitle,\n\
        int&amp; rollupFlags,\n\
        int&amp; rollupCategory) {\n\
\n\
        if (paramBlock.ID() == geo_creation_type) {\n\
            // this invokes the Qt translation system:\n\
            //: this is a comment for the translation team\n\
            rollupTitle = GeosphereCreationType::tr(&quot;Creation Method (Qt)&quot;);\n\
            return new GeosphereCreationType();\n\
        }\n\
        return nullptr;\n\
\n\
    };</code></pre><p>Here we set the rollout title. Note the special <em>//:</em> comment format, which is recognized by the Qt translation system. You will need to reorganize the block ID enums and move them above this code.</p>\n\
<p>Finally, let&apos;s modify the <code>geo_crtype_blk</code> param block definition itself to connect to Qt. Delete the rollout definition, and change <code>P_AUTO_UI</code> to <code>P_AUTO_UI_QT</code>. It should look like this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static ParamBlockDesc2 geo_crtype_blk ( geo_creation_type, _T(&quot;GeosphereCreationType&quot;), 0, &amp;gsphereDesc, P_CLASS_PARAMS + P_AUTO_UI_QT,\n\
    // params\n\
    geo_create_meth,  _T(&quot;creationMethod&quot;),         TYPE_INT,       0, IDS_CREATION_METHOD,\n\
        p_default,      1,\n\
        p_range,        0, 1,\n\
        p_end,\n\
    p_end\n\
    );</code></pre><p>We also need to change the ClassID and dll name so they do not conflict with the existing GeoSphere plug-in. Redefine <code>GEOSPHERE_CLASS_ID</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#define GEOSPHERE_CLASS_ID 277485056\n\
</code></pre><p>Import <em>GeosphereCreationType.h</em> in <em>GSPHERE.cpp</em>.</p>\n\
<p>Move the definition of the block ID enums (<code>geo_creation_type</code>, etc) to the top of the file.</p>\n\
<p>Change the target output name to <em>gsphereqt</em>.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>   <strong></strong> Be sure to add MaxQtBridge.lib to the Linker &gt; Input &gt; Additional Dependencies for the project.</p>\n\
\n\
</div></div>\n\
\n\
<p>Compile the project. Load the plug-in in 3ds Max and confirm that our rollout is being created by Qt:</p>\n\
<p><img src=\"./images/GUID-B1F06C06-B43E-42C5-8040-2D3F68E88E28-low.png\" alt></p>\n\
<p>Now let&apos;s add the Parameters rollout.</p>\n\
<p><img src=\"./images/GUID-66F76FCC-6EB5-428A-A5D6-6AF0BBC1DC7E-low.png\" alt>\n\
Create a new widget, add a Double Spin Box, a regular Spin Box, three radio buttons in a group box, and five check boxes. Let&apos;s add a button to illustrate how to hook up slots, even though there isn&apos;t one on the original roll-out. Remember to add a ButtonGroup named &quot;baseType&quot; (to associate it with the baseType parameter), and add all the radio buttons to it. Finally, add labels for the spin boxes, and lay them out approximately like they appear on the rollout we&apos;re trying to re-create:</p>\n\
<p><img src=\"./images/GUID-EAD8C2EF-4CD5-472F-A0DA-93673CAEADF0-low.png\" alt></p>\n\
<p>Select the group box, and apply a grid layout. Apply a grid layout to the whole widget, and then select <strong>Adjust Size</strong>. The final layout should look like this:</p>\n\
<p><img src=\"./images/qt_tutorial_parameters_rollup.png\" alt></p>\n\
<p><strong>Optional:</strong> You can tweak the layout a bit in Qt Designer to get slightly better results.  In our example, the spin boxes look a bit narrow, and appear to float over to the left side of the rollup.  This is because the grid layout has placed them in the second column of the grid, and the checkboxes below cause the first column to be quite wide.  To make the layout look a bit better:</p>\n\
<ul>\n\
<li>Drag the resize handle for each checkbox over to the left, so each checbox spans both columns.</li>\n\
<li>Drag the resize hanlde of the sayHi button over to the left, so the button spans both columns as well.</li>\n\
<li>Select the form QWidget, and find the <strong>layoutColumnStretch</strong> property.  Set this to <code>0,1</code>.  The result should be wider spin boxes:</li>\n\
</ul>\n\
<p><img src=\"./images/qt_tutorial_parameters_rollup2.png\" alt></p>\n\
<p>The 3ds Max SDK provides some customized Qt widgets that will work better in the Max UI. For example, the customized spin boxes will respect user decimal place settings. The headers for these widgets are located in * maxsdk\\include\\Qt* . Your project must link against  <em>MaxQtBridge.lib</em>  to use these widgets. To convert the Radius spinbox, right-click on it and select &quot;Promote To...&quot;</p>\n\
<p>Enter &quot;MaxSDK::QmaxDoubleSpinBox&quot; in Promoted class name.</p>\n\
<p>Enter &quot;Qt/qmaxspinbox.h&quot; in Header File.</p>\n\
<p>Check Global include.</p>\n\
<p>Click Add.</p>\n\
<p>If you had other float spin boxes in the widget, you could convert them by right-clicking and selecting &quot;Promote To &gt; MaxSDK::QmaxDoubleSpinBox&quot;.</p>\n\
<p>You can also promote the &quot;Segments&quot; spinbox to a MaxSDK::QMaxSpinBox using a similar procedure.</p>\n\
<p>Rename the widgets to match their corresponding parameter names. Name the button &quot;sayHi&quot;. Finally, rename the form &quot;GeosphereParameters&quot; to match the name of the wrapper class we will create next.</p>\n\
<p><img src=\"./images/GUID-7413D331-59AB-4345-80A5-793B3EB54239-low.png\" alt></p>\n\
<p>Save the form as  <em>gsphere_parameters.ui</em>  in the  <em>gsphereqt</em>  project directory.</p>\n\
<p>In Visual Studio, add the new form to the resources directory.</p>\n\
<p>Let&apos;s add the wrapper class for this rollout. Create a new class and name it &quot;GeosphereParameters&quot;.</p>\n\
<p>This class is going to look a lot like the previous class:</p>\n\
<p>In  <em>GeosphereParameters.h</em> :</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#pragma once\n\
#include &lt;maxapi.h&gt;\n\
#include &lt;iparamb2.h&gt;\n\
#include &lt;iparamm2.h&gt;\n\
#include &lt;baseinterface.h&gt;\n\
#include &lt;Qt/QMaxParamBlockWidget.h&gt;\n\
\n\
namespace Ui {\n\
    class GeosphereParameters;\n\
};\n\
\n\
class GeosphereParameters :public MaxSDK::QMaxParamBlockWidget\n\
{\n\
    Q_OBJECT\n\
\n\
public:\n\
    explicit GeosphereParameters(QWidget* parent = nullptr);\n\
    ~GeosphereParameters();\n\
\n\
    // boilerplate\n\
    virtual void SetParamBlock(ReferenceMaker* owner, IParamBlock2* const param_block) {};\n\
    virtual void UpdateUI(const TimeValue t) {};\n\
    virtual void UpdateParameterUI(const TimeValue t, const ParamID param_id, const int tab_index) {};\n\
\n\
protected slots:\n\
    void on_sayHi_clicked();\n\
\n\
private:\n\
    Ui::GeosphereParameters* ui;\n\
};</code></pre><p>The <code>slots</code> keyword is a Qt mechanism. Using the Qt signal-slot mechanism and a special naming convention (<code>on_&lt;NAME_OF_MY_WIDGET&gt;_&lt;NAME_OF_THE_SIGNAL&gt;()</code>) here allows us to easily connect a button-click to that function. This connection is setup during <code>ui-&gt;setupUI(this)</code> - see the next code block.</p>\n\
<p>And in  <em>GeosphereParameters.cpp</em> :</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#include &quot;GeosphereParameters.h&quot;\n\
#include &quot;ui_gsphere_parameters.h&quot;\n\
\n\
\n\
GeosphereParameters::GeosphereParameters(QWidget* parent)\n\
    :QMaxParamBlockWidget(/*parent*/),\n\
    ui(new Ui::GeosphereParameters())\n\
{\n\
    ui-&gt;setupUi(this);\n\
    // map radio button IDs to ints in the paramblock:\n\
    // If a button is clicked, the button group will emit a change\n\
    // which will trigger a change in the param block, and vice versa.\n\
    ui-&gt;baseType-&gt;setId(ui-&gt;octa, 0);\n\
    ui-&gt;baseType-&gt;setId(ui-&gt;tetra, 1);\n\
    ui-&gt;baseType-&gt;setId(ui-&gt;icosa, 2);\n\
\n\
}\n\
void GeosphereParameters::GeosphereParameters::on_sayHi_clicked() {\n\
    QMessageBox::information(this, &quot;Hi&quot;, &quot;Hi from Qt&quot;);\n\
}\n\
\n\
GeosphereParameters::~GeosphereParameters()\n\
{\n\
    delete ui;\n\
}</code></pre><p>Add a new condition in <code>GSphereClasDesc::CreateQtWidget()</code> to handle the new rollout creation:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">       if (paramBlock.ID() == geo_params) {\n\
            rollupTitle = GeosphereParameters::tr(&quot;Parameters (Qt)&quot;);\n\
            return new GeosphereParameters();\n\
        }</code></pre><p>And finally edit the param block for keyboard entry, changing <code>P_AUTO_UI</code> to <code>P_AUTO_UI_QT</code>, and remove the rollout definition and p_ui definitions:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static ParamBlockDesc2 geo_param_blk ( geo_params, _T(&quot;GeosphereParameters&quot;),  0, &amp;gsphereDesc, P_AUTO_CONSTRUCT + P_AUTO_UI_QT, PBLOCK_REF_NO,\n\
\n\
    // params\n\
    geo_hemi,   _T(&quot;hemisphere&quot;),       TYPE_BOOL,      P_ANIMATABLE,               IDS_HEMI,\n\
        p_default,      FALSE,\n\
        p_end,\n\
    geo_segs,   _T(&quot;segs&quot;),             TYPE_INT,       P_ANIMATABLE,   IDS_RB_SEGS,\n\
        p_default,      4,\n\
        p_range,        MIN_SEGMENTS, MAX_SEGMENTS,\n\
        p_end,\n\
    geo_radius,  _T(&quot;radius&quot;),          TYPE_FLOAT,     P_ANIMATABLE + P_RESET_DEFAULT,     IDS_RB_RADIUS,\n\
        p_default,      0.0,\n\
        p_ms_default,   25.0,\n\
        p_range,        MIN_RADIUS, MAX_RADIUS,\n\
        p_end,\n\
    geo_type,   _T(&quot;baseType&quot;),         TYPE_INT,       0,              IDS_BASETYPE,\n\
        p_default,      2,\n\
        p_range,        0, 2,\n\
        p_end,\n\
    geo_basepivot,  _T(&quot;baseToPivot&quot;),  TYPE_BOOL,      0,              IDS_BASEPIVOT,\n\
        p_default,      FALSE,\n\
        p_end,\n\
    geo_smooth,     _T(&quot;smooth&quot;),       TYPE_BOOL,      P_ANIMATABLE,               IDS_RB_SMOOTH,\n\
        p_default,      TRUE,\n\
        p_end,\n\
    geo_mapping,    _T(&quot;mapCoords&quot;),    TYPE_BOOL,      0,              IDS_MAPPING,\n\
        p_default,      TRUE,\n\
        p_end,\n\
    p_end\n\
    );</code></pre><p>If you compile and run, you&apos;ll will see our new rollouts, but notice that changing parameters on a <code>gsphereQt</code> object in modify mode does not update the object in the viewport. We will need to use a ParamBlock Accessor to detect param changes and trigger a redraw.</p>\n\
<p>Add a <code>PBAccessor</code> after the <code>GSphereClassDesc</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class GeoPBAccessor : PBAccessor {\n\
    virtual void Get(PB2Value&amp; v, ReferenceMaker* owner, ParamID id, int tabIndex, TimeValue t, Interval &amp;valid) override\n\
    {\n\
        Interface* ip = GetCOREInterface();\n\
        ip-&gt;RedrawViews(t, REDRAW_NORMAL);\n\
    }\n\
};</code></pre><p>Create a static instance of this accessor:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">static GeoPBAccessor _accessor;\n\
</code></pre><p>And add the accessor to each parameter in the <code>GeosphereParameters</code> paramblock:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">p_accessor, &amp;_accessor,</code></pre><p>At this point we have a working plug-in that uses Qt for two of its rollouts, and we&apos;ve seen how to hook up widgets such as spinners, checkboxes and radio buttons to ParamBlocks. We&apos;ve seen how to use the Max widget extensions, such as <code>QmaxDoubleSpinBox</code>, and we&apos;ve also seen how to react to Qt signals such as button clicks. To further explore Qt in Max, have a look at the  <em>maxsdk\\samples\\systems\\sunlight</em>  plug-in, which is entirely Qt-based.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../using_qt_with_3ds_max_plug-ins.html\">Using Qt with 3ds Max Plug-ins</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";