var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"manual_widget_binding\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Manual Widget Binding</h1></div><p>As of <strong>3ds Max 2024</strong> you can manually bind Qt widgets to param blocks.  You can do this once during widget initialization, or (more usefully) re-bind widgets dynamically when the param changes, for example with tab-type params with <code>P_VARIABLE_SIZE</code>.  The example below is summarized from the SDK howto sample under <em>maxsdk/howto/ui/qtObjectDemo</em>, which includes a rollup with a spinner that dynamically rebinds to a variable-sized tab param.</p>\n\
<p>To manually bind a widget:</p>\n\
<ul>\n\
<li>register a <code>QMaxParamBlockWidget::PostConnectUI()</code> callback for your param</li>\n\
<li>in that callback use <code>IParamMap2::ConnectUI()</code> to create a new connection between a widget and a param</li>\n\
</ul>\n\
<p>Then you can react to changes in the param size and unbind/rebind the Qt widget accordingly.</p>\n\
<p>For example, the <em>QtObjectDemo</em> defines this param:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">    pb_float_tab, _T(&quot;floatTabParam&quot;), TYPE_FLOAT_TAB, 0, P_VARIABLE_SIZE | P_ANIMATABLE, 0,\n\
        p_end,</code></pre>\n\
<p>The <code>QtVariableFloatTabRollupWidget</code> defines a <code>PostConnect()</code> callback that connects a spinner to the <code>pb_float_tab</code> param:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">void QtVariableFloatTabRollupWidget::PostConnectUI(const MapID paramMapID)\n\
{\n\
    if (mPBlock &amp;&amp; mParamMapID == paramMapID)\n\
    {\n\
        // Manually bind the tab parameter with the single &apos;Edit next&apos; spinner,\n\
        // since we want to bind it to a specific tab param index.\n\
        // The automatic binding, based on the spinners Qt objectName matching the parameter name,\n\
        // won&apos;t work for us due to the changing of the current tab index.\n\
        auto map = mPBlock-&gt;GetMap(mParamMapID);\n\
        map-&gt;ConnectUI(pb_float_tab, ui-&gt;spinner_tab_val, mCurTabIdx);\n\
    }\n\
\n\
    UpdateLabelTabValues(GetCOREInterface()-&gt;GetTime());\n\
}</code></pre>\n\
<p>It also defines a method for changing the offset index for the <code>pb_float_tab</code> param.  This method then calls the general <code>ConnectUI(QMaxParamBlockWidget)</code> on the param map as a whole, which in turn will call the callback (above) to connect the specific <code>pb_float_tab</code> parameter:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">void QtVariableFloatTabRollupWidget::SetTabOffset(int newTabOffset)\n\
{\n\
    // In this example we use the automatic binding of Qt UI controls to parameters.\n\
    // We have two spinners in the UI which show us a section of the tab parameter values.\n\
    // For getting the auto binding to work, the Qt objectName of the UI control need\n\
    // to match the parameter name plus the according tab index suffix.\n\
    // For this reason when moving the displayed tab section up/down, we change the\n\
    // spinners tab index suffix in the objectName here.\n\
    // The more clever approach would have been using the parameter specific\n\
    // IParamMap::ConnnectUI instead of the renaming and then doing a re-connect all parameters,\n\
    // like it is done on the single spinner example in this demo.\n\
    // But we also want to demonstrate alternatives here.\n\
    int tabCount = mPBlock-&gt;Count(pb_float_tab);\n\
    mTabOffset = std::clamp(newTabOffset, 0, tabCount - 2);\n\
    ui-&gt;floatTabParam_tab_1-&gt;setObjectName(&quot;floatTabParam_tab_&quot; + QString::number(mTabOffset + 1));\n\
    ui-&gt;floatTabParam_tab_2-&gt;setObjectName(&quot;floatTabParam_tab_&quot; + QString::number(mTabOffset + 2));\n\
    // Re-connect all parameter with the UI again, so that the automatic parameter binding can do its magic.\n\
    auto map = mPBlock-&gt;GetMap(mParamMapID);\n\
    map-&gt;ConnectUI(this);\n\
}</code></pre>\n\
<p>See the <em>maxsdk/howto/ui/qtObjectDemo</em> project for more details of the implementation for this example.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../using_qt_with_3ds_max_plug-ins.html\">Using Qt with 3ds Max Plug-ins</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";