var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"using_multi_res_icons\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Using Multi-Res Icons in Max and Qt</h1></div><p>Beginning with 3ds Max 2017, the 3ds Max user interface supports multi-resolution icons that can be used to\n\
display crisp and appealing icons on High-DPI displays. 3ds Max\n\
picks the best fitting icon resource based on the available sizes for an optimal\n\
user experience. For details on the naming conventions of these icon files,\n\
see <code>MaxSDK::LoadMaxMultiResIcon()</code> in the SDK Reference.</p>\n\
<p>As of <strong>3ds Max 2023.1 Update</strong> Scalable Vector Graphics (SVG) files are also supported.</p>\n\
<p>There are three options for accessing icons for use in Qt-based user interfaces:</p>\n\
<ul>\n\
<li>Use standard 3ds Max icons</li>\n\
<li>Use custom icons compiled with the Qt Resource System</li>\n\
<li>Use custom icons loaded from the file system at run time.</li>\n\
</ul>\n\
<h2 id=\"using-standard-3ds-max-icons\">Using Standard 3ds Max Icons</h2><p>3ds Max uses a large library of icons, and these can be accessed by plug-ins by using <code>LoadMaxMultiResIcon()</code> and passing in a path and icon name. Be aware that these icons may be altered in upcoming versions of 3ds Max.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MaxSDK::LoadMaxMultiResIcon( &quot;Common/Delete&quot; );</code></pre><p>You can view the path and name of icons in the 3ds Max UI in the <a href=\"./../../icon_guide/icon_guide.html\">Icon Guide</a>.  \n\
You can also set <code>CustomControlsOptions.PrintIconPaths=true</code> in the Scripting Listener, and then hover the mouse over an icon.  </p>\n\
<hr>\n\
<h2 id=\"custom-icons-compiled-by-qt\">Custom Icons Compiled by Qt</h2><p>You can use the Qt Resource System with your plug-in. Qt provides a method to compile resources into\n\
resource-bundles, which can be linked into the application / dll or loaded dynamically at run-time.  The sample plug-in in <em>maxsdk\\howto\\ui\\qtIconsDemo</em> illustrates how to load icons dynamically using this approach.</p>\n\
<p>The benefit of using a dynamically-loaded resource-bundle is that you have the ability to change\n\
the resources at run-time, as shown in the qtIconsDemo example to support the Dark and Light\n\
theme switching, without having to change anything on the UI code side.</p>\n\
<p>The 3ds Max internal multi-res icon system takes care of reloading the\n\
icons after a theme change, so exchanging the resource-bundle right before that\n\
is all that needs to be done on a plug-in side.</p>\n\
<p>The resources are defined by a .qrc file, with a format that looks like this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">    &lt;!DOCTYPE RCC&gt;&lt;RCC version=&quot;1.0&quot;&gt;\n\
    &lt;qresource prefix=&quot;/qtIconsDemo/Icons&quot;&gt;\n\
        &lt;file&gt;Array_24.png&lt;/file&gt;\n\
        &lt;file&gt;Array_30.png&lt;/file&gt;\n\
        &lt;file&gt;Array_36.png&lt;/file&gt;\n\
        &lt;file&gt;Array_48.png&lt;/file&gt;\n\
        &lt;file&gt;Array.svg&lt;/file&gt;\n\
        &lt;file&gt;Snapshot_24.png&lt;/file&gt;\n\
        &lt;file&gt;Snapshot_30.png&lt;/file&gt;\n\
        &lt;file&gt;Snapshot_36.png&lt;/file&gt;\n\
        &lt;file&gt;Snapshot_48.png&lt;/file&gt;\n\
        &lt;file&gt;Snapshot.svg&lt;/file&gt;\n\
        &lt;/qresource&gt;\n\
    &lt;/RCC&gt;</code></pre><p>Note the prefix attribute in the .qrc file, which specifies the\n\
path where this resource can be found:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MaxSDK::LoadMaxMultiResIcon( &quot;:/qtIconsDemo/Icons/Array&quot; );</code></pre><p>Also note the colon <strong>:</strong> at the beginning of the path. That is the Qt convention for indicating a resource and not a file-system path.</p>\n\
<p>In the example above we supply pixel perfect PNG images for specific sizes, and an SVG version that is used at other sizes.  You can supply only SVG images (which is similar) but may want to provide a PNG for the smallest size to ensure it is scaled correctly.</p>\n\
<p>The .qrc file is then compiled using Qt&apos;s RCC compiler. See the custom build-step in\n\
the qtIconsDemo project.  In that sample, the icon set for the light and dark themes are each copied to a temporary directory, the .qrc file is copied to the same location, and RCC is called to compile the icons. The usage looks like this:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">rcc.exe qtIconsDemo.qrc -binary -o qtIconsDemo.rcc</code></pre><p>This .rcc file then has to be registered as a resource-bundle at run-time using\n\
this call:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">QResource::registerResource( path_to_my_bundle.rcc )</code></pre><p>and can be unregistered using</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">QResource::unregisterResource( path_to_my_bundle.rcc );</code></pre><p>Note that registering the resource-bundle does not load anything into\n\
memory, as the resources are loaded later on demand. If dynamic change of the\n\
resources in a bundle is not needed (for example, if you do not need to support light and dark theming), the qrc can also be simply compiled into a <code>.cpp</code> file that can be compiled into a dll.</p>\n\
<p>For more detailed information see the <a href=\"http://doc.qt.io/qt-4.8/resources.html\" target=\"_blank\">Qt Resource System documenation</a>.</p>\n\
<blockquote>\n\
<p>To make the qtIconsDemo sample work, make sure to copy the qtIconsDemo_Dark.rcc and\n\
qtIconsDemo_Light.rcc files into the UI_ln folder inside the 3dsmax installation.</p>\n\
</blockquote>\n\
<hr>\n\
<h2 id=\"load-icons-from-the-file-system-at-run-time\">Load Icons from the File System at Run-time</h2><p>Your plug-in can load icons in PNG in SVG format from the file system as required.  This is the easiest way to quickly do some testing, but\n\
probably not the optimal solution for production code used by customers.</p>\n\
<p><code>LoadMaxMultiResIcon()</code> can load icons under <em>UI_ln/Icons</em> in the 3ds Max install location.  Pass the sub-path to the function such as:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">MaxSDK::LoadMaxMultiResIcon( &quot;qtIconsDemo/Cut&quot; );</code></pre><p>This will look into that folders and load one of those files:</p>\n\
<p>UI_ln/Icons/Dark/qtIconsDemo/Cut_24.png\n\
UI_ln/Icons/Dark/qtIconsDemo/Cut_30.png\n\
UI_ln/Icons/Dark/qtIconsDemo/Cut_36.png\n\
UI_ln/Icons/Dark/qtIconsDemo/Cut_48.png</p>\n\
<p>UI_ln/Icons/Light/qtIconsDemo/Cut_24.png\n\
UI_ln/Icons/Light/qtIconsDemo/Cut_30.png\n\
UI_ln/Icons/Light/qtIconsDemo/Cut_36.png\n\
UI_ln/Icons/Light/qtIconsDemo/Cut_48.png</p>\n\
<p>Depending on the current theme of 3dsmax and the display resolution.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> To make the qtIconsDemo sample work, make sure to copy the contents of the UI_ln\n\
folder in the project directory into the UI_ln folder of the 3ds Max\n\
installation.</div>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../using_qt_with_3ds_max_plug-ins.html\">Using Qt with 3ds Max Plug-ins</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";