var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"making_controllers_lockable\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Making Controllers Lockable</h1></div><p>Follow these steps to make a controller lockable. Also, note that the samples in <code>maxsdk\\\\samples\\\\controllers</code> are lockable.</p>\n\
<ol>\n\
<li><p>Include the header file <code>ILockedTracks.h</code>.</p>\n\
</li>\n\
<li><p>Derive your controller plug-in publicly from <code>ILockedTrackImp</code> directly or from a class that derives from <code>ILockedTrackImp</code> such as <code>StdLockableControl</code> or <code>LockableControl</code>.</p>\n\
<ul>\n\
<li>When possible prefer <code>StdLockableControl</code> or <code>LockableControl</code> because they already contain default functions needed to properly lock your controller. For example they contain all of the logic needed to determine whether or not an item is really locked, and handles the default mechanism for the unlocking and locking of parent and child sub-animatables.</li>\n\
<li>Make sure to keep the same logic if you override the default implementations of <code>Animatable::GetInterface()</code>, <code>Control::IsReplaceable()</code>, and <code>Control::CanApplyEaseMultCurves()</code>.</li>\n\
</ul>\n\
</li>\n\
<li><p>Make sure the <code>Animatable::GetInterface()</code> function is defined because it is where the locked track interface resides. At a minimum you need to implement it as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void* GetInterface(ULONG id)\n\
{\n\
  switch (id) {\n\
    case I_LOCKED:\n\
      return (ILockedTrackImp*) this;\n\
    default: \n\
      return Control::GetInterface(id);  \n\
  }\n\
}</code></pre></li>\n\
<li><p>If your controller is a leaf controller, you need to disable the appending of ease and mult curves to your controller. To do this, you need to modify the <code>Control::CanApplyEaseMultCurves()</code> function to return false when the controller is locked. If it is not a leaf controller, there is no need to worry about this function because you cannot apply eases or mult controllers to non-leaf controllers.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">BOOL CanApplyEaseMultCurves() {\n\
  return !GetLocked(); \n\
}</code></pre></li>\n\
<li><p>When a controller is locked it cannot be replaced, so you need to modify the <code>Control::IsReplaceable()</code> function as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">  BOOL IsReplaceable() {\n\
    return !GetLocked();\n\
  }</code></pre></li>\n\
<li><p>Make sure that the <code>ReferenceMaker::Load()</code> and <code>ReferenceMaker::Save()</code> functions are implemented and that you save and load the <code>ILockedTrackImp::mLocked</code> member variable as shown in the following example:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">#define LOCK_CHUNK 0x2535\n\
IOResult MasterPointControlImp::Save(ISave *isave)\n\
{\n\
  Control::Save(isave); // save ORT\n\
  ULONG nb;\n\
  int on = (mLocked==true) ? 1 :0;\n\
  isave-&gt;BeginChunk(LOCK_CHUNK);\n\
  isave-&gt;Write(&amp;on,sizeof(on),&amp;nb);  \n\
  isave-&gt;EndChunk(); \n\
  return IO_OK;\n\
}\n\
IOResult MasterPointControlImp::Load(ILoad *iload)\n\
{\n\
  ULONG nb;\n\
  IOResult res;\n\
  res = Control::Load(iload); // Load ORT\n\
  if (res!=IO_OK)  return res;\n\
  while (IO_OK==(res=iload-&gt;OpenChunk()))\n\
  {\n\
    ULONG curID = iload-&gt;CurChunkID();\n\
    if (curID==LOCK_CHUNK)\n\
    {\n\
      int on;\n\
      res=iload-&gt;Read(&amp;on,sizeof(on),&amp;nb);\n\
      mLocked = on ? true : false;\n\
    }\n\
    iload-&gt;CloseChunk();\n\
    if (res!=IO_OK)  return res;\n\
  }\n\
  return IO_OK; \n\
}</code></pre></li>\n\
<li><p>In your implementation of <code>ReferenceTarget::Clone()</code>, make sure to clone the inherited value of <code>ILockedTrackImp::mLocked</code>.</p>\n\
</li>\n\
<li><p>In your implementation of <code>Control::Copy()</code>, make sure to copy the value of <code>mLocked</code>.</p>\n\
</li>\n\
<li><p>Note that only the functions <code>Save()</code>, <code>Load()</code>, <code>Clone()</code>, and <code>Copy</code> require explicit use of the <code>ILockedTrackImp::mLocked</code> variable. Other functions which need to query the locked status of a controller must use the <code>ILockedTrackImp::GetLocked()</code> function because that takes into consideration any active overrides or container states to determine if the controller is locked.</p>\n\
</li>\n\
<li><p>Inside the controller&apos;s implementation of <code>Animatable::AssignController()</code>, you need to check both to see if the controller is locked, in which case you do not assign the controller, and the controller that is getting assigned over is also not locked. You must also modify <code>Animatable::CanAssignController()</code>, but that function cannot be used reliably to determine if a controller might get assigned.</p>\n\
</li>\n\
<li><p>Some functions need to be locked: meaning that they return immediately if <code>ILockedTrackImp::GetLocked()</code> returns <code>true</code>.</p>\n\
</li>\n\
<li><p>If a controller is locked it must lock the functions <code>Animatable::BeginEditParams()</code> and <code>Animatable::EditTrackParams()</code>. You might also need to modify <code>Animatable::EndEditParams()</code> accordingly. If you do normally allow the editing of track parameters, you must return <code>TRACKPARAMS_NONE</code> from the <code>Animatable::TrackParamsType()</code> function when the track is locked.</p>\n\
</li>\n\
<li><p>In addition to the <code>Animatable</code> functions mentioned above, other <code>Animatable</code> functions that must be locked are those that modify the keys or other information in your controller, such as <code>Animatable::CopyKeysFromTime()</code>, <code>Animatable::DeleteKeyAtTime()</code>, <code>Animatable::DeleteKeys()</code>, <code>Animatable::DeleteTime()</code>, <code>Animatable::ReverseTime()</code>, <code>Animatable::ScaleTime()</code>, <code>Animatable::InsertTime()</code>, <code>Animatable::MapKeys()</code>, <code>Animatable::AddNewKey()</code>, <code>Animatable::MoveKeys()</code>, <code>Animatable::ScaleKeyValues()</code>, <code>Animatable::SetSelKeyCoordsExpr()</code>, <code>Animatable::CommitSetKeyBuffer()</code>, <code>Animatable::RevertSetKeyBuffer()</code>, and others.</p>\n\
</li>\n\
<li><p>You must also lock schematic view functions that change links, such as <code>Animtable::SvCanConcludeLink()</code>, and <code>Animatable::SVLinkChild()</code>.</p>\n\
</li>\n\
<li><p>You usually do not have to lock functions that set selection (<code>Animatable::SetKeyByIndex</code>, and others) or flag keys (<code>Animatable::FlagKey()</code>) because in most circumstances the selection or flag state of a key does not change how that controller computes its value. In general, only lock operations will have an effect when <code>Control::GetValue()</code> is called.</p>\n\
</li>\n\
<li><p>The control functions that need to be locked include <code>Control::SetValue()</code>, <code>Control::CreateLockKey()</code>, and any function that sets a sub-controller (such as <code>Control::SetPositionController()</code>, or <code>Control::SetRollController()</code>).</p>\n\
</li>\n\
<li><p>Control functions that must never get locked include <code>Control::GetValue()</code> and <code>Control::ChangeParents()</code>.</p>\n\
</li>\n\
<li><p>For most controllers, the only IK function that needs to get locked is <code>Control::PasteIKParams()</code>. The other functions are needed to be left unlocked or IK might not work.</p>\n\
</li>\n\
<li><p>If the controller inherits from <code>IKeyControl</code>, then some of its functions also need to be locked. They include <code>IKeyControl::SetKey()</code> and <code>IKeyControl::AppendKey()</code>. Note that <code>IKeyControl::GetKey()</code> returns a pointer to the key which in theory the client might modify, but cannot be locked because it is the only way a client can examine the contents of a key.</p>\n\
</li>\n\
<li><p>For PRS controllers, you will need to lock the setting of inheritance flags in <code>Control::SetInheritanceFlags()</code> Do not lock <code>Control::ChangeParents()</code> because when a parent changes you will want the keys to modify, if needed. Also, you might need to unlock sub controllers, using <code>ILockedTracksMan::PushUberUnLockOverride()</code> and <code>ILockedTracksMan::PopUberUnLockOverride()</code> when changing parents before calling <code>Control::SetValue()</code> on them.</p>\n\
</li>\n\
<li><p>If you need to unlock something temporarily, the best way is to use the <code>ILockedTracksMan::PushUberUnLockOverride()</code> function to set up an override that disables all locks, and then the <code>ILockedTracksMan::PopUberUnLockOverride()</code> function to revert the override unlock state. This is safer than individual unlocking the controller because if a controller is in an imported container, it cannot get unlocked through the other SDK functions.</p>\n\
</li>\n\
<li><p>We currently do not lock exposed SDK or MAXScript published functions that might set internal values, which are normally locked. Usually, this is not a problem because we prevent display of the default UI.</p>\n\
</li>\n\
</ol>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../writing_animation_controller_plu.html\">Writing Animation Controller Plug-ins</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";