var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"writing_maxscript_plug-ins\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Writing MAXScript Plug-ins</h1></div><p>MAXScript plug-ins have the file extension <code>.dlx</code>.  They can extend MAXScript in several ways, including:</p>\n\
<ul>\n\
<li>Defining system globals</li>\n\
<li>Adding structs, functions, and classes</li>\n\
<li>Adding custom UI controls</li>\n\
<li>Extending existing scripted plug-ins</li>\n\
</ul>\n\
<blockquote>\n\
<p><strong>Note</strong>: There are several sample DLX projects in the 3ds Max sdk howto and samples directories.  See the <a href=\"./../about_the_3ds_max_sdk/sample_plug-ins/maxscript_extension_plug-ins.html\">MAXScript Extension Plug-ins</a> topic for a list of these projects.  The <code>howto\\intervalarray</code> and <code>howto\\testdlx projects</code> are simple examples that illustrate how to add different types of extensions.</p>\n\
</blockquote>\n\
<p>Other plug-in types use the Function Publishing method to expose themselves to MAXScript.  DLX plug-ins expose functions by extending the Primitive class, and using the def_visible_primitive macro from <code>maxscript\\macros\\define_instantiation_functions.h</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">// Declare C++ function and register it with MAXScript\n\
#include &lt;maxscript\\macros\\define_instantiation_functions.h&gt;\n\
    def_visible_primitive(IntervalArray, &quot;IntervalArray&quot;);</code></pre>\n\
<blockquote>\n\
<p><strong>Note</strong>: DLX plug-ins cannot be loaded or viewed in the Plug-in Manager.  When writing and testing a DLX plug-in, you need to add the path to the compiled plug-in to the <strong>3rd Party Plug-Ins</strong> paths (under Customize &gt; Configure User and System Paths), and re-start 3dx Max.</p>\n\
</blockquote>\n\
<h2 id=\"def-and-dll-entry\">DEF and DLL Entry</h2><p>DLX plug-ins must define a DLL entry point that sets the language locale and disables threading when the DLL is loaded.  This file also exports the DLL&apos;s initialization function, the description, and the version.</p>\n\
<p>MAXScript plug-ins do not generally use <code>ClassDesc</code> so their <code>.cpp</code> and <code>.def</code> files are simpler as they do not export the functions <code>GetClassDesc()</code> and <code>GetNumClasses()</code>. </p>\n\
<p>The following is a typical MAXScript plug-in <code>.def</code> file:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">LIBRARY IntervalArray.dlx\n\
EXPORTS\n\
   LibDescription       @1       PRIVATE\n\
   LibInit              @2       PRIVATE\n\
   LibVersion           @3       PRIVATE\n\
SECTIONS\n\
   .data READ WRITE</code></pre><p>The following is the implementation of the IntervalArray howto sample <code>DllMain.cpp</code> file (in <code>maxsdk/howto/maxscript/intervalarray/</code>):</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">//DLLMain.cpp\n\
\n\
#include &lt;maxscript/maxscript.h&gt;\n\
HINSTANCE hInstance;\n\
extern void IntervalArrayInit ();\n\
// ========================================================\n\
// Grab onto this DLL&apos;s instance handle\n\
BOOL WINAPI DllMain(HINSTANCE DLLhinst, DWORD fdwReason, LPVOID lpvReserved)\n\
{\n\
    switch(fdwReason)\n\
    {\n\
    case DLL_PROCESS_ATTACH:\n\
        MaxSDK::Util::UseLanguagePackLocale();\n\
        hInstance = DLLhinst;\n\
        DisableThreadLibraryCalls(hInstance);\n\
        break;\n\
    }\n\
    return TRUE;\n\
}\n\
__declspec(dllexport) void LibInit()\n\
{\n\
    //TODO: Put any code for initializing your plugin here. \n\
    //In this case it is IntervalArrayInit(), which will be defined in IntervalArray.cpp,\n\
    //However in this example IntervalArrayInit() is empty.\n\
    IntervalArrayInit();\n\
}\n\
__declspec(dllexport) const TCHAR* LibDescription()\n\
{\n\
    //TODO: Put code in here telling what your plugin does.\n\
    return _T(&quot;Interval Array Function&quot;);\n\
}\n\
__declspec(dllexport) ULONG LibVersion()\n\
{\n\
    //Return the version of the Max SDK\n\
    return VERSION_3DSMAX;\n\
}</code></pre>\n\
<blockquote>\n\
<p><strong>Note</strong>: There are several sample DLX projects in the 3ds Max sdk howto and samples directories.  See the <a href=\"./../about_the_3ds_max_sdk/sample_plug-ins/maxscript_extension_plug-ins.html\">MAXScript Extension Plug-ins</a> topic for a list of these projects.  The <code>howto\\intervalarray</code> and <code>howto\\testdlx projects</code> are simple examples that illustrate how to add different types of extensions.</p>\n\
</blockquote>\n\
<p>Other plug-in types use the Function Publishing method to expose themselves to MAXScript.  DLX plug-ins expose functions by extending the Primitive class, and using the def_visible_primitive macro from <code>maxscript\\macros\\define_instantiation_functions.h</code>:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">// Declare C++ function and register it with MAXScript\n\
#include &lt;maxscript\\macros\\define_instantiation_functions.h&gt;\n\
    def_visible_primitive(IntervalArray, &quot;IntervalArray&quot;);</code></pre>\n\
<blockquote>\n\
<p><strong>Note</strong>: DLX plug-ins cannot be loaded or viewed in the Plug-in Manager.  When writing and testing a DLX plug-in, you need to add the path to the compiled plug-in to the <strong>3rd Party Plug-Ins</strong> paths (under Customize &gt; Configure User and System Paths), and re-start 3dx Max.</p>\n\
</blockquote>\n\
<h2 id=\"def-and-dll-entry\">DEF and DLL Entry</h2><p>The DLL entry definition is simpler than for other plug-in types, as MAXScript plug-ins do not generally use <code>ClassDesc</code> so their <code>.cpp</code> and <code>.def</code> files do not export the functions <code>GetClassDesc()</code> and <code>GetNumClasses()</code>. </p>\n\
<p>They only need to export:</p>\n\
<ul>\n\
<li><code>LibInit()</code> - all initialization is performed here</li>\n\
<li><code>LibDescription()</code> - a short description of the DLL</li>\n\
<li><code>LibVersion()</code> - returns the supported version of 3ds Max (the SDK version used to build the DLL)</li>\n\
</ul>\n\
<p>The following is a typical MAXScript plug-in <code>.def</code> file:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">LIBRARY IntervalArray.dlx\n\
EXPORTS\n\
   LibDescription       @1       PRIVATE\n\
   LibInit              @2       PRIVATE\n\
   LibVersion           @3       PRIVATE\n\
SECTIONS\n\
   .data READ WRITE</code></pre>\n\
<p>DLX plug-ins must define a DLL entry point that sets the language locale and disables threading when the DLL is loaded.  </p>\n\
<p>The following is the implementation of the IntervalArray howto sample <code>DllMain.cpp</code> file (in <code>maxsdk/howto/maxscript/intervalarray/</code>):</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">//DLLMain.cpp\n\
\n\
#include &lt;maxscript/maxscript.h&gt;\n\
HINSTANCE hInstance;\n\
extern void IntervalArrayInit ();\n\
// ========================================================\n\
// Entry point for this DLL.  See https://docs.microsoft.com/en-us/windows/desktop/dlls/dllmain\n\
// Here we set up the user language, store the DLL handle, and disable threading for system calls\n\
// because 3dsmax.exe is not multi-threaded.\n\
BOOL WINAPI DllMain(HINSTANCE DLLhinst, DWORD fdwReason, LPVOID lpvReserved)\n\
{\n\
    switch(fdwReason)\n\
    {\n\
    case DLL_PROCESS_ATTACH:\n\
        MaxSDK::Util::UseLanguagePackLocale();\n\
        hInstance = DLLhinst;\n\
        DisableThreadLibraryCalls(hInstance);\n\
        break;\n\
    }\n\
    return TRUE;\n\
}\n\
__declspec(dllexport) void LibInit()\n\
{\n\
    // This function is called when the DLL is loaded to do any initialization. \n\
    // In this case it is IntervalArrayInit(), which is defined in IntervalArray.cpp,\n\
    IntervalArrayInit();\n\
}\n\
__declspec(dllexport) const TCHAR* LibDescription()\n\
{\n\
    // Return a short description string here\n\
    return _T(&quot;Interval Array Function&quot;);\n\
}\n\
__declspec(dllexport) ULONG LibVersion()\n\
{\n\
    // Returns the version of the Max SDK, used to check compatibility when loaded by Max\n\
    return VERSION_3DSMAX;\n\
}</code></pre>\n\
<p>Finally, here is the actual implementation of the IntervalArray type in IntervalArray.cpp:</p>\n\
<pre class=\"pre codeblock\"><code class=\"lang-cpp prettyprint\">#include &lt;maxscript/maxscript.h&gt;\n\
#include &lt;maxscript/foundation/numbers.h&gt;\n\
#include &lt;maxscript/foundation/arrays.h&gt;\n\
\n\
void IntervalArrayInit()\n\
{\n\
    //Todo: Place initialization code here. This gets called when Maxscript goes live\n\
    //during max startup.\n\
}\n\
\n\
// Declare C++ function and register it with MAXScript\n\
#include &lt;maxscript\\macros\\define_instantiation_functions.h&gt;\n\
    def_visible_primitive(IntervalArray, &quot;IntervalArray&quot;);\n\
\n\
Value* IntervalArray_cf(Value **arg_list, int count)\n\
{\n\
    //--------------------------------------------------------\n\
    //Maxscript usage:\n\
    //--------------------------------------------------------\n\
    // &lt;array&gt; IntervalArray &lt;Start:Number&gt; &lt;End:Number&gt; &lt;steps:Integer&gt;\n\
    check_arg_count(IntervalArray, 3, count);\n\
    Value* pBegin = arg_list[0];\n\
    Value* pEnd = arg_list[1];\n\
    Value* pStep = arg_list[2];\n\
\n\
    //First example of how to type check an argument\n\
    if ( ! (is_number(pBegin)))\n\
    {\n\
        throw RuntimeError(_T(&quot;Expected a Number for the first argument, in function IntervalArray&quot;));\n\
    }\n\
    if ( ! (is_number(pEnd)))\n\
    {\n\
        throw RuntimeError(_T(&quot;Expected a Number for the second argument, in function IntervalArray&quot;));\n\
    }\n\
    //Second example of how to type check an argument\n\
    integer_type_check(pStep, _T(&quot;Expected an Integer for the step size&quot; ));\n\
\n\
    float begin = pBegin-&gt;to_float();\n\
    float end   = pEnd-&gt;to_float();\n\
    int steps   = pStep-&gt;to_int();\n\
\n\
\n\
    if (steps &lt;= 0) { throw RuntimeError(_T(&quot;Expected a positive Number for Steps, in function IntervalArray&quot;));}\n\
    MAXScript_TLS* _tls = (MAXScript_TLS*)TlsGetValue(thread_locals_index);\n\
    one_typed_value_local_tls(Array* rArray);\n\
    vl.rArray = new Array (steps);\n\
\n\
    float data = begin;\n\
    float increment = ((end-begin)/steps);\n\
\n\
    for (int i = 0 ; i &lt;= steps ; i++)\n\
    {\n\
        Value* aTempNumber = Float::heap_intern(data);\n\
        vl.rArray-&gt;append(aTempNumber);\n\
        data += increment;\n\
    }\n\
    return_value_tls(vl.rArray);\n\
}\n\
</code></pre>\n\
<h3 id=\"see-also\">See Also</h3><ul>\n\
<li><a href=\"plug-in_basics.html\"\">Plug-in Basics</a></li>\n\
<li><a href=\"advanced_plug-in_topics.html\"\">Advanced Plug-in Topics</a></li>\n\
<li><a href=\"./../3ds_max_sdk_features/maxscript_scripting_engine.html\">MAXScript Scripting Engine</a>.</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../writing_plug-ins.html\">Writing Plug-ins</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";