var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"managing_modifier_parameters\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Managing Modifier Parameters</h1></div><p>A modifier plug-in may have parameters which are animated or which are constant. The developer decides which of the modifier&apos;s parameters may be animated. Typically in 3ds Max, every parameter is animatable (provided it makes sense). For instance, the angle and direction parameters of the Bend modifier are animated while the Axis of the bend is not.</p>\n\
<p>For a parameter to be animatable, it must have a controller to control the animation. Different parameter types require different controller types. For instance, a floating point value, like the angle parameter, requires a floating point controller. A node transformation matrix requires a transform controller. The full details of parameter management using controllers is covered in the <a href=\"./../../3ds_max_sdk_features/animation_time_and_controllers.html\">Animation, Time, and Controllers</a> section. The parameter block system greatly simplifies the animating of parameters.</p>\n\
<p>One of the main purposes of a parameter block is to manage the complexity of maintaining different controllers for different parameters. The developer&apos;s job is simplified because to store and retrieve animated values, one simply uses the methods <code>GetValue()</code> and <code>SetValue()</code>. The parameter block stores values in an efficient a manner as possible. If the value hasn&apos;t yet been animated, that is, <code>SetValue()</code> hasn&apos;t been called with time not equal to 0, then a constant value is stored. When <code>SetValue()</code> is called with time not equal to 0, and Max&apos;s &apos;Animate&apos; button is on, a new instance of the default controller for the parameter type is plugged in to the parameter block and initialized with the parameter&apos;s values. Plug-ins are required to display their animated parameters in the track view. If all the plug-in&apos;s parameters are handled by a parameter block, then the parameter block will take care of this task as well.</p>\n\
<h2 id=\"bend-example\">Bend Example</h2><p>In the Bend example, a parameter block is used to handle the Angle, Direction, Axis, DoRegion, From and To parameters. The base class <code>SimpleMod</code> and <code>SimpleMod2</code> manages this parameter block. The parameter block is part of the SimpleMod2 class:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class SimpleMod2 : publicSimpleMod {\n\
     public:\n\
          IParamBlock2* pblock2;</code></pre><p>Getting a value at any time is as simple as calling</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">float to;\n\
u-&gt;pblock2-&gt;GetValue(bend_to,t,to,FOREVER);</code></pre><p>or</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">float from = 45.4\n\
u-&gt;pblock2-&gt;SetValue(bend_to,t,from);</code></pre><p><img src=\"./images/GUID-D31ABBE6-9C23-4821-9C22-67A062717852-low.jpg\" alt></p>\n\
<p>See the topics on <a href=\"./../../3ds_max_sdk_features/plug-ins/parameter_blocks.html\">Parameter Blocks</a> for more info.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../writing_modifier_plug-ins.html\">Writing Modifier Plug-ins</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";