var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"implementing_sub-object_selectio\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Implementing Sub-Object Selection</h1></div><p>A modifier registers its sub-object selection levels when <code>Animatable::BeginEditParams()</code> method is called. The method <code>Interface::RegisterSubObjectTypes()</code> can be used to handle this. It provides the system with a list of strings. These strings are the names of the sub-object selection levels that will appear in the Sub-Object drop down list. For example, the following code would register two sub-object levels, &quot;Gizmo&quot; and &quot;Center&quot;.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">const TCHAR *ptype[] = { &quot;Gizmo&quot;, &quot;Center&quot; };\n\
GetCOREInterface()-&gt;RegisterSubObjectTypes( ptype, 2 );</code></pre><p>When the user presses the Sub-Object button or changes the selection in the sub-object drop down list the modifier is notified. The system calls the Modifier&apos;s <code>ActivateSubObjSel()</code> method. In its implementation of this method the modifier is expected to provide an instance of a class derived from <code>CommandMode</code> to support Move, Rotate, Uniform scale, Non-uniform scale, and Squash. These modes replace their object level counterparts although the user still uses the move/rotate/scale tool bar buttons to activate these modes.</p>\n\
<p>Users will expect sub-object move, rotate and scale transformations to behave like their object level counterparts. To make this very easy to implement, there are a set of standard modes defined by the system that a developer may use. These modes handle all this logic internally -- the developer simply uses them. Most modifiers will be able to use these modes. The sample code below demonstrates how the modes are used. This example is from the implementation of class <code>SimpleMod</code>.</p>\n\
<p>In the class declaration the mode pointers are defined:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class SimpleMod : public Modifier\n\
{\n\
   //...\n\
   protected:\n\
     static  MoveModBoxCMode *moveMode;\n\
     static  RotateModBoxCMode *rotMode;\n\
     static  UScaleModBoxCMode *uscaleMode;\n\
     static  NUScaleModBoxCMode *nuscaleMode;\n\
     static  SquashModBoxCMode *squashMode;\n\
     static  SelectModBoxCMode *selectMode;\n\
     //...\n\
};</code></pre><p>In the <code>BeginEditParams()</code> method the modes are allocated:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void SimpleMod::BeginEditParams( IObjParam *ip, ULONG flags,\n\
                Animatable *prev )\n\
{\n\
   //...\n\
   // Create sub object editing modes.\n\
   moveMode   = newMoveModBoxCMode(this,ip);\n\
   rotMode     = new RotateModBoxCMode(this,ip);\n\
   uscaleMode   = newUScaleModBoxCMode(this,ip);\n\
   nuscaleMode = new NUScaleModBoxCMode(this,ip);\n\
   squashMode   = newSquashModBoxCMode(this,ip);\n\
   //...\n\
}</code></pre><p>In the method <code>ActivateSubobjSel()</code> the modes are activated. This method is defined in <code>BaseObject</code>.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void SimpleMod::ActivateSubobjSel(int level, XFormModes&amp; modes )\n\
{\n\
   switch ( level )\n\
   {\n\
     case 1: // Modifier box\n\
      modes = XFormModes( moveMode, rotMode,\n\
                nuscaleMode, uscaleMode,\n\
                squashMode,NULL);\n\
      break;\n\
     case 2: // Modifier Center\n\
      modes = XFormModes(moveMode,NULL,NULL,NULL,NULL,NULL);\n\
      break;\n\
   }\n\
   //...\n\
}</code></pre><p>When the item is finished being edited the modes can be deleted:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">void SimpleMod::EndEditParams(   IObjParam *ip,\n\
                ULONG flags,\n\
                Animatable *next)\n\
{\n\
   //...\n\
   ip-&gt;DeleteMode(moveMode);\n\
   ip-&gt;DeleteMode(rotMode);\n\
   ip-&gt;DeleteMode(uscaleMode);\n\
   ip-&gt;DeleteMode(nuscaleMode);\n\
   ip-&gt;DeleteMode(squashMode);\n\
\n\
   if ( moveMode ) deletemoveMode;\n\
   moveMode = NULL;\n\
\n\
   if ( rotMode ) deleterotMode;\n\
   rotMode = NULL;\n\
\n\
   if ( uscaleMode ) deleteuscaleMode;\n\
   uscaleMode = NULL;\n\
\n\
   if ( nuscaleMode ) delete nuscaleMode;\n\
   nuscaleMode = NULL;\n\
\n\
   if ( squashMode ) deletesquashMode;\n\
   squashMode = NULL;\n\
}</code></pre><p>Both the standard transformation modes and custom modes are expected to treat sub-object selection in the same way 3ds Max handles object selection. This means all the keypress options, such as <em>Ctrl</em> to add to a selection and <em>Alt</em> to subtract from a selection, should work in the same manner. To avoid forcing plug-in developers to implement all this logic, a <code>SelectionProcessor</code> class is provided. This class represents a mouse proc derived from the <code>MouseCallback</code> class. When you create an instance of the <code>SelectionProcessor</code> class you give its constructor a pointer to another <code>MouseCallback</code> -- the mouse procedure to handle the mode being implemented. The <code>SelectionProcessor</code> receives the mouse input first. It processes the mouse input using a standard selection logic and then calls the <code>MouseCallback</code> it was given.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>   <strong></strong> For working with modifier and sub-object selection, two classes are defined: <code>GenModSelectionProcessor</code> and <code>SubModSelectionProcessor</code>.</p>\n\
\n\
</div></div>\n\
\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../sub-object_selection.html\">Sub-Object Selection</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";