var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"converting_objects_to_compatible\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Converting Objects to Compatible Types</h1></div><p>3ds Max objects aren&apos;t always in a form suitable for a particular type of modifier. Consider the case of a 3ds Max user putting a volume select modifier on a procedural sphere in order to select some vertices within it. A procedural sphere is not defined by vertices and faces. Rather it is defined by it procedural definition, i.e. its radius, segment count, hemisphere setting, etc. But a user can put a volume select modifier on a sphere. The way this is handled is that 3ds Max asks the object if it can be converted to a triangle mesh object (which has vertices and faces). If the sphere responds yes, then 3ds Max asks the sphere to convert itself to one. Then, once the objects is in the form of the triangle mesh, the modifier may alter it. There are two methods that procedural objects implement to allow this to happen. These are <code>Object::CanConvertToType()</code> and <code>Object::ConvertToType()</code>.</p>\n\
<p>The code executed by the procedural sphere implementation of <code>Object::CanConvertToType()</code> is as follows:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">int SphereObject::CanConvertToType(Class_ID obtype)\n\
{\n\
   if (obtype==patchObjectClassID || obtype==defObjectClassID ||\n\
      obtype==triObjectClassID || obtype==EDITABLE_SURF_CLASS_ID)\n\
   {\n\
     return 1;\n\
   }\n\
   else\n\
   {\n\
     return  SimpleObject::CanConvertToType(obtype);\n\
   }\n\
}\n\
\n\
int  SimpleObject::CanConvertToType(Class_ID obtype)\n\
{\n\
   if (obtype==defObjectClassID || obtype==mapObjectClassID ||\n\
      obtype==triObjectClassID || obtype==patchObjectClassID)\n\
   {\n\
     return 1;\n\
   } \n\
   return Object::CanConvertToType(obtype);\n\
}\n\
\n\
int Object::CanConvertToType(Class_ID obtype)\n\
{\n\
   return obtype==ClassID();\n\
}\n\
</code></pre><p>Note that the sphere returns TRUE when asked if it can convert itself to a patch object, a deformable object (a generic object with points that can be modified), a triangle mesh object, and an editable surf object (NURBS object). In other cases it calls the base class method from <code>SimpleObject</code>. If <code>SimpleObject</code> doesn&apos;t recognize the type it calls its base class method from <code>Object</code>. The <code>Object</code> implementation returns nonzero if the object is asked to convert to itself, otherwise zero.</p>\n\
<p>When 3ds Max needs to actually have the object convert itself to the type required by the modifier it calls <code>ConvertToType()</code> on the object as passes the needed <code>Class_ID</code>. Below is the code from the procedural sphere&apos;s implementation.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">Object* SphereObject::ConvertToType(TimeValue t, Class_ID obtype)\n\
{\n\
   if (obtype == patchObjectClassID)\n\
   {\n\
     Interval valid = FOREVER;\n\
     float radius;\n\
     int smooth, genUVs;\n\
     pblock-&gt;GetValue(PB_RADIUS,t,radius,valid);\n\
     pblock-&gt;GetValue(PB_SMOOTH,t,smooth,valid); \n\
     pblock-&gt;GetValue(PB_GENUVS,t,genUVs,valid);\n\
     PatchObject *ob = new PatchObject();\n\
     BuildSpherePatch(ob-&gt;patch,radius,smooth,genUVs);\n\
     ob-&gt;SetChannelValidity(TOPO_CHAN_NUM,valid);\n\
     ob-&gt;SetChannelValidity(GEOM_CHAN_NUM,valid);\n\
     ob-&gt;UnlockObject();\n\
     return ob;\n\
   }\n\
   elseif (obtype == EDITABLE_SURF_CLASS_ID)\n\
   {\n\
     Interval valid = FOREVER;\n\
     float radius, hemi;\n\
     int recenter, genUVs;\n\
     pblock-&gt;GetValue(PB_RADIUS,t,radius,valid);\n\
     pblock-&gt;GetValue(PB_HEMI,t,hemi,valid); \n\
     pblock-&gt;GetValue(PB_RECENTER, t,recenter,valid);\n\
     pblock-&gt;GetValue(PB_GENUVS,t,genUVs,valid);\n\
     Object *ob = BuildNURBSSphere(radius, hemi, recenter,genUVs);\n\
     ob-&gt;SetChannelValidity(TOPO_CHAN_NUM,valid);\n\
     ob-&gt;SetChannelValidity(GEOM_CHAN_NUM,valid);\n\
     ob-&gt;UnlockObject();\n\
     return ob;\n\
   }\n\
   else\n\
   {\n\
     return  SimpleObject::ConvertToType(t,obtype);\n\
   }\n\
}</code></pre><p>The cases handled above cover converting to a patch or NURBS representation. The other cases are handled by the base classes.</p>\n\
<p>In summary, an object must implement two methods to allow a modifier to operate upon it. These are <code>Object::CanConvertToType()</code> and <code>Object::ConvertToType()</code>.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../writing_object_plug-ins.html\">Writing Object Plug-ins</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";