var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"dialog_based_creation_of_objects\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Dialog Based Creation of Objects</h1></div><p>The <code>Interface::NonMouseCreate()</code> method can be used to create a new object / node without going through the usual create mouse procedure sequence. This call must be made when the system is ready to create -- that is during execution of the system-provided default object creation process. To understand how this works consider the example of the Sphere object.</p>\n\
<p>When the user selects sphere from the 3ds Max UI, an instance of the sphere is created in memory (but not in the scene) and its parameters can be edited in the roll-out panel. The sphere&apos;s user interface allows the user to type in a center position and radius value. When and then press a &apos;Create&apos; button. When the button is pressed a new sphere in the scene will appear with these parameters. This is done through a call to <code>Interface::NonMouseCreate()</code> .</p>\n\
<p>When the <code>Interface::NonMouseCreate()</code> method is called, 3ds Max checks if a sphere has been created but is not yet in the scene. If so it creates a copy of the object, places it in the scene by attaching it to a node, and immediately deletes the original object. If there is no sphere <code>Interface::NonMouseCreate()</code> will create an object and place into the scene immediately using the default parameters.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> When you call <code>Interface::NonMouseCreate()</code> from your plug-in object (assuming it is the one being created), it will be deleted immediately. You should return immediately and call no more methods, and access no fields on the object, since its &quot;this&quot; pointer is invalid.</div>\n\
<p>The method <code>Interface::NonMouseCreate()</code> takes a single <code>Matrix3</code> argument which is the transformation relative to the construction plane. In the code below, the translation part of this matrix is set to the values the user typed in for the X, Y, and Z location of the sphere.</p>\n\
<p>The following code is from the file <code>\\\\samples\\\\objects\\\\sphere.cpp</code>. This is the code that is executed if the user presses the &apos;Create&apos; button in the &apos;Keyboard Entry&apos; rollup page. This allows the user to type in the values for the sphere&apos;s position and radius using the keyboard and then create the object.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">\n\
BOOL SphereTypeInDlgProc::DlgProc(\n\
     TimeValue t,\n\
     IParamMap *map,\n\
     HWND hWnd,\n\
     UINT msg,\n\
     WPARAM wParam,\n\
     LPARAM lParam)\n\
{\n\
   switch (msg)\n\
   {\n\
     case WM_COMMAND:\n\
     switch (LOWORD(wParam))\n\
     {\n\
      case IDC_TI_CREATE:\n\
       {\n\
         // The user has pressed the create button...\n\
         // If the radius is zero there is nothing to create\n\
         // so just return.\n\
         if (so-&gt;crtRadius==0.0) return TRUE;\n\
\n\
         // We only want to set the value if the object is\n\
         // not in the scene.\n\
         if (so-&gt;TestAFlag(A_OBJ_CREATING)) {\n\
           so-&gt;pblock-&gt;SetValue(PB_RADIUS,0,so-&gt;crtRadius);\n\
         }\n\
\n\
         // Create an identity matrix\n\
         Matrix3 tm(1);\n\
\n\
         // Update the translation portion of the matrix to the\n\
         // creation position type-in parameter\n\
         tm.SetTrans(so-&gt;crtPos);\n\
\n\
         // Call the interface method to create the node...\n\
         // This will use the values in the objects pblock.\n\
         so-&gt;ip-&gt;NonMouseCreate(tm);\n\
         so-&gt;suspendSnap = FALSE;\n\
\n\
         // NOTE that calling NonMouseCreate() will create a copy\n\
          // of this object and cause this object to be deleted.\n\
          // DO NOT DO ANYTHING BUT RETURN.\n\
         return TRUE;\n\
       }\n\
     }\n\
     break;\n\
   }\n\
   return FALSE;\n\
}\n\
\n\
\n\
\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../object_creation_methods.html\">Object Creation Methods</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";