var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"handling_mouse_procedures\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Handling Mouse Procedures</h1></div><p>3ds Max uses a callback mechanism to allow objects to be constructed using the mouse. To support mouse construction a plug-in must return an instance of a class derived from <code>CreateMouseCallBack</code> in its implementation of the function: <code>BaseObject::GetCreateMouseCallBack()</code>.</p>\n\
<p>In the Widget how-to sample the following call-back class is defined:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">class WidgetCreateCallBack\n\
  : public  CreateMouseCallBack\n\
{\n\
    IPoint2 sp0;  // First point in screen coordinates\n\
    Widget *ob;   // Pointer to the object\n\
    Point3 p0;    // First point in world coordinates\n\
    Point3 p1;    // Second point in world coordinates.\n\
\n\
public: \n\
\n\
    int proc(ViewExp *vpt, int msg, int point, int flags, IPoint2 m, Matrix3&amp; mat);\n\
};</code></pre><p>A plug-in&apos;s call-back class must override the <code>CreateMouseCallBack::proc()</code> method. This method will be called by 3ds Max until the creation procedure is completed (e.g. returns <code>CREATE_STOP</code>) or abandoned (e.g. returns <code>CREATE_ABORT</code>). When the creation procedure starts the <code>point</code> parameter has the value 0. The method is called every time the mouse is moved, giving the plug-in a chance to update itself in the viewport (e.g. through a call to <code>ParamBlockDesc2::InvalidateUI()</code>).Each time a new point is added manually by the user (e.g. byclicking the mouse) the <code>point</code> parameter is increased by one. The following is an example implementation used by the Widget how-to sample:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">int WidgetCreateCallBack::proc(ViewExp *vpt, int msg, int point,\n\
  intflags, IPoint2 m, Matrix3&amp; mat)\n\
{\n\
     if (msg == MOUSE_POINT || msg == MOUSE_MOVE)\n\
     {\n\
        switch(point)\n\
        {\n\
             case 0:\n\
            {\n\
                 // When point == 0 the message should only be a MOUSE_POINT\n\
                 DbgAssert(msg == MOUSE_POINT);\n\
\n\
                 // prevent snapping to self\n\
                 ob-&gt;suspendSnap = TRUE;\n\
\n\
                 // store screen coordinates        \n\
                 sp0 = m;\n\
\n\
                 // store world coordinates\n\
                 p0 = vpt-&gt;SnapPoint(m, m, NULL, SNAP_IN_PLANE);   \n\
\n\
                 // Update the matrix             \n\
                 mat.SetTrans(p0);\n\
                 break;\n\
            }\n\
             case 1:\n\
            {\n\
                 // store screen coordinates        \n\
                 ob-&gt;suspendSnap = TRUE;\n\
\n\
                 // store world coordinates        \n\
                 p1 = vpt-&gt;SnapPoint(m, m, NULL, SNAP_IN_PLANE);\n\
\n\
                 // Compute the size parameter by scaling using a constant\n\
                 float speedFactor = 24.0f;\n\
                 float theSize = (Length(p1 - p0) / speedFactor);\n\
\n\
                 // Set the overall size in parameter block\n\
                 ob-&gt;pblock2-&gt;SetValue(widget_size, ob-&gt;ip-&gt;GetTime(), theSize);\n\
\n\
                 // Invalidate the parameter block&apos;s UI\n\
                 // this triggers the mesh to be invalidated\n\
                 widget_param_blk.InvalidateUI();\n\
                 break;\n\
            }\n\
             case 2:\n\
            {\n\
                 // Indicate creation is complete\n\
                 return CREATE_STOP;\n\
            }\n\
        }\n\
    }\n\
    else\n\
    {\n\
        if (msg == MOUSE_ABORT)\n\
        {\n\
            // Indicate creation is abandoned\n\
            return CREATE_ABORT;\n\
        }\n\
    }\n\
    return TRUE;\n\
}</code></pre><p>The most common messages that your <code>proc</code> method will handle: <code>MOUSE_MOVE</code>, <code>MOUSE_POINT</code>, and <code>MOUSE_ABORT</code>. A <code>MOUSE_POINT</code> message is generated when the user first starts the drawing process and every time the user releases the left mouse button, until the mouse drawing process is completed (i.e. your function returns <code>TRUE</code>) or the process is aborted (i.e. the user presses escape). A list of more messages can be found here.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../object_creation_methods.html\">Object Creation Methods</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";