var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max SDK Help</title>\n\
    <meta name=\"topicid\" content=\"ring_array_creation_process\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2024/ENU/Max-Developer-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Ring Array Creation Process</h1></div><p>The Ring Array&apos;s implementation of <code>ClassDesc::BeginCreate()</code> installs a command mode that manages the creation process. This command mode is an instance of class <code>RingDriverCreateMode</code>. See the section on <a href=\"./../../3ds_max_sdk_features/user_interface/command_modes_and_mouse_procedur.html\">Command Modes and Mouse Procedures</a> for more information about command modes. Below is the method of <code>ClassDesc</code> to begin the creation of the Ring Array. Note how the command mode is pushed on the command stack.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">int RingDriverClassDesc::BeginCreate(Interface *i)\n\
{\n\
    SuspendSetKeyMode();\n\
    IObjCreate *iob = i-&gt;GetIObjCreate();\n\
\n\
   // This creates a new ring driver object and starts the editing\n\
   // of the objects parameters.\n\
    theRingDriverCreateMode.Begin( iob, this );\n\
\n\
   // This sets the creation command mode as the active mode\n\
   // at the top of the command stack.\n\
    iob-&gt;PushCommandMode( &amp;theRingDriverCreateMode );\n\
\n\
   return TRUE;\n\
}</code></pre><p>A command mode has a mouse proc that handles the user / mouse interaction while the command mode is in effect. A method of the command mode, <code>MouseProc()</code>, is expected to return an instance of a class derived from <code>MouseCallBack</code>. This is the procedure that defines the user / mouse interaction during the system creation. The <code>RingDriverCreateMode</code> returns a pointer to an instance of <code>RingDriverCreationManager</code>. The <code>RingDriverCreationManager</code> is derived from both <code>MouseCallback</code> and <code>ReferenceMaker</code>. This allows it to handle the user / mouse interaction and to make references to the nodes it creates in the scene. Below is the code for the command mode:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// This is the command mode that manages the overall process when\n\
// the system is created.\n\
class RingDriverCreateMode : public CommandMode \n\
{\n\
   // This instance of RingDriverCreationManager handles the user/mouse\n\
   // interaction as the Ring Array is created.\n\
    RingDriverCreationManager proc;\n\
   public:\n\
\n\
   // These two methods just call the creation proc method of\n\
   // the same name.\n\
   // This creates a new ring driver object and starts the editing\n\
   // of the objects parameters. This is called just before the\n\
   // command mode is pushed on the stack to begin the creation\n\
   // process.\n\
   void Begin( IObjCreate *ioc, ClassDesc *desc )\n\
   { proc.Begin( ioc, desc ); }\n\
\n\
   // This terminates the editing of the ring driver&apos;s parameters.\n\
   // This is called just before the command mode is removed from\n\
   // the command stack.\n\
   void End() { proc.End(); }\n\
\n\
   // This returns the type of command mode this is. See the online\n\
   // help under this method for a list of the available choices.\n\
   // In this case we are a creation command mode.\n\
   int Class() { return CREATE_COMMAND; }\n\
\n\
   // Returns the ID of the command mode. This value should be the\n\
   // constant CID_USER plus some random value chosen\n\
   // by the developer.\n\
   int ID() { return CID_RINGCREATE; }\n\
\n\
   // This method returns a pointer to the mouse proc that will\n\
   // handle the user/mouse interaction. It also establishes\n\
   // the number of points that may be accepted by the mouse proc.\n\
   // In this case we set the number of points to 100000. The user\n\
   // process will terminate prior to this many points being\n\
   // entered when the mouse proc returns FALSE.\n\
   // The mouse proc returned from this method is an instance of\n\
   // RingDriverCreationManager. Note that that class is derived\n\
   // from MouseCallBack.\n\
   MouseCallBack *MouseProc(int *numPoints)\n\
   { *numPoints = 100000; return &amp;proc; }\n\
\n\
   // This method is called to flag nodes in the foreground plane.\n\
   // We just return the standard CHANGE_FG_SELECTED value\n\
   // to indicate that selected nodes will go into the foreground.\n\
   // This allows the system to speed up screen redraws. See the\n\
   // Advanced Topics section on Foreground / Background planes\n\
   // for more details.\n\
    ChangeForegroundCallback *ChangeFGProc()\n\
   { return CHANGE_FG_SELECTED; }\n\
\n\
   // This method returns TRUE if the command mode needs to\n\
   // change the foreground proc (using ChangeFGProc()) and FALSE\n\
   // if it does not.\n\
   BOOL ChangeFG( CommandMode *oldMode )\n\
   { return (oldMode-&gt;ChangeFGProc() != CHANGE_FG_SELECTED); }\n\
\n\
   // This method is called when a command mode becomes active. We\n\
   // don&apos;t need to do anything at this time so our implementation\n\
   // is NULL\n\
   void EnterMode() {}\n\
\n\
   // This method is called when the command mode is replaced by\n\
   // another mode. Again, we don&apos;t need to do anything special here.\n\
   void ExitMode() {}\n\
};</code></pre><p>Note that the <code>MouseProc()</code> method returns a number of points it uses in its creation process. This implementation returns 100000. The user will never actually enter 100000 points -- the process is aborted when the mouse proc returns FALSE. This happens after the user has released the mouse button and the process is finished, or if the user aborts the process.</p>\n\
<p>The mouse proc handles the user / mouse interaction. This process works as follows: When the user first presses the mouse button the nodes are created in the scene -- a single central node (the parent) and the user-specified number of child nodes. The children are arranged in a circle around the parent. As the user holds the mouse button down and drags, the radius of the system is changed to reflect the mouse position. When the user releases the mouse button the creation process is finished and the nodes positions are set. If the user right clicks the mouse while the left mouse button is still down the process is aborted. What happens in detail during creation is described below:</p>\n\
<p>When the user first presses the left mouse button in the viewport the system first creates the dummy object at the center of the system. It does this by calling <code>Interface::CreateInstance()</code>. This method creates a new instance of a plug-in class by specifying the <code>Class_ID</code> and <code>SuperClassID</code> of the plug-in. A node in the scene is then created from this instance using <code>Interface::CreateObjectNode()</code>. Next the method <code>GenBoxObject::SetParams()</code> is called to set the parameters used by the box objects. At this point, a single dummy object has been created in the scene. The code to do this is shown below:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// Snap the inital mouse point\n\
mat.IdentityMatrix();\n\
center = vpx-&gt;SnapPoint(m,m,NULL,SNAP_IN_PLANE);\n\
mat.SetTrans(center);\n\
\n\
// This method creates an instance of a plug-in object\n\
// given its SuperClassID and ClassID. Here we create\n\
// a dummy object at the center of the Ring Array system.\n\
DummyObject *dumObj = (DummyObject *)createInterface-&gt;\n\
CreateInstance(HELPER_CLASS_ID,Class_ID(DUMMY_CLASS_ID,0));\n\
assert(dumObj);\n\
\n\
// This method creates a node given the instance.\n\
dummyNode = createInterface-&gt;CreateObjectNode(dumObj);\n\
\n\
// This sets the size of the dummy object\n\
// box representation\n\
dumObj-&gt;SetBox(Box3(Point3(-DUMSZ,-DUMSZ,-DUMSZ),\n\
Point3(DUMSZ,DUMSZ,DUMSZ)));</code></pre><p>The plug-in then creates another dummy object for the driven nodes. The code to do this is similar:</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// These are the instances that surround the center of\n\
// the Ring Array\n\
GenBoxObject *ob = ( GenBoxObject *)createInterface-&gt;\n\
CreateInstance(GEOMOBJECT_CLASS_ID,Class_ID(BOXOBJ_CLASS_ID,0));\n\
ob-&gt;SetParams(BOXSZ,BOXSZ,BOXSZ,1,1,1,FALSE);</code></pre><p>The next step in the creation process is to create the user-specified number of ring nodes, those that surround the center. The loop below is where this is done.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">// This is the loop that creates the nodes in the ring\n\
// array, sets their controller, and attaches them to\n\
// the parent node.\n\
for (int i=0; i&lt;theDriver-&gt; GetNum(createInterface-&gt;GetTime()); i++)\n\
{\n\
   // Create a new node given the instance.\n\
   newNode = createInterface-&gt;CreateObjectNode(ob);\n\
\n\
   // Create a new driven controller of the driver control.\n\
   // This constructor creates a reference from the driven\n\
   // controller to the driver. This is done so the\n\
   // system is aware of the dependency between the\n\
   // driven controller and the RingDriver object.\n\
   DrivenControl* driven = new DrivenControl(theDriver,i);\n\
\n\
   // Set the transform controller used by the node.\n\
   newNode-&gt;SetTMController(driven);\n\
\n\
   // Attach the new node as a child of the central node.\n\
   dummyNode-&gt;AttachChild(newNode);\n\
\n\
   // Set the new node as the &apos;i-th&apos; driven node\n\
   // of the driver object.\n\
   theDriver-&gt;SetDrivenNode(i,newNode);\n\
}</code></pre><p>This code fragment demonstrates several concepts.</p>\n\
<p>The statement <code>newNode</code> = createInterface-&gt;CreateObjectNode(ob); calls a method of the interface class that creates a new node in the scene given an object pointer. The object pointer in this case is the dummy helper object.</p>\n\
<p>The statement <code>DrivenControl</code>* driven = new DrivenControl(theDriver,i); creates a new driven controller object. It passes the RingDriver object and the index of this node to the constructor.</p>\n\
<p>The statement <code>newNode</code>-&gt;SetTMController(driven); establishes the transform controller used by the node. We set it to our driven controller (and again, the driven controller just uses the driver controller to get all of its values).</p>\n\
<p>The statement <code>dummyNode</code>-&gt;AttachChild(newNode); sets the new node as a child of the dummy node at the center. In this way the child nodes will inherit the transformation of the parent node (for example, as it is moved or rotated in the scene). The attached nodes are still of course free to move on their own, independent of the parent (but driven by the driver).</p>\n\
<p>The statement <code>theDriver</code>-&gt;SetDrivenNode(i,newNode); is used to add the node to the list of nodes maintained by the driver.</p>\n\
<p>As the user drags the mouse while the left button is still down, the plug-in mouse proc receives <code>MOUSE_MOVE</code> messages. The plug-in responds by updating the radius value and redrawing the viewports.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">case MOUSE_MOVE:\n\
   if (node0)\n\
   {\n\
     r = (float)fabs(vpx-&gt;SnapLength(vpx-&gt;GetCPDisp(center,Point3(0,1,0),pt0,m)));\n\
     theDriver-&gt;SetRad(0,r);\n\
     theDriver-&gt;radSpin-&gt;SetValue(r, FALSE );\n\
     createInterface-&gt;RedrawViews(createInterface-&gt;GetTime(),REDRAW_NORMAL,theDriver);\n\
   }\n\
   res = TRUE;\n\
   break;</code></pre><p>When the user releases the mouse button to end the drag operation they have finalized the creation process.</p>\n\
<pre class=\"pre codeblock\"><code class=\"language-cpp prettyprint\">else\n\
{\n\
   // The mouse has been released - finish the creation\n\
   // process. Select the first node so if we go into\n\
   // the motion branch we&apos;ll see its parameters.\n\
   // We set the flag to ignore the selection change\n\
   // because the NotifyRefChanged() method receives\n\
   // a message when the node is selected or deselected\n\
   // and terminates the parameter editing process. This\n\
   // flag causes it to ignore the selection change.\n\
   ignoreSelectionChange = TRUE;\n\
   createInterface-&gt;SelectNode( theDriver-&gt;GetDrivenNode(0) );\n\
   ignoreSelectionChange = FALSE;\n\
\n\
   // This registers the undo object with the system so\n\
   // the user may undo the creation.\n\
   theHold.Accept(IDS_CREATE);\n\
\n\
   // Set the return value to FALSE to indicate the\n\
   // creation process is finished. This mouse proc will\n\
   // no longer be called until the user creates another\n\
   // Ring Array.\n\
   res = FALSE;\n\
}</code></pre><p>The code above shows how the Ring Array is created. When the mouse proc has finished the user is still in the Ring Array command mode and may create another Ring Array. The system calls <code>ClassDesc::EndCreate()</code> when the object creation process is finished. This may happen if the user changes to another branch of the command panel or selects a tool from the toolbar that replaces the creation command mode.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../writing_system_plug-ins.html\">Writing System Plug-ins</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";